@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __ABOUT_H__
@@ -27,13 +26,13 @@
   _("GNU Image Manipulation Program")
 
 #define GIMP_COPYRIGHT \
-  _("Copyright © 1995-2008\n" \
+  _("Copyright © 1995-2009\n" \
     "Spencer Kimball, Peter Mattis and the GIMP Development Team")
 
 #define GIMP_LICENSE \
-  _("GIMP is free software; you can redistribute it and/or modify it "   \
+  _("GIMP is free software: you can redistribute it and/or modify it "   \
     "under the terms of the GNU General Public License as published by " \
-    "the Free Software Foundation; either version 2 of the License, or " \
+    "the Free Software Foundation; either version 3 of the License, or " \
     "(at your option) any later version."                                \
     "\n\n"                                                               \
     "GIMP is distributed in the hope that it will be useful, "           \
@@ -42,8 +41,7 @@
     "GNU General Public License for more details."                       \
     "\n\n"                                                               \
     "You should have received a copy of the GNU General Public License " \
-    "along with GIMP; if not, write to the Free Software Foundation, "   \
-    "Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.")
+    "along with GIMP.  If not, see <http://www.gnu.org/licenses/>.")
 
 
 #endif /* __ABOUT_H__ */
