@@ -37,11 +37,13 @@
 #include "widgets/gimpcontainerview.h"
 #include "widgets/gimpdock.h"
 #include "widgets/gimpdockable.h"
+#include "widgets/gimpdockwindow.h"
 #include "widgets/gimpimageeditor.h"
 #include "widgets/gimpitemtreeview.h"
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpimagewindow.h"
 #include "display/gimpnavigationeditor.h"
 #include "display/gimpstatusbar.h"
 
@@ -62,6 +64,8 @@
 #include "dockable-actions.h"
 #include "documents-actions.h"
 #include "drawable-actions.h"
+#include "dynamics-actions.h"
+#include "dynamics-editor-actions.h"
 #include "edit-actions.h"
 #include "error-console-actions.h"
 #include "file-actions.h"
@@ -83,6 +87,8 @@
 #include "text-editor-actions.h"
 #include "text-tool-actions.h"
 #include "tool-options-actions.h"
+#include "tool-preset-actions.h"
+#include "tool-preset-editor-actions.h"
 #include "tools-actions.h"
 #include "vectors-actions.h"
 #include "view-actions.h"
@@ -142,6 +148,12 @@ static const GimpActionFactoryEntry action_groups[] =
   { "drawable", N_("Drawable"), GIMP_STOCK_LAYER,
     drawable_actions_setup,
     drawable_actions_update },
+  { "dynamics", N_("Paint Dynamics"), GIMP_STOCK_DYNAMICS,
+    dynamics_actions_setup,
+    dynamics_actions_update },
+  { "dynamics-editor", N_("Paint Dynamics Editor"), GIMP_STOCK_DYNAMICS,
+    dynamics_editor_actions_setup,
+    dynamics_editor_actions_update },
   { "edit", N_("Edit"), GTK_STOCK_EDIT,
     edit_actions_setup,
     edit_actions_update },
@@ -160,6 +172,12 @@ static const GimpActionFactoryEntry action_groups[] =
   { "gradients", N_("Gradients"), GIMP_STOCK_GRADIENT,
     gradients_actions_setup,
     gradients_actions_update },
+  { "tool-preset", N_("Tool Presets"), GIMP_STOCK_TOOL_PRESET,
+    tool_preset_actions_setup,
+    tool_preset_actions_update },
+  { "tool-preset-editor", N_("Tool Preset Editor"), GIMP_STOCK_TOOL_PRESET,
+    tool_preset_editor_actions_setup,
+    tool_preset_editor_actions_update },
   { "help", N_("Help"), GTK_STOCK_HELP,
     help_actions_setup,
     help_actions_update },
@@ -262,10 +280,17 @@ action_data_get_gimp (gpointer data)
 
   if (GIMP_IS_DISPLAY (data))
     return ((GimpDisplay *) data)->gimp;
+  else if (GIMP_IS_IMAGE_WINDOW (data))
+    {
+      GimpDisplayShell *shell = gimp_image_window_get_active_shell (data);
+      return shell ? shell->display->gimp : NULL;
+    }
   else if (GIMP_IS_GIMP (data))
     return data;
   else if (GIMP_IS_DOCK (data))
     context = gimp_dock_get_context (((GimpDock *) data));
+  else if (GIMP_IS_DOCK_WINDOW (data))
+    context = gimp_dock_window_get_context (((GimpDockWindow *) data));
   else if (GIMP_IS_CONTAINER_VIEW (data))
     context = gimp_container_view_get_context ((GimpContainerView *) data);
   else if (GIMP_IS_CONTAINER_EDITOR (data))
@@ -289,10 +314,17 @@ action_data_get_context (gpointer data)
 
   if (GIMP_IS_DISPLAY (data))
     return gimp_get_user_context (((GimpDisplay *) data)->gimp);
+  else if (GIMP_IS_IMAGE_WINDOW (data))
+    {
+      GimpDisplayShell *shell = gimp_image_window_get_active_shell (data);
+      return shell ? gimp_get_user_context (shell->display->gimp) : NULL;
+    }
   else if (GIMP_IS_GIMP (data))
     return gimp_get_user_context (data);
   else if (GIMP_IS_DOCK (data))
     return gimp_dock_get_context ((GimpDock *) data);
+  else if (GIMP_IS_DOCK_WINDOW (data))
+    return gimp_dock_window_get_context (((GimpDockWindow *) data));
   else if (GIMP_IS_CONTAINER_VIEW (data))
     return gimp_container_view_get_context ((GimpContainerView *) data);
   else if (GIMP_IS_CONTAINER_EDITOR (data))
@@ -309,16 +341,24 @@ GimpImage *
 action_data_get_image (gpointer data)
 {
   GimpContext *context = NULL;
+  GimpDisplay *display = NULL;
 
   if (! data)
     return NULL;
 
   if (GIMP_IS_DISPLAY (data))
-    return ((GimpDisplay *) data)->image;
+    display = (GimpDisplay *) data;
+  else if (GIMP_IS_IMAGE_WINDOW (data))
+    {
+      GimpDisplayShell *shell = gimp_image_window_get_active_shell (data);
+      display = shell ? shell->display : NULL;
+    }
   else if (GIMP_IS_GIMP (data))
     context = gimp_get_user_context (data);
   else if (GIMP_IS_DOCK (data))
     context = gimp_dock_get_context ((GimpDock *) data);
+  else if (GIMP_IS_DOCK_WINDOW (data))
+    context = gimp_dock_window_get_context (((GimpDockWindow *) data));
   else if (GIMP_IS_ITEM_TREE_VIEW (data))
     return gimp_item_tree_view_get_image ((GimpItemTreeView *) data);
   else if (GIMP_IS_IMAGE_EDITOR (data))
@@ -328,6 +368,8 @@ action_data_get_image (gpointer data)
 
   if (context)
     return gimp_context_get_image (context);
+  else if (display)
+    return gimp_display_get_image (display);
 
   return NULL;
 }
@@ -342,10 +384,17 @@ action_data_get_display (gpointer data)
 
   if (GIMP_IS_DISPLAY (data))
     return data;
+  else if (GIMP_IS_IMAGE_WINDOW (data))
+    {
+      GimpDisplayShell *shell = gimp_image_window_get_active_shell (data);
+      return shell ? shell->display : NULL;
+    }
   else if (GIMP_IS_GIMP (data))
     context = gimp_get_user_context (data);
   else if (GIMP_IS_DOCK (data))
     context = gimp_dock_get_context ((GimpDock *) data);
+  else if (GIMP_IS_DOCK_WINDOW (data))
+    context = gimp_dock_window_get_context (((GimpDockWindow *) data));
   else if (GIMP_IS_NAVIGATION_EDITOR (data))
     context = ((GimpNavigationEditor *) data)->context;
 
@@ -355,6 +404,20 @@ action_data_get_display (gpointer data)
   return NULL;
 }
 
+GimpDisplayShell *
+action_data_get_shell (gpointer data)
+{
+  GimpDisplay      *display = NULL;
+  GimpDisplayShell *shell   = NULL;
+
+  display = action_data_get_display (data);
+
+  if (display)
+    shell = gimp_display_get_shell (display);
+
+  return shell;
+}
+
 GtkWidget *
 action_data_get_widget (gpointer data)
 {
@@ -371,11 +434,27 @@ action_data_get_widget (gpointer data)
     return data;
 
   if (display)
-    return display->shell;
+    return GTK_WIDGET (gimp_display_get_shell (display));
 
   return dialogs_get_toolbox ();
 }
 
+gint
+action_data_sel_count (gpointer data)
+{
+  if (GIMP_IS_CONTAINER_EDITOR (data))
+    {
+      GimpContainerEditor  *editor;
+
+      editor = GIMP_CONTAINER_EDITOR (data);
+      return gimp_container_view_get_selected (editor->view, NULL);
+    }
+  else
+    {
+      return 0;
+    }
+}
+
 gdouble
 action_select_value (GimpActionSelectType  select_type,
                      gdouble               value,
@@ -601,8 +680,9 @@ action_message (GimpDisplay *display,
                 const gchar *format,
                 ...)
 {
-  GimpDisplayShell *shell    = GIMP_DISPLAY_SHELL (display->shell);
-  const gchar      *stock_id = NULL;
+  GimpDisplayShell *shell     = gimp_display_get_shell (display);
+  GimpStatusbar    *statusbar = gimp_display_shell_get_statusbar (shell);
+  const gchar      *stock_id  = NULL;
   va_list           args;
 
   if (GIMP_IS_TOOL_OPTIONS (object))
@@ -617,8 +697,7 @@ action_message (GimpDisplay *display,
     }
 
   va_start (args, format);
-  gimp_statusbar_push_temp_valist (GIMP_STATUSBAR (shell->statusbar),
-                                   GIMP_MESSAGE_INFO, stock_id,
-                                   format, args);
+  gimp_statusbar_push_temp_valist (statusbar, GIMP_MESSAGE_INFO,
+                                   stock_id, format, args);
   va_end (args);
 }
