@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -40,14 +39,14 @@
 static const GimpActionEntry brush_editor_actions[] =
 {
   { "brush-editor-popup", GIMP_STOCK_BRUSH,
-    N_("Brush Editor Menu"), NULL, NULL, NULL,
+    NC_("brush-editor-action", "Brush Editor Menu"), NULL, NULL, NULL,
     GIMP_HELP_BRUSH_EDITOR_DIALOG }
 };
 
 static const GimpToggleActionEntry brush_editor_toggle_actions[] =
 {
   { "brush-editor-edit-active", GIMP_STOCK_LINKED,
-    N_("Edit Active Brush"), NULL, NULL,
+    NC_("brush-editor-action", "Edit Active Brush"), NULL, NULL,
     G_CALLBACK (data_editor_edit_active_cmd_callback),
     FALSE,
     GIMP_HELP_BRUSH_EDITOR_EDIT_ACTIVE }
@@ -57,11 +56,11 @@ static const GimpToggleActionEntry brush_editor_toggle_actions[] =
 void
 brush_editor_actions_setup (GimpActionGroup *group)
 {
-  gimp_action_group_add_actions (group,
+  gimp_action_group_add_actions (group, "brush-editor-action",
                                  brush_editor_actions,
                                  G_N_ELEMENTS (brush_editor_actions));
 
-  gimp_action_group_add_toggle_actions (group,
+  gimp_action_group_add_toggle_actions (group, "brush-editor-action",
                                         brush_editor_toggle_actions,
                                         G_N_ELEMENTS (brush_editor_toggle_actions));
 }
