@@ -81,7 +81,7 @@ channels_edit_attributes_cmd_callback (GtkAction *action,
                                         action_data_get_context (data),
                                         widget,
                                         &channel->color,
-                                        gimp_object_get_name (GIMP_OBJECT (channel)),
+                                        gimp_object_get_name (channel),
                                         _("Channel Attributes"),
                                         "gimp-channel-edit",
                                         GTK_STOCK_EDIT,
@@ -113,7 +113,7 @@ channels_new_cmd_callback (GtkAction *action,
                                         widget,
                                         &channel_color,
                                         channel_name ? channel_name :
-                                        _("New Channel"),
+                                        _("Channel"),
                                         _("New Channel"),
                                         "gimp-channel-new",
                                         GIMP_STOCK_CHANNEL,
@@ -159,8 +159,7 @@ channels_new_last_vals_cmd_callback (GtkAction *action,
                                _("New Channel"));
 
   new_channel = gimp_channel_new (image, width, height,
-                                  channel_name ? channel_name :
-                                  _("New Channel"), &color);
+                                  channel_name, &color);
 
   gimp_drawable_fill_by_type (GIMP_DRAWABLE (new_channel),
                               action_data_get_context (data),
@@ -267,7 +266,7 @@ channels_duplicate_cmd_callback (GtkAction *action,
        *  the latter would add a duplicated group inside itself instead of
        *  above it
        */
-      parent = GIMP_CHANNEL (gimp_viewable_get_parent (GIMP_VIEWABLE (channel)));
+      parent = gimp_channel_get_parent (channel);
     }
 
   gimp_image_add_channel (image, new_channel, parent, -1, TRUE);
@@ -335,6 +334,7 @@ channels_new_channel_response (GtkWidget            *widget,
 
       if (channel_name)
         g_free (channel_name);
+
       channel_name =
         g_strdup (gtk_entry_get_text (GTK_ENTRY (options->name_entry)));
 
@@ -392,7 +392,7 @@ channels_edit_channel_response (GtkWidget            *widget,
       gimp_color_button_get_color (GIMP_COLOR_BUTTON (options->color_panel),
                                    &color);
 
-      if (strcmp (new_name, gimp_object_get_name (GIMP_OBJECT (channel))))
+      if (strcmp (new_name, gimp_object_get_name (channel)))
         name_changed = TRUE;
 
       if (gimp_rgba_distance (&color, &channel->color) > 0.0001)
