@@ -511,7 +511,7 @@ context_brush_spacing_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH (brush) && gimp_data_is_writable (GIMP_DATA (brush)))
     {
       action_select_property ((GimpActionSelectType) value,
                               action_data_get_display (data),
@@ -532,7 +532,8 @@ context_brush_shape_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       GimpBrushGenerated *generated = GIMP_BRUSH_GENERATED (brush);
       GimpDisplay        *display;
@@ -564,7 +565,8 @@ context_brush_radius_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       GimpBrushGenerated *generated = GIMP_BRUSH_GENERATED (brush);
       GimpDisplay        *display;
@@ -623,7 +625,8 @@ context_brush_spikes_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       action_select_property ((GimpActionSelectType) value,
                               action_data_get_display (data),
@@ -644,7 +647,8 @@ context_brush_hardness_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       action_select_property ((GimpActionSelectType) value,
                               action_data_get_display (data),
@@ -665,7 +669,8 @@ context_brush_aspect_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       action_select_property ((GimpActionSelectType) value,
                               action_data_get_display (data),
@@ -686,7 +691,8 @@ context_brush_angle_cmd_callback (GtkAction *action,
 
   brush = gimp_context_get_brush (context);
 
-  if (GIMP_IS_BRUSH_GENERATED (brush) && GIMP_DATA (brush)->writable)
+  if (GIMP_IS_BRUSH_GENERATED (brush) &&
+      gimp_data_is_writable (GIMP_DATA (brush)))
     {
       GimpBrushGenerated *generated = GIMP_BRUSH_GENERATED (brush);
       GimpDisplay        *display;
@@ -862,17 +868,14 @@ context_set_color_index (gint      index,
 static GimpPaletteEditor *
 context_get_palette_editor (void)
 {
-  GimpDialogFactory *dialog_factory;
-  GimpSessionInfo   *info;
+  GimpSessionInfo *info = NULL;
 
-  dialog_factory = gimp_dialog_factory_from_name ("dock");
+  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (gimp_dialog_factory_get_singleton ()), NULL);
 
-  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
-
-  info = gimp_dialog_factory_find_session_info (dialog_factory,
+  info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                 "gimp-palette-editor");
-  if (info && info->widget)
-    return GIMP_PALETTE_EDITOR (gtk_bin_get_child (GTK_BIN (info->widget)));
+  if (info && gimp_session_info_get_widget (info))
+    return GIMP_PALETTE_EDITOR (gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (info))));
 
   return NULL;
 }
@@ -880,17 +883,14 @@ context_get_palette_editor (void)
 static GimpColormapEditor *
 context_get_colormap_editor (void)
 {
-  GimpDialogFactory *dialog_factory;
-  GimpSessionInfo   *info;
+  GimpSessionInfo *info = NULL;
 
-  dialog_factory = gimp_dialog_factory_from_name ("dock");
+  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (gimp_dialog_factory_get_singleton ()), NULL);
 
-  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
-
-  info = gimp_dialog_factory_find_session_info (dialog_factory,
+  info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                 "gimp-indexed-palette");
-  if (info && info->widget)
-    return GIMP_COLORMAP_EDITOR (gtk_bin_get_child (GTK_BIN (info->widget)));
+  if (info && gimp_session_info_get_widget (info))
+    return GIMP_COLORMAP_EDITOR (gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (info))));
 
   return NULL;
 }
