@@ -31,6 +31,7 @@
 #include "base/tile.h"
 
 #include "core/gimp.h"
+#include "core/gimp-utils.h"
 #include "core/gimpcontext.h"
 #include "core/gimpimage.h"
 #include "core/gimplayer.h"
@@ -44,6 +45,7 @@
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpimagewindow.h"
 
 #include "widgets/gimpaction.h"
 #include "widgets/gimpmenufactory.h"
@@ -119,7 +121,7 @@ debug_dump_menus_cmd_callback (GtkAction *action,
 {
   GList *list;
 
-  for (list = global_menu_factory->registered_menus;
+  for (list = gimp_menu_factory_get_registered_menus (global_menu_factory);
        list;
        list = g_list_next (list))
     {
@@ -160,7 +162,7 @@ debug_dump_managers_cmd_callback (GtkAction *action,
 {
   GList *list;
 
-  for (list = global_menu_factory->registered_menus;
+  for (list = gimp_menu_factory_get_registered_menus (global_menu_factory);
        list;
        list = g_list_next (list))
     {
@@ -186,18 +188,21 @@ void
 debug_dump_keyboard_shortcuts_cmd_callback (GtkAction *action,
                                             gpointer   data)
 {
-  GimpDisplay      *display     = NULL;
-  GimpDisplayShell *shell       = NULL;
-  GtkAccelGroup    *accel_group = NULL;
-  GList            *group_it    = NULL;
-  GList            *strings     = NULL;
+  GimpDisplay      *display;
+  GimpImageWindow  *window;
+  GtkUIManager     *manager;
+  GtkAccelGroup    *accel_group;
+  GList            *group_it;
+  GList            *strings = NULL;
   return_if_no_display (display, data);
 
-  shell       = GIMP_DISPLAY_SHELL (display->shell);
-  accel_group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (shell->menubar_manager));
+  window  = gimp_display_shell_get_window (gimp_display_get_shell (display));
+  manager = GTK_UI_MANAGER (gimp_image_window_get_ui_manager (window));
+
+  accel_group = gtk_ui_manager_get_accel_group (manager);
 
   /* Gather formated strings of keyboard shortcuts */
-  for (group_it = gtk_ui_manager_get_action_groups (GTK_UI_MANAGER (shell->menubar_manager));
+  for (group_it = gtk_ui_manager_get_action_groups (manager);
        group_it;
        group_it = g_list_next (group_it))
     {
@@ -287,18 +292,17 @@ debug_benchmark_projection (GimpImage *image)
 {
   GimpProjection *projection = gimp_image_get_projection (image);
   TileManager    *tiles;
-  GTimer         *timer;
   gint            x, y;
 
-  gimp_image_update (image,
-                     0, 0,
-                     gimp_image_get_width  (image),
-                     gimp_image_get_height (image));
+  gimp_image_invalidate (image,
+                         0, 0,
+                         gimp_image_get_width  (image),
+                         gimp_image_get_height (image));
   gimp_projection_flush_now (projection);
 
   tiles = gimp_pickable_get_tiles (GIMP_PICKABLE (projection));
 
-  timer = g_timer_new ();
+  GIMP_TIMER_START ();
 
   for (x = 0; x < tile_manager_width (tiles); x += TILE_WIDTH)
     {
@@ -310,10 +314,7 @@ debug_benchmark_projection (GimpImage *image)
         }
     }
 
-  g_print ("Validation of the entire projection took %.0f ms\n",
-           1000 * g_timer_elapsed (timer, NULL));
-
-  g_timer_destroy (timer);
+  GIMP_TIMER_END ("Validation of the entire projection");
 
   g_object_unref (image);
 
@@ -384,15 +385,19 @@ debug_dump_menus_recurse_menu (GtkWidget *menu,
                                gint       depth,
                                gchar     *path)
 {
+  GList *children;
   GList *list;
 
-  for (list = GTK_MENU_SHELL (menu)->children; list; list = g_list_next (list))
+  children = gtk_container_get_children (GTK_CONTAINER (menu));
+
+  for (list = children; list; list = g_list_next (list))
     {
       GtkWidget *menu_item = GTK_WIDGET (list->data);
       GtkWidget *child     = gtk_bin_get_child (GTK_BIN (menu_item));
 
       if (GTK_IS_LABEL (child))
         {
+          GtkWidget   *submenu;
           const gchar *label;
           gchar       *full_path;
           gchar       *help_page;
@@ -411,13 +416,16 @@ debug_dump_menus_recurse_menu (GtkWidget *menu,
           g_free (format_str);
           g_free (help_page);
 
-          if (GTK_MENU_ITEM (menu_item)->submenu)
-            debug_dump_menus_recurse_menu (GTK_MENU_ITEM (menu_item)->submenu,
-                                           depth + 1, full_path);
+          submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
+
+          if (submenu)
+            debug_dump_menus_recurse_menu (submenu, depth + 1, full_path);
 
           g_free (full_path);
         }
     }
+
+  g_list_free (children);
 }
 
 static void
