@@ -34,8 +34,6 @@
 #include "widgets/gimpdocked.h"
 #include "widgets/gimpsessioninfo.h"
 
-#include "dialogs/dialogs.h"
-
 #include "dockable-commands.h"
 
 
@@ -54,6 +52,7 @@ dockable_add_tab_cmd_callback (GtkAction   *action,
   if (value)
     {
       GtkWidget *dockable;
+      GimpDock  *dock;
       gchar     *identifier;
       gchar     *p;
 
@@ -64,10 +63,10 @@ dockable_add_tab_cmd_callback (GtkAction   *action,
       if (p)
         *p = '\0';
 
-      dockable =
-        gimp_dialog_factory_dockable_new (gimp_dock_get_dialog_factory (dockbook->dock),
-                                          dockbook->dock,
-                                          identifier, -1);
+      dock     = gimp_dockbook_get_dock (dockbook);
+      dockable = gimp_dialog_factory_dockable_new (gimp_dock_get_dialog_factory (dock),
+                                                   dock,
+                                                   identifier, -1);
 
       g_free (identifier);
 
@@ -75,7 +74,7 @@ dockable_add_tab_cmd_callback (GtkAction   *action,
        *  existing singleton dockable, so check if it already is
        *  attached to a dockbook.
        */
-      if (dockable && ! GIMP_DOCKABLE (dockable)->dockbook)
+      if (dockable && ! gimp_dockable_get_dockbook (GIMP_DOCKABLE (dockable)))
         gimp_dockbook_add (dockbook, GIMP_DOCKABLE (dockable), -1);
     }
 }
@@ -151,20 +150,27 @@ dockable_toggle_view_cmd_callback (GtkAction *action,
           substring = strstr (identifier, "grid");
 
           if (substring && view_type == GIMP_VIEW_TYPE_GRID)
-            return;
+            {
+              g_free (identifier);
+              return;
+            }
 
           if (! substring)
             {
               substring = strstr (identifier, "list");
 
               if (substring && view_type == GIMP_VIEW_TYPE_LIST)
-                return;
+                {
+                  g_free (identifier);
+                  return;
+                }
             }
 
           if (substring)
             {
               GimpContainerView *old_view;
               GtkWidget         *new_dockable;
+              GimpDock          *dock;
               gint               view_size = -1;
 
               if (view_type == GIMP_VIEW_TYPE_LIST)
@@ -177,12 +183,11 @@ dockable_toggle_view_cmd_callback (GtkAction *action,
               if (old_view)
                 view_size = gimp_container_view_get_view_size (old_view, NULL);
 
-
-              new_dockable =
-                gimp_dialog_factory_dockable_new (gimp_dock_get_dialog_factory (dockbook->dock),
-                                                  dockbook->dock,
-                                                  identifier,
-                                                  view_size);
+              dock         = gimp_dockbook_get_dock (dockbook);
+              new_dockable = gimp_dialog_factory_dockable_new (gimp_dock_get_dialog_factory (dock),
+                                                               dock,
+                                                               identifier,
+                                                               view_size);
 
               if (new_dockable)
                 {
@@ -198,21 +203,21 @@ dockable_toggle_view_cmd_callback (GtkAction *action,
 
                   show = gimp_docked_get_show_button_bar (old);
                   gimp_docked_set_show_button_bar (new, show);
-                }
 
-              /*  Maybe gimp_dialog_factory_dockable_new() returned
-               *  an already existing singleton dockable, so check
-               *  if it already is attached to a dockbook.
-               */
-              if (new_dockable && ! GIMP_DOCKABLE (new_dockable)->dockbook)
-                {
-                  gimp_dockbook_add (dockbook, GIMP_DOCKABLE (new_dockable),
-                                     page_num);
+                  /*  Maybe gimp_dialog_factory_dockable_new() returned
+                   *  an already existing singleton dockable, so check
+                   *  if it already is attached to a dockbook.
+                   */
+                  if (! gimp_dockable_get_dockbook (GIMP_DOCKABLE (new_dockable)))
+                    {
+                      gimp_dockbook_add (dockbook, GIMP_DOCKABLE (new_dockable),
+                                         page_num);
 
-                  gimp_dockbook_remove (dockbook, dockable);
+                      gimp_dockbook_remove (dockbook, dockable);
 
-                  gtk_notebook_set_current_page (GTK_NOTEBOOK (dockbook),
-                                                 page_num);
+                      gtk_notebook_set_current_page (GTK_NOTEBOOK (dockbook),
+                                                     page_num);
+                    }
                 }
             }
 
@@ -261,7 +266,7 @@ dockable_tab_style_cmd_callback (GtkAction *action,
   tab_style = (GimpTabStyle)
     gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
 
-  if (dockable && dockable->tab_style != tab_style)
+  if (dockable && gimp_dockable_get_tab_style (dockable) != tab_style)
     {
       GtkWidget *tab_widget;
 
