@@ -263,6 +263,8 @@ edit_actions_update (GimpActionGroup *group,
   gchar        *undo_name    = NULL;
   gchar        *redo_name    = NULL;
   gchar        *fade_name    = NULL;
+  gboolean      writable     = FALSE;
+  gboolean      children     = FALSE;
   gboolean      undo_enabled = FALSE;
   gboolean      fade_enabled = FALSE;
 
@@ -270,25 +272,35 @@ edit_actions_update (GimpActionGroup *group,
     {
       drawable = gimp_image_get_active_drawable (image);
 
+      if (drawable)
+        {
+          writable = ! gimp_item_is_content_locked (GIMP_ITEM (drawable));
+
+          if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+            children = TRUE;
+        }
+
       undo_enabled = gimp_image_undo_is_enabled (image);
 
       if (undo_enabled)
         {
-          GimpUndo *undo = gimp_undo_stack_peek (image->undo_stack);
-          GimpUndo *redo = gimp_undo_stack_peek (image->redo_stack);
+          GimpUndoStack *undo_stack = gimp_image_get_undo_stack (image);
+          GimpUndoStack *redo_stack = gimp_image_get_redo_stack (image);
+          GimpUndo      *undo       = gimp_undo_stack_peek (undo_stack);
+          GimpUndo      *redo       = gimp_undo_stack_peek (redo_stack);
 
           if (undo)
             {
               undo_name =
                 g_strdup_printf (_("_Undo %s"),
-                                 gimp_object_get_name (GIMP_OBJECT (undo)));
+                                 gimp_object_get_name (undo));
             }
 
           if (redo)
             {
               redo_name =
                 g_strdup_printf (_("_Redo %s"),
-                                 gimp_object_get_name (GIMP_OBJECT (redo)));
+                                 gimp_object_get_name (redo));
             }
 
           undo = gimp_image_undo_get_fadeable (image);
@@ -303,7 +315,7 @@ edit_actions_update (GimpActionGroup *group,
             {
               fade_name =
                 g_strdup_printf (_("_Fade %s..."),
-                                 gimp_object_get_name (GIMP_OBJECT (undo)));
+                                 gimp_object_get_name (undo));
             }
         }
     }
@@ -329,22 +341,25 @@ edit_actions_update (GimpActionGroup *group,
   g_free (redo_name);
   g_free (fade_name);
 
-  SET_SENSITIVE ("edit-cut",                drawable);
+  SET_SENSITIVE ("edit-cut",                writable && !children);
   SET_SENSITIVE ("edit-copy",               drawable);
   SET_SENSITIVE ("edit-copy-visible",       image);
-  /*             "edit-paste" is always enabled  */
+  SET_SENSITIVE ("edit-paste",              !image || (!drawable ||
+                                                       (writable && !children)));
   SET_SENSITIVE ("edit-paste-as-new-layer", image);
-  SET_SENSITIVE ("edit-paste-into",         image);
+  SET_SENSITIVE ("edit-paste-into",         image && (!drawable ||
+                                                      (writable  && !children)));
 
-  SET_SENSITIVE ("edit-named-cut",          drawable);
+  SET_SENSITIVE ("edit-named-cut",          writable && !children);
   SET_SENSITIVE ("edit-named-copy",         drawable);
   SET_SENSITIVE ("edit-named-copy-visible", drawable);
-  SET_SENSITIVE ("edit-named-paste",        image);
+  SET_SENSITIVE ("edit-named-paste",        image && (!drawable ||
+                                                      (writable && !children)));
 
-  SET_SENSITIVE ("edit-clear",              drawable);
-  SET_SENSITIVE ("edit-fill-fg",            drawable);
-  SET_SENSITIVE ("edit-fill-bg",            drawable);
-  SET_SENSITIVE ("edit-fill-pattern",       drawable);
+  SET_SENSITIVE ("edit-clear",              writable && !children);
+  SET_SENSITIVE ("edit-fill-fg",            writable && !children);
+  SET_SENSITIVE ("edit-fill-bg",            writable && !children);
+  SET_SENSITIVE ("edit-fill-pattern",       writable && !children);
 
 #undef SET_LABEL
 #undef SET_SENSITIVE
