@@ -26,6 +26,7 @@
 
 #include "core/gimpimage.h"
 #include "core/gimplayer.h"
+#include "core/gimplayer-floating-sel.h"
 #include "core/gimplayermask.h"
 
 #include "text/gimptextlayer.h"
@@ -98,8 +99,8 @@ static const GimpActionEntry layers_actions[] =
     GIMP_HELP_LAYER_NEW_FROM_VISIBLE },
 
   { "layers-new-group", GTK_STOCK_DIRECTORY,
-    NC_("layers-action", "_New Group Layer..."), NULL,
-    NC_("layers-action", "Create a new group layer and add it to the image"),
+    NC_("layers-action", "New Layer _Group..."), NULL,
+    NC_("layers-action", "Create a new layer group and add it to the image"),
     G_CALLBACK (layers_new_group_cmd_callback),
     GIMP_HELP_LAYER_NEW },
 
@@ -148,10 +149,16 @@ static const GimpActionEntry layers_actions[] =
 
   { "layers-merge-down", GIMP_STOCK_MERGE_DOWN,
     NC_("layers-action", "Merge Do_wn"), NULL,
-    NC_("layers-action", "Merge this layer with the one below it"),
+    NC_("layers-action", "Merge this layer with the first visible layer below it"),
     G_CALLBACK (layers_merge_down_cmd_callback),
     GIMP_HELP_LAYER_MERGE_DOWN },
 
+  { "layers-merge-group", NULL,
+    NC_("layers-action", "Merge Layer Group"), NULL,
+    NC_("layers-action", "Merge the layer group's layers into one normal layer"),
+    G_CALLBACK (layers_merge_group_cmd_callback),
+    GIMP_HELP_LAYER_MERGE_GROUP },
+
   { "layers-merge-layers", NULL,
     NC_("layers-action", "Merge _Visible Layers..."), NULL,
     NC_("layers-action", "Merge all visible layers into one layer"),
@@ -491,18 +498,22 @@ void
 layers_actions_update (GimpActionGroup *group,
                        gpointer         data)
 {
-  GimpImage     *image      = action_data_get_image (data);
-  GimpLayer     *layer      = NULL;
-  GimpLayerMask *mask       = NULL;     /*  layer mask             */
-  gboolean       fs         = FALSE;    /*  floating sel           */
-  gboolean       ac         = FALSE;    /*  active channel         */
-  gboolean       sel        = FALSE;
-  gboolean       alpha      = FALSE;    /*  alpha channel present  */
-  gboolean       indexed    = FALSE;    /*  is indexed             */
-  gboolean       lock_alpha = FALSE;
-  gboolean       text_layer = FALSE;
-  GList         *next       = NULL;
-  GList         *prev       = NULL;
+  GimpImage     *image          = action_data_get_image (data);
+  GimpLayer     *layer          = NULL;
+  GimpLayerMask *mask           = NULL;     /*  layer mask             */
+  gboolean       fs             = FALSE;    /*  floating sel           */
+  gboolean       ac             = FALSE;    /*  active channel         */
+  gboolean       sel            = FALSE;
+  gboolean       alpha          = FALSE;    /*  alpha channel present  */
+  gboolean       indexed        = FALSE;    /*  is indexed             */
+  gboolean       lock_alpha     = FALSE;
+  gboolean       can_lock_alpha = FALSE;
+  gboolean       text_layer     = FALSE;
+  gboolean       writable       = FALSE;
+  gboolean       children       = FALSE;
+  GList         *next           = NULL;
+  GList         *next_visible   = NULL;
+  GList         *prev           = NULL;
 
   if (image)
     {
@@ -518,9 +529,14 @@ layers_actions_update (GimpActionGroup *group,
           GList *layer_list;
           GList *list;
 
-          mask       = gimp_layer_get_mask (layer);
-          lock_alpha = gimp_layer_get_lock_alpha (layer);
-          alpha      = gimp_drawable_has_alpha (GIMP_DRAWABLE (layer));
+          mask           = gimp_layer_get_mask (layer);
+          lock_alpha     = gimp_layer_get_lock_alpha (layer);
+          can_lock_alpha = gimp_layer_can_lock_alpha (layer);
+          alpha          = gimp_drawable_has_alpha (GIMP_DRAWABLE (layer));
+          writable       = ! gimp_item_is_content_locked (GIMP_ITEM (layer));
+
+          if (gimp_viewable_get_children (GIMP_VIEWABLE (layer)))
+            children = TRUE;
 
           layer_list = gimp_item_get_container_iter (GIMP_ITEM (layer));
 
@@ -530,10 +546,26 @@ layers_actions_update (GimpActionGroup *group,
             {
               prev = g_list_previous (list);
               next = g_list_next (list);
+
+              for (next_visible = next;
+                   next_visible;
+                   next_visible = g_list_next (next_visible))
+                {
+                  if (gimp_item_get_visible (next_visible->data))
+                    {
+                      /*  "next_visible" is actually "next_visible" and
+                       *  "writable" and "not group"
+                       */
+                      if (gimp_item_is_content_locked (next_visible->data) ||
+                          gimp_viewable_get_children (next_visible->data))
+                        next_visible = NULL;
+
+                      break;
+                    }
+                }
             }
 
-          if (layer)
-            text_layer = gimp_drawable_is_text_layer (GIMP_DRAWABLE (layer));
+          text_layer = gimp_drawable_is_text_layer (GIMP_DRAWABLE (layer));
         }
     }
 
@@ -543,14 +575,27 @@ layers_actions_update (GimpActionGroup *group,
         gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
 #define SET_ACTIVE(action,condition) \
         gimp_action_group_set_action_active (group, action, (condition) != 0)
+#define SET_LABEL(action,label) \
+        gimp_action_group_set_action_label (group, action, label)
 
   SET_VISIBLE   ("layers-text-tool",        text_layer && !ac);
   SET_SENSITIVE ("layers-edit-attributes",  layer && !fs && !ac);
 
+  if (layer && gimp_layer_is_floating_sel (layer))
+    {
+      SET_LABEL ("layers-new",             C_("layers-action", "To _New Layer"));
+      SET_LABEL ("layers-new-last-values", C_("layers-action", "To _New Layer"));
+    }
+  else
+    {
+      SET_LABEL ("layers-new",             C_("layers-action", "_New Layer..."));
+      SET_LABEL ("layers-new-last-values", C_("layers-action", "_New Layer"));
+    }
+
   SET_SENSITIVE ("layers-new",              image);
   SET_SENSITIVE ("layers-new-last-values",  image);
   SET_SENSITIVE ("layers-new-from-visible", image);
-  SET_SENSITIVE ("layers-new-group",        image);
+  SET_SENSITIVE ("layers-new-group",        image && !indexed);
   SET_SENSITIVE ("layers-duplicate",        layer && !fs && !ac);
   SET_SENSITIVE ("layers-delete",           layer && !ac);
 
@@ -565,7 +610,9 @@ layers_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("layers-lower-to-bottom",  layer && !fs && !ac && next);
 
   SET_SENSITIVE ("layers-anchor",           layer &&  fs && !ac);
-  SET_SENSITIVE ("layers-merge-down",       layer && !fs && !ac && next);
+  SET_SENSITIVE ("layers-merge-down",       layer && !fs && !ac && next_visible);
+  SET_VISIBLE   ("layers-merge-group",      children);
+  SET_SENSITIVE ("layers-merge-group",      layer && !fs && !ac && children);
   SET_SENSITIVE ("layers-merge-layers",     layer && !fs && !ac);
   SET_SENSITIVE ("layers-flatten-image",    layer && !fs && !ac);
 
@@ -577,21 +624,21 @@ layers_actions_update (GimpActionGroup *group,
   SET_VISIBLE   ("layers-text-selection-subtract",  text_layer && !ac);
   SET_VISIBLE   ("layers-text-selection-intersect", text_layer && !ac);
 
-  SET_SENSITIVE ("layers-resize",          layer && !ac);
-  SET_SENSITIVE ("layers-resize-to-image", layer && !ac);
-  SET_SENSITIVE ("layers-scale",           layer && !ac);
+  SET_SENSITIVE ("layers-resize",          writable && !ac);
+  SET_SENSITIVE ("layers-resize-to-image", writable && !ac);
+  SET_SENSITIVE ("layers-scale",           writable && !ac);
 
-  SET_SENSITIVE ("layers-crop",            layer && sel);
+  SET_SENSITIVE ("layers-crop",            writable && sel);
 
-  SET_SENSITIVE ("layers-alpha-add",       layer && !fs && !alpha);
-  SET_SENSITIVE ("layers-alpha-remove",    layer && !fs &&  alpha);
+  SET_SENSITIVE ("layers-alpha-add",       writable && !children && !fs && !alpha);
+  SET_SENSITIVE ("layers-alpha-remove",    writable && !children && !fs &&  alpha);
 
-  SET_SENSITIVE ("layers-lock-alpha", layer);
+  SET_SENSITIVE ("layers-lock-alpha", can_lock_alpha);
   SET_ACTIVE    ("layers-lock-alpha", lock_alpha);
 
-  SET_SENSITIVE ("layers-mask-add",    layer && !fs && !ac && !mask);
-  SET_SENSITIVE ("layers-mask-apply",  layer && !fs && !ac &&  mask);
-  SET_SENSITIVE ("layers-mask-delete", layer && !fs && !ac &&  mask);
+  SET_SENSITIVE ("layers-mask-add",    layer    && !fs && !ac && !mask && !children);
+  SET_SENSITIVE ("layers-mask-apply",  writable && !fs && !ac &&  mask && !children);
+  SET_SENSITIVE ("layers-mask-delete", layer    && !fs && !ac &&  mask);
 
   SET_SENSITIVE ("layers-mask-edit",    layer && !fs && !ac &&  mask);
   SET_SENSITIVE ("layers-mask-show",    layer && !fs && !ac &&  mask);
@@ -614,4 +661,5 @@ layers_actions_update (GimpActionGroup *group,
 #undef SET_VISIBLE
 #undef SET_SENSITIVE
 #undef SET_ACTIVE
+#undef SET_LABEL
 }
