@@ -59,6 +59,7 @@
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpimagewindow.h"
 
 #include "tools/gimptexttool.h"
 #include "tools/tool_manager.h"
@@ -198,7 +199,7 @@ layers_edit_attributes_cmd_callback (GtkAction *action,
                                      layer,
                                      action_data_get_context (data),
                                      widget,
-                                     gimp_object_get_name (GIMP_OBJECT (layer)),
+                                     gimp_object_get_name (layer),
                                      layer_fill_type,
                                      _("Layer Attributes"),
                                      "gimp-layer-edit",
@@ -247,7 +248,7 @@ layers_new_cmd_callback (GtkAction *action,
   dialog = layer_options_dialog_new (image, NULL,
                                      action_data_get_context (data),
                                      widget,
-                                     layer_name ? layer_name : _("New Layer"),
+                                     layer_name ? layer_name : _("Layer"),
                                      layer_fill_type,
                                      _("New Layer"),
                                      "gimp-layer-new",
@@ -321,7 +322,7 @@ layers_new_last_vals_cmd_callback (GtkAction *action,
 
   new_layer = gimp_layer_new (image, width, height,
                               gimp_image_base_type_with_alpha (image),
-                              layer_name ? layer_name : _("New Layer"),
+                              layer_name,
                               opacity, mode);
 
   gimp_drawable_fill_by_type (GIMP_DRAWABLE (new_layer),
@@ -348,6 +349,8 @@ layers_new_from_visible_cmd_callback (GtkAction *action,
 
   pickable = GIMP_PICKABLE (gimp_image_get_projection (image));
 
+  gimp_pickable_flush (pickable);
+
   layer = gimp_layer_new_from_tiles (gimp_pickable_get_tiles (pickable),
                                      image,
                                      gimp_image_base_type_with_alpha (image),
@@ -460,7 +463,6 @@ layers_duplicate_cmd_callback (GtkAction *action,
   GimpImage *image;
   GimpLayer *layer;
   GimpLayer *new_layer;
-  GimpLayer *parent;
   return_if_no_layer (image, layer, data);
 
   new_layer = GIMP_LAYER (gimp_item_duplicate (GIMP_ITEM (layer),
@@ -470,9 +472,9 @@ layers_duplicate_cmd_callback (GtkAction *action,
    *  the latter would add a duplicated group inside itself instead of
    *  above it
    */
-  parent = GIMP_LAYER (gimp_viewable_get_parent (GIMP_VIEWABLE (layer)));
-
-  gimp_image_add_layer (image, new_layer, parent, -1, TRUE);
+  gimp_image_add_layer (image, new_layer,
+                        gimp_layer_get_parent (layer), -1,
+                        TRUE);
 
   gimp_image_flush (image);
 }
@@ -501,7 +503,19 @@ layers_merge_down_cmd_callback (GtkAction *action,
   return_if_no_layer (image, layer, data);
 
   gimp_image_merge_down (image, layer, action_data_get_context (data),
-                         GIMP_EXPAND_AS_NECESSARY);
+                         GIMP_EXPAND_AS_NECESSARY, NULL);
+  gimp_image_flush (image);
+}
+
+void
+layers_merge_group_cmd_callback (GtkAction *action,
+                                 gpointer   data)
+{
+  GimpImage *image;
+  GimpLayer *layer;
+  return_if_no_layer (image, layer, data);
+
+  gimp_image_merge_group_layer (image, GIMP_GROUP_LAYER (layer));
   gimp_image_flush (image);
 }
 
@@ -586,15 +600,19 @@ void
 layers_resize_cmd_callback (GtkAction *action,
                             gpointer   data)
 {
-  GimpImage *image;
-  GimpLayer *layer;
-  GtkWidget *widget;
-  GtkWidget *dialog;
+  GimpDisplay *display = NULL;
+  GimpImage   *image;
+  GimpLayer   *layer;
+  GtkWidget   *widget;
+  GtkWidget   *dialog;
   return_if_no_layer (image, layer, data);
   return_if_no_widget (widget, data);
 
-  if (layer_resize_unit != GIMP_UNIT_PERCENT && GIMP_IS_DISPLAY (data))
-    layer_resize_unit = GIMP_DISPLAY_SHELL (GIMP_DISPLAY (data)->shell)->unit;
+  if (GIMP_IS_IMAGE_WINDOW (data))
+    display = action_data_get_display (data);
+
+  if (layer_resize_unit != GIMP_UNIT_PERCENT && display)
+    layer_resize_unit = gimp_display_get_shell (display)->unit;
 
   dialog = resize_dialog_new (GIMP_VIEWABLE (layer),
                               action_data_get_context (data),
@@ -624,15 +642,19 @@ void
 layers_scale_cmd_callback (GtkAction *action,
                            gpointer   data)
 {
-  GimpImage *image;
-  GimpLayer *layer;
-  GtkWidget *widget;
-  GtkWidget *dialog;
+  GimpDisplay *display = NULL;
+  GimpImage   *image;
+  GimpLayer   *layer;
+  GtkWidget   *widget;
+  GtkWidget   *dialog;
   return_if_no_layer (image, layer, data);
   return_if_no_widget (widget, data);
 
-  if (layer_scale_unit != GIMP_UNIT_PERCENT && GIMP_IS_DISPLAY (data))
-    layer_scale_unit = GIMP_DISPLAY_SHELL (GIMP_DISPLAY (data)->shell)->unit;
+  if (GIMP_IS_IMAGE_WINDOW (data))
+    display = action_data_get_display (data);
+
+  if (layer_scale_unit != GIMP_UNIT_PERCENT && display)
+    layer_scale_unit = gimp_display_get_shell (display)->unit;
 
   if (layer_scale_interp == -1)
     layer_scale_interp = image->gimp->config->interpolation_type;
@@ -645,7 +667,7 @@ layers_scale_cmd_callback (GtkAction *action,
                              layer_scale_unit,
                              layer_scale_interp,
                              layers_scale_layer_callback,
-                             GIMP_IS_DISPLAY (data) ? data : NULL);
+                             display);
 
   gtk_widget_show (dialog);
 }
@@ -968,6 +990,7 @@ layers_new_layer_response (GtkWidget          *widget,
 
       if (layer_name)
         g_free (layer_name);
+
       layer_name =
         g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->name_entry)));
 
@@ -1019,7 +1042,7 @@ layers_edit_layer_response (GtkWidget          *widget,
 
       new_name = gtk_entry_get_text (GTK_ENTRY (dialog->name_entry));
 
-      if (strcmp (new_name, gimp_object_get_name (GIMP_OBJECT (layer))))
+      if (strcmp (new_name, gimp_object_get_name (layer)))
         {
           GError *error = NULL;
 
