@@ -38,7 +38,6 @@
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
 
-#include "dialogs/dialogs.h"
 #include "dialogs/stroke-dialog.h"
 
 #include "actions.h"
@@ -144,24 +143,27 @@ select_feather_cmd_callback (GtkAction *action,
                              gpointer   data)
 {
   GimpDisplay *display;
+  GimpImage   *image;
   GtkWidget   *dialog;
   gdouble      xres;
   gdouble      yres;
   return_if_no_display (display, data);
 
-  gimp_image_get_resolution (display->image, &xres, &yres);
+  image = gimp_display_get_image (display);
+
+  gimp_image_get_resolution (image, &xres, &yres);
 
   dialog = gimp_query_size_box (_("Feather Selection"),
-                                display->shell,
+                                GTK_WIDGET (gimp_display_get_shell (display)),
                                 gimp_standard_help_func,
                                 GIMP_HELP_SELECTION_FEATHER,
                                 _("Feather selection by"),
                                 select_feather_radius, 0, 32767, 3,
-                                GIMP_DISPLAY_SHELL (display->shell)->unit,
+                                gimp_display_get_shell (display)->unit,
                                 MIN (xres, yres),
                                 FALSE,
-                                G_OBJECT (display->image), "disconnect",
-                                select_feather_callback, display->image);
+                                G_OBJECT (image), "disconnect",
+                                select_feather_callback, image);
   gtk_widget_show (dialog);
 }
 
@@ -181,25 +183,28 @@ select_shrink_cmd_callback (GtkAction *action,
                             gpointer   data)
 {
   GimpDisplay *display;
+  GimpImage   *image;
   GtkWidget   *dialog;
   GtkWidget   *button;
   gdouble      xres;
   gdouble      yres;
   return_if_no_display (display, data);
 
-  gimp_image_get_resolution (display->image, &xres, &yres);
+  image = gimp_display_get_image (display);
+
+  gimp_image_get_resolution (image, &xres, &yres);
 
   dialog = gimp_query_size_box (_("Shrink Selection"),
-                                display->shell,
+                                GTK_WIDGET (gimp_display_get_shell (display)),
                                 gimp_standard_help_func,
                                 GIMP_HELP_SELECTION_SHRINK,
                                 _("Shrink selection by"),
                                 select_shrink_pixels, 1, 32767, 0,
-                                GIMP_DISPLAY_SHELL (display->shell)->unit,
+                                gimp_display_get_shell (display)->unit,
                                 MIN (xres, yres),
                                 FALSE,
-                                G_OBJECT (display->image), "disconnect",
-                                select_shrink_callback, display->image);
+                                G_OBJECT (image), "disconnect",
+                                select_shrink_callback, image);
 
   button = gtk_check_button_new_with_mnemonic (_("_Shrink from image border"));
 
@@ -219,24 +224,27 @@ select_grow_cmd_callback (GtkAction *action,
                           gpointer   data)
 {
   GimpDisplay *display;
+  GimpImage   *image;
   GtkWidget   *dialog;
   gdouble      xres;
   gdouble      yres;
   return_if_no_display (display, data);
 
-  gimp_image_get_resolution (display->image, &xres, &yres);
+  image = gimp_display_get_image (display);
+
+  gimp_image_get_resolution (image, &xres, &yres);
 
   dialog = gimp_query_size_box (_("Grow Selection"),
-                                display->shell,
+                                GTK_WIDGET (gimp_display_get_shell (display)),
                                 gimp_standard_help_func,
                                 GIMP_HELP_SELECTION_GROW,
                                 _("Grow selection by"),
                                 select_grow_pixels, 1, 32767, 0,
-                                GIMP_DISPLAY_SHELL (display->shell)->unit,
+                                gimp_display_get_shell (display)->unit,
                                 MIN (xres, yres),
                                 FALSE,
-                                G_OBJECT (display->image), "disconnect",
-                                select_grow_callback, display->image);
+                                G_OBJECT (image), "disconnect",
+                                select_grow_callback, image);
   gtk_widget_show (dialog);
 }
 
@@ -245,25 +253,28 @@ select_border_cmd_callback (GtkAction *action,
                             gpointer   data)
 {
   GimpDisplay *display;
+  GimpImage   *image;
   GtkWidget   *dialog;
   GtkWidget   *button;
   gdouble      xres;
   gdouble      yres;
   return_if_no_display (display, data);
 
-  gimp_image_get_resolution (display->image, &xres, &yres);
+  image = gimp_display_get_image (display);
+
+  gimp_image_get_resolution (image, &xres, &yres);
 
   dialog = gimp_query_size_box (_("Border Selection"),
-                                display->shell,
+                                GTK_WIDGET (gimp_display_get_shell (display)),
                                 gimp_standard_help_func,
                                 GIMP_HELP_SELECTION_BORDER,
                                 _("Border selection by"),
                                 select_border_radius, 1, 32767, 0,
-                                GIMP_DISPLAY_SHELL (display->shell)->unit,
+                                gimp_display_get_shell (display)->unit,
                                 MIN (xres, yres),
                                 FALSE,
-                                G_OBJECT (display->image), "disconnect",
-                                select_border_callback, display->image);
+                                G_OBJECT (image), "disconnect",
+                                select_border_callback, image);
 
   /* Feather button */
   button = gtk_check_button_new_with_mnemonic (_("_Feather border"));
@@ -307,7 +318,7 @@ select_save_cmd_callback (GtkAction *action,
   gimp_selection_save (GIMP_SELECTION (gimp_image_get_mask (image)));
   gimp_image_flush (image);
 
-  gimp_dialog_factory_dialog_raise (global_dock_factory,
+  gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
                                     gtk_widget_get_screen (widget),
                                     "gimp-channel-list", -1);
 }
