@@ -134,7 +134,7 @@ vectors_edit_attributes_cmd_callback (GtkAction *action,
   options = vectors_options_dialog_new (image, vectors,
                                         action_data_get_context (data),
                                         widget,
-                                        gimp_object_get_name (GIMP_OBJECT (vectors)),
+                                        gimp_object_get_name (vectors),
                                         _("Path Attributes"),
                                         "gimp-vectors-edit",
                                         GTK_STOCK_EDIT,
@@ -162,7 +162,7 @@ vectors_new_cmd_callback (GtkAction *action,
                                         action_data_get_context (data),
                                         widget,
                                         vectors_name ? vectors_name :
-                                        _("New Path"),
+                                        _("Path"),
                                         _("New Path"),
                                         "gimp-vectors-new",
                                         GIMP_STOCK_PATH,
@@ -184,8 +184,7 @@ vectors_new_last_vals_cmd_callback (GtkAction *action,
   GimpVectors *new_vectors;
   return_if_no_image (image, data);
 
-  new_vectors = gimp_vectors_new (image,
-                                  vectors_name ? vectors_name : _("New Path"));
+  new_vectors = gimp_vectors_new (image, vectors_name);
 
   gimp_image_add_vectors (image, new_vectors,
                           GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
@@ -248,7 +247,6 @@ vectors_duplicate_cmd_callback (GtkAction *action,
   GimpImage   *image;
   GimpVectors *vectors;
   GimpVectors *new_vectors;
-  GimpVectors *parent;
   return_if_no_vectors (image, vectors, data);
 
   new_vectors = GIMP_VECTORS (gimp_item_duplicate (GIMP_ITEM (vectors),
@@ -258,9 +256,9 @@ vectors_duplicate_cmd_callback (GtkAction *action,
    *  the latter would add a duplicated group inside itself instead of
    *  above it
    */
-  parent = GIMP_VECTORS (gimp_viewable_get_parent (GIMP_VIEWABLE (vectors)));
-
-  gimp_image_add_vectors (image, new_vectors, parent, -1, TRUE);
+  gimp_image_add_vectors (image, new_vectors,
+                          gimp_vectors_get_parent (vectors), -1,
+                          TRUE);
 
   gimp_image_flush (image);
 }
@@ -614,6 +612,37 @@ vectors_linked_cmd_callback (GtkAction *action,
     }
 }
 
+void
+vectors_lock_content_cmd_callback (GtkAction *action,
+                                   gpointer   data)
+{
+  GimpImage   *image;
+  GimpVectors *vectors;
+  gboolean     locked;
+  return_if_no_vectors (image, vectors, data);
+
+  locked = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+
+  if (locked != gimp_item_get_lock_content (GIMP_ITEM (vectors)))
+    {
+#if 0
+      GimpUndo *undo;
+#endif
+      gboolean  push_undo = TRUE;
+
+#if 0
+      undo = gimp_image_undo_can_compress (image, GIMP_TYPE_ITEM_UNDO,
+                                           GIMP_UNDO_ITEM_LINKED);
+
+      if (undo && GIMP_ITEM_UNDO (undo)->item == GIMP_ITEM (vectors))
+        push_undo = FALSE;
+#endif
+
+      gimp_item_set_lock_content (GIMP_ITEM (vectors), locked, push_undo);
+      gimp_image_flush (image);
+    }
+}
+
 
 /*  private functions  */
 
@@ -655,7 +684,7 @@ vectors_edit_vectors_response (GtkWidget            *widget,
 
       new_name = gtk_entry_get_text (GTK_ENTRY (options->name_entry));
 
-      if (strcmp (new_name, gimp_object_get_name (GIMP_OBJECT (vectors))))
+      if (strcmp (new_name, gimp_object_get_name (vectors)))
         {
           gimp_item_rename (GIMP_ITEM (vectors), new_name, NULL);
           gimp_image_flush (options->image);
