@@ -26,6 +26,7 @@
 
 #include "actions-types.h"
 
+#include "config/gimpdisplayoptions.h"
 #include "config/gimpguiconfig.h"
 
 #include "core/gimp.h"
@@ -39,11 +40,11 @@
 #include "widgets/gimpwidgets-utils.h"
 
 #include "display/gimpdisplay.h"
-#include "display/gimpdisplayoptions.h"
 #include "display/gimpdisplayshell.h"
 #include "display/gimpdisplayshell-appearance.h"
 #include "display/gimpdisplayshell-scale.h"
 #include "display/gimpdisplayshell-selection.h"
+#include "display/gimpimagewindow.h"
 
 #include "actions.h"
 #include "view-actions.h"
@@ -85,7 +86,7 @@ static const GimpActionEntry view_actions[] =
     GIMP_HELP_FILE_CLOSE },
 
   { "view-zoom-fit-in", GTK_STOCK_ZOOM_FIT,
-    NC_("view-action", "_Fit Image in Window"), "<shift><control>R",
+    NC_("view-action", "_Fit Image in Window"), "<shift><control>J",
     NC_("view-action", "Adjust the zoom ratio so that the image becomes fully visible"),
     G_CALLBACK (view_zoom_fit_in_cmd_callback),
     GIMP_HELP_VIEW_ZOOM_FIT_IN },
@@ -115,7 +116,7 @@ static const GimpActionEntry view_actions[] =
     GIMP_HELP_DISPLAY_FILTER_DIALOG },
 
   { "view-shrink-wrap", GTK_STOCK_ZOOM_FIT,
-    NC_("view-action", "Shrink _Wrap"), "<control>R",
+    NC_("view-action", "Shrink _Wrap"), "<control>J",
     NC_("view-action", "Reduce the image window to the size of the image display"),
     G_CALLBACK (view_shrink_wrap_cmd_callback),
     GIMP_HELP_VIEW_SHRINK_WRAP },
@@ -523,7 +524,7 @@ view_actions_setup (GimpActionGroup *group)
   view_actions_check_type_notify (GIMP_DISPLAY_CONFIG (group->gimp->config),
                                   NULL, group);
 
-  if (GIMP_IS_DISPLAY (group->user_data) ||
+  if (GIMP_IS_IMAGE_WINDOW (group->user_data) ||
       GIMP_IS_GIMP (group->user_data))
     {
       /*  add window actions only if the context of the group is
@@ -546,19 +547,27 @@ view_actions_update (GimpActionGroup *group,
   gchar              *label          = NULL;
   gboolean            fullscreen     = FALSE;
   gboolean            revert_enabled = FALSE;   /* able to revert zoom? */
+  gboolean            use_gegl       = FALSE;
 
   if (display)
     {
-      image = display->image;
-      shell = GIMP_DISPLAY_SHELL (display->shell);
+      GimpImageWindow *window;
 
-      fullscreen = gimp_display_shell_get_fullscreen (shell);
+      image  = gimp_display_get_image (display);
+      shell  = gimp_display_get_shell (display);
+      window = gimp_display_shell_get_window (shell);
+
+      if (window)
+        fullscreen = gimp_image_window_get_fullscreen (window);
 
       options = (image ?
                  (fullscreen ? shell->fullscreen_options : shell->options) :
                  shell->no_image_options);
 
       revert_enabled = gimp_display_shell_scale_can_revert (shell);
+
+      if (image)
+        use_gegl = gimp_image_get_projection (image)->use_gegl;
     }
 
 #define SET_ACTIVE(action,condition) \
@@ -665,13 +674,19 @@ view_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("view-shrink-wrap", image);
   SET_SENSITIVE ("view-fullscreen",  image);
   SET_ACTIVE    ("view-fullscreen",  display && fullscreen);
-  SET_ACTIVE    ("view-use-gegl",    image && display->image->projection->use_gegl);
+  SET_ACTIVE    ("view-use-gegl",    use_gegl);
 
-  if (GIMP_IS_DISPLAY (group->user_data) ||
+  if (GIMP_IS_IMAGE_WINDOW (group->user_data) ||
       GIMP_IS_GIMP (group->user_data))
     {
+      GtkWidget *window = NULL;
+
+      if (shell)
+        window = gtk_widget_get_toplevel (GTK_WIDGET (shell));
+
       /*  see view_actions_setup()  */
-      window_actions_update (group, GTK_WIDGET (shell));
+      if (GTK_IS_WINDOW (window))
+        window_actions_update (group, window);
     }
 
 #undef SET_ACTIVE
