@@ -25,22 +25,24 @@
 
 #include "actions-types.h"
 
+#include "config/gimpdisplayoptions.h"
 #include "config/gimpguiconfig.h"
 
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpimage.h"
+#include "core/gimpgrouplayer.h"
 #include "core/gimpprojection.h"
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplay-foreach.h"
-#include "display/gimpdisplayoptions.h"
 #include "display/gimpdisplayshell.h"
 #include "display/gimpdisplayshell-appearance.h"
 #include "display/gimpdisplayshell-filter-dialog.h"
 #include "display/gimpdisplayshell-scale.h"
 #include "display/gimpdisplayshell-scale-dialog.h"
 #include "display/gimpdisplayshell-scroll.h"
+#include "display/gimpimagewindow.h"
 
 #include "widgets/gimpactiongroup.h"
 #include "widgets/gimpcolordialog.h"
@@ -48,8 +50,6 @@
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpuimanager.h"
 
-#include "dialogs/dialogs.h"
-
 #include "actions.h"
 #include "view-commands.h"
 
@@ -63,7 +63,7 @@
 
 #define IS_ACTIVE_DISPLAY(display) \
   ((display) == \
-   gimp_context_get_display (gimp_get_user_context ((display)->image->gimp)))
+   gimp_context_get_display (gimp_get_user_context ((display)->gimp)))
 
 
 /*  local function prototypes  */
@@ -84,10 +84,10 @@ view_new_cmd_callback (GtkAction *action,
   GimpDisplayShell *shell;
   return_if_no_display (display, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
-  gimp_create_display (display->image->gimp,
-                       display->image,
+  gimp_create_display (display->gimp,
+                       gimp_display_get_image (display),
                        shell->unit, gimp_zoom_model_get_factor (shell->zoom));
 }
 
@@ -98,7 +98,7 @@ view_zoom_fit_in_cmd_callback (GtkAction *action,
   GimpDisplay *display;
   return_if_no_display (display, data);
 
-  gimp_display_shell_scale_fit_in (GIMP_DISPLAY_SHELL (display->shell));
+  gimp_display_shell_scale_fit_in (gimp_display_get_shell (display));
 }
 
 void
@@ -108,7 +108,7 @@ view_zoom_fill_cmd_callback (GtkAction *action,
   GimpDisplay *display;
   return_if_no_display (display, data);
 
-  gimp_display_shell_scale_fill (GIMP_DISPLAY_SHELL (display->shell));
+  gimp_display_shell_scale_fill (gimp_display_get_shell (display));
 }
 
 void
@@ -118,7 +118,7 @@ view_zoom_revert_cmd_callback (GtkAction *action,
   GimpDisplay *display;
   return_if_no_display (display, data);
 
-  gimp_display_shell_scale_revert (GIMP_DISPLAY_SHELL (display->shell));
+  gimp_display_shell_scale_revert (gimp_display_get_shell (display));
 }
 
 void
@@ -126,11 +126,8 @@ view_zoom_cmd_callback (GtkAction *action,
                         gint       value,
                         gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   switch ((GimpActionSelectType) value)
     {
@@ -204,12 +201,9 @@ view_zoom_explicit_cmd_callback (GtkAction *action,
                                  GtkAction *current,
                                  gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
   gint              value;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   value = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
 
@@ -227,11 +221,8 @@ void
 view_zoom_other_cmd_callback (GtkAction *action,
                               gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   /* check if we are activated by the user or from
    * view_actions_set_zoom()
@@ -252,16 +243,19 @@ view_dot_for_dot_cmd_callback (GtkAction *action,
   gboolean          active;
   return_if_no_display (display, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
   if (active != shell->dot_for_dot)
     {
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
       gimp_display_shell_scale_set_dot_for_dot (shell, active);
 
-      SET_ACTIVE (shell->menubar_manager, "view-dot-for-dot",
-                  shell->dot_for_dot);
+      if (window)
+        SET_ACTIVE (gimp_image_window_get_ui_manager (window),
+                    "view-dot-for-dot", shell->dot_for_dot);
 
       if (IS_ACTIVE_DISPLAY (display))
         SET_ACTIVE (shell->popup_manager, "view-dot-for-dot",
@@ -274,24 +268,22 @@ view_scroll_horizontal_cmd_callback (GtkAction *action,
                                      gint       value,
                                      gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
   gdouble           offset;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   offset = action_select_value ((GimpActionSelectType) value,
-                                shell->hsbdata->value,
-                                shell->hsbdata->lower,
-                                shell->hsbdata->upper -
-                                shell->hsbdata->page_size,
-                                shell->hsbdata->lower,
+                                gtk_adjustment_get_value (shell->hsbdata),
+                                gtk_adjustment_get_lower (shell->hsbdata),
+                                gtk_adjustment_get_upper (shell->hsbdata) -
+                                gtk_adjustment_get_page_size (shell->hsbdata),
+                                gtk_adjustment_get_lower (shell->hsbdata),
                                 1,
-                                shell->hsbdata->step_increment,
-                                shell->hsbdata->page_increment,
+                                gtk_adjustment_get_step_increment (shell->hsbdata),
+                                gtk_adjustment_get_page_increment (shell->hsbdata),
                                 0,
                                 FALSE);
+
   gtk_adjustment_set_value (shell->hsbdata, offset);
 }
 
@@ -300,24 +292,22 @@ view_scroll_vertical_cmd_callback (GtkAction *action,
                                    gint       value,
                                    gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
   gdouble           offset;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   offset = action_select_value ((GimpActionSelectType) value,
-                                shell->vsbdata->value,
-                                shell->vsbdata->lower,
-                                shell->vsbdata->upper -
-                                shell->vsbdata->page_size,
-                                shell->vsbdata->lower,
+                                gtk_adjustment_get_value (shell->vsbdata),
+                                gtk_adjustment_get_lower (shell->vsbdata),
+                                gtk_adjustment_get_upper (shell->vsbdata) -
+                                gtk_adjustment_get_page_size (shell->vsbdata),
+                                gtk_adjustment_get_lower (shell->vsbdata),
                                 1,
-                                shell->vsbdata->step_increment,
-                                shell->vsbdata->page_increment,
+                                gtk_adjustment_get_step_increment (shell->vsbdata),
+                                gtk_adjustment_get_page_increment (shell->vsbdata),
                                 0,
                                 FALSE);
+
   gtk_adjustment_set_value (shell->vsbdata, offset);
 }
 
@@ -325,14 +315,11 @@ void
 view_navigation_window_cmd_callback (GtkAction *action,
                                      gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  return_if_no_display (display, data);
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  gimp_dialog_factory_dialog_raise (global_dock_factory,
-                                    gtk_widget_get_screen (display->shell),
+  gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
+                                    gtk_widget_get_screen (GTK_WIDGET (shell)),
                                     "gimp-navigation-view", -1);
 }
 
@@ -340,11 +327,8 @@ void
 view_display_filters_cmd_callback (GtkAction *action,
                                    gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   if (! shell->filters_dialog)
     {
@@ -362,14 +346,12 @@ void
 view_toggle_selection_cmd_callback (GtkAction *action,
                                     gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_selection (shell))
+    return;
 
   gimp_display_shell_set_show_selection (shell, active);
 }
@@ -378,14 +360,12 @@ void
 view_toggle_layer_boundary_cmd_callback (GtkAction *action,
                                          gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_layer (shell))
+    return;
 
   gimp_display_shell_set_show_layer (shell, active);
 }
@@ -394,14 +374,12 @@ void
 view_toggle_menubar_cmd_callback (GtkAction *action,
                                   gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_menubar (shell))
+    return;
 
   gimp_display_shell_set_show_menubar (shell, active);
 }
@@ -410,14 +388,12 @@ void
 view_toggle_rulers_cmd_callback (GtkAction *action,
                                  gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_rulers (shell))
+    return;
 
   gimp_display_shell_set_show_rulers (shell, active);
 }
@@ -426,14 +402,12 @@ void
 view_toggle_scrollbars_cmd_callback (GtkAction *action,
                                      gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_scrollbars (shell))
+    return;
 
   gimp_display_shell_set_show_scrollbars (shell, active);
 }
@@ -442,14 +416,12 @@ void
 view_toggle_statusbar_cmd_callback (GtkAction *action,
                                     gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_statusbar (shell))
+    return;
 
   gimp_display_shell_set_show_statusbar (shell, active);
 }
@@ -458,14 +430,12 @@ void
 view_toggle_guides_cmd_callback (GtkAction *action,
                                  gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_guides (shell))
+    return;
 
   gimp_display_shell_set_show_guides (shell, active);
 }
@@ -474,14 +444,12 @@ void
 view_toggle_grid_cmd_callback (GtkAction *action,
                                gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_grid (shell))
+    return;
 
   gimp_display_shell_set_show_grid (shell, active);
 }
@@ -490,14 +458,12 @@ void
 view_toggle_sample_points_cmd_callback (GtkAction *action,
                                         gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_show_sample_points (shell))
+    return;
 
   gimp_display_shell_set_show_sample_points (shell, active);
 }
@@ -506,14 +472,12 @@ void
 view_snap_to_guides_cmd_callback (GtkAction *action,
                                   gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_snap_to_guides (shell))
+    return;
 
   gimp_display_shell_set_snap_to_guides (shell, active);
 }
@@ -522,14 +486,12 @@ void
 view_snap_to_grid_cmd_callback (GtkAction *action,
                                 gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_snap_to_grid (shell))
+    return;
 
   gimp_display_shell_set_snap_to_grid (shell, active);
 }
@@ -538,14 +500,12 @@ void
 view_snap_to_canvas_cmd_callback (GtkAction *action,
                                   gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_snap_to_canvas (shell))
+    return;
 
   gimp_display_shell_set_snap_to_canvas (shell, active);
 }
@@ -554,14 +514,12 @@ void
 view_snap_to_vectors_cmd_callback (GtkAction *action,
                                    gpointer   data)
 {
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
-  return_if_no_display (display, data);
+  gboolean          active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (active == gimp_display_shell_get_snap_to_vectors (shell))
+    return;
 
   gimp_display_shell_set_snap_to_vectors (shell, active);
 }
@@ -572,14 +530,19 @@ view_padding_color_cmd_callback (GtkAction *action,
                                  gpointer   data)
 {
   GimpDisplay        *display;
+  GimpImageWindow    *window;
   GimpDisplayShell   *shell;
   GimpDisplayOptions *options;
   gboolean            fullscreen;
   return_if_no_display (display, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell  = gimp_display_get_shell (display);
+  window = gimp_display_shell_get_window (shell);
 
-  fullscreen = gimp_display_shell_get_fullscreen (shell);
+  if (window)
+    fullscreen = gimp_image_window_get_fullscreen (window);
+  else
+    fullscreen = FALSE;
 
   if (fullscreen)
     options = shell->fullscreen_options;
@@ -608,20 +571,23 @@ view_padding_color_cmd_callback (GtkAction *action,
 
         if (! color_dialog)
           {
+            GimpImage        *image = gimp_display_get_image (display);
+            GimpDisplayShell *shell = gimp_display_get_shell (display);
+
             color_dialog =
-              gimp_color_dialog_new (GIMP_VIEWABLE (display->image),
+              gimp_color_dialog_new (GIMP_VIEWABLE (image),
                                      action_data_get_context (data),
                                      _("Set Canvas Padding Color"),
                                      GTK_STOCK_SELECT_COLOR,
                                      _("Set Custom Canvas Padding Color"),
-                                     display->shell,
+                                     GTK_WIDGET (shell),
                                      NULL, NULL,
                                      &options->padding_color,
                                      FALSE, FALSE);
 
             g_signal_connect (color_dialog, "update",
                               G_CALLBACK (view_padding_color_dialog_update),
-                              display->shell);
+                              shell);
 
             g_object_set_data_full (G_OBJECT (shell), "padding-color-dialog",
                                     color_dialog,
@@ -657,14 +623,10 @@ void
 view_shrink_wrap_cmd_callback (GtkAction *action,
                                gpointer   data)
 {
-  GimpDisplay *display;
   GimpDisplayShell *shell;
-  return_if_no_display (display, data);
+  return_if_no_shell (shell, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
-  gimp_display_shell_scale_shrink_wrap (shell,
-                                        FALSE);
+  gimp_display_shell_scale_shrink_wrap (shell, FALSE);
 }
 
 void
@@ -673,14 +635,20 @@ view_fullscreen_cmd_callback (GtkAction *action,
 {
   GimpDisplay      *display;
   GimpDisplayShell *shell;
-  gboolean          active;
+  GimpImageWindow  *window;
   return_if_no_display (display, data);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell  = gimp_display_get_shell (display);
+  window = gimp_display_shell_get_window (shell);
 
-  active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  if (window)
+    {
+      gboolean active;
 
-  gimp_display_shell_set_fullscreen (shell, active);
+      active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+
+      gimp_image_window_set_fullscreen (window, active);
+    }
 }
 
 void
@@ -688,18 +656,32 @@ view_use_gegl_cmd_callback (GtkAction *action,
                             gpointer   data)
 {
   GimpImage        *image;
-  GimpDisplay      *display;
   GimpDisplayShell *shell;
+  GList            *layers;
+  GList            *list;
   gboolean          active;
   return_if_no_image (image, data);
-  return_if_no_display (display, data);
-
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  return_if_no_shell (shell, data);
 
   active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
-  image->projection->use_gegl = active;
-  gimp_image_update (image, 0, 0, image->width, image->height);
+  gimp_image_get_projection (image)->use_gegl = active;
+
+  layers = gimp_image_get_layer_list (image);
+
+  for (list = layers; list; list = g_list_next (list))
+    {
+      GimpLayer *layer = list->data;
+
+      if (GIMP_IS_GROUP_LAYER (layer))
+        GIMP_GROUP_LAYER (layer)->projection->use_gegl = active;
+    }
+
+  g_list_free (layers);
+
+  gimp_image_invalidate (image, 0, 0,
+                         gimp_image_get_width  (image),
+                         gimp_image_get_height (image));
   gimp_image_flush (image);
 }
 
@@ -712,10 +694,16 @@ view_padding_color_dialog_update (GimpColorDialog      *dialog,
                                   GimpColorDialogState  state,
                                   GimpDisplayShell     *shell)
 {
+  GimpImageWindow    *window;
   GimpDisplayOptions *options;
   gboolean            fullscreen;
 
-  fullscreen = gimp_display_shell_get_fullscreen (shell);
+  window = gimp_display_shell_get_window (shell);
+
+  if (window)
+    fullscreen = gimp_image_window_get_fullscreen (window);
+  else
+    fullscreen = FALSE;
 
   if (fullscreen)
     options = shell->fullscreen_options;
