@@ -41,14 +41,14 @@ window_close_cmd_callback (GtkAction *action,
   GtkWidget *widget;
   return_if_no_widget (widget, data);
 
-  if (! GTK_WIDGET_TOPLEVEL (widget))
+  if (! gtk_widget_is_toplevel (widget))
     widget = gtk_widget_get_toplevel (widget);
 
-  if (widget && widget->window)
+  if (widget && gtk_widget_get_window (widget));
     {
       GdkEvent *event = gdk_event_new (GDK_DELETE);
 
-      event->any.window     = g_object_ref (widget->window);
+      event->any.window     = g_object_ref (gtk_widget_get_window (widget));
       event->any.send_event = TRUE;
 
       gtk_main_do_event (event);
@@ -136,7 +136,7 @@ window_move_to_screen_cmd_callback (GtkAction *action,
   GdkScreen *screen;
   return_if_no_widget (widget, data);
 
-  if (! GTK_WIDGET_TOPLEVEL (widget))
+  if (! gtk_widget_is_toplevel (widget))
     widget = gtk_widget_get_toplevel (widget);
 
   screen = g_object_get_data (G_OBJECT (current), "screen");
