@@ -24,6 +24,9 @@
 
 GType         gimp_tile_manager_get_type     (void) G_GNUC_CONST;
 
+#ifdef GIMP_UNSTABLE
+void          tile_manager_exit              (void);
+#endif
 
 /* Creates a new tile manager with the specified size */
 TileManager * tile_manager_new               (gint width,
@@ -35,6 +38,10 @@ TileManager * tile_manager_new               (gint width,
 TileManager * tile_manager_ref               (TileManager *tm);
 void          tile_manager_unref             (TileManager *tm);
 
+/* Make a copy of the tile manager.
+ */
+TileManager * tile_manager_duplicate         (TileManager *tm);
+
 /* Set the validate procedure for the tile manager.  The validate
  *  procedure is called when an invalid tile is referenced. If the
  *  procedure is NULL, then the tile is set to valid and its memory is
