@@ -37,17 +37,20 @@
 #include "gimp-intl.h"
 
 
-#define DEFAULT_BRUSH     "Round Fuzzy"
-#define DEFAULT_PATTERN   "Pine"
-#define DEFAULT_PALETTE   "Default"
-#define DEFAULT_GRADIENT  "FG to BG (RGB)"
-#define DEFAULT_FONT      "Sans"
-#define DEFAULT_COMMENT   "Created with GIMP"
+#define DEFAULT_BRUSH       "Round Fuzzy"
+#define DEFAULT_DYNAMICS    "Basic Dynamics"
+#define DEFAULT_PATTERN     "Pine"
+#define DEFAULT_PALETTE     "Default"
+#define DEFAULT_GRADIENT    "FG to BG (RGB)"
+#define DEFAULT_TOOL_PRESET "Current Options"
+#define DEFAULT_FONT        "Sans"
+#define DEFAULT_COMMENT     "Created with GIMP"
 
 
 enum
 {
   PROP_0,
+  PROP_LANGUAGE,
   PROP_INTERPOLATION_TYPE,
   PROP_PLUG_IN_PATH,
   PROP_MODULE_PATH,
@@ -55,20 +58,27 @@ enum
   PROP_ENVIRON_PATH,
   PROP_BRUSH_PATH,
   PROP_BRUSH_PATH_WRITABLE,
+  PROP_DYNAMICS_PATH,
+  PROP_DYNAMICS_PATH_WRITABLE,
   PROP_PATTERN_PATH,
   PROP_PATTERN_PATH_WRITABLE,
   PROP_PALETTE_PATH,
   PROP_PALETTE_PATH_WRITABLE,
   PROP_GRADIENT_PATH,
   PROP_GRADIENT_PATH_WRITABLE,
+  PROP_TOOL_PRESET_PATH,
+  PROP_TOOL_PRESET_PATH_WRITABLE,
   PROP_FONT_PATH,
   PROP_FONT_PATH_WRITABLE,
   PROP_DEFAULT_BRUSH,
+  PROP_DEFAULT_DYNAMICS,
   PROP_DEFAULT_PATTERN,
   PROP_DEFAULT_PALETTE,
   PROP_DEFAULT_GRADIENT,
+  PROP_DEFAULT_TOOL_PRESET,
   PROP_DEFAULT_FONT,
   PROP_GLOBAL_BRUSH,
+  PROP_GLOBAL_DYNAMICS,
   PROP_GLOBAL_PATTERN,
   PROP_GLOBAL_PALETTE,
   PROP_GLOBAL_GRADIENT,
@@ -128,12 +138,18 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
   object_class->set_property = gimp_core_config_set_property;
   object_class->get_property = gimp_core_config_get_property;
 
+  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_LANGUAGE,
+                                   "language", LANGUAGE_BLURB,
+                                   NULL,  /* take from environment */
+                                   GIMP_PARAM_STATIC_STRINGS |
+                                   GIMP_CONFIG_PARAM_RESTART);
   GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_INTERPOLATION_TYPE,
                                  "interpolation-type",
                                  INTERPOLATION_TYPE_BLURB,
                                  GIMP_TYPE_INTERPOLATION_TYPE,
                                  GIMP_INTERPOLATION_CUBIC,
                                  GIMP_PARAM_STATIC_STRINGS);
+
   path = gimp_config_build_plug_in_path ("plug-ins");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_PLUG_IN_PATH,
                                  "plug-in-path", PLUG_IN_PATH_BLURB,
@@ -141,6 +157,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_plug_in_path ("modules");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_MODULE_PATH,
                                  "module-path", MODULE_PATH_BLURB,
@@ -148,6 +165,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_plug_in_path ("interpreters");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_INTERPRETER_PATH,
                                  "interpreter-path", INTERPRETER_PATH_BLURB,
@@ -155,6 +173,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_plug_in_path ("environ");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_ENVIRON_PATH,
                                  "environ-path", ENVIRON_PATH_BLURB,
@@ -162,6 +181,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_data_path ("brushes");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_BRUSH_PATH,
                                  "brush-path", BRUSH_PATH_BLURB,
@@ -169,6 +189,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_writable_path ("brushes");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_BRUSH_PATH_WRITABLE,
                                  "brush-path-writable",
@@ -177,6 +198,24 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
+  path = gimp_config_build_data_path ("dynamics");
+  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_DYNAMICS_PATH,
+                                 "dynamics-path", DYNAMICS_PATH_BLURB,
+                                 GIMP_CONFIG_PATH_DIR_LIST, path,
+                                 GIMP_PARAM_STATIC_STRINGS |
+                                 GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
+  path = gimp_config_build_writable_path ("dynamics");
+  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_DYNAMICS_PATH_WRITABLE,
+                                 "dynamics-path-writable",
+                                 DYNAMICS_PATH_WRITABLE_BLURB,
+                                 GIMP_CONFIG_PATH_DIR_LIST, path,
+                                 GIMP_PARAM_STATIC_STRINGS |
+                                 GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
   path = gimp_config_build_data_path ("patterns");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_PATTERN_PATH,
                                  "pattern-path", PATTERN_PATH_BLURB,
@@ -184,6 +223,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_writable_path ("patterns");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_PATTERN_PATH_WRITABLE,
                                  "pattern-path-writable",
@@ -192,6 +232,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_data_path ("palettes");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_PALETTE_PATH,
                                  "palette-path", PALETTE_PATH_BLURB,
@@ -199,6 +240,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_writable_path ("palettes");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_PALETTE_PATH_WRITABLE,
                                  "palette-path-writable",
@@ -207,6 +249,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_data_path ("gradients");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_GRADIENT_PATH,
                                  "gradient-path", GRADIENT_PATH_BLURB,
@@ -214,6 +257,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
   path = gimp_config_build_writable_path ("gradients");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_GRADIENT_PATH_WRITABLE,
                                  "gradient-path-writable",
@@ -222,6 +266,24 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_RESTART);
   g_free (path);
+
+  path = gimp_config_build_data_path ("tool-presets");
+  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_TOOL_PRESET_PATH,
+                                 "tool-preset-path", TOOL_PRESET_PATH_BLURB,
+                                 GIMP_CONFIG_PATH_DIR_LIST, path,
+                                 GIMP_PARAM_STATIC_STRINGS |
+                                 GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
+  path = gimp_config_build_writable_path ("tool-presets");
+  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_TOOL_PRESET_PATH_WRITABLE,
+                                 "tool-preset-path-writable",
+                                 TOOL_PRESET_PATH_WRITABLE_BLURB,
+                                 GIMP_CONFIG_PATH_DIR_LIST, path,
+                                 GIMP_PARAM_STATIC_STRINGS |
+                                 GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
   path = gimp_config_build_data_path ("fonts");
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_FONT_PATH,
                                  "font-path", FONT_PATH_BLURB,
@@ -229,6 +291,7 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                  GIMP_PARAM_STATIC_STRINGS |
                                  GIMP_CONFIG_PARAM_CONFIRM);
   g_free (path);
+
   GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_FONT_PATH_WRITABLE,
                                  "font-path-writable", NULL,
                                  GIMP_CONFIG_PATH_DIR_LIST, NULL,
@@ -238,6 +301,10 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                    "default-brush", DEFAULT_BRUSH_BLURB,
                                    DEFAULT_BRUSH,
                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_DEFAULT_DYNAMICS,
+                                   "default-dynamics", DEFAULT_DYNAMICS_BLURB,
+                                   DEFAULT_DYNAMICS,
+                                   GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_DEFAULT_PATTERN,
                                    "default-pattern", DEFAULT_PATTERN_BLURB,
                                    DEFAULT_PATTERN,
@@ -250,6 +317,10 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                    "default-gradient", DEFAULT_GRADIENT_BLURB,
                                    DEFAULT_GRADIENT,
                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_DEFAULT_TOOL_PRESET,
+                                   "default-tool-preset", DEFAULT_TOOL_PRESET_BLURB,
+                                   DEFAULT_TOOL_PRESET,
+                                   GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_DEFAULT_FONT,
                                    "default-font", DEFAULT_FONT_BLURB,
                                    DEFAULT_FONT,
@@ -258,6 +329,10 @@ gimp_core_config_class_init (GimpCoreConfigClass *klass)
                                     "global-brush", GLOBAL_BRUSH_BLURB,
                                     TRUE,
                                     GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_GLOBAL_DYNAMICS,
+                                    "global-dynamics", GLOBAL_DYNAMICS_BLURB,
+                                    TRUE,
+                                    GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_GLOBAL_PATTERN,
                                     "global-pattern", GLOBAL_PATTERN_BLURB,
                                     TRUE,
@@ -372,13 +447,16 @@ static void
 gimp_core_config_init (GimpCoreConfig *config)
 {
   config->default_image = g_object_new (GIMP_TYPE_TEMPLATE,
+                                        "name",    "Default Image",
                                         "comment", DEFAULT_COMMENT,
                                         NULL);
   g_signal_connect (config->default_image, "notify",
                     G_CALLBACK (gimp_core_config_default_image_notify),
                     config);
 
-  config->default_grid = g_object_new (GIMP_TYPE_GRID, NULL);
+  config->default_grid = g_object_new (GIMP_TYPE_GRID,
+                                       "name", "Default Grid",
+                                       NULL);
   g_signal_connect (config->default_grid, "notify",
                     G_CALLBACK (gimp_core_config_default_grid_notify),
                     config);
@@ -394,24 +472,31 @@ gimp_core_config_finalize (GObject *object)
 {
   GimpCoreConfig *core_config = GIMP_CORE_CONFIG (object);
 
+  g_free (core_config->language);
   g_free (core_config->plug_in_path);
   g_free (core_config->module_path);
   g_free (core_config->interpreter_path);
   g_free (core_config->environ_path);
   g_free (core_config->brush_path);
   g_free (core_config->brush_path_writable);
+  g_free (core_config->dynamics_path);
+  g_free (core_config->dynamics_path_writable);
   g_free (core_config->pattern_path);
   g_free (core_config->pattern_path_writable);
   g_free (core_config->palette_path);
   g_free (core_config->palette_path_writable);
   g_free (core_config->gradient_path);
   g_free (core_config->gradient_path_writable);
+  g_free (core_config->tool_preset_path);
+  g_free (core_config->tool_preset_path_writable);
   g_free (core_config->font_path);
   g_free (core_config->font_path_writable);
   g_free (core_config->default_brush);
+  g_free (core_config->default_dynamics);
   g_free (core_config->default_pattern);
   g_free (core_config->default_palette);
   g_free (core_config->default_gradient);
+  g_free (core_config->default_tool_preset);
   g_free (core_config->default_font);
   g_free (core_config->plug_in_rc_path);
 
@@ -437,6 +522,10 @@ gimp_core_config_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_LANGUAGE:
+      g_free (core_config->language);
+      core_config->language = g_value_dup_string (value);
+      break;
     case PROP_INTERPOLATION_TYPE:
       core_config->interpolation_type = g_value_get_enum (value);
       break;
@@ -464,6 +553,14 @@ gimp_core_config_set_property (GObject      *object,
       g_free (core_config->brush_path_writable);
       core_config->brush_path_writable = g_value_dup_string (value);
       break;
+    case PROP_DYNAMICS_PATH:
+      g_free (core_config->dynamics_path);
+      core_config->dynamics_path = g_value_dup_string (value);
+      break;
+    case PROP_DYNAMICS_PATH_WRITABLE:
+      g_free (core_config->dynamics_path_writable);
+      core_config->dynamics_path_writable = g_value_dup_string (value);
+      break;
     case PROP_PATTERN_PATH:
       g_free (core_config->pattern_path);
       core_config->pattern_path = g_value_dup_string (value);
@@ -488,6 +585,14 @@ gimp_core_config_set_property (GObject      *object,
       g_free (core_config->gradient_path_writable);
       core_config->gradient_path_writable = g_value_dup_string (value);
       break;
+    case PROP_TOOL_PRESET_PATH:
+      g_free (core_config->tool_preset_path);
+      core_config->tool_preset_path = g_value_dup_string (value);
+      break;
+    case PROP_TOOL_PRESET_PATH_WRITABLE:
+      g_free (core_config->tool_preset_path_writable);
+      core_config->tool_preset_path_writable = g_value_dup_string (value);
+      break;
     case PROP_FONT_PATH:
       g_free (core_config->font_path);
       core_config->font_path = g_value_dup_string (value);
@@ -500,6 +605,10 @@ gimp_core_config_set_property (GObject      *object,
       g_free (core_config->default_brush);
       core_config->default_brush = g_value_dup_string (value);
       break;
+    case PROP_DEFAULT_DYNAMICS:
+      g_free (core_config->default_dynamics);
+      core_config->default_dynamics = g_value_dup_string (value);
+      break;
     case PROP_DEFAULT_PATTERN:
       g_free (core_config->default_pattern);
       core_config->default_pattern = g_value_dup_string (value);
@@ -512,6 +621,10 @@ gimp_core_config_set_property (GObject      *object,
       g_free (core_config->default_gradient);
       core_config->default_gradient = g_value_dup_string (value);
       break;
+    case PROP_DEFAULT_TOOL_PRESET:
+      g_free (core_config->default_tool_preset);
+      core_config->default_tool_preset = g_value_dup_string (value);
+      break;
     case PROP_DEFAULT_FONT:
       g_free (core_config->default_font);
       core_config->default_font = g_value_dup_string (value);
@@ -519,6 +632,9 @@ gimp_core_config_set_property (GObject      *object,
     case PROP_GLOBAL_BRUSH:
       core_config->global_brush = g_value_get_boolean (value);
       break;
+    case PROP_GLOBAL_DYNAMICS:
+      core_config->global_dynamics = g_value_get_boolean (value);
+      break;
     case PROP_GLOBAL_PATTERN:
       core_config->global_pattern = g_value_get_boolean (value);
       break;
@@ -606,6 +722,9 @@ gimp_core_config_get_property (GObject    *object,
 
   switch (property_id)
     {
+    case PROP_LANGUAGE:
+      g_value_set_string (value, core_config->language);
+      break;
     case PROP_INTERPOLATION_TYPE:
       g_value_set_enum (value, core_config->interpolation_type);
       break;
@@ -627,6 +746,12 @@ gimp_core_config_get_property (GObject    *object,
     case PROP_BRUSH_PATH_WRITABLE:
       g_value_set_string (value, core_config->brush_path_writable);
       break;
+    case PROP_DYNAMICS_PATH:
+      g_value_set_string (value, core_config->dynamics_path);
+      break;
+    case PROP_DYNAMICS_PATH_WRITABLE:
+      g_value_set_string (value, core_config->dynamics_path_writable);
+      break;
     case PROP_PATTERN_PATH:
       g_value_set_string (value, core_config->pattern_path);
       break;
@@ -645,6 +770,12 @@ gimp_core_config_get_property (GObject    *object,
     case PROP_GRADIENT_PATH_WRITABLE:
       g_value_set_string (value, core_config->gradient_path_writable);
       break;
+    case PROP_TOOL_PRESET_PATH:
+      g_value_set_string (value, core_config->tool_preset_path);
+      break;
+    case PROP_TOOL_PRESET_PATH_WRITABLE:
+      g_value_set_string (value, core_config->tool_preset_path_writable);
+      break;
     case PROP_FONT_PATH:
       g_value_set_string (value, core_config->font_path);
       break;
@@ -654,6 +785,9 @@ gimp_core_config_get_property (GObject    *object,
     case PROP_DEFAULT_BRUSH:
       g_value_set_string (value, core_config->default_brush);
       break;
+    case PROP_DEFAULT_DYNAMICS:
+      g_value_set_string (value, core_config->default_dynamics);
+      break;
     case PROP_DEFAULT_PATTERN:
       g_value_set_string (value, core_config->default_pattern);
       break;
@@ -663,12 +797,18 @@ gimp_core_config_get_property (GObject    *object,
     case PROP_DEFAULT_GRADIENT:
       g_value_set_string (value, core_config->default_gradient);
       break;
+    case PROP_DEFAULT_TOOL_PRESET:
+      g_value_set_string (value, core_config->default_tool_preset);
+      break;
     case PROP_DEFAULT_FONT:
       g_value_set_string (value, core_config->default_font);
       break;
     case PROP_GLOBAL_BRUSH:
       g_value_set_boolean (value, core_config->global_brush);
       break;
+    case PROP_GLOBAL_DYNAMICS:
+      g_value_set_boolean (value, core_config->global_dynamics);
+      break;
     case PROP_GLOBAL_PATTERN:
       g_value_set_boolean (value, core_config->global_pattern);
       break;
