@@ -21,8 +21,6 @@
 #ifndef __GIMP_DISPLAY_CONFIG_H__
 #define __GIMP_DISPLAY_CONFIG_H__
 
-#include "display/display-enums.h"
-
 #include "config/gimpcoreconfig.h"
 
 
@@ -73,6 +71,7 @@ struct _GimpDisplayConfig
   GimpSpaceBarAction  space_bar_action;
   GimpRGB             xor_color;
   GimpZoomQuality     zoom_quality;
+  gboolean            use_event_history;
 };
 
 struct _GimpDisplayConfigClass
