@@ -21,8 +21,6 @@
 #ifndef __GIMP_GUI_CONFIG_H__
 #define __GIMP_GUI_CONFIG_H__
 
-#include "widgets/widgets-enums.h"
-
 #include "config/gimpdisplayconfig.h"
 
 
@@ -48,6 +46,8 @@ struct _GimpGuiConfig
   gboolean             restore_session;
   gboolean             save_tool_options;
   gboolean             show_tooltips;
+  gboolean             hide_docks;
+  gboolean             single_window_mode;
   gboolean             tearoff_menus;
   gboolean             can_change_accels;
   gboolean             save_accels;
@@ -67,7 +67,6 @@ struct _GimpGuiConfig
   gchar               *web_browser;
   gboolean             user_manual_online;
   gchar               *user_manual_online_uri;
-  GimpWindowHint       toolbox_window_hint;
   GimpWindowHint       dock_window_hint;
   GimpCursorFormat     cursor_format;
 
