@@ -19,6 +19,16 @@ N_("When enabled, an image will become the active image when its image " \
 
 #define BRUSH_PATH_WRITABLE_BLURB ""
 
+#define DYNAMICS_PATH_BLURB \
+N_("Sets the dynamics search path.")
+
+#define DYNAMICS_PATH_WRITABLE_BLURB ""
+
+#define TOOL_PRESET_PATH_BLURB \
+N_("Sets the dynamics search path.")
+
+#define TOOL_PRESET_PATH_WRITABLE_BLURB ""
+
 #define CANVAS_PADDING_COLOR_BLURB \
 N_("Sets the canvas padding color used if the padding mode is set to " \
    "custom color.")
@@ -49,6 +59,14 @@ N_("Context-dependent mouse pointers are helpful.  They are enabled by " \
 "Specify a default brush.  The brush is searched for in the " \
 "specified brush path."
 
+#define DEFAULT_DYNAMICS_BLURB \
+"Specify a default dynamics.  The dynamics is searched for in the " \
+"specified dynamics path."
+
+#define DEFAULT_TOOL_PRESET_BLURB \
+"Specify a default tool preset.  The tool preset is searched for in the " \
+"specified tool prests path."
+
 #define DEFAULT_DOT_FOR_DOT_BLURB \
 N_("When enabled, this will ensure that each pixel of an image gets " \
    "mapped to a pixel on the screen.")
@@ -101,8 +119,9 @@ N_("Tools such as fuzzy-select and bucket fill find regions based on a " \
 "Sets the default settings used when an image is viewed in fullscreen mode."
 
 #define DOCK_WINDOW_HINT_BLURB \
-N_("The window type hint that is set on dock windows. This may affect " \
-   "the way your window manager decorates and handles dock windows.")
+N_("The window type hint that is set on dock windows and the toolbox " \
+   "window. This may affect the way your window manager decorates and " \
+   "handles these windows.")
 
 #define ENVIRON_PATH_BLURB \
 "Sets the environ search path."
@@ -134,6 +153,9 @@ N_("The window type hint that is set on dock windows. This may affect " \
 #define GLOBAL_BRUSH_BLURB \
 N_("When enabled, the selected brush will be used for all tools.")
 
+#define GLOBAL_DYNAMICS_BLURB \
+N_("When enabled, the selected dynamics will be used for all tools.")
+
 #define GLOBAL_FONT_BLURB \
 "When enabled, the selected font will be used for all tools."
 
@@ -185,6 +207,9 @@ N_("Sets the level of interpolation used for scaling and other " \
 #define INTERPRETER_PATH_BLURB \
 "Sets the interpreter search path."
 
+#define LANGUAGE_BLURB \
+N_("Specifies the language to use for the user interface.")
+
 #define LAST_OPENED_SIZE_BLURB \
 N_("How many recently opened image filenames to keep on the File menu.")
 
@@ -342,6 +367,12 @@ N_("When enabled, the sample points are visible by default. This can also be " \
 #define SHOW_TOOLTIPS_BLURB \
 N_("Show a tooltip when the pointer hovers over an item.")
 
+#define SINGLE_WINDOW_MODE_BLURB \
+N_("Use GIMP in a single-window mode.")
+
+#define HIDE_DOCKS_BLURB \
+N_("Hide docks and other windows, leaving only image windows.")
+
 #define SPACE_BAR_ACTION_BLURB \
 N_("What to do when the space bar is pressed in the image window.")
 
@@ -404,10 +435,6 @@ N_("Show the currently active image in the toolbox.")
 #define TOOLBOX_WILBER_BLURB \
 "Show the GIMP mascot at the top of the toolbox."
 
-#define TOOLBOX_WINDOW_HINT_BLURB \
-N_("The window type hint that is set on the toolbox. This may affect " \
-   "how your window manager decorates and handles the toolbox window.")
-
 #define TRANSPARENCY_TYPE_BLURB \
 N_("Sets the manner in which transparency is displayed in images.")
 
@@ -449,5 +476,8 @@ N_("When enabled, pressing F1 will open the help browser.")
 #define ZOOM_QUALITY_BLURB \
 "There's a tradeoff between speed and quality of the zoomed-out display."
 
+#define DEFAULT_USE_EVENT_HISTORY_BLURB \
+"Bugs in event history buffer are frequent so in case of cursor " \
+"offset problems turning it off helps."
 
 #endif  /* __GIMP_RC_BLURBS_H__ */
