@@ -410,16 +410,19 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_DRAWABLE,                 /*< desc="Layer/Channel"               >*/
   GIMP_UNDO_DRAWABLE_MOD,             /*< desc="Layer/Channel modification"  >*/
   GIMP_UNDO_MASK,                     /*< desc="Selection mask"              >*/
+  GIMP_UNDO_ITEM_REORDER,             /*< desc="Reorder item"                >*/
   GIMP_UNDO_ITEM_RENAME,              /*< desc="Rename item"                 >*/
   GIMP_UNDO_ITEM_DISPLACE,            /*< desc="Move item"                   >*/
   GIMP_UNDO_ITEM_VISIBILITY,          /*< desc="Item visibility"             >*/
   GIMP_UNDO_ITEM_LINKED,              /*< desc="Link/Unlink item"            >*/
   GIMP_UNDO_LAYER_ADD,                /*< desc="New layer"                   >*/
   GIMP_UNDO_LAYER_REMOVE,             /*< desc="Delete layer"                >*/
-  GIMP_UNDO_LAYER_REORDER,            /*< desc="Reorder layer"               >*/
   GIMP_UNDO_LAYER_MODE,               /*< desc="Set layer mode"              >*/
   GIMP_UNDO_LAYER_OPACITY,            /*< desc="Set layer opacity"           >*/
   GIMP_UNDO_LAYER_LOCK_ALPHA,         /*< desc="Lock/Unlock alpha channel"   >*/
+  GIMP_UNDO_GROUP_LAYER_SUSPEND,      /*< desc="Suspend group layer resize"  >*/
+  GIMP_UNDO_GROUP_LAYER_RESUME,       /*< desc="Resume group layer resize"   >*/
+  GIMP_UNDO_GROUP_LAYER_CONVERT,      /*< desc="Convert group layer"         >*/
   GIMP_UNDO_TEXT_LAYER,               /*< desc="Text layer"                  >*/
   GIMP_UNDO_TEXT_LAYER_MODIFIED,      /*< desc="Text layer modification"     >*/
   GIMP_UNDO_LAYER_MASK_ADD,           /*< desc="Add layer mask"              >*/
@@ -428,12 +431,10 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_LAYER_MASK_SHOW,          /*< desc="Show layer mask"             >*/
   GIMP_UNDO_CHANNEL_ADD,              /*< desc="New channel"                 >*/
   GIMP_UNDO_CHANNEL_REMOVE,           /*< desc="Delete channel"              >*/
-  GIMP_UNDO_CHANNEL_REORDER,          /*< desc="Reorder channel"             >*/
   GIMP_UNDO_CHANNEL_COLOR,            /*< desc="Channel color"               >*/
   GIMP_UNDO_VECTORS_ADD,              /*< desc="New path"                    >*/
   GIMP_UNDO_VECTORS_REMOVE,           /*< desc="Delete path"                 >*/
   GIMP_UNDO_VECTORS_MOD,              /*< desc="Path modification"           >*/
-  GIMP_UNDO_VECTORS_REORDER,          /*< desc="Reorder path"                >*/
   GIMP_UNDO_FS_TO_LAYER,              /*< desc="Floating selection to layer" >*/
   GIMP_UNDO_TRANSFORM,                /*< desc="Transform"                   >*/
   GIMP_UNDO_PAINT,                    /*< desc="Paint"                       >*/
@@ -583,6 +584,26 @@ typedef enum  /*< pdb-skip >*/
 } GimpColorProfilePolicy;
 
 
+#define GIMP_TYPE_DYNAMICS_OUTPUT_TYPE (gimp_dynamics_output_type_get_type ())
+
+GType gimp_dynamics_output_type_get_type (void) G_GNUC_CONST;
+
+typedef enum  /*< pdb-skip >*/
+{
+  GIMP_DYNAMICS_OUTPUT_OPACITY,      /*< desc="Opacity"      >*/
+  GIMP_DYNAMICS_OUTPUT_SIZE,         /*< desc="Size"         >*/
+  GIMP_DYNAMICS_OUTPUT_ANGLE,        /*< desc="Angle"        >*/
+  GIMP_DYNAMICS_OUTPUT_COLOR,        /*< desc="Color"        >*/
+  GIMP_DYNAMICS_OUTPUT_HARDNESS,     /*< desc="Hardness"     >*/
+  GIMP_DYNAMICS_OUTPUT_FORCE,        /*< desc="Force"        >*/
+  GIMP_DYNAMICS_OUTPUT_ASPECT_RATIO, /*< desc="Aspect ratio" >*/
+  GIMP_DYNAMICS_OUTPUT_SPACING,      /*< desc="Spacing"      >*/
+  GIMP_DYNAMICS_OUTPUT_RATE,         /*< desc="Rate"         >*/
+  GIMP_DYNAMICS_OUTPUT_FLOW,         /*< desc="Flow"         >*/
+  GIMP_DYNAMICS_OUTPUT_JITTER,       /*< desc="Jitter"       >*/
+} GimpDynamicsOutputType;
+
+
 /*
  * non-registered enums; register them if needed
  */
@@ -590,47 +611,51 @@ typedef enum  /*< pdb-skip >*/
 
 typedef enum  /*< pdb-skip, skip >*/
 {
-  GIMP_CONTEXT_FIRST_PROP      =  2,
+  GIMP_CONTEXT_FIRST_PROP       =  2,
 
-  GIMP_CONTEXT_PROP_IMAGE      =  GIMP_CONTEXT_FIRST_PROP,
-  GIMP_CONTEXT_PROP_DISPLAY    =  3,
-  GIMP_CONTEXT_PROP_TOOL       =  4,
-  GIMP_CONTEXT_PROP_PAINT_INFO =  5,
-  GIMP_CONTEXT_PROP_FOREGROUND =  6,
-  GIMP_CONTEXT_PROP_BACKGROUND =  7,
-  GIMP_CONTEXT_PROP_OPACITY    =  8,
-  GIMP_CONTEXT_PROP_PAINT_MODE =  9,
-  GIMP_CONTEXT_PROP_BRUSH      = 10,
-  GIMP_CONTEXT_PROP_PATTERN    = 11,
-  GIMP_CONTEXT_PROP_GRADIENT   = 12,
-  GIMP_CONTEXT_PROP_PALETTE    = 13,
-  GIMP_CONTEXT_PROP_FONT       = 14,
-  GIMP_CONTEXT_PROP_BUFFER     = 15,
-  GIMP_CONTEXT_PROP_IMAGEFILE  = 16,
-  GIMP_CONTEXT_PROP_TEMPLATE   = 17,
+  GIMP_CONTEXT_PROP_IMAGE       =  GIMP_CONTEXT_FIRST_PROP,
+  GIMP_CONTEXT_PROP_DISPLAY     =  3,
+  GIMP_CONTEXT_PROP_TOOL        =  4,
+  GIMP_CONTEXT_PROP_PAINT_INFO  =  5,
+  GIMP_CONTEXT_PROP_FOREGROUND  =  6,
+  GIMP_CONTEXT_PROP_BACKGROUND  =  7,
+  GIMP_CONTEXT_PROP_OPACITY     =  8,
+  GIMP_CONTEXT_PROP_PAINT_MODE  =  9,
+  GIMP_CONTEXT_PROP_BRUSH       = 10,
+  GIMP_CONTEXT_PROP_DYNAMICS    = 11,
+  GIMP_CONTEXT_PROP_PATTERN     = 12,
+  GIMP_CONTEXT_PROP_GRADIENT    = 13,
+  GIMP_CONTEXT_PROP_PALETTE     = 14,
+  GIMP_CONTEXT_PROP_TOOL_PRESET = 15,
+  GIMP_CONTEXT_PROP_FONT        = 16,
+  GIMP_CONTEXT_PROP_BUFFER      = 17,
+  GIMP_CONTEXT_PROP_IMAGEFILE   = 18,
+  GIMP_CONTEXT_PROP_TEMPLATE    = 19,
 
-  GIMP_CONTEXT_LAST_PROP       = GIMP_CONTEXT_PROP_TEMPLATE
+  GIMP_CONTEXT_LAST_PROP        = GIMP_CONTEXT_PROP_TEMPLATE
 } GimpContextPropType;
 
 
 typedef enum  /*< pdb-skip, skip >*/
 {
-  GIMP_CONTEXT_IMAGE_MASK      = 1 <<  2,
-  GIMP_CONTEXT_DISPLAY_MASK    = 1 <<  3,
-  GIMP_CONTEXT_TOOL_MASK       = 1 <<  4,
-  GIMP_CONTEXT_PAINT_INFO_MASK = 1 <<  5,
-  GIMP_CONTEXT_FOREGROUND_MASK = 1 <<  6,
-  GIMP_CONTEXT_BACKGROUND_MASK = 1 <<  7,
-  GIMP_CONTEXT_OPACITY_MASK    = 1 <<  8,
-  GIMP_CONTEXT_PAINT_MODE_MASK = 1 <<  9,
-  GIMP_CONTEXT_BRUSH_MASK      = 1 << 10,
-  GIMP_CONTEXT_PATTERN_MASK    = 1 << 11,
-  GIMP_CONTEXT_GRADIENT_MASK   = 1 << 12,
-  GIMP_CONTEXT_PALETTE_MASK    = 1 << 13,
-  GIMP_CONTEXT_FONT_MASK       = 1 << 14,
-  GIMP_CONTEXT_BUFFER_MASK     = 1 << 15,
-  GIMP_CONTEXT_IMAGEFILE_MASK  = 1 << 16,
-  GIMP_CONTEXT_TEMPLATE_MASK   = 1 << 17,
+  GIMP_CONTEXT_IMAGE_MASK       = 1 <<  2,
+  GIMP_CONTEXT_DISPLAY_MASK     = 1 <<  3,
+  GIMP_CONTEXT_TOOL_MASK        = 1 <<  4,
+  GIMP_CONTEXT_PAINT_INFO_MASK  = 1 <<  5,
+  GIMP_CONTEXT_FOREGROUND_MASK  = 1 <<  6,
+  GIMP_CONTEXT_BACKGROUND_MASK  = 1 <<  7,
+  GIMP_CONTEXT_OPACITY_MASK     = 1 <<  8,
+  GIMP_CONTEXT_PAINT_MODE_MASK  = 1 <<  9,
+  GIMP_CONTEXT_BRUSH_MASK       = 1 << 10,
+  GIMP_CONTEXT_DYNAMICS_MASK    = 1 << 11,
+  GIMP_CONTEXT_PATTERN_MASK     = 1 << 12,
+  GIMP_CONTEXT_GRADIENT_MASK    = 1 << 13,
+  GIMP_CONTEXT_PALETTE_MASK     = 1 << 14,
+  GIMP_CONTEXT_TOOL_PRESET_MASK = 1 << 15,
+  GIMP_CONTEXT_FONT_MASK        = 1 << 16,
+  GIMP_CONTEXT_BUFFER_MASK      = 1 << 17,
+  GIMP_CONTEXT_IMAGEFILE_MASK   = 1 << 18,
+  GIMP_CONTEXT_TEMPLATE_MASK    = 1 << 19,
 
   /*  aliases  */
   GIMP_CONTEXT_PAINT_PROPS_MASK = (GIMP_CONTEXT_FOREGROUND_MASK |
@@ -638,6 +663,7 @@ typedef enum  /*< pdb-skip, skip >*/
                                    GIMP_CONTEXT_OPACITY_MASK    |
                                    GIMP_CONTEXT_PAINT_MODE_MASK |
                                    GIMP_CONTEXT_BRUSH_MASK      |
+                                   GIMP_CONTEXT_DYNAMICS_MASK   |
                                    GIMP_CONTEXT_PATTERN_MASK    |
                                    GIMP_CONTEXT_GRADIENT_MASK),
   GIMP_CONTEXT_ALL_PROPS_MASK   = (GIMP_CONTEXT_IMAGE_MASK      |
