@@ -29,9 +29,6 @@
 
 /*  defines  */
 
-#define GIMP_OPACITY_TRANSPARENT      0.0
-#define GIMP_OPACITY_OPAQUE           1.0
-
 #define GIMP_COORDS_MIN_PRESSURE      0.0
 #define GIMP_COORDS_MAX_PRESSURE      1.0
 #define GIMP_COORDS_DEFAULT_PRESSURE  1.0
@@ -46,12 +43,15 @@
 
 #define GIMP_COORDS_DEFAULT_VELOCITY  0.0
 
+#define GIMP_COORDS_DEFAULT_DIRECTION 0.0
+
 #define GIMP_COORDS_DEFAULT_VALUES    { 0.0, 0.0, \
                                         GIMP_COORDS_DEFAULT_PRESSURE, \
                                         GIMP_COORDS_DEFAULT_TILT,     \
                                         GIMP_COORDS_DEFAULT_TILT,     \
                                         GIMP_COORDS_DEFAULT_WHEEL,    \
-                                        GIMP_COORDS_DEFAULT_VELOCITY }
+                                        GIMP_COORDS_DEFAULT_VELOCITY, \
+                                        GIMP_COORDS_DEFAULT_DIRECTION }
 
 
 /*  base classes  */
@@ -75,6 +75,11 @@ typedef struct _GimpItemStack         GimpItemStack;
 typedef struct _GimpToolPresets       GimpToolPresets;
 
 
+/*  not really a container  */
+
+typedef struct _GimpItemTree          GimpItemTree;
+
+
 /*  context objects  */
 
 typedef struct _GimpContext         GimpContext;
@@ -98,10 +103,13 @@ typedef struct _GimpBrushClipboard   GimpBrushClipboard;
 typedef struct _GimpBrushGenerated   GimpBrushGenerated;
 typedef struct _GimpBrushPipe        GimpBrushPipe;
 typedef struct _GimpCurve            GimpCurve;
+typedef struct _GimpDynamics         GimpDynamics;
+typedef struct _GimpDynamicsOutput   GimpDynamicsOutput;
 typedef struct _GimpGradient         GimpGradient;
 typedef struct _GimpPalette          GimpPalette;
 typedef struct _GimpPattern          GimpPattern;
 typedef struct _GimpPatternClipboard GimpPatternClipboard;
+typedef struct _GimpToolPreset       GimpToolPreset;
 typedef struct _GimpTagCache         GimpTagCache;
 
 
@@ -129,6 +137,7 @@ typedef struct _GimpLayerMaskUndo     GimpLayerMaskUndo;
 typedef struct _GimpLayerMaskPropUndo GimpLayerMaskPropUndo;
 typedef struct _GimpLayerUndo         GimpLayerUndo;
 typedef struct _GimpLayerPropUndo     GimpLayerPropUndo;
+typedef struct _GimpGroupLayerUndo    GimpGroupLayerUndo;
 typedef struct _GimpMaskUndo          GimpMaskUndo;
 typedef struct _GimpGuideUndo         GimpGuideUndo;
 typedef struct _GimpSamplePointUndo   GimpSamplePointUndo;
