@@ -57,8 +57,6 @@ static GimpBuffer * gimp_edit_extract         (GimpImage            *image,
                                                GimpContext          *context,
                                                gboolean              cut_pixels,
                                                GError              **error);
-static GimpBuffer * gimp_edit_make_buffer     (Gimp                 *gimp,
-                                               TileManager          *tiles);
 static gboolean     gimp_edit_fill_internal   (GimpImage            *image,
                                                GimpDrawable         *drawable,
                                                GimpContext          *context,
@@ -131,13 +129,16 @@ gimp_edit_copy_visible (GimpImage    *image,
                         GimpContext  *context,
                         GError      **error)
 {
-  GimpBuffer *buffer;
+  GimpProjection *projection;
+  GimpBuffer     *buffer;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  buffer = gimp_edit_extract (image, GIMP_PICKABLE (image->projection),
+  projection = gimp_image_get_projection (image);
+
+  buffer = gimp_edit_extract (image, GIMP_PICKABLE (projection),
                               context, FALSE, error);
 
   if (buffer)
@@ -257,7 +258,7 @@ gimp_edit_paste (GimpImage    *image,
 
   /*  Start a group undo  */
   gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_PASTE,
-                               _("Paste"));
+                               C_("undo-type", "Paste"));
 
   /*  If there is a selection mask clear it--
    *  this might not always be desired, but in general,
@@ -278,65 +279,6 @@ gimp_edit_paste (GimpImage    *image,
   return layer;
 }
 
-GimpImage *
-gimp_edit_paste_as_new (Gimp       *gimp,
-                        GimpImage  *invoke,
-                        GimpBuffer *paste)
-{
-  GimpImage     *image;
-  GimpLayer     *layer;
-  GimpImageType  type;
-
-  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
-  g_return_val_if_fail (invoke == NULL || GIMP_IS_IMAGE (invoke), NULL);
-  g_return_val_if_fail (GIMP_IS_BUFFER (paste), NULL);
-
-  switch (tile_manager_bpp (paste->tiles))
-    {
-    case 1: type = GIMP_GRAY_IMAGE;  break;
-    case 2: type = GIMP_GRAYA_IMAGE; break;
-    case 3: type = GIMP_RGB_IMAGE;   break;
-    case 4: type = GIMP_RGBA_IMAGE;  break;
-    default:
-      g_return_val_if_reached (NULL);
-      break;
-    }
-
-  /*  create a new image  (always of type GIMP_RGB)  */
-  image = gimp_create_image (gimp,
-                             gimp_buffer_get_width (paste),
-                             gimp_buffer_get_height (paste),
-                             GIMP_IMAGE_TYPE_BASE_TYPE (type),
-                             TRUE);
-  gimp_image_undo_disable (image);
-
-  if (invoke)
-    {
-      gdouble xres;
-      gdouble yres;
-
-      gimp_image_get_resolution (invoke, &xres, &yres);
-      gimp_image_set_resolution (image, xres, yres);
-      gimp_image_set_unit (image, gimp_image_get_unit (invoke));
-    }
-
-  layer = gimp_layer_new_from_tiles (paste->tiles, image, type,
-                                     _("Pasted Layer"),
-                                     GIMP_OPACITY_OPAQUE, GIMP_NORMAL_MODE);
-
-  if (! layer)
-    {
-      g_object_unref (image);
-      return NULL;
-    }
-
-  gimp_image_add_layer (image, layer, NULL, 0, TRUE);
-
-  gimp_image_undo_enable (image);
-
-  return image;
-}
-
 const gchar *
 gimp_edit_named_cut (GimpImage     *image,
                      const gchar   *name,
@@ -362,7 +304,7 @@ gimp_edit_named_cut (GimpImage     *image,
       gimp_container_add (image->gimp->named_buffers, GIMP_OBJECT (buffer));
       g_object_unref (buffer);
 
-      return gimp_object_get_name (GIMP_OBJECT (buffer));
+      return gimp_object_get_name (buffer);
     }
 
   return NULL;
@@ -393,7 +335,7 @@ gimp_edit_named_copy (GimpImage     *image,
       gimp_container_add (image->gimp->named_buffers, GIMP_OBJECT (buffer));
       g_object_unref (buffer);
 
-      return gimp_object_get_name (GIMP_OBJECT (buffer));
+      return gimp_object_get_name (buffer);
     }
 
   return NULL;
@@ -405,14 +347,17 @@ gimp_edit_named_copy_visible (GimpImage    *image,
                               GimpContext  *context,
                               GError      **error)
 {
-  GimpBuffer *buffer;
+  GimpProjection *projection;
+  GimpBuffer     *buffer;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  buffer = gimp_edit_extract (image, GIMP_PICKABLE (image->projection),
+  projection = gimp_image_get_projection (image);
+
+  buffer = gimp_edit_extract (image, GIMP_PICKABLE (projection),
                               context, FALSE, error);
 
   if (buffer)
@@ -421,7 +366,7 @@ gimp_edit_named_copy_visible (GimpImage    *image,
       gimp_container_add (image->gimp->named_buffers, GIMP_OBJECT (buffer));
       g_object_unref (buffer);
 
-      return gimp_object_get_name (GIMP_OBJECT (buffer));
+      return gimp_object_get_name (buffer);
     }
 
   return NULL;
@@ -440,7 +385,7 @@ gimp_edit_clear (GimpImage    *image,
   return gimp_edit_fill_internal (image, drawable, context,
                                   GIMP_TRANSPARENT_FILL,
                                   GIMP_OPACITY_OPAQUE, GIMP_ERASE_MODE,
-                                  _("Clear"));
+                                  C_("undo-type", "Clear"));
 }
 
 gboolean
@@ -459,23 +404,23 @@ gimp_edit_fill (GimpImage    *image,
   switch (fill_type)
     {
     case GIMP_FOREGROUND_FILL:
-      undo_desc = _("Fill with Foreground Color");
+      undo_desc = C_("undo-type", "Fill with Foreground Color");
       break;
 
     case GIMP_BACKGROUND_FILL:
-      undo_desc = _("Fill with Background Color");
+      undo_desc = C_("undo-type", "Fill with Background Color");
       break;
 
     case GIMP_WHITE_FILL:
-      undo_desc = _("Fill with White");
+      undo_desc = C_("undo-type", "Fill with White");
       break;
 
     case GIMP_TRANSPARENT_FILL:
-      undo_desc = _("Fill with Transparency");
+      undo_desc = C_("undo-type", "Fill with Transparency");
       break;
 
     case GIMP_PATTERN_FILL:
-      undo_desc = _("Fill with Pattern");
+      undo_desc = C_("undo-type", "Fill with Pattern");
       break;
 
     case GIMP_NO_FILL:
@@ -523,7 +468,7 @@ gimp_edit_fade (GimpImage   *image,
 
       gimp_drawable_apply_region (drawable, &src2PR,
                                   TRUE,
-                                  gimp_object_get_name (GIMP_OBJECT (undo)),
+                                  gimp_object_get_name (undo),
                                   gimp_context_get_opacity (context),
                                   gimp_context_get_paint_mode (context),
                                   NULL, NULL,
@@ -552,7 +497,7 @@ gimp_edit_extract (GimpImage     *image,
   TileManager *tiles;
 
   if (cut_pixels)
-    gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_CUT, _("Cut"));
+    gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_CUT, C_("undo-type", "Cut"));
 
   /*  Cut/copy the mask portion from the image  */
   tiles = gimp_selection_extract (GIMP_SELECTION (gimp_image_get_mask (image)),
@@ -562,14 +507,16 @@ gimp_edit_extract (GimpImage     *image,
   if (cut_pixels)
     gimp_image_undo_group_end (image);
 
-  return gimp_edit_make_buffer (image->gimp, tiles);
-}
+  if (tiles)
+    {
+      GimpBuffer *buffer = gimp_buffer_new (tiles, _("Global Buffer"), FALSE);
 
-static GimpBuffer *
-gimp_edit_make_buffer (Gimp        *gimp,
-                       TileManager *tiles)
-{
-  return tiles ? gimp_buffer_new (tiles, _("Global Buffer"), FALSE) : NULL;
+      tile_manager_unref (tiles);
+
+      return buffer;
+    }
+
+  return NULL;
 }
 
 static gboolean
