@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-2003 Spencer Kimball, Peter Mattis, and others
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,15 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <stdlib.h>
 
-#include <glib-object.h>
+#include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
@@ -105,8 +104,8 @@ static void  gimp_transform_region_lanczos (TileManager       *orig_tiles,
                                             GimpProgress      *progress);
 
 static inline void  untransform_coords     (const GimpMatrix3 *m,
-                                            gint               x,
-                                            gint               y,
+                                            const gint         x,
+                                            const gint         y,
                                             gdouble           *tu,
                                             gdouble           *tv,
                                             gdouble           *tw);
@@ -117,51 +116,51 @@ static inline void  normalize_coords       (const gint         coords,
                                             gdouble           *u,
                                             gdouble           *v);
 
-static inline gboolean supersample_dtest   (gdouble u0,
-                                            gdouble v0,
-                                            gdouble u1,
-                                            gdouble v1,
-                                            gdouble u2,
-                                            gdouble v2,
-                                            gdouble u3,
-                                            gdouble v3);
+static inline gboolean supersample_dtest   (const gdouble u0,
+                                            const gdouble v0,
+                                            const gdouble u1,
+                                            const gdouble v1,
+                                            const gdouble u2,
+                                            const gdouble v2,
+                                            const gdouble u3,
+                                            const gdouble v3);
 
 static void     sample_adapt      (TileManager   *tm,
-                                   gdouble        uc,
-                                   gdouble        vc,
-                                   gdouble        u0,
-                                   gdouble        v0,
-                                   gdouble        u1,
-                                   gdouble        v1,
-                                   gdouble        u2,
-                                   gdouble        v2,
-                                   gdouble        u3,
-                                   gdouble        v3,
-                                   gint           level,
+                                   const gdouble  xc,
+                                   const gdouble  yc,
+                                   const gdouble  x0,
+                                   const gdouble  y0,
+                                   const gdouble  x1,
+                                   const gdouble  y1,
+                                   const gdouble  x2,
+                                   const gdouble  y2,
+                                   const gdouble  x3,
+                                   const gdouble  y3,
+                                   const gint     level,
                                    guchar        *color,
                                    const guchar  *bg_color,
                                    gint           bpp,
                                    gint           alpha);
 
 static void     sample_linear     (PixelSurround *surround,
-                                   gdouble        u,
-                                   gdouble        v,
+                                   const gdouble  u,
+                                   const gdouble  v,
                                    guchar        *color,
-                                   gint           bytes,
-                                   gint           alpha);
+                                   const gint     bytes,
+                                   const gint     alpha);
 static void     sample_cubic      (PixelSurround *surround,
-                                   gdouble        u,
-                                   gdouble        v,
+                                   const gdouble  u,
+                                   const gdouble  v,
                                    guchar        *color,
-                                   gint           bytes,
-                                   gint           alpha);
+                                   const gint     bytes,
+                                   const gint     alpha);
 static void     sample_lanczos    (PixelSurround *surround,
                                    const gfloat  *lanczos,
-                                   gdouble        u,
-                                   gdouble        v,
+                                   const gdouble  u,
+                                   const gdouble  v,
                                    guchar        *color,
-                                   gint           bytes,
-                                   gint           alpha);
+                                   const gint     bytes,
+                                   const gint     alpha);
 
 
 /*  public functions  */
@@ -211,18 +210,18 @@ gimp_transform_region (GimpPickable          *pickable,
   switch (GIMP_IMAGE_TYPE_BASE_TYPE (pickable_type))
     {
     case GIMP_RGB:
-      bg_color[ALPHA_PIX] = TRANSPARENT_OPACITY;
-      alpha = ALPHA_PIX;
+      bg_color[ALPHA] = TRANSPARENT_OPACITY;
+      alpha = ALPHA;
       break;
 
     case GIMP_GRAY:
-      bg_color[ALPHA_G_PIX] = TRANSPARENT_OPACITY;
-      alpha = ALPHA_G_PIX;
+      bg_color[ALPHA_G] = TRANSPARENT_OPACITY;
+      alpha = ALPHA_G;
       break;
 
     case GIMP_INDEXED:
-      bg_color[ALPHA_I_PIX] = TRANSPARENT_OPACITY;
-      alpha = ALPHA_I_PIX;
+      bg_color[ALPHA_I] = TRANSPARENT_OPACITY;
+      alpha = ALPHA_I;
       /*  If the image is indexed color, ignore interpolation value  */
       interpolation_type = GIMP_INTERPOLATION_NONE;
       break;
@@ -394,7 +393,8 @@ gimp_transform_region_linear (TileManager        *orig_tiles,
   gint           n;
   gpointer       pr;
 
-  surround = pixel_surround_new (orig_tiles, 2, 2, bg_color);
+  surround = pixel_surround_new (orig_tiles, 2, 2, PIXEL_SURROUND_BACKGROUND);
+  pixel_surround_set_bg (surround, bg_color);
 
   uinc = m->coeff[0][0];
   vinc = m->coeff[1][0];
@@ -496,7 +496,8 @@ gimp_transform_region_cubic (TileManager        *orig_tiles,
   gint           n;
   gpointer       pr;
 
-  surround = pixel_surround_new (orig_tiles, 4, 4, bg_color);
+  surround = pixel_surround_new (orig_tiles, 4, 4, PIXEL_SURROUND_BACKGROUND);
+  pixel_surround_set_bg (surround, bg_color);
 
   uinc = m->coeff[0][0];
   vinc = m->coeff[1][0];
@@ -599,7 +600,9 @@ gimp_transform_region_lanczos (TileManager       *orig_tiles,
   gpointer       pr;
 
   surround = pixel_surround_new (orig_tiles,
-                                 LANCZOS_WIDTH2, LANCZOS_WIDTH2, bg_color);
+                                 LANCZOS_WIDTH2, LANCZOS_WIDTH2,
+                                 PIXEL_SURROUND_BACKGROUND);
+  pixel_surround_set_bg (surround, bg_color);
 
   /* allocate and fill lanczos lookup table */
   lanczos = create_lanczos_lookup ();
@@ -685,8 +688,8 @@ gimp_transform_region_lanczos (TileManager       *orig_tiles,
 
 static inline void
 untransform_coords (const GimpMatrix3 *m,
-                    gint               x,
-                    gint               y,
+                    const gint         x,
+                    const gint         y,
                     gdouble           *tu,
                     gdouble           *tv,
                     gdouble           *tw)
@@ -751,16 +754,16 @@ normalize_coords (const gint     coords,
    */
 static void
 sample_linear (PixelSurround *surround,
-               gdouble        u,
-               gdouble        v,
+               const gdouble  u,
+               const gdouble  v,
                guchar        *color,
-               gint           bytes,
-               gint           alpha)
+               const gint     bytes,
+               const gint     alpha)
 {
   gdouble       a_val, a_recip;
   gint          i;
-  gint          iu = floor (u);
-  gint          iv = floor (v);
+  const gint    iu = floor (u);
+  const gint    iv = floor (v);
   gint          rowstride;
   gdouble       du, dv;
   const guchar *alphachan;
@@ -836,23 +839,21 @@ sample_linear (PixelSurround *surround,
 */
 static void
 sample_bi (TileManager  *tm,
-           gint          x,
-           gint          y,
+           const gint    x,
+           const gint    y,
            guchar       *color,
            const guchar *bg_color,
-           gint          bpp,
-           gint          alpha)
+           const gint    bpp,
+           const gint    alpha)
 {
-  guchar C[4][4];
-  gint   i;
-  gint   xscale = (x & (FIXED_UNIT-1));
-  gint   yscale = (y & (FIXED_UNIT-1));
-
-  gint   x0 = x >> FIXED_SHIFT;
-  gint   y0 = y >> FIXED_SHIFT;
-  gint   x1 = x0 + 1;
-  gint   y1 = y0 + 1;
-
+  const gint xscale = (x & (FIXED_UNIT-1));
+  const gint yscale = (y & (FIXED_UNIT-1));
+  const gint x0 = x >> FIXED_SHIFT;
+  const gint y0 = y >> FIXED_SHIFT;
+  const gint x1 = x0 + 1;
+  const gint y1 = y0 + 1;
+  guchar     C[4][4];
+  gint       i;
 
   /*  fill the color with default values, since read_pixel_data_1
    *  does nothing, when accesses are out of bounds.
@@ -873,18 +874,13 @@ sample_bi (TileManager  *tm,
                       lerp (C[2][alpha], C[3][alpha], yscale), xscale);
 
   if (color[alpha])
-    { /* to avoid problems, calculate with premultiplied alpha */
+    {
+      /* to avoid problems, calculate with premultiplied alpha */
       for (i = 0; i < alpha; i++)
-        {
-          C[0][i] = (C[0][i] * C[0][alpha] / 255);
-          C[1][i] = (C[1][i] * C[1][alpha] / 255);
-          C[2][i] = (C[2][i] * C[2][alpha] / 255);
-          C[3][i] = (C[3][i] * C[3][alpha] / 255);
-        }
-
-      for (i = 0; i < alpha; i++)
-        color[i] = lerp (lerp (C[0][i], C[1][i], yscale),
-                         lerp (C[2][i], C[3][i], yscale), xscale);
+        color[i] = lerp (lerp (C[0][i] * C[0][alpha] / 255,
+                               C[1][i] * C[1][alpha] / 255, yscale),
+                         lerp (C[2][i] * C[2][alpha] / 255,
+                               C[3][i] * C[3][alpha] / 255, yscale), xscale);
     }
   else
     {
@@ -902,10 +898,10 @@ sample_bi (TileManager  *tm,
  * subdivision should be used.
  */
 static inline gboolean
-supersample_test (gint x0, gint y0,
-                  gint x1, gint y1,
-                  gint x2, gint y2,
-                  gint x3, gint y3)
+supersample_test (const gint x0, const gint y0,
+                  const gint x1, const gint y1,
+                  const gint x2, const gint y2,
+                  const gint x3, const gint y3)
 {
   return (abs (x0 - x1) > FIXED_UNIT ||
           abs (x1 - x2) > FIXED_UNIT ||
@@ -925,10 +921,10 @@ supersample_test (gint x0, gint y0,
  * rotations.
  */
 static inline gboolean
-supersample_dtest (gdouble x0, gdouble y0,
-                   gdouble x1, gdouble y1,
-                   gdouble x2, gdouble y2,
-                   gdouble x3, gdouble y3)
+supersample_dtest (const gdouble x0, const gdouble y0,
+                   const gdouble x1, const gdouble y1,
+                   const gdouble x2, const gdouble y2,
+                   const gdouble x3, const gdouble y3)
 {
   return (fabs (x0 - x1) > G_SQRT2 ||
           fabs (x1 - x2) > G_SQRT2 ||
@@ -948,22 +944,22 @@ supersample_dtest (gdouble x0, gdouble y0,
 */
 static void
 get_sample (TileManager  *tm,
-            gint          xc,
-            gint          yc,
-            gint          x0,
-            gint          y0,
-            gint          x1,
-            gint          y1,
-            gint          x2,
-            gint          y2,
-            gint          x3,
-            gint          y3,
+            const gint    xc,
+            const gint    yc,
+            const gint    x0,
+            const gint    y0,
+            const gint    x1,
+            const gint    y1,
+            const gint    x2,
+            const gint    y2,
+            const gint    x3,
+            const gint    y3,
             gint         *cc,
-            gint          level,
+            const gint    level,
             guint        *color,
             const guchar *bg_color,
-            gint          bpp,
-            gint          alpha)
+            const gint    bpp,
+            const gint    alpha)
 {
   if (!level || !supersample_test (x0, y0, x1, y1, x2, y2, x3, y3))
     {
@@ -1027,22 +1023,22 @@ get_sample (TileManager  *tm,
 }
 
 static void
-sample_adapt (TileManager  *tm,
-              gdouble       xc,
-              gdouble       yc,
-              gdouble       x0,
-              gdouble       y0,
-              gdouble       x1,
-              gdouble       y1,
-              gdouble       x2,
-              gdouble       y2,
-              gdouble       x3,
-              gdouble       y3,
-              gint          level,
-              guchar       *color,
-              const guchar *bg_color,
-              gint          bpp,
-              gint          alpha)
+sample_adapt (TileManager   *tm,
+              const gdouble  xc,
+              const gdouble  yc,
+              const gdouble  x0,
+              const gdouble  y0,
+              const gdouble  x1,
+              const gdouble  y1,
+              const gdouble  x2,
+              const gdouble  y2,
+              const gdouble  x3,
+              const gdouble  y3,
+              const gint     level,
+              guchar        *color,
+              const guchar  *bg_color,
+              const gint     bpp,
+              const gint     alpha)
 {
     gint  cc = 0;
     gint  i;
@@ -1092,11 +1088,11 @@ sample_adapt (TileManager  *tm,
 /*  Note: cubic function no longer clips result. */
 /*  Inlining this function makes sample_cubic() run about 10% faster. (Sven) */
 static inline gdouble
-gimp_drawable_transform_cubic (gdouble dx,
-                               gint    jm1,
-                               gint    j,
-                               gint    jp1,
-                               gint    jp2)
+gimp_drawable_transform_cubic (const gdouble dx,
+                               const gint    jm1,
+                               const gint    j,
+                               const gint    jp1,
+                               const gint    jp2)
 {
   gdouble result;
 
@@ -1127,16 +1123,16 @@ gimp_drawable_transform_cubic (gdouble dx,
    */
 static void
 sample_cubic (PixelSurround *surround,
-              gdouble        u,
-              gdouble        v,
+              const gdouble  u,
+              const gdouble  v,
               guchar        *color,
-              gint           bytes,
-              gint           alpha)
+              const gint     bytes,
+              const gint     alpha)
 {
   gdouble       a_val, a_recip;
   gint          i;
-  gint          iu = floor(u);
-  gint          iv = floor(v);
+  const gint    iu = floor(u);
+  const gint    iv = floor(v);
   gint          rowstride;
   gdouble       du, dv;
   const guchar *data;
@@ -1207,11 +1203,11 @@ sample_cubic (PixelSurround *surround,
 static void
 sample_lanczos (PixelSurround *surround,
                 const gfloat  *lanczos,
-                gdouble        u,
-                gdouble        v,
+                const gdouble  u,
+                const gdouble  v,
                 guchar        *color,
-                gint           bytes,
-                gint           alpha)
+                const gint     bytes,
+                const gint     alpha)
 {
   gdouble       x_kernel[LANCZOS_WIDTH2]; /* 1-D kernels of window coeffs */
   gdouble       y_kernel[LANCZOS_WIDTH2];
