@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis, and others
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -190,6 +189,12 @@ gimp_transform_resize_boundary (const GimpMatrix3   *inv,
                                   ((gdouble) u2 - u1) / (v2 - v1),
                                   x1, y1, x2, y2);
       break;
+
+    case GIMP_TRANSFORM_RESIZE_CLIP:
+      /* Remove warning about not handling all enum values. We handle
+       * this case in the beginning of the function
+       */
+      break;
     }
 
   /* ensure that resulting rectangle has at least area 1 */
