@@ -57,7 +57,8 @@
 #include "gimpbuffer.h"
 #include "gimpcontext.h"
 #include "gimpdatafactory.h"
-#include "gimptagcache.h"
+#include "gimpdynamics.h"
+#include "gimpdynamics-load.h"
 #include "gimpdocumentlist.h"
 #include "gimpgradient-load.h"
 #include "gimpgradient.h"
@@ -72,8 +73,11 @@
 #include "gimppattern.h"
 #include "gimppatternclipboard.h"
 #include "gimpprogress.h"
+#include "gimptagcache.h"
 #include "gimptemplate.h"
 #include "gimptoolinfo.h"
+#include "gimptoolpreset.h"
+#include "gimptoolpreset-load.h"
 
 #include "gimp-intl.h"
 
@@ -191,6 +195,7 @@ gimp_init (Gimp *gimp)
   gimp->be_verbose       = FALSE;
   gimp->no_data          = FALSE;
   gimp->no_interface     = FALSE;
+  gimp->show_gui         = TRUE;
   gimp->use_shm          = FALSE;
   gimp->message_handler  = GIMP_CONSOLE;
   gimp->stack_trace_mode = GIMP_STACK_TRACE_NEVER;
@@ -231,6 +236,8 @@ gimp_init (Gimp *gimp)
 
   gimp->next_display_ID     = 1;
 
+  gimp->image_windows       = NULL;
+
   gimp->global_buffer       = NULL;
   gimp->named_buffers       = gimp_list_new (GIMP_TYPE_BUFFER, TRUE);
   gimp_object_set_static_name (GIMP_OBJECT (gimp->named_buffers),
@@ -238,9 +245,11 @@ gimp_init (Gimp *gimp)
 
   gimp->fonts               = NULL;
   gimp->brush_factory       = NULL;
+  gimp->dynamics_factory    = NULL;
   gimp->pattern_factory     = NULL;
   gimp->gradient_factory    = NULL;
   gimp->palette_factory     = NULL;
+  gimp->tool_preset_factory = NULL;
 
   gimp->tag_cache           = NULL;
 
@@ -277,6 +286,9 @@ gimp_dispose (GObject *object)
   if (gimp->brush_factory)
     gimp_data_factory_data_free (gimp->brush_factory);
 
+  if (gimp->dynamics_factory)
+    gimp_data_factory_data_free (gimp->dynamics_factory);
+
   if (gimp->pattern_factory)
     gimp_data_factory_data_free (gimp->pattern_factory);
 
@@ -286,6 +298,9 @@ gimp_dispose (GObject *object)
   if (gimp->palette_factory)
     gimp_data_factory_data_free (gimp->palette_factory);
 
+  if (gimp->tool_preset_factory)
+    gimp_data_factory_data_free (gimp->tool_preset_factory);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -321,6 +336,8 @@ gimp_finalize (GObject *object)
 
   if (gimp->tool_info_list)
     {
+      gimp_container_foreach (gimp->tool_info_list,
+                              (GFunc) g_object_run_dispose, NULL);
       g_object_unref (gimp->tool_info_list);
       gimp->tool_info_list = NULL;
     }
@@ -339,6 +356,12 @@ gimp_finalize (GObject *object)
       gimp->brush_factory = NULL;
     }
 
+  if (gimp->dynamics_factory)
+    {
+      g_object_unref (gimp->dynamics_factory);
+      gimp->dynamics_factory = NULL;
+    }
+
   if (gimp->pattern_factory)
     {
       g_object_unref (gimp->pattern_factory);
@@ -357,6 +380,12 @@ gimp_finalize (GObject *object)
       gimp->palette_factory = NULL;
     }
 
+  if (gimp->tool_preset_factory)
+    {
+      g_object_unref (gimp->tool_preset_factory);
+      gimp->tool_preset_factory = NULL;
+    }
+
   if (gimp->tag_cache)
     {
       g_object_unref (gimp->tag_cache);
@@ -478,12 +507,16 @@ gimp_get_memsize (GimpObject *object,
                                       gui_size);
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->brush_factory),
                                       gui_size);
+  memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->dynamics_factory),
+                                      gui_size);
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->pattern_factory),
                                       gui_size);
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->gradient_factory),
                                       gui_size);
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->palette_factory),
                                       gui_size);
+  memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->tool_preset_factory),
+                                      gui_size);
 
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->tag_cache),
                                       gui_size);
@@ -526,6 +559,11 @@ gimp_real_initialize (Gimp               *gimp,
     { gimp_brush_pipe_load,      GIMP_BRUSH_PIPE_FILE_EXTENSION,      FALSE }
   };
 
+  static const GimpDataFactoryLoaderEntry dynamics_loader_entries[] =
+  {
+    { gimp_dynamics_load,        GIMP_DYNAMICS_FILE_EXTENSION,        TRUE  }
+  };
+
   static const GimpDataFactoryLoaderEntry pattern_loader_entries[] =
   {
     { gimp_pattern_load,         GIMP_PATTERN_FILE_EXTENSION,         FALSE },
@@ -545,6 +583,11 @@ gimp_real_initialize (Gimp               *gimp,
     { gimp_palette_load,         NULL /* legacy loader */,            TRUE  }
   };
 
+  static const GimpDataFactoryLoaderEntry tool_preset_loader_entries[] =
+  {
+    { gimp_tool_preset_load,     GIMP_TOOL_PRESET_FILE_EXTENSION,     TRUE  }
+  };
+
   GimpData *clipboard_brush;
   GimpData *clipboard_pattern;
 
@@ -566,6 +609,17 @@ gimp_real_initialize (Gimp               *gimp,
   gimp_object_set_static_name (GIMP_OBJECT (gimp->brush_factory),
                                "brush factory");
 
+  gimp->dynamics_factory =
+    gimp_data_factory_new (gimp,
+                           GIMP_TYPE_DYNAMICS,
+                           "dynamics-path", "dynamics-path-writable",
+                           dynamics_loader_entries,
+                           G_N_ELEMENTS (dynamics_loader_entries),
+                           gimp_dynamics_new,
+                           gimp_dynamics_get_standard);
+  gimp_object_set_static_name (GIMP_OBJECT (gimp->dynamics_factory),
+                               "dynamics factory");
+
   gimp->pattern_factory =
     gimp_data_factory_new (gimp,
                            GIMP_TYPE_PATTERN,
@@ -599,6 +653,17 @@ gimp_real_initialize (Gimp               *gimp,
   gimp_object_set_static_name (GIMP_OBJECT (gimp->palette_factory),
                                "palette factory");
 
+  gimp->tool_preset_factory =
+    gimp_data_factory_new (gimp,
+                           GIMP_TYPE_TOOL_PRESET,
+                           "tool-preset-path", "tool-preset-path-writable",
+                           tool_preset_loader_entries,
+                           G_N_ELEMENTS (tool_preset_loader_entries),
+                           gimp_tool_preset_new,
+                           gimp_tool_preset_get_standard);
+  gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_preset_factory),
+                               "tool preset factory");
+
   gimp->tag_cache = gimp_tag_cache_new ();
 
   gimp_paint_init (gimp);
@@ -665,9 +730,11 @@ gimp_real_exit (Gimp     *gimp,
   gimp_tag_cache_save (gimp->tag_cache);
 
   gimp_data_factory_data_save (gimp->brush_factory);
+  gimp_data_factory_data_save (gimp->dynamics_factory);
   gimp_data_factory_data_save (gimp->pattern_factory);
   gimp_data_factory_data_save (gimp->gradient_factory);
   gimp_data_factory_data_save (gimp->palette_factory);
+  gimp_data_factory_data_save (gimp->tool_preset_factory);
 
   gimp_fonts_reset (gimp);
 
@@ -711,6 +778,37 @@ gimp_new (const gchar       *name,
   return gimp;
 }
 
+/**
+ * gimp_set_show_gui:
+ * @gimp:
+ * @show:
+ *
+ * Test cases that tests the UI typically don't want any windows to be
+ * presented during the test run. Allow them to set this.
+ **/
+void
+gimp_set_show_gui (Gimp     *gimp,
+                   gboolean  show_gui)
+{
+  g_return_if_fail (GIMP_IS_GIMP (gimp));
+
+  gimp->show_gui = show_gui;
+}
+
+/**
+ * gimp_get_show_gui:
+ * @gimp:
+ *
+ * Returns: %TRUE if the GUI should be shown, %FALSE otherwise.
+ **/
+gboolean
+gimp_get_show_gui (Gimp *gimp)
+{
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), FALSE);
+
+  return gimp->show_gui;
+}
+
 static void
 gimp_global_config_notify (GObject    *global_config,
                            GParamSpec *param_spec,
@@ -853,44 +951,62 @@ gimp_restore (Gimp               *gimp,
 
   /*  initialize the list of gimp brushes    */
   status_callback (NULL, _("Brushes"), 0.1);
-  gimp_data_factory_data_init (gimp->brush_factory, gimp->no_data);
+  gimp_data_factory_data_init (gimp->brush_factory, gimp->user_context,
+                               gimp->no_data);
+
+  /*  initialize the list of gimp dynamics   */
+  status_callback (NULL, _("Dynamics"), 0.2);
+  gimp_data_factory_data_init (gimp->dynamics_factory, gimp->user_context,
+                               gimp->no_data);
 
   /*  initialize the list of gimp patterns   */
-  status_callback (NULL, _("Patterns"), 0.2);
-  gimp_data_factory_data_init (gimp->pattern_factory, gimp->no_data);
+  status_callback (NULL, _("Patterns"), 0.3);
+  gimp_data_factory_data_init (gimp->pattern_factory, gimp->user_context,
+                               gimp->no_data);
 
   /*  initialize the list of gimp palettes   */
-  status_callback (NULL, _("Palettes"), 0.3);
-  gimp_data_factory_data_init (gimp->palette_factory, gimp->no_data);
+  status_callback (NULL, _("Palettes"), 0.4);
+  gimp_data_factory_data_init (gimp->palette_factory, gimp->user_context,
+                               gimp->no_data);
 
   /*  initialize the list of gimp gradients  */
-  status_callback (NULL, _("Gradients"), 0.4);
-  gimp_data_factory_data_init (gimp->gradient_factory, gimp->no_data);
+  status_callback (NULL, _("Gradients"), 0.5);
+  gimp_data_factory_data_init (gimp->gradient_factory, gimp->user_context,
+                               gimp->no_data);
 
   /*  initialize the list of fonts  */
-  status_callback (NULL, _("Fonts (this may take a while)"), 0.5);
+  status_callback (NULL, _("Fonts (this may take a while)"), 0.6);
   if (! gimp->no_fonts)
     gimp_fonts_load (gimp);
 
+  /*  initialize the list of gimp tool presets   */
+  status_callback (NULL, _("Tool Presets"), 0.65);
+  gimp_data_factory_data_init (gimp->tool_preset_factory, gimp->user_context,
+                               gimp->no_data);
+
   /*  initialize the template list  */
-  status_callback (NULL, _("Templates"), 0.6);
+  status_callback (NULL, _("Templates"), 0.7);
   gimp_templates_load (gimp);
 
   /*  initialize the module list  */
-  status_callback (NULL, _("Modules"), 0.7);
+  status_callback (NULL, _("Modules"), 0.8);
   gimp_modules_load (gimp);
 
   /* update tag cache */
-  status_callback (NULL, _("Updating tag cache"), 0.8);
+  status_callback (NULL, _("Updating tag cache"), 0.9);
   gimp_tag_cache_load (gimp->tag_cache);
   gimp_tag_cache_add_container (gimp->tag_cache,
                                 gimp_data_factory_get_container (gimp->brush_factory));
+  gimp_tag_cache_add_container (gimp->tag_cache,
+                                gimp_data_factory_get_container (gimp->dynamics_factory));
   gimp_tag_cache_add_container (gimp->tag_cache,
                                 gimp_data_factory_get_container (gimp->pattern_factory));
   gimp_tag_cache_add_container (gimp->tag_cache,
                                 gimp_data_factory_get_container (gimp->gradient_factory));
   gimp_tag_cache_add_container (gimp->tag_cache,
                                 gimp_data_factory_get_container (gimp->palette_factory));
+  gimp_tag_cache_add_container (gimp->tag_cache,
+                                gimp_data_factory_get_container (gimp->tool_preset_factory));
 
   g_signal_emit (gimp, gimp_signals[RESTORE], 0, status_callback);
 }
@@ -950,6 +1066,14 @@ gimp_get_display_iter (Gimp *gimp)
   return GIMP_LIST (gimp->displays)->list;
 }
 
+GList *
+gimp_get_image_windows (Gimp *gimp)
+{
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
+
+  return g_list_copy (gimp->image_windows);
+}
+
 GList *
 gimp_get_tool_info_iter (Gimp *gimp)
 {
