@@ -46,6 +46,7 @@ struct _Gimp
   gboolean                no_data;
   gboolean                no_fonts;
   gboolean                no_interface;
+  gboolean                show_gui;
   gboolean                use_shm;
   GimpMessageHandlerType  message_handler;
   gboolean                console_messages;
@@ -84,15 +85,19 @@ struct _Gimp
   GimpContainer          *displays;
   gint                    next_display_ID;
 
+  GList                  *image_windows;
+
   GimpBuffer             *global_buffer;
   GimpContainer          *named_buffers;
 
   GimpContainer          *fonts;
 
   GimpDataFactory        *brush_factory;
+  GimpDataFactory        *dynamics_factory;
   GimpDataFactory        *pattern_factory;
   GimpDataFactory        *gradient_factory;
   GimpDataFactory        *palette_factory;
+  GimpDataFactory        *tool_preset_factory;
 
   GimpTagCache           *tag_cache;
 
@@ -133,7 +138,7 @@ struct _GimpClass
 
   /*  emitted if an image is loaded and opened with a display  */
   void     (* image_opened)   (Gimp               *gimp,
-			       const gchar        *uri);
+                               const gchar        *uri);
 };
 
 
@@ -149,6 +154,9 @@ Gimp         * gimp_new                  (const gchar         *name,
                                           gboolean             console_messages,
                                           GimpStackTraceMode   stack_trace_mode,
                                           GimpPDBCompatMode    pdb_compat_mode);
+void           gimp_set_show_gui         (Gimp                *gimp,
+                                          gboolean             show_gui);
+gboolean       gimp_get_show_gui         (Gimp                *gimp);
 
 void           gimp_load_config          (Gimp                *gimp,
                                           const gchar         *alternate_system_gimprc,
@@ -164,6 +172,7 @@ void           gimp_exit                 (Gimp                *gimp,
 
 GList        * gimp_get_image_iter       (Gimp                *gimp);
 GList        * gimp_get_display_iter     (Gimp                *gimp);
+GList        * gimp_get_image_windows    (Gimp                *gimp);
 GList        * gimp_get_tool_info_iter   (Gimp                *gimp);
 
 void           gimp_set_global_buffer    (Gimp                *gimp,
@@ -202,7 +211,7 @@ void           gimp_message_literal      (Gimp                *gimp,
                                           const gchar         *message);
 
 void           gimp_image_opened         (Gimp                *gimp,
-					  const gchar         *uri);
+                                          const gchar         *uri);
 
 gboolean       gimp_use_gegl             (Gimp                *gimp);
 
