@@ -29,23 +29,27 @@
 #include "gimpbrush-transform.h"
 
 #include "base/temp-buf.h"
+#include "base/pixel-region.h"
 
+#include "paint-funcs/paint-funcs.h"
 
 /*  local function prototypes  */
 
-static void  gimp_brush_transform_matrix       (GimpBrush         *brush,
-                                                gdouble            scale_x,
-                                                gdouble            scale_y,
-                                                gdouble            angle,
-                                                GimpMatrix3       *matrix);
-static void  gimp_brush_transform_bounding_box (GimpBrush         *brush,
-                                                const GimpMatrix3 *matrix,
-                                                gint              *x,
-                                                gint              *y,
-                                                gint              *width,
-                                                gint              *height);
-
+static void    gimp_brush_transform_bounding_box (TempBuf           *brush,
+                                                  const GimpMatrix3 *matrix,
+                                                  gint              *x,
+                                                  gint              *y,
+                                                  gint              *width,
+                                                  gint              *height);
 
+static gdouble gimp_brush_transform_array_sum    (gfloat            *arr,
+                                                  gint               len);
+static void    gimp_brush_transform_fill_blur_kernel (gfloat            *arr,
+                                                      gint               len);
+static gint    gimp_brush_transform_blur_kernel_size (gint height,
+                                                      gint width,
+                                                      gdouble hardness);
+#define MAX_BLUR_KERNEL   15
 /*  public functions  */
 
 void
@@ -59,12 +63,10 @@ gimp_brush_real_transform_size (GimpBrush *brush,
   GimpMatrix3 matrix;
   gint        x, y;
 
-  if (aspect_ratio < 1.0)
-    gimp_brush_transform_matrix (brush, scale * aspect_ratio, scale, angle, &matrix);
-  else
-    gimp_brush_transform_matrix (brush, scale, scale / aspect_ratio, angle, &matrix);
+  gimp_brush_transform_matrix (brush->mask->width, brush->mask->height,
+                               scale, aspect_ratio, angle, &matrix);
 
-  gimp_brush_transform_bounding_box (brush, &matrix, &x, &y, width, height);
+  gimp_brush_transform_bounding_box (brush->mask, &matrix, &x, &y, width, height);
 }
 
 /*
@@ -89,19 +91,26 @@ gimp_brush_real_transform_size (GimpBrush *brush,
  * than the input brush size.
  *
  * There are no floating point calculations in the inner loop for speed.
+ *
+ * Some variables end with the suffix _i to indicate they have been
+ * premultiplied by int_multiple
  */
 TempBuf *
 gimp_brush_real_transform_mask (GimpBrush *brush,
                                 gdouble    scale,
                                 gdouble    aspect_ratio,
-                                gdouble    angle)
+                                gdouble    angle,
+                                gdouble    hardness)
 {
   TempBuf      *result;
+  TempBuf      *source;
   guchar       *dest;
   const guchar *src;
   GimpMatrix3   matrix;
   gint          src_width;
   gint          src_height;
+  gint          src_width_minus_one;
+  gint          src_height_minus_one;
   gint          dest_width;
   gint          dest_height;
   gint          x, y;
@@ -111,24 +120,22 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
   gdouble       src_tl_to_tr_delta_y;
   gdouble       src_tl_to_bl_delta_x;
   gdouble       src_tl_to_bl_delta_y;
-  gint          src_walk_ux;
-  gint          src_walk_uy;
-  gint          src_walk_vx;
-  gint          src_walk_vy;
+  gint          src_walk_ux_i;
+  gint          src_walk_uy_i;
+  gint          src_walk_vx_i;
+  gint          src_walk_vy_i;
   gint          src_space_cur_pos_x;
   gint          src_space_cur_pos_y;
-  gint          src_space_row_start_x;
-  gint          src_space_row_start_y;
+  gint          src_space_cur_pos_x_i;
+  gint          src_space_cur_pos_y_i;
+  gint          src_space_row_start_x_i;
+  gint          src_space_row_start_y_i;
   const guchar *src_walker;
   const guchar *pixel_next;
   const guchar *pixel_below;
   const guchar *pixel_below_next;
   gint          opposite_x, distance_from_true_x;
   gint          opposite_y, distance_from_true_y;
-  gint          src_height_times_int_multiple;
-  gint          src_width_times_int_multiple;
-  gint          src_heightm1_times_int_multiple;
-  gint          src_widthm1_times_int_multiple;
 
   /*
    * tl, tr etc are used because it is easier to visualize top left,
@@ -136,7 +143,7 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
    * rectangle.
    */
   const gint fraction_bits = 12;
-  const gint int_multiple  = pow(2,fraction_bits);
+  const gint int_multiple  = pow (2, fraction_bits);
 
   /* In inner loop's bilinear calculation, two numbers that were each
    * previously multiplied by int_multiple are multiplied together.
@@ -160,19 +167,20 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
    */
   const guint fraction_bitmask = pow(2, fraction_bits) - 1 ;
 
+  source = brush->mask;
 
-   if (aspect_ratio < 1.0)
-    gimp_brush_transform_matrix (brush, scale * aspect_ratio, scale, angle, &matrix);
-  else
-    gimp_brush_transform_matrix (brush, scale, scale / aspect_ratio, angle, &matrix);
+  gimp_brush_transform_matrix (source->height, source->width,
+                               scale, aspect_ratio, angle, &matrix);
 
   if (gimp_matrix3_is_identity (&matrix))
-    return temp_buf_copy (brush->mask, NULL);
+    return temp_buf_copy (source, NULL);
 
-  src_width  = brush->mask->width;
-  src_height = brush->mask->height;
+  src_width  = source->width;
+  src_height = source->height;
+  src_width_minus_one  = src_width  - 1;
+  src_height_minus_one = src_height - 1;
 
-  gimp_brush_transform_bounding_box (brush, &matrix,
+  gimp_brush_transform_bounding_box (source, &matrix,
                                      &x, &y, &dest_width, &dest_height);
   gimp_matrix3_translate (&matrix, -x, -y);
   gimp_matrix3_invert (&matrix);
@@ -180,7 +188,7 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
   result = temp_buf_new (dest_width, dest_height, 1, 0, 0, NULL);
 
   dest = temp_buf_get_data (result);
-  src  = temp_buf_get_data (brush->mask);
+  src  = temp_buf_get_data (source);
 
   /* prevent disappearance of 1x1 pixel brush at some rotations when
      scaling < 1 */
@@ -210,34 +218,29 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
   src_tl_to_bl_delta_y = bly - tly;
 
   /* speed optimized, note conversion to int precision */
-  src_walk_ux = (gint) ((src_tl_to_tr_delta_x / dest_width)* int_multiple);
-  src_walk_uy = (gint) ((src_tl_to_tr_delta_y / dest_width)* int_multiple);
-  src_walk_vx = (gint) ((src_tl_to_bl_delta_x / dest_height)* int_multiple);
-  src_walk_vy = (gint) ((src_tl_to_bl_delta_y / dest_height)* int_multiple);
+  src_walk_ux_i = (gint) ((src_tl_to_tr_delta_x / dest_width)* int_multiple);
+  src_walk_uy_i = (gint) ((src_tl_to_tr_delta_y / dest_width)* int_multiple);
+  src_walk_vx_i = (gint) ((src_tl_to_bl_delta_x / dest_height)* int_multiple);
+  src_walk_vy_i = (gint) ((src_tl_to_bl_delta_y / dest_height)* int_multiple);
 
   /* initialize current position in source space to the start position (tl)
    * speed optimized, note conversion to int precision
    */
-  src_space_cur_pos_x    = (gint) (tlx* int_multiple);
-  src_space_cur_pos_y    = (gint) (tly* int_multiple);
-  src_space_row_start_x  = (gint) (tlx* int_multiple);
-  src_space_row_start_y  = (gint) (tly* int_multiple);
+  src_space_cur_pos_x_i   = (gint) (tlx* int_multiple);
+  src_space_cur_pos_y_i   = (gint) (tly* int_multiple);
+  src_space_cur_pos_x     = (gint) (src_space_cur_pos_x_i >> fraction_bits);
+  src_space_cur_pos_y     = (gint) (src_space_cur_pos_y_i >> fraction_bits);
+  src_space_row_start_x_i = (gint) (tlx* int_multiple);
+  src_space_row_start_y_i = (gint) (tly* int_multiple);
 
-  src_walker = src;
-
-  src_height_times_int_multiple = src_height << fraction_bits; /* mult by int_multiple */
-  src_width_times_int_multiple  = src_width  << fraction_bits; /* mult by int_multiple */
-
-  src_heightm1_times_int_multiple = src_height_times_int_multiple - int_multiple;
-  src_widthm1_times_int_multiple  = src_width_times_int_multiple - int_multiple;
 
   for (y = 0; y < dest_height; y++)
     {
       for (x = 0; x < dest_width; x++)
         {
-          if (src_space_cur_pos_x > src_width_times_int_multiple  ||
+          if (src_space_cur_pos_x > src_width_minus_one ||
               src_space_cur_pos_x < 0     ||
-              src_space_cur_pos_y > src_height_times_int_multiple ||
+              src_space_cur_pos_y > src_height_minus_one ||
               src_space_cur_pos_y < 0)
               /* no corresponding pixel in source space */
             {
@@ -246,15 +249,15 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
           else /* reverse transformed point hits source pixel */
             {
               src_walker = src
-                        + (src_space_cur_pos_y>>fraction_bits) * src_width
-                        + (src_space_cur_pos_x>>fraction_bits);
+                        + src_space_cur_pos_y * src_width
+                        + src_space_cur_pos_x;
 
               /* bottom right corner
                * no pixel below, reuse current pixel instead
                * no next pixel to the right so reuse current pixel instead
                */
-              if (src_space_cur_pos_y > (src_heightm1_times_int_multiple) &&
-                  src_space_cur_pos_x > (src_widthm1_times_int_multiple) )
+              if (src_space_cur_pos_y == src_height_minus_one &&
+                  src_space_cur_pos_x == src_width_minus_one )
                 {
                   pixel_next       = src_walker;
                   pixel_below      = src_walker;
@@ -263,7 +266,7 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
 
               /* bottom edge pixel row, except rightmost corner
                * no pixel below, reuse current pixel instead  */
-              else if (src_space_cur_pos_y > (src_heightm1_times_int_multiple))
+              else if (src_space_cur_pos_y == src_height_minus_one)
                 {
                   pixel_next       = src_walker + 1;
                   pixel_below      = src_walker;
@@ -272,7 +275,7 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
 
               /* right edge pixel column, except bottom corner
                * no next pixel to the right so reuse current pixel instead */
-              else if (src_space_cur_pos_x > (src_widthm1_times_int_multiple))
+              else if (src_space_cur_pos_x == src_width_minus_one)
                 {
                   pixel_next       = src_walker;
                   pixel_below      = src_walker + src_width;
@@ -287,28 +290,64 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
                   pixel_below_next = pixel_below + 1;
                 }
 
-              distance_from_true_x = src_space_cur_pos_x & fraction_bitmask;
-              distance_from_true_y = src_space_cur_pos_y & fraction_bitmask;
+              distance_from_true_x = src_space_cur_pos_x_i & fraction_bitmask;
+              distance_from_true_y = src_space_cur_pos_y_i & fraction_bitmask;
               opposite_x =  int_multiple - distance_from_true_x;
               opposite_y =  int_multiple - distance_from_true_y;
 
-              *dest = ((src_walker[0] * opposite_x + pixel_next[0] * distance_from_true_x) * opposite_y +
-                       (pixel_below[0] * opposite_x + pixel_below_next[0] *distance_from_true_x) * distance_from_true_y
+              *dest = ((src_walker[0] * opposite_x +
+                        pixel_next[0] * distance_from_true_x) * opposite_y +
+                       (pixel_below[0] * opposite_x +
+                        pixel_below_next[0] *distance_from_true_x) * distance_from_true_y
                        ) >> recovery_bits;
             }
 
-          src_space_cur_pos_x+=src_walk_ux;
-          src_space_cur_pos_y+=src_walk_uy;
+          src_space_cur_pos_x_i+=src_walk_ux_i;
+          src_space_cur_pos_y_i+=src_walk_uy_i;
+
+          src_space_cur_pos_x = src_space_cur_pos_x_i >> fraction_bits;
+          src_space_cur_pos_y = src_space_cur_pos_y_i >> fraction_bits;
+
           dest ++;
         } /* end for x */
 
-        src_space_row_start_x +=src_walk_vx;
-        src_space_row_start_y +=src_walk_vy;
-        src_space_cur_pos_x = src_space_row_start_x;
-        src_space_cur_pos_y = src_space_row_start_y;
+        src_space_row_start_x_i +=src_walk_vx_i;
+        src_space_row_start_y_i +=src_walk_vy_i;
+        src_space_cur_pos_x_i = src_space_row_start_x_i;
+        src_space_cur_pos_y_i = src_space_row_start_y_i;
+
+        src_space_cur_pos_x = src_space_cur_pos_x_i >> fraction_bits;
+        src_space_cur_pos_y = src_space_cur_pos_y_i >> fraction_bits;
 
     } /* end for y */
 
+  if (hardness < 1.0)
+    {
+      TempBuf      *blur_src;
+      PixelRegion  srcPR;
+      PixelRegion  destPR;
+      gint kernel_size = gimp_brush_transform_blur_kernel_size ( result->height,
+                                                                 result->width,
+                                                                 hardness);
+      gint kernel_len  = kernel_size * kernel_size;
+      gfloat blur_kernel [kernel_len];
+
+      gimp_brush_transform_fill_blur_kernel ( blur_kernel, kernel_len);
+
+      blur_src = temp_buf_copy (result, NULL);
+
+      pixel_region_init_temp_buf (&srcPR, blur_src,
+                                  blur_src->x, blur_src->y,
+                                  blur_src->width, blur_src->height);
+      pixel_region_init_temp_buf (&destPR, result,
+                                  result->x, result->y,
+                                  result->width, result->height);
+
+      convolve_region (&srcPR, &destPR, blur_kernel, kernel_size,
+                       gimp_brush_transform_array_sum (blur_kernel, kernel_len),
+                       GIMP_NORMAL_CONVOL, FALSE);
+    }
+
   return result;
 }
 
@@ -339,19 +378,26 @@ gimp_brush_real_transform_mask (GimpBrush *brush,
  * than the input brush size.
  *
  * There are no floating point calculations in the inner loop for speed.
+ *
+ * Some variables end with the suffix _i to indicate they have been
+ * premultiplied by int_multiple
  */
 TempBuf *
 gimp_brush_real_transform_pixmap (GimpBrush *brush,
                                   gdouble    scale,
                                   gdouble    aspect_ratio,
-                                  gdouble    angle)
+                                  gdouble    angle,
+                                  gdouble    hardness)
 {
   TempBuf      *result;
+  TempBuf      *source;
   guchar       *dest;
   const guchar *src;
   GimpMatrix3   matrix;
   gint          src_width;
   gint          src_height;
+  gint          src_width_minus_one;
+  gint          src_height_minus_one;
   gint          dest_width;
   gint          dest_height;
   gint          x, y;
@@ -361,24 +407,22 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
   gdouble       src_tl_to_tr_delta_y;
   gdouble       src_tl_to_bl_delta_x;
   gdouble       src_tl_to_bl_delta_y;
-  gint          src_walk_ux;
-  gint          src_walk_uy;
-  gint          src_walk_vx;
-  gint          src_walk_vy;
+  gint          src_walk_ux_i;
+  gint          src_walk_uy_i;
+  gint          src_walk_vx_i;
+  gint          src_walk_vy_i;
   gint          src_space_cur_pos_x;
   gint          src_space_cur_pos_y;
-  gint          src_space_row_start_x;
-  gint          src_space_row_start_y;
+  gint          src_space_cur_pos_x_i;
+  gint          src_space_cur_pos_y_i;
+  gint          src_space_row_start_x_i;
+  gint          src_space_row_start_y_i;
   const guchar *src_walker;
   const guchar *pixel_next;
   const guchar *pixel_below;
   const guchar *pixel_below_next;
   gint          opposite_x, distance_from_true_x;
   gint          opposite_y, distance_from_true_y;
-  gint          src_height_times_int_multiple;
-  gint          src_width_times_int_multiple;
-  gint          src_heightm1_times_int_multiple;
-  gint          src_widthm1_times_int_multiple;
 
   /*
    * tl, tr etc are used because it is easier to visualize top left,
@@ -410,19 +454,20 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
    */
   const guint fraction_bitmask = pow(2, fraction_bits)- 1 ;
 
+  source = brush->pixmap;
 
-  if (aspect_ratio < 1.0)
-    gimp_brush_transform_matrix (brush, scale * aspect_ratio, scale, angle, &matrix);
-  else
-    gimp_brush_transform_matrix (brush, scale, scale / aspect_ratio, angle, &matrix);
+  gimp_brush_transform_matrix (source->height, source->width,
+                               scale, aspect_ratio, angle, &matrix);
 
   if (gimp_matrix3_is_identity (&matrix))
-    return temp_buf_copy (brush->pixmap, NULL);
+    return temp_buf_copy (source, NULL);
 
-  src_width  = brush->pixmap->width;
-  src_height = brush->pixmap->height;
+  src_width  = source->width;
+  src_height = source->height;
+  src_width_minus_one  = src_width  - 1;
+  src_height_minus_one = src_height - 1;
 
-  gimp_brush_transform_bounding_box (brush, &matrix,
+  gimp_brush_transform_bounding_box (source, &matrix,
                                      &x, &y, &dest_width, &dest_height);
   gimp_matrix3_translate (&matrix, -x, -y);
   gimp_matrix3_invert (&matrix);
@@ -430,7 +475,7 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
   result = temp_buf_new (dest_width, dest_height, 3, 0, 0, NULL);
 
   dest = temp_buf_get_data (result);
-  src  = temp_buf_get_data (brush->pixmap);
+  src  = temp_buf_get_data (source);
 
   gimp_matrix3_transform_point (&matrix, 0,          0,           &tlx, &tly);
   gimp_matrix3_transform_point (&matrix, dest_width, 0,           &trx, &try);
@@ -451,34 +496,29 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
   src_tl_to_bl_delta_y = bly - tly;
 
   /* speed optimized, note conversion to int precision */
-  src_walk_ux = (gint) ((src_tl_to_tr_delta_x / dest_width)* int_multiple);
-  src_walk_uy = (gint) ((src_tl_to_tr_delta_y / dest_width)* int_multiple);
-  src_walk_vx = (gint) ((src_tl_to_bl_delta_x / dest_height)* int_multiple);
-  src_walk_vy = (gint) ((src_tl_to_bl_delta_y / dest_height)* int_multiple);
+  src_walk_ux_i = (gint) ((src_tl_to_tr_delta_x / dest_width)* int_multiple);
+  src_walk_uy_i = (gint) ((src_tl_to_tr_delta_y / dest_width)* int_multiple);
+  src_walk_vx_i = (gint) ((src_tl_to_bl_delta_x / dest_height)* int_multiple);
+  src_walk_vy_i = (gint) ((src_tl_to_bl_delta_y / dest_height)* int_multiple);
 
   /* initialize current position in source space to the start position (tl)
    * speed optimized, note conversion to int precision
    */
-  src_space_cur_pos_x    = (gint) (tlx* int_multiple);
-  src_space_cur_pos_y    = (gint) (tly* int_multiple);
-  src_space_row_start_x  = (gint) (tlx* int_multiple);
-  src_space_row_start_y  = (gint) (tly* int_multiple);
+  src_space_cur_pos_x_i    = (gint) (tlx* int_multiple);
+  src_space_cur_pos_y_i    = (gint) (tly* int_multiple);
+  src_space_cur_pos_x      = (gint) (src_space_cur_pos_x_i >> fraction_bits);
+  src_space_cur_pos_y      = (gint) (src_space_cur_pos_y_i >> fraction_bits);
+  src_space_row_start_x_i  = (gint) (tlx* int_multiple);
+  src_space_row_start_y_i  = (gint) (tly* int_multiple);
 
-  src_walker = src;
-
-  src_height_times_int_multiple = src_height << fraction_bits; /* mult by int_multiple */
-  src_width_times_int_multiple  = src_width  << fraction_bits; /* mult by int_multiple */
-
-  src_heightm1_times_int_multiple = src_height_times_int_multiple - int_multiple;
-  src_widthm1_times_int_multiple  = src_width_times_int_multiple - int_multiple;
 
   for (y = 0; y < dest_height; y++)
     {
       for (x = 0; x < dest_width; x++)
         {
-          if (src_space_cur_pos_x > src_width_times_int_multiple  ||
+          if (src_space_cur_pos_x > src_width_minus_one  ||
               src_space_cur_pos_x < 0     ||
-              src_space_cur_pos_y > src_height_times_int_multiple ||
+              src_space_cur_pos_y > src_height_minus_one ||
               src_space_cur_pos_y < 0)
               /* no corresponding pixel in source space */
             {
@@ -490,15 +530,15 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
             {
               src_walker = src
                         + 3 * (
-                          (src_space_cur_pos_y >> fraction_bits) * src_width
-                        + (src_space_cur_pos_x >> fraction_bits));
+                          src_space_cur_pos_y * src_width
+                        + src_space_cur_pos_x);
 
               /* bottom right corner
                * no pixel below, reuse current pixel instead
                * no next pixel to the right so reuse current pixel instead
                */
-              if (src_space_cur_pos_y > (src_heightm1_times_int_multiple) &&
-                  src_space_cur_pos_x > (src_widthm1_times_int_multiple) )
+              if (src_space_cur_pos_y == src_height_minus_one &&
+                  src_space_cur_pos_x == src_width_minus_one )
                 {
                   pixel_next  = src_walker;
                   pixel_below = src_walker;
@@ -507,7 +547,7 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
 
               /* bottom edge pixel row, except rightmost corner
                * no pixel below, reuse current pixel instead  */
-              else if (src_space_cur_pos_y > (src_heightm1_times_int_multiple))
+              else if (src_space_cur_pos_y == src_height_minus_one)
                 {
                   pixel_next  = src_walker + 3;
                   pixel_below = src_walker;
@@ -516,7 +556,7 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
 
               /* right edge pixel column, except bottom corner
                * no next pixel to the right so reuse current pixel instead */
-              else if (src_space_cur_pos_x > (src_widthm1_times_int_multiple))
+              else if (src_space_cur_pos_x == src_width_minus_one)
                 {
                   pixel_next  = src_walker;
                   pixel_below = src_walker + src_width * 3;
@@ -531,80 +571,183 @@ gimp_brush_real_transform_pixmap (GimpBrush *brush,
                   pixel_below_next = pixel_below + 3;
                 }
 
-              distance_from_true_x = src_space_cur_pos_x & fraction_bitmask;
-              distance_from_true_y = src_space_cur_pos_y & fraction_bitmask;
+              distance_from_true_x = src_space_cur_pos_x_i & fraction_bitmask;
+              distance_from_true_y = src_space_cur_pos_y_i & fraction_bitmask;
               opposite_x =  int_multiple - distance_from_true_x;
               opposite_y =  int_multiple - distance_from_true_y;
 
-              dest[0] = ((src_walker[0] * opposite_x + pixel_next[0] * distance_from_true_x) * opposite_y +
-                         (pixel_below[0] * opposite_x + pixel_below_next[0] *distance_from_true_x) * distance_from_true_y
+              dest[0] = ((src_walker[0] * opposite_x +
+                          pixel_next[0] * distance_from_true_x) * opposite_y +
+                         (pixel_below[0] * opposite_x +
+                          pixel_below_next[0] *distance_from_true_x) * distance_from_true_y
                         ) >> recovery_bits;
 
-              dest[1] = ((src_walker[1] * opposite_x + pixel_next[1] * distance_from_true_x) * opposite_y +
-                         (pixel_below[1] * opposite_x + pixel_below_next[1] *distance_from_true_x) * distance_from_true_y
+              dest[1] = ((src_walker[1] * opposite_x +
+                          pixel_next[1] * distance_from_true_x) * opposite_y +
+                         (pixel_below[1] * opposite_x +
+                          pixel_below_next[1] *distance_from_true_x) * distance_from_true_y
                         ) >> recovery_bits;
 
-              dest[2] = ((src_walker[2] * opposite_x + pixel_next[2] * distance_from_true_x) * opposite_y +
-                         (pixel_below[2] * opposite_x + pixel_below_next[2] *distance_from_true_x) * distance_from_true_y
+              dest[2] = ((src_walker[2] * opposite_x +
+                          pixel_next[2] * distance_from_true_x) * opposite_y +
+                         (pixel_below[2] * opposite_x +
+                          pixel_below_next[2] *distance_from_true_x) * distance_from_true_y
                         ) >> recovery_bits;
             }
 
-          src_space_cur_pos_x += src_walk_ux;
-          src_space_cur_pos_y += src_walk_uy;
+          src_space_cur_pos_x_i += src_walk_ux_i;
+          src_space_cur_pos_y_i += src_walk_uy_i;
+
+          src_space_cur_pos_x = src_space_cur_pos_x_i >> fraction_bits;
+          src_space_cur_pos_y = src_space_cur_pos_y_i >> fraction_bits;
+
           dest += 3;
         } /* end for x */
-        src_space_row_start_x +=src_walk_vx;
-        src_space_row_start_y +=src_walk_vy;
-        src_space_cur_pos_x = src_space_row_start_x;
-        src_space_cur_pos_y = src_space_row_start_y;
 
+        src_space_row_start_x_i +=src_walk_vx_i;
+        src_space_row_start_y_i +=src_walk_vy_i;
+        src_space_cur_pos_x_i = src_space_row_start_x_i;
+        src_space_cur_pos_y_i = src_space_row_start_y_i;
+
+        src_space_cur_pos_x = src_space_cur_pos_x_i >> fraction_bits;
+        src_space_cur_pos_y = src_space_cur_pos_y_i >> fraction_bits;
     } /* end for y */
 
+  if (hardness < 1.0)
+    {
+      TempBuf      *blur_src;
+      PixelRegion  srcPR;
+      PixelRegion  destPR;
+      gint kernel_size = gimp_brush_transform_blur_kernel_size ( result->height,
+                                                                 result->width,
+                                                                 hardness);
+      gint kernel_len  = kernel_size * kernel_size;
+      gfloat blur_kernel [kernel_len];
+
+      gimp_brush_transform_fill_blur_kernel ( blur_kernel, kernel_len);
+
+      blur_src = temp_buf_copy (result, NULL);
+
+      pixel_region_init_temp_buf (&srcPR, blur_src, blur_src->x, blur_src->y, blur_src->width, blur_src->height);
+      pixel_region_init_temp_buf (&destPR, result, result->x, result->y, result->width, result->height);
+
+      convolve_region (&srcPR, &destPR, blur_kernel, kernel_size,
+                       gimp_brush_transform_array_sum(blur_kernel, kernel_len),
+                       GIMP_NORMAL_CONVOL, FALSE);
+
+    }
 
   return result;
 }
 
 
-/*  private functions  */
 
-static void
-gimp_brush_transform_matrix (GimpBrush   *brush,
-                             gdouble      scale_x,
-                             gdouble      scale_y,
+void
+gimp_brush_transform_matrix (gdouble      width,
+                             gdouble      height,
+                             gdouble      scale,
+                             gdouble      aspect_ratio,
                              gdouble      angle,
                              GimpMatrix3 *matrix)
 {
-  const gdouble center_x = brush->mask->width  / 2;
-  const gdouble center_y = brush->mask->height / 2;
+  const gdouble center_x = width  / 2;
+  const gdouble center_y = height / 2;
+  gdouble scale_x;
+  gdouble scale_y;
+
+  if (aspect_ratio < 1.0)
+    {
+        scale_x = scale * aspect_ratio;
+        scale_y = scale;
+    }
+  else
+    {
+      scale_x = scale;
+      scale_y = scale / aspect_ratio;
+    }
 
   gimp_matrix3_identity (matrix);
-  gimp_matrix3_translate (matrix, - center_x, - center_y);
-  gimp_matrix3_rotate (matrix, -2 * G_PI * angle);
-  gimp_matrix3_translate (matrix, center_x, center_y);
   gimp_matrix3_scale (matrix, scale_x, scale_y);
+  gimp_matrix3_translate (matrix, - center_x * scale_x, - center_y * scale_y);
+  gimp_matrix3_rotate (matrix, -2 * G_PI * angle);
+  gimp_matrix3_translate (matrix, center_x * scale_x, center_y * scale_y);
+
 }
 
+/*  private functions  */
+
 static void
-gimp_brush_transform_bounding_box (GimpBrush         *brush,
+gimp_brush_transform_bounding_box (TempBuf           *brush,
                                    const GimpMatrix3 *matrix,
                                    gint              *x,
                                    gint              *y,
                                    gint              *width,
                                    gint              *height)
 {
-  const gdouble  w = brush->mask->width;
-  const gdouble  h = brush->mask->height;
+  const gdouble  w = brush->width;
+  const gdouble  h = brush->height;
   gdouble        x1, x2, x3, x4;
   gdouble        y1, y2, y3, y4;
+  gdouble        temp_x;
+  gdouble        temp_y;
 
   gimp_matrix3_transform_point (matrix, 0, 0, &x1, &y1);
   gimp_matrix3_transform_point (matrix, w, 0, &x2, &y2);
   gimp_matrix3_transform_point (matrix, 0, h, &x3, &y3);
   gimp_matrix3_transform_point (matrix, w, h, &x4, &y4);
 
-  *x = floor (MIN (MIN (x1, x2), MIN (x3, x4)));
-  *y = floor (MIN (MIN (y1, y2), MIN (y3, y4)));
+  temp_x = MIN (MIN (x1, x2), MIN (x3, x4));
+  temp_y = MIN (MIN (y1, y2), MIN (y3, y4));
+
+  *width  = (gint) ceil (MAX (MAX (x1, x2), MAX (x3, x4)) - temp_x);
+  *height = (gint) ceil (MAX (MAX (y1, y2), MAX (y3, y4)) - temp_y);
+
+  *x = floor (temp_x);
+  *y = floor (temp_y);
+
+  /* Transform size can not be less than 1 px */
+  *width  = MAX(1, *width);
+  *height = MAX(1, *height);
 
-  *width  = (gint) (ceil  (MAX (MAX (x1, x2), MAX (x3, x4))) - *x);
-  *height = (gint) (ceil  (MAX (MAX (y1, y2), MAX (y3, y4))) - *y);
+}
+
+static gdouble
+gimp_brush_transform_array_sum (gfloat *arr, gint len)
+{
+  int i;
+  gfloat total = 0;
+  for (i = 0; i < len; i++)
+    {
+      total += arr [i];
+    }
+  return total;
+}
+
+static void
+gimp_brush_transform_fill_blur_kernel (gfloat *arr, gint len)
+{
+  int i;
+  gint half_point = ((gint) len / 2) + 1;
+
+  for (i = 0; i < len; i++)
+    {
+      if (i < half_point)
+        arr [i] = half_point - i;
+      else arr [i] = i - half_point;
+    }
+}
+
+static gint
+gimp_brush_transform_blur_kernel_size (gint height,
+                                       gint width,
+                                       gdouble hardness)
+{
+
+  gint kernel_size = MIN (MAX_BLUR_KERNEL, MIN(width, height)) *
+                        ((MIN (width, height) * (1.0 - hardness)) / MIN (width, height));
+
+  /*Kernel size must be odd*/
+  if ( kernel_size % 2 == 0) kernel_size++;
+
+  return kernel_size;
 }
