@@ -32,11 +32,20 @@ void      gimp_brush_real_transform_size   (GimpBrush *brush,
 TempBuf * gimp_brush_real_transform_mask   (GimpBrush *brush,
                                             gdouble    scale,
                                             gdouble    aspect_ratio,
-                                            gdouble    angle);
+                                            gdouble    angle,
+                                            gdouble    hardness);
 TempBuf * gimp_brush_real_transform_pixmap (GimpBrush *brush,
                                             gdouble    scale,
                                             gdouble    aspect_ratio,
-                                            gdouble    angle);
+                                            gdouble    angle,
+                                            gdouble    hardness);
+
+void        gimp_brush_transform_matrix    (gdouble            width,
+                                            gdouble            height,
+                                            gdouble            scale,
+                                            gdouble            aspect_ratio,
+                                            gdouble            angle,
+                                            GimpMatrix3       *matrix);
 
 
 #endif  /*  __GIMP_BRUSH_SCALE_H__  */
