@@ -277,13 +277,13 @@ gimp_brush_get_new_preview (GimpViewable *viewable,
 
       if (scale != 1.0)
         {
-          mask_buf = gimp_brush_transform_mask (brush, scale, 1.0, 0.0);
+          mask_buf = gimp_brush_transform_mask (brush, scale, 1.0, 0.0, 1.0);
 
           if (! mask_buf)
             mask_buf = temp_buf_new (1, 1, 1, 0, 0, transp);
 
           if (pixmap_buf)
-            pixmap_buf = gimp_brush_transform_pixmap (brush, scale, 1.0, 0.0);
+            pixmap_buf = gimp_brush_transform_pixmap (brush, scale, 1.0, 0.0, 1.0);
 
           mask_width  = mask_buf->width;
           mask_height = mask_buf->height;
@@ -344,7 +344,7 @@ gimp_brush_get_description (GimpViewable  *viewable,
   GimpBrush *brush = GIMP_BRUSH (viewable);
 
   return g_strdup_printf ("%s (%d × %d)",
-                          GIMP_OBJECT (brush)->name,
+                          gimp_object_get_name (brush),
                           brush->mask->width,
                           brush->mask->height);
 }
@@ -399,7 +399,8 @@ gimp_brush_get_checksum (GimpTagged *tagged)
 /*  public functions  */
 
 GimpData *
-gimp_brush_new (const gchar *name)
+gimp_brush_new (GimpContext *context,
+                const gchar *name)
 {
   g_return_val_if_fail (name != NULL, NULL);
 
@@ -409,20 +410,19 @@ gimp_brush_new (const gchar *name)
 }
 
 GimpData *
-gimp_brush_get_standard (void)
+gimp_brush_get_standard (GimpContext *context)
 {
   static GimpData *standard_brush = NULL;
 
   if (! standard_brush)
     {
-      standard_brush = gimp_brush_new ("Standard");
+      standard_brush = gimp_brush_new (context, "Standard");
 
-      standard_brush->dirty = FALSE;
-      gimp_data_make_internal (standard_brush,
-                               "gimp-brush-standard");
+      gimp_data_clean (standard_brush);
+      gimp_data_make_internal (standard_brush, "gimp-brush-standard");
 
-      /*  set ref_count to 2 --> never swap the standard brush  */
-      g_object_ref (standard_brush);
+      g_object_add_weak_pointer (G_OBJECT (standard_brush),
+                                 (gpointer *) &standard_brush);
     }
 
   return standard_brush;
@@ -485,23 +485,25 @@ TempBuf *
 gimp_brush_transform_mask (GimpBrush *brush,
                            gdouble    scale,
                            gdouble    aspect_ratio,
-                           gdouble    angle)
+                           gdouble    angle,
+                           gdouble    hardness)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
   g_return_val_if_fail (scale > 0.0, NULL);
   g_return_val_if_fail (aspect_ratio > 0.0, NULL);
 
-  if ((scale == 1.0) && (aspect_ratio == 1.0) && (angle == 0.0))
+  if ((scale == 1.0) && (aspect_ratio == 1.0) && (angle == 0.0) && (hardness == 1.0))
     return temp_buf_copy (brush->mask, NULL);
 
-  return GIMP_BRUSH_GET_CLASS (brush)->transform_mask (brush, scale, aspect_ratio, angle);
+  return GIMP_BRUSH_GET_CLASS (brush)->transform_mask (brush, scale, aspect_ratio, angle, hardness);
 }
 
 TempBuf *
 gimp_brush_transform_pixmap (GimpBrush *brush,
                              gdouble    scale,
                              gdouble    aspect_ratio,
-                             gdouble    angle)
+                             gdouble    angle,
+                             gdouble    hardness)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
   g_return_val_if_fail (brush->pixmap != NULL, NULL);
@@ -509,10 +511,10 @@ gimp_brush_transform_pixmap (GimpBrush *brush,
   g_return_val_if_fail (aspect_ratio > 0.0, NULL);
 
 
-  if ((scale == 1.0) && (aspect_ratio == 1.0) && (angle == 0.0))
+  if ((scale == 1.0) && (aspect_ratio == 1.0) && (angle == 0.0) && (hardness == 1.0))
     return temp_buf_copy (brush->pixmap, NULL);
 
-  return GIMP_BRUSH_GET_CLASS (brush)->transform_pixmap (brush, scale, aspect_ratio, angle);
+  return GIMP_BRUSH_GET_CLASS (brush)->transform_pixmap (brush, scale, aspect_ratio, angle, hardness);
 }
 
 TempBuf *
