@@ -65,11 +65,13 @@ struct _GimpBrushClass
   TempBuf   * (* transform_mask)   (GimpBrush        *brush,
                                     gdouble           scale_x,
                                     gdouble           scale_y,
-                                    gdouble           angle);
+                                    gdouble           angle,
+                                    gdouble           hardness);
   TempBuf   * (* transform_pixmap) (GimpBrush        *brush,
                                     gdouble           scale_x,
                                     gdouble           scale_y,
-                                    gdouble           angle);
+                                    gdouble           angle,
+                                    gdouble           hardness);
 
   /*  signals  */
   void        (* spacing_changed)  (GimpBrush        *brush);
@@ -78,8 +80,9 @@ struct _GimpBrushClass
 
 GType       gimp_brush_get_type         (void) G_GNUC_CONST;
 
-GimpData  * gimp_brush_new              (const gchar      *name);
-GimpData  * gimp_brush_get_standard     (void);
+GimpData  * gimp_brush_new              (GimpContext      *context,
+                                         const gchar      *name);
+GimpData  * gimp_brush_get_standard     (GimpContext      *context);
 
 GimpBrush * gimp_brush_select_brush     (GimpBrush        *brush,
                                          const GimpCoords *last_coords,
@@ -98,11 +101,13 @@ void        gimp_brush_transform_size   (GimpBrush        *brush,
 TempBuf   * gimp_brush_transform_mask   (GimpBrush        *brush,
                                          gdouble           scale_x,
                                          gdouble           scale_y,
-                                         gdouble           angle);
+                                         gdouble           angle,
+                                         gdouble           hardness);
 TempBuf   * gimp_brush_transform_pixmap (GimpBrush        *brush,
                                          gdouble           scale_x,
                                          gdouble           scale_y,
-                                         gdouble           angle);
+                                         gdouble           angle,
+                                         gdouble           hardness);
 
 TempBuf   * gimp_brush_get_mask         (const GimpBrush  *brush);
 TempBuf   * gimp_brush_get_pixmap       (const GimpBrush  *brush);
