@@ -74,7 +74,8 @@ static void          gimp_brush_generated_transform_size(GimpBrush    *gbrush,
 static TempBuf     * gimp_brush_generated_transform_mask(GimpBrush    *gbrush,
                                                          gdouble       scale,
                                                          gdouble       aspect_ratio,
-                                                         gdouble       angle);
+                                                         gdouble       angle,
+                                                         gdouble       hardness);
 
 static TempBuf     * gimp_brush_generated_calc          (GimpBrushGenerated      *brush,
                                                          GimpBrushGeneratedShape  shape,
@@ -279,7 +280,7 @@ gimp_brush_generated_duplicate (GimpData *data)
 {
   GimpBrushGenerated *brush = GIMP_BRUSH_GENERATED (data);
 
-  return gimp_brush_generated_new (GIMP_OBJECT (brush)->name,
+  return gimp_brush_generated_new (gimp_object_get_name (brush),
                                    brush->shape,
                                    brush->radius,
                                    brush->spikes,
@@ -299,23 +300,35 @@ gimp_brush_generated_transform_size (GimpBrush *gbrush,
   GimpBrushGenerated *brush = GIMP_BRUSH_GENERATED (gbrush);
   gint                half_width;
   gint                half_height;
+  gdouble             ratio;
 
-  /* Since generated brushes are symmetric the dont have intput
-   * for aspect ratios  < 1.0. its same as rotate by 90 degrees and
-   * 1 / ratio. So we fix the input up for this case.   */
-
-  if (aspect_ratio < 1.0)
+  if (aspect_ratio == 1.0)
     {
-      aspect_ratio = 1.0 / aspect_ratio;
-      angle = angle + 0.25;
+      ratio = brush->aspect_ratio;
     }
+  else
+    {
+      ratio = MIN (aspect_ratio, 20);
+      /* Since generated brushes are symmetric the dont have input
+       * for aspect ratios  < 1.0. its same as rotate by 90 degrees and
+       * 1 / ratio. So we fix the input up for this case.   */
+
+      if (ratio < 1.0)
+        {
+          ratio = MIN (1.0 / ratio, 20);
+          angle = angle + 0.25;
+        }
+    }
+
+
+
 
   gimp_brush_generated_get_half_size (brush,
                                       brush->shape,
                                       brush->radius * scale,
                                       brush->spikes,
                                       brush->hardness,
-                                      brush->aspect_ratio * aspect_ratio,
+                                      ratio,
                                       (brush->angle + 360 * angle),
                                       &half_width, &half_height,
                                       NULL, NULL, NULL, NULL);
@@ -328,26 +341,37 @@ static TempBuf *
 gimp_brush_generated_transform_mask (GimpBrush *gbrush,
                                      gdouble    scale,
                                      gdouble    aspect_ratio,
-                                     gdouble    angle)
+                                     gdouble    angle,
+                                     gdouble    hardness)
 {
   GimpBrushGenerated *brush  = GIMP_BRUSH_GENERATED (gbrush);
+  gdouble             ratio;
 
-  /* Since generated brushes are symmetric the dont have intput
-   * for aspect ratios  < 1.0. its same as rotate by 90 degrees and
-   * 1 / ratio. So we fix the input up for this case.   */
-
-  if (aspect_ratio < 1.0)
+  if (aspect_ratio == 1.0)
     {
-      aspect_ratio = 1.0 / aspect_ratio;
-      angle = angle + 0.25;
+      ratio = brush->aspect_ratio;
     }
+  else
+    {
+      ratio = MIN (aspect_ratio, 20);
+      /* Since generated brushes are symmetric the dont have input
+       * for aspect ratios  < 1.0. its same as rotate by 90 degrees and
+       * 1 / ratio. So we fix the input up for this case.   */
+
+      if (ratio < 1.0)
+        {
+          ratio = MIN (1.0 / ratio, 20);
+          angle = angle + 0.25;
+        }
+    }
+
 
   return gimp_brush_generated_calc (brush,
                                     brush->shape,
                                     brush->radius * scale ,
                                     brush->spikes,
-                                    brush->hardness,
-                                    brush->aspect_ratio * aspect_ratio,
+                                    brush->hardness * hardness,
+                                    ratio,
                                     (brush->angle + 360 * angle),
                                     NULL, NULL);
 }
@@ -558,6 +582,17 @@ gimp_brush_generated_get_half_size (GimpBrushGenerated      *gbrush,
   GimpVector2  x_axis;
   GimpVector2  y_axis;
 
+  /* Since floatongpoint is not really accurate,
+   * we need to round to limit the errors.
+   * Errors in some border cases resulted in
+   * different height and width reported for
+   * the same input value on calling procedure side.
+   * This became problem at the rise of dynamics that
+   * allows for any angle to turn up.
+   **/
+
+  angle_in_degrees = ROUND (angle_in_degrees * 1000.0) / 1000.0;
+
   s = sin (gimp_deg_to_rad (angle_in_degrees));
   c = cos (gimp_deg_to_rad (angle_in_degrees));
 
