@@ -28,8 +28,6 @@
 #include "base/tile-manager-preview.h"
 #include "base/temp-buf.h"
 
-#include "paint-funcs/paint-funcs.h"
-
 #include "gimpbuffer.h"
 
 
@@ -210,7 +208,7 @@ gimp_buffer_get_description (GimpViewable  *viewable,
   GimpBuffer *buffer = GIMP_BUFFER (viewable);
 
   return g_strdup_printf ("%s (%d × %d)",
-                          GIMP_OBJECT (buffer)->name,
+                          gimp_object_get_name (buffer),
                           gimp_buffer_get_width (buffer),
                           gimp_buffer_get_height (buffer));
 }
@@ -234,20 +232,9 @@ gimp_buffer_new (TileManager *tiles,
                          NULL);
 
   if (copy_pixels)
-    {
-      PixelRegion srcPR, destPR;
-
-      buffer->tiles = tile_manager_new (width, height,
-                                        tile_manager_bpp (tiles));
-
-      pixel_region_init (&srcPR, tiles, 0, 0, width, height, FALSE);
-      pixel_region_init (&destPR, buffer->tiles, 0, 0, width, height, TRUE);
-      copy_region (&srcPR, &destPR);
-    }
+    buffer->tiles = tile_manager_duplicate (tiles);
   else
-    {
-      buffer->tiles = tiles;
-    }
+    buffer->tiles = tile_manager_ref (tiles);
 
   return buffer;
 }
@@ -256,6 +243,7 @@ GimpBuffer *
 gimp_buffer_new_from_pixbuf (GdkPixbuf   *pixbuf,
                              const gchar *name)
 {
+  GimpBuffer  *buffer;
   TileManager *tiles;
   guchar      *pixels;
   PixelRegion  destPR;
@@ -284,7 +272,11 @@ gimp_buffer_new_from_pixbuf (GdkPixbuf   *pixbuf,
       pixel_region_set_row (&destPR, 0, y, width, pixels);
    }
 
-  return gimp_buffer_new (tiles, name, FALSE);
+  buffer = gimp_buffer_new (tiles, name, FALSE);
+
+  tile_manager_unref (tiles);
+
+  return buffer;
 }
 
 gint
