@@ -41,7 +41,6 @@
 #include "gimp.h"
 #include "gimp-utils.h"
 #include "gimpcontainer.h"
-#include "gimpdrawable-convert.h"
 #include "gimperror.h"
 #include "gimpimage.h"
 #include "gimpimage-quick-mask.h"
@@ -76,9 +75,8 @@ static gint64     gimp_channel_get_memsize   (GimpObject        *object,
 static gchar  * gimp_channel_get_description (GimpViewable      *viewable,
                                               gchar            **tooltip);
 
-static gboolean   gimp_channel_is_attached   (GimpItem          *item);
-static GimpContainer *
-                  gimp_channel_get_container (GimpItem          *item);
+static gboolean   gimp_channel_is_attached   (const GimpItem    *item);
+static GimpItemTree * gimp_channel_get_tree  (GimpItem          *item);
 static GimpItem * gimp_channel_duplicate     (GimpItem          *item,
                                               GType              new_type);
 static void       gimp_channel_convert       (GimpItem          *item,
@@ -251,7 +249,7 @@ gimp_channel_class_init (GimpChannelClass *klass)
   viewable_class->default_stock_id = "gimp-channel";
 
   item_class->is_attached          = gimp_channel_is_attached;
-  item_class->get_container        = gimp_channel_get_container;
+  item_class->get_tree             = gimp_channel_get_tree;
   item_class->duplicate            = gimp_channel_duplicate;
   item_class->convert              = gimp_channel_convert;
   item_class->translate            = gimp_channel_translate;
@@ -263,14 +261,14 @@ gimp_channel_class_init (GimpChannelClass *klass)
   item_class->stroke               = gimp_channel_stroke;
   item_class->get_node             = gimp_channel_get_node;
   item_class->default_name         = _("Channel");
-  item_class->rename_desc          = _("Rename Channel");
-  item_class->translate_desc       = _("Move Channel");
-  item_class->scale_desc           = _("Scale Channel");
-  item_class->resize_desc          = _("Resize Channel");
-  item_class->flip_desc            = _("Flip Channel");
-  item_class->rotate_desc          = _("Rotate Channel");
-  item_class->transform_desc       = _("Transform Channel");
-  item_class->stroke_desc          = _("Stroke Channel");
+  item_class->rename_desc          = C_("undo-type", "Rename Channel");
+  item_class->translate_desc       = C_("undo-type", "Move Channel");
+  item_class->scale_desc           = C_("undo-type", "Scale Channel");
+  item_class->resize_desc          = C_("undo-type", "Resize Channel");
+  item_class->flip_desc            = C_("undo-type", "Flip Channel");
+  item_class->rotate_desc          = C_("undo-type", "Rotate Channel");
+  item_class->transform_desc       = C_("undo-type", "Transform Channel");
+  item_class->stroke_desc          = C_("undo-type", "Stroke Channel");
 
   drawable_class->invalidate_boundary   = gimp_channel_invalidate_boundary;
   drawable_class->get_active_components = gimp_channel_get_active_components;
@@ -292,14 +290,14 @@ gimp_channel_class_init (GimpChannelClass *klass)
   klass->grow           = gimp_channel_real_grow;
   klass->shrink         = gimp_channel_real_shrink;
 
-  klass->feather_desc   = _("Feather Channel");
-  klass->sharpen_desc   = _("Sharpen Channel");
-  klass->clear_desc     = _("Clear Channel");
-  klass->all_desc       = _("Fill Channel");
-  klass->invert_desc    = _("Invert Channel");
-  klass->border_desc    = _("Border Channel");
-  klass->grow_desc      = _("Grow Channel");
-  klass->shrink_desc    = _("Shrink Channel");
+  klass->feather_desc   = C_("undo-type", "Feather Channel");
+  klass->sharpen_desc   = C_("undo-type", "Sharpen Channel");
+  klass->clear_desc     = C_("undo-type", "Clear Channel");
+  klass->all_desc       = C_("undo-type", "Fill Channel");
+  klass->invert_desc    = C_("undo-type", "Invert Channel");
+  klass->border_desc    = C_("undo-type", "Border Channel");
+  klass->grow_desc      = C_("undo-type", "Grow Channel");
+  klass->shrink_desc    = C_("undo-type", "Shrink Channel");
 }
 
 static void
@@ -366,7 +364,7 @@ gimp_channel_get_description (GimpViewable  *viewable,
                               gchar        **tooltip)
 {
   if (! strcmp (GIMP_IMAGE_QUICK_MASK_NAME,
-                gimp_object_get_name (GIMP_OBJECT (viewable))))
+                gimp_object_get_name (viewable)))
     {
       return g_strdup (_("Quick Mask"));
     }
@@ -376,20 +374,23 @@ gimp_channel_get_description (GimpViewable  *viewable,
 }
 
 static gboolean
-gimp_channel_is_attached (GimpItem *item)
+gimp_channel_is_attached (const GimpItem *item)
 {
-  return (GIMP_IS_IMAGE (gimp_item_get_image (item)) &&
-          gimp_container_have (gimp_item_get_image (item)->channels, GIMP_OBJECT (item)));
+  GimpImage *image = gimp_item_get_image (item);
+
+  return (GIMP_IS_IMAGE (image) &&
+          gimp_container_have (gimp_image_get_channels (image),
+                               GIMP_OBJECT (item)));
 }
 
-static GimpContainer *
-gimp_channel_get_container (GimpItem *item)
+static GimpItemTree *
+gimp_channel_get_tree (GimpItem *item)
 {
   if (gimp_item_is_attached (item))
     {
       GimpImage *image = gimp_item_get_image (item);
 
-      return gimp_image_get_channels (image);
+      return gimp_image_get_channel_tree (image);
     }
 
   return NULL;
@@ -429,29 +430,12 @@ static void
 gimp_channel_convert (GimpItem  *item,
                       GimpImage *dest_image)
 {
-  GimpChannel       *channel  = GIMP_CHANNEL (item);
-  GimpDrawable      *drawable = GIMP_DRAWABLE (item);
-  GimpImageBaseType  old_base_type;
+  GimpChannel  *channel  = GIMP_CHANNEL (item);
+  GimpDrawable *drawable = GIMP_DRAWABLE (item);
 
-  old_base_type = GIMP_IMAGE_TYPE_BASE_TYPE (gimp_drawable_type (drawable));
-
-  if (old_base_type != GIMP_GRAY)
+  if (! gimp_drawable_is_gray (drawable))
     {
-      TileManager   *new_tiles;
-      GimpImageType  new_type = GIMP_GRAY_IMAGE;
-
-      if (gimp_drawable_has_alpha (drawable))
-        new_type = GIMP_IMAGE_TYPE_WITH_ALPHA (new_type);
-
-      new_tiles = tile_manager_new (gimp_item_get_width  (item),
-                                    gimp_item_get_height (item),
-                                    GIMP_IMAGE_TYPE_BYTES (new_type));
-
-      gimp_drawable_convert_grayscale (drawable, new_tiles, old_base_type);
-
-      gimp_drawable_set_tiles (drawable, FALSE, NULL,
-                               new_tiles, new_type);
-      tile_manager_unref (new_tiles);
+      gimp_drawable_convert_type (drawable, NULL, GIMP_GRAY, FALSE);
     }
 
   if (gimp_drawable_has_alpha (drawable))
@@ -867,6 +851,8 @@ gimp_channel_get_node (GimpItem *item)
                        channel->color.b,
                        channel->color.a);
 
+  g_warn_if_fail (channel->color_node == NULL);
+
   channel->color_node = gegl_node_new_child (node,
                                              "operation", "gegl:color",
                                              "value",     color,
@@ -874,12 +860,16 @@ gimp_channel_get_node (GimpItem *item)
 
   g_object_unref (color);
 
+  g_warn_if_fail (channel->mask_node == NULL);
+
   channel->mask_node = gegl_node_new_child (node,
                                             "operation", "gegl:opacity",
                                             NULL);
   gegl_node_connect_to (channel->color_node, "output",
                         channel->mask_node,  "input");
 
+  g_warn_if_fail (channel->invert_node == NULL);
+
   channel->invert_node = gegl_node_new_child (node,
                                               "operation", "gegl:invert",
                                               NULL);
@@ -1664,13 +1654,14 @@ gimp_channel_new_from_component (GimpImage       *image,
                                  const gchar     *name,
                                  const GimpRGB   *color)
 {
-  GimpChannel *channel;
-  TileManager *projection;
-  PixelRegion  src;
-  PixelRegion  dest;
-  gint         width;
-  gint         height;
-  gint         pixel;
+  GimpProjection *projection;
+  GimpChannel    *channel;
+  TileManager    *proj_tiles;
+  PixelRegion     src;
+  PixelRegion     dest;
+  gint            width;
+  gint            height;
+  gint            pixel;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
 
@@ -1678,15 +1669,17 @@ gimp_channel_new_from_component (GimpImage       *image,
 
   g_return_val_if_fail (pixel != -1, NULL);
 
-  gimp_pickable_flush (GIMP_PICKABLE (image->projection));
+  projection = gimp_image_get_projection (image);
 
-  projection = gimp_pickable_get_tiles (GIMP_PICKABLE (image->projection));
-  width  = tile_manager_width  (projection);
-  height = tile_manager_height (projection);
+  gimp_pickable_flush (GIMP_PICKABLE (projection));
+
+  proj_tiles = gimp_pickable_get_tiles (GIMP_PICKABLE (projection));
+  width  = tile_manager_width  (proj_tiles);
+  height = tile_manager_height (proj_tiles);
 
   channel = gimp_channel_new (image, width, height, name, color);
 
-  pixel_region_init (&src, projection,
+  pixel_region_init (&src, proj_tiles,
                      0, 0, width, height, FALSE);
   pixel_region_init (&dest,
                      gimp_drawable_get_tiles (GIMP_DRAWABLE (channel)),
@@ -1697,6 +1690,14 @@ gimp_channel_new_from_component (GimpImage       *image,
   return channel;
 }
 
+GimpChannel *
+gimp_channel_get_parent (GimpChannel *channel)
+{
+  g_return_val_if_fail (GIMP_IS_CHANNEL (channel), NULL);
+
+  return GIMP_CHANNEL (gimp_viewable_get_parent (GIMP_VIEWABLE (channel)));
+}
+
 void
 gimp_channel_set_color (GimpChannel   *channel,
                         const GimpRGB *color,
@@ -1711,7 +1712,7 @@ gimp_channel_set_color (GimpChannel   *channel,
         {
           GimpImage *image = gimp_item_get_image (GIMP_ITEM (channel));
 
-          gimp_image_undo_push_channel_color (image, _("Set Channel Color"),
+          gimp_image_undo_push_channel_color (image, C_("undo-type", "Set Channel Color"),
                                               channel);
         }
 
@@ -1777,7 +1778,7 @@ gimp_channel_set_opacity (GimpChannel *channel,
         {
           GimpImage *image = gimp_item_get_image (GIMP_ITEM (channel));
 
-          gimp_image_undo_push_channel_color (image, _("Set Channel Opacity"),
+          gimp_image_undo_push_channel_color (image, C_("undo-type", "Set Channel Opacity"),
                                               channel);
         }
 
