@@ -410,7 +410,7 @@ gimp_curve_get_description (GimpViewable  *viewable,
 {
   GimpCurve *curve = GIMP_CURVE (viewable);
 
-  return g_strdup_printf ("%s", GIMP_OBJECT (curve)->name);
+  return g_strdup_printf ("%s", gimp_object_get_name (curve));
 }
 
 static void
@@ -434,12 +434,10 @@ gimp_curve_get_extension (GimpData *data)
 static GimpData *
 gimp_curve_duplicate (GimpData *data)
 {
-  GimpCurve *curve = GIMP_CURVE (data);
-  GimpCurve *new;
+  GimpCurve *new = g_object_new (GIMP_TYPE_CURVE, NULL);
 
-  new = g_object_new (GIMP_TYPE_CURVE,
-                      "curve-type", curve->curve_type,
-                      NULL);
+  gimp_config_copy (GIMP_CONFIG (data),
+                    GIMP_CONFIG (new), 0);
 
   return GIMP_DATA (new);
 }
@@ -532,7 +530,7 @@ gimp_curve_get_standard (void)
     {
       standard_curve = gimp_curve_new ("Standard");
 
-      standard_curve->dirty = FALSE;
+      gimp_data_clean (standard_curve);
       gimp_data_make_internal (standard_curve,
                                "gimp-curve-standard");
 
@@ -887,7 +885,7 @@ gimp_curve_calculate (GimpCurve *curve)
   gint  num_pts;
   gint  p1, p2, p3, p4;
 
-  if (GIMP_DATA (curve)->freeze_count > 0)
+  if (gimp_data_is_frozen (GIMP_DATA (curve)))
     return;
 
   points = g_newa (gint, curve->n_points);
