@@ -48,6 +48,11 @@
 #define GIMP_OBSOLETE_DATA_DIR_NAME "gimp-obsolete-files"
 
 
+typedef void (* GimpDataForeachFunc) (GimpDataFactory *factory,
+                                      GimpData        *data,
+                                      gpointer         user_data);
+
+
 struct _GimpDataFactoryPriv
 {
   Gimp                             *gimp;
@@ -69,12 +74,14 @@ struct _GimpDataFactoryPriv
 static void    gimp_data_factory_finalize     (GObject              *object);
 
 static void    gimp_data_factory_data_load    (GimpDataFactory      *factory,
+                                               GimpContext          *context,
                                                GHashTable           *cache);
 
 static gint64  gimp_data_factory_get_memsize  (GimpObject           *object,
                                                gint64               *gui_size);
 
-static gchar * gimp_data_factory_get_save_dir (GimpDataFactory      *factory);
+static gchar * gimp_data_factory_get_save_dir (GimpDataFactory      *factory,
+                                               GError              **error);
 
 static void    gimp_data_factory_load_data  (const GimpDatafileData *file_data,
                                              gpointer                data);
@@ -209,9 +216,11 @@ gimp_data_factory_new (Gimp                             *gimp,
 
 void
 gimp_data_factory_data_init (GimpDataFactory *factory,
+                             GimpContext     *context,
                              gboolean         no_data)
 {
   g_return_if_fail (GIMP_IS_DATA_FACTORY (factory));
+  g_return_if_fail (GIMP_IS_CONTEXT (context));
 
   /*  Freeze and thaw the container even if no_data,
    *  this creates the standard data that serves as fallback.
@@ -222,22 +231,39 @@ gimp_data_factory_data_init (GimpDataFactory *factory,
     {
       if (factory->priv->gimp->be_verbose)
         {
-          const gchar *name = gimp_object_get_name (GIMP_OBJECT (factory));
+          const gchar *name = gimp_object_get_name (factory);
 
           g_print ("Loading '%s' data\n", name ? name : "???");
         }
 
-      gimp_data_factory_data_load (factory, NULL);
+      gimp_data_factory_data_load (factory, context, NULL);
     }
 
   gimp_container_thaw (factory->priv->container);
 }
 
-typedef struct
+static void
+gimp_data_factory_refresh_cache_add (GimpDataFactory *factory,
+                                     GimpData        *data,
+                                     gpointer         user_data)
 {
-  GimpDataFactory *factory;
-  GHashTable      *cache;
-} GimpDataLoadContext;
+  const gchar *filename = gimp_data_get_filename (data);
+
+  if (filename)
+    {
+      GHashTable *cache = user_data;
+      GList      *list;
+
+      g_object_ref (data);
+
+      gimp_container_remove (factory->priv->container, GIMP_OBJECT (data));
+
+      list = g_hash_table_lookup (cache, filename);
+      list = g_list_prepend (list, data);
+
+      g_hash_table_insert (cache, (gpointer) filename, list);
+    }
+}
 
 static gboolean
 gimp_data_factory_refresh_cache_remove (gpointer key,
@@ -255,66 +281,39 @@ gimp_data_factory_refresh_cache_remove (gpointer key,
 }
 
 static void
-gimp_data_factory_data_move_to_cache (GimpDataFactory *factory,
-                                      gpointer         object,
-                                      gpointer         data)
+gimp_data_factory_data_foreach (GimpDataFactory     *factory,
+                                gboolean             skip_internal,
+                                GimpDataForeachFunc  callback,
+                                gpointer             user_data)
 {
-  GHashTable  *cache    = data;
-  GList       *list;
-  const gchar *filename = GIMP_DATA (object)->filename;
+  GList *list = GIMP_LIST (factory->priv->container)->list;
 
-  g_object_ref (object);
-
-  gimp_container_remove (factory->priv->container, GIMP_OBJECT (object));
-
-  list = g_hash_table_lookup (cache, filename);
-  list = g_list_prepend (list, object);
-
-  g_hash_table_insert (cache, (gpointer) filename, list);
-}
-
-static void
-gimp_data_factory_data_foreach (GimpDataFactory *factory,
-                                void (*callback) (GimpDataFactory *factory,
-                                                  gpointer         expr,
-                                                  gpointer         context),
-                                gpointer         context)
-{
-  GimpList *list;
-
-  if (gimp_container_is_empty (factory->priv->container))
-    return;
-
-  list = GIMP_LIST (factory->priv->container);
-
-  if (list->list)
+  if (skip_internal)
     {
-      if (GIMP_DATA (list->list->data)->internal)
-        {
-          /*  if there are internal objects in the list, skip them  */
-          GList *glist;
+      while (list && gimp_data_is_internal (GIMP_DATA (list->data)))
+        list = g_list_next (list);
+    }
 
-          for (glist = list->list; glist; glist = g_list_next (glist))
-            {
-              if (glist->next && ! GIMP_DATA (glist->next->data)->internal)
-                {
-                  while (glist->next)
-                    callback (factory, glist->next->data, context);
+  while (list)
+    {
+      GList *next = g_list_next (list);
 
-                  break;
-                }
-            }
-        }
-      else
-        {
-          while (list->list)
-            callback (factory, list->list->data, context);
-        }
+      callback (factory, list->data, user_data);
+
+      list = next;
     }
 }
 
+typedef struct
+{
+  GimpDataFactory *factory;
+  GimpContext     *context;
+  GHashTable      *cache;
+} GimpDataLoadContext;
+
 static void
 gimp_data_factory_data_load (GimpDataFactory *factory,
+                             GimpContext     *context,
                              GHashTable      *cache)
 {
   gchar *path;
@@ -329,10 +328,11 @@ gimp_data_factory_data_load (GimpDataFactory *factory,
     {
       GList               *writable_list = NULL;
       gchar               *tmp;
-      GimpDataLoadContext  context;
+      GimpDataLoadContext  load_context;
 
-      context.factory = factory;
-      context.cache   = cache;
+      load_context.factory = factory;
+      load_context.context = context;
+      load_context.cache   = cache;
 
       tmp = gimp_config_path_expand (path, TRUE, NULL);
       g_free (path);
@@ -350,12 +350,13 @@ gimp_data_factory_data_load (GimpDataFactory *factory,
                              WRITABLE_PATH_KEY, writable_list);
         }
 
-      gimp_datafiles_read_directories (path, G_FILE_TEST_EXISTS,
-                                       gimp_data_factory_load_data, &context);
+      gimp_datafiles_read_directories (path, G_FILE_TEST_IS_REGULAR,
+                                       gimp_data_factory_load_data,
+                                       &load_context);
 
       gimp_datafiles_read_directories (path, G_FILE_TEST_IS_DIR,
                                        gimp_data_factory_load_data_recursive,
-                                       &context);
+                                       &load_context);
 
       if (writable_path)
         {
@@ -368,68 +369,67 @@ gimp_data_factory_data_load (GimpDataFactory *factory,
   g_free (writable_path);
 }
 
-static void
-gimp_data_factory_data_reload (GimpDataFactory *factory)
+void
+gimp_data_factory_data_refresh (GimpDataFactory *factory,
+                                GimpContext     *context)
 {
-
   GHashTable *cache;
 
   g_return_if_fail (GIMP_IS_DATA_FACTORY (factory));
+  g_return_if_fail (GIMP_IS_CONTEXT (context));
 
   gimp_container_freeze (factory->priv->container);
 
+  /*  First, save all dirty data objects  */
+  gimp_data_factory_data_save (factory);
+
   cache = g_hash_table_new (g_str_hash, g_str_equal);
 
-  gimp_data_factory_data_foreach (factory,
-                                  gimp_data_factory_data_move_to_cache, cache);
+  gimp_data_factory_data_foreach (factory, TRUE,
+                                  gimp_data_factory_refresh_cache_add, cache);
 
-  /* Now the cache contains a filename => list-of-objects mapping of
-   * the old objects. So we should now traverse the directory and for
-   * each file load it only if it's mtime is newer.
+  /*  Now the cache contains a filename => list-of-objects mapping of
+   *  the old objects. So we should now traverse the directory and for
+   *  each file load it only if its mtime is newer.
    *
-   * Once a file was added, it is removed from the cache, so the only
-   * objects remaining there will be those that are not present on the
-   * disk (that have to be destroyed)
+   *  Once a file was added, it is removed from the cache, so the only
+   *  objects remaining there will be those that are not present on
+   *  the disk (that have to be destroyed)
    */
-  gimp_data_factory_data_load (factory, cache);
+  gimp_data_factory_data_load (factory, context, cache);
 
-  /* Now all the data is loaded. Free what remains in the cache. */
+  /*  Now all the data is loaded. Free what remains in the cache  */
   g_hash_table_foreach_remove (cache,
                                gimp_data_factory_refresh_cache_remove, NULL);
+
   g_hash_table_destroy (cache);
 
   gimp_container_thaw (factory->priv->container);
 }
 
-void
-gimp_data_factory_data_refresh (GimpDataFactory *factory)
-{
-  g_return_if_fail (GIMP_IS_DATA_FACTORY (factory));
-
-  gimp_container_freeze (factory->priv->container);
-
-  gimp_data_factory_data_save (factory);
-
-  gimp_data_factory_data_reload (factory);
-
-  gimp_container_thaw (factory->priv->container);
-}
-
 void
 gimp_data_factory_data_save (GimpDataFactory *factory)
 {
-  GList *list;
-  gchar *writable_dir;
+  GList  *list;
+  gchar  *writable_dir;
+  GError *error = NULL;
 
   g_return_if_fail (GIMP_IS_DATA_FACTORY (factory));
 
   if (gimp_container_is_empty (factory->priv->container))
     return;
 
-  writable_dir = gimp_data_factory_get_save_dir (factory);
+  writable_dir = gimp_data_factory_get_save_dir (factory, &error);
 
   if (! writable_dir)
-    return;
+    {
+      gimp_message (factory->priv->gimp, NULL, GIMP_MESSAGE_ERROR,
+                    _("Failed to save data:\n\n%s"),
+                    error->message);
+      g_clear_error (&error);
+
+      return;
+    }
 
   for (list = GIMP_LIST (factory->priv->container)->list;
        list;
@@ -437,10 +437,11 @@ gimp_data_factory_data_save (GimpDataFactory *factory)
     {
       GimpData *data = list->data;
 
-      if (! data->filename)
+      if (! gimp_data_get_filename (data))
         gimp_data_create_filename (data, writable_dir);
 
-      if (data->dirty && data->writable)
+      if (gimp_data_is_dirty (data) &&
+          gimp_data_is_writable (data))
         {
           GError *error = NULL;
 
@@ -465,8 +466,8 @@ gimp_data_factory_data_save (GimpDataFactory *factory)
 
 static void
 gimp_data_factory_remove_cb (GimpDataFactory *factory,
-                             gpointer         data,
-                             gpointer         context)
+                             GimpData        *data,
+                             gpointer         user_data)
 {
   gimp_container_remove (factory->priv->container, GIMP_OBJECT (data));
 }
@@ -478,22 +479,25 @@ gimp_data_factory_data_free (GimpDataFactory *factory)
 
   gimp_container_freeze (factory->priv->container);
 
-  gimp_data_factory_data_foreach (factory, gimp_data_factory_remove_cb, NULL);
+  gimp_data_factory_data_foreach (factory, TRUE,
+                                  gimp_data_factory_remove_cb, NULL);
 
   gimp_container_thaw (factory->priv->container);
 }
 
 GimpData *
 gimp_data_factory_data_new (GimpDataFactory *factory,
+                            GimpContext     *context,
                             const gchar     *name)
 {
   g_return_val_if_fail (GIMP_IS_DATA_FACTORY (factory), NULL);
+  g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail (*name != '\0', NULL);
 
   if (factory->priv->data_new_func)
     {
-      GimpData *data = factory->priv->data_new_func (name);
+      GimpData *data = factory->priv->data_new_func (context, name);
 
       if (data)
         {
@@ -522,14 +526,12 @@ gimp_data_factory_data_duplicate (GimpDataFactory *factory,
 
   if (new_data)
     {
-      const gchar *name;
+      const gchar *name = gimp_object_get_name (data);
       gchar       *ext;
       gint         copy_len;
       gint         number;
       gchar       *new_name;
 
-      name = gimp_object_get_name (GIMP_OBJECT (data));
-
       ext      = strrchr (name, '#');
       copy_len = strlen (_("copy"));
 
@@ -573,7 +575,7 @@ gimp_data_factory_data_delete (GimpDataFactory  *factory,
 
       gimp_container_remove (factory->priv->container, GIMP_OBJECT (data));
 
-      if (delete_from_disk && data->filename)
+      if (delete_from_disk && gimp_data_get_filename (data))
         retval = gimp_data_delete_from_disk (data, error);
 
       g_object_unref (data);
@@ -583,12 +585,14 @@ gimp_data_factory_data_delete (GimpDataFactory  *factory,
 }
 
 GimpData *
-gimp_data_factory_data_get_standard (GimpDataFactory *factory)
+gimp_data_factory_data_get_standard (GimpDataFactory *factory,
+                                     GimpContext     *context)
 {
   g_return_val_if_fail (GIMP_IS_DATA_FACTORY (factory), NULL);
+  g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
 
   if (factory->priv->data_get_standard_func)
-    return factory->priv->data_get_standard_func ();
+    return factory->priv->data_get_standard_func (context);
 
   return NULL;
 }
@@ -602,18 +606,23 @@ gimp_data_factory_data_save_single (GimpDataFactory  *factory,
   g_return_val_if_fail (GIMP_IS_DATA (data), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  if (! data->dirty)
+  if (! gimp_data_is_dirty (data))
     return TRUE;
 
-  if (! data->filename)
+  if (! gimp_data_get_filename (data))
     {
-      gchar *writable_dir = gimp_data_factory_get_save_dir (factory);
+      gchar  *writable_dir;
+      GError *my_error = NULL;
+
+      writable_dir = gimp_data_factory_get_save_dir (factory, &my_error);
 
       if (! writable_dir)
         {
           g_set_error (error, GIMP_DATA_ERROR, 0,
                        _("Failed to save data:\n\n%s"),
-                       _("You don't have a writable data folder configured."));
+                       my_error->message);
+          g_clear_error (&my_error);
+
           return FALSE;
         }
 
@@ -622,7 +631,7 @@ gimp_data_factory_data_save_single (GimpDataFactory  *factory,
       g_free (writable_dir);
     }
 
-  if (! data->writable)
+  if (! gimp_data_is_writable (data))
     return FALSE;
 
   if (! gimp_data_save (data, error))
@@ -677,7 +686,8 @@ gimp_data_factory_has_data_new_func (GimpDataFactory *factory)
 /*  private functions  */
 
 static gchar *
-gimp_data_factory_get_save_dir (GimpDataFactory *factory)
+gimp_data_factory_get_save_dir (GimpDataFactory  *factory,
+                                GError          **error)
 {
   gchar *path;
   gchar *writable_path;
@@ -700,21 +710,66 @@ gimp_data_factory_get_save_dir (GimpDataFactory *factory)
   g_free (writable_path);
   writable_path = tmp;
 
-  path_list     = gimp_path_parse (path,          16, TRUE, NULL);
-  writable_list = gimp_path_parse (writable_path, 16, TRUE, NULL);
+  path_list     = gimp_path_parse (path,          16, FALSE, NULL);
+  writable_list = gimp_path_parse (writable_path, 16, FALSE, NULL);
 
   g_free (path);
   g_free (writable_path);
 
-  for (list = writable_list; list; list = g_list_next (list))
+  if (writable_path)
     {
-      GList *found = g_list_find_custom (path_list,
-                                         list->data, (GCompareFunc) strcmp);
-      if (found)
+      gboolean found_any = FALSE;
+
+      for (list = writable_list; list; list = g_list_next (list))
         {
-          writable_dir = g_strdup (found->data);
-          break;
+          GList *found = g_list_find_custom (path_list,
+                                             list->data, (GCompareFunc) strcmp);
+          if (found)
+            {
+              const gchar *dir = found->data;
+
+              found_any = TRUE;
+
+              if (! g_file_test (dir, G_FILE_TEST_IS_DIR))
+                {
+                  /*  error out only if this is the last chance  */
+                  if (! list->next)
+                    {
+                      gchar *display_name = g_filename_display_name (dir);
+
+                      g_set_error (error, GIMP_DATA_ERROR, 0,
+                                   _("You have a writable data folder "
+                                     "configured (%s), but this folder does "
+                                     "not exist. Please create the folder or "
+                                     "fix your configuation in the "
+                                     "Preferences dialog's 'Folders' section."),
+                                   display_name);
+
+                      g_free (display_name);
+                    }
+                }
+              else
+                {
+                  writable_dir = g_strdup (dir);
+                  break;
+                }
+            }
         }
+
+      if (! writable_dir && ! found_any)
+        {
+          g_set_error (error, GIMP_DATA_ERROR, 0,
+                       _("You have a writable data folder configured, but this "
+                         "folder is not part of your data search path. You "
+                         "probably edited the gimprc file manually, "
+                         "please fix it in the Preferences dialog's 'Folders' "
+                         "section."));
+        }
+    }
+  else
+    {
+      g_set_error (error, GIMP_DATA_ERROR, 0,
+                   _("You don't have any writable data folder configured."));
     }
 
   gimp_path_free (path_list);
@@ -729,7 +784,7 @@ gimp_data_factory_load_data_recursive (const GimpDatafileData *file_data,
 {
   GimpDataLoadContext *context = data;
 
-  gimp_datafiles_read_directories (file_data->filename, G_FILE_TEST_EXISTS,
+  gimp_datafiles_read_directories (file_data->filename, G_FILE_TEST_IS_REGULAR,
                                    gimp_data_factory_load_data, context);
 
   gimp_datafiles_read_directories (file_data->filename, G_FILE_TEST_IS_DIR,
@@ -741,16 +796,19 @@ static void
 gimp_data_factory_load_data (const GimpDatafileData *file_data,
                              gpointer                data)
 {
-  GimpDataLoadContext *context = data;
-  GimpDataFactory     *factory = context->factory;
-  GHashTable          *cache   = context->cache;
-  gint                 i;
+  GimpDataLoadContext              *context = data;
+  GimpDataFactory                  *factory = context->factory;
+  GHashTable                       *cache   = context->cache;
+  const GimpDataFactoryLoaderEntry *loader;
+  gint                              i;
 
   for (i = 0; i < factory->priv->n_loader_entries; i++)
     {
-      if (! factory->priv->loader_entries[i].extension ||
+      loader = &factory->priv->loader_entries[i];
+
+      if (! loader->extension ||
           gimp_datafiles_check_extension (file_data->filename,
-                                          factory->priv->loader_entries[i].extension))
+                                          loader->extension))
         goto insert;
     }
 
@@ -766,14 +824,15 @@ gimp_data_factory_load_data (const GimpDatafileData *file_data,
       {
         GimpData *data = cached_data->data;
 
-        load_from_disk = (data->mtime == 0 || data->mtime != file_data->mtime);
+        load_from_disk = (gimp_data_get_mtime (data) == 0 ||
+                          gimp_data_get_mtime (data) != file_data->mtime);
 
         if (! load_from_disk)
           {
             GList *list;
 
             for (list = cached_data; list; list = g_list_next (list))
-              gimp_container_add (factory->priv->container, GIMP_OBJECT (list->data));
+              gimp_container_add (factory->priv->container, list->data);
           }
       }
 
@@ -782,8 +841,8 @@ gimp_data_factory_load_data (const GimpDatafileData *file_data,
         GList  *data_list;
         GError *error = NULL;
 
-        data_list = factory->priv->loader_entries[i].load_func (file_data->filename,
-                                                          &error);
+        data_list = loader->load_func (context->context,
+                                       file_data->filename, &error);
 
         if (G_LIKELY (data_list))
           {
@@ -799,7 +858,7 @@ gimp_data_factory_load_data (const GimpDatafileData *file_data,
                          g_list_find_custom (writable_list, file_data->dirname,
                                              (GCompareFunc) strcmp) != NULL);
 
-            writable = (deletable && factory->priv->loader_entries[i].writable);
+            writable = (deletable && loader->writable);
 
             for (list = data_list; list; list = g_list_next (list))
               {
@@ -807,8 +866,9 @@ gimp_data_factory_load_data (const GimpDatafileData *file_data,
 
                 gimp_data_set_filename (data, file_data->filename,
                                         writable, deletable);
-                data->mtime = file_data->mtime;
-                data->dirty = FALSE;
+                gimp_data_set_mtime (data, file_data->mtime);
+
+                gimp_data_clean (data);
 
                 if (strstr (file_data->dirname, GIMP_OBSOLETE_DATA_DIR_NAME))
                   gimp_container_add (factory->priv->container_obsolete,
