@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis, and others
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_DRAWABLE_TRANSFORM_H__
@@ -39,7 +38,6 @@ TileManager * gimp_drawable_transform_tiles_affine (GimpDrawable           *draw
                                                     const GimpMatrix3      *matrix,
                                                     GimpTransformDirection  direction,
                                                     GimpInterpolationType   interpolation_type,
-                                                    gboolean                supersample,
                                                     gint                    recursion_level,
                                                     GimpTransformResize     clip_result,
                                                     GimpProgress           *progress);
@@ -63,7 +61,6 @@ gboolean      gimp_drawable_transform_affine       (GimpDrawable           *draw
                                                     const GimpMatrix3      *matrix,
                                                     GimpTransformDirection  direction,
                                                     GimpInterpolationType   interpolation_type,
-                                                    gboolean                supersample,
                                                     gint                    recursion_level,
                                                     GimpTransformResize     clip_result,
                                                     GimpProgress           *progress);
