@@ -18,6 +18,7 @@
 #include "config.h"
 
 #include <gegl.h>
+#include <gegl-plugin.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
@@ -32,9 +33,15 @@
 #include "paint-funcs/paint-funcs.h"
 #include "paint-funcs/scale-region.h"
 
+#include "gegl/gimp-gegl-utils.h"
+
+#include "gimp.h" /* temp for gimp_use_gegl() */
+#include "gimp-utils.h" /* temp for GIMP_TIMER */
 #include "gimpchannel.h"
 #include "gimpcontext.h"
 #include "gimpdrawable-combine.h"
+#include "gimpdrawable-convert.h"
+#include "gimpdrawable-operation.h"
 #include "gimpdrawable-preview.h"
 #include "gimpdrawable-private.h"
 #include "gimpdrawable-shadow.h"
@@ -49,6 +56,8 @@
 #include "gimppreviewcache.h"
 #include "gimpprogress.h"
 
+#include "gimp-log.h"
+
 #include "gimp-intl.h"
 
 
@@ -125,6 +134,11 @@ static gint64  gimp_drawable_real_estimate_memsize (const GimpDrawable *drawable
                                                     gint               width,
                                                     gint               height);
 
+static void       gimp_drawable_real_convert_type  (GimpDrawable      *drawable,
+                                                    GimpImage         *dest_image,
+                                                    GimpImageBaseType  new_base_type,
+                                                    gboolean           push_undo);
+
 static TileManager * gimp_drawable_real_get_tiles  (GimpDrawable      *drawable);
 static void       gimp_drawable_real_set_tiles     (GimpDrawable      *drawable,
                                                     gboolean           push_undo,
@@ -156,6 +170,12 @@ static void       gimp_drawable_sync_source_node   (GimpDrawable      *drawable,
 static void       gimp_drawable_fs_notify          (GimpLayer         *fs,
                                                     const GParamSpec  *pspec,
                                                     GimpDrawable      *drawable);
+static void       gimp_drawable_fs_update          (GimpLayer         *fs,
+                                                    gint               x,
+                                                    gint               y,
+                                                    gint               width,
+                                                    gint               height,
+                                                    GimpDrawable      *drawable);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpDrawable, gimp_drawable, GIMP_TYPE_ITEM,
@@ -220,6 +240,7 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
   klass->estimate_memsize            = gimp_drawable_real_estimate_memsize;
   klass->invalidate_boundary         = NULL;
   klass->get_active_components       = NULL;
+  klass->convert_type                = gimp_drawable_real_convert_type;
   klass->apply_region                = gimp_drawable_real_apply_region;
   klass->replace_region              = gimp_drawable_real_replace_region;
   klass->get_tiles                   = gimp_drawable_real_get_tiles;
@@ -233,13 +254,12 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
 static void
 gimp_drawable_init (GimpDrawable *drawable)
 {
-  drawable->private   = G_TYPE_INSTANCE_GET_PRIVATE (drawable,
-                                                     GIMP_TYPE_DRAWABLE,
-                                                     GimpDrawablePrivate);
+  drawable->private = G_TYPE_INSTANCE_GET_PRIVATE (drawable,
+                                                   GIMP_TYPE_DRAWABLE,
+                                                   GimpDrawablePrivate);
 
-  drawable->bytes     = 0;
-  drawable->type      = -1;
-  drawable->has_alpha = FALSE;
+  drawable->bytes = 0;
+  drawable->type  = -1;
 }
 
 /* sorry for the evil casts */
@@ -317,8 +337,7 @@ gimp_drawable_invalidate_preview (GimpViewable *viewable)
 {
   GimpDrawable *drawable = GIMP_DRAWABLE (viewable);
 
-  if (GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview)
-    GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview (viewable);
+  GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview (viewable);
 
   drawable->private->preview_valid = FALSE;
 
@@ -341,17 +360,20 @@ static void
 gimp_drawable_visibility_changed (GimpItem *item)
 {
   GimpDrawable *drawable = GIMP_DRAWABLE (item);
+  GeglNode     *item_node;
 
   /*  don't use gimp_item_get_node() because that would create
    *  the node.
    */
-  if (item->node)
+  item_node = gimp_item_peek_node (item);
+
+  if (item_node)
     {
       GeglNode *input;
       GeglNode *output;
 
-      input  = gegl_node_get_input_proxy  (item->node, "input");
-      output = gegl_node_get_output_proxy (item->node, "output");
+      input  = gegl_node_get_input_proxy  (item_node, "input");
+      output = gegl_node_get_output_proxy (item_node, "output");
 
       if (gimp_item_get_visible (item) &&
           ! (GIMP_IS_LAYER (item) &&
@@ -405,7 +427,7 @@ gimp_drawable_duplicate (GimpItem *item,
                                gimp_item_get_width  (item),
                                gimp_item_get_height (item),
                                gimp_drawable_type (drawable),
-                               GIMP_OBJECT (new_drawable)->name);
+                               gimp_object_get_name (new_drawable));
 
       pixel_region_init (&srcPR, gimp_drawable_get_tiles (drawable),
                          0, 0,
@@ -435,30 +457,63 @@ gimp_drawable_scale (GimpItem              *item,
 {
   GimpDrawable *drawable = GIMP_DRAWABLE (item);
   TileManager  *new_tiles;
-  PixelRegion   srcPR, destPR;
 
   new_tiles = tile_manager_new (new_width, new_height, drawable->bytes);
 
-  pixel_region_init (&srcPR, gimp_drawable_get_tiles (drawable),
-                     0, 0,
-                     gimp_item_get_width  (item),
-                     gimp_item_get_height (item),
-                     FALSE);
-  pixel_region_init (&destPR, new_tiles,
-                     0, 0,
-                     new_width, new_height,
-                     TRUE);
+  GIMP_TIMER_START ();
 
-  /*  Scale the drawable -
-   *   If the drawable is indexed, then we don't use pixel-value
-   *   resampling because that doesn't necessarily make sense for indexed
-   *   images.
-   */
-  scale_region (&srcPR, &destPR,
-                gimp_drawable_is_indexed (drawable) ?
-                GIMP_INTERPOLATION_NONE : interpolation_type,
-                progress ? gimp_progress_update_and_flush : NULL,
-                progress);
+  if (gimp_use_gegl (gimp_item_get_image (item)->gimp) &&
+      ! gimp_drawable_is_indexed (drawable)            &&
+      interpolation_type != GIMP_INTERPOLATION_LANCZOS)
+    {
+      GeglNode *scale;
+
+      scale = g_object_new (GEGL_TYPE_NODE,
+                            "operation", "gegl:scale",
+                            NULL);
+
+      gegl_node_set (scale,
+                     "origin-x",   0.0,
+                     "origin-y",   0.0,
+                     "filter",     gimp_interpolation_to_gegl_filter (interpolation_type),
+                     "hard-edges", FALSE,
+                     "x",          ((gdouble) new_width /
+                                    gimp_item_get_width  (item)),
+                     "y",          ((gdouble) new_height /
+                                    gimp_item_get_height (item)),
+                     NULL);
+
+      gimp_drawable_apply_operation_to_tiles (drawable, progress, C_("undo-type", "Scale"),
+                                              scale, TRUE, new_tiles);
+      g_object_unref (scale);
+    }
+  else
+    {
+      PixelRegion srcPR, destPR;
+
+      pixel_region_init (&srcPR, gimp_drawable_get_tiles (drawable),
+                         0, 0,
+                         gimp_item_get_width  (item),
+                         gimp_item_get_height (item),
+                         FALSE);
+      pixel_region_init (&destPR, new_tiles,
+                         0, 0,
+                         new_width, new_height,
+                         TRUE);
+
+      /*  Scale the drawable -
+       *   If the drawable is indexed, then we don't use pixel-value
+       *   resampling because that doesn't necessarily make sense for indexed
+       *   images.
+       */
+      scale_region (&srcPR, &destPR,
+                    gimp_drawable_is_indexed (drawable) ?
+                    GIMP_INTERPOLATION_NONE : interpolation_type,
+                    progress ? gimp_progress_update_and_flush : NULL,
+                    progress);
+    }
+
+  GIMP_TIMER_END ("scaling");
 
   gimp_drawable_set_tiles_full (drawable, gimp_item_is_attached (item), NULL,
                                 new_tiles, gimp_drawable_type (drawable),
@@ -698,7 +753,7 @@ gimp_drawable_real_update (GimpDrawable *drawable,
       rect.width  = width;
       rect.height = height;
 
-      gegl_operation_invalidate (operation, &rect);
+      gegl_operation_invalidate (GEGL_OPERATION (operation), &rect, FALSE);
 
       g_object_unref (operation);
     }
@@ -714,6 +769,29 @@ gimp_drawable_real_estimate_memsize (const GimpDrawable *drawable,
   return (gint64) gimp_drawable_bytes (drawable) * width * height;
 }
 
+static void
+gimp_drawable_real_convert_type (GimpDrawable      *drawable,
+                                 GimpImage         *dest_image,
+                                 GimpImageBaseType  new_base_type,
+                                 gboolean           push_undo)
+{
+  g_return_if_fail (new_base_type != GIMP_INDEXED);
+
+  switch (new_base_type)
+    {
+    case GIMP_RGB:
+      gimp_drawable_convert_rgb (drawable, push_undo);
+      break;
+
+    case GIMP_GRAY:
+      gimp_drawable_convert_grayscale (drawable, push_undo);
+      break;
+
+    default:
+      break;
+    }
+}
+
 static TileManager *
 gimp_drawable_real_get_tiles (GimpDrawable *drawable)
 {
@@ -742,7 +820,7 @@ gimp_drawable_real_set_tiles (GimpDrawable *drawable,
 
   if (push_undo)
     gimp_image_undo_push_drawable_mod (gimp_item_get_image (item), undo_desc,
-                                       drawable);
+                                       drawable, FALSE);
 
   /*  ref new before unrefing old, they might be the same  */
   tile_manager_ref (tiles);
@@ -753,25 +831,18 @@ gimp_drawable_real_set_tiles (GimpDrawable *drawable,
   drawable->private->tiles = tiles;
   drawable->type           = type;
   drawable->bytes          = tile_manager_bpp (tiles);
-  drawable->has_alpha      = GIMP_IMAGE_TYPE_HAS_ALPHA (type);
 
   gimp_item_set_offset (item, offset_x, offset_y);
-
-  if (gimp_item_get_width  (item) != tile_manager_width (tiles) ||
-      gimp_item_get_height (item) != tile_manager_height (tiles))
-    {
-      item->width  = tile_manager_width (tiles);
-      item->height = tile_manager_height (tiles);
-
-      gimp_viewable_size_changed (GIMP_VIEWABLE (drawable));
-    }
+  gimp_item_set_size (item,
+                      tile_manager_width  (tiles),
+                      tile_manager_height (tiles));
 
   if (old_has_alpha != gimp_drawable_has_alpha (drawable))
     gimp_drawable_alpha_changed (drawable);
 
   if (drawable->private->tile_source_node)
     gegl_node_set (drawable->private->tile_source_node,
-                   "tile-manager", drawable->private->tiles,
+                   "tile-manager", gimp_drawable_get_tiles (drawable),
                    NULL);
 }
 
@@ -785,8 +856,10 @@ gimp_drawable_get_node (GimpItem *item)
 
   node = GIMP_ITEM_CLASS (parent_class)->get_node (item);
 
+  g_warn_if_fail (drawable->private->mode_node == NULL);
+
   drawable->private->mode_node = gegl_node_new_child (node,
-                                                      "operation", "gegl:normal",
+                                                      "operation", "gegl:over",
                                                       NULL);
 
   input  = gegl_node_get_input_proxy  (node, "input");
@@ -1067,6 +1140,40 @@ gimp_drawable_fs_notify (GimpLayer        *fs,
     }
 }
 
+static void
+gimp_drawable_fs_update (GimpLayer    *fs,
+                         gint          x,
+                         gint          y,
+                         gint          width,
+                         gint          height,
+                         GimpDrawable *drawable)
+{
+  gint fs_off_x, fs_off_y;
+  gint off_x, off_y;
+  gint dr_x, dr_y, dr_width, dr_height;
+
+  gimp_item_get_offset (GIMP_ITEM (fs), &fs_off_x, &fs_off_y);
+  gimp_item_get_offset (GIMP_ITEM (drawable), &off_x, &off_y);
+
+  if (gimp_rectangle_intersect (x + fs_off_x,
+                                y + fs_off_y,
+                                width,
+                                height,
+                                off_y,
+                                off_y,
+                                gimp_item_get_width  (GIMP_ITEM (drawable)),
+                                gimp_item_get_height (GIMP_ITEM (drawable)),
+                                &dr_x,
+                                &dr_y,
+                                &dr_width,
+                                &dr_height))
+    {
+      gimp_drawable_update (drawable,
+                            dr_x - off_x, dr_y - off_y,
+                            dr_width, dr_height);
+    }
+}
+
 
 /*  public functions  */
 
@@ -1098,9 +1205,8 @@ gimp_drawable_configure (GimpDrawable  *drawable,
   gimp_item_configure (GIMP_ITEM (drawable), image,
                        offset_x, offset_y, width, height, name);
 
-  drawable->type      = type;
-  drawable->bytes     = GIMP_IMAGE_TYPE_BYTES (type);
-  drawable->has_alpha = GIMP_IMAGE_TYPE_HAS_ALPHA (type);
+  drawable->type  = type;
+  drawable->bytes = GIMP_IMAGE_TYPE_BYTES (type);
 
   if (drawable->private->tiles)
     tile_manager_unref (drawable->private->tiles);
@@ -1113,7 +1219,7 @@ gimp_drawable_configure (GimpDrawable  *drawable,
 
   if (drawable->private->tile_source_node)
     gegl_node_set (drawable->private->tile_source_node,
-                   "tile-manager", drawable->private->tiles,
+                   "tile-manager", gimp_drawable_get_tiles (drawable),
                    NULL);
 }
 
@@ -1166,6 +1272,29 @@ gimp_drawable_get_active_components (const GimpDrawable *drawable,
     drawable_class->get_active_components (drawable, active);
 }
 
+void
+gimp_drawable_convert_type (GimpDrawable      *drawable,
+                            GimpImage         *dest_image,
+                            GimpImageBaseType  new_base_type,
+                            gboolean           push_undo)
+{
+  GimpImageType type;
+
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+  g_return_if_fail (dest_image == NULL || GIMP_IS_IMAGE (dest_image));
+  g_return_if_fail (new_base_type != GIMP_INDEXED || GIMP_IS_IMAGE (dest_image));
+
+  if (! gimp_item_is_attached (GIMP_ITEM (drawable)))
+    push_undo = FALSE;
+
+  type = gimp_drawable_type (drawable);
+
+  g_return_if_fail (new_base_type != GIMP_IMAGE_TYPE_BASE_TYPE (type));
+
+  GIMP_DRAWABLE_GET_CLASS (drawable)->convert_type (drawable, dest_image,
+                                                    new_base_type, push_undo);
+}
+
 void
 gimp_drawable_apply_region (GimpDrawable         *drawable,
                             PixelRegion          *src2PR,
@@ -1367,13 +1496,13 @@ gimp_drawable_set_tiles (GimpDrawable  *drawable,
 }
 
 void
-gimp_drawable_set_tiles_full (GimpDrawable       *drawable,
-                              gboolean            push_undo,
-                              const gchar        *undo_desc,
-                              TileManager        *tiles,
-                              GimpImageType       type,
-                              gint                offset_x,
-                              gint                offset_y)
+gimp_drawable_set_tiles_full (GimpDrawable  *drawable,
+                              gboolean       push_undo,
+                              const gchar   *undo_desc,
+                              TileManager   *tiles,
+                              GimpImageType  type,
+                              gint           offset_x,
+                              gint           offset_y)
 {
   GimpItem  *item;
   GimpImage *image;
@@ -1399,11 +1528,15 @@ gimp_drawable_set_tiles_full (GimpDrawable       *drawable,
                             gimp_item_get_height (item));
     }
 
+  g_object_freeze_notify (G_OBJECT (drawable));
+
   GIMP_DRAWABLE_GET_CLASS (drawable)->set_tiles (drawable,
                                                  push_undo, undo_desc,
                                                  tiles, type,
                                                  offset_x, offset_y);
 
+  g_object_thaw_notify (G_OBJECT (drawable));
+
   gimp_drawable_update (drawable,
                         0, 0,
                         gimp_item_get_width  (item),
@@ -1423,7 +1556,7 @@ gimp_drawable_get_source_node (GimpDrawable *drawable)
   drawable->private->tile_source_node =
     gegl_node_new_child (drawable->private->source_node,
                          "operation",    "gimp:tilemanager-source",
-                         "tile-manager", drawable->private->tiles,
+                         "tile-manager", gimp_drawable_get_tiles (drawable),
                          "linear",       TRUE,
                          NULL);
 
@@ -1438,7 +1571,7 @@ gimp_drawable_get_mode_node (GimpDrawable *drawable)
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
 
   if (! drawable->private->mode_node)
-    gimp_drawable_get_node (GIMP_ITEM (drawable));
+    gimp_item_get_node (GIMP_ITEM (drawable));
 
   return drawable->private->mode_node;
 }
@@ -1725,7 +1858,7 @@ gimp_drawable_has_alpha (const GimpDrawable *drawable)
 {
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), FALSE);
 
-  return drawable->has_alpha;
+  return GIMP_IMAGE_TYPE_HAS_ALPHA (gimp_drawable_type (drawable));
 }
 
 GimpImageType
@@ -1846,7 +1979,7 @@ gimp_drawable_attach_floating_sel (GimpDrawable *drawable,
   g_return_if_fail (gimp_item_is_attached (GIMP_ITEM (drawable)));
   g_return_if_fail (GIMP_IS_LAYER (floating_sel));
 
-  g_printerr ("%s\n", G_STRFUNC);
+  GIMP_LOG (FLOATING_SELECTION, "%s", G_STRFUNC);
 
   image = gimp_item_get_image (GIMP_ITEM (drawable));
 
@@ -1857,8 +1990,15 @@ gimp_drawable_attach_floating_sel (GimpDrawable *drawable,
 
   gimp_drawable_sync_source_node (drawable, FALSE);
 
-  /* FIXME: remove this hack when the floating sel is no layer any longer */
-  g_signal_emit_by_name (floating_sel, "visibility-changed");
+  g_signal_connect (floating_sel, "update",
+                    G_CALLBACK (gimp_drawable_fs_update),
+                    drawable);
+
+  gimp_drawable_fs_update (floating_sel,
+                           0, 0,
+                           gimp_item_get_width  (GIMP_ITEM (floating_sel)),
+                           gimp_item_get_height (GIMP_ITEM (floating_sel)),
+                           drawable);
 }
 
 void
@@ -1870,21 +2010,21 @@ gimp_drawable_detach_floating_sel (GimpDrawable *drawable,
   g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
   g_return_if_fail (GIMP_IS_LAYER (floating_sel));
 
-  g_printerr ("%s\n", G_STRFUNC);
+  GIMP_LOG (FLOATING_SELECTION, "%s", G_STRFUNC);
 
   image = gimp_item_get_image (GIMP_ITEM (drawable));
 
   gimp_drawable_sync_source_node (drawable, TRUE);
 
-  /* FIXME: remove this hack when the floating sel is no layer any longer */
-  g_signal_emit_by_name (floating_sel, "visibility-changed");
+  g_signal_handlers_disconnect_by_func (floating_sel,
+                                        gimp_drawable_fs_update,
+                                        drawable);
 
-  /*  Invalidate the preview of the obscured drawable.  We do this here
-   *  because it will not be done until the floating selection is removed,
-   *  at which point the obscured drawable's preview will not be declared
-   *  invalid.
-   */
-  gimp_viewable_invalidate_preview (GIMP_VIEWABLE (floating_sel));
+  gimp_drawable_fs_update (floating_sel,
+                           0, 0,
+                           gimp_item_get_width  (GIMP_ITEM (floating_sel)),
+                           gimp_item_get_height (GIMP_ITEM (floating_sel)),
+                           drawable);
 
   /*  clear the selection  */
   gimp_drawable_invalidate_boundary (GIMP_DRAWABLE (floating_sel));
