@@ -39,7 +39,6 @@ struct _GimpDrawable
 
   gint           bytes;              /* bytes per pixel                */
   GimpImageType  type;               /* type of drawable               */
-  gboolean       has_alpha;          /* drawable has alpha             */
 
   GimpDrawablePrivate *private;
 };
@@ -63,6 +62,10 @@ struct _GimpDrawableClass
   void          (* invalidate_boundary)   (GimpDrawable         *drawable);
   void          (* get_active_components) (const GimpDrawable   *drawable,
                                            gboolean             *active);
+  void          (* convert_type)          (GimpDrawable         *drawable,
+                                           GimpImage            *dest_image,
+                                           GimpImageBaseType     new_base_type,
+                                           gboolean              push_undo);
   void          (* apply_region)          (GimpDrawable         *drawable,
                                            PixelRegion          *src2PR,
                                            gboolean              push_undo,
@@ -140,6 +143,11 @@ void           gimp_drawable_invalidate_boundary (GimpDrawable       *drawable);
 void         gimp_drawable_get_active_components (const GimpDrawable *drawable,
                                                   gboolean           *active);
 
+void            gimp_drawable_convert_type       (GimpDrawable       *drawable,
+                                                  GimpImage          *dest_image,
+                                                  GimpImageBaseType   new_base_type,
+                                                  gboolean            push_undo);
+
 void            gimp_drawable_apply_region       (GimpDrawable       *drawable,
                                                   PixelRegion        *src2PR,
                                                   gboolean            push_undo,
