@@ -279,15 +279,33 @@ gimp_gradient_get_checksum (GimpTagged *tagged)
 
       while (segment)
         {
-          g_checksum_update (checksum, (const guchar *) &segment->left, sizeof (segment->left));
-          g_checksum_update (checksum, (const guchar *) &segment->middle, sizeof (segment->middle));
-          g_checksum_update (checksum, (const guchar *) &segment->right,sizeof (segment->right));
-          g_checksum_update (checksum, (const guchar *) &segment->left_color_type, sizeof (segment->left_color_type));
-          g_checksum_update (checksum, (const guchar *) &segment->left_color, sizeof (segment->left_color));
-          g_checksum_update (checksum, (const guchar *) &segment->right_color_type, sizeof (segment->right_color_type));
-          g_checksum_update (checksum, (const guchar *) &segment->right_color, sizeof (segment->right_color));
-          g_checksum_update (checksum, (const guchar *) &segment->type, sizeof (segment->type));
-          g_checksum_update (checksum, (const guchar *) &segment->color, sizeof (segment->color));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->left,
+                             sizeof (segment->left));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->middle,
+                             sizeof (segment->middle));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->right,
+                             sizeof (segment->right));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->left_color_type,
+                             sizeof (segment->left_color_type));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->left_color,
+                             sizeof (segment->left_color));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->right_color_type,
+                             sizeof (segment->right_color_type));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->right_color,
+                             sizeof (segment->right_color));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->type,
+                             sizeof (segment->type));
+          g_checksum_update (checksum,
+                             (const guchar *) &segment->color,
+                             sizeof (segment->color));
 
           segment = segment->next;
         }
@@ -300,10 +318,12 @@ gimp_gradient_get_checksum (GimpTagged *tagged)
   return checksum_string;
 }
 
+
 /*  public functions  */
 
 GimpData *
-gimp_gradient_new (const gchar *name)
+gimp_gradient_new (GimpContext *context,
+                   const gchar *name)
 {
   GimpGradient *gradient;
 
@@ -320,19 +340,19 @@ gimp_gradient_new (const gchar *name)
 }
 
 GimpData *
-gimp_gradient_get_standard (void)
+gimp_gradient_get_standard (GimpContext *context)
 {
   static GimpData *standard_gradient = NULL;
 
   if (! standard_gradient)
     {
-      standard_gradient = gimp_gradient_new ("Standard");
+      standard_gradient = gimp_gradient_new (context, "Standard");
 
-      standard_gradient->dirty = FALSE;
-      gimp_data_make_internal (standard_gradient,
-                               "gimp-gradient-standard");
+      gimp_data_clean (standard_gradient);
+      gimp_data_make_internal (standard_gradient, "gimp-gradient-standard");
 
-      g_object_ref (standard_gradient);
+      g_object_add_weak_pointer (G_OBJECT (standard_gradient),
+                                 (gpointer *) &standard_gradient);
     }
 
   return standard_gradient;
