@@ -27,6 +27,7 @@
 
 #include "gimpimage.h"
 #include "gimpimage-colormap.h"
+#include "gimpimage-private.h"
 #include "gimpimage-undo-push.h"
 
 #include "gimp-intl.h"
@@ -37,7 +38,7 @@ gimp_image_get_colormap (const GimpImage *image)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
 
-  return image->colormap;
+  return GIMP_IMAGE_GET_PRIVATE (image)->colormap;
 }
 
 gint
@@ -45,7 +46,7 @@ gimp_image_get_colormap_size (const GimpImage *image)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), 0);
 
-  return image->n_colors;
+  return GIMP_IMAGE_GET_PRIVATE (image)->n_colors;
 }
 
 void
@@ -54,32 +55,36 @@ gimp_image_set_colormap (GimpImage    *image,
                          gint          n_colors,
                          gboolean      push_undo)
 {
+  GimpImagePrivate *private;
+
   g_return_if_fail (GIMP_IS_IMAGE (image));
   g_return_if_fail (colormap != NULL || n_colors == 0);
   g_return_if_fail (n_colors >= 0 && n_colors <= 256);
 
-  if (push_undo)
-    gimp_image_undo_push_image_colormap (image, _("Set Colormap"));
+  private = GIMP_IMAGE_GET_PRIVATE (image);
 
-  if (image->colormap)
-    memset (image->colormap, 0, GIMP_IMAGE_COLORMAP_SIZE);
+  if (push_undo)
+    gimp_image_undo_push_image_colormap (image, C_("undo-type", "Set Colormap"));
+
+  if (private->colormap)
+    memset (private->colormap, 0, GIMP_IMAGE_COLORMAP_SIZE);
 
   if (colormap)
     {
-      if (! image->colormap)
-        image->colormap = g_new0 (guchar, GIMP_IMAGE_COLORMAP_SIZE);
+      if (! private->colormap)
+        private->colormap = g_new0 (guchar, GIMP_IMAGE_COLORMAP_SIZE);
 
-      memcpy (image->colormap, colormap, n_colors * 3);
+      memcpy (private->colormap, colormap, n_colors * 3);
     }
   else if (! gimp_image_base_type (image) == GIMP_INDEXED)
     {
-      if (image->colormap)
-        g_free (image->colormap);
+      if (private->colormap)
+        g_free (private->colormap);
 
-      image->colormap = NULL;
+      private->colormap = NULL;
     }
 
-  image->n_colors = n_colors;
+  private->n_colors = n_colors;
 
   gimp_image_colormap_changed (image, -1);
 }
@@ -89,15 +94,20 @@ gimp_image_get_colormap_entry (GimpImage *image,
                                gint       color_index,
                                GimpRGB   *color)
 {
+  GimpImagePrivate *private;
+
   g_return_if_fail (GIMP_IS_IMAGE (image));
-  g_return_if_fail (image->colormap != NULL);
-  g_return_if_fail (color_index >= 0 && color_index < image->n_colors);
+
+  private = GIMP_IMAGE_GET_PRIVATE (image);
+
+  g_return_if_fail (private->colormap != NULL);
+  g_return_if_fail (color_index >= 0 && color_index < private->n_colors);
   g_return_if_fail (color != NULL);
 
   gimp_rgba_set_uchar (color,
-                       image->colormap[color_index * 3],
-                       image->colormap[color_index * 3 + 1],
-                       image->colormap[color_index * 3 + 2],
+                       private->colormap[color_index * 3],
+                       private->colormap[color_index * 3 + 1],
+                       private->colormap[color_index * 3 + 2],
                        OPAQUE_OPACITY);
 }
 
@@ -107,19 +117,24 @@ gimp_image_set_colormap_entry (GimpImage     *image,
                                const GimpRGB *color,
                                gboolean       push_undo)
 {
+  GimpImagePrivate *private;
+
   g_return_if_fail (GIMP_IS_IMAGE (image));
-  g_return_if_fail (image->colormap != NULL);
-  g_return_if_fail (color_index >= 0 && color_index < image->n_colors);
+
+  private = GIMP_IMAGE_GET_PRIVATE (image);
+
+  g_return_if_fail (private->colormap != NULL);
+  g_return_if_fail (color_index >= 0 && color_index < private->n_colors);
   g_return_if_fail (color != NULL);
 
   if (push_undo)
     gimp_image_undo_push_image_colormap (image,
-                                         _("Change Colormap entry"));
+                                         C_("undo-type", "Change Colormap entry"));
 
   gimp_rgb_get_uchar (color,
-                      &image->colormap[color_index * 3],
-                      &image->colormap[color_index * 3 + 1],
-                      &image->colormap[color_index * 3 + 2]);
+                      &private->colormap[color_index * 3],
+                      &private->colormap[color_index * 3 + 1],
+                      &private->colormap[color_index * 3 + 2]);
 
   gimp_image_colormap_changed (image, color_index);
 }
@@ -128,20 +143,25 @@ void
 gimp_image_add_colormap_entry (GimpImage     *image,
                                const GimpRGB *color)
 {
+  GimpImagePrivate *private;
+
   g_return_if_fail (GIMP_IS_IMAGE (image));
-  g_return_if_fail (image->colormap != NULL);
-  g_return_if_fail (image->n_colors < 256);
+
+  private = GIMP_IMAGE_GET_PRIVATE (image);
+
+  g_return_if_fail (private->colormap != NULL);
+  g_return_if_fail (private->n_colors < 256);
   g_return_if_fail (color != NULL);
 
   gimp_image_undo_push_image_colormap (image,
-                                       _("Add Color to Colormap"));
+                                       C_("undo-type", "Add Color to Colormap"));
 
   gimp_rgb_get_uchar (color,
-                      &image->colormap[image->n_colors * 3],
-                      &image->colormap[image->n_colors * 3 + 1],
-                      &image->colormap[image->n_colors * 3 + 2]);
+                      &private->colormap[private->n_colors * 3],
+                      &private->colormap[private->n_colors * 3 + 1],
+                      &private->colormap[private->n_colors * 3 + 2]);
 
-  image->n_colors++;
+  private->n_colors++;
 
   gimp_image_colormap_changed (image, -1);
 }
