@@ -33,6 +33,8 @@
 #include "gimpimage-duplicate.h"
 #include "gimpimage-grid.h"
 #include "gimpimage-guides.h"
+#include "gimpimage-private.h"
+#include "gimpimage-undo.h"
 #include "gimpimage-sample-points.h"
 #include "gimplayer.h"
 #include "gimplayer-floating-sel.h"
@@ -138,7 +140,7 @@ gimp_image_duplicate (GimpImage *image)
   if (active_vectors)
     gimp_image_set_active_vectors (new_image, active_vectors);
 
-  /*  Copy state of all color channels  */
+  /*  Copy state of all color components  */
   gimp_image_duplicate_components (image, new_image);
 
   /*  Copy any guides  */
@@ -178,7 +180,7 @@ gimp_image_duplicate_save_source_uri (GimpImage *image,
                                       GimpImage *new_image)
 {
   g_object_set_data_full (G_OBJECT (new_image), "gimp-image-source-uri",
-                          g_strdup (gimp_object_get_name (GIMP_OBJECT (image))),
+                          g_strdup (gimp_object_get_name (image)),
                           (GDestroyNotify) g_free);
 }
 
@@ -202,7 +204,6 @@ gimp_image_duplicate_layers (GimpImage     *image,
 {
   GimpLayer *active_layer = NULL;
   GimpLayer *floating_selection;
-  GList     *all_layers;
   GList     *list;
   gint       count;
 
@@ -212,9 +213,7 @@ gimp_image_duplicate_layers (GimpImage     *image,
   if (floating_selection)
     *floating_sel_drawable = gimp_layer_get_floating_sel_drawable (floating_selection);
 
-  all_layers = gimp_image_get_layer_list (image);
-
-  for (list = all_layers, count = 0;
+  for (list = gimp_image_get_layer_iter (image), count = 0;
        list;
        list = g_list_next (list))
     {
@@ -225,19 +224,16 @@ gimp_image_duplicate_layers (GimpImage     *image,
                                                  new_image,
                                                  G_TYPE_FROM_INSTANCE (layer)));
 
-      g_object_set_data (G_OBJECT (layer), "gimp-image-duplicate-item",
-                         new_layer);
-
       /*  Make sure the copied layer doesn't say: "<old layer> copy"  */
       gimp_object_set_name (GIMP_OBJECT (new_layer),
-                            gimp_object_get_name (GIMP_OBJECT (layer)));
+                            gimp_object_get_name (layer));
 
       /*  Make sure that if the layer has a layer mask,
        *  its name isn't screwed up
        */
       if (new_layer->mask)
         gimp_object_set_name (GIMP_OBJECT (new_layer->mask),
-                              gimp_object_get_name (GIMP_OBJECT (layer->mask)));
+                              gimp_object_get_name (layer->mask));
 
       if (gimp_image_get_active_layer (image) == layer)
         active_layer = new_layer;
@@ -249,26 +245,10 @@ gimp_image_duplicate_layers (GimpImage     *image,
         *new_floating_sel_drawable = GIMP_DRAWABLE (new_layer);
 
       if (*floating_layer != new_layer)
-        {
-          GimpViewable *parent;
-          GimpLayer    *new_parent = NULL;
-
-          parent = gimp_viewable_get_parent (GIMP_VIEWABLE (layer));
-
-          if (parent)
-            new_parent = g_object_get_data (G_OBJECT (parent),
-                                            "gimp-image-duplicate-item");
-
-          gimp_image_add_layer (new_image, new_layer,
-                                new_parent, count++, FALSE);
-        }
+        gimp_image_add_layer (new_image, new_layer,
+                              NULL, count++, FALSE);
     }
 
-  for (list = all_layers; list; list = g_list_next (list))
-    g_object_set_data (list->data, "gimp-image-duplicate-item", NULL);
-
-  g_list_free (all_layers);
-
   return active_layer;
 }
 
@@ -279,31 +259,23 @@ gimp_image_duplicate_channels (GimpImage     *image,
                                GimpDrawable **new_floating_sel_drawable)
 {
   GimpChannel *active_channel = NULL;
-  GList       *all_channels;
   GList       *list;
   gint         count;
 
-  all_channels = gimp_image_get_channel_list (image);
-
-  for (list = all_channels, count = 0;
+  for (list = gimp_image_get_channel_iter (image), count = 0;
        list;
        list = g_list_next (list))
     {
       GimpChannel  *channel = list->data;
       GimpChannel  *new_channel;
-      GimpViewable *parent;
-      GimpChannel  *new_parent = NULL;
 
       new_channel = GIMP_CHANNEL (gimp_item_convert (GIMP_ITEM (channel),
                                                      new_image,
                                                      G_TYPE_FROM_INSTANCE (channel)));
 
-      g_object_set_data (G_OBJECT (channel), "gimp-image-duplicate-item",
-                         new_channel);
-
       /*  Make sure the copied channel doesn't say: "<old channel> copy"  */
       gimp_object_set_name (GIMP_OBJECT (new_channel),
-                            gimp_object_get_name (GIMP_OBJECT (channel)));
+                            gimp_object_get_name (channel));
 
       if (gimp_image_get_active_channel (image) == channel)
         active_channel = new_channel;
@@ -311,21 +283,10 @@ gimp_image_duplicate_channels (GimpImage     *image,
       if (floating_sel_drawable == GIMP_DRAWABLE (channel))
         *new_floating_sel_drawable = GIMP_DRAWABLE (new_channel);
 
-      parent = gimp_viewable_get_parent (GIMP_VIEWABLE (channel));
-
-      if (parent)
-        new_parent = g_object_get_data (G_OBJECT (parent),
-                                        "gimp-image-duplicate-item");
-
       gimp_image_add_channel (new_image, new_channel,
-                              new_parent, count++, FALSE);
+                              NULL, count++, FALSE);
     }
 
-  for (list = all_channels; list; list = g_list_next (list))
-    g_object_set_data (list->data, "gimp-image-duplicate-item", NULL);
-
-  g_list_free (all_channels);
-
   return active_channel;
 }
 
@@ -334,50 +295,31 @@ gimp_image_duplicate_vectors (GimpImage *image,
                               GimpImage *new_image)
 {
   GimpVectors *active_vectors = NULL;
-  GList       *all_vectors;
   GList       *list;
   gint         count;
 
-  all_vectors = gimp_image_get_vectors_list (image);
-
-  for (list = all_vectors, count = 0;
+  for (list = gimp_image_get_vectors_iter (image), count = 0;
        list;
        list = g_list_next (list))
     {
       GimpVectors  *vectors = list->data;
       GimpVectors  *new_vectors;
-      GimpViewable *parent;
-      GimpVectors  *new_parent = NULL;
 
       new_vectors = GIMP_VECTORS (gimp_item_convert (GIMP_ITEM (vectors),
                                                      new_image,
                                                      G_TYPE_FROM_INSTANCE (vectors)));
 
-      g_object_set_data (G_OBJECT (vectors), "gimp-image-duplicate-item",
-                         new_vectors);
-
       /*  Make sure the copied vectors doesn't say: "<old vectors> copy"  */
       gimp_object_set_name (GIMP_OBJECT (new_vectors),
-                            gimp_object_get_name (GIMP_OBJECT (vectors)));
+                            gimp_object_get_name (vectors));
 
       if (gimp_image_get_active_vectors (image) == vectors)
         active_vectors = new_vectors;
 
-      parent = gimp_viewable_get_parent (GIMP_VIEWABLE (vectors));
-
-      if (parent)
-        new_parent = g_object_get_data (G_OBJECT (parent),
-                                        "gimp-image-duplicate-item");
-
       gimp_image_add_vectors (new_image, new_vectors,
-                              new_parent, count++, FALSE);
+                              NULL, count++, FALSE);
     }
 
-  for (list = all_vectors; list; list = g_list_next (list))
-    g_object_set_data (list->data, "gimp-image-duplicate-item", NULL);
-
-  g_list_free (all_vectors);
-
   return active_vectors;
 }
 
@@ -385,14 +327,17 @@ static void
 gimp_image_duplicate_mask (GimpImage *image,
                            GimpImage *new_image)
 {
+  GimpChannel *mask;
+  GimpChannel *new_mask;
   TileManager *src_tiles;
   TileManager *dest_tiles;
   PixelRegion  srcPR, destPR;
 
-  src_tiles  =
-    gimp_drawable_get_tiles (GIMP_DRAWABLE (gimp_image_get_mask (image)));
-  dest_tiles =
-    gimp_drawable_get_tiles (GIMP_DRAWABLE (gimp_image_get_mask (new_image)));
+  mask     = gimp_image_get_mask (image);
+  new_mask = gimp_image_get_mask (new_image);
+
+  src_tiles  = gimp_drawable_get_tiles (GIMP_DRAWABLE (mask));
+  dest_tiles = gimp_drawable_get_tiles (GIMP_DRAWABLE (new_mask));
 
   pixel_region_init (&srcPR, src_tiles,
                      0, 0,
@@ -407,20 +352,22 @@ gimp_image_duplicate_mask (GimpImage *image,
 
   copy_region (&srcPR, &destPR);
 
-  new_image->selection_mask->bounds_known   = FALSE;
-  new_image->selection_mask->boundary_known = FALSE;
+  new_mask->bounds_known   = FALSE;
+  new_mask->boundary_known = FALSE;
 }
 
 static void
 gimp_image_duplicate_components (GimpImage *image,
                                  GimpImage *new_image)
 {
-  gint count;
+  GimpImagePrivate *private     = GIMP_IMAGE_GET_PRIVATE (image);
+  GimpImagePrivate *new_private = GIMP_IMAGE_GET_PRIVATE (new_image);
+  gint              count;
 
   for (count = 0; count < MAX_CHANNELS; count++)
     {
-      new_image->visible[count] = image->visible[count];
-      new_image->active[count]  = image->active[count];
+      new_private->visible[count] = private->visible[count];
+      new_private->active[count]  = private->active[count];
     }
 }
 
@@ -476,26 +423,32 @@ static void
 gimp_image_duplicate_grid (GimpImage *image,
                            GimpImage *new_image)
 {
-  if (image->grid)
-    gimp_image_set_grid (new_image, image->grid, FALSE);
+  if (gimp_image_get_grid (image))
+    gimp_image_set_grid (new_image, gimp_image_get_grid (image), FALSE);
 }
 
 static void
 gimp_image_duplicate_quick_mask (GimpImage *image,
                                  GimpImage *new_image)
 {
-  new_image->quick_mask_state    = image->quick_mask_state;
-  new_image->quick_mask_inverted = image->quick_mask_inverted;
-  new_image->quick_mask_color    = image->quick_mask_color;
+  GimpImagePrivate *private     = GIMP_IMAGE_GET_PRIVATE (image);
+  GimpImagePrivate *new_private = GIMP_IMAGE_GET_PRIVATE (new_image);
+
+  new_private->quick_mask_state    = private->quick_mask_state;
+  new_private->quick_mask_inverted = private->quick_mask_inverted;
+  new_private->quick_mask_color    = private->quick_mask_color;
 }
 
 static void
 gimp_image_duplicate_parasites (GimpImage *image,
                                 GimpImage *new_image)
 {
-  if (image->parasites)
+  GimpImagePrivate *private     = GIMP_IMAGE_GET_PRIVATE (image);
+  GimpImagePrivate *new_private = GIMP_IMAGE_GET_PRIVATE (new_image);
+
+  if (private->parasites)
     {
-      g_object_unref (new_image->parasites);
-      new_image->parasites = gimp_parasite_list_copy (image->parasites);
+      g_object_unref (new_private->parasites);
+      new_private->parasites = gimp_parasite_list_copy (private->parasites);
     }
 }
