@@ -29,6 +29,7 @@
 #include "gimpgrid.h"
 #include "gimpimage.h"
 #include "gimpimage-grid.h"
+#include "gimpimage-private.h"
 #include "gimpimage-undo-push.h"
 
 #include "gimp-intl.h"
@@ -39,7 +40,7 @@ gimp_image_get_grid (GimpImage *image)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
 
-  return image->grid;
+  return GIMP_IMAGE_GET_PRIVATE (image)->grid;
 }
 
 void
@@ -47,14 +48,18 @@ gimp_image_set_grid (GimpImage *image,
                      GimpGrid  *grid,
                      gboolean   push_undo)
 {
+  GimpImagePrivate *private;
+
   g_return_if_fail (GIMP_IS_IMAGE (image));
   g_return_if_fail (GIMP_IS_GRID (grid));
 
-  if (gimp_config_is_equal_to (GIMP_CONFIG (image->grid), GIMP_CONFIG (grid)))
+  private = GIMP_IMAGE_GET_PRIVATE (image);
+
+  if (gimp_config_is_equal_to (GIMP_CONFIG (private->grid), GIMP_CONFIG (grid)))
     return;
 
   if (push_undo)
-    gimp_image_undo_push_image_grid (image, _("Grid"), image->grid);
+    gimp_image_undo_push_image_grid (image, C_("undo-type", "Grid"), private->grid);
 
-  gimp_config_sync (G_OBJECT (grid), G_OBJECT (image->grid), 0);
+  gimp_config_sync (G_OBJECT (grid), G_OBJECT (private->grid), 0);
 }
