@@ -84,7 +84,7 @@ gimp_image_scale (GimpImage             *image,
   g_object_freeze_notify (G_OBJECT (image));
 
   gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_IMAGE_SCALE,
-                               _("Scale Image"));
+                               C_("undo-type", "Scale Image"));
 
   old_width   = gimp_image_get_width  (image);
   old_height  = gimp_image_get_height (image);
@@ -150,6 +150,10 @@ gimp_image_scale (GimpImage             *image,
       gimp_sub_progress_set_step (GIMP_SUB_PROGRESS (sub_progress),
                                   progress_current++, progress_steps);
 
+      /*  group layers are updated automatically  */
+      if (gimp_viewable_get_children (GIMP_VIEWABLE (item)))
+        continue;
+
       if (! gimp_item_scale_by_factors (item,
                                         img_scale_w, img_scale_h,
                                         interpolation_type, sub_progress))
@@ -164,7 +168,9 @@ gimp_image_scale (GimpImage             *image,
     }
 
   /*  Scale all Guides  */
-  for (list = gimp_image_get_guides (image); list; list = g_list_next (list))
+  for (list = gimp_image_get_guides (image);
+       list;
+       list = g_list_next (list))
     {
       GimpGuide *guide    = list->data;
       gint       position = gimp_guide_get_position (guide);
@@ -264,6 +270,9 @@ gimp_image_scale_check (const GimpImage *image,
                                              GIMP_ITEM_TYPE_LAYERS |
                                              GIMP_ITEM_TYPE_CHANNELS,
                                              GIMP_ITEM_SET_ALL);
+
+  gimp_image_item_list_filter (NULL, drawables, TRUE, FALSE);
+
   drawables = g_list_prepend (drawables, gimp_image_get_mask (image));
 
   scalable_size = 0;
@@ -302,8 +311,8 @@ gimp_image_scale_check (const GimpImage *image,
             "scalable_size = %"G_GINT64_FORMAT"  scaled_size = %"G_GINT64_FORMAT,
             scalable_size, scaled_size);
 
-  undo_size = gimp_object_get_memsize (GIMP_OBJECT (image->undo_stack), NULL);
-  redo_size = gimp_object_get_memsize (GIMP_OBJECT (image->redo_stack), NULL);
+  undo_size = gimp_object_get_memsize (GIMP_OBJECT (gimp_image_get_undo_stack (image)), NULL);
+  redo_size = gimp_object_get_memsize (GIMP_OBJECT (gimp_image_get_redo_stack (image)), NULL);
 
   /*  the fixed part of the image's memsize w/o any undo information  */
   fixed_size = current_size - undo_size - redo_size - scalable_size;
@@ -327,6 +336,10 @@ gimp_image_scale_check (const GimpImage *image,
     {
       GimpItem *item = list->data;
 
+      /*  group layers are updated automatically  */
+      if (gimp_viewable_get_children (GIMP_VIEWABLE (item)))
+        continue;
+
       if (! gimp_item_check_scaling (item, new_width, new_height))
         {
           g_list_free (all_layers);
