@@ -78,17 +78,6 @@
 #define GIMP_IMAGE_ACTIVE_PARENT           ((gpointer) 1)
 
 
-typedef struct _GimpImageFlushAccumulator GimpImageFlushAccumulator;
-
-struct _GimpImageFlushAccumulator
-{
-  gboolean alpha_changed;
-  gboolean mask_changed;
-  gboolean floating_selection_changed;
-  gboolean preview_invalidated;
-};
-
-
 #define GIMP_TYPE_IMAGE            (gimp_image_get_type ())
 #define GIMP_IMAGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIMP_TYPE_IMAGE, GimpImage))
 #define GIMP_IMAGE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GIMP_TYPE_IMAGE, GimpImageClass))
@@ -101,80 +90,9 @@ typedef struct _GimpImageClass GimpImageClass;
 
 struct _GimpImage
 {
-  GimpViewable       parent_instance;
+  GimpViewable  parent_instance;
 
-  Gimp              *gimp;                  /*  the GIMP the image belongs to*/
-
-  gint               ID;                    /*  provides a unique ID         */
-
-  GimpPlugInProcedure *load_proc;           /*  procedure used for loading   */
-  GimpPlugInProcedure *save_proc;           /*  last save procedure used     */
-
-  gchar             *display_name;          /*  display basename             */
-  gint               width, height;         /*  width and height attributes  */
-  gdouble            xresolution;           /*  image x-res, in dpi          */
-  gdouble            yresolution;           /*  image y-res, in dpi          */
-  GimpUnit           resolution_unit;       /*  resolution unit              */
-  GimpImageBaseType  base_type;             /*  base gimp_image type         */
-
-  guchar            *colormap;              /*  colormap (for indexed)       */
-  gint               n_colors;              /*  # of colors (for indexed)    */
-
-  gint               dirty;                 /*  dirty flag -- # of ops       */
-  guint              dirty_time;            /*  time when image became dirty */
-  gint               undo_freeze_count;     /*  counts the _freeze's         */
-
-  gint               export_dirty;          /*  'dirty' but for export       */
-
-  gint               instance_count;        /*  number of instances          */
-  gint               disp_count;            /*  number of displays           */
-
-  GimpTattoo         tattoo_state;          /*  the last used tattoo         */
-
-  GimpProjection    *projection;            /*  projection layers & channels */
-  GeglNode          *graph;                 /*  GEGL projection graph        */
-
-  GList             *guides;                /*  guides                       */
-  GimpGrid          *grid;                  /*  grid                         */
-  GList             *sample_points;         /*  color sample points          */
-
-  /*  Layer/Channel attributes  */
-  GimpContainer     *layers;                /*  the list of layers           */
-  GimpContainer     *channels;              /*  the list of masks            */
-  GimpContainer     *vectors;               /*  the list of vectors          */
-  GSList            *layer_stack;           /*  the layers in MRU order      */
-
-  GQuark             layer_alpha_handler;
-  GQuark             channel_name_changed_handler;
-  GQuark             channel_color_changed_handler;
-
-  GimpLayer         *active_layer;          /*  the active layer             */
-  GimpChannel       *active_channel;        /*  the active channel           */
-  GimpVectors       *active_vectors;        /*  the active vectors           */
-
-  GimpLayer         *floating_sel;          /*  the FS layer                 */
-  GimpChannel       *selection_mask;        /*  the selection mask channel   */
-
-  GimpParasiteList  *parasites;             /*  Plug-in parasite data        */
-
-  gboolean           visible[MAX_CHANNELS]; /*  visible channels             */
-  gboolean           active[MAX_CHANNELS];  /*  active channels              */
-
-  gboolean           quick_mask_state;      /*  TRUE if quick mask is on       */
-  gboolean           quick_mask_inverted;   /*  TRUE if quick mask is inverted */
-  GimpRGB            quick_mask_color;      /*  rgba triplet of the color      */
-
-  /*  Undo apparatus  */
-  GimpUndoStack     *undo_stack;            /*  stack for undo operations    */
-  GimpUndoStack     *redo_stack;            /*  stack for redo operations    */
-  gint               group_count;           /*  nested undo groups           */
-  GimpUndoType       pushing_undo_group;    /*  undo group status flag       */
-
-  /*  Preview  */
-  TempBuf           *preview;               /*  the projection preview       */
-
-  /*  Signal emmision accumulator  */
-  GimpImageFlushAccumulator  flush_accum;
+  Gimp         *gimp;  /*  the GIMP the image belongs to  */
 };
 
 struct _GimpImageClass
@@ -297,6 +215,9 @@ void       gimp_image_floating_selection_changed (GimpImage          *image);
 GimpChannel   * gimp_image_get_mask              (const GimpImage    *image);
 void            gimp_image_mask_changed          (GimpImage          *image);
 
+
+/*  image components  */
+
 gint            gimp_image_get_component_index   (const GimpImage    *image,
                                                   GimpChannelType     channel);
 
@@ -305,16 +226,23 @@ void            gimp_image_set_component_active  (GimpImage          *image,
                                                   gboolean            active);
 gboolean        gimp_image_get_component_active  (const GimpImage    *image,
                                                   GimpChannelType     type);
+void            gimp_image_get_active_array      (const GimpImage    *image,
+                                                  gboolean           *components);
 
 void            gimp_image_set_component_visible (GimpImage          *image,
                                                   GimpChannelType     type,
                                                   gboolean            visible);
 gboolean        gimp_image_get_component_visible (const GimpImage    *image,
                                                   GimpChannelType     type);
+void            gimp_image_get_visible_array     (const GimpImage    *image,
+                                                  gboolean           *components);
+
+
+/*  emitting image signals  */
 
 void            gimp_image_mode_changed          (GimpImage          *image);
 void            gimp_image_alpha_changed         (GimpImage          *image);
-void            gimp_image_update                (GimpImage          *image,
+void            gimp_image_invalidate            (GimpImage          *image,
                                                   gint                x,
                                                   gint                y,
                                                   gint                width,
@@ -337,26 +265,22 @@ void            gimp_image_size_changed_detailed (GimpImage          *image,
                                                   gint                previous_origin_y,
                                                   gint                previous_width,
                                                   gint                previous_height);
-
-
-/*  undo  */
-
-gboolean        gimp_image_undo_is_enabled       (const GimpImage    *image);
-gboolean        gimp_image_undo_enable           (GimpImage          *image);
-gboolean        gimp_image_undo_disable          (GimpImage          *image);
-gboolean        gimp_image_undo_freeze           (GimpImage          *image);
-gboolean        gimp_image_undo_thaw             (GimpImage          *image);
 void            gimp_image_undo_event            (GimpImage          *image,
                                                   GimpUndoEvent       event,
                                                   GimpUndo           *undo);
+
+
+/*  dirty counters  */
+
 gint            gimp_image_dirty                 (GimpImage          *image,
                                                   GimpDirtyMask       dirty_mask);
 gint            gimp_image_clean                 (GimpImage          *image,
                                                   GimpDirtyMask       dirty_mask);
 void            gimp_image_clean_all             (GimpImage          *image);
+void            gimp_image_export_clean_all      (GimpImage          *image);
 gint            gimp_image_is_dirty              (const GimpImage    *image);
 gboolean        gimp_image_is_export_dirty       (const GimpImage    *image);
-void            gimp_image_export_clean_all      (GimpImage          *image);
+gint            gimp_image_get_dirty_time        (const GimpImage    *image);
 
 
 /*  flush this image's displays  */
@@ -364,6 +288,16 @@ void            gimp_image_export_clean_all      (GimpImage          *image);
 void            gimp_image_flush                 (GimpImage          *image);
 
 
+/*  display / instance counters  */
+
+gint            gimp_image_get_display_count     (const GimpImage    *image);
+void            gimp_image_inc_display_count     (GimpImage          *image);
+void            gimp_image_dec_display_count     (GimpImage          *image);
+
+gint            gimp_image_get_instance_count    (const GimpImage    *image);
+void            gimp_image_inc_instance_count    (GimpImage          *image);
+
+
 /*  color transforms / utilities  */
 
 void            gimp_image_get_foreground        (const GimpImage    *image,
@@ -420,6 +354,10 @@ GimpProjection * gimp_image_get_projection       (const GimpImage    *image);
 
 /*  layers / channels / vectors  */
 
+GimpItemTree  * gimp_image_get_layer_tree        (const GimpImage    *image);
+GimpItemTree  * gimp_image_get_channel_tree      (const GimpImage    *image);
+GimpItemTree  * gimp_image_get_vectors_tree      (const GimpImage    *image);
+
 GimpContainer * gimp_image_get_layers            (const GimpImage    *image);
 GimpContainer * gimp_image_get_channels          (const GimpImage    *image);
 GimpContainer * gimp_image_get_vectors           (const GimpImage    *image);
@@ -553,12 +491,7 @@ gboolean        gimp_image_reorder_vectors       (GimpImage          *image,
                                                   gint                new_index,
                                                   gboolean            push_undo,
                                                   const gchar        *undo_desc);
-gboolean        gimp_image_layer_boundary        (const GimpImage    *image,
-                                                  BoundSeg          **segs,
-                                                  gint               *n_segs);
-GimpLayer     * gimp_image_pick_correlate_layer  (const GimpImage    *image,
-                                                  gint                x,
-                                                  gint                y);
+
 gboolean    gimp_image_coords_in_active_pickable (GimpImage          *image,
                                                   const GimpCoords   *coords,
                                                   gboolean            sample_merged,
