@@ -9,9 +9,9 @@
  * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
  *                          Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,8 +20,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_IMAGEFILE_H__
@@ -47,7 +46,6 @@ struct _GimpImagefile
 
   Gimp               *gimp;
   GimpThumbnail      *thumbnail;
-  gchar              *mime_type;
   gchar              *description;
   gboolean            static_desc;
 };
@@ -64,6 +62,8 @@ GType           gimp_imagefile_get_type              (void) G_GNUC_CONST;
 
 GimpImagefile * gimp_imagefile_new                   (Gimp          *gimp,
                                                       const gchar   *uri);
+void            gimp_imagefile_set_mime_type         (GimpImagefile *imagefile,
+                                                      const gchar   *mime_type);
 void            gimp_imagefile_update                (GimpImagefile *imagefile);
 void            gimp_imagefile_create_thumbnail      (GimpImagefile *imagefile,
                                                       GimpContext   *context,
