@@ -35,6 +35,7 @@
 #include "gimpimage-undo-push.h"
 #include "gimpitem.h"
 #include "gimpitem-preview.h"
+#include "gimpitemtree.h"
 #include "gimplist.h"
 #include "gimpmarshal.h"
 #include "gimpparasitelist.h"
@@ -49,6 +50,7 @@ enum
   REMOVED,
   VISIBILITY_CHANGED,
   LINKED_CHANGED,
+  LOCK_CONTENT_CHANGED,
   LAST_SIGNAL
 };
 
@@ -61,55 +63,56 @@ enum
   PROP_OFFSET_X,
   PROP_OFFSET_Y,
   PROP_VISIBLE,
-  PROP_LINKED
+  PROP_LINKED,
+  PROP_LOCK_CONTENT
 };
 
 
 /*  local function prototypes  */
 
-static void       gimp_item_set_property      (GObject       *object,
-                                               guint          property_id,
-                                               const GValue  *value,
-                                               GParamSpec    *pspec);
-static void       gimp_item_get_property      (GObject       *object,
-                                               guint          property_id,
-                                               GValue        *value,
-                                               GParamSpec    *pspec);
-static void       gimp_item_finalize          (GObject       *object);
+static void       gimp_item_set_property            (GObject        *object,
+                                                     guint           property_id,
+                                                     const GValue   *value,
+                                                     GParamSpec     *pspec);
+static void       gimp_item_get_property            (GObject        *object,
+                                                     guint           property_id,
+                                                     GValue         *value,
+                                                     GParamSpec     *pspec);
+static void       gimp_item_finalize                (GObject        *object);
 
-static gint64     gimp_item_get_memsize       (GimpObject    *object,
-                                               gint64        *gui_size);
+static gint64     gimp_item_get_memsize             (GimpObject     *object,
+                                                     gint64         *gui_size);
 
-static void       gimp_item_real_visibility_changed
-                                              (GimpItem      *item);
+static void       gimp_item_real_visibility_changed (GimpItem       *item);
 
-static GimpItem * gimp_item_real_duplicate    (GimpItem      *item,
-                                               GType          new_type);
-static void       gimp_item_real_convert      (GimpItem      *item,
-                                               GimpImage     *dest_image);
-static gboolean   gimp_item_real_rename       (GimpItem      *item,
-                                               const gchar   *new_name,
-                                               const gchar   *undo_desc,
-                                               GError       **error);
-static void       gimp_item_real_translate    (GimpItem      *item,
-                                               gint           offset_x,
-                                               gint           offset_y,
-                                               gboolean       push_undo);
-static void       gimp_item_real_scale        (GimpItem      *item,
-                                               gint           new_width,
-                                               gint           new_height,
-                                               gint           new_offset_x,
-                                               gint           new_offset_y,
-                                               GimpInterpolationType  interpolation,
-                                               GimpProgress  *progress);
-static void       gimp_item_real_resize       (GimpItem      *item,
-                                               GimpContext   *context,
-                                               gint           new_width,
-                                               gint           new_height,
-                                               gint           offset_x,
-                                               gint           offset_y);
-static GeglNode * gimp_item_real_get_node     (GimpItem      *item);
-static void       gimp_item_sync_offset_node  (GimpItem      *item);
+static gboolean   gimp_item_real_is_content_locked  (const GimpItem *item);
+static GimpItem * gimp_item_real_duplicate          (GimpItem       *item,
+                                                     GType           new_type);
+static void       gimp_item_real_convert            (GimpItem       *item,
+                                                     GimpImage      *dest_image);
+static gboolean   gimp_item_real_rename             (GimpItem       *item,
+                                                     const gchar    *new_name,
+                                                     const gchar    *undo_desc,
+                                                     GError        **error);
+static void       gimp_item_real_translate          (GimpItem       *item,
+                                                     gint            offset_x,
+                                                     gint            offset_y,
+                                                     gboolean        push_undo);
+static void       gimp_item_real_scale              (GimpItem       *item,
+                                                     gint            new_width,
+                                                     gint            new_height,
+                                                     gint            new_offset_x,
+                                                     gint            new_offset_y,
+                                                     GimpInterpolationType interpolation,
+                                                     GimpProgress   *progress);
+static void       gimp_item_real_resize             (GimpItem       *item,
+                                                     GimpContext    *context,
+                                                     gint            new_width,
+                                                     gint            new_height,
+                                                     gint            offset_x,
+                                                     gint            offset_y);
+static GeglNode * gimp_item_real_get_node           (GimpItem       *item);
+static void       gimp_item_sync_offset_node        (GimpItem       *item);
 
 
 G_DEFINE_TYPE (GimpItem, gimp_item, GIMP_TYPE_VIEWABLE)
@@ -153,6 +156,15 @@ gimp_item_class_init (GimpItemClass *klass)
                   gimp_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+  gimp_item_signals[LOCK_CONTENT_CHANGED] =
+    g_signal_new ("lock-content-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpItemClass, lock_content_changed),
+                  NULL, NULL,
+                  gimp_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   object_class->set_property       = gimp_item_set_property;
   object_class->get_property       = gimp_item_get_property;
   object_class->finalize           = gimp_item_finalize;
@@ -165,9 +177,11 @@ gimp_item_class_init (GimpItemClass *klass)
   klass->removed                   = NULL;
   klass->visibility_changed        = gimp_item_real_visibility_changed;
   klass->linked_changed            = NULL;
+  klass->lock_content_changed      = NULL;
 
   klass->is_attached               = NULL;
-  klass->get_container             = NULL;
+  klass->is_content_locked         = gimp_item_real_is_content_locked;
+  klass->get_tree                  = NULL;
   klass->duplicate                 = gimp_item_real_duplicate;
   klass->convert                   = gimp_item_real_convert;
   klass->rename                    = gimp_item_real_rename;
@@ -225,6 +239,12 @@ gimp_item_class_init (GimpItemClass *klass)
                                    g_param_spec_boolean ("linked", NULL, NULL,
                                                          FALSE,
                                                          GIMP_PARAM_READABLE));
+
+  g_object_class_install_property (object_class, PROP_LOCK_CONTENT,
+                                   g_param_spec_boolean ("lock-content",
+                                                         NULL, NULL,
+                                                         FALSE,
+                                                         GIMP_PARAM_READABLE));
 }
 
 static void
@@ -232,19 +252,20 @@ gimp_item_init (GimpItem *item)
 {
   g_object_force_floating (G_OBJECT (item));
 
-  item->ID          = 0;
-  item->tattoo      = 0;
-  item->image       = NULL;
-  item->parasites   = gimp_parasite_list_new ();
-  item->width       = 0;
-  item->height      = 0;
-  item->offset_x    = 0;
-  item->offset_y    = 0;
-  item->visible     = TRUE;
-  item->linked      = FALSE;
-  item->removed     = FALSE;
-  item->node        = NULL;
-  item->offset_node = NULL;
+  item->ID           = 0;
+  item->tattoo       = 0;
+  item->image        = NULL;
+  item->parasites    = gimp_parasite_list_new ();
+  item->width        = 0;
+  item->height       = 0;
+  item->offset_x     = 0;
+  item->offset_y     = 0;
+  item->visible      = TRUE;
+  item->linked       = FALSE;
+  item->lock_content = FALSE;
+  item->removed      = FALSE;
+  item->node         = NULL;
+  item->offset_node  = NULL;
 }
 
 static void
@@ -292,6 +313,9 @@ gimp_item_get_property (GObject    *object,
     case PROP_LINKED:
       g_value_set_boolean (value, item->linked);
       break;
+    case PROP_LOCK_CONTENT:
+      g_value_set_boolean (value, item->lock_content);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -362,6 +386,17 @@ gimp_item_real_visibility_changed (GimpItem *item)
     }
 }
 
+static gboolean
+gimp_item_real_is_content_locked (const GimpItem *item)
+{
+  GimpItem *parent = gimp_item_get_parent (item);
+
+  if (parent && gimp_item_is_content_locked (parent))
+    return TRUE;
+
+  return item->lock_content;
+}
+
 static GimpItem *
 gimp_item_real_duplicate (GimpItem *item,
                           GType     new_type)
@@ -380,10 +415,11 @@ gimp_item_real_duplicate (GimpItem *item,
     gint         number;
     gint         len;
 
-    name = gimp_object_get_name (GIMP_OBJECT (item));
+    name = gimp_object_get_name (item);
 
     g_return_val_if_fail (name != NULL, NULL);
 
+
     ext = strrchr (name, '#');
     len = strlen (_("copy"));
 
@@ -414,8 +450,12 @@ gimp_item_real_duplicate (GimpItem *item,
   g_object_unref (new_item->parasites);
   new_item->parasites = gimp_parasite_list_copy (item->parasites);
 
-  new_item->visible = gimp_item_get_visible (item);
-  new_item->linked  = gimp_item_get_linked (item);
+  gimp_item_set_visible (new_item, gimp_item_get_visible (item), FALSE);
+  gimp_item_set_linked  (new_item, gimp_item_get_linked (item),  FALSE);
+
+  if (gimp_item_can_lock_content (new_item))
+    gimp_item_set_lock_content (new_item, gimp_item_get_lock_content (item),
+                                FALSE);
 
   return new_item;
 }
@@ -434,9 +474,10 @@ gimp_item_real_rename (GimpItem     *item,
                        GError      **error)
 {
   if (gimp_item_is_attached (item))
-    gimp_image_undo_push_item_rename (item->image, undo_desc, item);
-
-  gimp_object_set_name (GIMP_OBJECT (item), new_name);
+    gimp_item_tree_rename_item (gimp_item_get_tree (item), item,
+                                new_name, TRUE, undo_desc);
+  else
+    gimp_object_set_name (GIMP_OBJECT (item), new_name);
 
   return TRUE;
 }
@@ -621,10 +662,11 @@ gimp_item_configure (GimpItem    *item,
 
   gimp_item_set_offset (item, offset_x, offset_y);
 
-  if (name)
+  if (name && strlen (name))
     gimp_object_set_name (GIMP_OBJECT (item), name);
   else
-    gimp_object_set_static_name (GIMP_OBJECT (item), _("Unnamed"));
+    gimp_object_set_static_name (GIMP_OBJECT (item),
+                                 GIMP_ITEM_GET_CLASS (item)->default_name);
 
   g_object_thaw_notify (G_OBJECT (item));
 }
@@ -636,34 +678,56 @@ gimp_item_configure (GimpItem    *item,
  * Returns: %TRUE if the item is attached to an image, %FALSE otherwise.
  */
 gboolean
-gimp_item_is_attached (GimpItem *item)
+gimp_item_is_attached (const GimpItem *item)
 {
-  GimpViewable *parent;
+  GimpItem *parent;
 
   g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
 
-  parent = gimp_viewable_get_parent (GIMP_VIEWABLE (item));
+  parent = gimp_item_get_parent (item);
 
   if (parent)
-    return gimp_item_is_attached (GIMP_ITEM (parent));
+    return gimp_item_is_attached (parent);
 
   return GIMP_ITEM_GET_CLASS (item)->is_attached (item);
 }
 
+GimpItem *
+gimp_item_get_parent (const GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), NULL);
+
+  return GIMP_ITEM (gimp_viewable_get_parent (GIMP_VIEWABLE (item)));
+}
+
+GimpItemTree *
+gimp_item_get_tree (GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), NULL);
+
+  if (GIMP_ITEM_GET_CLASS (item)->get_tree)
+    return GIMP_ITEM_GET_CLASS (item)->get_tree (item);
+
+  return NULL;
+}
+
 GimpContainer *
 gimp_item_get_container (GimpItem *item)
 {
-  GimpViewable *parent;
+  GimpItem     *parent;
+  GimpItemTree *tree;
 
   g_return_val_if_fail (GIMP_IS_ITEM (item), NULL);
 
-  parent = gimp_viewable_get_parent (GIMP_VIEWABLE (item));
+  parent = gimp_item_get_parent (item);
 
   if (parent)
     return gimp_viewable_get_children (GIMP_VIEWABLE (parent));
 
-  if (GIMP_ITEM_GET_CLASS (item)->get_container)
-    return GIMP_ITEM_GET_CLASS (item)->get_container (item);
+  tree = gimp_item_get_tree (item);
+
+  if (tree)
+    return tree->container;
 
   return NULL;
 }
@@ -698,6 +762,35 @@ gimp_item_get_index (GimpItem *item)
   return -1;
 }
 
+GList *
+gimp_item_get_path (GimpItem *item)
+{
+  GimpContainer *container;
+  GList         *path = NULL;
+
+  g_return_val_if_fail (GIMP_IS_ITEM (item), NULL);
+  g_return_val_if_fail (gimp_item_is_attached (item), NULL);
+
+  container = gimp_item_get_container (item);
+
+  while (container)
+    {
+      guint32 index = gimp_container_get_child_index (container,
+                                                      GIMP_OBJECT (item));
+
+      path = g_list_prepend (path, GUINT_TO_POINTER (index));
+
+      item = gimp_item_get_parent (item);
+
+      if (item)
+        container = gimp_item_get_container (item);
+      else
+        container = NULL;
+    }
+
+  return path;
+}
+
 /**
  * gimp_item_duplicate:
  * @item:     The #GimpItem to duplicate.
@@ -773,7 +866,7 @@ gimp_item_rename (GimpItem     *item,
   if (! new_name || ! *new_name)
     new_name = item_class->default_name;
 
-  if (strcmp (new_name, gimp_object_get_name (GIMP_OBJECT (item))))
+  if (strcmp (new_name, gimp_object_get_name (item)))
     return item_class->rename (item, new_name, item_class->rename_desc, error);
 
   return TRUE;
@@ -807,6 +900,36 @@ gimp_item_get_height (const GimpItem *item)
   return item->height;
 }
 
+void
+gimp_item_set_size (GimpItem *item,
+                    gint      width,
+                    gint      height)
+{
+  g_return_if_fail (GIMP_IS_ITEM (item));
+
+  if (item->width  != width ||
+      item->height != height)
+    {
+      g_object_freeze_notify (G_OBJECT (item));
+
+      if (item->width != width)
+        {
+          item->width = width;
+          g_object_notify (G_OBJECT (item), "width");
+        }
+
+      if (item->height != height)
+        {
+          item->height = height;
+          g_object_notify (G_OBJECT (item), "height");
+        }
+
+      g_object_thaw_notify (G_OBJECT (item));
+
+      gimp_viewable_size_changed (GIMP_VIEWABLE (item));
+    }
+}
+
 /**
  * gimp_item_get_offset:
  * @item:     The #GimpItem to check.
@@ -1299,6 +1422,14 @@ gimp_item_get_node (GimpItem *item)
   return GIMP_ITEM_GET_CLASS (item)->get_node (item);
 }
 
+GeglNode *
+gimp_item_peek_node (GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), NULL);
+
+  return item->node;
+}
+
 GeglNode *
 gimp_item_get_offset_node (GimpItem *item)
 {
@@ -1409,7 +1540,7 @@ gimp_item_parasite_attach (GimpItem           *item,
           /* do a group in case we have attach_parent set */
           gimp_image_undo_group_start (item->image,
                                        GIMP_UNDO_GROUP_PARASITE_ATTACH,
-                                       _("Attach Parasite"));
+                                       C_("undo-type", "Attach Parasite"));
 
           gimp_image_undo_push_item_parasite (item->image, NULL, item, &copy);
         }
@@ -1419,7 +1550,7 @@ gimp_item_parasite_attach (GimpItem           *item,
                                         (item, gimp_parasite_name (&copy))))
         {
           gimp_image_undo_push_cantundo (item->image,
-                                         _("Attach Parasite to Item"));
+                                         C_("undo-type", "Attach Parasite to Item"));
         }
     }
 
@@ -1461,14 +1592,14 @@ gimp_item_parasite_detach (GimpItem    *item,
   if (gimp_parasite_is_undoable (parasite))
     {
       gimp_image_undo_push_item_parasite_remove (item->image,
-                                                 _("Remove Parasite from Item"),
+                                                 C_("undo-type", "Remove Parasite from Item"),
                                                  item,
                                                  gimp_parasite_name (parasite));
     }
   else if (gimp_parasite_is_persistent (parasite))
     {
       gimp_image_undo_push_cantundo (item->image,
-                                     _("Remove Parasite from Item"));
+                                     C_("undo-type", "Remove Parasite from Item"));
     }
 
   gimp_parasite_list_remove (item->parasites, name);
@@ -1512,14 +1643,6 @@ gimp_item_parasite_list (const GimpItem *item,
   return list;
 }
 
-gboolean
-gimp_item_get_visible (const GimpItem *item)
-{
-  g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
-
-  return item->visible;
-}
-
 void
 gimp_item_set_visible (GimpItem *item,
                        gboolean  visible,
@@ -1527,6 +1650,8 @@ gimp_item_set_visible (GimpItem *item,
 {
   g_return_if_fail (GIMP_IS_ITEM (item));
 
+  visible = visible ? TRUE : FALSE;
+
   if (gimp_item_get_visible (item) != visible)
     {
       if (push_undo && gimp_item_is_attached (item))
@@ -1537,7 +1662,7 @@ gimp_item_set_visible (GimpItem *item,
             gimp_image_undo_push_item_visibility (image, NULL, item);
         }
 
-      item->visible = visible ? TRUE : FALSE;
+      item->visible = visible;
 
       g_signal_emit (item, gimp_item_signals[VISIBILITY_CHANGED], 0);
 
@@ -1545,6 +1670,14 @@ gimp_item_set_visible (GimpItem *item,
     }
 }
 
+gboolean
+gimp_item_get_visible (const GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
+
+  return item->visible;
+}
+
 void
 gimp_item_set_linked (GimpItem *item,
                       gboolean  linked,
@@ -1552,6 +1685,8 @@ gimp_item_set_linked (GimpItem *item,
 {
   g_return_if_fail (GIMP_IS_ITEM (item));
 
+  linked = linked ? TRUE : FALSE;
+
   if (gimp_item_get_linked (item) != linked)
     {
       if (push_undo && gimp_item_is_attached (item))
@@ -1562,7 +1697,7 @@ gimp_item_set_linked (GimpItem *item,
             gimp_image_undo_push_item_linked (image, NULL, item);
         }
 
-      item->linked = linked ? TRUE : FALSE;
+      item->linked = linked;
 
       g_signal_emit (item, gimp_item_signals[LINKED_CHANGED], 0);
 
@@ -1578,6 +1713,60 @@ gimp_item_get_linked (const GimpItem *item)
   return item->linked;
 }
 
+void
+gimp_item_set_lock_content (GimpItem *item,
+                            gboolean  lock_content,
+                            gboolean  push_undo)
+{
+  g_return_if_fail (GIMP_IS_ITEM (item));
+  g_return_if_fail (gimp_item_can_lock_content (item));
+
+  lock_content = lock_content ? TRUE : FALSE;
+
+  if (gimp_item_get_lock_content (item) != lock_content)
+    {
+      if (push_undo && gimp_item_is_attached (item))
+        {
+          /* Right now I don't think this should be pushed. */
+#if 0
+          GimpImage *image = gimp_item_get_image (item);
+
+          gimp_image_undo_push_item_lock_content (image, NULL, item);
+#endif
+        }
+
+      item->lock_content = lock_content;
+
+      g_signal_emit (item, gimp_item_signals[LOCK_CONTENT_CHANGED], 0);
+
+      g_object_notify (G_OBJECT (item), "lock-content");
+    }
+}
+
+gboolean
+gimp_item_get_lock_content (const GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
+
+  return item->lock_content;
+}
+
+gboolean
+gimp_item_can_lock_content (const GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
+
+  return TRUE;
+}
+
+gboolean
+gimp_item_is_content_locked (const GimpItem *item)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM (item), FALSE);
+
+  return GIMP_ITEM_GET_CLASS (item)->is_content_locked (item);
+}
+
 gboolean
 gimp_item_is_in_set (GimpItem    *item,
                      GimpItemSet  set)
