@@ -46,10 +46,11 @@ struct _GimpItem
   gint              width, height;      /*  size in pixels           */
   gint              offset_x, offset_y; /*  pixel offset in image    */
 
-  gboolean          visible;            /*  control visibility       */
-  gboolean          linked;             /*  control linkage          */
+  guint             visible      : 1;   /*  control visibility       */
+  guint             linked       : 1;   /*  control linkage          */
+  guint             lock_content : 1;   /*  content editability      */
 
-  gboolean          removed;            /*  removed from the image?  */
+  guint             removed : 1;        /*  removed from the image?  */
 
   GeglNode         *node;               /*  the GEGL node to plug
                                             into the graph           */
@@ -61,13 +62,15 @@ struct _GimpItemClass
   GimpViewableClass  parent_class;
 
   /*  signals  */
-  void            (* removed)            (GimpItem               *item);
-  void            (* visibility_changed) (GimpItem               *item);
-  void            (* linked_changed)     (GimpItem               *item);
+  void            (* removed)              (GimpItem             *item);
+  void            (* visibility_changed)   (GimpItem             *item);
+  void            (* linked_changed)       (GimpItem             *item);
+  void            (* lock_content_changed) (GimpItem             *item);
 
   /*  virtual functions  */
-  gboolean        (* is_attached)        (GimpItem               *item);
-  GimpContainer * (* get_container)      (GimpItem               *item);
+  gboolean        (* is_attached)        (const GimpItem         *item);
+  gboolean        (* is_content_locked)  (const GimpItem         *item);
+  GimpItemTree  * (* get_tree)           (GimpItem               *item);
   GimpItem      * (* duplicate)          (GimpItem               *item,
                                           GType                   new_type);
   void            (* convert)            (GimpItem               *item,
@@ -138,11 +141,15 @@ GType           gimp_item_get_type           (void) G_GNUC_CONST;
 void            gimp_item_removed            (GimpItem           *item);
 gboolean        gimp_item_is_removed         (const GimpItem     *item);
 
-gboolean        gimp_item_is_attached        (GimpItem           *item);
+gboolean        gimp_item_is_attached        (const GimpItem     *item);
 
+GimpItem      * gimp_item_get_parent         (const GimpItem     *item);
+
+GimpItemTree  * gimp_item_get_tree           (GimpItem           *item);
 GimpContainer * gimp_item_get_container      (GimpItem           *item);
 GList         * gimp_item_get_container_iter (GimpItem           *item);
 gint            gimp_item_get_index          (GimpItem           *item);
+GList         * gimp_item_get_path           (GimpItem           *item);
 
 void            gimp_item_configure          (GimpItem           *item,
                                               GimpImage          *image,
@@ -163,6 +170,9 @@ gboolean        gimp_item_rename             (GimpItem           *item,
 
 gint            gimp_item_get_width          (const GimpItem     *item);
 gint            gimp_item_get_height         (const GimpItem     *item);
+void            gimp_item_set_size           (GimpItem           *item,
+                                              gint                width,
+                                              gint                height);
 
 void            gimp_item_get_offset         (const GimpItem     *item,
                                               gint               *offset_x,
@@ -237,6 +247,7 @@ gboolean        gimp_item_stroke             (GimpItem           *item,
                                               GError            **error);
 
 GeglNode      * gimp_item_get_node           (GimpItem           *item);
+GeglNode      * gimp_item_peek_node          (GimpItem           *item);
 GeglNode      * gimp_item_get_offset_node    (GimpItem           *item);
 
 
@@ -261,16 +272,23 @@ const GimpParasite * gimp_item_parasite_find (const GimpItem     *item,
 gchar        ** gimp_item_parasite_list      (const GimpItem     *item,
                                               gint               *count);
 
-gboolean        gimp_item_get_visible        (const GimpItem     *item);
 void            gimp_item_set_visible        (GimpItem           *item,
                                               gboolean            visible,
                                               gboolean            push_undo);
+gboolean        gimp_item_get_visible        (const GimpItem     *item);
 
 void            gimp_item_set_linked         (GimpItem           *item,
                                               gboolean            linked,
                                               gboolean            push_undo);
 gboolean        gimp_item_get_linked         (const GimpItem     *item);
 
+void            gimp_item_set_lock_content   (GimpItem           *item,
+                                              gboolean            lock_content,
+                                              gboolean            push_undo);
+gboolean        gimp_item_get_lock_content   (const GimpItem     *item);
+gboolean        gimp_item_can_lock_content   (const GimpItem     *item);
+gboolean        gimp_item_is_content_locked  (const GimpItem     *item);
+
 gboolean        gimp_item_is_in_set          (GimpItem           *item,
                                               GimpItemSet         set);
 
