@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,13 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <glib-object.h>
+#include <gegl.h>
 
 #include "core-types.h"
 
@@ -107,8 +106,9 @@ gimp_item_undo_set_property (GObject      *object,
   switch (property_id)
     {
     case PROP_ITEM:
-      item_undo->item = (GimpItem *) g_value_dup_object (value);
+      item_undo->item = g_value_dup_object (value);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -128,6 +128,7 @@ gimp_item_undo_get_property (GObject    *object,
     case PROP_ITEM:
       g_value_set_object (value, item_undo->item);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -140,11 +141,11 @@ gimp_item_undo_free (GimpUndo     *undo,
 {
   GimpItemUndo *item_undo = GIMP_ITEM_UNDO (undo);
 
-  GIMP_UNDO_CLASS (parent_class)->free (undo, undo_mode);
-
   if (item_undo->item)
     {
       g_object_unref (item_undo->item);
       item_undo->item = NULL;
     }
+
+  GIMP_UNDO_CLASS (parent_class)->free (undo, undo_mode);
 }
