@@ -96,9 +96,8 @@ static gchar    * gimp_layer_get_description    (GimpViewable       *viewable,
                                                  gchar             **tooltip);
 
 static void       gimp_layer_removed            (GimpItem           *item);
-static gboolean   gimp_layer_is_attached        (GimpItem           *item);
-static GimpContainer *
-                  gimp_layer_get_container      (GimpItem           *item);
+static gboolean   gimp_layer_is_attached        (const GimpItem     *item);
+static GimpItemTree * gimp_layer_get_tree       (GimpItem           *item);
 static GimpItem * gimp_layer_duplicate          (GimpItem           *item,
                                                  GType               new_type);
 static void       gimp_layer_convert            (GimpItem           *item,
@@ -143,6 +142,7 @@ static void       gimp_layer_transform          (GimpItem           *item,
                                                  gint                recursion_level,
                                                  GimpTransformResize    clip_result,
                                                  GimpProgress       *progress);
+static GeglNode * gimp_layer_get_node           (GimpItem           *item);
 
 static gint64  gimp_layer_estimate_memsize      (const GimpDrawable *drawable,
                                                  gint                width,
@@ -150,7 +150,10 @@ static gint64  gimp_layer_estimate_memsize      (const GimpDrawable *drawable,
 static void    gimp_layer_invalidate_boundary   (GimpDrawable       *drawable);
 static void    gimp_layer_get_active_components (const GimpDrawable *drawable,
                                                  gboolean           *active);
-static GeglNode * gimp_layer_get_node           (GimpItem           *item);
+static void    gimp_layer_convert_type          (GimpDrawable       *drawable,
+                                                 GimpImage          *dest_image,
+                                                 GimpImageBaseType   new_base_type,
+                                                 gboolean            push_undo);
 
 static gint    gimp_layer_get_opacity_at        (GimpPickable       *pickable,
                                                  gint                x,
@@ -169,6 +172,8 @@ static void       gimp_layer_layer_mask_update  (GimpDrawable       *layer_mask,
                                                  gint                height,
                                                  GimpLayer          *layer);
 
+static void       gimp_layer_sync_mode_node     (GimpLayer          *layer);
+
 
 G_DEFINE_TYPE_WITH_CODE (GimpLayer, gimp_layer, GIMP_TYPE_DRAWABLE,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_PICKABLE,
@@ -238,7 +243,7 @@ gimp_layer_class_init (GimpLayerClass *klass)
 
   item_class->removed                 = gimp_layer_removed;
   item_class->is_attached             = gimp_layer_is_attached;
-  item_class->get_container           = gimp_layer_get_container;
+  item_class->get_tree                = gimp_layer_get_tree;
   item_class->duplicate               = gimp_layer_duplicate;
   item_class->convert                 = gimp_layer_convert;
   item_class->rename                  = gimp_layer_rename;
@@ -250,17 +255,18 @@ gimp_layer_class_init (GimpLayerClass *klass)
   item_class->transform               = gimp_layer_transform;
   item_class->get_node                = gimp_layer_get_node;
   item_class->default_name            = _("Layer");
-  item_class->rename_desc             = _("Rename Layer");
-  item_class->translate_desc          = _("Move Layer");
-  item_class->scale_desc              = _("Scale Layer");
-  item_class->resize_desc             = _("Resize Layer");
-  item_class->flip_desc               = _("Flip Layer");
-  item_class->rotate_desc             = _("Rotate Layer");
-  item_class->transform_desc          = _("Transform Layer");
+  item_class->rename_desc             = C_("undo-type", "Rename Layer");
+  item_class->translate_desc          = C_("undo-type", "Move Layer");
+  item_class->scale_desc              = C_("undo-type", "Scale Layer");
+  item_class->resize_desc             = C_("undo-type", "Resize Layer");
+  item_class->flip_desc               = C_("undo-type", "Flip Layer");
+  item_class->rotate_desc             = C_("undo-type", "Rotate Layer");
+  item_class->transform_desc          = C_("undo-type", "Transform Layer");
 
   drawable_class->estimate_memsize      = gimp_layer_estimate_memsize;
   drawable_class->invalidate_boundary   = gimp_layer_invalidate_boundary;
   drawable_class->get_active_components = gimp_layer_get_active_components;
+  drawable_class->convert_type          = gimp_layer_convert_type;
   drawable_class->project_region        = gimp_layer_project_region;
 
   klass->opacity_changed              = NULL;
@@ -439,8 +445,7 @@ gimp_layer_invalidate_preview (GimpViewable *viewable)
 {
   GimpLayer *layer = GIMP_LAYER (viewable);
 
-  if (GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview)
-    GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview (viewable);
+  GIMP_VIEWABLE_CLASS (parent_class)->invalidate_preview (viewable);
 
   if (gimp_layer_is_floating_sel (layer))
     floating_sel_invalidate (layer);
@@ -453,79 +458,13 @@ gimp_layer_get_description (GimpViewable  *viewable,
   if (gimp_layer_is_floating_sel (GIMP_LAYER (viewable)))
     {
       return g_strdup_printf (_("Floating Selection\n(%s)"),
-                              gimp_object_get_name (GIMP_OBJECT (viewable)));
+                              gimp_object_get_name (viewable));
     }
 
   return GIMP_VIEWABLE_CLASS (parent_class)->get_description (viewable,
                                                               tooltip);
 }
 
-static void
-gimp_layer_get_active_components (const GimpDrawable *drawable,
-                                  gboolean           *active)
-{
-  GimpLayer *layer = GIMP_LAYER (drawable);
-  GimpImage *image = gimp_item_get_image (GIMP_ITEM (drawable));
-  gint       i;
-
-  /*  first copy the image active channels  */
-  for (i = 0; i < MAX_CHANNELS; i++)
-    active[i] = image->active[i];
-
-  if (gimp_drawable_has_alpha (drawable) && layer->lock_alpha)
-    active[gimp_drawable_bytes (drawable) - 1] = FALSE;
-}
-
-static GeglNode *
-gimp_layer_get_node (GimpItem *item)
-{
-  GimpDrawable *drawable = GIMP_DRAWABLE (item);
-  GimpLayer    *layer    = GIMP_LAYER (item);
-  GeglNode     *node;
-  GeglNode     *offset_node;
-  GeglNode     *source;
-  GeglNode     *mode_node;
-
-  node = GIMP_ITEM_CLASS (parent_class)->get_node (item);
-
-  source = gimp_drawable_get_source_node (drawable);
-  gegl_node_add_child (node, source);
-
-  layer->opacity_node = gegl_node_new_child (node,
-                                             "operation", "gegl:opacity",
-                                             "value",     layer->opacity,
-                                             NULL);
-  gegl_node_connect_to (source,              "output",
-                        layer->opacity_node, "input");
-
-  if (layer->mask)
-    {
-      GeglNode *mask;
-
-      mask = gimp_drawable_get_source_node (GIMP_DRAWABLE (layer->mask));
-
-      gegl_node_connect_to (mask,                "output",
-                            layer->opacity_node, "aux");
-    }
-
-  offset_node = gimp_item_get_offset_node (GIMP_ITEM (layer));
-
-  gegl_node_connect_to (layer->opacity_node, "output",
-                        offset_node,         "input");
-
-  mode_node = gimp_drawable_get_mode_node (drawable);
-
-  gegl_node_set (mode_node,
-                 "operation",  "gimp:point-layer-mode",
-                 "blend-mode", layer->mode,
-                 NULL);
-
-  gegl_node_connect_to (offset_node, "output",
-                        mode_node,   "aux");
-
-  return node;
-}
-
 static void
 gimp_layer_removed (GimpItem *item)
 {
@@ -539,20 +478,23 @@ gimp_layer_removed (GimpItem *item)
 }
 
 static gboolean
-gimp_layer_is_attached (GimpItem *item)
+gimp_layer_is_attached (const GimpItem *item)
 {
-  return (GIMP_IS_IMAGE (gimp_item_get_image (item)) &&
-          gimp_container_have (gimp_item_get_image (item)->layers, GIMP_OBJECT (item)));
+  GimpImage *image = gimp_item_get_image (item);
+
+  return (GIMP_IS_IMAGE (image) &&
+          gimp_container_have (gimp_image_get_layers (image),
+                               GIMP_OBJECT (item)));
 }
 
-static GimpContainer *
-gimp_layer_get_container (GimpItem *item)
+static GimpItemTree *
+gimp_layer_get_tree (GimpItem *item)
 {
   if (gimp_item_is_attached (item))
     {
       GimpImage *image = gimp_item_get_image (item);
 
-      return gimp_image_get_layers (image);
+      return gimp_image_get_layer_tree (image);
     }
 
   return NULL;
@@ -573,12 +515,12 @@ gimp_layer_duplicate (GimpItem *item,
       GimpLayer *layer     = GIMP_LAYER (item);
       GimpLayer *new_layer = GIMP_LAYER (new_item);
 
-      gimp_layer_set_mode       (new_layer,
-                                 gimp_layer_get_mode (layer), FALSE);
-      gimp_layer_set_opacity    (new_layer,
-                                 gimp_layer_get_opacity (layer), FALSE);
-      gimp_layer_set_lock_alpha (new_layer,
-                                 gimp_layer_get_lock_alpha (layer), FALSE);
+      gimp_layer_set_mode    (new_layer, gimp_layer_get_mode (layer),    FALSE);
+      gimp_layer_set_opacity (new_layer, gimp_layer_get_opacity (layer), FALSE);
+
+      if (gimp_layer_can_lock_alpha (new_layer))
+        gimp_layer_set_lock_alpha (new_layer,
+                                   gimp_layer_get_lock_alpha (layer), FALSE);
 
       /*  duplicate the layer mask if necessary  */
       if (layer->mask)
@@ -607,60 +549,7 @@ gimp_layer_convert (GimpItem  *item,
   new_base_type = gimp_image_base_type (dest_image);
 
   if (old_base_type != new_base_type)
-    {
-      TileManager   *new_tiles;
-      GimpImageType  new_type;
-
-      new_type = GIMP_IMAGE_TYPE_FROM_BASE_TYPE (new_base_type);
-
-      if (gimp_drawable_has_alpha (drawable))
-        new_type = GIMP_IMAGE_TYPE_WITH_ALPHA (new_type);
-
-      new_tiles = tile_manager_new (gimp_item_get_width  (item),
-                                    gimp_item_get_height (item),
-                                    GIMP_IMAGE_TYPE_BYTES (new_type));
-
-      switch (new_base_type)
-        {
-        case GIMP_RGB:
-          gimp_drawable_convert_rgb (drawable,
-                                     new_tiles,
-                                     old_base_type);
-          break;
-
-        case GIMP_GRAY:
-          gimp_drawable_convert_grayscale (drawable,
-                                           new_tiles,
-                                           old_base_type);
-          break;
-
-        case GIMP_INDEXED:
-          {
-            PixelRegion layerPR;
-            PixelRegion newPR;
-
-            pixel_region_init (&layerPR, gimp_drawable_get_tiles (drawable),
-                               0, 0,
-                               gimp_item_get_width  (item),
-                               gimp_item_get_height (item),
-                               FALSE);
-            pixel_region_init (&newPR, new_tiles,
-                               0, 0,
-                               gimp_item_get_width  (item),
-                               gimp_item_get_height (item),
-                               TRUE);
-
-            gimp_layer_transform_color (dest_image,
-                                        &layerPR, gimp_drawable_type (drawable),
-                                        &newPR,   new_type);
-          }
-          break;
-        }
-
-      gimp_drawable_set_tiles (drawable, FALSE, NULL,
-                               new_tiles, new_type);
-      tile_manager_unref (new_tiles);
-    }
+    gimp_drawable_convert_type (drawable, dest_image, new_base_type, FALSE);
 
   if (layer->mask)
     gimp_item_set_image (GIMP_ITEM (layer->mask), dest_image);
@@ -857,6 +746,55 @@ gimp_layer_transform (GimpItem               *item,
                          clip_result, progress);
 }
 
+static GeglNode *
+gimp_layer_get_node (GimpItem *item)
+{
+  GimpDrawable *drawable = GIMP_DRAWABLE (item);
+  GimpLayer    *layer    = GIMP_LAYER (item);
+  GeglNode     *node;
+  GeglNode     *offset_node;
+  GeglNode     *source;
+  GeglNode     *mode_node;
+
+  node = GIMP_ITEM_CLASS (parent_class)->get_node (item);
+
+  source = gimp_drawable_get_source_node (drawable);
+  gegl_node_add_child (node, source);
+
+  g_warn_if_fail (layer->opacity_node == NULL);
+
+  layer->opacity_node = gegl_node_new_child (node,
+                                             "operation", "gegl:opacity",
+                                             "value",     layer->opacity,
+                                             NULL);
+  gegl_node_connect_to (source,              "output",
+                        layer->opacity_node, "input");
+
+  if (layer->mask)
+    {
+      GeglNode *mask;
+
+      mask = gimp_drawable_get_source_node (GIMP_DRAWABLE (layer->mask));
+
+      gegl_node_connect_to (mask,                "output",
+                            layer->opacity_node, "aux");
+    }
+
+  offset_node = gimp_item_get_offset_node (GIMP_ITEM (layer));
+
+  gegl_node_connect_to (layer->opacity_node, "output",
+                        offset_node,         "input");
+
+  gimp_layer_sync_mode_node (layer);
+
+  mode_node = gimp_drawable_get_mode_node (drawable);
+
+  gegl_node_connect_to (offset_node, "output",
+                        mode_node,   "aux");
+
+  return node;
+}
+
 static gint64
 gimp_layer_estimate_memsize (const GimpDrawable *drawable,
                              gint                width,
@@ -904,6 +842,74 @@ gimp_layer_invalidate_boundary (GimpDrawable *drawable)
     floating_sel_invalidate (layer);
 }
 
+static void
+gimp_layer_get_active_components (const GimpDrawable *drawable,
+                                  gboolean           *active)
+{
+  GimpLayer *layer = GIMP_LAYER (drawable);
+  GimpImage *image = gimp_item_get_image (GIMP_ITEM (drawable));
+
+  /*  first copy the image active channels  */
+  gimp_image_get_active_array (image, active);
+
+  if (gimp_drawable_has_alpha (drawable) && layer->lock_alpha)
+    active[gimp_drawable_bytes (drawable) - 1] = FALSE;
+}
+
+static void
+gimp_layer_convert_type (GimpDrawable      *drawable,
+                         GimpImage         *dest_image,
+                         GimpImageBaseType  new_base_type,
+                         gboolean           push_undo)
+{
+  switch (new_base_type)
+    {
+    case GIMP_RGB:
+    case GIMP_GRAY:
+      GIMP_DRAWABLE_CLASS (parent_class)->convert_type (drawable, dest_image,
+                                                        new_base_type,
+                                                        push_undo);
+      break;
+
+    case GIMP_INDEXED:
+      {
+        GimpItem      *item = GIMP_ITEM (drawable);
+        TileManager   *new_tiles;
+        GimpImageType  new_type;
+        PixelRegion    layerPR;
+        PixelRegion    newPR;
+
+        new_type = GIMP_IMAGE_TYPE_FROM_BASE_TYPE (new_base_type);
+
+        if (gimp_drawable_has_alpha (drawable))
+          new_type = GIMP_IMAGE_TYPE_WITH_ALPHA (new_type);
+
+        new_tiles = tile_manager_new (gimp_item_get_width  (item),
+                                      gimp_item_get_height (item),
+                                      GIMP_IMAGE_TYPE_BYTES (new_type));
+
+        pixel_region_init (&layerPR, gimp_drawable_get_tiles (drawable),
+                           0, 0,
+                           gimp_item_get_width  (item),
+                           gimp_item_get_height (item),
+                           FALSE);
+        pixel_region_init (&newPR, new_tiles,
+                           0, 0,
+                           gimp_item_get_width  (item),
+                           gimp_item_get_height (item),
+                           TRUE);
+
+        gimp_layer_transform_color (dest_image,
+                                    &layerPR, gimp_drawable_type (drawable),
+                                    &newPR,   new_type);
+
+        gimp_drawable_set_tiles (drawable, push_undo, NULL,
+                                 new_tiles, new_type);
+        tile_manager_unref (new_tiles);
+      }
+    }
+}
+
 static gint
 gimp_layer_get_opacity_at (GimpPickable *pickable,
                            gint          x,
@@ -955,8 +961,9 @@ gimp_layer_transform_color (GimpImage     *image,
                             PixelRegion   *destPR,
                             GimpImageType  dest_type)
 {
-  GimpImageBaseType base_type = GIMP_IMAGE_TYPE_BASE_TYPE (src_type);
-  gboolean          alpha     = GIMP_IMAGE_TYPE_HAS_ALPHA (src_type);
+  GimpImageBaseType base_type  = GIMP_IMAGE_TYPE_BASE_TYPE (src_type);
+  gboolean          src_alpha  = GIMP_IMAGE_TYPE_HAS_ALPHA (src_type);
+  gboolean          dest_alpha = GIMP_IMAGE_TYPE_HAS_ALPHA (dest_type);
   gpointer          pr;
 
   for (pr = pixel_regions_register (2, srcPR, destPR);
@@ -977,9 +984,9 @@ gimp_layer_transform_color (GimpImage     *image,
             {
               gimp_image_transform_color (image, dest_type, d, base_type, s);
 
-              /*  alpha channel  */
-              d[destPR->bytes - 1] = (alpha ?
-                                      s[srcPR->bytes - 1] : OPAQUE_OPACITY);
+              if (dest_alpha)
+                d[destPR->bytes - 1] = (src_alpha ?
+                                        s[srcPR->bytes - 1] : OPAQUE_OPACITY);
 
               s += srcPR->bytes;
               d += destPR->bytes;
@@ -1009,6 +1016,58 @@ gimp_layer_layer_mask_update (GimpDrawable *drawable,
     }
 }
 
+static void
+gimp_layer_sync_mode_node (GimpLayer *layer)
+{
+  if (layer->opacity_node)
+    {
+      GeglNode *mode_node;
+
+      mode_node = gimp_drawable_get_mode_node (GIMP_DRAWABLE (layer));
+
+      switch (layer->mode)
+        {
+        case GIMP_DISSOLVE_MODE:
+        case GIMP_BEHIND_MODE:
+        case GIMP_MULTIPLY_MODE:
+        case GIMP_SCREEN_MODE:
+        case GIMP_OVERLAY_MODE:
+        case GIMP_DIFFERENCE_MODE:
+        case GIMP_ADDITION_MODE:
+        case GIMP_SUBTRACT_MODE:
+        case GIMP_DARKEN_ONLY_MODE:
+        case GIMP_LIGHTEN_ONLY_MODE:
+        case GIMP_HUE_MODE:
+        case GIMP_SATURATION_MODE:
+        case GIMP_COLOR_MODE:
+        case GIMP_VALUE_MODE:
+        case GIMP_DIVIDE_MODE:
+        case GIMP_DODGE_MODE:
+        case GIMP_BURN_MODE:
+        case GIMP_HARDLIGHT_MODE:
+        case GIMP_SOFTLIGHT_MODE:
+        case GIMP_GRAIN_EXTRACT_MODE:
+        case GIMP_GRAIN_MERGE_MODE:
+        case GIMP_COLOR_ERASE_MODE:
+        case GIMP_ERASE_MODE:
+        case GIMP_REPLACE_MODE:
+        case GIMP_ANTI_ERASE_MODE:
+          gegl_node_set (mode_node,
+                         "operation",  "gimp:point-layer-mode",
+                         "blend-mode", layer->mode,
+                         NULL);
+          break;
+
+        default:
+          gegl_node_set (mode_node,
+                         "operation",
+                         gimp_layer_mode_to_gegl_operation (layer->mode),
+                         NULL);
+          break;
+        }
+    }
+}
+
 
 /*  public functions  */
 
@@ -1282,6 +1341,14 @@ gimp_layer_new_from_region (PixelRegion          *region,
   return new_layer;
 }
 
+GimpLayer *
+gimp_layer_get_parent (GimpLayer *layer)
+{
+  g_return_val_if_fail (GIMP_IS_LAYER (layer), NULL);
+
+  return GIMP_LAYER (gimp_viewable_get_parent (GIMP_VIEWABLE (layer)));
+}
+
 GimpLayerMask *
 gimp_layer_get_mask (const GimpLayer *layer)
 {
@@ -1329,7 +1396,7 @@ gimp_layer_add_mask (GimpLayer      *layer,
     }
 
   if (push_undo)
-    gimp_image_undo_push_layer_mask_add (image, _("Add Layer Mask"),
+    gimp_image_undo_push_layer_mask_add (image, C_("undo-type", "Add Layer Mask"),
                                          layer, mask);
 
   layer->mask = g_object_ref_sink (mask);
@@ -1389,7 +1456,7 @@ gimp_layer_create_mask (const GimpLayer *layer,
   image    = gimp_item_get_image (item);
 
   mask_name = g_strdup_printf (_("%s mask"),
-                               gimp_object_get_name (GIMP_OBJECT (layer)));
+                               gimp_object_get_name (layer));
 
   mask = gimp_layer_mask_new (image,
                               gimp_item_get_width  (item),
@@ -1443,7 +1510,7 @@ gimp_layer_create_mask (const GimpLayer *layer,
               guchar *alpha_ptr;
 
               gimp_drawable_push_undo (drawable,
-                                       _("Transfer Alpha to Mask"),
+                                       C_("undo-type", "Transfer Alpha to Mask"),
                                        0, 0,
                                        gimp_item_get_width  (item),
                                        gimp_item_get_height (item),
@@ -1529,25 +1596,20 @@ gimp_layer_create_mask (const GimpLayer *layer,
 
     case GIMP_ADD_COPY_MASK:
       {
-        TileManager   *copy_tiles = NULL;
-        GimpImageType  layer_type;
+        TileManager *copy_tiles = NULL;
 
-        layer_type = drawable->type;
-
-        if (GIMP_IMAGE_TYPE_BASE_TYPE (layer_type) != GIMP_GRAY)
+        if (! gimp_drawable_is_gray (drawable))
           {
             GimpImageType copy_type;
 
-            copy_type = (GIMP_IMAGE_TYPE_HAS_ALPHA (layer_type) ?
+            copy_type = (gimp_drawable_has_alpha (drawable) ?
                          GIMP_GRAYA_IMAGE : GIMP_GRAY_IMAGE);
 
             copy_tiles = tile_manager_new (gimp_item_get_width  (item),
                                            gimp_item_get_height (item),
                                            GIMP_IMAGE_TYPE_BYTES (copy_type));
 
-            gimp_drawable_convert_grayscale (drawable,
-                                             copy_tiles,
-                                             GIMP_IMAGE_TYPE_BASE_TYPE (layer_type));
+            gimp_drawable_convert_tiles_grayscale (drawable, copy_tiles);
 
             pixel_region_init (&srcPR, copy_tiles,
                                0, 0,
@@ -1618,8 +1680,8 @@ gimp_layer_apply_mask (GimpLayer         *layer,
     {
       gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_LAYER_APPLY_MASK,
                                    (mode == GIMP_MASK_APPLY) ?
-                                   _("Apply Layer Mask") :
-                                   _("Delete Layer Mask"));
+                                   C_("undo-type", "Apply Layer Mask") :
+                                   C_("undo-type", "Delete Layer Mask"));
 
       gimp_image_undo_push_layer_mask_remove (image, NULL, layer, mask);
 
@@ -1738,7 +1800,7 @@ gimp_layer_add_alpha (GimpLayer *layer)
   /*  Set the new tiles  */
   gimp_drawable_set_tiles (GIMP_DRAWABLE (layer),
                            gimp_item_is_attached (GIMP_ITEM (layer)),
-                           _("Add Alpha Channel"),
+                           C_("undo-type", "Add Alpha Channel"),
                            new_tiles, new_type);
   tile_manager_unref (new_tiles);
 }
@@ -1792,7 +1854,7 @@ gimp_layer_flatten (GimpLayer   *layer,
   /*  Set the new tiles  */
   gimp_drawable_set_tiles (GIMP_DRAWABLE (layer),
                            gimp_item_is_attached (GIMP_ITEM (layer)),
-                           _("Remove Alpha Channel"),
+                           C_("undo-type", "Remove Alpha Channel"),
                            new_tiles, new_type);
   tile_manager_unref (new_tiles);
 }
@@ -1812,7 +1874,7 @@ gimp_layer_resize_to_image (GimpLayer   *layer,
   image = gimp_item_get_image (GIMP_ITEM (layer));
 
   gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_ITEM_RESIZE,
-                               _("Layer to Image Size"));
+                               C_("undo-type", "Layer to Image Size"));
 
   gimp_item_get_offset (GIMP_ITEM (layer), &offset_x, &offset_y);
   gimp_item_resize (GIMP_ITEM (layer), context,
@@ -1836,31 +1898,34 @@ gimp_layer_boundary (GimpLayer *layer,
 
   item = GIMP_ITEM (layer);
 
-  /*  Create the four boundary segments that encompass this
-   *  layer's boundary.
-   */
-  new_segs  = g_new (BoundSeg, 4);
-  *num_segs = 4;
-
-  /*  if the layer is a floating selection  */
   if (gimp_layer_is_floating_sel (layer))
     {
-      if (GIMP_IS_CHANNEL (gimp_layer_get_floating_sel_drawable (layer)))
+      GimpDrawable *fs_drawable;
+
+      fs_drawable = gimp_layer_get_floating_sel_drawable (layer);
+
+      if (GIMP_IS_CHANNEL (fs_drawable))
         {
           /*  if the owner drawable is a channel, just return nothing  */
 
-          g_free (new_segs);
           *num_segs = 0;
+
           return NULL;
         }
       else
         {
           /*  otherwise, set the layer to the owner drawable  */
 
-          layer = GIMP_LAYER (gimp_layer_get_floating_sel_drawable (layer));
+          layer = GIMP_LAYER (fs_drawable);
         }
     }
 
+  /*  Create the four boundary segments that encompass this
+   *  layer's boundary.
+   */
+  new_segs  = g_new (BoundSeg, 4);
+  *num_segs = 4;
+
   gimp_item_get_offset (item, &offset_x, &offset_y);
 
   new_segs[0].x1   = offset_x;
@@ -1996,17 +2061,7 @@ gimp_layer_set_mode (GimpLayer            *layer,
       g_signal_emit (layer, layer_signals[MODE_CHANGED], 0);
       g_object_notify (G_OBJECT (layer), "mode");
 
-      if (layer->opacity_node)
-        {
-          GeglNode *mode_node;
-
-          mode_node = gimp_drawable_get_mode_node (GIMP_DRAWABLE (layer));
-
-          gegl_node_set (mode_node,
-                         "operation",  "gimp:point-layer-mode",
-                         "blend-mode", layer->mode,
-                         NULL);
-        }
+      gimp_layer_sync_mode_node (layer);
 
       gimp_drawable_update (GIMP_DRAWABLE (layer),
                             0, 0,
@@ -2029,6 +2084,7 @@ gimp_layer_set_lock_alpha (GimpLayer *layer,
                            gboolean   push_undo)
 {
   g_return_if_fail (GIMP_IS_LAYER (layer));
+  g_return_if_fail (gimp_layer_can_lock_alpha (layer));
 
   lock_alpha = lock_alpha ? TRUE : FALSE;
 
@@ -2055,3 +2111,14 @@ gimp_layer_get_lock_alpha (const GimpLayer *layer)
 
   return layer->lock_alpha;
 }
+
+gboolean
+gimp_layer_can_lock_alpha (const GimpLayer *layer)
+{
+  g_return_val_if_fail (GIMP_IS_LAYER (layer), FALSE);
+
+  if (gimp_viewable_get_children (GIMP_VIEWABLE (layer)))
+    return FALSE;
+
+  return TRUE;
+}
