@@ -96,6 +96,8 @@ GimpLayer     * gimp_layer_new_from_region     (PixelRegion          *region,
                                                 gdouble               opacity,
                                                 GimpLayerModeEffects  mode);
 
+GimpLayer     * gimp_layer_get_parent          (GimpLayer            *layer);
+
 GimpLayerMask * gimp_layer_get_mask            (const GimpLayer      *layer);
 GimpLayerMask * gimp_layer_create_mask         (const GimpLayer      *layer,
                                                 GimpAddMaskType       mask_type,
@@ -136,6 +138,7 @@ void            gimp_layer_set_lock_alpha      (GimpLayer            *layer,
                                                 gboolean              lock_alpha,
                                                 gboolean              push_undo);
 gboolean        gimp_layer_get_lock_alpha      (const GimpLayer      *layer);
+gboolean        gimp_layer_can_lock_alpha      (const GimpLayer      *layer);
 
 
 #endif /* __GIMP_LAYER_H__ */
