@@ -46,10 +46,18 @@ extern void gimp_marshal_BOOLEAN__STRING (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:BOOLEAN (./gimpmarshal.list:31) */
+/* BOOLEAN:STRING,FLAGS (./gimpmarshal.list:30) */
+extern void gimp_marshal_BOOLEAN__STRING_FLAGS (GClosure     *closure,
+                                                GValue       *return_value,
+                                                guint         n_param_values,
+                                                const GValue *param_values,
+                                                gpointer      invocation_hint,
+                                                gpointer      marshal_data);
+
+/* VOID:BOOLEAN (./gimpmarshal.list:32) */
 #define gimp_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN
 
-/* VOID:BOOLEAN,INT,INT,INT,INT (./gimpmarshal.list:32) */
+/* VOID:BOOLEAN,INT,INT,INT,INT (./gimpmarshal.list:33) */
 extern void gimp_marshal_VOID__BOOLEAN_INT_INT_INT_INT (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -57,10 +65,10 @@ extern void gimp_marshal_VOID__BOOLEAN_INT_INT_INT_INT (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:BOXED (./gimpmarshal.list:33) */
+/* VOID:BOXED (./gimpmarshal.list:34) */
 #define gimp_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
 
-/* VOID:BOXED,ENUM (./gimpmarshal.list:34) */
+/* VOID:BOXED,ENUM (./gimpmarshal.list:35) */
 extern void gimp_marshal_VOID__BOXED_ENUM (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -68,10 +76,10 @@ extern void gimp_marshal_VOID__BOXED_ENUM (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:DOUBLE (./gimpmarshal.list:35) */
+/* VOID:DOUBLE (./gimpmarshal.list:36) */
 #define gimp_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
 
-/* VOID:DOUBLE,DOUBLE (./gimpmarshal.list:36) */
+/* VOID:DOUBLE,DOUBLE (./gimpmarshal.list:37) */
 extern void gimp_marshal_VOID__DOUBLE_DOUBLE (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -79,7 +87,7 @@ extern void gimp_marshal_VOID__DOUBLE_DOUBLE (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:DOUBLE,DOUBLE,DOUBLE,DOUBLE (./gimpmarshal.list:37) */
+/* VOID:DOUBLE,DOUBLE,DOUBLE,DOUBLE (./gimpmarshal.list:38) */
 extern void gimp_marshal_VOID__DOUBLE_DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
@@ -87,10 +95,10 @@ extern void gimp_marshal_VOID__DOUBLE_DOUBLE_DOUBLE_DOUBLE (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
-/* VOID:ENUM (./gimpmarshal.list:38) */
+/* VOID:ENUM (./gimpmarshal.list:39) */
 #define gimp_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM
 
-/* VOID:ENUM,ENUM,BOXED,INT (./gimpmarshal.list:39) */
+/* VOID:ENUM,ENUM,BOXED,INT (./gimpmarshal.list:40) */
 extern void gimp_marshal_VOID__ENUM_ENUM_BOXED_INT (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -98,7 +106,7 @@ extern void gimp_marshal_VOID__ENUM_ENUM_BOXED_INT (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:ENUM,INT (./gimpmarshal.list:40) */
+/* VOID:ENUM,INT (./gimpmarshal.list:41) */
 extern void gimp_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -106,7 +114,7 @@ extern void gimp_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (./gimpmarshal.list:41) */
+/* VOID:ENUM,INT,BOOLEAN (./gimpmarshal.list:42) */
 extern void gimp_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -114,7 +122,7 @@ extern void gimp_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:ENUM,OBJECT (./gimpmarshal.list:42) */
+/* VOID:ENUM,OBJECT (./gimpmarshal.list:43) */
 extern void gimp_marshal_VOID__ENUM_OBJECT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -122,7 +130,7 @@ extern void gimp_marshal_VOID__ENUM_OBJECT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:ENUM,POINTER (./gimpmarshal.list:43) */
+/* VOID:ENUM,POINTER (./gimpmarshal.list:44) */
 extern void gimp_marshal_VOID__ENUM_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -130,13 +138,13 @@ extern void gimp_marshal_VOID__ENUM_POINTER (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:FLAGS (./gimpmarshal.list:44) */
+/* VOID:FLAGS (./gimpmarshal.list:45) */
 #define gimp_marshal_VOID__FLAGS	g_cclosure_marshal_VOID__FLAGS
 
-/* VOID:INT (./gimpmarshal.list:45) */
+/* VOID:INT (./gimpmarshal.list:46) */
 #define gimp_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 
-/* VOID:INT,INT (./gimpmarshal.list:46) */
+/* VOID:INT,INT (./gimpmarshal.list:47) */
 extern void gimp_marshal_VOID__INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -144,7 +152,7 @@ extern void gimp_marshal_VOID__INT_INT (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* VOID:INT,INT,INT,INT (./gimpmarshal.list:47) */
+/* VOID:INT,INT,INT,INT (./gimpmarshal.list:48) */
 extern void gimp_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -152,10 +160,10 @@ extern void gimp_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:OBJECT (./gimpmarshal.list:48) */
+/* VOID:OBJECT (./gimpmarshal.list:49) */
 #define gimp_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (./gimpmarshal.list:49) */
+/* VOID:OBJECT,BOOLEAN (./gimpmarshal.list:50) */
 extern void gimp_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -163,7 +171,7 @@ extern void gimp_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,INT (./gimpmarshal.list:50) */
+/* VOID:OBJECT,INT (./gimpmarshal.list:51) */
 extern void gimp_marshal_VOID__OBJECT_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -171,7 +179,7 @@ extern void gimp_marshal_VOID__OBJECT_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (./gimpmarshal.list:51) */
+/* VOID:OBJECT,OBJECT (./gimpmarshal.list:52) */
 extern void gimp_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -179,7 +187,7 @@ extern void gimp_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,POINTER (./gimpmarshal.list:52) */
+/* VOID:OBJECT,POINTER (./gimpmarshal.list:53) */
 extern void gimp_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -187,10 +195,10 @@ extern void gimp_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:POINTER (./gimpmarshal.list:53) */
+/* VOID:POINTER (./gimpmarshal.list:54) */
 #define gimp_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:POINTER,BOXED (./gimpmarshal.list:54) */
+/* VOID:POINTER,BOXED (./gimpmarshal.list:55) */
 extern void gimp_marshal_VOID__POINTER_BOXED (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -198,7 +206,7 @@ extern void gimp_marshal_VOID__POINTER_BOXED (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:POINTER,ENUM (./gimpmarshal.list:55) */
+/* VOID:POINTER,ENUM (./gimpmarshal.list:56) */
 extern void gimp_marshal_VOID__POINTER_ENUM (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -206,10 +214,10 @@ extern void gimp_marshal_VOID__POINTER_ENUM (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING (./gimpmarshal.list:56) */
+/* VOID:STRING (./gimpmarshal.list:57) */
 #define gimp_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,BOOLEAN,UINT,FLAGS (./gimpmarshal.list:57) */
+/* VOID:STRING,BOOLEAN,UINT,FLAGS (./gimpmarshal.list:58) */
 extern void gimp_marshal_VOID__STRING_BOOLEAN_UINT_FLAGS (GClosure     *closure,
                                                           GValue       *return_value,
                                                           guint         n_param_values,
@@ -217,7 +225,7 @@ extern void gimp_marshal_VOID__STRING_BOOLEAN_UINT_FLAGS (GClosure     *closure,
                                                           gpointer      invocation_hint,
                                                           gpointer      marshal_data);
 
-/* VOID:STRING,FLAGS (./gimpmarshal.list:58) */
+/* VOID:STRING,FLAGS (./gimpmarshal.list:59) */
 extern void gimp_marshal_VOID__STRING_FLAGS (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -225,7 +233,7 @@ extern void gimp_marshal_VOID__STRING_FLAGS (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./gimpmarshal.list:59) */
+/* VOID:STRING,STRING,STRING (./gimpmarshal.list:60) */
 extern void gimp_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -233,7 +241,7 @@ extern void gimp_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* VOID:VOID (./gimpmarshal.list:60) */
+/* VOID:VOID (./gimpmarshal.list:61) */
 #define gimp_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 
 G_END_DECLS
