@@ -267,12 +267,13 @@ gimp_palette_get_description (GimpViewable  *viewable,
   GimpPalette *palette = GIMP_PALETTE (viewable);
 
   return g_strdup_printf ("%s (%d)",
-                          GIMP_OBJECT (palette)->name,
+                          gimp_object_get_name (palette),
                           palette->n_colors);
 }
 
 GimpData *
-gimp_palette_new (const gchar *name)
+gimp_palette_new (GimpContext *context,
+                  const gchar *name)
 {
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail (*name != '\0', NULL);
@@ -283,19 +284,19 @@ gimp_palette_new (const gchar *name)
 }
 
 GimpData *
-gimp_palette_get_standard (void)
+gimp_palette_get_standard (GimpContext *context)
 {
   static GimpData *standard_palette = NULL;
 
   if (! standard_palette)
     {
-      standard_palette = gimp_palette_new ("Standard");
+      standard_palette = gimp_palette_new (context, "Standard");
 
-      standard_palette->dirty = FALSE;
-      gimp_data_make_internal (standard_palette,
-                               "gimp-palette-standard");
+      gimp_data_clean (standard_palette);
+      gimp_data_make_internal (standard_palette, "gimp-palette-standard");
 
-      g_object_ref (standard_palette);
+      g_object_add_weak_pointer (G_OBJECT (standard_palette),
+                                 (gpointer *) &standard_palette);
     }
 
   return standard_palette;
