@@ -461,6 +461,8 @@ gimp_stroke_options_prepare (GimpStrokeOptions *options,
 
             if (config->global_brush)
               global_props |= GIMP_CONTEXT_BRUSH_MASK;
+            if (config->global_dynamics)
+              global_props |= GIMP_CONTEXT_DYNAMICS_MASK;
             if (config->global_pattern)
               global_props |= GIMP_CONTEXT_PATTERN_MASK;
             if (config->global_palette)
