@@ -44,7 +44,9 @@
 #include "gimp-intl.h"
 
 
-#define GIMP_TAG_CACHE_FILE "tags.xml"
+#define GIMP_TAG_CACHE_FILE  "tags.xml"
+
+/* #define DEBUG_GIMP_TAG_CACHE  1 */
 
 
 enum
@@ -137,7 +139,8 @@ gimp_tag_cache_init (GimpTagCache *cache)
                                              GIMP_TYPE_TAG_CACHE,
                                              GimpTagCachePriv);
 
-  cache->priv->records    = g_array_new (FALSE, FALSE, sizeof(GimpTagCacheRecord));
+  cache->priv->records    = g_array_new (FALSE, FALSE,
+                                         sizeof (GimpTagCacheRecord));
   cache->priv->containers = NULL;
 }
 
@@ -151,8 +154,8 @@ gimp_tag_cache_finalize (GObject *object)
     {
       for (i = 0; i < cache->priv->records->len; i++)
         {
-          GimpTagCacheRecord *rec =
-            &g_array_index (cache->priv->records, GimpTagCacheRecord, i);
+          GimpTagCacheRecord *rec = &g_array_index (cache->priv->records,
+                                                    GimpTagCacheRecord, i);
 
           g_list_foreach (rec->tags, (GFunc) g_object_unref, NULL);
           g_list_free (rec->tags);
@@ -235,17 +238,22 @@ gimp_tag_cache_add_object (GimpTagCache *cache,
 {
   gchar  *identifier;
   GQuark  identifier_quark = 0;
-  gchar  *checksum_string;
-  GQuark  checksum_quark;
-  GList  *list;
-  gint    i;
 
   identifier = gimp_tagged_get_identifier (tagged);
-  identifier_quark = g_quark_try_string (identifier);
-  g_free (identifier);
+
+  if (identifier)
+    {
+      identifier_quark = g_quark_try_string (identifier);
+      g_free (identifier);
+    }
 
   if (! gimp_tagged_get_tags (tagged))
     {
+      gchar  *checksum;
+      GQuark  checksum_quark = 0;
+      GList  *list;
+      gint    i;
+
       if (identifier_quark)
         {
           for (i = 0; i < cache->priv->records->len; i++)
@@ -266,9 +274,13 @@ gimp_tag_cache_add_object (GimpTagCache *cache,
             }
         }
 
-      checksum_string = gimp_tagged_get_checksum (tagged);
-      checksum_quark = g_quark_try_string (checksum_string);
-      g_free (checksum_string);
+      checksum = gimp_tagged_get_checksum (tagged);
+
+      if (checksum)
+        {
+          checksum_quark = g_quark_try_string (checksum);
+          g_free (checksum);
+        }
 
       if (checksum_quark)
         {
@@ -279,9 +291,12 @@ gimp_tag_cache_add_object (GimpTagCache *cache,
 
               if (rec->checksum == checksum_quark)
                 {
+#if DEBUG_GIMP_TAG_CACHE
                   g_printerr ("remapping identifier: %s ==> %s\n",
-                              g_quark_to_string (rec->identifier),
-                              g_quark_to_string (identifier_quark));
+                              rec->identifier ? g_quark_to_string (rec->identifier) : "(NULL)",
+                              identifier_quark ? g_quark_to_string (identifier_quark) : "(NULL)");
+#endif
+
                   rec->identifier = identifier_quark;
 
                   for (list = rec->tags; list; list = g_list_next (list))
@@ -308,9 +323,7 @@ static void
 gimp_tag_cache_tagged_to_cache_record_foreach (GimpTagged  *tagged,
                                                GList      **cache_records)
 {
-  gchar *identifier;
-
-  identifier = gimp_tagged_get_identifier (tagged);
+  gchar *identifier = gimp_tagged_get_identifier (tagged);
 
   if (identifier)
     {
@@ -321,7 +334,7 @@ gimp_tag_cache_tagged_to_cache_record_foreach (GimpTagged  *tagged,
 
       cache_rec->identifier = g_quark_from_string (identifier);
       cache_rec->checksum   = g_quark_from_string (checksum);
-      cache_rec->tags = g_list_copy (gimp_tagged_get_tags (tagged));
+      cache_rec->tags       = g_list_copy (gimp_tagged_get_tags (tagged));
 
       g_free (checksum);
 
@@ -352,7 +365,8 @@ gimp_tag_cache_save (GimpTagCache *cache)
   saved_records = NULL;
   for (i = 0; i < cache->priv->records->len; i++)
     {
-      GimpTagCacheRecord *current_record = &g_array_index (cache->priv->records, GimpTagCacheRecord, i);
+      GimpTagCacheRecord *current_record = &g_array_index (cache->priv->records,
+                                                           GimpTagCacheRecord, i);
 
       if (! current_record->referenced && current_record->tags)
         {
@@ -369,7 +383,8 @@ gimp_tag_cache_save (GimpTagCache *cache)
         }
     }
 
-  for (iterator = cache->priv->containers; iterator;
+  for (iterator = cache->priv->containers;
+       iterator;
        iterator = g_list_next (iterator))
     {
       gimp_container_foreach (GIMP_CONTAINER (iterator->data),
@@ -380,6 +395,7 @@ gimp_tag_cache_save (GimpTagCache *cache)
   buf = g_string_new ("");
   g_string_append (buf, "<?xml version='1.0' encoding='UTF-8'?>\n");
   g_string_append (buf, "<tags>\n");
+
   for (iterator = saved_records; iterator; iterator = g_list_next (iterator))
     {
       GimpTagCacheRecord *cache_rec = iterator->data;
@@ -393,7 +409,8 @@ gimp_tag_cache_save (GimpTagCache *cache)
                               g_quark_to_string (cache_rec->checksum));
       g_free (identifier_string);
 
-      for (tag_iterator = cache_rec->tags; tag_iterator;
+      for (tag_iterator = cache_rec->tags;
+           tag_iterator;
            tag_iterator = g_list_next (tag_iterator))
         {
           tag_string = g_markup_escape_text (gimp_tag_get_name (GIMP_TAG (tag_iterator->data)), -1);
@@ -407,6 +424,7 @@ gimp_tag_cache_save (GimpTagCache *cache)
   g_string_append (buf, "</tags>\n");
 
   filename = g_build_filename (gimp_directory (), GIMP_TAG_CACHE_FILE, NULL);
+
   if (! g_file_set_contents (filename, buf->str, buf->len, &error))
     {
       g_printerr ("Error while saving tag cache: %s\n", error->message);
@@ -414,10 +432,12 @@ gimp_tag_cache_save (GimpTagCache *cache)
     }
 
   g_free (filename);
+
   g_string_free (buf, TRUE);
 
-  for (iterator = saved_records; iterator;
-      iterator = g_list_next (iterator))
+  for (iterator = saved_records;
+       iterator;
+       iterator = g_list_next (iterator))
     {
       GimpTagCacheRecord *cache_rec = iterator->data;
 
@@ -460,16 +480,18 @@ gimp_tag_cache_load (GimpTagCache *cache)
   markup_parser.error         = gimp_tag_cache_load_error;
 
   xml_parser = gimp_xml_parser_new (&markup_parser, &parse_data);
-  if (! gimp_xml_parser_parse_file (xml_parser, filename, &error))
-    {
-      g_printerr ("Failed to parse tag cache.\n");
-    }
-  else
+
+  if (gimp_xml_parser_parse_file (xml_parser, filename, &error))
     {
       cache->priv->records = g_array_append_vals (cache->priv->records,
                                                   parse_data.records->data,
                                                   parse_data.records->len);
     }
+  else
+    {
+      g_printerr ("Failed to parse tag cache: %s\n",
+                  error ? error->message : NULL);
+    }
 
   g_free (filename);
   gimp_xml_parser_free (xml_parser);
@@ -544,7 +566,7 @@ gimp_tag_cache_load_text (GMarkupParseContext  *context,
 
   current_element = g_markup_parse_context_get_element (context);
 
-  if (current_element && strcmp (current_element, "tag") == 0)
+  if (g_strcmp0 (current_element, "tag") == 0)
     {
       if (text_len >= sizeof (buffer))
         {
