@@ -213,8 +213,7 @@ gimp_tool_presets_new (GimpToolInfo *tool_info)
                           "policy",        GIMP_CONTAINER_POLICY_STRONG,
                           NULL);
 
-  name = g_strdup_printf ("%s options",
-                          gimp_object_get_name (GIMP_OBJECT (tool_info)));
+  name = g_strdup_printf ("%s options", gimp_object_get_name (tool_info));
 
   gimp_object_take_name (GIMP_OBJECT (presets), name);
 
@@ -256,10 +255,10 @@ gimp_tool_presets_save (GimpToolPresets  *presets,
 
       gimp_tool_options_create_folder ();
 
-      footer = g_strdup_printf ("end of %s", GIMP_OBJECT (presets)->name);
+      footer = g_strdup_printf ("end of %s", gimp_object_get_name (presets));
 
       retval = gimp_config_serialize_to_file (GIMP_CONFIG (presets), filename,
-                                              GIMP_OBJECT (presets)->name,
+                                              gimp_object_get_name (presets),
                                               footer,
                                               NULL, error);
 
