@@ -37,19 +37,16 @@
 #include "widgets/gimpchanneltreeview.h"
 #include "widgets/gimpcoloreditor.h"
 #include "widgets/gimpcolormapeditor.h"
-#include "widgets/gimpcontainergridview.h"
-#include "widgets/gimpcontainertreeview.h"
-#include "widgets/gimpcontainerview-utils.h"
-#include "widgets/gimpdataeditor.h"
 #include "widgets/gimpdevicestatus.h"
 #include "widgets/gimpdialogfactory.h"
-#include "widgets/gimpdockable.h"
+#include "widgets/gimpdockwindow.h"
 #include "widgets/gimpdocumentview.h"
+#include "widgets/gimpdynamicseditor.h"
+#include "widgets/gimpdynamicsfactoryview.h"
 #include "widgets/gimperrorconsole.h"
 #include "widgets/gimperrordialog.h"
 #include "widgets/gimpfontview.h"
 #include "widgets/gimpgradienteditor.h"
-#include "widgets/gimphelp-ids.h"
 #include "widgets/gimphistogrameditor.h"
 #include "widgets/gimpimageview.h"
 #include "widgets/gimplayertreeview.h"
@@ -61,6 +58,8 @@
 #include "widgets/gimptemplateview.h"
 #include "widgets/gimptoolbox.h"
 #include "widgets/gimptooloptionseditor.h"
+#include "widgets/gimptoolpresetfactoryview.h"
+#include "widgets/gimptoolpreseteditor.h"
 #include "widgets/gimpundoeditor.h"
 #include "widgets/gimpvectorstreeview.h"
 
@@ -74,6 +73,7 @@
 #include "file-open-location-dialog.h"
 #include "file-save-dialog.h"
 #include "image-new-dialog.h"
+#include "input-devices-dialog.h"
 #include "keyboard-shortcuts-dialog.h"
 #include "module-dialog.h"
 #include "palette-import-dialog.h"
@@ -98,6 +98,7 @@ static void   dialogs_indexed_palette_selected (GimpColormapEditor *editor,
 GtkWidget *
 dialogs_image_new_new (GimpDialogFactory *factory,
                        GimpContext       *context,
+                       GimpUIManager     *ui_manager,
                        gint               view_size)
 {
   return image_new_dialog_new (context);
@@ -106,6 +107,7 @@ dialogs_image_new_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_file_open_new (GimpDialogFactory *factory,
                        GimpContext       *context,
+                       GimpUIManager     *ui_manager,
                        gint               view_size)
 {
   return file_open_dialog_new (context->gimp);
@@ -114,6 +116,7 @@ dialogs_file_open_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_file_open_location_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return file_open_location_dialog_new (context->gimp);
@@ -122,6 +125,7 @@ dialogs_file_open_location_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_file_save_new (GimpDialogFactory *factory,
                        GimpContext       *context,
+                       GimpUIManager     *ui_manager,
                        gint               view_size)
 {
   return file_save_dialog_new (context->gimp, FALSE);
@@ -130,6 +134,7 @@ dialogs_file_save_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_file_export_new (GimpDialogFactory *factory,
                          GimpContext       *context,
+                         GimpUIManager     *ui_manager,
                          gint               view_size)
 {
   return file_save_dialog_new (context->gimp, TRUE);
@@ -138,6 +143,7 @@ dialogs_file_export_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_preferences_get (GimpDialogFactory *factory,
                          GimpContext       *context,
+                         GimpUIManager     *ui_manager,
                          gint               view_size)
 {
   return preferences_dialog_create (context->gimp);
@@ -146,14 +152,25 @@ dialogs_preferences_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_keyboard_shortcuts_get (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return keyboard_shortcuts_dialog_new (context->gimp);
 }
 
+GtkWidget *
+dialogs_input_devices_get (GimpDialogFactory *factory,
+                           GimpContext       *context,
+                           GimpUIManager     *ui_manager,
+                           gint               view_size)
+{
+  return input_devices_dialog_new (context->gimp);
+}
+
 GtkWidget *
 dialogs_module_get (GimpDialogFactory *factory,
                     GimpContext       *context,
+                    GimpUIManager     *ui_manager,
                     gint               view_size)
 {
   return module_dialog_new (context->gimp);
@@ -162,6 +179,7 @@ dialogs_module_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_palette_import_get (GimpDialogFactory *factory,
                             GimpContext       *context,
+                            GimpUIManager     *ui_manager,
                             gint               view_size)
 {
   return palette_import_dialog_new (context);
@@ -170,6 +188,7 @@ dialogs_palette_import_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_tips_get (GimpDialogFactory *factory,
                   GimpContext       *context,
+                  GimpUIManager     *ui_manager,
                   gint               view_size)
 {
   return tips_dialog_create (context->gimp);
@@ -178,6 +197,7 @@ dialogs_tips_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_about_get (GimpDialogFactory *factory,
                    GimpContext       *context,
+                   GimpUIManager     *ui_manager,
                    gint               view_size)
 {
   return about_dialog_create (context);
@@ -186,6 +206,7 @@ dialogs_about_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_error_get (GimpDialogFactory *factory,
                    GimpContext       *context,
+                   GimpUIManager     *ui_manager,
                    gint               view_size)
 {
   return gimp_error_dialog_new (_("GIMP Message"));
@@ -194,6 +215,7 @@ dialogs_error_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_close_all_get (GimpDialogFactory *factory,
                        GimpContext       *context,
+                       GimpUIManager     *ui_manager,
                        gint               view_size)
 {
   return close_all_dialog_new (context->gimp);
@@ -202,6 +224,7 @@ dialogs_close_all_get (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_quit_get (GimpDialogFactory *factory,
                   GimpContext       *context,
+                  GimpUIManager     *ui_manager,
                   gint               view_size)
 {
   return quit_dialog_new (context->gimp);
@@ -213,25 +236,49 @@ dialogs_quit_get (GimpDialogFactory *factory,
 /***********/
 
 GtkWidget *
-dialogs_toolbox_get (GimpDialogFactory *factory,
+dialogs_toolbox_new (GimpDialogFactory *factory,
                      GimpContext       *context,
+                     GimpUIManager     *ui_manager,
                      gint               view_size)
 {
-  /*  we pass "global_dock_factory", _not_ "global_toolbox_factory" to
-   *  the toolbox constructor, because the global_toolbox_factory has no
-   *  dockables registered
-   */
-  return gimp_toolbox_new (global_dock_factory, context);
+  return gimp_toolbox_new (factory,
+                           context,
+                           ui_manager);
+}
+
+GtkWidget *
+dialogs_toolbox_dock_window_new (GimpDialogFactory *factory,
+                                 GimpContext       *context,
+                                 GimpUIManager     *ui_manager,
+                                 gint               view_size)
+{
+  return gimp_dock_window_new ("gimp-toolbox",
+                               "<Toolbox>",
+                               TRUE,
+                               factory,
+                               context);
 }
 
 GtkWidget *
 dialogs_dock_new (GimpDialogFactory *factory,
                   GimpContext       *context,
+                  GimpUIManager     *ui_manager,
                   gint               view_size)
 {
-  return gimp_menu_dock_new (factory,
-                             context->gimp->images,
-                             context->gimp->displays);
+  return gimp_menu_dock_new ();
+}
+
+GtkWidget *
+dialogs_dock_window_new (GimpDialogFactory *factory,
+                         GimpContext       *context,
+                         GimpUIManager     *ui_manager,
+                         gint               view_size)
+{
+  return gimp_dock_window_new ("gimp-dock",
+                               "<Dock>",
+                               FALSE,
+                               factory,
+                               context);
 }
 
 
@@ -239,49 +286,22 @@ dialogs_dock_new (GimpDialogFactory *factory,
 /*  dockables  */
 /***************/
 
-/*****  the dockable constructor  *****/
-
-GtkWidget *
-dialogs_dockable_constructor (GimpDialogFactory      *factory,
-                              GimpDialogFactoryEntry *entry,
-                              GimpContext            *context,
-                              gint                    view_size)
-{
-  GtkWidget *dockable = NULL;
-  GtkWidget *widget;
-
-  widget = entry->new_func (factory, context, view_size);
-
-  if (widget)
-    {
-      dockable = gimp_dockable_new (entry->name, entry->blurb,
-                                    entry->stock_id, entry->help_id);
-      gtk_container_add (GTK_CONTAINER (dockable), widget);
-      gtk_widget_show (widget);
-
-      /* EEK */
-      g_object_set_data (G_OBJECT (dockable), "gimp-dialog-identifier",
-                         entry->identifier);
-    }
-
-  return dockable;
-}
-
-
 /*****  singleton dialogs  *****/
 
 GtkWidget *
 dialogs_tool_options_new (GimpDialogFactory *factory,
                           GimpContext       *context,
+                          GimpUIManager     *ui_manager,
                           gint               view_size)
 {
   return gimp_tool_options_editor_new (context->gimp,
-                                       factory->menu_factory);
+                                       gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_device_status_new (GimpDialogFactory *factory,
                            GimpContext       *context,
+                           GimpUIManager     *ui_manager,
                            gint               view_size)
 {
   return gimp_device_status_new (context->gimp);
@@ -290,18 +310,20 @@ dialogs_device_status_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_error_console_new (GimpDialogFactory *factory,
                            GimpContext       *context,
+                           GimpUIManager     *ui_manager,
                            gint               view_size)
 {
   return gimp_error_console_new (context->gimp,
-                                 factory->menu_factory);
+                                 gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_cursor_view_new (GimpDialogFactory *factory,
                          GimpContext       *context,
+                         GimpUIManager     *ui_manager,
                          gint               view_size)
 {
-  return gimp_cursor_view_new (factory->menu_factory);
+  return gimp_cursor_view_new (gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
@@ -310,18 +332,20 @@ dialogs_cursor_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_image_list_view_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   return gimp_image_view_new (GIMP_VIEW_TYPE_LIST,
                               context->gimp->images,
                               context,
                               view_size, 1,
-                              factory->menu_factory);
+                              gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_brush_list_view_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   return gimp_brush_factory_view_new (GIMP_VIEW_TYPE_LIST,
@@ -329,31 +353,46 @@ dialogs_brush_list_view_new (GimpDialogFactory *factory,
                                       context,
                                       TRUE,
                                       view_size, 1,
-                                      factory->menu_factory);
+                                      gimp_dialog_factory_get_menu_factory (factory));
+}
+
+GtkWidget *
+dialogs_dynamics_list_view_new (GimpDialogFactory *factory,
+                                GimpContext       *context,
+                                GimpUIManager     *ui_manager,
+                                gint               view_size)
+{
+  return gimp_dynamics_factory_view_new (GIMP_VIEW_TYPE_LIST,
+                                         context->gimp->dynamics_factory,
+                                         context,
+                                         view_size, 1,
+                                         gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_pattern_list_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   return gimp_pattern_factory_view_new (GIMP_VIEW_TYPE_LIST,
                                         context->gimp->pattern_factory,
                                         context,
                                         view_size, 1,
-                                        factory->menu_factory);
+                                        gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_gradient_list_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_data_factory_view_new (GIMP_VIEW_TYPE_LIST,
                                      context->gimp->gradient_factory,
                                      context,
                                      view_size, 1,
-                                     factory->menu_factory, "<Gradients>",
+                                     gimp_dialog_factory_get_menu_factory (factory), "<Gradients>",
                                      "/gradients-popup",
                                      "gradients");
 }
@@ -361,13 +400,14 @@ dialogs_gradient_list_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_palette_list_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   return gimp_data_factory_view_new (GIMP_VIEW_TYPE_LIST,
                                      context->gimp->palette_factory,
                                      context,
                                      view_size, 1,
-                                     factory->menu_factory, "<Palettes>",
+                                     gimp_dialog_factory_get_menu_factory (factory), "<Palettes>",
                                      "/palettes-popup",
                                      "palettes");
 }
@@ -375,49 +415,66 @@ dialogs_palette_list_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_font_list_view_new (GimpDialogFactory *factory,
                             GimpContext       *context,
+                            GimpUIManager     *ui_manager,
                             gint               view_size)
 {
   return gimp_font_view_new (GIMP_VIEW_TYPE_LIST,
                              context->gimp->fonts,
                              context,
                              view_size, 1,
-                             factory->menu_factory);
+                             gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_buffer_list_view_new (GimpDialogFactory *factory,
                               GimpContext       *context,
+                              GimpUIManager     *ui_manager,
                               gint               view_size)
 {
   return gimp_buffer_view_new (GIMP_VIEW_TYPE_LIST,
                                context->gimp->named_buffers,
                                context,
                                view_size, 1,
-                               factory->menu_factory);
+                               gimp_dialog_factory_get_menu_factory (factory));
+}
+
+GtkWidget *
+dialogs_tool_preset_list_view_new (GimpDialogFactory *factory,
+                                GimpContext       *context,
+                                GimpUIManager     *ui_manager,
+                                gint               view_size)
+{
+  return gimp_tool_preset_factory_view_new (GIMP_VIEW_TYPE_LIST,
+                                            context->gimp->tool_preset_factory,
+                                            context,
+                                            view_size, 1,
+                                            gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_document_list_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_document_view_new (GIMP_VIEW_TYPE_LIST,
                                  context->gimp->documents,
                                  context,
                                  view_size, 0,
-                                 factory->menu_factory);
+                                 gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_template_list_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_template_view_new (GIMP_VIEW_TYPE_LIST,
                                  context->gimp->templates,
                                  context,
                                  view_size, 0,
-                                 factory->menu_factory);
+                                 gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
@@ -426,18 +483,20 @@ dialogs_template_list_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_image_grid_view_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   return gimp_image_view_new (GIMP_VIEW_TYPE_GRID,
                               context->gimp->images,
                               context,
                               view_size, 1,
-                              factory->menu_factory);
+                              gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_brush_grid_view_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   return gimp_brush_factory_view_new (GIMP_VIEW_TYPE_GRID,
@@ -445,31 +504,33 @@ dialogs_brush_grid_view_new (GimpDialogFactory *factory,
                                       context,
                                       TRUE,
                                       view_size, 1,
-                                      factory->menu_factory);
+                                      gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_pattern_grid_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   return gimp_pattern_factory_view_new (GIMP_VIEW_TYPE_GRID,
                                         context->gimp->pattern_factory,
                                         context,
                                         view_size, 1,
-                                        factory->menu_factory);
+                                        gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_gradient_grid_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_data_factory_view_new (GIMP_VIEW_TYPE_GRID,
                                      context->gimp->gradient_factory,
                                      context,
                                      view_size, 1,
-                                     factory->menu_factory, "<Gradients>",
+                                     gimp_dialog_factory_get_menu_factory (factory), "<Gradients>",
                                      "/gradients-popup",
                                      "gradients");
 }
@@ -477,13 +538,14 @@ dialogs_gradient_grid_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_palette_grid_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   return gimp_data_factory_view_new (GIMP_VIEW_TYPE_GRID,
                                      context->gimp->palette_factory,
                                      context,
                                      view_size, 1,
-                                     factory->menu_factory, "<Palettes>",
+                                     gimp_dialog_factory_get_menu_factory (factory), "<Palettes>",
                                      "/palettes-popup",
                                      "palettes");
 }
@@ -491,49 +553,53 @@ dialogs_palette_grid_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_font_grid_view_new (GimpDialogFactory *factory,
                             GimpContext       *context,
+                            GimpUIManager     *ui_manager,
                             gint               view_size)
 {
   return gimp_font_view_new (GIMP_VIEW_TYPE_GRID,
                              context->gimp->fonts,
                              context,
                              view_size, 1,
-                             factory->menu_factory);
+                             gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_buffer_grid_view_new (GimpDialogFactory *factory,
                               GimpContext       *context,
+                              GimpUIManager     *ui_manager,
                               gint               view_size)
 {
   return gimp_buffer_view_new (GIMP_VIEW_TYPE_GRID,
                                context->gimp->named_buffers,
                                context,
                                view_size, 1,
-                               factory->menu_factory);
+                               gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_document_grid_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_document_view_new (GIMP_VIEW_TYPE_GRID,
                                  context->gimp->documents,
                                  context,
                                  view_size, 0,
-                                 factory->menu_factory);
+                                 gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_template_grid_view_new (GimpDialogFactory *factory,
                                 GimpContext       *context,
+                                GimpUIManager     *ui_manager,
                                 gint               view_size)
 {
   return gimp_template_view_new (GIMP_VIEW_TYPE_GRID,
                                  context->gimp->templates,
                                  context,
                                  view_size, 0,
-                                 factory->menu_factory);
+                                 gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
@@ -542,6 +608,7 @@ dialogs_template_grid_view_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_layer_list_view_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   if (view_size < 1)
@@ -550,13 +617,14 @@ dialogs_layer_list_view_new (GimpDialogFactory *factory,
   return gimp_item_tree_view_new (GIMP_TYPE_LAYER_TREE_VIEW,
                                   view_size, 2,
                                   gimp_context_get_image (context),
-                                  factory->menu_factory, "<Layers>",
+                                  gimp_dialog_factory_get_menu_factory (factory), "<Layers>",
                                   "/layers-popup");
 }
 
 GtkWidget *
 dialogs_channel_list_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   if (view_size < 1)
@@ -565,13 +633,14 @@ dialogs_channel_list_view_new (GimpDialogFactory *factory,
   return gimp_item_tree_view_new (GIMP_TYPE_CHANNEL_TREE_VIEW,
                                   view_size, 1,
                                   gimp_context_get_image (context),
-                                  factory->menu_factory, "<Channels>",
+                                  gimp_dialog_factory_get_menu_factory (factory), "<Channels>",
                                   "/channels-popup");
 }
 
 GtkWidget *
 dialogs_vectors_list_view_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
   if (view_size < 1)
@@ -580,18 +649,19 @@ dialogs_vectors_list_view_new (GimpDialogFactory *factory,
   return gimp_item_tree_view_new (GIMP_TYPE_VECTORS_TREE_VIEW,
                                   view_size, 1,
                                   gimp_context_get_image (context),
-                                  factory->menu_factory, "<Vectors>",
+                                  gimp_dialog_factory_get_menu_factory (factory), "<Vectors>",
                                   "/vectors-popup");
 }
 
 GtkWidget *
 dialogs_colormap_editor_new (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   GtkWidget *view;
 
-  view = gimp_colormap_editor_new (factory->menu_factory);
+  view = gimp_colormap_editor_new (gimp_dialog_factory_get_menu_factory (factory));
 
   g_signal_connect (view, "selected",
                     G_CALLBACK (dialogs_indexed_palette_selected),
@@ -603,6 +673,7 @@ dialogs_colormap_editor_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_histogram_editor_new (GimpDialogFactory *factory,
                               GimpContext       *context,
+                              GimpUIManager     *ui_manager,
                               gint               view_size)
 {
   return gimp_histogram_editor_new ();
@@ -611,26 +682,29 @@ dialogs_histogram_editor_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_selection_editor_new (GimpDialogFactory *factory,
                               GimpContext       *context,
+                              GimpUIManager     *ui_manager,
                               gint               view_size)
 {
-  return gimp_selection_editor_new (factory->menu_factory);
+  return gimp_selection_editor_new (gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_undo_editor_new (GimpDialogFactory *factory,
                          GimpContext       *context,
+                         GimpUIManager     *ui_manager,
                          gint               view_size)
 {
   return gimp_undo_editor_new (context->gimp->config,
-                               factory->menu_factory);
+                               gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_sample_point_editor_new (GimpDialogFactory *factory,
                                  GimpContext       *context,
+                                 GimpUIManager     *ui_manager,
                                  gint               view_size)
 {
-  return gimp_sample_point_editor_new (factory->menu_factory);
+  return gimp_sample_point_editor_new (gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
@@ -639,9 +713,10 @@ dialogs_sample_point_editor_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_navigation_editor_new (GimpDialogFactory *factory,
                                GimpContext       *context,
+                               GimpUIManager     *ui_manager,
                                gint               view_size)
 {
-  return gimp_navigation_editor_new (factory->menu_factory);
+  return gimp_navigation_editor_new (gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
@@ -650,6 +725,7 @@ dialogs_navigation_editor_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_color_editor_new (GimpDialogFactory *factory,
                           GimpContext       *context,
+                          GimpUIManager     *ui_manager,
                           gint               view_size)
 {
   return gimp_color_editor_new (context);
@@ -663,28 +739,51 @@ dialogs_color_editor_new (GimpDialogFactory *factory,
 GtkWidget *
 dialogs_brush_editor_get (GimpDialogFactory *factory,
                           GimpContext       *context,
+                          GimpUIManager     *ui_manager,
                           gint               view_size)
 {
   return gimp_brush_editor_new (context,
-                                factory->menu_factory);
+                                gimp_dialog_factory_get_menu_factory (factory));
+}
+
+GtkWidget *
+dialogs_dynamics_editor_get (GimpDialogFactory *factory,
+                             GimpContext       *context,
+                             GimpUIManager     *ui_manager,
+                             gint               view_size)
+{
+  return gimp_dynamics_editor_new (context,
+                                   gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_gradient_editor_get (GimpDialogFactory *factory,
                              GimpContext       *context,
+                             GimpUIManager     *ui_manager,
                              gint               view_size)
 {
   return gimp_gradient_editor_new (context,
-                                   factory->menu_factory);
+                                   gimp_dialog_factory_get_menu_factory (factory));
 }
 
 GtkWidget *
 dialogs_palette_editor_get (GimpDialogFactory *factory,
                             GimpContext       *context,
+                            GimpUIManager     *ui_manager,
                             gint               view_size)
 {
   return gimp_palette_editor_new (context,
-                                  factory->menu_factory);
+                                  gimp_dialog_factory_get_menu_factory (factory));
+}
+
+GtkWidget *
+dialogs_tool_preset_editor_get (GimpDialogFactory *factory,
+                                GimpContext       *context,
+                                GimpUIManager     *ui_manager,
+                                gint               view_size)
+{
+  return gimp_tool_preset_editor_new (context,
+                                      gimp_dialog_factory_get_menu_factory (factory));
 }
 
 
