@@ -1,6 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
+ * dialogs.c
+ * Copyright (C) 2010 Martin Nordholts <martinn@src.gnome.org>
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
@@ -30,9 +33,11 @@
 #include "core/gimplist.h"
 
 #include "widgets/gimpdialogfactory.h"
+#include "widgets/gimpdockwindow.h"
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimpsessioninfo.h"
+#include "widgets/gimptoolbox.h"
 
 #include "dialogs.h"
 #include "dialogs-constructors.h"
@@ -40,24 +45,144 @@
 #include "gimp-intl.h"
 
 
-GimpDialogFactory *global_dialog_factory  = NULL;
-GimpDialogFactory *global_dock_factory    = NULL;
-GimpDialogFactory *global_toolbox_factory = NULL;
-GimpDialogFactory *global_display_factory = NULL;
-
-GimpContainer     *global_recent_docks    = NULL;
+GimpContainer *global_recent_docks = NULL;
 
 
 #define FOREIGN(id, singleton, remember_size) \
-  { id, NULL, NULL, NULL, NULL, \
-    NULL, 0, singleton,  TRUE, remember_size, FALSE }
+  { id             /* identifier       */, \
+    NULL           /* name             */, \
+    NULL           /* blurb            */, \
+    NULL           /* stock_id         */, \
+    NULL           /* help_id          */, \
+    NULL           /* new_func         */, \
+    0              /* view_size        */, \
+    singleton      /* singleton        */, \
+    TRUE           /* session_managed  */, \
+    remember_size  /* remember_size    */, \
+    FALSE          /* remember_if_open */, \
+    TRUE           /* hideable         */, \
+    FALSE          /* dockable         */}
+
+#define FOREIGN_NOT_HIDEABLE(id, singleton, remember_size) \
+  { id             /* identifier       */, \
+    NULL           /* name             */, \
+    NULL           /* blurb            */, \
+    NULL           /* stock_id         */, \
+    NULL           /* help_id          */, \
+    NULL           /* new_func         */, \
+    0              /* view_size        */, \
+    singleton      /* singleton        */, \
+    TRUE           /* session_managed  */, \
+    remember_size  /* remember_size    */, \
+    FALSE          /* remember_if_open */, \
+    FALSE          /* hideable         */, \
+    FALSE          /* dockable         */}
 
 #define TOPLEVEL(id, new_func, singleton, session_managed, remember_size) \
-  { id, NULL, NULL, NULL, NULL, \
-    new_func, 0, singleton, session_managed, remember_size, FALSE }
+  { id               /* identifier       */, \
+    NULL             /* name             */, \
+    NULL             /* blurb            */, \
+    NULL             /* stock_id         */, \
+    NULL             /* help_id          */, \
+    new_func         /* new_func         */, \
+    0                /* view_size        */, \
+    singleton        /* singleton        */, \
+    session_managed  /* session_managed  */, \
+    remember_size    /* remember_size    */, \
+    FALSE            /* remember_if_open */, \
+    TRUE             /* hideable         */, \
+    FALSE            /* dockable         */}
+
+#define DOCKABLE(id, name, blurb, stock_id, help_id, new_func, view_size, singleton) \
+  { id         /* identifier       */, \
+    name       /* name             */, \
+    blurb      /* blurb            */, \
+    stock_id   /* stock_id         */, \
+    help_id    /* help_id          */, \
+    new_func   /* new_func         */, \
+    view_size  /* view_size        */, \
+    singleton  /* singleton        */, \
+    FALSE      /* session_managed  */, \
+    FALSE      /* remember_size    */, \
+    TRUE       /* remember_if_open */, \
+    TRUE       /* hideable         */, \
+    TRUE       /* dockable         */}
+
+#define DOCK(id, new_func) \
+  { id         /* identifier       */, \
+    NULL       /* name             */, \
+    NULL       /* blurb            */, \
+    NULL       /* stock_id         */, \
+    NULL       /* help_id          */, \
+    new_func   /* new_func         */, \
+    0          /* view_size        */, \
+    FALSE      /* singleton        */, \
+    FALSE      /* session_managed  */, \
+    FALSE      /* remember_size    */, \
+    FALSE      /* remember_if_open */, \
+    TRUE       /* hideable         */, \
+    FALSE      /* dockable         */}
+
+#define DOCK_WINDOW(id, new_func) \
+  { id         /* identifier       */, \
+    NULL       /* name             */, \
+    NULL       /* blurb            */, \
+    NULL       /* stock_id         */, \
+    NULL       /* help_id          */, \
+    new_func   /* new_func         */, \
+    0          /* view_size        */, \
+    FALSE      /* singleton        */, \
+    TRUE       /* session_managed  */, \
+    TRUE       /* remember_size    */, \
+    TRUE       /* remember_if_open */, \
+    TRUE       /* hideable         */, \
+    FALSE      /* dockable         */}
+
+#define LISTGRID(id, name, blurb, stock_id, help_id, view_size) \
+  { "gimp-"#id"-list"             /* identifier       */,  \
+    name                          /* name             */,  \
+    blurb                         /* blurb            */,  \
+    stock_id                      /* stock_id         */,  \
+    help_id                       /* help_id          */,  \
+    dialogs_##id##_list_view_new  /* new_func         */,  \
+    view_size                     /* view_size        */,  \
+    FALSE                         /* singleton        */,  \
+    FALSE                         /* session_managed  */,  \
+    FALSE                         /* remember_size    */,  \
+    TRUE                          /* remember_if_open */,  \
+    TRUE                          /* hideable         */,  \
+    TRUE                          /* dockable         */}, \
+  { "gimp-"#id"-grid"             /* identifier       */,  \
+    name                          /* name             */,  \
+    blurb                         /* blurb            */,  \
+    stock_id                      /* stock_id         */,  \
+    help_id                       /* help_id          */,  \
+    dialogs_##id##_grid_view_new  /* new_func         */,  \
+    view_size                     /* view_size        */,  \
+    FALSE                         /* singleton        */,  \
+    FALSE                         /* session_managed  */,  \
+    FALSE                         /* remember_size    */,  \
+    TRUE                          /* remember_if_open */,  \
+    TRUE                          /* hideable         */,  \
+    TRUE                          /* dockable         */}
+
+#define LIST(id, new_func, name, blurb, stock_id, help_id, view_size) \
+  { "gimp-"#id"-list"                   /* identifier       */, \
+    name                                /* name             */, \
+    blurb                               /* blurb            */, \
+    stock_id                            /* stock_id         */, \
+    help_id                             /* help_id          */, \
+    dialogs_##new_func##_list_view_new  /* new_func         */, \
+    view_size                           /* view_size        */, \
+    FALSE                               /* singleton        */, \
+    FALSE                               /* session_managed  */, \
+    FALSE                               /* remember_size    */, \
+    TRUE                                /* remember_if_open */, \
+    TRUE                                /* hideable         */, \
+    TRUE                                /* dockable         */}
 
 
-static const GimpDialogFactoryEntry toplevel_entries[] =
+static const GimpDialogFactoryEntry entries[] =
 {
   /*  foreign toplevels without constructor  */
   FOREIGN ("gimp-brightness-contrast-tool-dialog", TRUE,  FALSE),
@@ -102,6 +227,8 @@ static const GimpDialogFactoryEntry toplevel_entries[] =
   /*  singleton toplevels  */
   TOPLEVEL ("gimp-preferences-dialog",
             dialogs_preferences_get,        TRUE, TRUE,  FALSE),
+  TOPLEVEL ("gimp-input-devices-dialog",
+            dialogs_input_devices_get,      TRUE, TRUE,  FALSE),
   TOPLEVEL ("gimp-keyboard-shortcuts-dialog",
             dialogs_keyboard_shortcuts_get, TRUE, TRUE,  TRUE),
   TOPLEVEL ("gimp-module-dialog",
@@ -117,23 +244,20 @@ static const GimpDialogFactoryEntry toplevel_entries[] =
   TOPLEVEL ("gimp-close-all-dialog",
             dialogs_close_all_get,          TRUE, FALSE, FALSE),
   TOPLEVEL ("gimp-quit-dialog",
-            dialogs_quit_get,               TRUE, FALSE, FALSE)
-};
+            dialogs_quit_get,               TRUE, FALSE, FALSE),
 
-#define DOCKABLE(id,name,blurb,stock_id,help_id,\
-                 new_func,view_size,singleton) \
-  { id, name, blurb, stock_id, help_id, \
-    new_func, view_size, singleton, FALSE, FALSE, TRUE }
+  /*  docks  */
+  DOCK ("gimp-dock",
+        dialogs_dock_new),
+  DOCK ("gimp-toolbox",
+        dialogs_toolbox_new),
 
-#define LISTGRID(id,name,blurb,stock_id,help_id,\
-                 view_size) \
-  { "gimp-"#id"-list", name, blurb, stock_id, help_id, \
-    dialogs_##id##_list_view_new, view_size, FALSE, FALSE, FALSE, TRUE }, \
-  { "gimp-"#id"-grid", name, blurb, stock_id, help_id, \
-    dialogs_##id##_grid_view_new, view_size, FALSE, FALSE, FALSE, TRUE }
+  /*  dock windows  */
+  DOCK_WINDOW ("gimp-dock-window",
+               dialogs_dock_window_new),
+  DOCK_WINDOW ("gimp-toolbox-window",
+               dialogs_toolbox_dock_window_new),
 
-static const GimpDialogFactoryEntry dock_entries[] =
-{
   /*  singleton dockables  */
   DOCKABLE ("gimp-tool-options",
             N_("Tool Options"), NULL, GIMP_STOCK_TOOL_OPTIONS,
@@ -172,6 +296,12 @@ static const GimpDialogFactoryEntry dock_entries[] =
   LISTGRID (template, N_("Templates"), N_("Image Templates"), GIMP_STOCK_TEMPLATE,
             GIMP_HELP_TEMPLATE_DIALOG, GIMP_VIEW_SIZE_SMALL),
 
+  /* Some things do not have grids, so just list */
+  LIST (dynamics, dynamics, N_("Paint Dynamics"), NULL, GIMP_STOCK_DYNAMICS,
+        GIMP_HELP_DYNAMICS_DIALOG, GIMP_VIEW_SIZE_MEDIUM),
+  LIST (tool-preset, tool_preset, N_("Tool Presets"), NULL, GIMP_STOCK_TOOL_PRESET,
+        GIMP_HELP_TOOL_PRESET_DIALOG, GIMP_VIEW_SIZE_MEDIUM),
+
   /*  image related  */
   DOCKABLE ("gimp-layer-list",
             N_("Layers"), NULL, GIMP_STOCK_LAYERS,
@@ -223,6 +353,10 @@ static const GimpDialogFactoryEntry dock_entries[] =
             N_("Brush Editor"), NULL, GIMP_STOCK_BRUSH,
             GIMP_HELP_BRUSH_EDITOR_DIALOG,
             dialogs_brush_editor_get, 0, TRUE),
+  DOCKABLE ("gimp-dynamics-editor",
+            N_("Paint Dynamics Editor"), NULL, GIMP_STOCK_DYNAMICS,
+            GIMP_HELP_DYNAMICS_EDITOR_DIALOG,
+            dialogs_dynamics_editor_get, 0, TRUE),
   DOCKABLE ("gimp-gradient-editor",
             N_("Gradient Editor"), NULL, GIMP_STOCK_GRADIENT,
             GIMP_HELP_GRADIENT_EDITOR_DIALOG,
@@ -230,7 +364,17 @@ static const GimpDialogFactoryEntry dock_entries[] =
   DOCKABLE ("gimp-palette-editor",
             N_("Palette Editor"), NULL, GIMP_STOCK_PALETTE,
             GIMP_HELP_PALETTE_EDITOR_DIALOG,
-            dialogs_palette_editor_get, 0, TRUE)
+            dialogs_palette_editor_get, 0, TRUE),
+  DOCKABLE ("gimp-tool-preset-editor",
+            N_("Tool Preset Editor"), NULL, GIMP_STOCK_TOOL_PRESET,
+            GIMP_HELP_TOOL_PRESET_EDITOR_DIALOG,
+            dialogs_tool_preset_editor_get, 0, TRUE),
+
+  /*  image windows  */
+  FOREIGN_NOT_HIDEABLE ("gimp-empty-image-window",
+                        TRUE, TRUE),
+  FOREIGN_NOT_HIDEABLE ("gimp-single-image-window",
+                        TRUE, TRUE)
 };
 
 
@@ -240,77 +384,32 @@ void
 dialogs_init (Gimp            *gimp,
               GimpMenuFactory *menu_factory)
 {
-  gint i;
+  GimpDialogFactory *factory = NULL;
+  gint               i       = 0;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (GIMP_IS_MENU_FACTORY (menu_factory));
 
-  global_dialog_factory = gimp_dialog_factory_new ("toplevel",
-                                                   gimp_get_user_context (gimp),
-                                                   menu_factory,
-                                                   NULL,
-                                                   TRUE);
+  factory = gimp_dialog_factory_new ("toplevel",
+                                     gimp_get_user_context (gimp),
+                                     menu_factory);
+  gimp_dialog_factory_set_singleton (factory);
 
-  global_toolbox_factory = gimp_dialog_factory_new ("toolbox",
-                                                    gimp_get_user_context (gimp),
-                                                    menu_factory,
-                                                    dialogs_toolbox_get,
-                                                    TRUE);
-  gimp_dialog_factory_set_constructor (global_toolbox_factory,
-                                       dialogs_dockable_constructor);
-
-  global_dock_factory = gimp_dialog_factory_new ("dock",
-                                                 gimp_get_user_context (gimp),
-                                                 menu_factory,
-                                                 dialogs_dock_new,
-                                                 TRUE);
-  gimp_dialog_factory_set_constructor (global_dock_factory,
-                                       dialogs_dockable_constructor);
-
-  global_display_factory = gimp_dialog_factory_new ("display",
-                                                    gimp_get_user_context (gimp),
-                                                    menu_factory,
-                                                    NULL,
-                                                    FALSE);
-
-  for (i = 0; i < G_N_ELEMENTS (toplevel_entries); i++)
-    gimp_dialog_factory_register_entry (global_dialog_factory,
-                                        toplevel_entries[i].identifier,
-                                        gettext (toplevel_entries[i].name),
-                                        gettext (toplevel_entries[i].blurb),
-                                        toplevel_entries[i].stock_id,
-                                        toplevel_entries[i].help_id,
-                                        toplevel_entries[i].new_func,
-                                        toplevel_entries[i].view_size,
-                                        toplevel_entries[i].singleton,
-                                        toplevel_entries[i].session_managed,
-                                        toplevel_entries[i].remember_size,
-                                        toplevel_entries[i].remember_if_open);
-
-  for (i = 0; i < G_N_ELEMENTS (dock_entries); i++)
-    gimp_dialog_factory_register_entry (global_dock_factory,
-                                        dock_entries[i].identifier,
-                                        gettext (dock_entries[i].name),
-                                        gettext (dock_entries[i].blurb),
-                                        dock_entries[i].stock_id,
-                                        dock_entries[i].help_id,
-                                        dock_entries[i].new_func,
-                                        dock_entries[i].view_size,
-                                        dock_entries[i].singleton,
-                                        dock_entries[i].session_managed,
-                                        dock_entries[i].remember_size,
-                                        dock_entries[i].remember_if_open);
-
-  gimp_dialog_factory_register_entry (global_display_factory,
-                                      "gimp-empty-image-window",
-                                      NULL, NULL,
-                                      NULL, NULL,
-                                      NULL,
-                                      -1,
-                                      TRUE,
-                                      TRUE,
-                                      TRUE,
-                                      FALSE);
+  for (i = 0; i < G_N_ELEMENTS (entries); i++)
+    gimp_dialog_factory_register_entry (gimp_dialog_factory_get_singleton (),
+                                        entries[i].identifier,
+                                        gettext (entries[i].name),
+                                        gettext (entries[i].blurb),
+                                        entries[i].stock_id,
+                                        entries[i].help_id,
+                                        entries[i].new_func,
+                                        entries[i].view_size,
+                                        entries[i].singleton,
+                                        entries[i].session_managed,
+                                        entries[i].remember_size,
+                                        entries[i].remember_if_open,
+                                        entries[i].hideable,
+                                        entries[i].dockable);
 
   global_recent_docks = gimp_list_new (GIMP_TYPE_SESSION_INFO, FALSE);
 }
@@ -320,34 +419,15 @@ dialogs_exit (Gimp *gimp)
 {
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
-  if (global_dialog_factory)
+  if (gimp_dialog_factory_get_singleton ())
     {
-      g_object_unref (global_dialog_factory);
-      global_dialog_factory = NULL;
-    }
+      /* run dispose manually so the factory destroys its dialogs, which
+       * might in turn directly or indirectly ref the factory
+       */
+      g_object_run_dispose (G_OBJECT (gimp_dialog_factory_get_singleton ()));
 
-  /*  destroy the "global_toolbox_factory" _before_ destroying the
-   *  "global_dock_factory" because the "global_toolbox_factory" owns
-   *  dockables which were created by the "global_dock_factory".  This
-   *  way they are properly removed from the "global_dock_factory", which
-   *  would complain about stale entries otherwise.
-   */
-  if (global_toolbox_factory)
-    {
-      g_object_unref (global_toolbox_factory);
-      global_toolbox_factory = NULL;
-    }
-
-  if (global_dock_factory)
-    {
-      g_object_unref (global_dock_factory);
-      global_dock_factory = NULL;
-    }
-
-  if (global_display_factory)
-    {
-      g_object_unref (global_display_factory);
-      global_display_factory = NULL;
+      g_object_unref (gimp_dialog_factory_get_singleton ());
+      gimp_dialog_factory_set_singleton (NULL);
     }
 
   if (global_recent_docks)
@@ -357,6 +437,23 @@ dialogs_exit (Gimp *gimp)
     }
 }
 
+static void
+dialogs_ensure_factory_entry_on_recent_dock (GimpSessionInfo *info)
+{
+  if (! gimp_session_info_get_factory_entry (info))
+    {
+      GimpDialogFactoryEntry *entry = NULL;
+
+      /* The recent docks container only contains session infos for
+       * dock windows
+       */
+      entry = gimp_dialog_factory_find_entry (gimp_dialog_factory_get_singleton (),
+                                              "gimp-dock-window");
+
+      gimp_session_info_set_factory_entry (info, entry);
+    }
+}
+
 void
 dialogs_load_recent_docks (Gimp *gimp)
 {
@@ -380,6 +477,13 @@ dialogs_load_recent_docks (Gimp *gimp)
       g_clear_error (&error);
     }
 
+  /* In GIMP 2.6 dockrc did not contain the factory entries for the
+   * session infos, so set that up manually if needed
+   */
+  gimp_container_foreach (global_recent_docks,
+                          (GFunc) dialogs_ensure_factory_entry_on_recent_dock,
+                          NULL);
+
   gimp_list_reverse (GIMP_LIST (global_recent_docks));
 
   g_free (filename);
@@ -416,13 +520,14 @@ dialogs_get_toolbox (void)
 {
   GList *list;
 
-  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (global_toolbox_factory), NULL);
+  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (gimp_dialog_factory_get_singleton ()), NULL);
 
-  for (list = global_toolbox_factory->open_dialogs;
+  for (list = gimp_dialog_factory_get_open_dialogs (gimp_dialog_factory_get_singleton ());
        list;
        list = g_list_next (list))
     {
-      if (GTK_WIDGET_TOPLEVEL (list->data))
+      if (GIMP_IS_DOCK_WINDOW (list->data) &&
+          gimp_dock_window_has_toolbox (list->data))
         return list->data;
     }
 
