@@ -125,6 +125,15 @@ file_open_dialog_response (GtkWidget *open_dialog,
 
   gimp_file_dialog_set_sensitive (dialog, FALSE);
 
+  /* When we are going to open new image windows, unset the transient
+   * window. We don't need it since we will use gdk_window_raise() to
+   * keep the dialog on top. And if we don't do it, then the dialog
+   * will pull the image window it was invoked from on top of all the
+   * new opened image windows, and we don't want that to happen.
+   */
+  if (! dialog->open_as_layers)
+    gtk_window_set_transient_for (GTK_WINDOW (open_dialog), NULL);
+
   for (list = uris; list; list = g_slist_next (list))
     {
       gchar *filename = file_utils_filename_from_uri (list->data);
@@ -171,7 +180,10 @@ file_open_dialog_response (GtkWidget *open_dialog,
             {
               success = TRUE;
 
-              gdk_window_raise (open_dialog->window);
+              /* Make the dialog stay on top of all images we open if
+               * we open say 10 at once
+               */
+              gdk_window_raise (gtk_widget_get_window (open_dialog));
             }
         }
 
