@@ -103,7 +103,7 @@ file_save_dialog_new (Gimp     *gimp,
       dialog = gimp_file_dialog_new (gimp,
                                      GIMP_FILE_CHOOSER_ACTION_EXPORT,
                                      _("Export Image"), "gimp-file-export",
-                                     GTK_STOCK_SAVE,
+                                     _("_Export"),
                                      GIMP_HELP_FILE_EXPORT);
 
       state = g_object_get_data (G_OBJECT (gimp), "gimp-file-export-dialog-state");
@@ -260,7 +260,7 @@ file_save_dialog_response (GtkWidget *save_dialog,
 
                       display = GIMP_DISPLAY_SHELL (parent)->display;
 
-                      if (! gimp_image_is_dirty (display->image))
+                      if (! gimp_image_is_dirty (gimp_display_get_image (display)))
                         gimp_display_close (display);
                     }
                 }
