@@ -3,9 +3,9 @@
  *
  * Copyright (C) 2003  Henrik Brix Andersen <brix@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -69,11 +69,15 @@ grid_dialog_new (GimpImage   *image,
   GimpGrid  *grid_backup;
   GtkWidget *dialog;
   GtkWidget *editor;
+  gdouble    xres;
+  gdouble    yres;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (parent == NULL || GTK_IS_WIDGET (parent), NULL);
 
+  gimp_image_get_resolution (image, &xres, &yres);
+
   grid = gimp_image_get_grid (GIMP_IMAGE (image));
   grid_backup = gimp_config_duplicate (GIMP_CONFIG (grid));
 
@@ -102,10 +106,9 @@ grid_dialog_new (GimpImage   *image,
                     G_CALLBACK (grid_dialog_response),
                     dialog);
 
-  editor = gimp_grid_editor_new (grid, context,
-                                 image->xresolution, image->yresolution);
+  editor = gimp_grid_editor_new (grid, context, xres, yres);
   gtk_container_set_border_width (GTK_CONTAINER (editor), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
                      editor);
 
   gtk_widget_show (editor);
