@@ -314,7 +314,7 @@ palette_import_dialog_new (GimpContext *context)
   dialog->entry = gtk_entry_new ();
   gtk_entry_set_text (GTK_ENTRY (dialog->entry),
                       gradient ?
-                      GIMP_OBJECT (gradient)->name : _("New import"));
+                      gimp_object_get_name (gradient) : _("New import"));
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
                              _("Palette _name:"), 0.0, 0.5,
                              dialog->entry, 2, FALSE);
@@ -476,7 +476,7 @@ palette_import_gradient_changed (GimpContext  *context,
   if (gradient && dialog->import_type == GRADIENT_IMPORT)
     {
       gtk_entry_set_text (GTK_ENTRY (dialog->entry),
-                          GIMP_OBJECT (gradient)->name);
+                          gimp_object_get_name (gradient));
 
       palette_import_make_palette (dialog);
     }
@@ -654,7 +654,7 @@ palette_import_grad_callback (GtkWidget    *widget,
   gradient = gimp_context_get_gradient (dialog->context);
 
   gtk_entry_set_text (GTK_ENTRY (dialog->entry),
-                      GIMP_OBJECT (gradient)->name);
+                      gimp_object_get_name (gradient));
 
   palette_import_set_sensitive (dialog);
 
@@ -734,7 +734,7 @@ palette_import_image_add (GimpContainer *container,
                           GimpImage     *image,
                           ImportDialog  *dialog)
 {
-  if (! GTK_WIDGET_IS_SENSITIVE (dialog->image_radio))
+  if (! gtk_widget_is_sensitive (dialog->image_radio))
     {
       gtk_widget_set_sensitive (dialog->image_radio, TRUE);
       gimp_context_set_image (dialog->context, image);
@@ -807,11 +807,13 @@ palette_import_make_palette (ImportDialog *dialog)
         if (gimp_image_base_type (image) == GIMP_INDEXED)
           {
             palette = gimp_palette_import_from_indexed_image (image,
+                                                              dialog->context,
                                                               palette_name);
           }
         else if (sample_merged)
           {
             palette = gimp_palette_import_from_image (image,
+                                                      dialog->context,
                                                       palette_name,
                                                       n_colors,
                                                       threshold,
@@ -824,6 +826,7 @@ palette_import_make_palette (ImportDialog *dialog)
             drawable = GIMP_DRAWABLE (gimp_image_get_active_layer (image));
 
             palette = gimp_palette_import_from_drawable (drawable,
+                                                         dialog->context,
                                                          palette_name,
                                                          n_colors,
                                                          threshold,
@@ -840,7 +843,8 @@ palette_import_make_palette (ImportDialog *dialog)
         filename =
           gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog->file_chooser));
 
-        palette = gimp_palette_import_from_file (filename,
+        palette = gimp_palette_import_from_file (dialog->context,
+                                                 filename,
                                                  palette_name, &error);
         g_free (filename);
 
@@ -868,8 +872,6 @@ palette_import_make_palette (ImportDialog *dialog)
       dialog->palette = palette;
     }
 
-  if (dialog->palette && dialog->palette->n_colors > 0)
-    gtk_widget_hide (dialog->no_colors_label);
-  else
-    gtk_widget_show (dialog->no_colors_label);
+  gtk_widget_set_visible (dialog->no_colors_label,
+                          dialog->palette && dialog->palette->n_colors > 0);
 }
