@@ -38,7 +38,6 @@
 #include "widgets/gimpcontainercombobox.h"
 #include "widgets/gimpcontainerview.h"
 #include "widgets/gimpcontrollerlist.h"
-#include "widgets/gimpdeviceinfo.h"
 #include "widgets/gimpdevices.h"
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpgrideditor.h"
@@ -100,9 +99,8 @@ static void   prefs_resolution_calibrate_callback (GtkWidget  *widget,
                                                    GtkWidget  *entry);
 static void   prefs_input_devices_dialog          (GtkWidget  *widget,
                                                    Gimp       *gimp);
-static void   prefs_input_dialog_able_callback    (GtkWidget  *widget,
-                                                   GdkDevice  *device,
-                                                   gpointer    data);
+static void   prefs_keyboard_shortcuts_dialog     (GtkWidget  *widget,
+                                                   Gimp       *gimp);
 static void   prefs_menus_save_callback           (GtkWidget  *widget,
                                                    Gimp       *gimp);
 static void   prefs_menus_clear_callback          (GtkWidget  *widget,
@@ -493,58 +491,16 @@ static void
 prefs_input_devices_dialog (GtkWidget *widget,
                             Gimp      *gimp)
 {
-  static GtkWidget *input_dialog = NULL;
-
-  if (input_dialog)
-    {
-      gtk_window_present (GTK_WINDOW (input_dialog));
-      return;
-    }
-
-  input_dialog = g_object_new (GTK_TYPE_INPUT_DIALOG,
-                               "title", _("Configure Input Devices"),
-                               NULL);
-
-  g_object_add_weak_pointer (G_OBJECT (input_dialog),
-                             (gpointer) &input_dialog);
-
-  gtk_window_set_transient_for (GTK_WINDOW (input_dialog),
-                                GTK_WINDOW (prefs_dialog));
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (input_dialog), TRUE);
-
-  g_signal_connect_swapped (GTK_INPUT_DIALOG (input_dialog)->save_button,
-                            "clicked",
-                            G_CALLBACK (gimp_devices_save),
-                            gimp);
-
-  g_signal_connect_swapped (GTK_INPUT_DIALOG (input_dialog)->close_button,
-                            "clicked",
-                            G_CALLBACK (gtk_widget_destroy),
-                            input_dialog);
-
-  g_signal_connect (input_dialog, "enable-device",
-                    G_CALLBACK (prefs_input_dialog_able_callback),
-                    NULL);
-  g_signal_connect (input_dialog, "disable-device",
-                    G_CALLBACK (prefs_input_dialog_able_callback),
-                    NULL);
-
-  gtk_widget_show (input_dialog);
-}
-
-static void
-prefs_input_dialog_able_callback (GtkWidget *widget,
-                                  GdkDevice *device,
-                                  gpointer   data)
-{
-  gimp_device_info_changed_by_device (device);
+  gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
+                                    gtk_widget_get_screen (widget),
+                                    "gimp-input-devices-dialog", 0);
 }
 
 static void
 prefs_keyboard_shortcuts_dialog (GtkWidget *widget,
                                  Gimp      *gimp)
 {
-  gimp_dialog_factory_dialog_raise (gimp_dialog_factory_from_name ("toplevel"),
+  gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
                                     gtk_widget_get_screen (widget),
                                     "gimp-keyboard-shortcuts-dialog", 0);
 }
@@ -1220,6 +1176,22 @@ prefs_boolean_combo_box_add (GObject      *config,
   return combo;
 }
 
+static GtkWidget *
+prefs_language_combo_box_add (GObject      *config,
+                              const gchar  *property_name,
+                              GtkBox       *vbox)
+{
+  GtkWidget *combo = gimp_prop_language_combo_box_new (config, property_name);
+
+  if (combo)
+    {
+      gtk_box_pack_start (vbox, combo, FALSE, FALSE, 0);
+      gtk_widget_show (combo);
+    }
+
+  return combo;
+}
+
 static GtkWidget *
 prefs_spin_button_add (GObject      *config,
                        const gchar  *property_name,
@@ -1619,6 +1591,15 @@ prefs_dialog_new (Gimp       *gimp,
                                      &top_iter,
                                      page_index++);
 
+  /*  Language  */
+
+  /*  Only add the language entry if the iso-codes package is available.  */
+#ifdef HAVE_ISO_CODES
+  vbox2 = prefs_frame_new (_("Language"), GTK_CONTAINER (vbox), FALSE);
+
+  prefs_language_combo_box_add (object, "language", GTK_BOX (vbox2));
+#endif
+
   /*  Previews  */
   vbox2 = prefs_frame_new (_("Previews"), GTK_CONTAINER (vbox), FALSE);
 
@@ -1932,6 +1913,9 @@ prefs_dialog_new (Gimp       *gimp,
   prefs_check_button_add_with_icon (object, "global-brush",
                                     _("_Brush"),    GIMP_STOCK_BRUSH,
                                     GTK_BOX (vbox2), size_group);
+  prefs_check_button_add_with_icon (object, "global-dynamics",
+                                    _("_Dynamics"), GIMP_STOCK_DYNAMICS,
+                                    GTK_BOX (vbox2), size_group);
   prefs_check_button_add_with_icon (object, "global-pattern",
                                     _("_Pattern"),  GIMP_STOCK_PATTERN,
                                     GTK_BOX (vbox2), size_group);
@@ -2629,14 +2613,10 @@ prefs_dialog_new (Gimp       *gimp,
   vbox2 = prefs_frame_new (_("Window Manager Hints"),
                            GTK_CONTAINER (vbox), FALSE);
 
-  table = prefs_table_new (2, GTK_CONTAINER (vbox2));
-
-  prefs_enum_combo_box_add (object, "toolbox-window-hint", 0, 0,
-                            _("Hint for the _toolbox:"),
-                            GTK_TABLE (table), 0, size_group);
+  table = prefs_table_new (1, GTK_CONTAINER (vbox2));
 
   prefs_enum_combo_box_add (object, "dock-window-hint", 0, 0,
-                            _("Hint for other _docks:"),
+                            _("Hint for _docks and toolbox:"),
                             GTK_TABLE (table), 1, size_group);
 
   vbox2 = prefs_frame_new (_("Focus"),
@@ -2741,6 +2721,10 @@ prefs_dialog_new (Gimp       *gimp,
         GIMP_HELP_PREFS_FOLDERS_BRUSHES,
         N_("Select Brush Folders"),
         "brush-path", "brush-path-writable" },
+      { N_("Dynamics"), N_("Dynamics Folders"), "folders-dynamics",
+        GIMP_HELP_PREFS_FOLDERS_DYNAMICS,
+        N_("Select Dynamics Folders"),
+        "dynamics-path", "dynamics-path-writable" },
       { N_("Patterns"), N_("Pattern Folders"), "folders-patterns",
         GIMP_HELP_PREFS_FOLDERS_PATTERNS,
         N_("Select Pattern Folders"),
