@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -103,8 +103,8 @@ scale_dialog_new (GimpViewable          *viewable,
 
       image = gimp_item_get_image (item);
 
-      width  = gimp_item_width (item);
-      height = gimp_item_height (item);
+      width  = gimp_item_get_width  (item);
+      height = gimp_item_get_height (item);
 
       text = _("Layer Size");
     }
@@ -162,7 +162,8 @@ scale_dialog_new (GimpViewable          *viewable,
 
   vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     vbox);
   gtk_widget_show (vbox);
 
   frame = gimp_frame_new (text);
@@ -206,7 +207,7 @@ scale_dialog_new (GimpViewable          *viewable,
                                  _("Indexed color layers are always scaled "
                                    "without interpolation. The chosen "
                                    "interpolation type will affect channels "
-                                   "and masks only."));
+                                   "and layer masks only."));
 
       gtk_container_set_border_width (GTK_CONTAINER (box), 0);
       gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
@@ -279,8 +280,8 @@ scale_dialog_reset (ScaleDialog *private)
 
       image = gimp_item_get_image (item);
 
-      width  = gimp_item_width (item);
-      height = gimp_item_height (item);
+      width  = gimp_item_get_width  (item);
+      height = gimp_item_get_height (item);
     }
   else
     {
