@@ -35,6 +35,9 @@
 #include "gimp-intl.h"
 
 
+#define MAX_BATCH_SIZE 32000
+
+
 enum
 {
   PROP_0,
@@ -44,27 +47,31 @@ enum
 
 /*  local function prototypes  */
 
-static void    gimp_canvas_set_property  (GObject         *object,
-                                          guint            property_id,
-                                          const GValue    *value,
-                                          GParamSpec      *pspec);
-static void    gimp_canvas_get_property  (GObject         *object,
-                                          guint            property_id,
-                                          GValue          *value,
-                                          GParamSpec      *pspec);
+static void       gimp_canvas_set_property    (GObject         *object,
+                                               guint            property_id,
+                                               const GValue    *value,
+                                               GParamSpec      *pspec);
+static void       gimp_canvas_get_property    (GObject         *object,
+                                               guint            property_id,
+                                               GValue          *value,
+                                               GParamSpec      *pspec);
 
-static void    gimp_canvas_realize       (GtkWidget       *widget);
-static void    gimp_canvas_unrealize     (GtkWidget       *widget);
-static void    gimp_canvas_size_allocate (GtkWidget       *widget,
-                                          GtkAllocation   *allocation);
-static void    gimp_canvas_style_set     (GtkWidget       *widget,
-                                          GtkStyle        *prev_style);
+static void       gimp_canvas_realize         (GtkWidget       *widget);
+static void       gimp_canvas_unrealize       (GtkWidget       *widget);
+static void       gimp_canvas_style_set       (GtkWidget       *widget,
+                                               GtkStyle        *prev_style);
+static gboolean   gimp_canvas_focus_in_event  (GtkWidget       *widget,
+                                               GdkEventFocus   *event);
+static gboolean   gimp_canvas_focus_out_event (GtkWidget       *widget,
+                                               GdkEventFocus   *event);
+static gboolean   gimp_canvas_focus           (GtkWidget       *widget,
+                                               GtkDirectionType direction);
 
-static GdkGC * gimp_canvas_gc_new       (GimpCanvas      *canvas,
-                                         GimpCanvasStyle  style);
+static GdkGC    * gimp_canvas_gc_new          (GimpCanvas      *canvas,
+                                               GimpCanvasStyle  style);
 
 
-G_DEFINE_TYPE (GimpCanvas, gimp_canvas, GTK_TYPE_CONTAINER)
+G_DEFINE_TYPE (GimpCanvas, gimp_canvas, GIMP_TYPE_OVERLAY_BOX)
 
 #define parent_class gimp_canvas_parent_class
 
@@ -160,13 +167,15 @@ gimp_canvas_class_init (GimpCanvasClass *klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  object_class->set_property  = gimp_canvas_set_property;
-  object_class->get_property  = gimp_canvas_get_property;
+  object_class->set_property    = gimp_canvas_set_property;
+  object_class->get_property    = gimp_canvas_get_property;
 
-  widget_class->realize       = gimp_canvas_realize;
-  widget_class->unrealize     = gimp_canvas_unrealize;
-  widget_class->size_allocate = gimp_canvas_size_allocate;
-  widget_class->style_set     = gimp_canvas_style_set;
+  widget_class->realize         = gimp_canvas_realize;
+  widget_class->unrealize       = gimp_canvas_unrealize;
+  widget_class->style_set       = gimp_canvas_style_set;
+  widget_class->focus_in_event  = gimp_canvas_focus_in_event;
+  widget_class->focus_out_event = gimp_canvas_focus_out_event;
+  widget_class->focus           = gimp_canvas_focus;
 
   g_object_class_install_property (object_class, PROP_CONFIG,
                                    g_param_spec_object ("config", NULL, NULL,
@@ -181,8 +190,9 @@ gimp_canvas_init (GimpCanvas *canvas)
   GtkWidget *widget = GTK_WIDGET (canvas);
   gint       i;
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
-
+  gtk_widget_set_double_buffered (widget, FALSE);
+  gtk_widget_set_can_focus (widget, TRUE);
+  gtk_widget_add_events (widget, GIMP_CANVAS_EVENT_MASK);
   gtk_widget_set_extension_events (widget, GDK_EXTENSION_EVENTS_ALL);
 
   for (i = 0; i < GIMP_CANVAS_NUM_STYLES; i++)
@@ -194,9 +204,9 @@ gimp_canvas_init (GimpCanvas *canvas)
 
 static void
 gimp_canvas_set_property (GObject      *object,
-                           guint         property_id,
-                           const GValue *value,
-                           GParamSpec   *pspec)
+                          guint         property_id,
+                          const GValue *value,
+                          GParamSpec   *pspec)
 {
   GimpCanvas *canvas = GIMP_CANVAS (object);
 
@@ -233,32 +243,9 @@ gimp_canvas_get_property (GObject    *object,
 static void
 gimp_canvas_realize (GtkWidget *widget)
 {
-  GimpCanvas    *canvas = GIMP_CANVAS (widget);
-  GdkWindowAttr  attributes;
-  gint           attributes_mask;
+  GimpCanvas *canvas = GIMP_CANVAS (widget);
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
-  attributes.x           = widget->allocation.x;
-  attributes.y           = widget->allocation.y;
-  attributes.width       = widget->allocation.width;
-  attributes.height      = widget->allocation.height;
-  attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.wclass      = GDK_INPUT_OUTPUT;
-  attributes.visual      = gtk_widget_get_visual (widget);
-  attributes.colormap    = gtk_widget_get_colormap (widget);
-  attributes.event_mask  = (gtk_widget_get_events (widget) |
-                            GIMP_CANVAS_EVENT_MASK);
-
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-
-  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-                                   &attributes, attributes_mask);
-  gdk_window_set_user_data (gtk_widget_get_window (widget), widget);
-
-  widget->style = gtk_style_attach (widget->style, widget->window);
-  gtk_style_set_background (widget->style, gtk_widget_get_window (widget),
-                            GTK_STATE_NORMAL);
+  GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
   canvas->stipple[0] =
     gdk_bitmap_create_from_data (gtk_widget_get_window (widget),
@@ -298,18 +285,6 @@ gimp_canvas_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
-static void
-gimp_canvas_size_allocate (GtkWidget     *widget,
-                           GtkAllocation *allocation)
-{
-  widget->allocation = *allocation;
-
-  if (GTK_WIDGET_REALIZED (widget))
-    gdk_window_move_resize (gtk_widget_get_window (widget),
-                            allocation->x, allocation->y,
-                            allocation->width, allocation->height);
-}
-
 static void
 gimp_canvas_style_set (GtkWidget *widget,
                        GtkStyle  *prev_style)
@@ -325,6 +300,44 @@ gimp_canvas_style_set (GtkWidget *widget,
     }
 }
 
+static gboolean
+gimp_canvas_focus_in_event (GtkWidget     *widget,
+                            GdkEventFocus *event)
+{
+  /*  don't allow the default impl to invalidate the whole widget,
+   *  we don't draw a focus indicator anyway.
+   */
+  return FALSE;
+}
+
+static gboolean
+gimp_canvas_focus_out_event (GtkWidget     *widget,
+                             GdkEventFocus *event)
+{
+  /*  see focus-in-event
+   */
+  return FALSE;
+}
+
+static gboolean
+gimp_canvas_focus (GtkWidget        *widget,
+                   GtkDirectionType  direction)
+{
+  GtkWidget *focus = gtk_container_get_focus_child (GTK_CONTAINER (widget));
+
+  /* override GtkContainer's focus() implementation which would always
+   * give focus to the canvas because it is focussable. Instead, try
+   * navigating in the focussed overlay child first, and use
+   * GtkContainer's default implementation only if that fails (which
+   * happens when focus navigation leaves the overlay child).
+   */
+
+  if (focus && gtk_widget_child_focus (focus, direction))
+    return TRUE;
+
+  return GTK_WIDGET_CLASS (parent_class)->focus (widget, direction);
+}
+
 /* Returns: %TRUE if the XOR color is not white */
 static gboolean
 gimp_canvas_get_xor_color (GimpCanvas *canvas,
@@ -351,7 +364,7 @@ gimp_canvas_gc_new (GimpCanvas      *canvas,
   GdkColor         fg   = { 0, 0, 0, 0 };
   GdkColor         bg   = { 0, 0, 0, 0 };
 
-  if (! GTK_WIDGET_REALIZED (canvas))
+  if (! gtk_widget_get_realized (GTK_WIDGET (canvas)))
     return NULL;
 
   switch (style)
@@ -388,6 +401,7 @@ gimp_canvas_gc_new (GimpCanvas      *canvas,
     case GIMP_CANVAS_STYLE_SELECTION_IN:
     case GIMP_CANVAS_STYLE_SELECTION_OUT:
     case GIMP_CANVAS_STYLE_LAYER_BOUNDARY:
+    case GIMP_CANVAS_STYLE_LAYER_GROUP_BOUNDARY:
     case GIMP_CANVAS_STYLE_GUIDE_NORMAL:
     case GIMP_CANVAS_STYLE_GUIDE_ACTIVE:
     case GIMP_CANVAS_STYLE_LAYER_MASK_ACTIVE:
@@ -458,6 +472,16 @@ gimp_canvas_gc_new (GimpCanvas      *canvas,
       bg.blue  = 0x0;
       break;
 
+    case GIMP_CANVAS_STYLE_LAYER_GROUP_BOUNDARY:
+      fg.red   = 0x0;
+      fg.green = 0x0;
+      fg.blue  = 0x0;
+
+      bg.red   = 0x0;
+      bg.green = 0xffff;
+      bg.blue  = 0xffff;
+      break;
+
     case GIMP_CANVAS_STYLE_GUIDE_NORMAL:
       fg.red   = 0x0;
       fg.green = 0x0;
@@ -545,34 +569,6 @@ gimp_canvas_new (GimpDisplayConfig *config)
                        NULL);
 }
 
-/**
- * gimp_canvas_scroll:
- * @canvas: the #GimpCanvas widget to scroll.
- * @offset_x: the x scroll amount.
- * @offset_y: the y scroll amount.
- *
- * Scrolls the canvas using gdk_window_scroll() and makes sure the result
- * is displayed immediately by calling gdk_window_process_updates().
- **/
-void
-gimp_canvas_scroll (GimpCanvas *canvas,
-                    gint        offset_x,
-                    gint        offset_y)
-{
-  GtkWidget *widget;
-  GdkWindow *window;
-
-  g_return_if_fail (GIMP_IS_CANVAS (canvas));
-
-  widget = GTK_WIDGET (canvas);
-  window = gtk_widget_get_window (widget);
-
-  gdk_window_scroll (window, offset_x, offset_y);
-
-  /*  Make sure expose events are processed before scrolling again  */
-  gdk_window_process_updates (window, FALSE);
-}
-
 /**
  * gimp_canvas_draw_cursor:
  * @canvas: the #GimpCanvas widget to draw on.
@@ -814,13 +810,13 @@ gimp_canvas_draw_segments (GimpCanvas      *canvas,
   if (! gimp_canvas_ensure_style (canvas, style))
     return;
 
-  while (num_segments >= 32000)
+  while (num_segments >= MAX_BATCH_SIZE)
     {
       gdk_draw_segments (gtk_widget_get_window (GTK_WIDGET (canvas)),
                          canvas->gc[style],
-                         segments, 32000);
-      num_segments -= 32000;
-      segments     += 32000;
+                         segments, MAX_BATCH_SIZE);
+      num_segments -= MAX_BATCH_SIZE;
+      segments     += MAX_BATCH_SIZE;
     }
 
   gdk_draw_segments (gtk_widget_get_window (GTK_WIDGET (canvas)),
@@ -910,22 +906,26 @@ void
 gimp_canvas_draw_drop_zone (GimpCanvas *canvas,
                             cairo_t    *cr)
 {
-  GtkWidget *widget = GTK_WIDGET (canvas);
-  GtkStyle  *style  = gtk_widget_get_style (widget);
-  gdouble    wilber_width;
-  gdouble    wilber_height;
-  gdouble    width;
-  gdouble    height;
-  gdouble    side;
-  gdouble    factor;
+  GtkWidget    *widget = GTK_WIDGET (canvas);
+  GtkStyle     *style  = gtk_widget_get_style (widget);
+  GtkStateType  state  = gtk_widget_get_state (widget);
+  GtkAllocation allocation;
+  gdouble       wilber_width;
+  gdouble       wilber_height;
+  gdouble       width;
+  gdouble       height;
+  gdouble       side;
+  gdouble       factor;
+
+  gtk_widget_get_allocation (widget, &allocation);
 
   gimp_cairo_wilber_get_size (cr, &wilber_width, &wilber_height);
 
   wilber_width  /= 2;
   wilber_height /= 2;
 
-  side = MIN (MIN (widget->allocation.width, widget->allocation.height),
-              MAX (widget->allocation.width, widget->allocation.height) / 2);
+  side = MIN (MIN (allocation.width, allocation.height),
+              MAX (allocation.width, allocation.height) / 2);
 
   width  = MAX (wilber_width,  side);
   height = MAX (wilber_height, side);
@@ -938,12 +938,12 @@ gimp_canvas_draw_drop_zone (GimpCanvas *canvas,
    */
   gimp_cairo_wilber (cr,
                      - wilber_width * 0.6,
-                     widget->allocation.height / factor - wilber_height * 1.1);
+                     allocation.height / factor - wilber_height * 1.1);
 
   cairo_set_source_rgba (cr,
-                         style->fg[widget->state].red   / 65535.0,
-                         style->fg[widget->state].green / 65535.0,
-                         style->fg[widget->state].blue  / 65535.0,
+                         style->fg[state].red   / 65535.0,
+                         style->fg[state].green / 65535.0,
+                         style->fg[state].blue  / 65535.0,
                          0.15);
   cairo_fill (cr);
 }
@@ -957,9 +957,9 @@ gimp_canvas_draw_drop_zone (GimpCanvas *canvas,
  * Sets a rectangular clipping area for the specified style.
  **/
 void
-gimp_canvas_set_clip_rect (GimpCanvas      *canvas,
-                           GimpCanvasStyle  style,
-                           GdkRectangle    *rect)
+gimp_canvas_set_clip_rect (GimpCanvas         *canvas,
+                           GimpCanvasStyle     style,
+                           const GdkRectangle *rect)
 {
   if (! canvas->gc[style])
     {
@@ -983,7 +983,7 @@ gimp_canvas_set_clip_rect (GimpCanvas      *canvas,
 void
 gimp_canvas_set_clip_region (GimpCanvas      *canvas,
                              GimpCanvasStyle  style,
-                             GdkRegion       *region)
+                             const GdkRegion *region)
 {
   if (! canvas->gc[style])
     {
@@ -1072,7 +1072,7 @@ gimp_canvas_set_bg_color (GimpCanvas *canvas,
   GdkColormap *colormap;
   GdkColor     gdk_color;
 
-  if (! GTK_WIDGET_REALIZED (widget))
+  if (! gtk_widget_get_realized (widget))
     return;
 
   gimp_rgb_get_gdk_color (color, &gdk_color);
