@@ -25,6 +25,8 @@
 
 #include "display-types.h"
 
+#include "config/gimpdisplayoptions.h"
+
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpimage.h"
@@ -39,62 +41,58 @@
 
 #include "gimpcanvas.h"
 #include "gimpdisplay.h"
-#include "gimpdisplayoptions.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-appearance.h"
+#include "gimpdisplayshell-expose.h"
 #include "gimpdisplayshell-selection.h"
+#include "gimpimagewindow.h"
 #include "gimpstatusbar.h"
 
 
-#define GET_OPTIONS(shell) \
-  (shell->display->image ? \
-   (gimp_display_shell_get_fullscreen (shell) ? \
-    shell->fullscreen_options : shell->options) : \
-   shell->no_image_options)
+/*  local function prototypes  */
 
-#define SET_ACTIVE(manager,action_name,active) \
-  { GimpActionGroup *group = \
-      gimp_ui_manager_get_action_group (manager, "view"); \
-    gimp_action_group_set_action_active (group, action_name, active); }
+static GimpDisplayOptions *
+              appearance_get_options       (GimpDisplayShell       *shell);
+static void   appearance_set_action_active (GimpDisplayShell       *shell,
+                                            const gchar            *action,
+                                            gboolean                active);
+static void   appearance_set_action_color  (GimpDisplayShell       *shell,
+                                            const gchar            *action,
+                                            const GimpRGB          *color);
 
-#define SET_COLOR(manager,action_name,color) \
-  { GimpActionGroup *group = \
-      gimp_ui_manager_get_action_group (manager, "view"); \
-    gimp_action_group_set_action_color (group, action_name, color, FALSE); }
-
-#define IS_ACTIVE_DISPLAY(shell) \
-  ((shell)->display == \
-   gimp_context_get_display (gimp_get_user_context \
-                             ((shell)->display->gimp)))
 
+/*  public functions  */
 
 void
 gimp_display_shell_appearance_update (GimpDisplayShell *shell)
 {
   GimpDisplayOptions *options;
-  gboolean            fullscreen;
+  GimpImageWindow    *window;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
+  window  = gimp_display_shell_get_window (shell);
 
-  fullscreen = gimp_display_shell_get_fullscreen (shell);
+  if (window)
+    {
+      gboolean fullscreen = gimp_image_window_get_fullscreen (window);
 
-  if (shell->menubar)
-    gtk_widget_set_name (GTK_WIDGET (shell->menubar),
-                         fullscreen ? "gimp-menubar-fullscreen" : NULL);
+      appearance_set_action_active (shell, "view-fullscreen", fullscreen);
 
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (shell->statusbar),
-                                     ! fullscreen);
+      gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (shell->statusbar),
+                                         ! fullscreen);
+    }
 
   gimp_display_shell_set_show_menubar       (shell,
                                              options->show_menubar);
+  gimp_display_shell_set_show_statusbar     (shell,
+                                             options->show_statusbar);
+
   gimp_display_shell_set_show_rulers        (shell,
                                              options->show_rulers);
   gimp_display_shell_set_show_scrollbars    (shell,
                                              options->show_scrollbars);
-  gimp_display_shell_set_show_statusbar     (shell,
-                                             options->show_statusbar);
   gimp_display_shell_set_show_selection     (shell,
                                              options->show_selection);
   gimp_display_shell_set_show_layer         (shell,
@@ -110,167 +108,35 @@ gimp_display_shell_appearance_update (GimpDisplayShell *shell)
                                              &options->padding_color);
 }
 
-void
-gimp_display_shell_set_fullscreen (GimpDisplayShell *shell,
-                                   gboolean          fullscreen)
-{
-  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-
-  if (fullscreen != gimp_display_shell_get_fullscreen (shell))
-    {
-      if (fullscreen)
-        gtk_window_fullscreen (GTK_WINDOW (shell));
-      else
-        gtk_window_unfullscreen (GTK_WINDOW (shell));
-    }
-}
-
-gboolean
-gimp_display_shell_get_fullscreen (const GimpDisplayShell *shell)
-{
-  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
-
-  return (shell->window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
-}
-
 void
 gimp_display_shell_set_show_menubar (GimpDisplayShell *shell,
                                      gboolean          show)
 {
   GimpDisplayOptions *options;
+  GimpImageWindow    *window;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
+  window  = gimp_display_shell_get_window (shell);
 
   g_object_set (options, "show-menubar", show, NULL);
 
-  if (shell->menubar)
+  if (window && gimp_image_window_get_active_shell (window) == shell)
     {
-      if (show)
-        gtk_widget_show (shell->menubar);
-      else
-        gtk_widget_hide (shell->menubar);
+      gimp_image_window_keep_canvas_pos (gimp_display_shell_get_window (shell));
+      gimp_image_window_set_show_menubar (window, show);
     }
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-menubar", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-menubar", show);
+  appearance_set_action_active (shell, "view-show-menubar", show);
 }
 
 gboolean
-gimp_display_shell_get_show_menubar (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_menubar (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_menubar;
-}
-
-void
-gimp_display_shell_set_show_rulers (GimpDisplayShell *shell,
-                                    gboolean          show)
-{
-  GimpDisplayOptions *options;
-  GtkTable           *table;
-
-  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-
-  options = GET_OPTIONS (shell);
-
-  g_object_set (options, "show-rulers", show, NULL);
-
-  table = GTK_TABLE (gtk_widget_get_parent (GTK_WIDGET (shell->canvas)));
-
-  if (show)
-    {
-      gtk_widget_show (shell->origin);
-      gtk_widget_show (shell->hrule);
-      gtk_widget_show (shell->vrule);
-
-      gtk_table_set_col_spacing (table, 0, 1);
-      gtk_table_set_row_spacing (table, 0, 1);
-    }
-  else
-    {
-      gtk_widget_hide (shell->origin);
-      gtk_widget_hide (shell->hrule);
-      gtk_widget_hide (shell->vrule);
-
-      gtk_table_set_col_spacing (table, 0, 0);
-      gtk_table_set_row_spacing (table, 0, 0);
-    }
-
-  SET_ACTIVE (shell->menubar_manager, "view-show-rulers", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-rulers", show);
-}
-
-gboolean
-gimp_display_shell_get_show_rulers (const GimpDisplayShell *shell)
-{
-  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
-
-  return GET_OPTIONS (shell)->show_rulers;
-}
-
-void
-gimp_display_shell_set_show_scrollbars (GimpDisplayShell *shell,
-                                        gboolean          show)
-{
-  GimpDisplayOptions *options;
-  GtkWidget          *parent;
-  GtkBox             *hbox;
-  GtkBox             *vbox;
-
-  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-
-  options = GET_OPTIONS (shell);
-
-  g_object_set (options, "show-scrollbars", show, NULL);
-
-  parent = gtk_widget_get_parent (shell->vsb);
-  hbox   = GTK_BOX (gtk_widget_get_parent (parent));
-
-  parent = gtk_widget_get_parent (shell->hsb);
-  vbox   = GTK_BOX (gtk_widget_get_parent (parent));
-
-  if (show)
-    {
-      gtk_widget_show (shell->nav_ebox);
-      gtk_widget_show (shell->hsb);
-      gtk_widget_show (shell->vsb);
-      gtk_widget_show (shell->quick_mask_button);
-      gtk_widget_show (shell->zoom_button);
-
-      gtk_box_set_spacing (hbox, 1);
-      gtk_box_set_spacing (vbox, 1);
-    }
-  else
-    {
-      gtk_widget_hide (shell->nav_ebox);
-      gtk_widget_hide (shell->hsb);
-      gtk_widget_hide (shell->vsb);
-      gtk_widget_hide (shell->quick_mask_button);
-      gtk_widget_hide (shell->zoom_button);
-
-      gtk_box_set_spacing (hbox, 0);
-      gtk_box_set_spacing (vbox, 0);
-    }
-
-  SET_ACTIVE (shell->menubar_manager, "view-show-scrollbars", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-scrollbars", show);
-}
-
-gboolean
-gimp_display_shell_get_show_scrollbars (const GimpDisplayShell *shell)
-{
-  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
-
-  return GET_OPTIONS (shell)->show_scrollbars;
+  return appearance_get_options (shell)->show_menubar;
 }
 
 void
@@ -281,24 +147,80 @@ gimp_display_shell_set_show_statusbar (GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-statusbar", show, NULL);
 
+  gimp_image_window_keep_canvas_pos (gimp_display_shell_get_window (shell));
   gimp_statusbar_set_visible (GIMP_STATUSBAR (shell->statusbar), show);
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-statusbar", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-statusbar", show);
+  appearance_set_action_active (shell, "view-show-statusbar", show);
 }
 
 gboolean
-gimp_display_shell_get_show_statusbar (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_statusbar (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_statusbar;
+  return appearance_get_options (shell)->show_statusbar;
+}
+
+void
+gimp_display_shell_set_show_rulers (GimpDisplayShell *shell,
+                                    gboolean          show)
+{
+  GimpDisplayOptions *options;
+
+  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+
+  options = appearance_get_options (shell);
+
+  g_object_set (options, "show-rulers", show, NULL);
+
+  gimp_image_window_keep_canvas_pos (gimp_display_shell_get_window (shell));
+  gtk_widget_set_visible (shell->origin, show);
+  gtk_widget_set_visible (shell->hrule, show);
+  gtk_widget_set_visible (shell->vrule, show);
+
+  appearance_set_action_active (shell, "view-show-rulers", show);
+}
+
+gboolean
+gimp_display_shell_get_show_rulers (GimpDisplayShell *shell)
+{
+  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
+
+  return appearance_get_options (shell)->show_rulers;
+}
+
+void
+gimp_display_shell_set_show_scrollbars (GimpDisplayShell *shell,
+                                        gboolean          show)
+{
+  GimpDisplayOptions *options;
+
+  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+
+  options = appearance_get_options (shell);
+
+  g_object_set (options, "show-scrollbars", show, NULL);
+
+  gimp_image_window_keep_canvas_pos (gimp_display_shell_get_window (shell));
+  gtk_widget_set_visible (shell->nav_ebox, show);
+  gtk_widget_set_visible (shell->hsb, show);
+  gtk_widget_set_visible (shell->vsb, show);
+  gtk_widget_set_visible (shell->quick_mask_button, show);
+  gtk_widget_set_visible (shell->zoom_button, show);
+
+  appearance_set_action_active (shell, "view-show-scrollbars", show);
+}
+
+gboolean
+gimp_display_shell_get_show_scrollbars (GimpDisplayShell *shell)
+{
+  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
+
+  return appearance_get_options (shell)->show_scrollbars;
 }
 
 void
@@ -309,24 +231,21 @@ gimp_display_shell_set_show_selection (GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-selection", show, NULL);
 
   gimp_display_shell_selection_set_hidden (shell, ! show);
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-selection", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-selection", show);
+  appearance_set_action_active (shell, "view-show-selection", show);
 }
 
 gboolean
-gimp_display_shell_get_show_selection (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_selection (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_selection;
+  return appearance_get_options (shell)->show_selection;
 }
 
 void
@@ -337,24 +256,21 @@ gimp_display_shell_set_show_layer (GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-layer-boundary", show, NULL);
 
-  gimp_display_shell_selection_layer_set_hidden (shell, ! show);
+  gimp_display_shell_selection_set_layer_hidden (shell, ! show);
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-layer-boundary", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-layer-boundary", show);
+  appearance_set_action_active (shell, "view-show-layer-boundary", show);
 }
 
 gboolean
-gimp_display_shell_get_show_layer (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_layer (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_layer_boundary;
+  return appearance_get_options (shell)->show_layer_boundary;
 }
 
 void
@@ -367,7 +283,7 @@ gimp_display_shell_set_show_transform (GimpDisplayShell *shell,
 }
 
 gboolean
-gimp_display_shell_get_show_transform (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_transform (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
@@ -379,31 +295,30 @@ gimp_display_shell_set_show_guides (GimpDisplayShell *shell,
                                     gboolean          show)
 {
   GimpDisplayOptions *options;
+  GimpImage          *image;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-guides", show, NULL);
 
-  if (shell->display->image &&
-      gimp_image_get_guides (shell->display->image))
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_image_get_guides (image))
     {
       gimp_display_shell_expose_full (shell);
     }
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-guides", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-guides", show);
+  appearance_set_action_active (shell, "view-show-guides", show);
 }
 
 gboolean
-gimp_display_shell_get_show_guides (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_guides (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_guides;
+  return appearance_get_options (shell)->show_guides;
 }
 
 void
@@ -411,31 +326,30 @@ gimp_display_shell_set_show_grid (GimpDisplayShell *shell,
                                   gboolean          show)
 {
   GimpDisplayOptions *options;
+  GimpImage          *image;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-grid", show, NULL);
 
-  if (shell->display->image &&
-      gimp_image_get_grid (shell->display->image))
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_image_get_grid (image))
     {
       gimp_display_shell_expose_full (shell);
     }
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-grid", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-grid", show);
+  appearance_set_action_active (shell, "view-show-grid", show);
 }
 
 gboolean
-gimp_display_shell_get_show_grid (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_grid (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_grid;
+  return appearance_get_options (shell)->show_grid;
 }
 
 void
@@ -443,31 +357,30 @@ gimp_display_shell_set_show_sample_points (GimpDisplayShell *shell,
                                            gboolean          show)
 {
   GimpDisplayOptions *options;
+  GimpImage          *image;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   g_object_set (options, "show-sample-points", show, NULL);
 
-  if (shell->display->image &&
-      gimp_image_get_sample_points (shell->display->image))
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_image_get_sample_points (image))
     {
       gimp_display_shell_expose_full (shell);
     }
 
-  SET_ACTIVE (shell->menubar_manager, "view-show-sample-points", show);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_ACTIVE (shell->popup_manager, "view-show-sample-points", show);
+  appearance_set_action_active (shell, "view-show-sample-points", show);
 }
 
 gboolean
-gimp_display_shell_get_show_sample_points (const GimpDisplayShell *shell)
+gimp_display_shell_get_show_sample_points (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
-  return GET_OPTIONS (shell)->show_sample_points;
+  return appearance_get_options (shell)->show_sample_points;
 }
 
 void
@@ -480,15 +393,12 @@ gimp_display_shell_set_snap_to_grid (GimpDisplayShell *shell,
     {
       shell->snap_to_grid = snap ? TRUE : FALSE;
 
-      SET_ACTIVE (shell->menubar_manager, "view-snap-to-grid", snap);
-
-      if (IS_ACTIVE_DISPLAY (shell))
-        SET_ACTIVE (shell->popup_manager, "view-snap-to-grid", snap);
+      appearance_set_action_active (shell, "view-snap-to-grid", snap);
     }
 }
 
 gboolean
-gimp_display_shell_get_snap_to_grid (const GimpDisplayShell *shell)
+gimp_display_shell_get_snap_to_grid (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
@@ -505,15 +415,12 @@ gimp_display_shell_set_snap_to_guides (GimpDisplayShell *shell,
     {
       shell->snap_to_guides = snap ? TRUE : FALSE;
 
-      SET_ACTIVE (shell->menubar_manager, "view-snap-to-guides", snap);
-
-      if (IS_ACTIVE_DISPLAY (shell))
-        SET_ACTIVE (shell->popup_manager, "view-snap-to-guides", snap);
+      appearance_set_action_active (shell, "view-snap-to-guides", snap);
     }
 }
 
 gboolean
-gimp_display_shell_get_snap_to_guides (const GimpDisplayShell *shell)
+gimp_display_shell_get_snap_to_guides (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
@@ -530,15 +437,12 @@ gimp_display_shell_set_snap_to_canvas (GimpDisplayShell *shell,
     {
       shell->snap_to_canvas = snap ? TRUE : FALSE;
 
-      SET_ACTIVE (shell->menubar_manager, "view-snap-to-canvas", snap);
-
-      if (IS_ACTIVE_DISPLAY (shell))
-        SET_ACTIVE (shell->popup_manager, "view-snap-to-canvas", snap);
+      appearance_set_action_active (shell, "view-snap-to-canvas", snap);
     }
 }
 
 gboolean
-gimp_display_shell_get_snap_to_canvas (const GimpDisplayShell *shell)
+gimp_display_shell_get_snap_to_canvas (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
@@ -555,15 +459,12 @@ gimp_display_shell_set_snap_to_vectors (GimpDisplayShell *shell,
     {
       shell->snap_to_vectors = snap ? TRUE : FALSE;
 
-      SET_ACTIVE (shell->menubar_manager, "view-snap-to-vectors", snap);
-
-      if (IS_ACTIVE_DISPLAY (shell))
-        SET_ACTIVE (shell->popup_manager, "view-snap-to-vectors", snap);
+      appearance_set_action_active (shell, "view-snap-to-vectors", snap);
     }
 }
 
 gboolean
-gimp_display_shell_get_snap_to_vectors (const GimpDisplayShell *shell)
+gimp_display_shell_get_snap_to_vectors (GimpDisplayShell *shell)
 {
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
 
@@ -581,7 +482,7 @@ gimp_display_shell_set_padding (GimpDisplayShell      *shell,
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (padding_color != NULL);
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
   color   = *padding_color;
 
   switch (padding_mode)
@@ -619,18 +520,14 @@ gimp_display_shell_set_padding (GimpDisplayShell      *shell,
 
   gimp_canvas_set_bg_color (GIMP_CANVAS (shell->canvas), &color);
 
-  SET_COLOR (shell->menubar_manager, "view-padding-color-menu",
-             &options->padding_color);
-
-  if (IS_ACTIVE_DISPLAY (shell))
-    SET_COLOR (shell->popup_manager, "view-padding-color-menu",
-               &options->padding_color);
+  appearance_set_action_color (shell, "view-padding-color-menu",
+                               &options->padding_color);
 
   gimp_display_shell_expose_full (shell);
 }
 
 void
-gimp_display_shell_get_padding (const GimpDisplayShell *shell,
+gimp_display_shell_get_padding (GimpDisplayShell       *shell,
                                 GimpCanvasPaddingMode  *padding_mode,
                                 GimpRGB                *padding_color)
 {
@@ -638,7 +535,7 @@ gimp_display_shell_get_padding (const GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  options = GET_OPTIONS (shell);
+  options = appearance_get_options (shell);
 
   if (padding_mode)
     *padding_mode = options->padding_mode;
@@ -646,3 +543,88 @@ gimp_display_shell_get_padding (const GimpDisplayShell *shell,
   if (padding_color)
     *padding_color = options->padding_color;
 }
+
+
+/*  private functions  */
+
+static GimpDisplayOptions *
+appearance_get_options (GimpDisplayShell *shell)
+{
+  if (gimp_display_get_image (shell->display))
+    {
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+      if (window && gimp_image_window_get_fullscreen (window))
+        return shell->fullscreen_options;
+      else
+        return shell->options;
+    }
+
+  return shell->no_image_options;
+}
+
+static void
+appearance_set_action_active (GimpDisplayShell *shell,
+                              const gchar      *action,
+                              gboolean          active)
+{
+  GimpImageWindow *window = gimp_display_shell_get_window (shell);
+  GimpContext     *context;
+
+  if (window && gimp_image_window_get_active_shell (window) == shell)
+    {
+      GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
+      GimpActionGroup *action_group;
+
+      action_group = gimp_ui_manager_get_action_group (manager, "view");
+
+      if (action_group)
+        gimp_action_group_set_action_active (action_group, action, active);
+    }
+
+  context = gimp_get_user_context (shell->display->gimp);
+
+  if (shell->display == gimp_context_get_display (context))
+    {
+      GimpActionGroup *action_group;
+
+      action_group = gimp_ui_manager_get_action_group (shell->popup_manager,
+                                                       "view");
+
+      if (action_group)
+        gimp_action_group_set_action_active (action_group, action, active);
+    }
+}
+
+static void
+appearance_set_action_color (GimpDisplayShell *shell,
+                             const gchar      *action,
+                             const GimpRGB    *color)
+{
+  GimpImageWindow *window = gimp_display_shell_get_window (shell);
+  GimpContext     *context;
+
+  if (window && gimp_image_window_get_active_shell (window) == shell)
+    {
+      GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
+      GimpActionGroup *action_group;
+
+      action_group = gimp_ui_manager_get_action_group (manager, "view");
+
+      if (action_group)
+        gimp_action_group_set_action_color (action_group, action, color, FALSE);
+    }
+
+  context = gimp_get_user_context (shell->display->gimp);
+
+  if (shell->display == gimp_context_get_display (context))
+    {
+      GimpActionGroup *action_group;
+
+      action_group = gimp_ui_manager_get_action_group (shell->popup_manager,
+                                                       "view");
+
+      if (action_group)
+        gimp_action_group_set_action_color (action_group, action, color, FALSE);
+    }
+}
