@@ -51,6 +51,8 @@
 #include "widgets/gimpcontrollers.h"
 #include "widgets/gimpcontrollerkeyboard.h"
 #include "widgets/gimpcontrollerwheel.h"
+#include "widgets/gimpdeviceinfo.h"
+#include "widgets/gimpdeviceinfo-coords.h"
 #include "widgets/gimpdevices.h"
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpuimanager.h"
@@ -72,6 +74,7 @@
 #include "gimpdisplayshell-selection.h"
 #include "gimpdisplayshell-title.h"
 #include "gimpdisplayshell-transform.h"
+#include "gimpimagewindow.h"
 #include "gimpnavigationeditor.h"
 
 #include "gimp-log.h"
@@ -80,6 +83,7 @@
 
 /*  local function prototypes  */
 
+static void       gimp_display_shell_toggle_hide_docks        (GimpDisplayShell *shell);
 static void       gimp_display_shell_vscrollbar_update        (GtkAdjustment    *adjustment,
                                                                GimpDisplayShell *shell);
 static void       gimp_display_shell_hscrollbar_update        (GtkAdjustment    *adjustment,
@@ -118,10 +122,10 @@ gimp_display_shell_events (GtkWidget        *widget,
   gboolean  set_display = FALSE;
 
   /*  are we in destruction?  */
-  if (! shell->display || ! shell->display->shell)
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
     return TRUE;
 
-  gimp = shell->display->gimp;
+  gimp = gimp_display_get_gimp (shell->display);
 
   switch (event->type)
     {
@@ -230,16 +234,19 @@ gimp_display_shell_canvas_realize (GtkWidget        *canvas,
 {
   GimpCanvasPaddingMode padding_mode;
   GimpRGB               padding_color;
+  GtkAllocation         allocation;
 
-  gtk_widget_grab_focus (shell->canvas);
+  gtk_widget_grab_focus (canvas);
 
   gimp_display_shell_get_padding (shell, &padding_mode, &padding_color);
   gimp_display_shell_set_padding (shell, padding_mode, &padding_color);
 
+  gtk_widget_get_allocation (canvas, &allocation);
+
   gimp_display_shell_title_update (shell);
 
-  shell->disp_width  = canvas->allocation.width;
-  shell->disp_height = canvas->allocation.height;
+  shell->disp_width  = allocation.width;
+  shell->disp_height = allocation.height;
 
   /*  set up the scrollbar observers  */
   g_signal_connect (shell->hsbdata, "value-changed",
@@ -269,7 +276,7 @@ gimp_display_shell_canvas_size_allocate (GtkWidget        *widget,
                                          GimpDisplayShell *shell)
 {
   /*  are we in destruction?  */
-  if (! shell->display || ! shell->display->shell)
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
     return;
 
   if ((shell->disp_width  != allocation->width) ||
@@ -285,7 +292,7 @@ gimp_display_shell_canvas_size_allocate (GtkWidget        *widget,
           gint    offset_x;
           gint    offset_y;
 
-          /* FIXMEEEEE!!! */
+          /* FIXME: The code is a bit of a mess */
 
           /*  multiply the zoom_factor with the ratio of the new and
            *  old canvas diagonals
@@ -360,45 +367,81 @@ gimp_display_shell_canvas_size_allocate (GtkWidget        *widget,
     }
 }
 
+static gboolean
+gimp_display_shell_is_double_buffered (GimpDisplayShell *shell)
+{
+  /*  always double-buffer if there are overlay children or a
+   *  transform preview, or they will flicker badly
+   */
+  if (GIMP_OVERLAY_BOX (shell->canvas)->children ||
+      gimp_display_shell_get_show_transform (shell))
+    return TRUE;
+
+  return FALSE;
+}
+
 gboolean
 gimp_display_shell_canvas_expose (GtkWidget        *widget,
                                   GdkEventExpose   *eevent,
                                   GimpDisplayShell *shell)
 {
   /*  are we in destruction?  */
-  if (! shell->display || ! shell->display->shell)
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
+    return TRUE;
+
+  /*  ignore events on overlays  */
+  if (eevent->window == gtk_widget_get_window (widget))
     {
-      return TRUE;
+      if (gimp_display_get_image (shell->display))
+        {
+          gimp_display_shell_pause (shell);
+
+          if (gimp_display_shell_is_double_buffered (shell))
+            gdk_window_begin_paint_region (eevent->window, eevent->region);
+
+          gimp_display_shell_canvas_expose_image (shell, eevent);
+        }
+      else
+        {
+          gimp_display_shell_canvas_expose_drop_zone (shell, eevent);
+        }
     }
 
-  if (shell->display->image)
-    {
-      gimp_display_shell_canvas_expose_image (shell, eevent);
+  return FALSE;
+}
 
-      /* Return TRUE here to avoid redrawing the image when it gets the
-       * keyboard focus.
-       */
-      return TRUE;
-    }
-  else
-    {
-      gimp_display_shell_canvas_expose_drop_zone (shell, eevent);
+gboolean
+gimp_display_shell_canvas_expose_after (GtkWidget        *widget,
+                                        GdkEventExpose   *eevent,
+                                        GimpDisplayShell *shell)
+{
+  /*  are we in destruction?  */
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
+    return TRUE;
 
-      /* Return FALSE here so that the drag indicator is drawn around
-       * the empty canvas during DND operations.
-       */
-      return FALSE;
+  /*  ignore events on overlays  */
+  if (eevent->window == gtk_widget_get_window (widget))
+    {
+      if (gimp_display_get_image (shell->display))
+        {
+          if (gimp_display_shell_is_double_buffered (shell))
+            gdk_window_end_paint (eevent->window);
+
+          gimp_display_shell_resume (shell);
+        }
     }
+
+  return FALSE;
 }
 
 static void
 gimp_display_shell_check_device_cursor (GimpDisplayShell *shell)
 {
-  GdkDevice *current_device;
+  GimpDeviceInfo *current_device;
 
   current_device = gimp_devices_get_current (shell->display->gimp);
 
-  shell->draw_cursor = ! current_device->has_cursor;
+  shell->draw_cursor = ! gimp_device_info_has_cursor (current_device);
 }
 
 static void
@@ -436,7 +479,7 @@ gimp_display_shell_space_pressed (GimpDisplayShell *shell,
                                   GdkModifierType   state,
                                   guint32           time)
 {
-  Gimp *gimp = shell->display->gimp;
+  Gimp *gimp = gimp_display_get_gimp (shell->display);
 
   if (shell->space_pressed)
     return;
@@ -450,9 +493,9 @@ gimp_display_shell_space_pressed (GimpDisplayShell *shell,
       {
         GimpCoords coords;
 
-        gimp_display_shell_get_device_coords (shell,
-                                              gimp_devices_get_current (gimp),
-                                              &coords);
+        gimp_device_info_get_device_coords (gimp_devices_get_current (gimp),
+                                            gtk_widget_get_window (shell->canvas),
+                                            &coords);
 
         gimp_display_shell_start_scrolling (shell, coords.x, coords.y);
 
@@ -471,7 +514,7 @@ gimp_display_shell_space_pressed (GimpDisplayShell *shell,
           return;
 
         shell->space_shaded_tool =
-          gimp_object_get_name (GIMP_OBJECT (active_tool->tool_info));
+          gimp_object_get_name (active_tool->tool_info);
 
         gimp_context_set_tool (gimp_get_user_context (gimp),
                                gimp_get_tool_info (gimp, "gimp-move-tool"));
@@ -492,7 +535,7 @@ gimp_display_shell_space_released (GimpDisplayShell *shell,
                                    GdkModifierType   state,
                                    guint32           time)
 {
-  Gimp *gimp = shell->display->gimp;
+  Gimp *gimp = gimp_display_get_gimp (shell->display);
 
   if (! shell->space_pressed && ! shell->space_release_pending)
     return;
@@ -529,7 +572,7 @@ gimp_display_shell_update_focus (GimpDisplayShell *shell,
                                  GimpCoords       *image_coords,
                                  GdkModifierType   state)
 {
-  Gimp *gimp = shell->display->gimp;
+  Gimp *gimp = gimp_display_get_gimp (shell->display);
 
   tool_manager_focus_display_active (gimp, shell->display);
   tool_manager_modifier_state_active (gimp, state, shell->display);
@@ -570,7 +613,7 @@ gimp_display_shell_canvas_no_image_events (GtkWidget        *canvas,
         if (kevent->keyval == GDK_Tab ||
             kevent->keyval == GDK_ISO_Left_Tab)
           {
-            gimp_dialog_factories_toggle ();
+            gimp_display_shell_toggle_hide_docks (shell);
             return TRUE;
           }
       }
@@ -601,19 +644,32 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
   gboolean         return_val       = FALSE;
   gboolean         update_sw_cursor = FALSE;
 
-  g_return_val_if_fail (GTK_WIDGET_REALIZED (canvas), FALSE);
+  g_return_val_if_fail (gtk_widget_get_realized (canvas), FALSE);
 
   /*  are we in destruction?  */
-  if (! shell->display || ! shell->display->shell)
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
     return TRUE;
 
   /*  set the active display before doing any other canvas event processing  */
   if (gimp_display_shell_events (canvas, event, shell))
     return TRUE;
 
+  /*  ignore events on overlays, but make sure key events go through
+   *  anyway because they are always originating from the toplevel
+   */
+  if (((GdkEventAny *) event)->window != gtk_widget_get_window (canvas))
+    {
+      if ((event->type != GDK_KEY_PRESS &&
+           event->type != GDK_KEY_RELEASE) ||
+          ! gtk_widget_has_focus (canvas))
+        {
+          return FALSE;
+        }
+    }
+
   display = shell->display;
-  gimp    = display->gimp;
-  image   = display->image;
+  gimp    = gimp_display_get_gimp (display);
+  image   = gimp_display_get_image (display);
 
   if (! image)
     {
@@ -629,12 +685,14 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
       device_changed = TRUE;
     }
 
-  gimp_display_shell_get_event_coords (shell, event,
-                                       gimp_devices_get_current (gimp),
-                                       &display_coords);
-  gimp_display_shell_get_event_state (shell, event,
-                                      gimp_devices_get_current (gimp),
-                                      &state);
+  gimp_device_info_get_event_coords (gimp_devices_get_current (gimp),
+                                     gtk_widget_get_window (canvas),
+                                     event,
+                                     &display_coords);
+  gimp_device_info_get_event_state (gimp_devices_get_current (gimp),
+                                    gtk_widget_get_window (canvas),
+                                    event,
+                                    &state);
   time = gdk_event_get_time (event);
 
   /*  GimpCoords passed to tools are ALWAYS in image coordinates  */
@@ -662,7 +720,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
   /*  If the device (and maybe the tool) has changed, update the new
    *  tool's state
    */
-  if (device_changed && GTK_WIDGET_HAS_FOCUS (canvas))
+  if (device_changed && gtk_widget_has_focus (canvas))
     {
       gimp_display_shell_update_focus (shell, &image_coords, state);
     }
@@ -735,7 +793,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           {
             GIMP_LOG (TOOL_EVENTS, "event (display %p): FOCUS_IN", display);
 
-            GTK_WIDGET_SET_FLAGS (canvas, GTK_HAS_FOCUS);
+            if (G_UNLIKELY (! gtk_widget_has_focus (canvas)))
+              g_warning ("%s: FOCUS_IN but canvas has no focus", G_STRFUNC);
 
             /*  press modifier keys when the canvas gets the focus
              *
@@ -751,7 +810,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           {
             GIMP_LOG (TOOL_EVENTS, "event (display %p): FOCUS_OUT", display);
 
-            GTK_WIDGET_UNSET_FLAGS (canvas, GTK_HAS_FOCUS);
+            if (G_LIKELY (gtk_widget_has_focus (canvas)))
+              g_warning ("%s: FOCUS_OUT but canvas has focus", G_STRFUNC);
 
             /*  reset it here to be prepared for the next
              *  FOCUS_IN / BUTTON_PRESS confusion
@@ -766,11 +826,6 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                                              shell->proximity,
                                              display);
           }
-
-        /*  stop the signal because otherwise gtk+ exposes the whole
-         *  canvas to get the non-existant focus indicator drawn
-         */
-        return_val = TRUE;
       }
       break;
 
@@ -784,11 +839,11 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
          *  canvas immediately, therefore we do this before logging
          *  the BUTTON_PRESS.
          */
-        if (! GTK_WIDGET_HAS_FOCUS (canvas))
+        if (! gtk_widget_has_focus (canvas))
           gtk_widget_grab_focus (canvas);
 
-        GIMP_LOG (TOOL_EVENTS, "event (display %p): BUTTON_PRESS (%d)",
-                  display, bevent->button);
+        GIMP_LOG (TOOL_EVENTS, "event (display %p): BUTTON_PRESS (%d @ %0.0f:%0.0f)",
+                  display, bevent->button, bevent->x, bevent->y);
 
         /*  if the toplevel window didn't have focus, the above
          *  gtk_widget_grab_focus() didn't set the canvas' HAS_FOCUS
@@ -796,7 +851,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
          *
          *  this happens in "click to focus" mode.
          */
-        if (! GTK_WIDGET_HAS_FOCUS (canvas))
+        if (! gtk_widget_has_focus (canvas))
           {
             /*  do the things a FOCUS_IN event would do and set a flag
              *  preventing it from doing the same.
@@ -864,7 +919,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                  *  the wacom driver is known to report crappy hints
                  *  (#6901) --mitch
                  */
-                if (gimp_devices_get_current (gimp) ==
+                if (gimp_devices_get_current (gimp)->device ==
                     gdk_display_get_core_pointer (gdk_display))
                   {
                     event_mask |= GDK_POINTER_MOTION_HINT_MASK;
@@ -981,8 +1036,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
       {
         GdkEventButton *bevent = (GdkEventButton *) event;
 
-        GIMP_LOG (TOOL_EVENTS, "event (display %p): 2BUTTON_PRESS (%d)",
-                  display, bevent->button);
+        GIMP_LOG (TOOL_EVENTS, "event (display %p): 2BUTTON_PRESS (%d @ %0.0f:%0.0f)",
+                  display, bevent->button, bevent->x, bevent->y);
 
         if (gimp->busy)
           return TRUE;
@@ -1009,8 +1064,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
       {
         GdkEventButton *bevent = (GdkEventButton *) event;
 
-        GIMP_LOG (TOOL_EVENTS, "event (display %p): 3BUTTON_PRESS (%d)",
-                  display, bevent->button);
+        GIMP_LOG (TOOL_EVENTS, "event (display %p): 3BUTTON_PRESS (%d @ %0.0f:%0.0f)",
+                  display, bevent->button, bevent->x, bevent->y);
 
         if (gimp->busy)
           return TRUE;
@@ -1037,8 +1092,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
       {
         GdkEventButton *bevent = (GdkEventButton *) event;
 
-        GIMP_LOG (TOOL_EVENTS, "event (display %p): BUTTON_RELEASE (%d)",
-                  display, bevent->button);
+        GIMP_LOG (TOOL_EVENTS, "event (display %p): BUTTON_RELEASE (%d @ %0.0f:%0.0f)",
+                  display, bevent->button, bevent->x, bevent->y);
 
         gimp_display_shell_autoscroll_stop (shell);
 
@@ -1221,7 +1276,8 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
         GdkEventMotion *mevent            = (GdkEventMotion *) event;
         GdkEvent       *compressed_motion = NULL;
 
-        GIMP_LOG (TOOL_EVENTS, "event (display %p): MOTION_NOTIFY", display);
+        GIMP_LOG (TOOL_EVENTS, "event (display %p): MOTION_NOTIFY (%0.0f:%0.0f %d)",
+                  display, mevent->x, mevent->y, mevent->time);
 
         if (gimp->busy)
           return TRUE;
@@ -1238,12 +1294,16 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
 
         if (compressed_motion && ! shell->scrolling)
           {
-            GdkDevice *device = gimp_devices_get_current (gimp);
+            GimpDeviceInfo *device = gimp_devices_get_current (gimp);
 
-            gimp_display_shell_get_event_coords (shell, compressed_motion,
-                                                 device, &display_coords);
-            gimp_display_shell_get_event_state (shell, compressed_motion,
-                                                device, &state);
+            gimp_device_info_get_event_coords (device,
+                                               gtk_widget_get_window (canvas),
+                                               compressed_motion,
+                                               &display_coords);
+            gimp_device_info_get_event_state (device,
+                                              gtk_widget_get_window (canvas),
+                                              compressed_motion,
+                                              &state);
             time = gdk_event_get_time (event);
 
             /*  GimpCoords passed to tools are ALWAYS in image coordinates  */
@@ -1322,23 +1382,26 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                  */
                 if ((gimp_tool_control_get_motion_mode (active_tool->control) ==
                      GIMP_MOTION_MODE_EXACT) &&
+                    shell->display->config->use_event_history &&
                     gdk_device_get_history (mevent->device, mevent->window,
                                             shell->last_read_motion_time + 1,
                                             mevent->time - 1,
                                             &history_events,
                                             &n_history_events))
                   {
-                    gint i;
+                    GimpDeviceInfo *device;
+                    gint            i;
 
-                    tool_manager_control_active (gimp, GIMP_TOOL_ACTION_PAUSE, display);
+                    device = gimp_device_info_get_by_device (mevent->device);
+
+                    tool_manager_control_active (gimp, GIMP_TOOL_ACTION_PAUSE,
+                                                 display);
 
                     for (i = 0; i < n_history_events; i++)
                       {
-
-                        gimp_display_shell_get_time_coords (shell,
-                                                            mevent->device,
-                                                            history_events[i],
-                                                            &display_coords);
+                        gimp_device_info_get_time_coords (device,
+                                                          history_events[i],
+                                                          &display_coords);
 
                         /*  GimpCoords passed to tools are ALWAYS in
                          *  image coordinates
@@ -1374,13 +1437,13 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                          shell->last_read_motion_time = history_events[i]->time;
                       }
 
-                      tool_manager_control_active (gimp, GIMP_TOOL_ACTION_RESUME, display);
+                    tool_manager_control_active (gimp, GIMP_TOOL_ACTION_RESUME,
+                                                 display);
 
                     gdk_device_free_history (history_events, n_history_events);
                   }
                 else
                   {
-
                     /* Early removal of useless events saves CPU time.
                      */
                     if (gimp_display_shell_eval_event (shell,
@@ -1404,22 +1467,22 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
             /* Early removal of useless events saves CPU time.
              * Smoothing is 0.0 here for coasting.
              */
-
             if (gimp_display_shell_eval_event (shell,
                                                &image_coords,
                                                0.0,
                                                time))
               {
-                  /* then update the tool. */
-                  GimpCoords *buf_coords = &g_array_index (shell->event_queue,
-                                                           GimpCoords, 0);
-                  tool_manager_oper_update_active (gimp,
-                                                   buf_coords, state,
-                                                   shell->proximity,
-                                                   display);
-                  /* remove used event */
-                  g_array_remove_index (shell->event_queue, 0);
+                /* then update the tool. */
+                GimpCoords *buf_coords = &g_array_index (shell->event_queue,
+                                                         GimpCoords, 0);
 
+                tool_manager_oper_update_active (gimp,
+                                                 buf_coords, state,
+                                                 shell->proximity,
+                                                 display);
+
+                /* remove used event */
+                g_array_remove_index (shell->event_queue, 0);
               }
 
             gimp_display_shell_push_event_history (shell, &image_coords);
@@ -1527,7 +1590,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                   }
                 else
                   {
-                    gimp_dialog_factories_toggle ();
+                    gimp_display_shell_toggle_hide_docks (shell);
                   }
 
                 return_val = TRUE;
@@ -1595,6 +1658,18 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           {
             tool_manager_focus_display_active (gimp, display);
 
+            if (gimp_tool_control_get_wants_all_key_events (active_tool->control))
+              {
+                if (tool_manager_key_release_active (gimp, kevent, display))
+                  {
+                    /* FIXME: need to do some of the stuff below, like
+                     * calling oper_update()
+                     */
+
+                    return TRUE;
+                  }
+              }
+
             switch (kevent->keyval)
               {
               case GDK_space:
@@ -1708,7 +1783,7 @@ gimp_display_shell_ruler_button_press (GtkWidget        *widget,
   if (display->gimp->busy)
     return TRUE;
 
-  if (! display->image)
+  if (! gimp_display_get_image (display))
     return TRUE;
 
   if (event->type == GDK_BUTTON_PRESS && event->button == 1)
@@ -1750,7 +1825,7 @@ gimp_display_shell_ruler_button_press (GtkWidget        *widget,
 
       if (active_tool)
         {
-          if (! GTK_WIDGET_HAS_FOCUS (shell->canvas))
+          if (! gtk_widget_has_focus (shell->canvas))
             {
               gimp_display_shell_update_focus (shell, NULL, event->state);
 
@@ -1826,14 +1901,22 @@ gimp_display_shell_quick_mask_button_press (GtkWidget        *widget,
                                             GdkEventButton   *bevent,
                                             GimpDisplayShell *shell)
 {
-  if (! shell->display->image)
+  if (! gimp_display_get_image (shell->display))
     return TRUE;
 
   if ((bevent->type == GDK_BUTTON_PRESS) && (bevent->button == 3))
     {
-      gimp_ui_manager_ui_popup (shell->menubar_manager, "/quick-mask-popup",
-                                GTK_WIDGET (shell),
-                                NULL, NULL, NULL, NULL);
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+      if (window)
+        {
+          GimpUIManager *manager = gimp_image_window_get_ui_manager (window);
+
+          gimp_ui_manager_ui_popup (manager,
+                                    "/quick-mask-popup",
+                                    GTK_WIDGET (shell),
+                                    NULL, NULL, NULL, NULL);
+        }
 
       return TRUE;
     }
@@ -1845,13 +1928,14 @@ void
 gimp_display_shell_quick_mask_toggled (GtkWidget        *widget,
                                        GimpDisplayShell *shell)
 {
-  gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+  GimpImage *image  = gimp_display_get_image (shell->display);
+  gboolean   active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-  if (active != gimp_image_get_quick_mask_state (shell->display->image))
+  if (active != gimp_image_get_quick_mask_state (image))
     {
-      gimp_image_set_quick_mask_state (shell->display->image, active);
+      gimp_image_set_quick_mask_state (image, active);
 
-      gimp_image_flush (shell->display->image);
+      gimp_image_flush (image);
     }
 }
 
@@ -1860,7 +1944,7 @@ gimp_display_shell_nav_button_press (GtkWidget        *widget,
                                      GdkEventButton   *bevent,
                                      GimpDisplayShell *shell)
 {
-  if (! shell->display->image)
+  if (! gimp_display_get_image (shell->display))
     return TRUE;
 
   if ((bevent->type == GDK_BUTTON_PRESS) && (bevent->button == 1))
@@ -1873,10 +1957,10 @@ gimp_display_shell_nav_button_press (GtkWidget        *widget,
 
 
 /* Event delay timeout handler & generic event flusher */
+
 gboolean
 gimp_display_shell_flush_event_queue (GimpDisplayShell *shell)
 {
-
   GimpTool *active_tool = tool_manager_get_active (shell->display->gimp);
 
   shell->event_delay = FALSE;
@@ -1904,16 +1988,17 @@ gimp_display_shell_flush_event_queue (GimpDisplayShell *shell)
 
 
 /*  private functions  */
+
 static void
 gimp_display_shell_process_tool_event_queue (GimpDisplayShell *shell,
                                              GdkModifierType   state,
                                              guint32           time)
 {
-  gint i;
-  gint keep = 0;
-  GdkModifierType event_state;
-  GimpCoords keep_event;
-  GimpCoords *buf_coords = NULL;
+  gint             i;
+  gint             keep = 0;
+  GdkModifierType  event_state;
+  GimpCoords       keep_event;
+  GimpCoords      *buf_coords = NULL;
 
   if (shell->event_delay)
     {
@@ -1945,7 +2030,6 @@ gimp_display_shell_process_tool_event_queue (GimpDisplayShell *shell,
                                   time,
                                   event_state,
                                   shell->display);
-
     }
 
   tool_manager_control_active (shell->display->gimp,
@@ -1964,24 +2048,43 @@ gimp_display_shell_process_tool_event_queue (GimpDisplayShell *shell,
     }
 }
 
+static void
+gimp_display_shell_toggle_hide_docks (GimpDisplayShell *shell)
+{
+  GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+  if (window)
+    gimp_ui_manager_activate_action (gimp_image_window_get_ui_manager (window),
+                                     "windows",
+                                     "windows-hide-docks");
+}
+
 static void
 gimp_display_shell_vscrollbar_update (GtkAdjustment    *adjustment,
                                       GimpDisplayShell *shell)
 {
-  gimp_display_shell_scroll (shell,
-                             0,
-                             gtk_adjustment_get_value (adjustment) -
-                             shell->offset_y);
+  /* If we are panning with mouse, scrollbars are to be ignored
+   * or they will cause jitter in motion
+   */
+  if (! shell->scrolling)
+    gimp_display_shell_scroll (shell,
+                               0,
+                               gtk_adjustment_get_value (adjustment) -
+                               shell->offset_y);
 }
 
 static void
 gimp_display_shell_hscrollbar_update (GtkAdjustment    *adjustment,
                                       GimpDisplayShell *shell)
 {
-  gimp_display_shell_scroll (shell,
-                             gtk_adjustment_get_value (adjustment) -
-                             shell->offset_x,
-                             0);
+  /* If we are panning with mouse, scrollbars are to be ignored
+   * or they will cause jitter in motion 
+   */
+  if (! shell->scrolling)
+    gimp_display_shell_scroll (shell,
+                               gtk_adjustment_get_value (adjustment) -
+                               shell->offset_x,
+                               0);
 }
 
 static gboolean
@@ -1998,9 +2101,11 @@ gimp_display_shell_hscrollbar_update_range (GtkRange         *range,
       (scroll == GTK_SCROLL_PAGE_FORWARD))
     return FALSE;
 
+  g_object_freeze_notify (G_OBJECT (shell->hsbdata));
+
   gimp_display_shell_scroll_setup_hscrollbar (shell, value);
 
-  gtk_adjustment_changed (shell->hsbdata);
+  g_object_thaw_notify (G_OBJECT (shell->hsbdata)); /* emits "changed" */
 
   return FALSE;
 }
@@ -2019,9 +2124,11 @@ gimp_display_shell_vscrollbar_update_range (GtkRange         *range,
       (scroll == GTK_SCROLL_PAGE_FORWARD))
     return FALSE;
 
+  g_object_freeze_notify (G_OBJECT (shell->vsbdata));
+
   gimp_display_shell_scroll_setup_vscrollbar (shell, value);
 
-  gtk_adjustment_changed (shell->vsbdata);
+  g_object_thaw_notify (G_OBJECT (shell->vsbdata)); /* emits "changed" */
 
   return FALSE;
 }
@@ -2109,82 +2216,112 @@ static void
 gimp_display_shell_canvas_expose_image (GimpDisplayShell *shell,
                                         GdkEventExpose   *eevent)
 {
-  GdkRegion    *region = NULL;
+  GdkRegion    *clear_region;
+  GdkRegion    *image_region;
+  GdkRectangle  image_rect;
   GdkRectangle *rects;
   gint          n_rects;
   gint          i;
 
-  /*  If the call to gimp_display_shell_pause() would cause a redraw,
-   *  we need to make sure that no XOR drawing happens on areas that
-   *  have already been cleared by the windowing system.
+  /*  first, clear the exposed part of the region that is outside the
+   *  image, which is the exposed region minus the image rectangle
    */
-  if (shell->paused_count == 0)
+
+  clear_region = gdk_region_copy (eevent->region);
+
+  image_rect.x = - shell->offset_x;
+  image_rect.y = - shell->offset_y;
+  gimp_display_shell_draw_get_scaled_image_size (shell,
+                                                 &image_rect.width,
+                                                 &image_rect.height);
+  image_region = gdk_region_rectangle (&image_rect);
+
+  gdk_region_subtract (clear_region, image_region);
+  gdk_region_destroy (image_region);
+
+  if (! gdk_region_empty (clear_region))
     {
-      GdkRectangle  area;
+      gdk_region_get_rectangles (clear_region, &rects, &n_rects);
 
-      area.x      = 0;
-      area.y      = 0;
-      area.width  = shell->disp_width;
-      area.height = shell->disp_height;
+      for (i = 0; i < n_rects; i++)
+        gdk_window_clear_area (gtk_widget_get_window (shell->canvas),
+                               rects[i].x,
+                               rects[i].y,
+                               rects[i].width,
+                               rects[i].height);
 
-      region = gdk_region_rectangle (&area);
-
-      gdk_region_subtract (region, eevent->region);
-
-      gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
-                                   GIMP_CANVAS_STYLE_XOR, region);
-      gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
-                                   GIMP_CANVAS_STYLE_XOR_DASHED, region);
+      g_free (rects);
     }
 
-  gimp_display_shell_pause (shell);
+  /*  then, draw the exposed part of the region that is inside the
+   *  image, which is the exposed region minus the region used for
+   *  clearing above
+   */
 
-  if (region)
+  image_region = gdk_region_copy (eevent->region);
+
+  gdk_region_subtract (image_region, clear_region);
+  gdk_region_destroy (clear_region);
+
+  if (! gdk_region_empty (image_region))
     {
-      gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
-                                   GIMP_CANVAS_STYLE_XOR, NULL);
-      gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
-                                   GIMP_CANVAS_STYLE_XOR_DASHED, NULL);
-      gdk_region_destroy (region);
+      gdk_region_get_rectangles (image_region, &rects, &n_rects);
+
+      for (i = 0; i < n_rects; i++)
+        gimp_display_shell_draw_area (shell,
+                                      rects[i].x,
+                                      rects[i].y,
+                                      rects[i].width,
+                                      rects[i].height);
+
+      g_free (rects);
     }
 
-  gdk_region_get_rectangles (eevent->region, &rects, &n_rects);
+  gdk_region_destroy (image_region);
 
-  for (i = 0; i < n_rects; i++)
-    gimp_display_shell_draw_area (shell,
-                                  rects[i].x,
-                                  rects[i].y,
-                                  rects[i].width,
-                                  rects[i].height);
-
-  g_free (rects);
+  /*  finally, draw all the remaining image window stuff on top
+   */
 
   /* draw the transform tool preview */
   gimp_display_shell_preview_transform (shell);
 
   /* draw the grid */
-  gimp_display_shell_draw_grid (shell, &eevent->area);
+  gimp_display_shell_draw_grid (shell, eevent->region);
 
   /* draw the guides */
-  gimp_display_shell_draw_guides (shell);
+  gimp_display_shell_draw_guides (shell, eevent->region);
 
   /* draw the sample points */
-  gimp_display_shell_draw_sample_points (shell);
+  gimp_display_shell_draw_sample_points (shell, eevent->region);
 
   /* and the cursor (if we have a software cursor) */
   gimp_display_shell_draw_cursor (shell);
 
   /* restart (and recalculate) the selection boundaries */
   gimp_display_shell_selection_control (shell, GIMP_SELECTION_ON);
-
-  gimp_display_shell_resume (shell);
 }
 
 static void
 gimp_display_shell_canvas_expose_drop_zone (GimpDisplayShell *shell,
                                             GdkEventExpose   *eevent)
 {
-  cairo_t *cr;
+  cairo_t      *cr;
+  GdkRectangle *rects;
+  gint          n_rects;
+  gint          i;
+
+  gdk_region_get_rectangles (eevent->region, &rects, &n_rects);
+
+  for (i = 0; i < n_rects; i++)
+    {
+      gdk_window_clear_area (gtk_widget_get_window (shell->canvas),
+                             rects[i].x,
+                             rects[i].y,
+                             rects[i].width,
+                             rects[i].height);
+    }
+
+  g_free (rects);
 
   cr = gdk_cairo_create (gtk_widget_get_window (shell->canvas));
   gdk_cairo_region (cr, eevent->region);
