@@ -43,6 +43,7 @@
 #include "gimpdisplay.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-close.h"
+#include "gimpimagewindow.h"
 
 #include "gimp-intl.h"
 
@@ -72,7 +73,7 @@ gimp_display_shell_close (GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  image = shell->display->image;
+  image = gimp_display_get_image (shell->display);
 
   /*  FIXME: gimp_busy HACK not really appropriate here because we only
    *  want to prevent the busy image and display to be closed.  --Mitch
@@ -84,10 +85,10 @@ gimp_display_shell_close (GimpDisplayShell *shell,
    *  it before nuking it--this only applies if its the last view
    *  to an image canvas.  (a image with disp_count = 1)
    */
-  if (! kill_it                   &&
-      image                       &&
-      image->disp_count == 1      &&
-      gimp_image_is_dirty (image) &&
+  if (! kill_it                                 &&
+      image                                     &&
+      gimp_image_get_display_count (image) == 1 &&
+      gimp_image_is_dirty (image)               &&
       shell->display->config->confirm_on_close)
     {
       /*  If there's a save dialog active for this image, then raise it.
@@ -110,11 +111,17 @@ gimp_display_shell_close (GimpDisplayShell *shell,
     }
   else
     {
-      /* Activate the action instead of simply calling gimp_exit(), so
-       * the quit action's sensitivity is taken into account.
-       */
-      gimp_ui_manager_activate_action (shell->menubar_manager,
-                                       "file", "file-quit");
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+      if (window)
+        {
+          GimpUIManager *manager = gimp_image_window_get_ui_manager (window);
+
+          /* Activate the action instead of simply calling gimp_exit(), so
+           * the quit action's sensitivity is taken into account.
+           */
+          gimp_ui_manager_activate_action (manager, "file", "file-quit");
+        }
     }
 }
 
@@ -141,7 +148,7 @@ gimp_display_shell_close_dialog (GimpDisplayShell *shell,
       return;
     }
 
-  uri = gimp_object_get_name (GIMP_OBJECT (image));
+  uri = gimp_object_get_name (image);
 
   title = g_strdup_printf (_("Close %s"), gimp_image_get_display_name (image));
 
@@ -211,7 +218,7 @@ gimp_display_shell_close_name_changed (GimpImage      *image,
 {
   GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (box));
 
-  if (window)
+  if (GTK_IS_WINDOW (window))
     {
       gchar *title = g_strdup_printf (_("Close %s"),
 				      gimp_image_get_display_name (image));
@@ -230,14 +237,15 @@ gimp_display_shell_close_name_changed (GimpImage      *image,
 static gboolean
 gimp_display_shell_close_time_changed (GimpMessageBox *box)
 {
-  GimpImage *image  = g_object_get_data (G_OBJECT (box), "gimp-image");
+  GimpImage *image      = g_object_get_data (G_OBJECT (box), "gimp-image");
+  gint       dirty_time = gimp_image_get_dirty_time (image);
 
-  if (image->dirty_time)
+  if (dirty_time)
     {
       gint hours   = 0;
       gint minutes = 0;
 
-      gimp_time_since (image->dirty_time, &hours, &minutes);
+      gimp_time_since (dirty_time, &hours, &minutes);
 
       if (hours > 0)
         {
@@ -295,8 +303,19 @@ gimp_display_shell_close_response (GtkWidget        *widget,
       break;
 
     case RESPONSE_SAVE:
-      gimp_ui_manager_activate_action (shell->menubar_manager,
-                                       "file", "file-save-and-close");
+      {
+        GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+        if (window)
+          {
+            GimpUIManager *manager = gimp_image_window_get_ui_manager (window);
+
+            /* FIXME image window: set this display active */
+
+            gimp_ui_manager_activate_action (manager,
+                                             "file", "file-save-and-close");
+          }
+      }
       break;
 
     default:
