@@ -35,6 +35,7 @@
 #include "gimpcursorview.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-cursor.h"
+#include "gimpdisplayshell-expose.h"
 #include "gimpdisplayshell-transform.h"
 #include "gimpstatusbar.h"
 
@@ -124,14 +125,14 @@ gimp_display_shell_update_cursor (GimpDisplayShell    *shell,
                                   gdouble              image_x,
                                   gdouble              image_y)
 {
-  GimpDialogFactory *factory;
+  GimpStatusbar     *statusbar;
   GimpSessionInfo   *session_info;
   GimpImage         *image;
   gboolean           new_cursor;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  image = shell->display->image;
+  image = gimp_display_get_image (shell->display);
 
   new_cursor = (shell->draw_cursor &&
                 shell->proximity   &&
@@ -159,17 +160,17 @@ gimp_display_shell_update_cursor (GimpDisplayShell    *shell,
   /*  use the passed image_coords for the statusbar because they are
    *  possibly snapped...
    */
-  gimp_statusbar_update_cursor (GIMP_STATUSBAR (shell->statusbar),
-                                precision, image_x, image_y);
+  statusbar = gimp_display_shell_get_statusbar (shell);
 
-  factory = gimp_dialog_factory_from_name ("dock");
-  session_info = gimp_dialog_factory_find_session_info (factory,
+  gimp_statusbar_update_cursor (statusbar, precision, image_x, image_y);
+
+  session_info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                         "gimp-cursor-view");
-  if (session_info && session_info->widget)
+  if (session_info && gimp_session_info_get_widget (session_info))
     {
       GtkWidget *cursor_view;
 
-      cursor_view = gtk_bin_get_child (GTK_BIN (session_info->widget));
+      cursor_view = gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (session_info)));
 
       if (cursor_view)
         {
@@ -182,7 +183,7 @@ gimp_display_shell_update_cursor (GimpDisplayShell    *shell,
                                                &t_x, &t_y, FALSE, FALSE);
 
           gimp_cursor_view_update_cursor (GIMP_CURSOR_VIEW (cursor_view),
-                                          shell->display->image, shell->unit,
+                                          image, shell->unit,
                                           t_x, t_y);
         }
     }
@@ -191,21 +192,22 @@ gimp_display_shell_update_cursor (GimpDisplayShell    *shell,
 void
 gimp_display_shell_clear_cursor (GimpDisplayShell *shell)
 {
-  GimpDialogFactory *factory;
+  GimpStatusbar     *statusbar;
   GimpSessionInfo   *session_info;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  gimp_statusbar_clear_cursor (GIMP_STATUSBAR (shell->statusbar));
+  statusbar = gimp_display_shell_get_statusbar (shell);
 
-  factory = gimp_dialog_factory_from_name ("dock");
-  session_info = gimp_dialog_factory_find_session_info (factory,
+  gimp_statusbar_clear_cursor (statusbar);
+
+  session_info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                         "gimp-cursor-view");
-  if (session_info && session_info->widget)
+  if (session_info && gimp_session_info_get_widget (session_info))
     {
       GtkWidget *cursor_view;
 
-      cursor_view = gtk_bin_get_child (GTK_BIN (session_info->widget));
+      cursor_view = gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (session_info)));
 
       if (cursor_view)
         gimp_cursor_view_clear_cursor (GIMP_CURSOR_VIEW (cursor_view));
@@ -230,7 +232,7 @@ gimp_display_shell_real_set_cursor (GimpDisplayShell   *shell,
     {
       shell->current_cursor = cursor_type;
 
-      if (GTK_WIDGET_DRAWABLE (shell->canvas))
+      if (gtk_widget_is_drawable (shell->canvas))
         gdk_window_set_cursor (gtk_widget_get_window (shell->canvas), NULL);
 
       return;
