@@ -33,8 +33,8 @@
 #include "core/gimpcontext.h"
 #include "core/gimpdrawable-bucket-fill.h"
 #include "core/gimpimage.h"
-#include "core/gimpimage-colormap.h"
 #include "core/gimpimage-merge.h"
+#include "core/gimpimage-new.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimplayer.h"
 #include "core/gimplayermask.h"
@@ -182,7 +182,7 @@ static void
 gimp_display_shell_dnd_flush (GimpDisplayShell *shell,
                               GimpImage        *image)
 {
-  gtk_window_present (GTK_WINDOW (shell));
+  gimp_display_shell_present (shell);
 
   gimp_image_flush (image);
 
@@ -198,10 +198,9 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
                                   gpointer      data)
 {
   GimpDisplayShell *shell     = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image     = shell->display->image;
+  GimpImage        *image     = gimp_display_get_image (shell->display);
   GType             new_type;
   GimpItem         *new_item;
-  gboolean          new_image = FALSE;
 
   GIMP_LOG (DND, NULL);
 
@@ -210,34 +209,12 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
 
   if (! image)
     {
-      GimpImage         *src_image = gimp_item_get_image (GIMP_ITEM (viewable));
-      GimpDrawable      *drawable  = GIMP_DRAWABLE (viewable);
-      GimpImageBaseType  type;
-      gdouble            xres;
-      gdouble            yres;
-
-      type = GIMP_IMAGE_TYPE_BASE_TYPE (gimp_drawable_type (drawable));
-
-      image = gimp_create_image (shell->display->gimp,
-                                 gimp_item_get_width  (GIMP_ITEM (viewable)),
-                                 gimp_item_get_height (GIMP_ITEM (viewable)),
-                                 type, TRUE);
-      gimp_image_undo_disable (image);
-
-      if (type == GIMP_INDEXED)
-        gimp_image_set_colormap (image,
-                                 gimp_image_get_colormap (src_image),
-                                 gimp_image_get_colormap_size (src_image),
-                                 FALSE);
-
-      gimp_image_get_resolution (src_image, &xres, &yres);
-      gimp_image_set_resolution (image, xres, yres);
-      gimp_image_set_unit (image, gimp_image_get_unit (src_image));
-
-      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0);
+      image = gimp_image_new_from_drawable (shell->display->gimp,
+                                            GIMP_DRAWABLE (viewable));
+      gimp_create_display (shell->display->gimp, image, GIMP_UNIT_PIXEL, 1.0);
       g_object_unref (image);
 
-      new_image = TRUE;
+      return;
     }
 
   if (GIMP_IS_LAYER (viewable))
@@ -254,8 +231,7 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
       gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_PASTE,
                                    _("Drop New Layer"));
 
-      if (! new_image)
-        gimp_display_shell_dnd_position_item (shell, new_item);
+      gimp_display_shell_dnd_position_item (shell, new_item);
 
       gimp_item_set_visible (new_item, TRUE, FALSE);
       gimp_item_set_linked (new_item, FALSE, FALSE);
@@ -267,9 +243,6 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
 
       gimp_display_shell_dnd_flush (shell, image);
     }
-
-  if (new_image)
-    gimp_image_undo_enable (image);
 }
 
 static void
@@ -280,7 +253,7 @@ gimp_display_shell_drop_vectors (GtkWidget    *widget,
                                  gpointer      data)
 {
   GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image = shell->display->image;
+  GimpImage        *image = gimp_display_get_image (shell->display);
   GimpItem         *new_item;
 
   GIMP_LOG (DND, NULL);
@@ -319,7 +292,7 @@ gimp_display_shell_drop_svg (GtkWidget     *widget,
                              gpointer       data)
 {
   GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image = shell->display->image;
+  GimpImage        *image = gimp_display_get_image (shell->display);
   GError           *error  = NULL;
 
   GIMP_LOG (DND, NULL);
@@ -353,7 +326,7 @@ gimp_display_shell_dnd_bucket_fill (GimpDisplayShell   *shell,
                                     const GimpRGB      *color,
                                     GimpPattern        *pattern)
 {
-  GimpImage    *image = shell->display->image;
+  GimpImage    *image = gimp_display_get_image (shell->display);
   GimpDrawable *drawable;
 
   if (shell->display->gimp->busy)
@@ -367,6 +340,22 @@ gimp_display_shell_dnd_bucket_fill (GimpDisplayShell   *shell,
   if (! drawable)
     return;
 
+  if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+    {
+      gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                            GIMP_MESSAGE_ERROR,
+                            _("Cannot modify the pixels of layer groups."));
+      return;
+    }
+
+  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+    {
+      gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                            GIMP_MESSAGE_ERROR,
+                            _("The active layer's pixels are locked."));
+      return;
+    }
+
   /* FIXME: there should be a virtual method for this that the
    *        GimpTextLayer can override.
    */
@@ -429,7 +418,8 @@ gimp_display_shell_drop_buffer (GtkWidget    *widget,
                                 gpointer      data)
 {
   GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image = shell->display->image;
+  GimpImage        *image = gimp_display_get_image (shell->display);
+  GimpDrawable     *drawable;
   GimpBuffer       *buffer;
   gint              x, y, width, height;
 
@@ -441,14 +431,34 @@ gimp_display_shell_drop_buffer (GtkWidget    *widget,
   if (! image)
     return;
 
+  drawable = gimp_image_get_active_drawable (image);
+
+  if (drawable)
+    {
+      if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+        {
+          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                                GIMP_MESSAGE_ERROR,
+                                _("Cannot modify the pixels of layer groups."));
+          return;
+        }
+
+      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+        {
+          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                                GIMP_MESSAGE_ERROR,
+                                _("The active layer's pixels are locked."));
+          return;
+        }
+    }
+
   buffer = GIMP_BUFFER (viewable);
 
   gimp_display_shell_untransform_viewport (shell, &x, &y, &width, &height);
 
   /* FIXME: popup a menu for selecting "Paste Into" */
 
-  gimp_edit_paste (image, gimp_image_get_active_drawable (image),
-                   buffer, FALSE,
+  gimp_edit_paste (image, drawable, buffer, FALSE,
                    x, y, width, height);
 
   gimp_display_shell_dnd_flush (shell, image);
@@ -462,14 +472,14 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
                                   gpointer   data)
 {
   GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image   = shell->display->image;
+  GimpImage        *image   = gimp_display_get_image (shell->display);
   GimpContext      *context = gimp_get_user_context (shell->display->gimp);
   GList            *list;
   gboolean          open_as_layers;
 
   GIMP_LOG (DND, NULL);
 
-  open_as_layers = (shell->display->image != NULL);
+  open_as_layers = (image != NULL);
 
   for (list = uri_list; list; list = g_list_next (list))
     {
@@ -508,7 +518,7 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
               warn = TRUE;
             }
         }
-      else if (shell->display->image)
+      else if (gimp_display_get_image (shell->display))
         {
           /*  open any subsequent images in a new display  */
           GimpImage *new_image;
@@ -560,7 +570,7 @@ gimp_display_shell_drop_component (GtkWidget       *widget,
                                    gpointer         data)
 {
   GimpDisplayShell *shell      = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *dest_image = shell->display->image;
+  GimpImage        *dest_image = gimp_display_get_image (shell->display);
   GimpChannel      *channel;
   GimpItem         *new_item;
   const gchar      *desc;
@@ -610,16 +620,25 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
                                 gpointer   data)
 {
   GimpDisplayShell *shell     = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image     = shell->display->image;
+  GimpImage        *image     = gimp_display_get_image (shell->display);
   GimpLayer        *new_layer;
   GimpImageType     image_type;
-  gboolean          new_image = FALSE;
 
   GIMP_LOG (DND, NULL);
 
   if (shell->display->gimp->busy)
     return;
 
+  if (! image)
+    {
+      image = gimp_image_new_from_pixbuf (shell->display->gimp, pixbuf,
+                                          _("Dropped Buffer"));
+      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0);
+      g_object_unref (image);
+
+      return;
+    }
+
   switch (gdk_pixbuf_get_n_channels (pixbuf))
     {
     case 1: image_type = GIMP_GRAY_IMAGE;  break;
@@ -633,22 +652,6 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
       break;
     }
 
-  if (! image)
-    {
-      image = gimp_create_image (shell->display->gimp,
-                                 gdk_pixbuf_get_width (pixbuf),
-                                 gdk_pixbuf_get_height (pixbuf),
-                                 GIMP_IMAGE_TYPE_BASE_TYPE (image_type),
-                                 FALSE);
-
-      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0);
-      g_object_unref (image);
-
-      gimp_image_undo_disable (image);
-
-      new_image = TRUE;
-    }
-
   new_layer =
     gimp_layer_new_from_pixbuf (pixbuf, image, image_type,
                                 _("Dropped Buffer"),
@@ -663,8 +666,7 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
       gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_PASTE,
                                    _("Drop New Layer"));
 
-      if (! new_image)
-        gimp_display_shell_dnd_position_item (shell, new_item);
+      gimp_display_shell_dnd_position_item (shell, new_item);
 
       gimp_image_add_layer (image, new_layer,
                             GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
@@ -673,7 +675,4 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
 
       gimp_display_shell_dnd_flush (shell, image);
     }
-
-  if (new_image)
-    gimp_image_undo_enable (image);
 }
