@@ -32,6 +32,7 @@
 #include "core/gimpgrid.h"
 #include "core/gimpguide.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-grid.h"
 #include "core/gimpimage-guides.h"
 #include "core/gimpimage-sample-points.h"
 #include "core/gimpprojection.h"
@@ -74,9 +75,9 @@ static GdkGC * gimp_display_shell_get_pen_gc  (GimpDisplayShell *shell,
  *
  **/
 void
-gimp_display_shell_draw_get_scaled_image_size (const GimpDisplayShell *shell,
-                                               gint                   *w,
-                                               gint                   *h)
+gimp_display_shell_draw_get_scaled_image_size (GimpDisplayShell *shell,
+                                               gint             *w,
+                                               gint             *h)
 {
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
@@ -95,11 +96,12 @@ gimp_display_shell_draw_get_scaled_image_size (const GimpDisplayShell *shell,
  *
  **/
 void
-gimp_display_shell_draw_get_scaled_image_size_for_scale (const GimpDisplayShell *shell,
-                                                         gdouble                 scale,
-                                                         gint                   *w,
-                                                         gint                   *h)
+gimp_display_shell_draw_get_scaled_image_size_for_scale (GimpDisplayShell *shell,
+                                                         gdouble           scale,
+                                                         gint             *w,
+                                                         gint             *h)
 {
+  GimpImage      *image;
   GimpProjection *proj;
   TileManager    *tiles;
   gdouble         scale_x;
@@ -109,9 +111,12 @@ gimp_display_shell_draw_get_scaled_image_size_for_scale (const GimpDisplayShell
   gint            level_height;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-  g_return_if_fail (GIMP_IS_IMAGE (shell->display->image));
 
-  proj = gimp_image_get_projection (shell->display->image);
+  image = gimp_display_get_image (shell->display);
+
+  g_return_if_fail (GIMP_IS_IMAGE (image));
+
+  proj = gimp_image_get_projection (image);
 
   gimp_display_shell_calculate_scale_x_and_y (shell, scale, &scale_x, &scale_y);
 
@@ -127,9 +132,10 @@ gimp_display_shell_draw_get_scaled_image_size_for_scale (const GimpDisplayShell
 }
 
 void
-gimp_display_shell_draw_guide (const GimpDisplayShell *shell,
-                               GimpGuide              *guide,
-                               gboolean                active)
+gimp_display_shell_draw_guide (GimpDisplayShell   *shell,
+                               GimpGuide          *guide,
+                               const GdkRectangle *area,
+                               gboolean            active)
 {
   gint  position;
   gint  x1, y1, x2, y2;
@@ -152,11 +158,19 @@ gimp_display_shell_draw_guide (const GimpDisplayShell *shell,
     {
     case GIMP_ORIENTATION_HORIZONTAL:
       gimp_display_shell_transform_xy (shell, 0, position, &x, &y, FALSE);
+      if (area && (y < area->y || y >= area->y + area->height))
+        return;
+      if (y < y1 || y >= y2)
+        return;
       y1 = y2 = y;
       break;
 
     case GIMP_ORIENTATION_VERTICAL:
       gimp_display_shell_transform_xy (shell, position, 0, &x, &y, FALSE);
+      if (area && (x < area->x || x >= area->x + area->width))
+        return;
+      if (x < x1 || x >= x2)
+        return;
       x1 = x2 = x;
       break;
 
@@ -171,58 +185,86 @@ gimp_display_shell_draw_guide (const GimpDisplayShell *shell,
 }
 
 void
-gimp_display_shell_draw_guides (const GimpDisplayShell *shell)
+gimp_display_shell_draw_guides (GimpDisplayShell *shell,
+                                const GdkRegion  *region)
 {
+  GimpImage *image;
+
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+  g_return_if_fail (region != NULL);
 
-  if (shell->display->image &&
-      gimp_display_shell_get_show_guides (shell))
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_display_shell_get_show_guides (shell))
     {
-      GList *list;
+      GdkRectangle  area;
+      GList        *list;
 
-      for (list = gimp_image_get_guides (shell->display->image);
+      if (region)
+        {
+          gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
+                                       GIMP_CANVAS_STYLE_GUIDE_NORMAL,
+                                       region);
+          gdk_region_get_clipbox (region, &area);
+        }
+
+      for (list = gimp_image_get_guides (image);
            list;
            list = g_list_next (list))
         {
-          gimp_display_shell_draw_guide (shell, list->data, FALSE);
+          gimp_display_shell_draw_guide (shell, list->data,
+                                         region ? &area : NULL,
+                                         FALSE);
         }
+
+      if (region)
+        gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
+                                     GIMP_CANVAS_STYLE_GUIDE_NORMAL,
+                                     NULL);
     }
 }
 
 void
-gimp_display_shell_draw_grid (GimpDisplayShell   *shell,
-                              const GdkRectangle *area)
+gimp_display_shell_draw_grid (GimpDisplayShell *shell,
+                              const GdkRegion  *region)
 {
-  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-  g_return_if_fail (area != NULL);
+  GimpImage *image;
 
-  if (shell->display->image &&
-      gimp_display_shell_get_show_grid (shell))
+  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+  g_return_if_fail (region != NULL);
+
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_display_shell_get_show_grid (shell))
     {
-      GimpGrid   *grid;
-      GimpCanvas *canvas;
-      gdouble     x, y;
-      gint        x0, x1, x2, x3;
-      gint        y0, y1, y2, y3;
-      gint        x_real, y_real;
-      gdouble     x_offset, y_offset;
-      gint        width, height;
+      GimpCanvas   *canvas = GIMP_CANVAS (shell->canvas);
+      GimpGrid     *grid;
+      GdkRectangle  area;
+      GdkGC        *grid_gc;
+      gdouble       x, y;
+      gint          x0, x1, x2, x3;
+      gint          y0, y1, y2, y3;
+      gint          x_real, y_real;
+      gdouble       x_offset, y_offset;
+      gint          width, height;
 
 #define CROSSHAIR 2
 
-      grid = GIMP_GRID (shell->display->image->grid);
+      grid = gimp_image_get_grid (image);
       if (! grid)
         return;
 
       g_return_if_fail (grid->xspacing > 0 && grid->yspacing > 0);
 
-      x1 = area->x;
-      y1 = area->y;
-      x2 = area->x + area->width;
-      y2 = area->y + area->height;
+      gdk_region_get_clipbox (region, &area);
 
-      width  = gimp_image_get_width  (shell->display->image);
-      height = gimp_image_get_height (shell->display->image);
+      x1 = area.x;
+      y1 = area.y;
+      x2 = area.x + area.width;
+      y2 = area.y + area.height;
+
+      width  = gimp_image_get_width  (image);
+      height = gimp_image_get_height (image);
 
       x_offset = grid->xoffset;
       while (x_offset > 0)
@@ -232,10 +274,10 @@ gimp_display_shell_draw_grid (GimpDisplayShell   *shell,
       while (y_offset > 0)
         y_offset -= grid->yspacing;
 
-      canvas = GIMP_CANVAS (shell->canvas);
+      grid_gc = gimp_display_shell_get_grid_gc (shell, grid);
 
-      gimp_canvas_set_custom_gc (canvas,
-                                 gimp_display_shell_get_grid_gc (shell, grid));
+      gdk_gc_set_clip_region (grid_gc, region);
+      gimp_canvas_set_custom_gc (canvas, grid_gc);
 
       switch (grid->style)
         {
@@ -355,6 +397,7 @@ gimp_display_shell_draw_grid (GimpDisplayShell   *shell,
         }
 
       gimp_canvas_set_custom_gc (canvas, NULL);
+      gdk_gc_set_clip_region (grid_gc, NULL);
     }
 }
 
@@ -403,15 +446,17 @@ gimp_display_shell_draw_pen (GimpDisplayShell  *shell,
 }
 
 void
-gimp_display_shell_draw_sample_point (const GimpDisplayShell *shell,
-                                      GimpSamplePoint        *sample_point,
-                                      gboolean                active)
+gimp_display_shell_draw_sample_point (GimpDisplayShell   *shell,
+                                      GimpSamplePoint    *sample_point,
+                                      const GdkRectangle *area,
+                                      gboolean            active)
 {
-  GimpCanvasStyle style;
-  gdouble         x, y;
-  gint            x1, x2;
-  gint            y1, y2;
-  gint            w, h;
+  GimpImage       *image;
+  GimpCanvasStyle  style;
+  gdouble          x, y;
+  gint             x1, x2;
+  gint             y1, y2;
+  gint             w, h;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (sample_point != NULL);
@@ -419,6 +464,8 @@ gimp_display_shell_draw_sample_point (const GimpDisplayShell *shell,
   if (sample_point->x < 0)
     return;
 
+  image = gimp_display_get_image (shell->display);
+
   gimp_display_shell_transform_xy_f (shell,
                                      sample_point->x + 0.5,
                                      sample_point->y + 0.5,
@@ -437,6 +484,12 @@ gimp_display_shell_draw_sample_point (const GimpDisplayShell *shell,
       y > h + GIMP_SAMPLE_POINT_DRAW_SIZE)
     return;
 
+  if (area && (x + GIMP_SAMPLE_POINT_DRAW_SIZE <  area->x               ||
+               y + GIMP_SAMPLE_POINT_DRAW_SIZE <  area->y               ||
+               x - GIMP_SAMPLE_POINT_DRAW_SIZE >= area->x + area->width ||
+               y - GIMP_SAMPLE_POINT_DRAW_SIZE >= area->y + area->height))
+    return;
+
   if (active)
     style = GIMP_CANVAS_STYLE_SAMPLE_POINT_ACTIVE;
   else
@@ -464,32 +517,53 @@ gimp_display_shell_draw_sample_point (const GimpDisplayShell *shell,
   gimp_canvas_draw_text (GIMP_CANVAS (shell->canvas), style,
                          x + 2, y + 2,
                          "%d",
-                         g_list_index (gimp_image_get_sample_points (shell->display->image),
+                         g_list_index (gimp_image_get_sample_points (image),
                                        sample_point) + 1);
 }
 
 void
-gimp_display_shell_draw_sample_points (const GimpDisplayShell *shell)
+gimp_display_shell_draw_sample_points (GimpDisplayShell *shell,
+                                       const GdkRegion  *region)
 {
+  GimpImage *image;
+
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+  g_return_if_fail (region != NULL);
 
-  if (shell->display->image &&
-      gimp_display_shell_get_show_sample_points (shell))
+  image = gimp_display_get_image (shell->display);
+
+  if (image && gimp_display_shell_get_show_sample_points (shell))
     {
-      GList *list;
+      GdkRectangle  area;
+      GList        *list;
 
-      for (list = gimp_image_get_sample_points (shell->display->image);
+      if (region)
+        {
+          gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
+                                       GIMP_CANVAS_STYLE_SAMPLE_POINT_NORMAL,
+                                       region);
+          gdk_region_get_clipbox (region, &area);
+        }
+
+      for (list = gimp_image_get_sample_points (image);
            list;
            list = g_list_next (list))
         {
-          gimp_display_shell_draw_sample_point (shell, list->data, FALSE);
+          gimp_display_shell_draw_sample_point (shell, list->data,
+                                                region ? &area : NULL,
+                                                FALSE);
         }
+
+      if (region)
+        gimp_canvas_set_clip_region (GIMP_CANVAS (shell->canvas),
+                                     GIMP_CANVAS_STYLE_SAMPLE_POINT_NORMAL,
+                                     region);
     }
 }
 
 void
-gimp_display_shell_draw_vector (const GimpDisplayShell *shell,
-                                GimpVectors            *vectors)
+gimp_display_shell_draw_vector (GimpDisplayShell *shell,
+                                GimpVectors      *vectors)
 {
   GimpStroke *stroke = NULL;
 
@@ -527,17 +601,20 @@ gimp_display_shell_draw_vector (const GimpDisplayShell *shell,
 }
 
 void
-gimp_display_shell_draw_vectors (const GimpDisplayShell *shell)
+gimp_display_shell_draw_vectors (GimpDisplayShell *shell)
 {
+  GimpImage *image;
+
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  if (shell->display->image &&
-      TRUE /* gimp_display_shell_get_show_vectors (shell) */)
+  image = gimp_display_get_image (shell->display);
+
+  if (image && TRUE /* gimp_display_shell_get_show_vectors (shell) */)
     {
       GList *all_vectors;
       GList *list;
 
-      all_vectors = gimp_image_get_vectors_list (shell->display->image);
+      all_vectors = gimp_image_get_vectors_list (image);
 
       for (list = all_vectors; list; list = list->next)
         {
@@ -552,7 +629,7 @@ gimp_display_shell_draw_vectors (const GimpDisplayShell *shell)
 }
 
 void
-gimp_display_shell_draw_cursor (const GimpDisplayShell *shell)
+gimp_display_shell_draw_cursor (GimpDisplayShell *shell)
 {
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
@@ -562,74 +639,52 @@ gimp_display_shell_draw_cursor (const GimpDisplayShell *shell)
 }
 
 void
-gimp_display_shell_draw_area (const GimpDisplayShell *shell,
-                              gint                    x,
-                              gint                    y,
-                              gint                    w,
-                              gint                    h)
+gimp_display_shell_draw_area (GimpDisplayShell *shell,
+                              gint              x,
+                              gint              y,
+                              gint              w,
+                              gint              h)
 {
-  gint sx, sy;
-  gint sw, sh;
+  GdkRectangle  rect;
+  gint          x2, y2;
+  gint          i, j;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+  g_return_if_fail (gimp_display_get_image (shell->display));
 
-  if (! shell->display->image)
-    return;
+  x2 = x + w;
+  y2 = y + h;
 
-  sx = - shell->offset_x;
-  sy = - shell->offset_y;
-
-  gimp_display_shell_draw_get_scaled_image_size (shell, &sw, &sh);
-
-  /*  check if the passed in area intersects with
-   *  both the display and the image
-   */
-  if (gimp_rectangle_intersect (x, y, w, h,
-                                0, 0, shell->disp_width,  shell->disp_height,
-                                &x, &y, &w, &h)
-      &&
-      gimp_rectangle_intersect (x, y, w, h,
-                                sx, sy, sw, sh,
-                                &x, &y, &w, &h))
+  if (shell->highlight)
     {
-      GdkRectangle  rect;
-      gint          x2, y2;
-      gint          i, j;
+      rect.x      = ceil  (shell->scale_x * shell->highlight->x);
+      rect.y      = ceil  (shell->scale_y * shell->highlight->y);
+      rect.width  = floor (shell->scale_x * shell->highlight->width);
+      rect.height = floor (shell->scale_y * shell->highlight->height);
+    }
 
-      x2 = x + w;
-      y2 = y + h;
-
-      if (shell->highlight)
+  /*  display the image in RENDER_BUF_WIDTH x RENDER_BUF_HEIGHT
+   *  sized chunks
+   */
+  for (i = y; i < y2; i += GIMP_DISPLAY_RENDER_BUF_HEIGHT)
+    {
+      for (j = x; j < x2; j += GIMP_DISPLAY_RENDER_BUF_WIDTH)
         {
-          rect.x      = ceil  (shell->scale_x * shell->highlight->x);
-          rect.y      = ceil  (shell->scale_y * shell->highlight->y);
-          rect.width  = floor (shell->scale_x * shell->highlight->width);
-          rect.height = floor (shell->scale_y * shell->highlight->height);
-        }
+          gint disp_xoffset, disp_yoffset;
+          gint dx, dy;
 
-      /*  display the image in RENDER_BUF_WIDTH x RENDER_BUF_HEIGHT
-       *  sized chunks
-       */
-      for (i = y; i < y2; i += GIMP_DISPLAY_RENDER_BUF_HEIGHT)
-        {
-          for (j = x; j < x2; j += GIMP_DISPLAY_RENDER_BUF_WIDTH)
-            {
-              gint disp_xoffset, disp_yoffset;
-              gint dx, dy;
+          dx = MIN (x2 - j, GIMP_DISPLAY_RENDER_BUF_WIDTH);
+          dy = MIN (y2 - i, GIMP_DISPLAY_RENDER_BUF_HEIGHT);
 
-              dx = MIN (x2 - j, GIMP_DISPLAY_RENDER_BUF_WIDTH);
-              dy = MIN (y2 - i, GIMP_DISPLAY_RENDER_BUF_HEIGHT);
+          gimp_display_shell_scroll_get_disp_offset (shell,
+                                                     &disp_xoffset,
+                                                     &disp_yoffset);
 
-              gimp_display_shell_scroll_get_disp_offset (shell,
-                                                         &disp_xoffset,
-                                                         &disp_yoffset);
-
-              gimp_display_shell_render (shell,
-                                         j - disp_xoffset,
-                                         i - disp_yoffset,
-                                         dx, dy,
-                                         shell->highlight ? &rect : NULL);
-            }
+          gimp_display_shell_render (shell,
+                                     j - disp_xoffset,
+                                     i - disp_yoffset,
+                                     dx, dy,
+                                     shell->highlight ? &rect : NULL);
         }
     }
 }
