@@ -27,27 +27,36 @@
 #include "display-types.h"
 
 #include "config/gimpdisplayconfig.h"
+#include "config/gimpdisplayoptions.h"
 
 #include "core/gimp.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-grid.h"
+#include "core/gimpimage-quick-mask.h"
 #include "core/gimpitem.h"
 #include "core/gimptreehandler.h"
 
+#include "file/file-utils.h"
+
 #include "widgets/gimpwidgets-utils.h"
 
 #include "gimpdisplay.h"
-#include "gimpdisplayoptions.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-appearance.h"
 #include "gimpdisplayshell-callbacks.h"
 #include "gimpdisplayshell-draw.h"
+#include "gimpdisplayshell-expose.h"
 #include "gimpdisplayshell-handlers.h"
 #include "gimpdisplayshell-icon.h"
 #include "gimpdisplayshell-scale.h"
 #include "gimpdisplayshell-scroll.h"
 #include "gimpdisplayshell-selection.h"
 #include "gimpdisplayshell-title.h"
+#include "gimpimagewindow.h"
+#include "gimpstatusbar.h"
+
+#include "gimp-intl.h"
 
 
 /*  local function prototypes  */
@@ -88,6 +97,12 @@ static void   gimp_display_shell_invalidate_preview_handler (GimpImage        *i
                                                              GimpDisplayShell *shell);
 static void   gimp_display_shell_profile_changed_handler    (GimpColorManaged *image,
                                                              GimpDisplayShell *shell);
+static void   gimp_display_shell_saved_handler              (GimpImage        *image,
+                                                             const gchar      *uri,
+                                                             GimpDisplayShell *shell);
+static void   gimp_display_shell_exported_handler           (GimpImage        *image,
+                                                             const gchar      *uri,
+                                                             GimpDisplayShell *shell);
 
 static void   gimp_display_shell_vectors_freeze_handler     (GimpVectors      *vectors,
                                                              GimpDisplayShell *shell);
@@ -130,13 +145,16 @@ static void   gimp_display_shell_quality_notify_handler     (GObject          *c
 void
 gimp_display_shell_connect (GimpDisplayShell *shell)
 {
-  GimpImage *image;
+  GimpImage     *image;
+  GimpContainer *vectors;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (GIMP_IS_DISPLAY (shell->display));
-  g_return_if_fail (GIMP_IS_IMAGE (shell->display->image));
 
-  image = shell->display->image;
+  image   = gimp_display_get_image (shell->display);
+  vectors = gimp_image_get_vectors (image);
+
+  g_return_if_fail (GIMP_IS_IMAGE (image));
 
   g_signal_connect (image, "clean",
                     G_CALLBACK (gimp_display_shell_clean_dirty_handler),
@@ -147,7 +165,7 @@ gimp_display_shell_connect (GimpDisplayShell *shell)
   g_signal_connect (image, "undo-event",
                     G_CALLBACK (gimp_display_shell_undo_event_handler),
                     shell);
-  g_signal_connect (image->grid, "notify",
+  g_signal_connect (gimp_image_get_grid (image), "notify",
                     G_CALLBACK (gimp_display_shell_grid_notify_handler),
                     shell);
   g_signal_connect (image, "name-changed",
@@ -177,24 +195,30 @@ gimp_display_shell_connect (GimpDisplayShell *shell)
   g_signal_connect (image, "profile-changed",
                     G_CALLBACK (gimp_display_shell_profile_changed_handler),
                     shell);
+  g_signal_connect (image, "saved",
+                    G_CALLBACK (gimp_display_shell_saved_handler),
+                    shell);
+  g_signal_connect (image, "exported",
+                    G_CALLBACK (gimp_display_shell_exported_handler),
+                    shell);
 
   shell->vectors_freeze_handler =
-    gimp_tree_handler_connect (image->vectors, "freeze",
+    gimp_tree_handler_connect (vectors, "freeze",
                                G_CALLBACK (gimp_display_shell_vectors_freeze_handler),
                                shell);
   shell->vectors_thaw_handler =
-    gimp_tree_handler_connect (image->vectors, "thaw",
+    gimp_tree_handler_connect (vectors, "thaw",
                                G_CALLBACK (gimp_display_shell_vectors_thaw_handler),
                                shell);
   shell->vectors_visible_handler =
-    gimp_tree_handler_connect (image->vectors, "visibility-changed",
+    gimp_tree_handler_connect (vectors, "visibility-changed",
                                G_CALLBACK (gimp_display_shell_vectors_visible_handler),
                                shell);
 
-  g_signal_connect (image->vectors, "add",
+  g_signal_connect (vectors, "add",
                     G_CALLBACK (gimp_display_shell_vectors_add_handler),
                     shell);
-  g_signal_connect (image->vectors, "remove",
+  g_signal_connect (vectors, "remove",
                     G_CALLBACK (gimp_display_shell_vectors_remove_handler),
                     shell);
 
@@ -266,15 +290,19 @@ gimp_display_shell_connect (GimpDisplayShell *shell)
 void
 gimp_display_shell_disconnect (GimpDisplayShell *shell)
 {
-  GimpImage *image;
+  GimpImage     *image;
+  GimpContainer *vectors;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (GIMP_IS_DISPLAY (shell->display));
-  g_return_if_fail (GIMP_IS_IMAGE (shell->display->image));
 
-  image = shell->display->image;
+  image = gimp_display_get_image (shell->display);
 
-  gimp_display_shell_icon_idle_stop (shell);
+  g_return_if_fail (GIMP_IS_IMAGE (image));
+
+  vectors = gimp_image_get_vectors (image);
+
+  gimp_display_shell_icon_update_stop (shell);
 
   if (shell->grid_gc)
     {
@@ -313,10 +341,10 @@ gimp_display_shell_disconnect (GimpDisplayShell *shell)
                                         gimp_display_shell_check_notify_handler,
                                         shell);
 
-  g_signal_handlers_disconnect_by_func (image->vectors,
+  g_signal_handlers_disconnect_by_func (vectors,
                                         gimp_display_shell_vectors_remove_handler,
                                         shell);
-  g_signal_handlers_disconnect_by_func (image->vectors,
+  g_signal_handlers_disconnect_by_func (vectors,
                                         gimp_display_shell_vectors_add_handler,
                                         shell);
 
@@ -329,6 +357,12 @@ gimp_display_shell_disconnect (GimpDisplayShell *shell)
   gimp_tree_handler_disconnect (shell->vectors_freeze_handler);
   shell->vectors_freeze_handler = NULL;
 
+  g_signal_handlers_disconnect_by_func (image,
+                                        gimp_display_shell_exported_handler,
+                                        shell);
+  g_signal_handlers_disconnect_by_func (image,
+                                        gimp_display_shell_saved_handler,
+                                        shell);
   g_signal_handlers_disconnect_by_func (image,
                                         gimp_display_shell_profile_changed_handler,
                                         shell);
@@ -356,7 +390,7 @@ gimp_display_shell_disconnect (GimpDisplayShell *shell)
   g_signal_handlers_disconnect_by_func (image,
                                         gimp_display_shell_name_changed_handler,
                                         shell);
-  g_signal_handlers_disconnect_by_func (image->grid,
+  g_signal_handlers_disconnect_by_func (gimp_image_get_grid (image),
                                         gimp_display_shell_grid_notify_handler,
                                         shell);
   g_signal_handlers_disconnect_by_func (image,
@@ -451,6 +485,7 @@ gimp_display_shell_quick_mask_changed_handler (GimpImage        *image,
                                                GimpDisplayShell *shell)
 {
   GtkImage *gtk_image;
+  gboolean  quick_mask_state;
 
   gtk_image = GTK_IMAGE (gtk_bin_get_child (GTK_BIN (shell->quick_mask_button)));
 
@@ -458,10 +493,12 @@ gimp_display_shell_quick_mask_changed_handler (GimpImage        *image,
                                    gimp_display_shell_quick_mask_toggled,
                                    shell);
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell->quick_mask_button),
-                                shell->display->image->quick_mask_state);
+  quick_mask_state = gimp_image_get_quick_mask_state (image);
 
-  if (shell->display->image->quick_mask_state)
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell->quick_mask_button),
+                                quick_mask_state);
+
+  if (quick_mask_state)
     gtk_image_set_from_stock (gtk_image, GIMP_STOCK_QUICK_MASK_ON,
                               GTK_ICON_SIZE_MENU);
   else
@@ -499,14 +536,19 @@ gimp_display_shell_size_changed_detailed_handler (GimpImage        *image,
 {
   if (shell->display->config->resize_windows_on_resize)
     {
-      /* If the window is resized just center the image in it when it
-       * has change size
-       */
-      gimp_display_shell_shrink_wrap (shell, FALSE);
+      GimpImageWindow *window = gimp_display_shell_get_window (shell);
+
+      if (window && gimp_image_window_get_active_shell (window) == shell)
+        {
+          /* If the window is resized just center the image in it when it
+           * has change size
+           */
+          gimp_image_window_shrink_wrap (window, FALSE);
+        }
     }
   else
     {
-      GimpImage *image                    = GIMP_IMAGE (shell->display->image);
+      GimpImage *image                    = gimp_display_get_image (shell->display);
       gint       new_width                = gimp_image_get_width  (image);
       gint       new_height               = gimp_image_get_height (image);
       gint       scaled_previous_origin_x = SCALEX (shell, previous_origin_x);
@@ -540,7 +582,7 @@ static void
 gimp_display_shell_invalidate_preview_handler (GimpImage        *image,
                                                GimpDisplayShell *shell)
 {
-  gimp_display_shell_icon_idle_update (shell);
+  gimp_display_shell_icon_update (shell);
 }
 
 static void
@@ -550,6 +592,35 @@ gimp_display_shell_profile_changed_handler (GimpColorManaged *image,
   gimp_color_managed_profile_changed (GIMP_COLOR_MANAGED (shell));
 }
 
+
+static void
+gimp_display_shell_saved_handler (GimpImage        *image,
+                                  const gchar      *uri,
+                                  GimpDisplayShell *shell)
+{
+  GimpStatusbar *statusbar = gimp_display_shell_get_statusbar (shell);
+  gchar         *filename  = file_utils_uri_display_name (uri);
+
+  gimp_statusbar_push_temp (statusbar, GIMP_MESSAGE_INFO,
+                            GTK_STOCK_SAVE, _("Image saved to '%s'"),
+                            filename);
+  g_free (filename);
+}
+
+static void
+gimp_display_shell_exported_handler (GimpImage        *image,
+                                     const gchar      *uri,
+                                     GimpDisplayShell *shell)
+{
+  GimpStatusbar *statusbar = gimp_display_shell_get_statusbar (shell);
+  gchar         *filename  = file_utils_uri_display_name (uri);
+
+  gimp_statusbar_push_temp (statusbar, GIMP_MESSAGE_INFO,
+                            GTK_STOCK_SAVE, _("Image exported to '%s'"),
+                            filename);
+  g_free (filename);
+}
+
 static void
 gimp_display_shell_vectors_freeze_handler (GimpVectors      *vectors,
                                            GimpDisplayShell *shell)
@@ -671,13 +742,19 @@ gimp_display_shell_padding_notify_handler (GObject          *config,
                                            GimpDisplayShell *shell)
 {
   GimpDisplayConfig     *display_config;
+  GimpImageWindow       *window;
   gboolean               fullscreen;
   GimpCanvasPaddingMode  padding_mode;
   GimpRGB                padding_color;
 
   display_config = shell->display->config;
 
-  fullscreen = gimp_display_shell_get_fullscreen (shell);
+  window = gimp_display_shell_get_window (shell);
+
+  if (window)
+    fullscreen = gimp_image_window_get_fullscreen (window);
+  else
+    fullscreen = FALSE;
 
   /*  if the user did not set the padding mode for this display explicitely  */
   if (! shell->fullscreen_options->padding_mode_set)
