@@ -41,7 +41,7 @@
 
 typedef struct
 {
-  GtkWidget *shell;
+  GtkWidget *window;
   GtkWidget *view;
   GtkWidget *label;
 
@@ -77,7 +77,7 @@ gimp_display_shell_layer_select_init (GimpDisplayShell *shell,
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
 
-  image = shell->display->image;
+  image = gimp_display_get_image (shell->display);
 
   layer = gimp_image_get_active_layer (image);
 
@@ -88,12 +88,12 @@ gimp_display_shell_layer_select_init (GimpDisplayShell *shell,
                                    image->gimp->config->layer_preview_size);
   layer_select_advance (layer_select, move);
 
-  gtk_window_set_screen (GTK_WINDOW (layer_select->shell),
+  gtk_window_set_screen (GTK_WINDOW (layer_select->window),
                          gtk_widget_get_screen (GTK_WIDGET (shell)));
 
-  gtk_widget_show (layer_select->shell);
+  gtk_widget_show (layer_select->window);
 
-  gdk_keyboard_grab (gtk_widget_get_window (layer_select->shell), FALSE, time);
+  gdk_keyboard_grab (gtk_widget_get_window (layer_select->window), FALSE, time);
 }
 
 
@@ -115,21 +115,22 @@ layer_select_new (GimpImage *image,
   layer_select->image      = image;
   layer_select->orig_layer = layer;
 
-  layer_select->shell = gtk_window_new (GTK_WINDOW_POPUP);
-  gtk_window_set_role (GTK_WINDOW (layer_select->shell), "gimp-layer-select");
-  gtk_window_set_title (GTK_WINDOW (layer_select->shell), _("Layer Select"));
-  gtk_window_set_position (GTK_WINDOW (layer_select->shell), GTK_WIN_POS_MOUSE);
-  gtk_widget_set_events (layer_select->shell, (GDK_KEY_PRESS_MASK   |
-                                               GDK_KEY_RELEASE_MASK |
-                                               GDK_BUTTON_PRESS_MASK));
+  layer_select->window = gtk_window_new (GTK_WINDOW_POPUP);
+  gtk_window_set_role (GTK_WINDOW (layer_select->window), "gimp-layer-select");
+  gtk_window_set_title (GTK_WINDOW (layer_select->window), _("Layer Select"));
+  gtk_window_set_position (GTK_WINDOW (layer_select->window), GTK_WIN_POS_MOUSE);
+  gtk_widget_set_events (layer_select->window,
+                         GDK_KEY_PRESS_MASK   |
+                         GDK_KEY_RELEASE_MASK |
+                         GDK_BUTTON_PRESS_MASK);
 
-  g_signal_connect (layer_select->shell, "event",
+  g_signal_connect (layer_select->window, "event",
                     G_CALLBACK (layer_select_events),
                     layer_select);
 
   frame1 = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_OUT);
-  gtk_container_add (GTK_CONTAINER (layer_select->shell), frame1);
+  gtk_container_add (GTK_CONTAINER (layer_select->window), frame1);
   gtk_widget_show (frame1);
 
   frame2 = gtk_frame_new (NULL);
@@ -159,7 +160,7 @@ layer_select_new (GimpImage *image,
   gtk_widget_show (alignment);
 
   /*  the layer name label */
-  layer_select->label = gtk_label_new (GIMP_OBJECT (layer)->name);
+  layer_select->label = gtk_label_new (gimp_object_get_name (layer));
   gtk_box_pack_start (GTK_BOX (hbox), layer_select->label, FALSE, FALSE, 0);
   gtk_widget_show (layer_select->label);
 
@@ -170,10 +171,10 @@ static void
 layer_select_destroy (LayerSelect *layer_select,
                       guint32      time)
 {
-  gdk_display_keyboard_ungrab (gtk_widget_get_display (layer_select->shell),
+  gdk_display_keyboard_ungrab (gtk_widget_get_display (layer_select->window),
                                                        time);
 
-  gtk_widget_destroy (layer_select->shell);
+  gtk_widget_destroy (layer_select->window);
 
   if (layer_select->orig_layer !=
       gimp_image_get_active_layer (layer_select->image))
@@ -188,10 +189,11 @@ static void
 layer_select_advance (LayerSelect *layer_select,
                       gint         move)
 {
-  GimpContainer *container;
-  GimpLayer     *current_layer;
-  GimpLayer     *next_layer;
-  gint           index;
+  GimpLayer *active_layer;
+  GimpLayer *next_layer;
+  GList     *layers;
+  gint       n_layers;
+  gint       index;
 
   if (move == 0)
     return;
@@ -200,31 +202,34 @@ layer_select_advance (LayerSelect *layer_select,
   if (gimp_image_get_floating_selection (layer_select->image))
     return;
 
-  current_layer = gimp_image_get_active_layer (layer_select->image);
+  active_layer = gimp_image_get_active_layer (layer_select->image);
 
-  container = gimp_item_get_container (GIMP_ITEM (current_layer));
-  index     = gimp_item_get_index (GIMP_ITEM (current_layer));
+  layers   = gimp_image_get_layer_list (layer_select->image);
+  n_layers = g_list_length (layers);
 
+  index = g_list_index (layers, active_layer);
   index += move;
 
   if (index < 0)
-    index = gimp_container_get_n_children (container) - 1;
-  else if (index >= gimp_container_get_n_children (container))
+    index = n_layers - 1;
+  else if (index >= n_layers)
     index = 0;
 
-  next_layer = GIMP_LAYER (gimp_container_get_child_by_index (container, index));
+  next_layer = g_list_nth_data (layers, index);
 
-  if (next_layer && next_layer != current_layer)
+  g_list_free (layers);
+
+  if (next_layer && next_layer != active_layer)
     {
-      current_layer = gimp_image_set_active_layer (layer_select->image,
-                                                   next_layer);
+      active_layer = gimp_image_set_active_layer (layer_select->image,
+                                                  next_layer);
 
-      if (current_layer)
+      if (active_layer)
         {
           gimp_view_set_viewable (GIMP_VIEW (layer_select->view),
-                                  GIMP_VIEWABLE (current_layer));
+                                  GIMP_VIEWABLE (active_layer));
           gtk_label_set_text (GTK_LABEL (layer_select->label),
-                              GIMP_OBJECT (current_layer)->name);
+                              gimp_object_get_name (active_layer));
         }
     }
 }
