@@ -56,8 +56,6 @@
                                                100% and 200% zoom)
                                              */
 
-#define GIMP_DISPLAY_RENDER_BUF_WIDTH  256
-
 
 typedef struct _RenderInfo  RenderInfo;
 
@@ -65,49 +63,49 @@ typedef void (* RenderFunc) (RenderInfo *info);
 
 struct _RenderInfo
 {
-  const GimpDisplayShell *shell;
-  TileManager            *src_tiles;
-  const guchar           *src;
-  guchar                 *dest;
-  gboolean                src_is_premult;
-  gint                    x, y;
-  gint                    w, h;
-  gdouble                 scalex;
-  gdouble                 scaley;
-  gint                    src_x;
-  gint                    src_y;
-  gint                    dest_bpp;
-  gint                    dest_bpl;
-  gint                    dest_width;
+  GimpDisplayShell *shell;
+  TileManager      *src_tiles;
+  const guchar     *src;
+  guchar           *dest;
+  gboolean          src_is_premult;
+  gint              x, y;
+  gint              w, h;
+  gdouble           scalex;
+  gdouble           scaley;
+  gint              src_x;
+  gint              src_y;
+  gint              dest_bpp;
+  gint              dest_bpl;
+  gint              dest_width;
 
-  gint                    zoom_quality;
+  gint              zoom_quality;
 
   /* Bresenham helpers */
-  gint                    x_dest_inc; /* amount to increment for each dest. pixel  */
-  gint                    x_src_dec;  /* amount to decrement for each source pixel */
-  gint64                  dx_start;   /* pixel fraction for first pixel            */
+  gint              x_dest_inc; /* amount to increment for each dest. pixel  */
+  gint              x_src_dec;  /* amount to decrement for each source pixel */
+  gint64            dx_start;   /* pixel fraction for first pixel            */
 
-  gint                    y_dest_inc;
-  gint                    y_src_dec;
-  gint64                  dy_start;
+  gint              y_dest_inc;
+  gint              y_src_dec;
+  gint64            dy_start;
 
-  gint                    footprint_x;
-  gint                    footprint_y;
-  gint                    footshift_x;
-  gint                    footshift_y;
+  gint              footprint_x;
+  gint              footprint_y;
+  gint              footshift_x;
+  gint              footshift_y;
 
-  gint64                  dy;
+  gint64            dy;
 };
 
-static void  gimp_display_shell_render_info_scale   (RenderInfo             *info,
-                                                     const GimpDisplayShell *shell,
-                                                     TileManager            *tiles,
-                                                     gint                    level,
-                                                     gboolean                is_premult);
+static void  gimp_display_shell_render_info_scale   (RenderInfo       *info,
+                                                     GimpDisplayShell *shell,
+                                                     TileManager      *tiles,
+                                                     gint              level,
+                                                     gboolean          is_premult);
 
-static void  gimp_display_shell_render_setup_notify (GObject                *config,
-                                                     GParamSpec             *param_spec,
-                                                     Gimp                   *gimp);
+static void  gimp_display_shell_render_setup_notify (GObject          *config,
+                                                     GParamSpec       *param_spec,
+                                                     Gimp             *gimp);
 
 
 static guchar *tile_buf    = NULL;
@@ -190,20 +188,20 @@ gimp_display_shell_render_setup_notify (GObject    *config,
 
 /*  Render Image functions  */
 
-static void           render_image_rgb_a         (RenderInfo             *info);
-static void           render_image_gray_a        (RenderInfo             *info);
+static void           render_image_rgb_a         (RenderInfo         *info);
+static void           render_image_gray_a        (RenderInfo         *info);
 
-static const guchar * render_image_tile_fault    (RenderInfo             *info);
+static const guchar * render_image_tile_fault    (RenderInfo         *info);
 
 
-static void  gimp_display_shell_render_highlight (const GimpDisplayShell *shell,
-                                                  gint                    x,
-                                                  gint                    y,
-                                                  gint                    w,
-                                                  gint                    h,
-                                                  const GdkRectangle     *highlight);
-static void  gimp_display_shell_render_mask      (const GimpDisplayShell *shell,
-                                                  RenderInfo             *info);
+static void  gimp_display_shell_render_highlight (GimpDisplayShell   *shell,
+                                                  gint                x,
+                                                  gint                y,
+                                                  gint                w,
+                                                  gint                h,
+                                                  const GdkRectangle *highlight);
+static void  gimp_display_shell_render_mask      (GimpDisplayShell   *shell,
+                                                  RenderInfo         *info);
 
 
 /*****************************************************************/
@@ -214,12 +212,12 @@ static void  gimp_display_shell_render_mask      (const GimpDisplayShell *shell,
 /*****************************************************************/
 
 void
-gimp_display_shell_render (const GimpDisplayShell *shell,
-                           gint                    x,
-                           gint                    y,
-                           gint                    w,
-                           gint                    h,
-                           const GdkRectangle     *highlight)
+gimp_display_shell_render (GimpDisplayShell   *shell,
+                           gint                x,
+                           gint                y,
+                           gint                w,
+                           gint                h,
+                           const GdkRectangle *highlight)
 {
   GimpProjection *projection;
   GimpImage      *image;
@@ -231,7 +229,7 @@ gimp_display_shell_render (const GimpDisplayShell *shell,
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (w > 0 && h > 0);
 
-  image = shell->display->image;
+  image = gimp_display_get_image (shell->display);
 
   projection = gimp_image_get_projection (image);
 
@@ -348,12 +346,12 @@ gimp_display_shell_render (const GimpDisplayShell *shell,
 /*  This function highlights the given area by dimming all pixels outside. */
 
 static void
-gimp_display_shell_render_highlight (const GimpDisplayShell *shell,
-                                     gint                    x,
-                                     gint                    y,
-                                     gint                    w,
-                                     gint                    h,
-                                     const GdkRectangle     *highlight)
+gimp_display_shell_render_highlight (GimpDisplayShell   *shell,
+                                     gint                x,
+                                     gint                y,
+                                     gint                w,
+                                     gint                h,
+                                     const GdkRectangle *highlight)
 {
   guchar       *buf  = shell->render_buf;
   GdkRectangle  rect;
@@ -413,8 +411,8 @@ gimp_display_shell_render_highlight (const GimpDisplayShell *shell,
 }
 
 static void
-gimp_display_shell_render_mask (const GimpDisplayShell *shell,
-                                RenderInfo             *info)
+gimp_display_shell_render_mask (GimpDisplayShell *shell,
+                                RenderInfo       *info)
 {
   gint y, ye;
   gint x, xe;
@@ -598,11 +596,11 @@ render_image_rgb_a (RenderInfo *info)
 }
 
 static void
-gimp_display_shell_render_info_scale (RenderInfo             *info,
-                                      const GimpDisplayShell *shell,
-                                      TileManager            *tiles,
-                                      gint                    level,
-                                      gboolean                is_premult)
+gimp_display_shell_render_info_scale (RenderInfo       *info,
+                                      GimpDisplayShell *shell,
+                                      TileManager      *tiles,
+                                      gint              level,
+                                      gboolean          is_premult)
 {
   info->src_tiles      = tiles;
   info->src_is_premult = is_premult;
