@@ -19,15 +19,19 @@
 #define __GIMP_DISPLAY_SHELL_RENDER_H__
 
 
-void   gimp_display_shell_render_init (Gimp                   *gimp);
-void   gimp_display_shell_render_exit (Gimp                   *gimp);
+#define GIMP_DISPLAY_RENDER_BUF_WIDTH  256
+#define GIMP_DISPLAY_RENDER_BUF_HEIGHT 256
 
-void   gimp_display_shell_render      (const GimpDisplayShell *shell,
-                                       gint                    x,
-                                       gint                    y,
-                                       gint                    w,
-                                       gint                    h,
-                                       const GdkRectangle     *highlight);
+
+void   gimp_display_shell_render_init (Gimp               *gimp);
+void   gimp_display_shell_render_exit (Gimp               *gimp);
+
+void   gimp_display_shell_render      (GimpDisplayShell   *shell,
+                                       gint                x,
+                                       gint                y,
+                                       gint                w,
+                                       gint                h,
+                                       const GdkRectangle *highlight);
 
 
 #endif  /*  __GIMP_DISPLAY_SHELL_RENDER_H__  */
