@@ -22,6 +22,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "libgimpbase/gimpbase.h"
@@ -31,9 +32,9 @@
 #include "config/gimpdisplayconfig.h"
 
 #include "core/gimpcontainer.h"
+#include "core/gimpdrawable.h"
 #include "core/gimpimage.h"
 #include "core/gimpitem.h"
-#include "core/gimpunit.h"
 
 #include "file/file-utils.h"
 #include "file/gimp-file.h"
@@ -65,11 +66,6 @@ static gint     gimp_display_shell_format_filename   (gchar            *buf,
 
 /*  public functions  */
 
-void
-gimp_display_shell_title_init (GimpDisplayShell *shell)
-{
-}
-
 void
 gimp_display_shell_title_update (GimpDisplayShell *shell)
 {
@@ -92,10 +88,11 @@ gimp_display_shell_update_title_idle (gpointer data)
 
   shell->title_idle_id = 0;
 
-  if (shell->display->image)
+  if (gimp_display_get_image (shell->display))
     {
       GimpDisplayConfig *config = shell->display->config;
       gchar              title[MAX_TITLE_BUF];
+      gchar              status[MAX_TITLE_BUF];
       gint               len;
 
       /* format the title */
@@ -107,21 +104,21 @@ gimp_display_shell_update_title_idle (gpointer data)
 
       g_strlcpy (title + len, GIMP_ACRONYM, sizeof (title) - len);
 
-      gtk_window_set_title (GTK_WINDOW (shell), title);
-
       /* format the statusbar */
-      gimp_display_shell_format_title (shell, title, sizeof (title),
+      gimp_display_shell_format_title (shell, status, sizeof (status),
                                        config->image_status_format);
 
-      gimp_statusbar_replace (GIMP_STATUSBAR (shell->statusbar), "title",
-                              NULL, "%s", title);
+      g_object_set (shell,
+                    "title",  title,
+                    "status", status,
+                    NULL);
     }
   else
     {
-      gtk_window_set_title (GTK_WINDOW (shell), GIMP_NAME);
-
-      gimp_statusbar_replace (GIMP_STATUSBAR (shell->statusbar), "title",
-                              NULL, " ");
+      g_object_set (shell,
+                    "title",  GIMP_NAME,
+                    "status", " ",
+                    NULL);
     }
 
   return FALSE;
@@ -138,6 +135,17 @@ gimp_display_shell_title_image_type (GimpImage *image)
   return name;
 }
 
+static const gchar *
+gimp_display_shell_title_drawable_type (GimpDrawable *drawable)
+{
+  const gchar *name = "";
+
+  gimp_enum_get_value (GIMP_TYPE_IMAGE_TYPE,
+                       gimp_drawable_type (drawable), NULL, NULL, &name, NULL);
+
+  return name;
+}
+
 static gint print (gchar       *buf,
                    gint         len,
                    gint         start,
@@ -178,8 +186,8 @@ gimp_display_shell_format_title (GimpDisplayShell *shell,
 
   g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), 0);
 
-  image = shell->display->image;
-  gimp  = shell->display->gimp;
+  image = gimp_display_get_image (shell->display);
+  gimp  = gimp_display_get_gimp (shell->display);
 
   if (! image)
     {
@@ -230,14 +238,25 @@ gimp_display_shell_format_title (GimpDisplayShell *shell,
               break;
 
             case 'i': /* instance */
-              i += print (title, title_len, i, "%d", shell->display->instance);
+              i += print (title, title_len, i, "%d",
+                          gimp_display_get_instance (shell->display));
               break;
 
-            case 't': /* type */
+            case 't': /* image type */
               i += print (title, title_len, i, "%s",
                           gimp_display_shell_title_image_type (image));
               break;
 
+            case 'T': /* drawable type */
+              {
+                GimpDrawable *drawable = gimp_image_get_active_drawable (image);
+
+                if (drawable)
+                  i += print (title, title_len, i, "%s",
+                              gimp_display_shell_title_drawable_type (drawable));
+              }
+              break;
+
             case 's': /* user source zoom factor */
               i += print (title, title_len, i, "%d", denom);
               break;
@@ -361,11 +380,10 @@ gimp_display_shell_format_title (GimpDisplayShell *shell,
                   gimp_image_get_resolution (image, &xres, &yres);
 
                   g_snprintf (unit_format, sizeof (unit_format), "%%.%df",
-                              _gimp_unit_get_digits (gimp, shell->unit) + 1);
+                              gimp_unit_get_digits (shell->unit) + 1);
                   i += print (title, title_len, i, unit_format,
-                              (gimp_image_get_width (image) *
-                               _gimp_unit_get_factor (gimp, shell->unit) /
-                               xres));
+                              gimp_pixels_to_units (gimp_image_get_width (image),
+                                                    shell->unit, xres));
                   break;
                 }
               /* else fallthru */
@@ -384,11 +402,10 @@ gimp_display_shell_format_title (GimpDisplayShell *shell,
                   gimp_image_get_resolution (image, &xres, &yres);
 
                   g_snprintf (unit_format, sizeof (unit_format), "%%.%df",
-                              _gimp_unit_get_digits (gimp, shell->unit) + 1);
+                              gimp_unit_get_digits (shell->unit) + 1);
                   i += print (title, title_len, i, unit_format,
-                              (gimp_image_get_height (image) *
-                               _gimp_unit_get_factor (gimp, shell->unit) /
-                               yres));
+                              gimp_pixels_to_units (gimp_image_get_height (image),
+                                                    shell->unit, yres));
                   break;
                 }
               /* else fallthru */
@@ -399,12 +416,12 @@ gimp_display_shell_format_title (GimpDisplayShell *shell,
 
             case 'u': /* unit symbol */
               i += print (title, title_len, i, "%s",
-                          _gimp_unit_get_symbol (gimp, shell->unit));
+                          gimp_unit_get_symbol (shell->unit));
               break;
 
             case 'U': /* unit abbreviation */
               i += print (title, title_len, i, "%s",
-                          _gimp_unit_get_abbreviation (gimp, shell->unit));
+                          gimp_unit_get_abbreviation (shell->unit));
               break;
 
               /* Other cool things to be added:
