@@ -167,7 +167,7 @@ gimp_navigation_editor_display_changed (GimpContext          *context,
   GimpDisplayShell *shell = NULL;
 
   if (display)
-    shell = GIMP_DISPLAY_SHELL (display->shell);
+    shell = gimp_display_get_shell (display);
 
   gimp_navigation_editor_set_shell (editor, shell);
 }
@@ -346,7 +346,7 @@ gimp_navigation_editor_new_private (GimpMenuFactory  *menu_factory,
   if (shell)
     {
       Gimp              *gimp   = shell->display->gimp;
-      GimpDisplayConfig *config = GIMP_DISPLAY_CONFIG (gimp->config);
+      GimpDisplayConfig *config = shell->display->config;
       GimpView          *view;
 
       editor = g_object_new (GIMP_TYPE_NAVIGATION_EDITOR, NULL);
@@ -467,8 +467,10 @@ gimp_navigation_editor_set_shell (GimpNavigationEditor *editor,
 
   if (editor->shell)
     {
+      GimpImage *image = gimp_display_get_image (shell->display);
+
       gimp_view_set_viewable (GIMP_VIEW (editor->view),
-                              GIMP_VIEWABLE (shell->display->image));
+                              GIMP_VIEWABLE (image));
 
       g_signal_connect (editor->shell, "scaled",
                         G_CALLBACK (gimp_navigation_editor_shell_scaled),
@@ -571,16 +573,19 @@ gimp_navigation_editor_scroll (GimpNavigationView   *view,
         {
         case GDK_SCROLL_LEFT:
         case GDK_SCROLL_UP:
-          value -= adj->page_increment / 2;
+          value -= gtk_adjustment_get_page_increment (adj) / 2;
           break;
 
         case GDK_SCROLL_RIGHT:
         case GDK_SCROLL_DOWN:
-          value += adj->page_increment / 2;
+          value += gtk_adjustment_get_page_increment (adj) / 2;
           break;
         }
 
-      value = CLAMP (value, adj->lower, adj->upper - adj->page_size);
+      value = CLAMP (value,
+                     gtk_adjustment_get_lower (adj),
+                     gtk_adjustment_get_upper (adj) -
+                     gtk_adjustment_get_page_size (adj));
 
       gtk_adjustment_set_value (adj, value);
     }
@@ -650,8 +655,10 @@ static void
 gimp_navigation_editor_shell_reconnect (GimpDisplayShell     *shell,
                                         GimpNavigationEditor *editor)
 {
+  GimpImage *image = gimp_display_get_image (shell->display);
+
   gimp_view_set_viewable (GIMP_VIEW (editor->view),
-                          GIMP_VIEWABLE (shell->display->image));
+                          GIMP_VIEWABLE (image));
 
   if (GIMP_EDITOR (editor)->ui_manager)
     gimp_ui_manager_update (GIMP_EDITOR (editor)->ui_manager,
