@@ -138,7 +138,7 @@ file_save (Gimp                *gimp,
     gimp_pdb_execute_procedure_by_name (image->gimp->pdb,
                                         gimp_get_user_context (gimp),
                                         progress, error,
-                                        GIMP_OBJECT (file_proc)->name,
+                                        gimp_object_get_name (file_proc),
                                         GIMP_TYPE_INT32,       run_mode,
                                         GIMP_TYPE_IMAGE_ID,    image_ID,
                                         GIMP_TYPE_DRAWABLE_ID, drawable_ID,
