@@ -4,9 +4,9 @@
  * gimpoperationthreshold.c
  * Copyright (C) 2007 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -54,7 +53,7 @@ gimp_operation_threshold_class_init (GimpOperationThresholdClass *klass)
   object_class->set_property   = gimp_operation_point_filter_set_property;
   object_class->get_property   = gimp_operation_point_filter_get_property;
 
-  operation_class->name        = "gimp-threshold";
+  operation_class->name        = "gimp:threshold";
   operation_class->categories  = "color";
   operation_class->description = "GIMP Threshold operation";
 
@@ -94,15 +93,15 @@ gimp_operation_threshold_process (GeglOperation       *operation,
     {
       gfloat value;
 
-      value = MAX (src[RED_PIX], src[GREEN_PIX]);
-      value = MAX (value, src[BLUE_PIX]);
+      value = MAX (src[RED], src[GREEN]);
+      value = MAX (value, src[BLUE]);
 
       value = (value >= config->low && value <= config->high) ? 1.0 : 0.0;
 
-      dest[RED_PIX]   = value;
-      dest[GREEN_PIX] = value;
-      dest[BLUE_PIX]  = value;
-      dest[ALPHA_PIX] = src[ALPHA_PIX];
+      dest[RED]   = value;
+      dest[GREEN] = value;
+      dest[BLUE]  = value;
+      dest[ALPHA] = src[ALPHA];
 
       src  += 4;
       dest += 4;
