@@ -4,9 +4,9 @@
  * gimpoperationtilesink.c
  * Copyright (C) 2007 Øyvind Kolås <pippin@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -26,7 +25,7 @@
 #include <gegl.h>
 #include <gegl-buffer.h>
 
-#include "gegl-types.h"
+#include "gimp-gegl-types.h"
 
 #include "base/base-types.h"
 #include "base/tile-manager.h"
@@ -94,7 +93,7 @@ gimp_operation_tile_sink_class_init (GimpOperationTileSinkClass *klass)
   object_class->set_property   = gimp_operation_tile_sink_set_property;
   object_class->get_property   = gimp_operation_tile_sink_get_property;
 
-  operation_class->name        = "gimp-tilemanager-sink";
+  operation_class->name        = "gimp:tilemanager-sink";
   operation_class->categories  = "output";
   operation_class->description = "GIMP TileManager sink";
 
@@ -196,18 +195,13 @@ gimp_operation_tile_sink_process (GeglOperation       *operation,
   GimpOperationTileSink *self = GIMP_OPERATION_TILE_SINK (operation);
   const Babl            *format;
   PixelRegion            destPR;
-  guint                  bpp;
   gpointer               pr;
 
   if (! self->tile_manager)
     return FALSE;
 
-  bpp = tile_manager_bpp (self->tile_manager);
-
-  if (self->linear)
-    format = gimp_bpp_to_babl_format_linear (bpp);
-  else
-    format = gimp_bpp_to_babl_format (bpp);
+  format = gimp_bpp_to_babl_format (tile_manager_bpp (self->tile_manager),
+                                    self->linear);
 
   pixel_region_init (&destPR, self->tile_manager,
                      result->x,     result->y,
