@@ -19,6 +19,7 @@
 
 #include "glib-object.h"
 
+#include "gimp-debug.h"
 #include "gimp-log.h"
 
 
@@ -30,21 +31,30 @@ gimp_log_init (void)
 {
   const gchar *env_log_val = g_getenv ("GIMP_LOG");
 
+  if (! env_log_val)
+    env_log_val = g_getenv ("GIMP_DEBUG");
+
   if (env_log_val)
     {
       const GDebugKey log_keys[] =
       {
-        { "tool-events",    GIMP_LOG_TOOL_EVENTS    },
-        { "tool-focus",     GIMP_LOG_TOOL_FOCUS     },
-        { "dnd",            GIMP_LOG_DND            },
-        { "help",           GIMP_LOG_HELP           },
-        { "dialog-factory", GIMP_LOG_DIALOG_FACTORY },
-        { "menus",          GIMP_LOG_MENUS          },
-        { "save-dialog",    GIMP_LOG_SAVE_DIALOG    },
-        { "image-scale",    GIMP_LOG_IMAGE_SCALE    },
-        { "shadow-tiles",   GIMP_LOG_SHADOW_TILES   },
-        { "scale",          GIMP_LOG_SCALE          },
-        { "wm",             GIMP_LOG_WM             }
+        { "tool-events",        GIMP_LOG_TOOL_EVENTS        },
+        { "tool-focus",         GIMP_LOG_TOOL_FOCUS         },
+        { "dnd",                GIMP_LOG_DND                },
+        { "help",               GIMP_LOG_HELP               },
+        { "dialog-factory",     GIMP_LOG_DIALOG_FACTORY     },
+        { "menus",              GIMP_LOG_MENUS              },
+        { "save-dialog",        GIMP_LOG_SAVE_DIALOG        },
+        { "image-scale",        GIMP_LOG_IMAGE_SCALE        },
+        { "shadow-tiles",       GIMP_LOG_SHADOW_TILES       },
+        { "scale",              GIMP_LOG_SCALE              },
+        { "wm",                 GIMP_LOG_WM                 },
+        { "floating-selection", GIMP_LOG_FLOATING_SELECTION },
+        { "shm",                GIMP_LOG_SHM                },
+        { "text-editing",       GIMP_LOG_TEXT_EDITING       },
+        { "key-events",         GIMP_LOG_KEY_EVENTS         },
+        { "auto-tab-style",     GIMP_LOG_AUTO_TAB_STYLE     },
+        { "instances",          GIMP_LOG_INSTANCES          }
       };
 
       /*  g_parse_debug_string() has special treatment of the string 'help',
@@ -56,6 +66,9 @@ gimp_log_init (void)
         gimp_log_flags = g_parse_debug_string (env_log_val,
                                                log_keys,
                                                G_N_ELEMENTS (log_keys));
+
+      if (gimp_log_flags & GIMP_LOG_INSTANCES)
+        gimp_debug_enable_instances ();
     }
 }
 
