@@ -71,6 +71,7 @@
 #include "dialogs/dialogs.h"
 
 #include "color-history.h"
+#include "gimpuiconfigurer.h"
 #include "gui.h"
 #include "gui-unique.h"
 #include "gui-vtable.h"
@@ -114,6 +115,9 @@ static void       gui_show_help_button_notify   (GimpGuiConfig      *gui_config,
 static void       gui_user_manual_notify        (GimpGuiConfig      *gui_config,
                                                  GParamSpec         *pspec,
                                                  Gimp               *gimp);
+static void       gui_single_window_mode_notify (GimpGuiConfig      *gui_config,
+                                                 GParamSpec         *pspec,
+                                                 GimpUIConfigurer   *ui_configurer);
 static void       gui_tearoff_menus_notify      (GimpGuiConfig      *gui_config,
                                                  GParamSpec         *pspec,
                                                  GtkUIManager       *manager);
@@ -130,13 +134,13 @@ static void       gui_menu_hide_tooltip         (GimpUIManager      *manager,
 static void       gui_display_changed           (GimpContext        *context,
                                                  GimpDisplay        *display,
                                                  Gimp               *gimp);
-static void       gui_display_remove            (GimpContainer      *displays);
 
 
 /*  private variables  */
 
-static Gimp          *the_gui_gimp     = NULL;
-static GimpUIManager *image_ui_manager = NULL;
+static Gimp             *the_gui_gimp     = NULL;
+static GimpUIManager    *image_ui_manager = NULL;
+static GimpUIConfigurer *ui_configurer    = NULL;
 
 
 /*  public functions  */
@@ -253,8 +257,8 @@ static gchar *
 gui_sanity_check (void)
 {
 #define GTK_REQUIRED_MAJOR 2
-#define GTK_REQUIRED_MINOR 16
-#define GTK_REQUIRED_MICRO 1
+#define GTK_REQUIRED_MINOR 20
+#define GTK_REQUIRED_MICRO 0
 
   const gchar *mismatch = gtk_check_version (GTK_REQUIRED_MAJOR,
                                              GTK_REQUIRED_MINOR,
@@ -382,9 +386,6 @@ gui_restore_callback (Gimp               *gimp,
   g_signal_connect (gimp_get_user_context (gimp), "display-changed",
                     G_CALLBACK (gui_display_changed),
                     gimp);
-  g_signal_connect (gimp->displays, "remove",
-                    G_CALLBACK (gui_display_remove),
-                    NULL);
 
   /* make sure the monitor resolution is valid */
   if (display_config->monitor_res_from_gdk               ||
@@ -445,6 +446,10 @@ gui_restore_after_callback (Gimp               *gimp,
   if (gui_config->restore_accels)
     menus_restore (gimp);
 
+  ui_configurer = g_object_new (GIMP_TYPE_UI_CONFIGURER,
+                                "gimp", gimp,
+                                NULL);
+
   image_ui_manager = gimp_menu_factory_manager_new (global_menu_factory,
                                                     "<Image>",
                                                     gimp,
@@ -498,6 +503,9 @@ gui_restore_after_callback (Gimp               *gimp,
   }
 #endif /* GDK_WINDOWING_QUARTZ */
 
+  g_signal_connect_object (gui_config, "notify::single-window-mode",
+                           G_CALLBACK (gui_single_window_mode_notify),
+                           ui_configurer, 0);
   g_signal_connect_object (gui_config, "notify::tearoff-menus",
                            G_CALLBACK (gui_tearoff_menus_notify),
                            image_ui_manager, 0);
@@ -516,17 +524,24 @@ gui_restore_after_callback (Gimp               *gimp,
 
   color_history_restore (gimp);
 
-  /*  create the empty display  */
-  display = GIMP_DISPLAY (gimp_create_display (gimp,
-                                               NULL, GIMP_UNIT_PIXEL, 1.0));
+  if (gimp_get_show_gui (gimp))
+    {
+      GimpDisplayShell *shell;
 
-  if (gui_config->restore_session)
-    session_restore (gimp);
+      /*  create the empty display  */
+      display = GIMP_DISPLAY (gimp_create_display (gimp,
+                                                   NULL,
+                                                   GIMP_UNIT_PIXEL,
+                                                   1.0));
 
-  windows_show_toolbox ();
+      shell = gimp_display_get_shell (display);
 
-  /*  move keyboard focus to the display  */
-  gtk_window_present (GTK_WINDOW (display->shell));
+      if (gui_config->restore_session)
+        session_restore (gimp);
+
+      /*  move keyboard focus to the display  */
+      gtk_window_present (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (shell))));
+    }
 
   /*  indicate that the application has finished loading  */
   gdk_notify_startup_complete ();
@@ -543,13 +558,22 @@ gui_exit_callback (Gimp     *gimp,
 
   if (! force && gimp_displays_dirty (gimp))
     {
-      gimp_dialog_factory_dialog_raise (global_dialog_factory,
+      gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
                                         gdk_screen_get_default (),
                                         "gimp-quit-dialog", -1);
 
       return TRUE; /* stop exit for now */
     }
 
+  /* Since single-window mode is not session managed yet, force
+   * disabling of the mode before exit to prevent loss of
+   * dockables. Make sure to do this _after_ we have asked about
+   * saving unsaved images.
+   */
+  g_object_set (gui_config,
+                "single-window-mode", FALSE,
+                NULL);
+
   gimp->message_handler = GIMP_CONSOLE;
 
   gui_unique_exit ();
@@ -568,9 +592,6 @@ gui_exit_callback (Gimp     *gimp,
   if (TRUE /* gui_config->save_controllers */)
     gimp_controllers_save (gimp);
 
-  g_signal_handlers_disconnect_by_func (gimp->displays,
-                                        gui_display_remove,
-                                        NULL);
   g_signal_handlers_disconnect_by_func (gimp_get_user_context (gimp),
                                         gui_display_changed,
                                         gimp);
@@ -603,15 +624,18 @@ gui_exit_after_callback (Gimp     *gimp,
   g_object_unref (image_ui_manager);
   image_ui_manager = NULL;
 
+  g_object_unref (ui_configurer);
+  ui_configurer = NULL;
+
   session_exit (gimp);
   menus_exit (gimp);
   actions_exit (gimp);
   gimp_display_shell_render_exit (gimp);
   gimp_render_exit (gimp);
 
-  dialogs_exit (gimp);
   gimp_controllers_exit (gimp);
   gimp_devices_exit (gimp);
+  dialogs_exit (gimp);
 
   g_signal_handlers_disconnect_by_func (gimp,
                                         G_CALLBACK (gui_global_buffer_changed),
@@ -653,6 +677,14 @@ gui_user_manual_notify (GimpGuiConfig *gui_config,
   gimp_help_user_manual_changed (gimp);
 }
 
+static void
+gui_single_window_mode_notify (GimpGuiConfig      *gui_config,
+                               GParamSpec         *pspec,
+                               GimpUIConfigurer   *ui_configurer)
+{
+  gimp_ui_configurer_configure (ui_configurer,
+                                gui_config->single_window_mode);
+}
 static void
 gui_tearoff_menus_notify (GimpGuiConfig *gui_config,
                           GParamSpec    *pspec,
@@ -666,14 +698,14 @@ gui_device_change_notify (Gimp *gimp)
 {
   GimpSessionInfo *session_info;
 
-  session_info = gimp_dialog_factory_find_session_info (global_dock_factory,
+  session_info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                         "gimp-device-status");
 
-  if (session_info && session_info->widget)
+  if (session_info && gimp_session_info_get_widget (session_info))
     {
       GtkWidget *device_status;
 
-      device_status = gtk_bin_get_child (GTK_BIN (session_info->widget));
+      device_status = gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (session_info)));
 
       gimp_device_status_update (GIMP_DEVICE_STATUS (device_status));
     }
@@ -695,9 +727,10 @@ gui_menu_show_tooltip (GimpUIManager *manager,
 
   if (display)
     {
-      GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+      GimpDisplayShell *shell     = gimp_display_get_shell (display);
+      GimpStatusbar    *statusbar = gimp_display_shell_get_statusbar (shell);
 
-      gimp_statusbar_push (GIMP_STATUSBAR (shell->statusbar), "menu-tooltip",
+      gimp_statusbar_push (statusbar, "menu-tooltip",
                            NULL, "%s", tooltip);
     }
 }
@@ -711,9 +744,10 @@ gui_menu_hide_tooltip (GimpUIManager *manager,
 
   if (display)
     {
-      GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+      GimpDisplayShell *shell     = gimp_display_get_shell (display);
+      GimpStatusbar    *statusbar = gimp_display_shell_get_statusbar (shell);
 
-      gimp_statusbar_pop (GIMP_STATUSBAR (shell->statusbar), "menu-tooltip");
+      gimp_statusbar_pop (statusbar, "menu-tooltip");
     }
 }
 
@@ -736,7 +770,7 @@ gui_display_changed (GimpContext *context,
             {
               GimpDisplay *display2 = list->data;
 
-              if (display2->image == image)
+              if (gimp_display_get_image (display2) == image)
                 {
                   gimp_context_set_display (context, display2);
 
@@ -754,12 +788,3 @@ gui_display_changed (GimpContext *context,
 
   gimp_ui_manager_update (image_ui_manager, display);
 }
-
-static void
-gui_display_remove (GimpContainer *displays)
-{
-  /* show the toolbox when the last image window is closed */
-
-  if (gimp_container_is_empty (displays))
-    windows_show_toolbox ();
-}
