@@ -50,6 +50,7 @@
 #include "dialogs/dialogs.h"
 
 #include "session.h"
+#include "gimp-log.h"
 
 #include "gimp-intl.h"
 
@@ -125,49 +126,86 @@ session_init (Gimp *gimp)
         case G_TOKEN_SYMBOL:
           if (scanner->value.v_symbol == GINT_TO_POINTER (SESSION_INFO))
             {
-              GimpDialogFactory *factory;
-              GimpSessionInfo   *info;
-              gchar             *factory_name;
-              gchar             *entry_name;
-              gboolean           skip = FALSE;
+              GimpDialogFactory      *factory      = NULL;
+              GimpSessionInfo        *info         = NULL;
+              gchar                  *factory_name = NULL;
+              gchar                  *entry_name   = NULL;
+              GimpDialogFactoryEntry *entry        = NULL;
 
               token = G_TOKEN_STRING;
 
               if (! gimp_scanner_parse_string (scanner, &factory_name))
                 break;
 
-              factory = gimp_dialog_factory_from_name (factory_name);
-              g_free (factory_name);
-
-              if (! factory)
-                break;
-
-              if (! gimp_scanner_parse_string (scanner, &entry_name))
-                break;
+              /* In versions <= GIMP 2.6 there was a "toolbox", a
+               * "dock", a "display" and a "toplevel" factory. These
+               * are now merged to a single gimp_dialog_factory_get_singleton (). We
+               * need the legacy name though, so keep it around.
+               */
+              factory = gimp_dialog_factory_get_singleton ();
 
               info = gimp_session_info_new ();
 
-              if (strcmp (entry_name, "dock"))
+              /* GIMP 2.6 has the entry name as part of the
+               * session-info header, so try to get it
+               */
+              gimp_scanner_parse_string (scanner, &entry_name);
+              if (entry_name)
                 {
-                  info->toplevel_entry = gimp_dialog_factory_find_entry (factory,
-                                                                         entry_name);
-                  skip = (info->toplevel_entry == NULL);
-                }
-
-              g_free (entry_name);
-
-              if (GIMP_CONFIG_GET_INTERFACE (info)->deserialize (GIMP_CONFIG (info),
-                                                                 scanner,
-                                                                 1,
-                                                                 NULL))
-                {
-                  if (! skip)
+                  /* Previously, GimpDock was a toplevel. That is why
+                   * versions <= GIMP 2.6 has "dock" as the entry name. We
+                   * want "dock" to be interpreted as 'dock window'
+                   * however so have some special-casing for that. When
+                   * the entry name is "dock" the factory name is either
+                   * "dock" or "toolbox".
+                   */
+                  if (strcmp (entry_name, "dock") == 0)
                     {
-                      factory->session_infos =
-                        g_list_append (factory->session_infos, info);
+                      entry =
+                        gimp_dialog_factory_find_entry (factory,
+                                                        (strcmp (factory_name, "toolbox") == 0 ?
+                                                         "gimp-toolbox-window" :
+                                                         "gimp-dock-window"));
                     }
                   else
                     {
+                      entry = gimp_dialog_factory_find_entry (factory,
+                                                              entry_name);
+                    }
+                }
+
+              /* We're done with these now */
+              g_free (factory_name);
+              g_free (entry_name);
+
+              /* We can get the factory entry either now (the GIMP <=
+               * 2.6 way), or when we deserialize (the GIMP 2.8 way)
+               */
+              if (entry)
+                {
+                  gimp_session_info_set_factory_entry (info, entry);
+                }
+
+              /* Always try to deserialize */
+              if (gimp_config_deserialize (GIMP_CONFIG (info), scanner, 1, NULL))
+                {
+                  /* Make sure we got a factory entry either the 2.6
+                   * or 2.8 way
+                   */
+                  if (gimp_session_info_get_factory_entry (info))
+                    {
+                      GIMP_LOG (DIALOG_FACTORY,
+                                "successfully parsed and added session info %p",
+                                info);
+
+                      gimp_dialog_factory_add_session_info (factory, info);
+                    }
+                  else
+                    {
+                      GIMP_LOG (DIALOG_FACTORY,
+                                "failed to parse session info %p, not adding",
+                                info);
+
                       g_object_unref (info);
                     }
                 }
@@ -230,7 +268,7 @@ session_restore (Gimp *gimp)
 {
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
-  gimp_dialog_factories_session_restore ();
+  gimp_dialog_factory_restore (gimp_dialog_factory_get_singleton ());
 }
 
 void
@@ -269,13 +307,17 @@ session_save (Gimp     *gimp,
   if (!writer)
     return;
 
-  gimp_dialog_factories_session_save (writer);
+  gimp_dialog_factory_save (gimp_dialog_factory_get_singleton (), writer);
   gimp_config_writer_linefeed (writer);
 
-  /* save last tip shown */
+  /* save last tip shown
+   *
+   * FIXME: Make last-tip-shown increment only when used within the
+   * session
+   */
   gimp_config_writer_open (writer, "last-tip-shown");
   gimp_config_writer_printf (writer, "%d",
-                             GIMP_GUI_CONFIG (gimp->config)->last_tip + 1);
+                             GIMP_GUI_CONFIG (gimp->config)->last_tip);
   gimp_config_writer_close (writer);
 
   if (! gimp_config_writer_finish (writer, "end of sessionrc", &error))
