@@ -207,6 +207,14 @@ menus_init (Gimp              *gimp,
                                       "brushes-menu.xml", plug_in_menus_setup,
                                       NULL);
 
+  gimp_menu_factory_manager_register (global_menu_factory, "<Dynamics>",
+                                      "dynamics",
+                                      "plug-in",
+                                      NULL,
+                                      "/dynamics-popup",
+                                      "dynamics-menu.xml", plug_in_menus_setup,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<Patterns>",
                                       "patterns",
                                       "plug-in",
@@ -231,6 +239,15 @@ menus_init (Gimp              *gimp,
                                       "palettes-menu.xml", plug_in_menus_setup,
                                       NULL);
 
+
+  gimp_menu_factory_manager_register (global_menu_factory, "<ToolPreset>",
+                                      "tool-preset",
+                                      "plug-in",
+                                      NULL,
+                                      "/tool-preset-popup",
+                                      "tool-preset-menu.xml", plug_in_menus_setup,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<Fonts>",
                                       "fonts",
                                       "plug-in",
@@ -275,6 +292,13 @@ menus_init (Gimp              *gimp,
                                       "brush-editor-menu.xml", NULL,
                                       NULL);
 
+  gimp_menu_factory_manager_register (global_menu_factory, "<DynamicsEditor>",
+                                      "dynamics-editor",
+                                      NULL,
+                                      "/dynamics-editor-popup",
+                                      "dynamics-editor-menu.xml", NULL,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<GradientEditor>",
                                       "gradient-editor",
                                       NULL,
@@ -289,6 +313,13 @@ menus_init (Gimp              *gimp,
                                       "palette-editor-menu.xml", NULL,
                                       NULL);
 
+  gimp_menu_factory_manager_register (global_menu_factory, "<ToolPresetEditor>",
+                                      "tool-preset-editor",
+                                      NULL,
+                                      "/tool-preset-editor-popup",
+                                      "tool-preset-editor-menu.xml", NULL,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<Selection>",
                                       "select",
                                       "vectors",
@@ -339,7 +370,7 @@ menus_init (Gimp              *gimp,
                                       "text-tool-menu.xml",
                                       NULL,
                                       NULL);
-  
+
   gimp_menu_factory_manager_register (global_menu_factory, "<CursorInfo>",
                                       "cursor-info",
                                       NULL,
@@ -362,7 +393,6 @@ menus_exit (Gimp *gimp)
 {
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (global_menu_factory != NULL);
-  g_return_if_fail (global_menu_factory->gimp == gimp);
 
   g_object_unref (global_menu_factory);
   global_menu_factory = NULL;
