@@ -32,15 +32,22 @@
 #include "paint-funcs/paint-funcs.h"
 
 #include "core/gimpbrush.h"
+#include "core/gimpbrushgenerated.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimpmarshal.h"
+#include "core/gimpbrush-transform.h"
 
 #include "gimpbrushcore.h"
 #include "gimpbrushcore-kernels.h"
+
+
 #include "gimppaintoptions.h"
 
+
 #include "gimp-intl.h"
 
 
@@ -49,6 +56,7 @@
 enum
 {
   SET_BRUSH,
+  SET_DYNAMICS,
   LAST_SIGNAL
 };
 
@@ -84,6 +92,8 @@ static TempBuf *gimp_brush_core_get_paint_area     (GimpPaintCore    *paint_core
 
 static void     gimp_brush_core_real_set_brush     (GimpBrushCore    *core,
                                                     GimpBrush        *brush);
+static void     gimp_brush_core_real_set_dynamics  (GimpBrushCore    *core,
+                                                    GimpDynamics     *dynamics);
 
 static inline void rotate_pointers                 (gulong          **p,
                                                     guint32           n);
@@ -147,17 +157,30 @@ gimp_brush_core_class_init (GimpBrushCoreClass *klass)
                   G_TYPE_NONE, 1,
                   GIMP_TYPE_BRUSH);
 
-  object_class->finalize  = gimp_brush_core_finalize;
+  core_signals[SET_DYNAMICS] =
+    g_signal_new ("set-dynamics",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GimpBrushCoreClass, set_dynamics),
+                  NULL, NULL,
+                  gimp_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  GIMP_TYPE_DYNAMICS);
 
-  paint_core_class->start          = gimp_brush_core_start;
-  paint_core_class->pre_paint      = gimp_brush_core_pre_paint;
-  paint_core_class->post_paint     = gimp_brush_core_post_paint;
-  paint_core_class->interpolate    = gimp_brush_core_interpolate;
-  paint_core_class->get_paint_area = gimp_brush_core_get_paint_area;
+  object_class->finalize                    = gimp_brush_core_finalize;
 
-  klass->handles_changing_brush     = FALSE;
-  klass->handles_transforming_brush = TRUE;
-  klass->set_brush                  = gimp_brush_core_real_set_brush;
+  paint_core_class->start                   = gimp_brush_core_start;
+  paint_core_class->pre_paint               = gimp_brush_core_pre_paint;
+  paint_core_class->post_paint              = gimp_brush_core_post_paint;
+  paint_core_class->interpolate             = gimp_brush_core_interpolate;
+  paint_core_class->get_paint_area          = gimp_brush_core_get_paint_area;
+
+  klass->handles_changing_brush             = FALSE;
+  klass->handles_transforming_brush         = TRUE;
+  klass->handles_dynamic_transforming_brush = TRUE;
+
+  klass->set_brush                          = gimp_brush_core_real_set_brush;
+  klass->set_dynamics                       = gimp_brush_core_real_set_dynamics;
 }
 
 static void
@@ -167,9 +190,12 @@ gimp_brush_core_init (GimpBrushCore *core)
 
   core->main_brush                   = NULL;
   core->brush                        = NULL;
+  core->dynamics                     = NULL;
   core->spacing                      = 1.0;
   core->scale                        = 1.0;
   core->angle                        = 1.0;
+  core->hardness                     = 1.0;
+  core->force                        = 1.0;
 
   core->pressure_brush               = NULL;
 
@@ -193,6 +219,7 @@ gimp_brush_core_init (GimpBrushCore *core)
   core->rand                         = g_rand_new ();
 
   core->brush_bound_segs             = NULL;
+  core->transformed_brush_bound_segs = NULL;
   core->n_brush_bound_segs           = 0;
   core->brush_bound_width            = 0;
   core->brush_bound_height           = 0;
@@ -280,6 +307,12 @@ gimp_brush_core_finalize (GObject *object)
       core->main_brush = NULL;
     }
 
+  if (core->dynamics)
+    {
+      g_object_unref (core->dynamics);
+      core->dynamics = NULL;
+    }
+
   if (core->brush_bound_segs)
     {
       g_free (core->brush_bound_segs);
@@ -289,6 +322,12 @@ gimp_brush_core_finalize (GObject *object)
       core->brush_bound_height = 0;
     }
 
+  if (core->transformed_brush_bound_segs)
+    {
+      g_free (core->transformed_brush_bound_segs);
+      core->transformed_brush_bound_segs   = NULL;
+    }
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -305,6 +344,7 @@ gimp_brush_core_pre_paint (GimpPaintCore    *paint_core,
     {
       GimpCoords last_coords;
       GimpCoords current_coords;
+      gdouble scale;
 
       gimp_paint_core_get_last_coords (paint_core, &last_coords);
       gimp_paint_core_get_current_coords (paint_core, &current_coords);
@@ -322,6 +362,25 @@ gimp_brush_core_pre_paint (GimpPaintCore    *paint_core,
         {
           return FALSE;
         }
+      /*No drawing anything if the scale is too small*/
+      if (GIMP_BRUSH_CORE_GET_CLASS (core)->handles_transforming_brush)
+        {
+          GimpImage *image = gimp_item_get_image (GIMP_ITEM (drawable));
+          gdouble    fade_point;
+
+          if (GIMP_BRUSH_CORE_GET_CLASS (core)->handles_dynamic_transforming_brush)
+            {
+              fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                                        paint_core->pixel_dist);
+
+              scale = paint_options->brush_scale *
+                      gimp_dynamics_output_get_linear_value (core->dynamics->size_output,
+                                                             &current_coords,
+                                                             paint_options,
+                                                             fade_point);
+              if (scale < 0.0001) return FALSE;
+            }
+        }
 
       if (GIMP_BRUSH_CORE_GET_CLASS (paint_core)->handles_changing_brush)
         {
@@ -358,28 +417,38 @@ gimp_brush_core_start (GimpPaintCore     *paint_core,
 {
   GimpBrushCore *core = GIMP_BRUSH_CORE (paint_core);
   GimpBrush     *brush;
+  GimpDynamics  *dynamics;
 
   brush = gimp_context_get_brush (GIMP_CONTEXT (paint_options));
 
   if (core->main_brush != brush)
     gimp_brush_core_set_brush (core, brush);
 
+  dynamics = gimp_context_get_dynamics (GIMP_CONTEXT (paint_options));
+
+  if (core->dynamics != dynamics)
+    gimp_brush_core_set_dynamics (core, dynamics);
+
   if (! core->main_brush)
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-			   _("No brushes available for use with this tool."));
+                           _("No brushes available for use with this tool."));
+      return FALSE;
+    }
+
+  if (! core->dynamics)
+    {
+      g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
+                           _("No paint dynamics available for use with this tool."));
       return FALSE;
     }
 
   if (GIMP_BRUSH_CORE_GET_CLASS (core)->handles_transforming_brush)
     {
-      core->scale = gimp_paint_options_get_dynamic_size (paint_options, coords,
-                                                         TRUE);
-
-      core->angle = gimp_paint_options_get_dynamic_angle (paint_options, coords);
-
-      core->aspect_ratio =
-        gimp_paint_options_get_dynamic_aspect_ratio (paint_options, coords);
+      gimp_brush_core_eval_transform_dynamics (paint_core,
+                                               drawable,
+                                               paint_options,
+                                               coords);
     }
 
   core->spacing = (gdouble) gimp_brush_get_spacing (core->main_brush) / 100.0;
@@ -425,7 +494,8 @@ gimp_brush_core_interpolate (GimpPaintCore    *paint_core,
                              GimpPaintOptions *paint_options,
                              guint32           time)
 {
-  GimpBrushCore *core = GIMP_BRUSH_CORE (paint_core);
+  GimpBrushCore *core  = GIMP_BRUSH_CORE (paint_core);
+  GimpImage     *image = gimp_item_get_image (GIMP_ITEM (drawable));
   GimpCoords     last_coords;
   GimpCoords     current_coords;
   GimpVector2    delta_vec;
@@ -445,12 +515,54 @@ gimp_brush_core_interpolate (GimpPaintCore    *paint_core,
   gdouble        pixel_initial;
   gdouble        xd, yd;
   gdouble        mag;
+  gdouble        dyn_spacing = core->spacing;
+  gdouble        fade_point;
+  gboolean       use_dyn_spacing;
 
   g_return_if_fail (GIMP_IS_BRUSH (core->brush));
 
   gimp_paint_core_get_last_coords (paint_core, &last_coords);
   gimp_paint_core_get_current_coords (paint_core, &current_coords);
 
+  /*Zero sized brushes are unfit for interpolate,
+   * so we just let paint core fail onits own
+   **/
+  if (core->scale == 0.0)
+    {
+      gimp_paint_core_set_last_coords (paint_core, &current_coords);
+
+      gimp_paint_core_paint (paint_core, drawable, paint_options,
+                             GIMP_PAINT_STATE_MOTION, time);
+      return;
+    }
+
+  fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                            paint_core->pixel_dist);
+
+  use_dyn_spacing =
+    gimp_dynamics_output_is_enabled (core->dynamics->spacing_output);
+
+  if (use_dyn_spacing)
+    {
+      dyn_spacing =
+        gimp_dynamics_output_get_linear_value (core->dynamics->spacing_output,
+                                               &current_coords,
+                                               paint_options,
+                                               fade_point);
+
+      /* Dynamic spacing assumes that the value set in core is the min
+       * value and the max is full 200% spacing. This approach differs
+       * from the usual factor from user input approach because making
+       * spacing smaller than the nominal value is unlikely and
+       * spacing has a hard defined max.
+       */
+      dyn_spacing = (core->spacing +
+                     ((2.0 - core->spacing) * ( 1.0 - dyn_spacing)));
+
+      /*  Limiting spacing to minimum 1%  */
+      dyn_spacing = MAX (core->spacing, dyn_spacing);
+    }
+
   gimp_avoid_exact_integer (&last_coords.x);
   gimp_avoid_exact_integer (&last_coords.y);
   gimp_avoid_exact_integer (&current_coords.x);
@@ -495,25 +607,6 @@ gimp_brush_core_interpolate (GimpPaintCore    *paint_core,
   pixel_dist    = gimp_vector2_length (&delta_vec);
   pixel_initial = paint_core->pixel_dist;
 
-  /*  FIXME: need to adapt the spacing to the size                      */
-  /*   lastscale = MIN (last_coords.pressure, 1/256);                   */
-  /*   curscale = MIN (current_coords.pressure,  1/256);                */
-  /*   spacing = core->spacing * sqrt (0.5 * (lastscale + curscale));   */
-
-  /*  Compute spacing parameters such that a brush position will be
-   *  made each time the line crosses the *center* of a pixel row or
-   *  column, according to whether the line is mostly horizontal or
-   *  mostly vertical. The term "stripe" will mean "column" if the
-   *  line is horizontalish; "row" if the line is verticalish.
-   *
-   *  We start by deriving coefficients for a new parameter 's':
-   *      s = t * st_factor + st_offset
-   *  such that the "nice" brush positions are the ones with *integer*
-   *  s values. (Actually the value of s will be 1/2 less than the nice
-   *  brush position's x or y coordinate - note that st_factor may
-   *  be negative!)
-   */
-
   if (delta_vec.x * delta_vec.x > delta_vec.y * delta_vec.y)
     {
       st_factor = delta_vec.x;
@@ -525,7 +618,20 @@ gimp_brush_core_interpolate (GimpPaintCore    *paint_core,
       st_offset = last_coords.y - 0.5;
     }
 
-  if (fabs (st_factor) > dist / core->spacing)
+  if (use_dyn_spacing)
+    {
+      gint s0;
+
+      num_points = dist / dyn_spacing;
+
+      s0 = (gint) floor (st_offset + 0.5);
+      t0 = (s0 - st_offset) / st_factor;
+      dt = dyn_spacing / dist;
+
+      if (num_points == 0)
+        return;
+    }
+  else if (fabs (st_factor) > dist / core->spacing)
     {
       /*  The stripe principle leads to brush positions that are spaced
        *  *closer* than the official brush spacing. Use the official
@@ -667,10 +773,18 @@ gimp_brush_core_interpolate (GimpPaintCore    *paint_core,
 
       if (core->jitter > 0.0)
         {
+          gdouble dyn_jitter;
           gdouble jitter_dist;
           gint32  jitter_angle;
 
-          jitter_dist  = g_rand_double_range (core->rand, 0, core->jitter);
+          dyn_jitter =
+            core->jitter *
+            gimp_dynamics_output_get_linear_value (core->dynamics->jitter_output,
+                                                   &current_coords,
+                                                   paint_options,
+                                                   fade_point);
+
+          jitter_dist  = g_rand_double_range (core->rand, 0, dyn_jitter);
           jitter_angle = g_rand_int_range (core->rand,
                                            0, BRUSH_CORE_JITTER_LUTSIZE);
 
@@ -721,13 +835,10 @@ gimp_brush_core_get_paint_area (GimpPaintCore    *paint_core,
 
   if (GIMP_BRUSH_CORE_GET_CLASS (core)->handles_transforming_brush)
     {
-      core->scale = gimp_paint_options_get_dynamic_size (paint_options, coords,
-                                                         TRUE);
-
-      core->angle = gimp_paint_options_get_dynamic_angle (paint_options, coords);
-
-      core->aspect_ratio =
-        gimp_paint_options_get_dynamic_aspect_ratio (paint_options, coords);
+      gimp_brush_core_eval_transform_dynamics (paint_core,
+                                               drawable,
+                                               paint_options,
+                                               coords);
     }
 
   core->scale = gimp_brush_core_clamp_brush_scale (core, core->scale);
@@ -798,6 +909,19 @@ gimp_brush_core_real_set_brush (GimpBrushCore *core,
     }
 }
 
+static void
+gimp_brush_core_real_set_dynamics (GimpBrushCore *core,
+                                   GimpDynamics  *dynamics)
+{
+  if (core->dynamics)
+    g_object_unref (core->dynamics);
+
+  core->dynamics = dynamics;
+
+  if (core->dynamics)
+    g_object_ref (core->dynamics);
+}
+
 void
 gimp_brush_core_set_brush (GimpBrushCore *core,
                            GimpBrush     *brush)
@@ -808,6 +932,16 @@ gimp_brush_core_set_brush (GimpBrushCore *core,
   g_signal_emit (core, core_signals[SET_BRUSH], 0, brush);
 }
 
+void
+gimp_brush_core_set_dynamics (GimpBrushCore *core,
+                              GimpDynamics  *dynamics)
+{
+  g_return_if_fail (GIMP_IS_BRUSH_CORE (core));
+  g_return_if_fail (dynamics == NULL || GIMP_IS_DYNAMICS (dynamics));
+
+  g_signal_emit (core, core_signals[SET_DYNAMICS], 0, dynamics);
+}
+
 void
 gimp_brush_core_create_bound_segs (GimpBrushCore    *core,
                                    GimpPaintOptions *paint_options)
@@ -829,8 +963,23 @@ gimp_brush_core_create_bound_segs (GimpBrushCore    *core,
     {
       scale = gimp_brush_core_clamp_brush_scale (core, scale);
 
-      mask = gimp_brush_transform_mask (core->main_brush,
-                                        scale, aspect_ratio, angle);
+      /* Generated brushes are a bit special*/
+      if (GIMP_IS_BRUSH_GENERATED (core->main_brush))
+        {
+          GimpBrushGenerated *generated_brush = GIMP_BRUSH_GENERATED (core->main_brush);
+
+          gdouble ratio = gimp_brush_generated_get_aspect_ratio (generated_brush);
+
+          gimp_brush_generated_set_aspect_ratio (generated_brush, 1.0);
+
+          mask = gimp_brush_transform_mask (core->main_brush,
+                                            1.0, 1.0, 0.0, 1.0);
+
+          gimp_brush_generated_set_aspect_ratio (generated_brush, ratio);
+        }
+      else
+        mask = gimp_brush_transform_mask (core->main_brush,
+                                          1.0, 1.0, 0.0, 1.0);
     }
 
   if (mask)
@@ -869,6 +1018,82 @@ gimp_brush_core_create_bound_segs (GimpBrushCore    *core,
     }
 }
 
+void
+gimp_brush_core_transform_bound_segs (GimpBrushCore    *core,
+                                      GimpPaintOptions *paint_options)
+{
+  gdouble      scale;
+  gdouble      angle;
+  gdouble      aspect_ratio;
+  gdouble      height;
+  gdouble      width;
+  GimpMatrix3  matrix;
+  gdouble      scale_x;
+  gdouble      scale_y;
+
+
+  g_return_if_fail (GIMP_IS_BRUSH_CORE (core));
+  g_return_if_fail (core->main_brush != NULL);
+  g_return_if_fail (core->brush_bound_segs != NULL);
+
+  scale = core->scale;
+  angle = core->angle;
+  aspect_ratio = core->aspect_ratio;
+
+  /* Generated brushes have their angle applied on top of base angle */
+  if (GIMP_IS_BRUSH_GENERATED (core->main_brush))
+    {
+      GimpBrushGenerated *generated_brush = GIMP_BRUSH_GENERATED (core->main_brush);
+
+      gdouble base_angle = gimp_brush_generated_get_angle (generated_brush);
+      angle = angle + base_angle / 360;
+
+      if (aspect_ratio == 1.0)
+        aspect_ratio = gimp_brush_generated_get_aspect_ratio (generated_brush);
+    }
+
+  height = core->brush_bound_width;
+  width  = core->brush_bound_height;
+
+  if (aspect_ratio < 1.0)
+    {
+        scale_x = scale * aspect_ratio;
+        scale_y = scale;
+    }
+  else
+    {
+      scale_x = scale;
+      scale_y = scale / aspect_ratio;
+    }
+
+  if (core->transformed_brush_bound_segs)
+    {
+      g_free (core->transformed_brush_bound_segs);
+      core->transformed_brush_bound_segs   = NULL;
+    }
+
+
+  if ((scale > 0.0) && (aspect_ratio > 0.0))
+    {
+
+      scale = gimp_brush_core_clamp_brush_scale (core, scale);
+
+
+      gimp_brush_transform_matrix (height, width,
+                                   scale, aspect_ratio, angle, &matrix);
+
+      core->transformed_brush_bound_segs
+                      = boundary_transform (core->brush_bound_segs,
+                                            &core->n_brush_bound_segs,
+                                            &matrix);
+
+      /* FIXME. Do noy use scale_x/scale_y */
+      core->transformed_brush_bound_width = core->brush_bound_width * scale_x;
+      core->transformed_brush_bound_height = core->brush_bound_height * scale_y;
+    }
+
+}
+
 void
 gimp_brush_core_paste_canvas (GimpBrushCore            *core,
                               GimpDrawable             *drawable,
@@ -877,13 +1102,13 @@ gimp_brush_core_paste_canvas (GimpBrushCore            *core,
                               gdouble                   image_opacity,
                               GimpLayerModeEffects      paint_mode,
                               GimpBrushApplicationMode  brush_hardness,
-                              gdouble                   dynamic_hardness,
+                              gdouble                   dynamic_force,
                               GimpPaintApplicationMode  mode)
 {
   TempBuf *brush_mask = gimp_brush_core_get_brush_mask (core,
                                                         coords,
                                                         brush_hardness,
-                                                        dynamic_hardness);
+                                                        dynamic_force);
 
   if (brush_mask)
     {
@@ -923,13 +1148,13 @@ gimp_brush_core_replace_canvas (GimpBrushCore            *core,
                                 gdouble                   brush_opacity,
                                 gdouble                   image_opacity,
                                 GimpBrushApplicationMode  brush_hardness,
-                                gdouble                   dynamic_hardness,
+                                gdouble                   dynamic_force,
                                 GimpPaintApplicationMode  mode)
 {
   TempBuf *brush_mask = gimp_brush_core_get_brush_mask (core,
                                                         coords,
                                                         brush_hardness,
-                                                        dynamic_hardness);
+                                                        dynamic_force);
 
   if (brush_mask)
     {
@@ -1336,7 +1561,10 @@ gimp_brush_core_transform_mask (GimpBrushCore *core,
   if (core->scale <= 0.0)
     return NULL; /* Should never happen now, with scale clamping. */
 
-  if ((core->scale == 1.0) && (core->angle == 0.0) && (core->aspect_ratio == 1.0))
+  if ((core->scale == 1.0) &&
+      (core->angle == 0.0) &&
+      (core->hardness == 1.0) &&
+      (core->aspect_ratio == 1.0))
     return brush->mask;
 
   gimp_brush_transform_size (brush,
@@ -1350,6 +1578,7 @@ gimp_brush_core_transform_mask (GimpBrushCore *core,
       height             == core->last_transform_height &&
       core->scale        == core->last_scale            &&
       core->angle        == core->last_angle            &&
+      core->hardness     == core->last_hardness         &&
       core->aspect_ratio == core->last_aspect_ratio)
     {
       return core->transform_brush;
@@ -1360,16 +1589,17 @@ gimp_brush_core_transform_mask (GimpBrushCore *core,
   core->last_transform_height = height;
   core->last_scale        = core->scale;
   core->last_angle        = core->angle;
+  core->last_hardness     = core->hardness;
   core->last_aspect_ratio = core->aspect_ratio;
 
-
   if (core->transform_brush)
     temp_buf_free (core->transform_brush);
 
   core->transform_brush = gimp_brush_transform_mask (brush,
                                                      core->scale,
                                                      core->aspect_ratio,
-                                                     core->angle);
+                                                     core->angle,
+                                                     core->hardness);
 
   core->cache_invalid       = TRUE;
   core->solid_cache_invalid = TRUE;
@@ -1387,7 +1617,10 @@ gimp_brush_core_transform_pixmap (GimpBrushCore *core,
   if (core->scale <= 0.0)
     return NULL;
 
-  if ((core->scale == 1.0) && (core->angle == 0.0) && (core->aspect_ratio == 1.0))
+  if ((core->scale        == 1.0) &&
+      (core->angle        == 0.0) &&
+      (core->hardness     == 1.0) &&
+      (core->aspect_ratio == 1.0))
     return brush->pixmap;
 
   gimp_brush_transform_size (brush,
@@ -1400,6 +1633,7 @@ gimp_brush_core_transform_pixmap (GimpBrushCore *core,
       width              == core->last_transform_pixmap_width  &&
       height             == core->last_transform_pixmap_height &&
       core->angle        == core->last_angle                   &&
+      core->hardness     == core->last_hardness                &&
       core->aspect_ratio == core->last_aspect_ratio)
     {
       return core->transform_pixmap;
@@ -1408,8 +1642,9 @@ gimp_brush_core_transform_pixmap (GimpBrushCore *core,
   core->last_transform_pixmap        = brush->pixmap;
   core->last_transform_pixmap_width  = width;
   core->last_transform_pixmap_height = height;
-  core->last_angle = core->angle;
-  core->last_aspect_ratio = core->aspect_ratio;
+  core->last_angle                   = core->angle;
+  core->last_hardness                = core->hardness;
+  core->last_aspect_ratio            = core->aspect_ratio;
 
   if (core->transform_pixmap)
     temp_buf_free (core->transform_pixmap);
@@ -1418,7 +1653,8 @@ gimp_brush_core_transform_pixmap (GimpBrushCore *core,
   core->transform_pixmap = gimp_brush_transform_pixmap (brush,
                                                         core->scale,
                                                         core->aspect_ratio,
-                                                        core->angle);
+                                                        core->angle,
+                                                        core->hardness);
 
   core->cache_invalid = TRUE;
 
@@ -1429,7 +1665,7 @@ TempBuf *
 gimp_brush_core_get_brush_mask (GimpBrushCore            *core,
                                 const GimpCoords         *coords,
                                 GimpBrushApplicationMode  brush_hardness,
-                                gdouble                   dynamic_hardness)
+                                gdouble                   dynamic_force)
 {
   TempBuf *mask;
 
@@ -1456,7 +1692,7 @@ gimp_brush_core_get_brush_mask (GimpBrushCore            *core,
       mask = gimp_brush_core_pressurize_mask (core, mask,
                                               coords->x,
                                               coords->y,
-                                              dynamic_hardness);
+                                              dynamic_force);
       break;
 
     default:
@@ -1466,6 +1702,59 @@ gimp_brush_core_get_brush_mask (GimpBrushCore            *core,
   return mask;
 }
 
+void
+gimp_brush_core_eval_transform_dynamics (GimpPaintCore     *paint_core,
+                                         GimpDrawable      *drawable,
+                                         GimpPaintOptions  *paint_options,
+                                         const GimpCoords  *coords)
+{
+  GimpBrushCore *core = GIMP_BRUSH_CORE (paint_core);
+  GimpImage *image;
+  gdouble    fade_point = 1.0;
+
+  core->scale = paint_options->brush_scale;
+  core->angle = paint_options->brush_angle;
+  core->aspect_ratio = paint_options->brush_aspect_ratio;
+
+  if (!GIMP_IS_DYNAMICS (core->dynamics))
+    return;
+
+  if (GIMP_BRUSH_CORE_GET_CLASS (core)->handles_dynamic_transforming_brush)
+    {
+      if (drawable)
+        {
+          image = gimp_item_get_image (GIMP_ITEM (drawable));
+          fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                                    paint_core->pixel_dist);
+        }
+
+      core->scale *=
+            gimp_dynamics_output_get_linear_value (core->dynamics->size_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
+
+      core->angle +=
+            gimp_dynamics_output_get_angular_value (core->dynamics->angle_output,
+                                                    coords,
+                                                    paint_options,
+                                                    fade_point);
+
+      core->hardness =
+            gimp_dynamics_output_get_linear_value (core->dynamics->hardness_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
+
+      core->aspect_ratio *=
+            gimp_dynamics_output_get_aspect_value (core->dynamics->aspect_ratio_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
+    }
+
+}
+
 
 /**************************************************/
 /*  Brush pipe utility functions                  */
