@@ -43,9 +43,12 @@ struct _GimpBrushCore
 
   GimpBrush     *main_brush;
   GimpBrush     *brush;
+  GimpDynamics  *dynamics;
   gdouble        spacing;
   gdouble        scale;
   gdouble        angle;
+  gdouble        hardness;
+  gdouble        force;
   gdouble        aspect_ratio;
 
   /*  brush buffers  */
@@ -61,6 +64,7 @@ struct _GimpBrushCore
   gint           last_transform_height;
   gdouble        last_scale;
   gdouble        last_angle;
+  gdouble        last_hardness;
   gdouble        last_aspect_ratio;
 
   TempBuf       *transform_pixmap;
@@ -81,9 +85,12 @@ struct _GimpBrushCore
 
   /*  don't use these...  */
   BoundSeg      *brush_bound_segs;
+  BoundSeg      *transformed_brush_bound_segs;
   gint           n_brush_bound_segs;
   gint           brush_bound_width;
   gint           brush_bound_height;
+  gint           transformed_brush_bound_width;
+  gint           transformed_brush_bound_height;
 };
 
 struct _GimpBrushCoreClass
@@ -94,11 +101,14 @@ struct _GimpBrushCoreClass
   gboolean            handles_changing_brush;
 
   /*  Set for tools that don't mind if the brush scales while painting  */
-
   gboolean            handles_transforming_brush;
+  /*  Set for tools that don't mind if the brush scales mid stroke  */
+  gboolean            handles_dynamic_transforming_brush;
 
-  void (* set_brush) (GimpBrushCore *core,
-                      GimpBrush     *brush);
+  void (* set_brush)    (GimpBrushCore *core,
+                         GimpBrush     *brush);
+  void (* set_dynamics) (GimpBrushCore *core,
+                         GimpDynamics  *brush);
 };
 
 
@@ -106,8 +116,14 @@ GType   gimp_brush_core_get_type       (void) G_GNUC_CONST;
 
 void    gimp_brush_core_set_brush      (GimpBrushCore            *core,
                                         GimpBrush                *brush);
+
+void    gimp_brush_core_set_dynamics   (GimpBrushCore            *core,
+                                        GimpDynamics             *dynamics);
+
 void    gimp_brush_core_create_bound_segs (GimpBrushCore         *core,
                                            GimpPaintOptions      *options);
+void gimp_brush_core_transform_bound_segs (GimpBrushCore    *core,
+                                           GimpPaintOptions *paint_options);
 
 void    gimp_brush_core_paste_canvas   (GimpBrushCore            *core,
                                         GimpDrawable             *drawable,
@@ -140,4 +156,9 @@ TempBuf * gimp_brush_core_get_brush_mask (GimpBrushCore            *core,
                                           gdouble                   dynamic_hardness);
 
 
+void gimp_brush_core_eval_transform_dynamics (GimpPaintCore     *paint_core,
+                                              GimpDrawable      *drawable,
+                                              GimpPaintOptions  *paint_options,
+                                              const GimpCoords  *coords);
+
 #endif  /*  __GIMP_BRUSH_CORE_H__  */
