@@ -33,6 +33,8 @@
 
 #include "core/gimp.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimppattern.h"
@@ -166,17 +168,17 @@ gimp_clone_motion (GimpSourceCore   *source_core,
   GimpCloneOptions  *options        = GIMP_CLONE_OPTIONS (paint_options);
   GimpSourceOptions *source_options = GIMP_SOURCE_OPTIONS (paint_options);
   GimpContext       *context        = GIMP_CONTEXT (paint_options);
+  GimpDynamics      *dynamics       = GIMP_BRUSH_CORE (paint_core)->dynamics;
+  GimpImage         *image          = gimp_item_get_image (GIMP_ITEM (drawable));
   GimpImage         *src_image      = NULL;
   GimpImageType      src_type       = 0;
   GimpImageType      dest_type;
-  GimpImage         *image;
   gpointer           pr = NULL;
   gint               y;
   PixelRegion        destPR;
   GimpPattern       *pattern = NULL;
-  gdouble            hardness;
-
-  image = gimp_item_get_image (GIMP_ITEM (drawable));
+  gdouble            fade_point;
+  gdouble            force;
 
   switch (options->clone_type)
     {
@@ -238,9 +240,13 @@ gimp_clone_motion (GimpSourceCore   *source_core,
         }
     }
 
-  opacity *= gimp_paint_options_get_dynamic_opacity (paint_options, coords);
+  fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                            paint_core->pixel_dist);
 
-  hardness = gimp_paint_options_get_dynamic_hardness (paint_options, coords);
+  force = gimp_dynamics_output_get_linear_value (dynamics->force_output,
+                                                    coords,
+                                                    paint_options,
+                                                    fade_point);
 
   gimp_brush_core_paste_canvas (GIMP_BRUSH_CORE (paint_core), drawable,
                                 coords,
@@ -248,7 +254,7 @@ gimp_clone_motion (GimpSourceCore   *source_core,
                                 gimp_context_get_opacity (context),
                                 gimp_context_get_paint_mode (context),
                                 gimp_paint_options_get_brush_mode (paint_options),
-                                hardness,
+                                force,
 
                                 /* In fixed mode, paint incremental so the
                                  * individual brushes are properly applied
