@@ -29,6 +29,8 @@
 
 #include "core/gimp.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimpimage.h"
 
 #include "gimperaser.h"
@@ -105,18 +107,26 @@ gimp_eraser_motion (GimpPaintCore    *paint_core,
                     GimpPaintOptions *paint_options,
                     const GimpCoords *coords)
 {
-  GimpEraserOptions *options = GIMP_ERASER_OPTIONS (paint_options);
-  GimpContext       *context = GIMP_CONTEXT (paint_options);
+  GimpEraserOptions *options  = GIMP_ERASER_OPTIONS (paint_options);
+  GimpContext       *context  = GIMP_CONTEXT (paint_options);
+  GimpDynamics      *dynamics = GIMP_BRUSH_CORE (paint_core)->dynamics;
   GimpImage         *image;
+  gdouble            fade_point;
   gdouble            opacity;
   TempBuf           *area;
   guchar             col[MAX_CHANNELS];
-  gdouble            hardness;
+  gdouble            force;
 
   image = gimp_item_get_image (GIMP_ITEM (drawable));
 
-  opacity = gimp_paint_options_get_fade (paint_options, image,
-                                         paint_core->pixel_dist);
+  fade_point = gimp_paint_options_get_fade (paint_options,
+                                            gimp_item_get_image (GIMP_ITEM (drawable)),
+                                            paint_core->pixel_dist);
+
+  opacity = gimp_dynamics_output_get_linear_value (dynamics->opacity_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
   if (opacity == 0.0)
     return;
 
@@ -135,9 +145,10 @@ gimp_eraser_motion (GimpPaintCore    *paint_core,
   color_pixels (temp_buf_get_data (area), col,
                 area->width * area->height, area->bytes);
 
-  opacity *= gimp_paint_options_get_dynamic_opacity (paint_options, coords);
-
-  hardness = gimp_paint_options_get_dynamic_hardness (paint_options, coords);
+  force = gimp_dynamics_output_get_linear_value (dynamics->force_output,
+                                                    coords,
+                                                    paint_options,
+                                                    fade_point);
 
   gimp_brush_core_paste_canvas (GIMP_BRUSH_CORE (paint_core), drawable,
                                 coords,
@@ -146,6 +157,6 @@ gimp_eraser_motion (GimpPaintCore    *paint_core,
                                 (options->anti_erase ?
                                  GIMP_ANTI_ERASE_MODE : GIMP_ERASE_MODE),
                                 gimp_paint_options_get_brush_mode (paint_options),
-                                hardness,
+                                force,
                                 paint_options->application_mode);
 }
