@@ -33,6 +33,8 @@
 
 #include "core/gimpbrush.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimppickable.h"
@@ -300,17 +302,18 @@ gimp_heal_laplace_iteration (gdouble *matrix,
                              gdouble *solution,
                              guchar  *mask)
 {
-  gint     rowstride = width * depth;
-  gint     i, j, k;
-  gdouble  tmp, diff;
-  gdouble  err       = 0.0;
+  const gint  rowstride = width * depth;
+  gint        i, j, k;
+  gdouble     tmp, diff;
+  gdouble     err       = 0.0;
 
   for (i = 0; i < height; i++)
     {
       for (j = 0; j < width; j++)
         {
-          if ((0 == *mask) || (i == 0) || (i == (height - 1)) ||
-              (j == 0) || (j == (height - 1)))
+          if ((0 == *mask) ||
+              (i == 0) || (i == (height - 1)) ||
+              (j == 0) || (j == (width - 1)))
             {
               /* do nothing at the boundary or outside mask */
               for (k = 0; k < depth; k++)
@@ -430,6 +433,8 @@ gimp_heal_motion (GimpSourceCore   *source_core,
 {
   GimpPaintCore *paint_core = GIMP_PAINT_CORE (source_core);
   GimpContext   *context    = GIMP_CONTEXT (paint_options);
+  GimpDynamics  *dynamics   = GIMP_BRUSH_CORE (paint_core)->dynamics;
+  GimpImage     *image      = gimp_item_get_image (GIMP_ITEM (drawable));
   TempBuf       *src;
   TempBuf       *temp;
   PixelRegion    origPR;
@@ -437,9 +442,16 @@ gimp_heal_motion (GimpSourceCore   *source_core,
   PixelRegion    destPR;
   GimpImageType  src_type;
   TempBuf       *mask_buf;
+  gdouble        fade_point;
   gdouble        hardness;
 
-  hardness = gimp_paint_options_get_dynamic_hardness (paint_options, coords);
+  fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                            paint_core->pixel_dist);
+
+  hardness = gimp_dynamics_output_get_linear_value (dynamics->hardness_output,
+                                                    coords,
+                                                    paint_options,
+                                                    fade_point);
 
   mask_buf = gimp_brush_core_get_brush_mask (GIMP_BRUSH_CORE (source_core),
                                              coords,
@@ -547,8 +559,6 @@ gimp_heal_motion (GimpSourceCore   *source_core,
 
   temp_buf_free (temp);
 
-  opacity *= gimp_paint_options_get_dynamic_opacity (paint_options, coords);
-
   /* replace the canvas with our healed data */
   gimp_brush_core_replace_canvas (GIMP_BRUSH_CORE (paint_core), drawable,
                                   coords,
