@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,13 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <glib-object.h>
+#include <gegl.h>
 
 #include "libgimpconfig/gimpconfig.h"
 
@@ -70,7 +69,7 @@ gimp_ink_options_class_init (GimpInkOptionsClass *klass)
   object_class->get_property = gimp_ink_options_get_property;
 
   GIMP_CONFIG_INSTALL_PROP_DOUBLE (object_class, PROP_SIZE,
-                                   "size", NULL,
+                                   "size", _("Ink Blob Size"),
                                    0.0, 200.0, 16.0,
                                    GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_DOUBLE (object_class, PROP_TILT_ANGLE,
@@ -97,11 +96,11 @@ gimp_ink_options_class_init (GimpInkOptionsClass *klass)
                                  GIMP_INK_BLOB_TYPE_ELLIPSE,
                                  GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_DOUBLE (object_class, PROP_BLOB_ASPECT,
-                                   "blob-aspect", NULL,
+                                   "blob-aspect", _("Ink Blob Aspect Ratio"),
                                    1.0, 10.0, 1.0,
                                    GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_DOUBLE (object_class, PROP_BLOB_ANGLE,
-                                   "blob-angle", NULL,
+                                   "blob-angle", _("Ink Blob Angle"),
                                    -90.0, 90.0, 0.0,
                                    GIMP_PARAM_STATIC_STRINGS);
 }
