@@ -26,32 +26,22 @@
                                         GIMP_CONTEXT_BACKGROUND_MASK | \
                                         GIMP_CONTEXT_OPACITY_MASK    | \
                                         GIMP_CONTEXT_PAINT_MODE_MASK | \
-                                        GIMP_CONTEXT_BRUSH_MASK
+                                        GIMP_CONTEXT_BRUSH_MASK      | \
+                                        GIMP_CONTEXT_DYNAMICS_MASK
 
 
-typedef struct _GimpDynamicOptions  GimpDynamicOptions;
 typedef struct _GimpFadeOptions     GimpFadeOptions;
 typedef struct _GimpJitterOptions   GimpJitterOptions;
 typedef struct _GimpGradientOptions GimpGradientOptions;
 
-struct _GimpDynamicOptions
-{
-  gboolean  opacity;
-  gboolean  hardness;
-  gboolean  rate;
-  gboolean  size;
-  gboolean  inverse_size;
-  gboolean  aspect_ratio;
-  gboolean  color;
-  gboolean  angle;
-  gdouble   prescale;
-};
 
 struct _GimpFadeOptions
 {
-  gboolean  use_fade;
-  gdouble   fade_length;
-  GimpUnit  fade_unit;
+  gboolean        use_fade;
+  gboolean        fade_reverse;
+  gdouble         fade_length;
+  GimpUnit        fade_unit;
+  GimpRepeatMode  fade_repeat;
 };
 
 struct _GimpJitterOptions
@@ -64,8 +54,6 @@ struct _GimpGradientOptions
 {
   gboolean        use_gradient;
   gboolean        gradient_reverse;
-  gdouble         gradient_length;
-  GimpUnit        gradient_unit;
   GimpRepeatMode  gradient_repeat;
 };
 
@@ -77,8 +65,6 @@ struct _GimpGradientOptions
 #define GIMP_IS_PAINT_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GIMP_TYPE_PAINT_OPTIONS))
 #define GIMP_PAINT_OPTIONS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GIMP_TYPE_PAINT_OPTIONS, GimpPaintOptionsClass))
 
-#define GIMP_PAINT_PRESSURE_SCALE 1.5
-#define GIMP_PAINT_VELOCITY_SCALE 1.0
 
 typedef struct _GimpPaintOptionsClass GimpPaintOptionsClass;
 
@@ -97,13 +83,6 @@ struct _GimpPaintOptions
 
   gboolean                  hard;
 
-  gboolean                  dynamics_expanded;
-  GimpDynamicOptions       *pressure_options;
-  GimpDynamicOptions       *velocity_options;
-  GimpDynamicOptions       *direction_options;
-  GimpDynamicOptions       *tilt_options;
-  GimpDynamicOptions       *random_options;
-
   GimpFadeOptions          *fade_options;
   GimpJitterOptions        *jitter_options;
   GimpGradientOptions      *gradient_options;
@@ -143,28 +122,5 @@ GimpBrushApplicationMode
              gimp_paint_options_get_brush_mode (GimpPaintOptions *paint_options);
 
 
-gdouble gimp_paint_options_get_dynamic_opacity (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
-gdouble gimp_paint_options_get_dynamic_size    (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords,
-                                                gboolean          use_dynamics);
-
-gdouble gimp_paint_options_get_dynamic_aspect_ratio
-                                               (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
-gdouble gimp_paint_options_get_dynamic_rate    (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
-gdouble gimp_paint_options_get_dynamic_color   (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
-gdouble gimp_paint_options_get_dynamic_angle   (GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
-gdouble gimp_paint_options_get_dynamic_hardness(GimpPaintOptions *paint_options,
-                                                const GimpCoords *coords);
-
 
 #endif  /*  __GIMP_PAINT_OPTIONS_H__  */
