@@ -31,6 +31,8 @@
 #include "core/gimp.h"
 #include "core/gimpbrush.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimpimage.h"
 #include "core/gimppickable.h"
 
@@ -94,6 +96,7 @@ gimp_smudge_class_init (GimpSmudgeClass *klass)
   paint_core_class->paint = gimp_smudge_paint;
 
   brush_core_class->handles_transforming_brush = TRUE;
+  brush_core_class->handles_dynamic_transforming_brush = FALSE;
 }
 
 static void
@@ -235,29 +238,35 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
                     GimpPaintOptions *paint_options,
                     const GimpCoords *coords)
 {
-  GimpSmudge        *smudge  = GIMP_SMUDGE (paint_core);
-  GimpSmudgeOptions *options = GIMP_SMUDGE_OPTIONS (paint_options);
-  GimpContext       *context = GIMP_CONTEXT (paint_options);
+  GimpSmudge        *smudge   = GIMP_SMUDGE (paint_core);
+  GimpSmudgeOptions *options  = GIMP_SMUDGE_OPTIONS (paint_options);
+  GimpContext       *context  = GIMP_CONTEXT (paint_options);
+  GimpDynamics      *dynamics = GIMP_BRUSH_CORE (paint_core)->dynamics;
   GimpImage         *image;
   TempBuf           *area;
   PixelRegion        srcPR, destPR, tempPR;
-  gdouble            rate;
+  gdouble            fade_point;
   gdouble            opacity;
+  gdouble            rate;
   gdouble            dynamic_rate;
   gint               x, y, w, h;
   gdouble            hardness;
 
-  image = gimp_item_get_image (GIMP_ITEM (drawable));
-
   if (gimp_drawable_is_indexed (drawable))
     return;
 
-  opacity = gimp_paint_options_get_fade (paint_options, image,
-                                         paint_core->pixel_dist);
+  image = gimp_item_get_image (GIMP_ITEM (drawable));
+
+  fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                            paint_core->pixel_dist);
+
+  opacity = gimp_dynamics_output_get_linear_value (dynamics->opacity_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
   if (opacity == 0.0)
     return;
 
-  /*  Get the paint area */
   area = gimp_paint_core_get_paint_area (paint_core, drawable, paint_options,
                                          coords);
   if (! area)
@@ -271,7 +280,11 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
                      area->x, area->y, area->width, area->height, FALSE);
 
   /* Enable dynamic rate */
-  dynamic_rate = gimp_paint_options_get_dynamic_rate (paint_options, coords);
+  dynamic_rate = gimp_dynamics_output_get_linear_value (dynamics->rate_output,
+                                                        coords,
+                                                        paint_options,
+                                                        fade_point);
+
   rate = (options->rate / 100.0) * dynamic_rate;
 
   /* The tempPR will be the built up buffer (for smudge) */
@@ -311,9 +324,10 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
   else
     copy_region (&tempPR, &destPR);
 
-  opacity *= gimp_paint_options_get_dynamic_opacity (paint_options, coords);
-
-  hardness = gimp_paint_options_get_dynamic_hardness (paint_options, coords);
+  hardness = gimp_dynamics_output_get_linear_value (dynamics->hardness_output,
+                                                    coords,
+                                                    paint_options,
+                                                    fade_point);
 
   gimp_brush_core_replace_canvas (GIMP_BRUSH_CORE (paint_core), drawable,
                                   coords,
