@@ -29,6 +29,8 @@
 
 #include "core/gimp.h"
 #include "core/gimpdrawable.h"
+#include "core/gimpdynamics.h"
+#include "core/gimpdynamicsoutput.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimppickable.h"
@@ -345,6 +347,7 @@ gimp_source_core_motion (GimpSourceCore   *source_core,
 {
   GimpPaintCore     *paint_core   = GIMP_PAINT_CORE (source_core);
   GimpSourceOptions *options      = GIMP_SOURCE_OPTIONS (paint_options);
+  GimpDynamics      *dynamics     = GIMP_BRUSH_CORE (paint_core)->dynamics;
   GimpImage         *image        = gimp_item_get_image (GIMP_ITEM (drawable));
   GimpPickable      *src_pickable = NULL;
   PixelRegion        srcPR;
@@ -355,10 +358,16 @@ gimp_source_core_motion (GimpSourceCore   *source_core,
   gint               paint_area_offset_y;
   gint               paint_area_width;
   gint               paint_area_height;
+  gdouble            fade_point;
   gdouble            opacity;
 
-  opacity = gimp_paint_options_get_fade (paint_options, image,
-                                         paint_core->pixel_dist);
+  fade_point = gimp_paint_options_get_fade (paint_options, image,
+                                            paint_core->pixel_dist);
+
+  opacity = gimp_dynamics_output_get_linear_value (dynamics->opacity_output,
+                                                   coords,
+                                                   paint_options,
+                                                   fade_point);
   if (opacity == 0.0)
     return;
 
@@ -377,7 +386,7 @@ gimp_source_core_motion (GimpSourceCore   *source_core,
           src_pickable = GIMP_PICKABLE (gimp_image_get_projection (src_image));
 
           gimp_item_get_offset (GIMP_ITEM (source_core->src_drawable),
-                             &off_x, &off_y);
+                                &off_x, &off_y);
 
           src_offset_x += off_x;
           src_offset_y += off_y;
