@@ -95,7 +95,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a CONFIG message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -108,7 +108,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TILE_ACK message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -117,7 +117,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TILE_DATA message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -134,7 +134,7 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RUN message.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       break;
@@ -236,7 +236,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried writing to invalid drawable %d (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     tile_info->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -248,7 +248,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried writing to drawable %d which was removed "
                     "from the image (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     tile_info->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -257,12 +257,40 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
 
   if (tile_info->shadow)
     {
+      /*  don't check whether the drawable is a group or locked here,
+       *  the plugin will get a proper error message when it tries to
+       *  merge the shadow tiles, which is much better than just
+       *  killing it.
+       */
       tm = gimp_drawable_get_shadow_tiles (drawable);
 
       gimp_plug_in_cleanup_add_shadow (plug_in, drawable);
     }
   else
     {
+      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+        {
+          gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "tried writing to a locked drawable %d (killing)",
+                        gimp_object_get_name (plug_in),
+                        gimp_filename_to_utf8 (plug_in->prog),
+                        tile_info->drawable_ID);
+          gimp_plug_in_close (plug_in, TRUE);
+          return;
+        }
+      else if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+        {
+          gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "tried writing to a group layer %d (killing)",
+                        gimp_object_get_name (plug_in),
+                        gimp_filename_to_utf8 (plug_in->prog),
+                        tile_info->drawable_ID);
+          gimp_plug_in_close (plug_in, TRUE);
+          return;
+        }
+
       tm = gimp_drawable_get_tiles (drawable);
     }
 
@@ -273,7 +301,7 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "requested invalid tile (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       return;
@@ -318,7 +346,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried reading from invalid drawable %d (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     request->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -330,7 +358,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "tried reading from drawable %d which was removed "
                     "from the image (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog),
                     request->drawable_ID);
       gimp_plug_in_close (plug_in, TRUE);
@@ -355,7 +383,7 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "requested invalid tile (killing)",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
       return;
@@ -478,7 +506,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
                             "Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical, proc_name);
             }
@@ -493,7 +521,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
               gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
                             "Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical);
             }
@@ -503,7 +531,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
                             "WARNING: Plug-In \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
-                            gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                            gimp_object_get_name (plug_in),
                             gimp_filename_to_utf8 (plug_in->prog),
                             canonical, procedure->deprecated);
             }
@@ -621,7 +649,7 @@ gimp_plug_in_handle_temp_proc_return (GimpPlugIn   *plug_in,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RETURN message while not running "
                     "a temporary procedure.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
@@ -660,7 +688,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
                         "attempted to install procedure \"%s\" "
                         "which fails to comply with the array parameter "
                         "passing standard.  Argument %d is noncompliant.",
-                        gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                        gimp_object_get_name (plug_in),
                         gimp_filename_to_utf8 (plug_in->prog),
                         canonical, i);
           g_free (canonical);
@@ -719,7 +747,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "attempted to install a procedure NULL parameter name.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       g_free (canonical);
       return;
@@ -730,7 +758,7 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "attempted to install a procedure with invalid UTF-8 strings.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       g_free (canonical);
       return;
@@ -864,7 +892,7 @@ gimp_plug_in_handle_extension_ack (GimpPlugIn *plug_in)
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent an EXTENSION_ACK message while not being started "
                     "as an extension.  This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
@@ -883,7 +911,7 @@ gimp_plug_in_handle_has_init (GimpPlugIn *plug_in)
                     "Plug-In \"%s\"\n(%s)\n\n"
                     "sent an HAS_INIT message while not in query().  "
                     "This should not happen.",
-                    gimp_object_get_name (GIMP_OBJECT (plug_in)),
+                    gimp_object_get_name (plug_in),
                     gimp_filename_to_utf8 (plug_in->prog));
       gimp_plug_in_close (plug_in, TRUE);
     }
