@@ -207,12 +207,6 @@ gimp_plug_in_manager_finalize (GObject *object)
       manager->plug_in_defs = NULL;
     }
 
-  if (manager->shm)
-    {
-      gimp_plug_in_shm_free (manager->shm);
-      manager->shm = NULL;
-    }
-
   if (manager->environ_table)
     {
       g_object_unref (manager->environ_table);
@@ -331,6 +325,15 @@ gimp_plug_in_manager_exit (GimpPlugInManager *manager)
 
   while (manager->open_plug_ins)
     gimp_plug_in_close (manager->open_plug_ins->data, TRUE);
+
+  /*  need to deatch from shared memory, we can't rely on exit()
+   *  cleaning up behind us (see bug #609026)
+   */
+  if (manager->shm)
+    {
+      gimp_plug_in_shm_free (manager->shm);
+      manager->shm = NULL;
+    }
 }
 
 void
@@ -346,8 +349,8 @@ gimp_plug_in_manager_add_procedure (GimpPlugInManager   *manager,
     {
       GimpPlugInProcedure *tmp_proc = list->data;
 
-      if (strcmp (GIMP_OBJECT (procedure)->name,
-                  GIMP_OBJECT (tmp_proc)->name) == 0)
+      if (strcmp (gimp_object_get_name (procedure),
+                  gimp_object_get_name (tmp_proc)) == 0)
         {
           GSList *list2;
 
@@ -355,7 +358,7 @@ gimp_plug_in_manager_add_procedure (GimpPlugInManager   *manager,
 
           g_printerr ("Removing duplicate PDB procedure '%s' "
                       "registered by '%s'\n",
-                      GIMP_OBJECT (tmp_proc)->name,
+                      gimp_object_get_name (tmp_proc),
                       gimp_filename_to_utf8 (tmp_proc->prog));
 
           /* search the plugin list to see if any plugins had references to
