@@ -150,7 +150,9 @@ text_get_extents (const gchar *fontname,
   g_return_val_if_fail (fontname != NULL, FALSE);
   g_return_val_if_fail (text != NULL, FALSE);
 
-  fontmap = PANGO_CAIRO_FONT_MAP (pango_cairo_font_map_new ());
+  fontmap = PANGO_CAIRO_FONT_MAP (pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT));
+  if (fontmap == NULL)
+    g_error ("You are using a Pango that has been built against a cairo that lacks the Freetype font backend");
   pango_cairo_font_map_set_resolution (fontmap, 72.0); /* FIXME: resolution */
   context = pango_cairo_font_map_create_context (fontmap);
   g_object_unref (fontmap);
