@@ -25,6 +25,7 @@
 #include <gegl.h>
 #include <pango/pangocairo.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpconfig/gimpconfig.h"
 
 #include "text-types.h"
@@ -35,11 +36,13 @@
 #include "paint-funcs/paint-funcs.h"
 
 #include "core/gimp.h"
+#include "core/gimp-utils.h"
 #include "core/gimpcontext.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
+#include "core/gimpitemtree.h"
 #include "core/gimpparasitelist.h"
 
 #include "gimptext.h"
@@ -95,7 +98,7 @@ static void       gimp_text_layer_push_undo      (GimpDrawable    *drawable,
                                                   gint             width,
                                                   gint             height);
 
-static void       gimp_text_layer_text_notify    (GimpTextLayer   *layer);
+static void       gimp_text_layer_text_changed   (GimpTextLayer   *layer);
 static gboolean   gimp_text_layer_render         (GimpTextLayer   *layer);
 static void       gimp_text_layer_render_layout  (GimpTextLayer   *layer,
                                                   GimpTextLayout  *layout);
@@ -349,6 +352,8 @@ gimp_text_layer_push_undo (GimpDrawable *drawable,
 }
 
 
+/*  public functions  */
+
 /**
  * gimp_text_layer_new:
  * @image: the #GimpImage the layer should belong to
@@ -367,7 +372,7 @@ gimp_text_layer_new (GimpImage *image,
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_TEXT (text), NULL);
 
-  if (! text->text)
+  if (! text->text && ! text->markup)
     return NULL;
 
   layer = g_object_new (GIMP_TYPE_TEXT_LAYER, NULL);
@@ -402,7 +407,7 @@ gimp_text_layer_set_text (GimpTextLayer *layer,
   if (layer->text)
     {
       g_signal_handlers_disconnect_by_func (layer->text,
-                                            G_CALLBACK (gimp_text_layer_text_notify),
+                                            G_CALLBACK (gimp_text_layer_text_changed),
                                             layer);
 
       g_object_unref (layer->text);
@@ -413,8 +418,8 @@ gimp_text_layer_set_text (GimpTextLayer *layer,
     {
       layer->text = g_object_ref (text);
 
-      g_signal_connect_object (text, "notify",
-                               G_CALLBACK (gimp_text_layer_text_notify),
+      g_signal_connect_object (text, "changed",
+                               G_CALLBACK (gimp_text_layer_text_changed),
                                layer, G_CONNECT_SWAPPED);
     }
 
@@ -455,7 +460,16 @@ gimp_text_layer_set (GimpTextLayer *layer,
   if (layer->modified)
     {
       gimp_image_undo_push_text_layer_modified (image, NULL, layer);
-      gimp_image_undo_push_drawable_mod (image, NULL, GIMP_DRAWABLE (layer));
+
+      /*  pass copy_tiles = TRUE so we not only ref the tiles; after
+       *  being a text layer again, undo doesn't care about the
+       *  layer's pixels any longer because they are generated, so
+       *  changing the text would happily overwrite the layer's
+       *  pixels, changing the pixels on the undo stack too without
+       *  any chance to ever undo again.
+       */
+      gimp_image_undo_push_drawable_mod (image, NULL,
+                                         GIMP_DRAWABLE (layer), TRUE);
     }
 
   gimp_image_undo_push_text_layer (image, undo_desc, layer, NULL);
@@ -505,8 +519,10 @@ gimp_drawable_is_text_layer (GimpDrawable *drawable)
 }
 
 
+/*  private functions  */
+
 static void
-gimp_text_layer_text_notify (GimpTextLayer *layer)
+gimp_text_layer_text_changed (GimpTextLayer *layer)
 {
   /*   If the text layer was created from a parasite, it's time to
    *   remove that parasite now.
@@ -576,9 +592,35 @@ gimp_text_layer_render (GimpTextLayer *layer)
     }
 
   if (layer->auto_rename)
-    gimp_object_set_name_safe (GIMP_OBJECT (layer),
-                               layer->text->text ?
-                               layer->text->text : _("Empty Text Layer"));
+    {
+      GimpItem *item = GIMP_ITEM (layer);
+      gchar    *name = NULL;
+
+      if (layer->text->text)
+        {
+          name = gimp_utf8_strtrim (layer->text->text, 30);
+        }
+      else if (layer->text->markup)
+        {
+          gchar *tmp = gimp_markup_extract_text (layer->text->markup);
+          name = gimp_utf8_strtrim (tmp, 30);
+          g_free (tmp);
+        }
+
+      if (! name)
+        name = g_strdup (_("Empty Text Layer"));
+
+      if (gimp_item_is_attached (item))
+        {
+          gimp_item_tree_rename_item (gimp_item_get_tree (item), item,
+                                      name, FALSE, NULL);
+          g_free (name);
+        }
+      else
+        {
+          gimp_object_take_name (GIMP_OBJECT (layer), name);
+        }
+    }
 
   gimp_text_layer_render_layout (layer, layout);
 
@@ -615,11 +657,13 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
 
   cr = cairo_create (surface);
 
-  gimp_text_layout_render (layout, cr, FALSE);
+  gimp_text_layout_render (layout, cr, layer->text->base_dir, FALSE);
 
   mask = tile_manager_new ( width, height, 1);
   pixel_region_init (&maskPR, mask, 0, 0, width, height, TRUE);
 
+  cairo_surface_flush (surface);
+
   data      = cairo_image_surface_get_data (surface);
   rowstride = cairo_image_surface_get_stride (surface);
 
