@@ -23,12 +23,12 @@
 #include <gegl.h>
 #include <pango/pangocairo.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "text-types.h"
 
 #include "core/gimpimage.h"
-#include "core/gimpunit.h"
 
 #include "gimptext.h"
 #include "gimptextlayout.h"
@@ -54,12 +54,10 @@ static PangoContext * gimp_text_get_pango_context (GimpText       *text,
                                                    gdouble         xres,
                                                    gdouble         yres);
 
-static gint           gimp_text_layout_pixel_size (Gimp           *gimp,
-                                                   gdouble         value,
+static gint           gimp_text_layout_pixel_size (gdouble         value,
                                                    GimpUnit        unit,
                                                    gdouble         res);
-static gint           gimp_text_layout_point_size (Gimp           *gimp,
-                                                   gdouble         value,
+static gint           gimp_text_layout_point_size (gdouble         value,
                                                    GimpUnit        unit,
                                                    gdouble         res);
 
@@ -123,8 +121,7 @@ gimp_text_layout_new (GimpText  *text,
 
   gimp_image_get_resolution (image, &xres, &yres);
 
-  size = gimp_text_layout_point_size (image->gimp,
-                                      text->font_size,
+  size = gimp_text_layout_point_size (text->font_size,
                                       text->unit,
                                       yres);
 
@@ -146,7 +143,9 @@ gimp_text_layout_new (GimpText  *text,
   pango_layout_set_font_description (layout->layout, font_desc);
   pango_font_description_free (font_desc);
 
-  if (text->text)
+  if (text->markup)
+    pango_layout_set_markup (layout->layout, text->markup, -1);
+  else if (text->text)
     pango_layout_set_text (layout->layout, text->text, -1);
   else
     pango_layout_set_text (layout->layout, NULL, 0);
@@ -176,28 +175,31 @@ gimp_text_layout_new (GimpText  *text,
       break;
     case GIMP_TEXT_BOX_FIXED:
       pango_layout_set_width (layout->layout,
-                              gimp_text_layout_pixel_size (image->gimp,
-                                                           text->box_width,
+                              gimp_text_layout_pixel_size (text->box_width,
                                                            text->box_unit,
                                                            xres));
       break;
     }
 
   pango_layout_set_indent (layout->layout,
-                           gimp_text_layout_pixel_size (image->gimp,
-                                                        text->indent,
+                           gimp_text_layout_pixel_size (text->indent,
                                                         text->unit,
                                                         xres));
   pango_layout_set_spacing (layout->layout,
-                            gimp_text_layout_pixel_size (image->gimp,
-                                                         text->line_spacing,
+                            gimp_text_layout_pixel_size (text->line_spacing,
                                                          text->unit,
                                                          yres));
   if (fabs (text->letter_spacing) > 0.1)
     {
-      PangoAttrList  *attrs = pango_attr_list_new ();
+      PangoAttrList  *attrs;
       PangoAttribute *attr;
 
+      attrs = pango_layout_get_attributes (layout->layout);
+      if (attrs)
+        pango_attr_list_ref (attrs);
+      else
+        attrs = pango_attr_list_new ();
+
       attr = pango_attr_letter_spacing_new (text->letter_spacing * PANGO_SCALE);
 
       attr->start_index = 0;
@@ -217,15 +219,20 @@ gimp_text_layout_new (GimpText  *text,
       break;
     case GIMP_TEXT_BOX_FIXED:
       layout->extents.width =
-        PANGO_PIXELS (gimp_text_layout_pixel_size (image->gimp,
-                                                   text->box_width,
+        PANGO_PIXELS (gimp_text_layout_pixel_size (text->box_width,
                                                    text->box_unit,
                                                    xres));
       layout->extents.height =
-        PANGO_PIXELS (gimp_text_layout_pixel_size (image->gimp,
-                                                   text->box_height,
+        PANGO_PIXELS (gimp_text_layout_pixel_size (text->box_height,
                                                    text->box_unit,
                                                    yres));
+
+/* #define VERBOSE */
+
+#ifdef VERBOSE
+      g_printerr ("extents set to %d x %d\n",
+                  layout->extents.width, layout->extents.height);
+#endif
       break;
     }
 
@@ -488,10 +495,10 @@ gimp_text_layout_position (GimpTextLayout *layout)
   logical.width = ceil ((gdouble) logical.width * layout->xres / layout->yres);
 
 #ifdef VERBOSE
-  g_print ("ink rect: %d x %d @ %d, %d\n",
-           ink.width, ink.height, ink.x, ink.y);
-  g_print ("logical rect: %d x %d @ %d, %d\n",
-           logical.width, logical.height, logical.x, logical.y);
+  g_printerr ("ink rect: %d x %d @ %d, %d\n",
+              ink.width, ink.height, ink.x, ink.y);
+  g_printerr ("logical rect: %d x %d @ %d, %d\n",
+              logical.width, logical.height, logical.x, logical.y);
 #endif
 
   if (ink.width < 1 || ink.height < 1)
@@ -507,6 +514,31 @@ gimp_text_layout_position (GimpTextLayout *layout)
   layout->extents.width  = x2 - x1;
   layout->extents.height = y2 - y1;
 
+  /* If the width of the layout is > 0, then the text-box is FIXED and
+   * the layout position should be offset if the alignment is centered
+   * or right-aligned, also adjust for RTL text direction.
+   */
+  if (pango_layout_get_width (layout->layout) > 0)
+    {
+      PangoAlignment    align    = pango_layout_get_alignment (layout->layout);
+      GimpTextDirection base_dir = layout->text->base_dir;
+      gint              width;
+
+      pango_layout_get_pixel_size (layout->layout, &width, NULL);
+
+      if ((base_dir == GIMP_TEXT_DIRECTION_LTR && align == PANGO_ALIGN_RIGHT) ||
+          (base_dir == GIMP_TEXT_DIRECTION_RTL && align == PANGO_ALIGN_LEFT))
+        {
+          layout->extents.x +=
+            PANGO_PIXELS (pango_layout_get_width (layout->layout)) - width;
+        }
+      else if (align == PANGO_ALIGN_CENTER)
+        {
+          layout->extents.x +=
+            (PANGO_PIXELS (pango_layout_get_width (layout->layout)) - width) / 2;
+       }
+    }
+
   if (layout->text->border > 0)
     {
       gint border = layout->text->border;
@@ -518,9 +550,9 @@ gimp_text_layout_position (GimpTextLayout *layout)
     }
 
 #ifdef VERBOSE
-  g_print ("layout extents: %d x %d @ %d, %d\n",
-           layout->extents.width, layout->extents.height,
-           layout->extents.x, layout->extents.y);
+  g_printerr ("layout extents: %d x %d @ %d, %d\n",
+              layout->extents.width, layout->extents.height,
+              layout->extents.x, layout->extents.y);
 #endif
 }
 
@@ -564,7 +596,9 @@ gimp_text_get_pango_context (GimpText *text,
   PangoCairoFontMap    *fontmap;
   cairo_font_options_t *options;
 
-  fontmap = PANGO_CAIRO_FONT_MAP (pango_cairo_font_map_new ());
+  fontmap = PANGO_CAIRO_FONT_MAP (pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT));
+  if (fontmap == NULL)
+    g_error ("You are using a Pango that has been built against a cairo that lacks the Freetype font backend");
 
   pango_cairo_font_map_set_resolution (fontmap, yres);
 
@@ -594,10 +628,9 @@ gimp_text_get_pango_context (GimpText *text,
 }
 
 static gint
-gimp_text_layout_pixel_size (Gimp     *gimp,
-                             gdouble   value,
-                             GimpUnit  unit,
-                             gdouble   res)
+gimp_text_layout_pixel_size (gdouble  value,
+                             GimpUnit unit,
+                             gdouble  res)
 {
   gdouble factor;
 
@@ -607,7 +640,7 @@ gimp_text_layout_pixel_size (Gimp     *gimp,
       return PANGO_SCALE * value;
 
     default:
-      factor = _gimp_unit_get_factor (gimp, unit);
+      factor = gimp_unit_get_factor (unit);
       g_return_val_if_fail (factor > 0.0, 0);
 
       return PANGO_SCALE * value * res / factor;
@@ -615,10 +648,9 @@ gimp_text_layout_pixel_size (Gimp     *gimp,
 }
 
 static gint
-gimp_text_layout_point_size (Gimp     *gimp,
-                             gdouble   value,
-                             GimpUnit  unit,
-                             gdouble   res)
+gimp_text_layout_point_size (gdouble  value,
+                             GimpUnit unit,
+                             gdouble  res)
 {
   gdouble factor;
 
@@ -630,13 +662,13 @@ gimp_text_layout_point_size (Gimp     *gimp,
     case GIMP_UNIT_PIXEL:
       g_return_val_if_fail (res > 0.0, 0);
       return (PANGO_SCALE * value *
-              _gimp_unit_get_factor (gimp, GIMP_UNIT_POINT) / res);
+              gimp_unit_get_factor (GIMP_UNIT_POINT) / res);
 
     default:
-      factor = _gimp_unit_get_factor (gimp, unit);
+      factor = gimp_unit_get_factor (unit);
       g_return_val_if_fail (factor > 0.0, 0);
 
       return (PANGO_SCALE * value *
-              _gimp_unit_get_factor (gimp, GIMP_UNIT_POINT) / factor);
+              gimp_unit_get_factor (GIMP_UNIT_POINT) / factor);
     }
 }
