@@ -213,6 +213,7 @@ void
 gimp_tools_exit (Gimp *gimp)
 {
   GList *default_order;
+  GList *list;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
@@ -224,6 +225,22 @@ gimp_tools_exit (Gimp *gimp)
 
   g_object_set_data (G_OBJECT (gimp), "gimp-tools-default-order", NULL);
 
+  for (list = gimp_get_tool_info_iter (gimp);
+       list;
+       list = g_list_next (list))
+    {
+      GimpToolInfo *tool_info = list->data;
+      GtkWidget    *options_gui;
+
+      options_gui = g_object_get_data (G_OBJECT (tool_info->tool_options),
+                                       "gimp-tool-options-gui");
+
+      gtk_widget_destroy (options_gui);
+
+      g_object_set_data (G_OBJECT (tool_info->tool_options),
+                         "gimp-tool-options-gui", NULL);
+    }
+
   tool_manager_exit (gimp);
 }
 
@@ -303,9 +320,19 @@ gimp_tools_restore (Gimp *gimp)
       GimpToolOptionsGUIFunc  options_gui_func;
       GtkWidget              *options_gui;
 
+      /*  copy all context properties except those the tool actually
+       *  uses, because the subsequent deserialize() on the tool
+       *  options will only set the properties that were set to
+       *  non-default values at the time of saving, and we want to
+       *  keep these default values as if they have been saved.
+       * (see bug #541586).
+       */
       gimp_context_copy_properties (gimp_get_user_context (gimp),
                                     GIMP_CONTEXT (tool_info->tool_options),
-                                    GIMP_CONTEXT_ALL_PROPS_MASK);
+                                    GIMP_CONTEXT_ALL_PROPS_MASK &~
+                                    (tool_info->context_props |
+                                     GIMP_CONTEXT_TOOL_MASK   |
+                                     GIMP_CONTEXT_PAINT_INFO_MASK));
 
       gimp_tool_options_deserialize (tool_info->tool_options, NULL);
 
@@ -331,8 +358,10 @@ gimp_tools_restore (Gimp *gimp)
           gtk_widget_show (label);
         }
 
-      g_object_set_data (G_OBJECT (tool_info->tool_options),
-                         "gimp-tool-options-gui", options_gui);
+      g_object_set_data_full (G_OBJECT (tool_info->tool_options),
+                              "gimp-tool-options-gui",
+                              g_object_ref_sink (options_gui),
+                              (GDestroyNotify) g_object_unref);
 
       if (tool_info->presets)
         gimp_tool_presets_load (tool_info->presets, NULL);
