@@ -153,7 +153,8 @@ gimp_blend_tool_initialize (GimpTool     *tool,
                             GimpDisplay  *display,
                             GError      **error)
 {
-  GimpDrawable *drawable = gimp_image_get_active_drawable (display->image);
+  GimpImage    *image    = gimp_display_get_image (display);
+  GimpDrawable *drawable = gimp_image_get_active_drawable (image);
 
   if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
     {
@@ -167,6 +168,20 @@ gimp_blend_tool_initialize (GimpTool     *tool,
       return FALSE;
     }
 
+  if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+    {
+      g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
+			   _("Cannot modify the pixels of layer groups."));
+      return FALSE;
+    }
+
+  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+    {
+      g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
+			   _("The active layer's pixels are locked."));
+      return FALSE;
+    }
+
   return TRUE;
 }
 
@@ -207,7 +222,7 @@ gimp_blend_tool_button_release (GimpTool              *tool,
   GimpBlendOptions *options       = GIMP_BLEND_TOOL_GET_OPTIONS (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_OPTIONS (options);
   GimpContext      *context       = GIMP_CONTEXT (options);
-  GimpImage        *image         = display->image;
+  GimpImage        *image         = gimp_display_get_image (display);
 
   gimp_tool_pop_status (tool, display);
 
@@ -253,6 +268,9 @@ gimp_blend_tool_button_release (GimpTool              *tool,
 
       gimp_image_flush (image);
     }
+
+  tool->display  = NULL;
+  tool->drawable = NULL;
 }
 
 static void
@@ -340,19 +358,19 @@ gimp_blend_tool_cursor_update (GimpTool         *tool,
                                GdkModifierType   state,
                                GimpDisplay      *display)
 {
-  switch (gimp_drawable_type (gimp_image_get_active_drawable (display->image)))
+  GimpImage          *image    = gimp_display_get_image (display);
+  GimpDrawable       *drawable = gimp_image_get_active_drawable (image);
+  GimpCursorModifier  modifier = GIMP_CURSOR_MODIFIER_NONE;
+
+  if (gimp_drawable_is_indexed (drawable)                   ||
+      gimp_viewable_get_children (GIMP_VIEWABLE (drawable)) ||
+      gimp_item_is_content_locked (GIMP_ITEM (drawable)))
     {
-    case GIMP_INDEXED_IMAGE:
-    case GIMP_INDEXEDA_IMAGE:
-      gimp_tool_control_set_cursor_modifier (tool->control,
-                                             GIMP_CURSOR_MODIFIER_BAD);
-      break;
-    default:
-      gimp_tool_control_set_cursor_modifier (tool->control,
-                                             GIMP_CURSOR_MODIFIER_NONE);
-      break;
+      modifier = GIMP_CURSOR_MODIFIER_BAD;
     }
 
+  gimp_tool_control_set_cursor_modifier (tool->control, modifier);
+
   GIMP_TOOL_CLASS (parent_class)->cursor_update (tool, coords, state, display);
 }
 
