@@ -207,18 +207,20 @@ gimp_brush_tool_oper_update (GimpTool         *tool,
 {
   GimpBrushTool    *brush_tool    = GIMP_BRUSH_TOOL (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_TOOL_GET_OPTIONS (tool);
+  GimpDrawable     *drawable      = gimp_image_get_active_drawable (gimp_display_get_image (display));
 
   gimp_draw_tool_pause (GIMP_DRAW_TOOL (tool));
 
   GIMP_TOOL_CLASS (parent_class)->oper_update (tool, coords, state,
                                                proximity, display);
 
-  if (! gimp_color_tool_is_enabled (GIMP_COLOR_TOOL (tool)) &&
-      gimp_image_get_active_drawable (display->image) && proximity)
+  if (! gimp_color_tool_is_enabled (GIMP_COLOR_TOOL (tool))             &&
+        drawable && proximity)
     {
       GimpPaintTool *paint_tool = GIMP_PAINT_TOOL (tool);
       GimpBrushCore *brush_core = GIMP_BRUSH_CORE (paint_tool->core);
       GimpBrush     *brush;
+      GimpDynamics  *dynamics;
 
       brush_tool->brush_x = coords->x;
       brush_tool->brush_y = coords->y;
@@ -227,6 +229,19 @@ gimp_brush_tool_oper_update (GimpTool         *tool,
 
       if (brush_core->main_brush != brush)
         gimp_brush_core_set_brush (brush_core, brush);
+
+      dynamics = gimp_context_get_dynamics (GIMP_CONTEXT (paint_options));
+
+      if (brush_core->dynamics != dynamics)
+        gimp_brush_core_set_dynamics (brush_core, dynamics);
+
+      if (GIMP_BRUSH_CORE_GET_CLASS (brush_core)->handles_transforming_brush)
+        {
+          gimp_brush_core_eval_transform_dynamics (paint_tool->core,
+                                                   drawable,
+                                                   paint_options,
+                                                   coords);
+        }
     }
 
   gimp_draw_tool_resume (GIMP_DRAW_TOOL (tool));
@@ -301,10 +316,13 @@ gimp_brush_tool_draw_brush (GimpBrushTool *brush_tool,
     }
 
   if (brush_core->brush_bound_segs)
+      gimp_brush_core_transform_bound_segs (brush_core, options);
+
+  if (brush_core->transformed_brush_bound_segs)
     {
-      GimpDisplayShell *shell  = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
-      gdouble           width  = brush_core->brush_bound_width;
-      gdouble           height = brush_core->brush_bound_height;
+      GimpDisplayShell *shell  = gimp_display_get_shell (draw_tool->display);
+      gdouble           width  = brush_core->transformed_brush_bound_width;
+      gdouble           height = brush_core->transformed_brush_bound_height;
 
       /*  don't draw the boundary if it becomes too small  */
       if (SCALEX (shell, width) > 4 && SCALEY (shell, height) > 4)
@@ -325,7 +343,7 @@ gimp_brush_tool_draw_brush (GimpBrushTool *brush_tool,
             }
 
           gimp_draw_tool_draw_boundary (draw_tool,
-                                        brush_core->brush_bound_segs,
+                                        brush_core->transformed_brush_bound_segs,
                                         brush_core->n_brush_bound_segs,
                                         x, y,
                                         FALSE);
@@ -350,6 +368,7 @@ gimp_brush_tool_brush_changed (GimpContext   *context,
 
   if (brush_core->main_brush != brush)
     gimp_brush_core_set_brush (brush_core, brush);
+
 }
 
 static void
@@ -368,7 +387,16 @@ gimp_brush_tool_set_brush (GimpBrushCore *brush_core,
                            GimpBrush     *brush,
                            GimpBrushTool *brush_tool)
 {
+  GimpPaintCore *paint_core = GIMP_PAINT_CORE (brush_core);
   gimp_draw_tool_pause (GIMP_DRAW_TOOL (brush_tool));
+
+  if (GIMP_BRUSH_CORE_GET_CLASS (brush_core)->handles_transforming_brush)
+    {
+      gimp_brush_core_eval_transform_dynamics (paint_core,
+                                               NULL,
+                                               GIMP_PAINT_TOOL_GET_OPTIONS (brush_tool),
+                                               &paint_core->cur_coords);
+    }
 }
 
 static void
