@@ -149,10 +149,9 @@ gimp_color_balance_tool_initialize (GimpTool     *tool,
                                     GimpDisplay  *display,
                                     GError      **error)
 {
-  GimpColorBalanceTool *cb_tool = GIMP_COLOR_BALANCE_TOOL (tool);
-  GimpDrawable         *drawable;
-
-  drawable = gimp_image_get_active_drawable (display->image);
+  GimpColorBalanceTool *cb_tool  = GIMP_COLOR_BALANCE_TOOL (tool);
+  GimpImage            *image    = gimp_display_get_image (display);
+  GimpDrawable         *drawable = gimp_image_get_active_drawable (image);
 
   if (! drawable)
     return FALSE;
@@ -166,9 +165,7 @@ gimp_color_balance_tool_initialize (GimpTool     *tool,
 
   gimp_config_reset (GIMP_CONFIG (cb_tool->config));
 
-  GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
-
-  return TRUE;
+  return GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
 }
 
 static GeglNode *
