@@ -145,7 +145,8 @@ gimp_colorize_tool_initialize (GimpTool     *tool,
                                GError      **error)
 {
   GimpColorizeTool *col_tool = GIMP_COLORIZE_TOOL (tool);
-  GimpDrawable     *drawable = gimp_image_get_active_drawable (display->image);
+  GimpImage        *image    = gimp_display_get_image (display);
+  GimpDrawable     *drawable = gimp_image_get_active_drawable (image);
 
   if (! drawable)
     return FALSE;
@@ -159,7 +160,10 @@ gimp_colorize_tool_initialize (GimpTool     *tool,
 
   gimp_config_reset (GIMP_CONFIG (col_tool->config));
 
-  GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
+  if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
+    {
+      return FALSE;
+    }
 
   gimp_image_map_tool_preview (GIMP_IMAGE_MAP_TOOL (tool));
 
