@@ -1,9 +1,9 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis, and others
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -37,7 +36,6 @@ enum
   PROP_0,
   PROP_SAMPLE_AVERAGE, /* overrides a GimpColorOptions property */
   PROP_PICK_MODE,
-  PROP_ADD_TO_PALETTE,
   PROP_USE_INFO_WINDOW
 };
 
@@ -74,10 +72,6 @@ gimp_color_picker_options_class_init (GimpColorPickerOptionsClass *klass)
                                  GIMP_TYPE_COLOR_PICK_MODE,
                                  GIMP_COLOR_PICK_MODE_FOREGROUND,
                                  GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_ADD_TO_PALETTE,
-                                    "add-to-palette", NULL,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_INFO_WINDOW,
                                     "use-info-window", NULL,
                                     FALSE,
@@ -105,12 +99,10 @@ gimp_color_picker_options_set_property (GObject      *object,
     case PROP_PICK_MODE:
       options->pick_mode = g_value_get_enum (value);
       break;
-    case PROP_ADD_TO_PALETTE:
-      options->add_to_palette = g_value_get_boolean (value);
-      break;
     case PROP_USE_INFO_WINDOW:
       options->use_info_window = g_value_get_boolean (value);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -134,12 +126,10 @@ gimp_color_picker_options_get_property (GObject    *object,
     case PROP_PICK_MODE:
       g_value_set_enum (value, options->pick_mode);
       break;
-    case PROP_ADD_TO_PALETTE:
-      g_value_set_boolean (value, options->add_to_palette);
-      break;
     case PROP_USE_INFO_WINDOW:
       g_value_set_boolean (value, options->use_info_window);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
