@@ -200,7 +200,7 @@ gimp_color_tool_control (GimpTool       *tool,
                          GimpDisplay    *display)
 {
   GimpColorTool    *color_tool = GIMP_COLOR_TOOL (tool);
-  GimpDisplayShell *shell      = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell      = gimp_display_get_shell (display);
 
   switch (action)
     {
@@ -210,15 +210,15 @@ gimp_color_tool_control (GimpTool       *tool,
     case GIMP_TOOL_ACTION_RESUME:
       if (color_tool->sample_point &&
           gimp_display_shell_get_show_sample_points (shell))
-        gimp_display_shell_draw_sample_point (shell,
-                                              color_tool->sample_point, TRUE);
+        gimp_display_shell_draw_sample_point (shell, color_tool->sample_point,
+                                              NULL, TRUE);
       break;
 
     case GIMP_TOOL_ACTION_HALT:
       if (color_tool->sample_point &&
           gimp_display_shell_get_show_sample_points (shell))
-        gimp_display_shell_draw_sample_point (shell,
-                                              color_tool->sample_point, FALSE);
+        gimp_display_shell_draw_sample_point (shell, color_tool->sample_point,
+                                              NULL, FALSE);
       break;
     }
 
@@ -234,7 +234,7 @@ gimp_color_tool_button_press (GimpTool            *tool,
                               GimpDisplay         *display)
 {
   GimpColorTool    *color_tool = GIMP_COLOR_TOOL (tool);
-  GimpDisplayShell *shell      = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell      = gimp_display_get_shell (display);
 
   /*  Chain up to activate the tool  */
   GIMP_TOOL_CLASS (parent_class)->button_press (tool, coords, time, state,
@@ -289,7 +289,8 @@ gimp_color_tool_button_release (GimpTool              *tool,
                                 GimpDisplay           *display)
 {
   GimpColorTool    *color_tool = GIMP_COLOR_TOOL (tool);
-  GimpDisplayShell *shell      = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell      = gimp_display_get_shell (display);
+  GimpImage        *image      = gimp_display_get_image (display);
 
   /*  Chain up to halt the tool  */
   GIMP_TOOL_CLASS (parent_class)->button_release (tool, coords, time, state,
@@ -326,7 +327,7 @@ gimp_color_tool_button_release (GimpTool              *tool,
         {
           if (color_tool->sample_point)
             {
-              gimp_image_remove_sample_point (display->image,
+              gimp_image_remove_sample_point (image,
                                               color_tool->sample_point, TRUE);
               color_tool->sample_point = NULL;
             }
@@ -335,7 +336,7 @@ gimp_color_tool_button_release (GimpTool              *tool,
         {
           if (color_tool->sample_point)
             {
-              gimp_image_move_sample_point (display->image,
+              gimp_image_move_sample_point (image,
                                             color_tool->sample_point,
                                             color_tool->sample_point_x,
                                             color_tool->sample_point_y,
@@ -344,7 +345,7 @@ gimp_color_tool_button_release (GimpTool              *tool,
           else
             {
               color_tool->sample_point =
-                gimp_image_add_sample_point_at_pos (display->image,
+                gimp_image_add_sample_point_at_pos (image,
                                                     color_tool->sample_point_x,
                                                     color_tool->sample_point_y,
                                                     TRUE);
@@ -352,11 +353,11 @@ gimp_color_tool_button_release (GimpTool              *tool,
         }
 
       gimp_display_shell_selection_control (shell, GIMP_SELECTION_RESUME);
-      gimp_image_flush (display->image);
+      gimp_image_flush (image);
 
       if (color_tool->sample_point)
         gimp_display_shell_draw_sample_point (shell, color_tool->sample_point,
-                                              TRUE);
+                                              NULL, TRUE);
 
       color_tool->moving_sample_point = FALSE;
       color_tool->sample_point_x      = -1;
@@ -376,7 +377,7 @@ gimp_color_tool_motion (GimpTool         *tool,
                         GimpDisplay      *display)
 {
   GimpColorTool    *color_tool = GIMP_COLOR_TOOL (tool);
-  GimpDisplayShell *shell      = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell      = gimp_display_get_shell (display);
 
   if (! color_tool->enabled)
     return;
@@ -470,7 +471,8 @@ gimp_color_tool_oper_update (GimpTool         *tool,
                              GimpDisplay      *display)
 {
   GimpColorTool    *color_tool   = GIMP_COLOR_TOOL (tool);
-  GimpDisplayShell *shell        = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell        = gimp_display_get_shell (display);
+  GimpImage        *image        = gimp_display_get_image (display);
   GimpSamplePoint  *sample_point = NULL;
 
   if (color_tool->enabled &&
@@ -478,25 +480,24 @@ gimp_color_tool_oper_update (GimpTool         *tool,
     {
       gint snap_distance;
 
-      snap_distance =
-        GIMP_DISPLAY_CONFIG (display->image->gimp->config)->snap_distance;
+      snap_distance = GIMP_DISPLAY_CONFIG (display->gimp->config)->snap_distance;
 
       sample_point =
-        gimp_image_find_sample_point (display->image,
+        gimp_image_find_sample_point (image,
                                       coords->x, coords->y,
                                       FUNSCALEX (shell, snap_distance),
                                       FUNSCALEY (shell, snap_distance));
     }
 
-  if (color_tool->sample_point && color_tool->sample_point != sample_point)
-    gimp_image_update_sample_point (shell->display->image,
-                                    color_tool->sample_point);
+  if (color_tool->sample_point &&
+      color_tool->sample_point != sample_point)
+    gimp_image_update_sample_point (image, color_tool->sample_point);
 
   color_tool->sample_point = sample_point;
 
   if (color_tool->sample_point)
     gimp_display_shell_draw_sample_point (shell, color_tool->sample_point,
-                                          TRUE);
+                                          NULL, TRUE);
 }
 
 static void
@@ -506,6 +507,7 @@ gimp_color_tool_cursor_update (GimpTool         *tool,
                                GimpDisplay      *display)
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (tool);
+  GimpImage     *image      = gimp_display_get_image (display);
 
   if (color_tool->enabled)
     {
@@ -520,7 +522,7 @@ gimp_color_tool_cursor_update (GimpTool         *tool,
         {
           GimpCursorModifier modifier = GIMP_CURSOR_MODIFIER_BAD;
 
-          if (gimp_image_coords_in_active_pickable (display->image, coords,
+          if (gimp_image_coords_in_active_pickable (image, coords,
                                                     color_tool->options->sample_merged,
                                                     FALSE))
             {
@@ -565,7 +567,7 @@ gimp_color_tool_draw (GimpDrawTool *draw_tool)
           if (color_tool->sample_point_x != -1 &&
               color_tool->sample_point_y != -1)
             {
-              GimpImage *image = draw_tool->display->image;
+              GimpImage *image = gimp_display_get_image (draw_tool->display);
 
               gimp_draw_tool_draw_line (draw_tool,
                                         0, color_tool->sample_point_y + 0.5,
@@ -607,12 +609,13 @@ gimp_color_tool_real_pick (GimpColorTool *color_tool,
                            GimpRGB       *color,
                            gint          *color_index)
 {
-  GimpTool *tool = GIMP_TOOL (color_tool);
+  GimpTool  *tool  = GIMP_TOOL (color_tool);
+  GimpImage *image = gimp_display_get_image (tool->display);
 
   g_return_val_if_fail (tool->display != NULL, FALSE);
   g_return_val_if_fail (tool->drawable != NULL, FALSE);
 
-  return gimp_image_pick_color (tool->display->image, tool->drawable, x, y,
+  return gimp_image_pick_color (image, tool->drawable, x, y,
                                 color_tool->options->sample_merged,
                                 color_tool->options->sample_average,
                                 color_tool->options->average_radius,
@@ -630,13 +633,10 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
 {
   GimpTool          *tool = GIMP_TOOL (color_tool);
   GimpContext       *context;
-  GimpDialogFactory *dialog_factory;
 
   /*  use this tool's own options here (NOT color_tool->options)  */
   context = GIMP_CONTEXT (gimp_tool_get_options (tool));
 
-  dialog_factory = gimp_dialog_factory_from_name ("dock");
-
   if (color_tool->pick_mode == GIMP_COLOR_PICK_MODE_FOREGROUND ||
       color_tool->pick_mode == GIMP_COLOR_PICK_MODE_BACKGROUND)
     {
@@ -644,13 +644,13 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
 
       if (GIMP_IMAGE_TYPE_IS_INDEXED (sample_type))
         {
-          info = gimp_dialog_factory_find_session_info (dialog_factory,
+          info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                         "gimp-indexed-palette");
-          if (info && info->widget)
+          if (info && gimp_session_info_get_widget (info))
             {
               GimpColormapEditor *editor;
 
-              editor = GIMP_COLORMAP_EDITOR (gtk_bin_get_child (GTK_BIN (info->widget)));
+              editor = GIMP_COLORMAP_EDITOR (gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (info))));
 
               gimp_colormap_editor_set_index (editor, color_index, NULL);
             }
@@ -658,14 +658,14 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
 
       if (TRUE)
         {
-          info = gimp_dialog_factory_find_session_info (dialog_factory,
+          info = gimp_dialog_factory_find_session_info (gimp_dialog_factory_get_singleton (),
                                                         "gimp-palette-editor");
-          if (info && info->widget)
+          if (info && gimp_session_info_get_widget (info))
             {
               GimpPaletteEditor *editor;
               gint               index;
 
-              editor = GIMP_PALETTE_EDITOR (gtk_bin_get_child (GTK_BIN (info->widget)));
+              editor = GIMP_PALETTE_EDITOR (gtk_bin_get_child (GTK_BIN (gimp_session_info_get_widget (info))));
 
               index = gimp_palette_editor_get_index (editor, color);
               if (index != -1)
@@ -689,11 +689,13 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
 
     case GIMP_COLOR_PICK_MODE_PALETTE:
       {
-        GdkScreen *screen;
-        GtkWidget *dockable;
+        GimpDisplayShell *shell = gimp_display_get_shell (tool->display);
+        GdkScreen        *screen;
+        GtkWidget        *dockable;
 
-        screen = gtk_widget_get_screen (tool->display->shell);
-        dockable = gimp_dialog_factory_dialog_raise (dialog_factory, screen,
+        screen = gtk_widget_get_screen (GTK_WIDGET (shell));
+        dockable = gimp_dialog_factory_dialog_raise (gimp_dialog_factory_get_singleton (),
+                                                     screen,
                                                      "gimp-palette-editor",
                                                      -1);
         if (dockable)
@@ -813,7 +815,7 @@ gimp_color_tool_start_sample_point (GimpTool    *tool,
 
   color_tool = GIMP_COLOR_TOOL (tool);
 
-  gimp_display_shell_selection_control (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_selection_control (gimp_display_get_shell (display),
                                         GIMP_SELECTION_PAUSE);
 
   tool->display = display;
@@ -821,8 +823,9 @@ gimp_color_tool_start_sample_point (GimpTool    *tool,
   gimp_tool_control_set_scroll_lock (tool->control, TRUE);
 
   if (color_tool->sample_point)
-    gimp_display_shell_draw_sample_point (GIMP_DISPLAY_SHELL (display->shell),
-                                          color_tool->sample_point, FALSE);
+    gimp_display_shell_draw_sample_point (gimp_display_get_shell (display),
+                                          color_tool->sample_point,
+                                          NULL, FALSE);
 
   color_tool->sample_point        = NULL;
   color_tool->moving_sample_point = TRUE;
