@@ -1,9 +1,9 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -42,11 +41,11 @@ static void   gimp_convolve_tool_modifier_key  (GimpTool         *tool,
                                                 GdkModifierType   state,
                                                 GimpDisplay      *display);
 static void   gimp_convolve_tool_cursor_update (GimpTool         *tool,
-                                                GimpCoords       *coords,
+                                                const GimpCoords *coords,
                                                 GdkModifierType   state,
                                                 GimpDisplay      *display);
 static void   gimp_convolve_tool_oper_update   (GimpTool         *tool,
-                                                GimpCoords       *coords,
+                                                const GimpCoords *coords,
                                                 GdkModifierType   state,
                                                 gboolean          proximity,
                                                 GimpDisplay      *display);
@@ -130,10 +129,10 @@ gimp_convolve_tool_modifier_key (GimpTool        *tool,
 }
 
 static void
-gimp_convolve_tool_cursor_update (GimpTool        *tool,
-                                  GimpCoords      *coords,
-                                  GdkModifierType  state,
-                                  GimpDisplay     *display)
+gimp_convolve_tool_cursor_update (GimpTool         *tool,
+                                  const GimpCoords *coords,
+                                  GdkModifierType   state,
+                                  GimpDisplay      *display)
 {
   GimpConvolveOptions *options = GIMP_CONVOLVE_TOOL_GET_OPTIONS (tool);
 
@@ -144,11 +143,11 @@ gimp_convolve_tool_cursor_update (GimpTool        *tool,
 }
 
 static void
-gimp_convolve_tool_oper_update (GimpTool        *tool,
-                                GimpCoords      *coords,
-                                GdkModifierType  state,
-                                gboolean         proximity,
-                                GimpDisplay     *display)
+gimp_convolve_tool_oper_update (GimpTool         *tool,
+                                const GimpCoords *coords,
+                                GdkModifierType   state,
+                                gboolean          proximity,
+                                GimpDisplay      *display)
 {
   GimpConvolveOptions *options = GIMP_CONVOLVE_TOOL_GET_OPTIONS (tool);
 
