@@ -309,20 +309,34 @@ gimp_crop_tool_execute (GimpRectangleTool  *rectangle,
                         gint                w,
                         gint                h)
 {
-  GimpTool        *tool;
-  GimpCropOptions *options;
-  GimpImage       *image;
-
-  tool    = GIMP_TOOL (rectangle);
-  options = GIMP_CROP_TOOL_GET_OPTIONS (tool);
+  GimpTool        *tool    = GIMP_TOOL (rectangle);
+  GimpCropOptions *options = GIMP_CROP_TOOL_GET_OPTIONS (tool);
+  GimpImage       *image   = gimp_display_get_image (tool->display);
 
   gimp_tool_pop_status (tool, tool->display);
 
-  image = tool->display->image;
-
   /* if rectangle exists, crop it */
   if (w > 0 && h > 0)
     {
+      if (options->layer_only)
+        {
+          GimpLayer *layer = gimp_image_get_active_layer (image);
+
+          if (! layer)
+            {
+              gimp_tool_message_literal (tool, tool->display,
+                                         _("There is no active layer to crop."));
+              return FALSE;
+            }
+
+          if (gimp_item_is_content_locked (GIMP_ITEM (layer)))
+            {
+              gimp_tool_message_literal (tool, tool->display,
+                                         _("The active layer's pixels are locked."));
+              return FALSE;
+            }
+        }
+
       gimp_image_crop (image, GIMP_CONTEXT (options),
                        x, y, w + x, h + y,
                        options->layer_only,
