@@ -209,7 +209,8 @@ gimp_curves_tool_initialize (GimpTool     *tool,
                              GError      **error)
 {
   GimpCurvesTool *c_tool   = GIMP_CURVES_TOOL (tool);
-  GimpDrawable   *drawable = gimp_image_get_active_drawable (display->image);
+  GimpImage      *image    = gimp_display_get_image (display);
+  GimpDrawable   *drawable = gimp_image_get_active_drawable (image);
   GimpHistogram  *histogram;
 
   if (! drawable)
@@ -224,7 +225,10 @@ gimp_curves_tool_initialize (GimpTool     *tool,
 
   gimp_config_reset (GIMP_CONFIG (c_tool->config));
 
-  GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
+  if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
+    {
+      return FALSE;
+    }
 
   /*  always pick colors  */
   gimp_color_tool_enable (GIMP_COLOR_TOOL (tool),
@@ -517,6 +521,8 @@ gimp_curves_tool_dialog (GimpImageMapTool *image_map_tool)
   gtk_widget_show (frame);
 
   tool->graph = gimp_curve_view_new ();
+  gimp_curve_view_set_range_x (GIMP_CURVE_VIEW (tool->graph), 0, 255);
+  gimp_curve_view_set_range_y (GIMP_CURVE_VIEW (tool->graph), 0, 255);
   gtk_widget_set_size_request (tool->graph,
                                GRAPH_SIZE + RADIUS * 2,
                                GRAPH_SIZE + RADIUS * 2);
@@ -734,6 +740,10 @@ gimp_curves_tool_config_notify (GObject        *object,
 
   if (! strcmp (pspec->name, "channel"))
     {
+      GimpRGB red;
+      GimpRGB green;
+      GimpRGB blue;
+
       gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (tool->channel_menu),
                                      config->channel);
 
@@ -775,6 +785,45 @@ gimp_curves_tool_config_notify (GObject        *object,
       gimp_color_bar_set_channel (GIMP_COLOR_BAR (tool->yrange),
                                   config->channel);
 
+      gimp_curve_view_remove_all_backgrounds (GIMP_CURVE_VIEW (tool->graph));
+
+      gimp_rgb_set (&red,   1.0, 0.0, 0.0);
+      gimp_rgb_set (&green, 0.0, 1.0, 0.0);
+      gimp_rgb_set (&blue,  0.0, 0.0, 1.0);
+
+      switch (config->channel)
+        {
+        case GIMP_HISTOGRAM_RED:
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_GREEN],
+                                          &green);
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_BLUE],
+                                          &blue);
+          break;
+
+        case GIMP_HISTOGRAM_GREEN:
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_RED],
+                                          &red);
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_BLUE],
+                                          &blue);
+          break;
+
+        case GIMP_HISTOGRAM_BLUE:
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_RED],
+                                          &red);
+          gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
+                                          config->curve[GIMP_HISTOGRAM_GREEN],
+                                          &green);
+          break;
+
+        default:
+          break;
+        }
+
       gimp_curve_view_set_curve (GIMP_CURVE_VIEW (tool->graph), curve);
 
       gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (tool->curve_type),
