@@ -112,9 +112,8 @@ gimp_desaturate_tool_initialize (GimpTool     *tool,
                                 GError      **error)
 {
   GimpDesaturateTool *desaturate_tool = GIMP_DESATURATE_TOOL (tool);
-  GimpDrawable       *drawable;
-
-  drawable = gimp_image_get_active_drawable (display->image);
+  GimpImage          *image           = gimp_display_get_image (display);
+  GimpDrawable       *drawable        = gimp_image_get_active_drawable (image);
 
   if (! drawable)
     return FALSE;
@@ -128,7 +127,10 @@ gimp_desaturate_tool_initialize (GimpTool     *tool,
 
   gimp_config_reset (GIMP_CONFIG (desaturate_tool->config));
 
-  GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
+  if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
+    {
+      return FALSE;
+    }
 
   gimp_int_radio_group_set_active (GTK_RADIO_BUTTON (desaturate_tool->button),
                                    desaturate_tool->config->mode);
