@@ -42,6 +42,12 @@
 #include "gimpdrawtool.h"
 
 
+#define DRAW_TIMEOUT 4
+#define USE_TIMEOUT  1
+
+
+static void          gimp_draw_tool_dispose      (GObject        *object);
+
 static gboolean      gimp_draw_tool_has_display  (GimpTool       *tool,
                                                   GimpDisplay    *display);
 static GimpDisplay * gimp_draw_tool_has_image    (GimpTool       *tool,
@@ -79,7 +85,10 @@ G_DEFINE_TYPE (GimpDrawTool, gimp_draw_tool, GIMP_TYPE_TOOL)
 static void
 gimp_draw_tool_class_init (GimpDrawToolClass *klass)
 {
-  GimpToolClass *tool_class = GIMP_TOOL_CLASS (klass);
+  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
+  GimpToolClass *tool_class   = GIMP_TOOL_CLASS (klass);
+
+  object_class->dispose   = gimp_draw_tool_dispose;
 
   tool_class->has_display = gimp_draw_tool_has_display;
   tool_class->has_image   = gimp_draw_tool_has_image;
@@ -97,6 +106,20 @@ gimp_draw_tool_init (GimpDrawTool *draw_tool)
   draw_tool->is_drawn     = FALSE;
 }
 
+static void
+gimp_draw_tool_dispose (GObject *object)
+{
+  GimpDrawTool *draw_tool = GIMP_DRAW_TOOL (object);
+
+  if (draw_tool->draw_timeout)
+    {
+      g_source_remove (draw_tool->draw_timeout);
+      draw_tool->draw_timeout = 0;
+    }
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static gboolean
 gimp_draw_tool_has_display (GimpTool    *tool,
                             GimpDisplay *display)
@@ -118,7 +141,7 @@ gimp_draw_tool_has_image (GimpTool  *tool,
 
   if (! display && draw_tool->display)
     {
-      if (image && draw_tool->display->image == image)
+      if (image && gimp_display_get_image (draw_tool->display) == image)
         display = draw_tool->display;
 
       /*  NULL image means any display  */
@@ -154,10 +177,24 @@ gimp_draw_tool_control (GimpTool       *tool,
   GIMP_TOOL_CLASS (parent_class)->control (tool, action, display);
 }
 
+#ifdef USE_TIMEOUT
+static gboolean
+gimp_draw_tool_draw_timeout (GimpDrawTool *draw_tool)
+{
+  draw_tool->draw_timeout = 0;
+
+  gimp_draw_tool_draw (draw_tool);
+
+  return FALSE;
+}
+#endif
+
 static void
 gimp_draw_tool_draw (GimpDrawTool *draw_tool)
 {
-  if (draw_tool->paused_count == 0 && draw_tool->display)
+  if (! draw_tool->draw_timeout    &&
+      draw_tool->paused_count == 0 &&
+      draw_tool->display)
     {
       GIMP_DRAW_TOOL_GET_CLASS (draw_tool)->draw (draw_tool);
 
@@ -175,13 +212,9 @@ void
 gimp_draw_tool_start (GimpDrawTool *draw_tool,
                       GimpDisplay  *display)
 {
-  GimpDisplayShell *shell;
-
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
-
   gimp_draw_tool_stop (draw_tool);
 
   draw_tool->display = display;
@@ -196,6 +229,12 @@ gimp_draw_tool_stop (GimpDrawTool *draw_tool)
 
   gimp_draw_tool_draw (draw_tool);
 
+  if (draw_tool->draw_timeout)
+    {
+      g_source_remove (draw_tool->draw_timeout);
+      draw_tool->draw_timeout = 0;
+    }
+
   draw_tool->display = NULL;
 }
 
@@ -215,23 +254,32 @@ gimp_draw_tool_pause (GimpDrawTool *draw_tool)
   gimp_draw_tool_draw (draw_tool);
 
   draw_tool->paused_count++;
+
+  if (draw_tool->draw_timeout)
+    {
+      g_source_remove (draw_tool->draw_timeout);
+      draw_tool->draw_timeout = 0;
+    }
 }
 
 void
 gimp_draw_tool_resume (GimpDrawTool *draw_tool)
 {
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
+  g_return_if_fail (draw_tool->paused_count > 0);
 
-  if (draw_tool->paused_count > 0)
-    {
-      draw_tool->paused_count--;
+  draw_tool->paused_count--;
 
-      gimp_draw_tool_draw (draw_tool);
-    }
-  else
-    {
-      g_warning ("%s: called with draw_tool->paused_count == 0", G_STRFUNC);
-    }
+#ifdef USE_TIMEOUT
+  if (draw_tool->paused_count == 0 && ! draw_tool->draw_timeout)
+    draw_tool->draw_timeout =
+      gdk_threads_add_timeout_full (G_PRIORITY_HIGH_IDLE,
+                                    DRAW_TIMEOUT,
+                                    (GSourceFunc) gimp_draw_tool_draw_timeout,
+                                    draw_tool, NULL);
+#else
+  gimp_draw_tool_draw (draw_tool);
+#endif
 }
 
 gboolean
@@ -282,7 +330,7 @@ gimp_draw_tool_calc_distance (GimpDrawTool *draw_tool,
  * distances.
  *
  * Returns: the square of the distance between the given points in
-            display coordinates
+ *          display coordinates
  **/
 gdouble
 gimp_draw_tool_calc_distance_square (GimpDrawTool *draw_tool,
@@ -299,7 +347,7 @@ gimp_draw_tool_calc_distance_square (GimpDrawTool *draw_tool,
   g_return_val_if_fail (GIMP_IS_DRAW_TOOL (draw_tool), 0.0);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), 0.0);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   gimp_display_shell_transform_xy_f (shell, x1, y1, &tx1, &ty1, FALSE);
   gimp_display_shell_transform_xy_f (shell, x2, y2, &tx2, &ty2, FALSE);
@@ -344,7 +392,7 @@ gimp_draw_tool_set_clip_rect (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   if (rect)
     {
@@ -400,7 +448,7 @@ gimp_draw_tool_draw_line (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x1, y1,
@@ -443,7 +491,7 @@ gimp_draw_tool_draw_dashed_line (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x1, y1,
@@ -479,12 +527,12 @@ gimp_draw_tool_draw_guide_line (GimpDrawTool        *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   x1 = 0;
   y1 = 0;
 
-  gdk_drawable_get_size (shell->canvas->window, &x2, &y2);
+  gdk_drawable_get_size (gtk_widget_get_window (shell->canvas), &x2, &y2);
 
   switch (orientation)
     {
@@ -535,7 +583,7 @@ gimp_draw_tool_draw_rectangle (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      MIN (x, x + width), MIN (y, y + height),
@@ -557,11 +605,19 @@ gimp_draw_tool_draw_rectangle (GimpDrawTool *draw_tool,
   h = PROJ_ROUND (MAX (0.0, ty2));
 
   if (w > 0 && h > 0)
-    gimp_canvas_draw_rectangle (GIMP_CANVAS (shell->canvas),
-                                GIMP_CANVAS_STYLE_XOR,
-                                filled,
-                                PROJ_ROUND (tx1), PROJ_ROUND (ty1),
-                                w - 1, h - 1);
+    {
+      if (! filled)
+        {
+          w--;
+          h--;
+        }
+
+      gimp_canvas_draw_rectangle (GIMP_CANVAS (shell->canvas),
+                                  GIMP_CANVAS_STYLE_XOR,
+                                  filled,
+                                  PROJ_ROUND (tx1), PROJ_ROUND (ty1),
+                                  w, h);
+    }
 }
 
 void
@@ -582,7 +638,7 @@ gimp_draw_tool_draw_arc (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      MIN (x, x + width), MIN (y, y + height),
@@ -600,13 +656,19 @@ gimp_draw_tool_draw_arc (GimpDrawTool *draw_tool,
 
   if (w > 0 && h > 0)
     {
+      if (! filled)
+        {
+          w--;
+          h--;
+        }
+
       if (w != 1 && h != 1)
         {
           gimp_canvas_draw_arc (GIMP_CANVAS (shell->canvas),
                                 GIMP_CANVAS_STYLE_XOR,
                                 filled,
                                 PROJ_ROUND (tx1), PROJ_ROUND (ty1),
-                                w - 1, h - 1,
+                                w, h,
                                 angle1, angle2);
         }
       else
@@ -618,7 +680,7 @@ gimp_draw_tool_draw_arc (GimpDrawTool *draw_tool,
                                       GIMP_CANVAS_STYLE_XOR,
                                       filled,
                                       PROJ_ROUND (tx1), PROJ_ROUND (ty1),
-                                      w - 1, h - 1);
+                                      w, h);
         }
     }
 }
@@ -638,7 +700,7 @@ gimp_draw_tool_draw_rectangle_by_anchor (GimpDrawTool   *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x, y,
@@ -680,7 +742,7 @@ gimp_draw_tool_draw_arc_by_anchor (GimpDrawTool  *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x, y,
@@ -719,7 +781,7 @@ gimp_draw_tool_draw_cross_by_anchor (GimpDrawTool  *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x, y,
@@ -855,7 +917,7 @@ gimp_draw_tool_draw_corner (GimpDrawTool   *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell  = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell  = gimp_display_get_shell (draw_tool->display);
   canvas = GIMP_CANVAS (shell->canvas);
 
   gimp_display_shell_transform_xy (shell, x1, y1, &tx1, &ty1, use_offsets);
@@ -1247,7 +1309,6 @@ gimp_draw_tool_draw_corner (GimpDrawTool   *draw_tool,
     }
 }
 
-
 void
 gimp_draw_tool_draw_lines (GimpDrawTool      *draw_tool,
                            const GimpVector2 *points,
@@ -1263,7 +1324,7 @@ gimp_draw_tool_draw_lines (GimpDrawTool      *draw_tool,
   if (points == NULL || n_points == 0)
     return;
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   coords = g_new (GdkPoint, n_points);
 
@@ -1301,7 +1362,7 @@ gimp_draw_tool_draw_strokes (GimpDrawTool     *draw_tool,
   if (n_points == 0)
     return;
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   coords = g_new (GdkPoint, n_points);
 
@@ -1357,7 +1418,7 @@ gimp_draw_tool_draw_boundary (GimpDrawTool   *draw_tool,
   g_return_if_fail (n_bound_segs > 0);
   g_return_if_fail (bound_segs != NULL);
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gdk_points = g_new0 (GdkPoint, n_bound_segs + 1);
   n_gdk_points = 0;
@@ -1453,13 +1514,10 @@ gimp_draw_tool_draw_boundary (GimpDrawTool   *draw_tool,
 }
 
 void
-gimp_draw_tool_draw_text_cursor (GimpDrawTool *draw_tool,
-                                 gdouble       x1,
-                                 gdouble       y1,
-                                 gdouble       x2,
-                                 gdouble       y2,
-                                 gboolean      overwrite,
-                                 gboolean      use_offsets)
+gimp_draw_tool_draw_text_cursor (GimpDrawTool   *draw_tool,
+                                 PangoRectangle *cursor,
+                                 gboolean        overwrite,
+                                 gboolean        use_offsets)
 {
   GimpDisplayShell *shell;
   gdouble           tx1, ty1;
@@ -1467,34 +1525,46 @@ gimp_draw_tool_draw_text_cursor (GimpDrawTool *draw_tool,
 
   g_return_if_fail (GIMP_IS_DRAW_TOOL (draw_tool));
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   gimp_display_shell_transform_xy_f (shell,
-                                     x1, y1,
+                                     cursor->x, cursor->y,
                                      &tx1, &ty1,
                                      use_offsets);
-  gimp_display_shell_transform_xy_f (shell,
-                                     x2, y2,
-                                     &tx2, &ty2,
-                                     use_offsets);
 
   if (overwrite)
     {
+      gint x, y;
+      gint width, height;
+
+      gimp_display_shell_transform_xy_f (shell,
+                                         cursor->x + cursor->width,
+                                         cursor->y + cursor->height,
+                                         &tx2, &ty2,
+                                         use_offsets);
+
+      x      = PROJ_ROUND (tx1);
+      y      = PROJ_ROUND (ty1);
+      width  = PROJ_ROUND (tx2 - tx1);
+      height = PROJ_ROUND (ty2 - ty1);
+
       gimp_canvas_draw_rectangle (GIMP_CANVAS (shell->canvas),
                                   GIMP_CANVAS_STYLE_XOR, FALSE,
-                                  PROJ_ROUND (tx1),
-                                  PROJ_ROUND (ty1),
-                                  PROJ_ROUND (tx2 - tx1),
-                                  PROJ_ROUND (ty2 - ty1));
+                                  x, y,
+                                  width, height);
       gimp_canvas_draw_rectangle (GIMP_CANVAS (shell->canvas),
                                   GIMP_CANVAS_STYLE_XOR, FALSE,
-                                  PROJ_ROUND (tx1) + 1,
-                                  PROJ_ROUND (ty1) + 1,
-                                  PROJ_ROUND (tx2 - tx1) - 2,
-                                  PROJ_ROUND (ty2 - ty1) - 2);
+                                  x + 1, y + 1,
+                                  width - 2, height - 2);
     }
   else
     {
+      gimp_display_shell_transform_xy_f (shell,
+                                         cursor->x,
+                                         cursor->y + cursor->height,
+                                         &tx2, &ty2,
+                                         use_offsets);
+
       /*  vertical line  */
       gimp_canvas_draw_line (GIMP_CANVAS (shell->canvas),
                              GIMP_CANVAS_STYLE_XOR,
@@ -1547,7 +1617,7 @@ gimp_draw_tool_on_handle (GimpDrawTool   *draw_tool,
   g_return_val_if_fail (GIMP_IS_DRAW_TOOL (draw_tool), FALSE);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), FALSE);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   gimp_display_shell_transform_xy_f (shell,
                                      x, y,
@@ -1809,7 +1879,7 @@ gimp_draw_tool_on_vectors (GimpDrawTool      *draw_tool,
   if (ret_stroke)        *ret_stroke         = NULL;
   if (ret_vectors)       *ret_vectors        = NULL;
 
-  all_vectors = gimp_image_get_vectors_list (display->image);
+  all_vectors = gimp_image_get_vectors_list (gimp_display_get_image (display));
 
   for (list = all_vectors; list; list = g_list_next (list))
     {
