@@ -50,8 +50,9 @@ struct _GimpDrawTool
                                  *  a different one than tool->display)
                                  */
 
-  gint            paused_count; /*  count to keep track of multiple pauses */
-  gboolean        is_drawn;     /*  is the stuff we draw currently visible */
+  gint            paused_count; /*  count to keep track of multiple pauses  */
+  gboolean        is_drawn;     /*  is the stuff we draw currently visible  */
+  guint           draw_timeout; /*  draw delay timeout ID                   */
 };
 
 struct _GimpDrawToolClass
@@ -203,10 +204,7 @@ void       gimp_draw_tool_draw_boundary            (GimpDrawTool     *draw_tool,
                                                     gboolean          use_offsets);
 
 void       gimp_draw_tool_draw_text_cursor         (GimpDrawTool     *draw_tool,
-                                                    gdouble           x1,
-                                                    gdouble           y1,
-                                                    gdouble           x2,
-                                                    gdouble           y2,
+                                                    PangoRectangle   *cursor,
                                                     gboolean          overwrite,
                                                     gboolean          use_offsets);
 
