@@ -571,7 +571,7 @@ gimp_foreground_select_tool_draw (GimpDrawTool *draw_tool)
 
   if (fg_select->stroke)
     {
-      gimp_display_shell_draw_pen (GIMP_DISPLAY_SHELL (draw_tool->display->shell),
+      gimp_display_shell_draw_pen (gimp_display_get_shell (draw_tool->display),
                                    (const GimpVector2 *)fg_select->stroke->data,
                                    fg_select->stroke->len,
                                    GIMP_CONTEXT (options),
@@ -583,7 +583,7 @@ gimp_foreground_select_tool_draw (GimpDrawTool *draw_tool)
 
   if (fg_select->mask)
     {
-      GimpDisplayShell   *shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+      GimpDisplayShell   *shell = gimp_display_get_shell (draw_tool->display);
       gint                x     = fg_select->last_coords.x;
       gint                y     = fg_select->last_coords.y;
       gdouble             radius;
@@ -624,7 +624,7 @@ gimp_foreground_select_tool_select (GimpFreeSelectTool *free_sel,
 {
   GimpForegroundSelectTool    *fg_select;
   GimpForegroundSelectOptions *options;
-  GimpImage                   *image = display->image;
+  GimpImage                   *image = gimp_display_get_image (display);
   GimpDrawable                *drawable;
   GimpScanConvert             *scan_convert;
   GimpChannel                 *mask;
@@ -732,7 +732,7 @@ gimp_foreground_select_tool_set_mask (GimpForegroundSelectTool *fg_select,
   if (mask)
     fg_select->mask = g_object_ref (mask);
 
-  gimp_display_shell_set_mask (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_set_mask (gimp_display_get_shell (display),
                                GIMP_DRAWABLE (mask), options->mask_color);
 
   if (mask)
@@ -761,10 +761,11 @@ gimp_foreground_select_tool_apply (GimpForegroundSelectTool *fg_select,
 {
   GimpTool             *tool    = GIMP_TOOL (fg_select);
   GimpSelectionOptions *options = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
+  GimpImage            *image   = gimp_display_get_image (display);
 
   g_return_if_fail (fg_select->mask != NULL);
 
-  gimp_channel_select_channel (gimp_image_get_mask (display->image),
+  gimp_channel_select_channel (gimp_image_get_mask (image),
                                C_("command", "Foreground Select"),
                                fg_select->mask, 0, 0,
                                options->operation,
@@ -774,7 +775,7 @@ gimp_foreground_select_tool_apply (GimpForegroundSelectTool *fg_select,
 
   gimp_tool_control (tool, GIMP_TOOL_ACTION_HALT, display);
 
-  gimp_image_flush (display->image);
+  gimp_image_flush (image);
 }
 
 static void
@@ -816,7 +817,7 @@ gimp_foreground_select_tool_push_stroke (GimpForegroundSelectTool    *fg_select,
                                          GimpDisplay                 *display,
                                          GimpForegroundSelectOptions *options)
 {
-  GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell = gimp_display_get_shell (display);
   FgSelectStroke   *stroke;
 
   g_return_if_fail (fg_select->stroke != NULL);
@@ -891,7 +892,7 @@ gimp_foreground_select_options_notify (GimpForegroundSelectOptions *options,
       GimpTool *tool = GIMP_TOOL (fg_select);
 
       if (tool->display)
-        gimp_display_shell_set_mask (GIMP_DISPLAY_SHELL (tool->display->shell),
+        gimp_display_shell_set_mask (gimp_display_get_shell (tool->display),
                                      GIMP_DRAWABLE (fg_select->mask),
                                      options->mask_color);
     }
