@@ -351,7 +351,7 @@ gimp_free_select_tool_should_close (GimpFreeSelectTool *fst,
   gdouble                    dist         = G_MAXDOUBLE;
 
   if (priv->polygon_modified       ||
-      priv->n_segment_indices <= 1 ||
+      priv->n_segment_indices <= 0 ||
       GIMP_TOOL (fst)->display == NULL)
     return FALSE;
 
@@ -485,9 +485,10 @@ gimp_free_select_tool_remove_last_segment (GimpFreeSelectTool *fst)
 
   gimp_draw_tool_pause (draw_tool);
 
-  priv->n_segment_indices--;
+  if (priv->n_segment_indices > 0)
+    priv->n_segment_indices--;
 
-  if (priv->n_segment_indices == 0)
+  if (priv->n_segment_indices <= 0)
     {
       gimp_free_select_tool_halt (fst);
     }
@@ -789,7 +790,7 @@ gimp_free_select_tool_commit (GimpFreeSelectTool *fst,
       gimp_free_select_tool_select (fst, display);
     }
 
-  gimp_image_flush (display->image);
+  gimp_image_flush (gimp_display_get_image (display));
 }
 
 static void
@@ -846,12 +847,13 @@ gimp_free_select_tool_handle_click (GimpFreeSelectTool *fst,
                                     guint32             time,
                                     GimpDisplay        *display)
 {
-  GimpFreeSelectToolPrivate *priv = GET_PRIVATE (fst);
+  GimpFreeSelectToolPrivate *priv  = GET_PRIVATE (fst);
+  GimpImage                 *image = gimp_display_get_image (display);
 
   /*  If there is a floating selection, anchor it  */
-  if (gimp_image_get_floating_selection (display->image))
+  if (gimp_image_get_floating_selection (image))
     {
-      floating_sel_anchor (gimp_image_get_floating_selection (display->image));
+      floating_sel_anchor (gimp_image_get_floating_selection (image));
       gimp_free_select_tool_halt (fst);
     }
   else
@@ -1513,30 +1515,64 @@ gimp_free_select_tool_active_modifier_key (GimpTool        *tool,
                                                        display);
 }
 
+/**
+ * gimp_free_select_tool_draw:
+ * @draw_tool:
+ *
+ * Draw the line segments and handles around segment vertices, the
+ * latter if the they are in proximity to cursor.
+ **/
 static void
 gimp_free_select_tool_draw (GimpDrawTool *draw_tool)
 {
-  GimpFreeSelectTool        *fst  = GIMP_FREE_SELECT_TOOL (draw_tool);
-  GimpFreeSelectToolPrivate *priv = GET_PRIVATE (fst);
-  GimpTool                  *tool = GIMP_TOOL (draw_tool);
-
+  GimpFreeSelectTool        *fst                   = GIMP_FREE_SELECT_TOOL (draw_tool);
+  GimpFreeSelectToolPrivate *priv                  = GET_PRIVATE (fst);
+  GimpTool                  *tool                  = GIMP_TOOL (draw_tool);
+  gboolean                   hovering_first_point  = FALSE;
+  gboolean                   handles_wants_to_show = FALSE;
+  GimpCoords                 coords                = { priv->last_coords.x,
+                                                       priv->last_coords.y,
+                                                       /* pad with 0 */ };
   if (! tool->display)
     return;
 
+  hovering_first_point =
+    gimp_free_select_tool_should_close (fst,
+                                        tool->display,
+                                        NO_CLICK_TIME_AVAILABLE,
+                                        &coords);
   gimp_draw_tool_draw_lines (draw_tool,
                              priv->points, priv->n_points,
                              FALSE, FALSE);
 
-  /* Draw handles around segment vertices in the proximity */
-  if (! priv->button1_down &&
+  /* We always show the handle for the first point, even with button1
+   * down, since releasing the button on the first point will close
+   * the polygon, so it's a significant state which we must give
+   * feedback for
+   */
+  handles_wants_to_show = (hovering_first_point ||
+                           ! priv->button1_down);
+
+  if (handles_wants_to_show &&
       ! priv->supress_handles)
     {
-      gint i;
+      gint i = 0;
+      gint n = 0;
 
-      for (i = 0; i < priv->n_segment_indices; i++)
+      /* If the first point is hovered while button1 is held down,
+       * only draw the first handle, the other handles are not
+       * relevant (see comment a few lines up)
+       */
+      if (priv->button1_down && hovering_first_point)
+        n = MIN (priv->n_segment_indices, 1);
+      else
+        n = priv->n_segment_indices;
+
+      for (i = 0; i < n; i++)
         {
-          GimpVector2 *point;
-          gdouble      dist;
+          GimpVector2   *point       = NULL;
+          gdouble        dist        = 0.0;
+          GimpHandleType handle_type = -1;
 
           point = &priv->points[priv->segment_indices[i]];
 
@@ -1547,15 +1583,20 @@ gimp_free_select_tool_draw (GimpDrawTool *draw_tool)
                                                        point->x,
                                                        point->y);
 
-          if (dist < POINT_SHOW_THRESHOLD_SQ)
-            {
-              gimp_draw_tool_draw_handle (draw_tool, GIMP_HANDLE_CIRCLE,
-                                          point->x,
-                                          point->y,
-                                          HANDLE_SIZE, HANDLE_SIZE,
-                                          GTK_ANCHOR_CENTER, FALSE);
+          /* If the cursor is over the point, fill, if it's just
+           * close, draw an outline
+           */
+          if (dist < POINT_GRAB_THRESHOLD_SQ)
+            handle_type = GIMP_HANDLE_FILLED_CIRCLE;
+          else if (dist < POINT_SHOW_THRESHOLD_SQ)
+            handle_type = GIMP_HANDLE_CIRCLE;
 
-            }
+          if (handle_type != -1)
+            gimp_draw_tool_draw_handle (draw_tool, handle_type,
+                                        point->x,
+                                        point->y,
+                                        HANDLE_SIZE, HANDLE_SIZE,
+                                        GTK_ANCHOR_CENTER, FALSE);
         }
     }
 
@@ -1578,8 +1619,9 @@ gimp_free_select_tool_real_select (GimpFreeSelectTool *fst,
 {
   GimpSelectionOptions      *options = GIMP_SELECTION_TOOL_GET_OPTIONS (fst);
   GimpFreeSelectToolPrivate *priv    = GET_PRIVATE (fst);
+  GimpImage                 *image   = gimp_display_get_image (display);
 
-  gimp_channel_select_polygon (gimp_image_get_mask (display->image),
+  gimp_channel_select_polygon (gimp_image_get_mask (image),
                                C_("command", "Free Select"),
                                priv->n_points,
                                priv->points,
