@@ -86,7 +86,7 @@ gimp_magnify_tool_register (GimpToolRegisterCallback  callback,
                 "gimp-zoom-tool",
                 _("Zoom"),
                 _("Zoom Tool: Adjust the zoom level"),
-                N_("tool|_Zoom"), "Z",
+                NC_("tool", "_Zoom"), "Z",
                 NULL, GIMP_HELP_TOOL_ZOOM,
                 GIMP_STOCK_TOOL_ZOOM,
                 data);
@@ -163,7 +163,7 @@ gimp_magnify_tool_button_release (GimpTool              *tool,
 {
   GimpMagnifyTool    *magnify = GIMP_MAGNIFY_TOOL (tool);
   GimpMagnifyOptions *options = GIMP_MAGNIFY_TOOL_GET_OPTIONS (tool);
-  GimpDisplayShell   *shell   = GIMP_DISPLAY_SHELL (tool->display->shell);
+  GimpDisplayShell   *shell   = gimp_display_get_shell (tool->display);
 
   gimp_draw_tool_stop (GIMP_DRAW_TOOL (tool));
 
@@ -224,12 +224,17 @@ gimp_magnify_tool_button_release (GimpTool              *tool,
 
         if (new_scale != current_scale)
           {
-            gdouble xres;
-            gdouble yres;
             gint    offset_x = 0;
             gint    offset_y = 0;
+            gdouble xres;
+            gdouble yres;
+            gdouble screen_xres;
+            gdouble screen_yres;
 
-            gimp_image_get_resolution (display->image, &xres, &yres);
+            gimp_image_get_resolution (gimp_display_get_image (display),
+                                       &xres, &yres);
+            gimp_display_shell_get_screen_resolution (shell,
+                                                      &screen_xres, &screen_yres);
 
             switch (options->zoom_type)
               {
@@ -244,11 +249,11 @@ gimp_magnify_tool_button_release (GimpTool              *tool,
                  *               offset
                  */
                 offset_x = RINT (new_scale * ((x1 + x2) / 2.0) *
-                                 SCREEN_XRES (shell) / xres -
+                                 screen_xres / xres -
                                  (shell->disp_width / 2.0));
 
                 offset_y = RINT (new_scale * ((y1 + y2) / 2.0) *
-                                 SCREEN_YRES (shell) / yres -
+                                 screen_yres / yres -
                                  (shell->disp_height / 2.0));
                 break;
 
@@ -265,14 +270,14 @@ gimp_magnify_tool_button_release (GimpTool              *tool,
                 offset_x = RINT (new_scale * UNSCALEX (shell,
                                                        shell->offset_x +
                                                        shell->disp_width / 2.0) *
-                                 SCREEN_XRES (shell) / xres -
+                                 screen_xres / xres -
                                  (SCALEX (shell, (x1 + x2) / 2.0) -
                                   shell->offset_x));
 
                 offset_y = RINT (new_scale * UNSCALEY (shell,
                                                        shell->offset_y +
                                                        shell->disp_height / 2.0) *
-                                 SCREEN_YRES (shell) / yres -
+                                 screen_yres / yres -
                                  (SCALEY (shell, (y1 + y2) / 2.0) -
                                   shell->offset_y));
                 break;
