@@ -26,6 +26,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
@@ -36,7 +37,6 @@
 #include "core/gimpimage-guides.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
-#include "core/gimpunit.h"
 
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimptooldialog.h"
@@ -44,6 +44,7 @@
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpdisplayshell-appearance.h"
 
 #include "gimpmeasureoptions.h"
 #include "gimpmeasuretool.h"
@@ -188,7 +189,8 @@ gimp_measure_tool_button_press (GimpTool            *tool,
 {
   GimpMeasureTool    *measure = GIMP_MEASURE_TOOL (tool);
   GimpMeasureOptions *options = GIMP_MEASURE_TOOL_GET_OPTIONS (tool);
-  GimpDisplayShell   *shell   = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell   *shell   = gimp_display_get_shell (display);
+  GimpImage          *image   = gimp_display_get_image (display);
   gint                i;
 
   /*  if we are changing displays, pop the statusbar of the old one  */
@@ -229,40 +231,40 @@ gimp_measure_tool_button_press (GimpTool            *tool,
                                    (measure->y[i] ==
                                     CLAMP (measure->y[i],
                                            0,
-                                           gimp_image_get_height (display->image))));
+                                           gimp_image_get_height (image))));
 
                   create_vguide = ((state & GDK_MOD1_MASK) &&
                                    (measure->x[i] ==
                                     CLAMP (measure->x[i],
                                            0,
-                                           gimp_image_get_width (display->image))));
+                                           gimp_image_get_width (image))));
 
                   if (create_hguide && create_vguide)
-                    gimp_image_undo_group_start (display->image,
+                    gimp_image_undo_group_start (image,
                                                  GIMP_UNDO_GROUP_GUIDE,
                                                  _("Add Guides"));
 
                   if (create_hguide)
                     {
-                      guide = gimp_image_add_hguide (display->image,
+                      guide = gimp_image_add_hguide (image,
                                                      measure->y[i],
                                                      TRUE);
-                      gimp_image_update_guide (display->image, guide);
+                      gimp_image_update_guide (image, guide);
                     }
 
                   if (create_vguide)
                     {
-                      guide = gimp_image_add_vguide (display->image,
+                      guide = gimp_image_add_vguide (image,
                                                      measure->x[i],
                                                      TRUE);
-                      gimp_image_update_guide (display->image, guide);
+                      gimp_image_update_guide (image, guide);
                     }
 
                   if (create_hguide && create_vguide)
-                    gimp_image_undo_group_end (display->image);
+                    gimp_image_undo_group_end (image);
 
                   if (create_hguide || create_vguide)
-                    gimp_image_flush (display->image);
+                    gimp_image_flush (image);
 
                   measure->function = GUIDING;
                   break;
@@ -330,17 +332,20 @@ gimp_measure_tool_button_press (GimpTool            *tool,
     }
 
   /*  create the info window if necessary  */
-  if (! measure->dialog && (options->use_info_window ||
-                            ! GTK_WIDGET_VISIBLE (shell->statusbar)))
+  if (! measure->dialog)
     {
-      measure->dialog = gimp_measure_tool_dialog_new (measure);
-      g_object_add_weak_pointer (G_OBJECT (measure->dialog),
-                                 (gpointer) &measure->dialog);
+      if (options->use_info_window ||
+          ! gimp_display_shell_get_show_statusbar (shell))
+        {
+          measure->dialog = gimp_measure_tool_dialog_new (measure);
+          g_object_add_weak_pointer (G_OBJECT (measure->dialog),
+                                     (gpointer) &measure->dialog);
+        }
     }
 
   if (measure->dialog)
     gimp_viewable_dialog_set_viewable (GIMP_VIEWABLE_DIALOG (measure->dialog),
-                                       GIMP_VIEWABLE (tool->display->image),
+                                       GIMP_VIEWABLE (image),
                                        GIMP_CONTEXT (options));
 }
 
@@ -737,7 +742,7 @@ gimp_measure_tool_draw (GimpDrawTool *draw_tool)
               gdouble           target;
               gdouble           arc_radius;
 
-              shell = GIMP_DISPLAY_SHELL (tool->display->shell);
+              shell = gimp_display_get_shell (tool->display);
 
               target     = FUNSCALEX (shell, (TARGET >> 1));
               arc_radius = FUNSCALEX (shell, ARC_RADIUS);
@@ -807,8 +812,8 @@ static void
 gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
                                  GimpDisplay     *display)
 {
-  GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
-  GimpImage        *image = display->image;
+  GimpDisplayShell *shell = gimp_display_get_shell (display);
+  GimpImage        *image = gimp_display_get_image (display);
   gint              ax, ay;
   gint              bx, by;
   gint              pixel_width;
@@ -844,15 +849,13 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
 
   gimp_image_get_resolution (image, &xres, &yres);
 
-  unit_width  = (_gimp_unit_get_factor (image->gimp, shell->unit) *
-                 pixel_width / xres);
-  unit_height = (_gimp_unit_get_factor (image->gimp, shell->unit) *
-                 pixel_height / yres);
+  unit_width  = gimp_pixels_to_units (pixel_width,  shell->unit, xres);
+  unit_height = gimp_pixels_to_units (pixel_height, shell->unit, yres);
 
   pixel_distance = sqrt (SQR (ax - bx) + SQR (ay - by));
-  unit_distance  = (_gimp_unit_get_factor (image->gimp, shell->unit) *
-                    sqrt (SQR ((gdouble)(ax - bx) / xres) +
-                          SQR ((gdouble)(ay - by) / yres)));
+  unit_distance  = (gimp_unit_get_factor (shell->unit) *
+                    sqrt (SQR ((gdouble) (ax - bx) / xres) +
+                          SQR ((gdouble) (ay - by) / yres)));
 
   if (measure->num_points != 3)
     bx = ax > 0 ? 1 : -1;
@@ -885,10 +888,10 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
     {
       g_snprintf (format, sizeof (format),
                   "%%.%df %s, %%.2f\302\260 (%%.%df × %%.%df)",
-                  _gimp_unit_get_digits (image->gimp, shell->unit),
-                  _gimp_unit_get_plural (image->gimp, shell->unit),
-                  _gimp_unit_get_digits (image->gimp, shell->unit),
-                  _gimp_unit_get_digits (image->gimp, shell->unit));
+                  gimp_unit_get_digits (shell->unit),
+                  gimp_unit_get_plural (shell->unit),
+                  gimp_unit_get_digits (shell->unit),
+                  gimp_unit_get_digits (shell->unit));
 
       gimp_tool_replace_status (GIMP_TOOL (measure), display, format,
                                 unit_distance, unit_angle,
@@ -901,7 +904,8 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
       gchar buf[128];
 
       g_snprintf (format, sizeof (format), "%%.%df",
-                  _gimp_unit_get_digits (image->gimp, shell->unit));
+                  gimp_unit_get_digits (shell->unit));
+
       /* Distance */
       g_snprintf (buf, sizeof (buf), "%.1f", pixel_distance);
       gtk_label_set_text (GTK_LABEL (measure->distance_label[0]), buf);
@@ -912,7 +916,7 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
           gtk_label_set_text (GTK_LABEL (measure->distance_label[1]), buf);
 
           gtk_label_set_text (GTK_LABEL (measure->unit_label[0]),
-                              _gimp_unit_get_plural (image->gimp, shell->unit));
+                              gimp_unit_get_plural (shell->unit));
         }
       else
         {
@@ -947,7 +951,7 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
           gtk_label_set_text (GTK_LABEL (measure->width_label[1]), buf);
 
           gtk_label_set_text (GTK_LABEL (measure->unit_label[2]),
-                              _gimp_unit_get_plural (image->gimp, shell->unit));
+                              gimp_unit_get_plural (shell->unit));
         }
       else
         {
@@ -965,7 +969,7 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
           gtk_label_set_text (GTK_LABEL (measure->height_label[1]), buf);
 
           gtk_label_set_text (GTK_LABEL (measure->unit_label[3]),
-                              _gimp_unit_get_plural (image->gimp, shell->unit));
+                              gimp_unit_get_plural (shell->unit));
         }
       else
         {
@@ -973,8 +977,8 @@ gimp_measure_tool_dialog_update (GimpMeasureTool *measure,
           gtk_label_set_text (GTK_LABEL (measure->unit_label[3]), NULL);
         }
 
-      if (GTK_WIDGET_VISIBLE (measure->dialog))
-        gdk_window_show (measure->dialog->window);
+      if (gtk_widget_get_visible (measure->dialog))
+        gdk_window_show (gtk_widget_get_window (measure->dialog));
       else
         gtk_widget_show (measure->dialog);
     }
