@@ -1,9 +1,9 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -167,19 +166,23 @@ gimp_move_options_gui (GimpToolOptions *tool_options)
   GObject   *config = G_OBJECT (tool_options);
   GtkWidget *vbox   = gimp_tool_options_gui (tool_options);
   GtkWidget *hbox;
+  GtkWidget *box;
   GtkWidget *label;
   GtkWidget *frame;
   gchar     *title;
 
-  hbox = gimp_prop_enum_stock_box_new (config, "move-type", "gimp", 0, 0);
+  hbox = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Move:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_box_reorder_child (GTK_BOX (hbox), label, 0);
   gtk_widget_show (label);
 
+  box = gimp_prop_enum_stock_box_new (config, "move-type", "gimp", 0, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), box, FALSE, FALSE, 0);
+  gtk_widget_show (box);
+
   /*  tool toggle  */
   title = g_strdup_printf (_("Tool Toggle  (%s)"),
                            gimp_get_mod_string (GDK_SHIFT_MASK));
