@@ -35,6 +35,7 @@
 #include "core/gimpguide.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-guides.h"
+#include "core/gimpimage-pick-layer.h"
 #include "core/gimplayer.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimplayermask.h"
@@ -180,7 +181,7 @@ gimp_move_tool_control (GimpTool       *tool,
                         GimpDisplay    *display)
 {
   GimpMoveTool     *move  = GIMP_MOVE_TOOL (tool);
-  GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell = gimp_display_get_shell (display);
 
   switch (action)
     {
@@ -189,12 +190,12 @@ gimp_move_tool_control (GimpTool       *tool,
 
     case GIMP_TOOL_ACTION_RESUME:
       if (move->guide && gimp_display_shell_get_show_guides (shell))
-        gimp_display_shell_draw_guide (shell, move->guide, TRUE);
+        gimp_display_shell_draw_guide (shell, move->guide, NULL, TRUE);
       break;
 
     case GIMP_TOOL_ACTION_HALT:
       if (move->guide && gimp_display_shell_get_show_guides (shell))
-        gimp_display_shell_draw_guide (shell, move->guide, FALSE);
+        gimp_display_shell_draw_guide (shell, move->guide, NULL, FALSE);
       break;
     }
 
@@ -210,8 +211,9 @@ gimp_move_tool_button_press (GimpTool            *tool,
                              GimpDisplay         *display)
 {
   GimpMoveTool     *move    = GIMP_MOVE_TOOL (tool);
-  GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (display->shell);
   GimpMoveOptions  *options = GIMP_MOVE_TOOL_GET_OPTIONS (tool);
+  GimpDisplayShell *shell   = gimp_display_get_shell (display);
+  GimpImage        *image   = gimp_display_get_image (display);
 
   tool->display = display;
 
@@ -233,9 +235,9 @@ gimp_move_tool_button_press (GimpTool            *tool,
                                          &vectors))
             {
               move->old_active_vectors =
-                gimp_image_get_active_vectors (display->image);
+                gimp_image_get_active_vectors (image);
 
-              gimp_image_set_active_vectors (display->image, vectors);
+              gimp_image_set_active_vectors (image, vectors);
             }
           else
             {
@@ -247,13 +249,10 @@ gimp_move_tool_button_press (GimpTool            *tool,
         {
           GimpGuide *guide;
           GimpLayer *layer;
-          gint       snap_distance;
-
-          snap_distance =
-            GIMP_DISPLAY_CONFIG (display->image->gimp->config)->snap_distance;
+          gint       snap_distance = display->config->snap_distance;
 
           if (gimp_display_shell_get_show_guides (shell) &&
-              (guide = gimp_image_find_guide (display->image,
+              (guide = gimp_image_find_guide (image,
                                               coords->x, coords->y,
                                               FUNSCALEX (shell, snap_distance),
                                               FUNSCALEY (shell, snap_distance))))
@@ -282,18 +281,18 @@ gimp_move_tool_button_press (GimpTool            *tool,
 
               return;
             }
-          else if ((layer = gimp_image_pick_correlate_layer (display->image,
-                                                             coords->x,
-                                                             coords->y)))
+          else if ((layer = gimp_image_pick_layer (image,
+                                                   coords->x,
+                                                   coords->y)))
             {
-              if (gimp_image_get_floating_selection (display->image) &&
+              if (gimp_image_get_floating_selection (image) &&
                   ! gimp_layer_is_floating_sel (layer))
                 {
                   /*  If there is a floating selection, and this aint it,
                    *  use the move tool to anchor it.
                    */
                   move->floating_layer =
-                    gimp_image_get_floating_selection (display->image);
+                    gimp_image_get_floating_selection (image);
 
                   gimp_tool_control_activate (tool->control);
 
@@ -301,10 +300,9 @@ gimp_move_tool_button_press (GimpTool            *tool,
                 }
               else
                 {
-                  move->old_active_layer =
-                    gimp_image_get_active_layer (display->image);
+                  move->old_active_layer = gimp_image_get_active_layer (image);
 
-                  gimp_image_set_active_layer (display->image, layer);
+                  gimp_image_set_active_layer (image, layer);
                 }
             }
           else
@@ -319,22 +317,20 @@ gimp_move_tool_button_press (GimpTool            *tool,
   switch (options->move_type)
     {
     case GIMP_TRANSFORM_TYPE_PATH:
-      if (gimp_image_get_active_vectors (display->image))
+      if (gimp_image_get_active_vectors (image))
         gimp_edit_selection_tool_start (tool, display, coords,
                                         GIMP_TRANSLATE_MODE_VECTORS, TRUE);
       break;
 
     case GIMP_TRANSFORM_TYPE_SELECTION:
-      if (! gimp_channel_is_empty (gimp_image_get_mask (display->image)))
+      if (! gimp_channel_is_empty (gimp_image_get_mask (image)))
         gimp_edit_selection_tool_start (tool, display, coords,
                                         GIMP_TRANSLATE_MODE_MASK, TRUE);
       break;
 
     case GIMP_TRANSFORM_TYPE_LAYER:
       {
-        GimpDrawable *drawable;
-
-        drawable = gimp_image_get_active_drawable (display->image);
+        GimpDrawable *drawable = gimp_image_get_active_drawable (image);
 
         if (GIMP_IS_LAYER_MASK (drawable))
           gimp_edit_selection_tool_start (tool, display, coords,
@@ -359,8 +355,9 @@ gimp_move_tool_button_release (GimpTool              *tool,
                                GimpDisplay           *display)
 {
   GimpMoveTool     *move   = GIMP_MOVE_TOOL (tool);
-  GimpDisplayShell *shell  = GIMP_DISPLAY_SHELL (display->shell);
-  GimpGuiConfig    *config = GIMP_GUI_CONFIG (display->image->gimp->config);
+  GimpGuiConfig    *config = GIMP_GUI_CONFIG (display->gimp->config);
+  GimpDisplayShell *shell  = gimp_display_get_shell (display);
+  GimpImage        *image  = gimp_display_get_image (display);
 
   if (gimp_tool_control_is_active (tool->control))
     gimp_tool_control_halt (tool->control);
@@ -412,7 +409,7 @@ gimp_move_tool_button_release (GimpTool              *tool,
         {
           if (move->guide)
             {
-              gimp_image_remove_guide (display->image, move->guide, TRUE);
+              gimp_image_remove_guide (image, move->guide, TRUE);
               move->guide = NULL;
             }
         }
@@ -420,7 +417,7 @@ gimp_move_tool_button_release (GimpTool              *tool,
         {
           if (move->guide)
             {
-              gimp_image_move_guide (display->image, move->guide,
+              gimp_image_move_guide (image, move->guide,
                                      move->guide_position, TRUE);
             }
           else
@@ -428,13 +425,13 @@ gimp_move_tool_button_release (GimpTool              *tool,
               switch (move->guide_orientation)
                 {
                 case GIMP_ORIENTATION_HORIZONTAL:
-                  move->guide = gimp_image_add_hguide (display->image,
+                  move->guide = gimp_image_add_hguide (image,
                                                        move->guide_position,
                                                        TRUE);
                   break;
 
                 case GIMP_ORIENTATION_VERTICAL:
-                  move->guide = gimp_image_add_vguide (display->image,
+                  move->guide = gimp_image_add_vguide (image,
                                                        move->guide_position,
                                                        TRUE);
                   break;
@@ -446,10 +443,10 @@ gimp_move_tool_button_release (GimpTool              *tool,
         }
 
       gimp_display_shell_selection_control (shell, GIMP_SELECTION_RESUME);
-      gimp_image_flush (display->image);
+      gimp_image_flush (image);
 
       if (move->guide)
-        gimp_display_shell_draw_guide (shell, move->guide, TRUE);
+        gimp_display_shell_draw_guide (shell, move->guide, NULL, TRUE);
 
       move->moving_guide      = FALSE;
       move->guide_position    = -1;
@@ -464,8 +461,7 @@ gimp_move_tool_button_release (GimpTool              *tool,
         {
           if (move->old_active_layer)
             {
-              gimp_image_set_active_layer (display->image,
-                                           move->old_active_layer);
+              gimp_image_set_active_layer (image, move->old_active_layer);
               move->old_active_layer = NULL;
 
               flush = TRUE;
@@ -473,8 +469,7 @@ gimp_move_tool_button_release (GimpTool              *tool,
 
           if (move->old_active_vectors)
             {
-              gimp_image_set_active_vectors (display->image,
-                                             move->old_active_vectors);
+              gimp_image_set_active_vectors (image, move->old_active_vectors);
               move->old_active_vectors = NULL;
 
               flush = TRUE;
@@ -492,7 +487,7 @@ gimp_move_tool_button_release (GimpTool              *tool,
         }
 
       if (flush)
-        gimp_image_flush (display->image);
+        gimp_image_flush (image);
     }
 }
 
@@ -505,7 +500,7 @@ gimp_move_tool_motion (GimpTool         *tool,
 
 {
   GimpMoveTool     *move  = GIMP_MOVE_TOOL (tool);
-  GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell = gimp_display_get_shell (display);
 
   if (move->moving_guide)
     {
@@ -655,7 +650,8 @@ gimp_move_tool_oper_update (GimpTool         *tool,
 {
   GimpMoveTool     *move    = GIMP_MOVE_TOOL (tool);
   GimpMoveOptions  *options = GIMP_MOVE_TOOL_GET_OPTIONS (tool);
-  GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (display->shell);
+  GimpDisplayShell *shell   = gimp_display_get_shell (display);
+  GimpImage        *image   = gimp_display_get_image (display);
   GimpGuide        *guide   = NULL;
 
   if (options->move_type == GIMP_TRANSFORM_TYPE_LAYER &&
@@ -663,23 +659,20 @@ gimp_move_tool_oper_update (GimpTool         *tool,
       gimp_display_shell_get_show_guides (shell)      &&
       proximity)
     {
-      gint snap_distance;
+      gint snap_distance = display->config->snap_distance;
 
-      snap_distance =
-        GIMP_DISPLAY_CONFIG (display->image->gimp->config)->snap_distance;
-
-      guide = gimp_image_find_guide (display->image, coords->x, coords->y,
+      guide = gimp_image_find_guide (image, coords->x, coords->y,
                                      FUNSCALEX (shell, snap_distance),
                                      FUNSCALEY (shell, snap_distance));
     }
 
   if (move->guide && move->guide != guide)
-    gimp_display_shell_draw_guide (shell, move->guide, FALSE);
+    gimp_display_shell_draw_guide (shell, move->guide, NULL, FALSE);
 
   move->guide = guide;
 
   if (move->guide)
-    gimp_display_shell_draw_guide (shell, move->guide, TRUE);
+    gimp_display_shell_draw_guide (shell, move->guide, NULL, TRUE);
 }
 
 static void
@@ -688,8 +681,9 @@ gimp_move_tool_cursor_update (GimpTool         *tool,
                               GdkModifierType   state,
                               GimpDisplay      *display)
 {
-  GimpDisplayShell   *shell       = GIMP_DISPLAY_SHELL (display->shell);
   GimpMoveOptions    *options     = GIMP_MOVE_TOOL_GET_OPTIONS (tool);
+  GimpDisplayShell   *shell       = gimp_display_get_shell (display);
+  GimpImage          *image       = gimp_display_get_image (display);
   GimpCursorType      cursor      = GIMP_CURSOR_MOUSE;
   GimpToolCursorType  tool_cursor = GIMP_TOOL_CURSOR_MOVE;
   GimpCursorModifier  modifier    = GIMP_CURSOR_MODIFIER_NONE;
@@ -701,7 +695,7 @@ gimp_move_tool_cursor_update (GimpTool         *tool,
 
       if (options->move_current)
         {
-          if (! gimp_image_get_active_vectors (display->image))
+          if (! gimp_image_get_active_vectors (image))
             modifier = GIMP_CURSOR_MODIFIER_BAD;
         }
       else
@@ -723,42 +717,39 @@ gimp_move_tool_cursor_update (GimpTool         *tool,
       tool_cursor = GIMP_TOOL_CURSOR_RECT_SELECT;
       modifier    = GIMP_CURSOR_MODIFIER_MOVE;
 
-      if (gimp_channel_is_empty (gimp_image_get_mask (display->image)))
+      if (gimp_channel_is_empty (gimp_image_get_mask (image)))
         modifier = GIMP_CURSOR_MODIFIER_BAD;
     }
   else if (options->move_current)
     {
-      if (! gimp_image_get_active_drawable (display->image))
+      if (! gimp_image_get_active_drawable (image))
         modifier = GIMP_CURSOR_MODIFIER_BAD;
     }
   else
     {
       GimpGuide *guide;
       GimpLayer *layer;
-      gint       snap_distance;
-
-      snap_distance =
-        GIMP_DISPLAY_CONFIG (display->image->gimp->config)->snap_distance;
+      gint       snap_distance = display->config->snap_distance;
 
       if (gimp_display_shell_get_show_guides (shell) &&
-          (guide = gimp_image_find_guide (display->image, coords->x, coords->y,
+          (guide = gimp_image_find_guide (image, coords->x, coords->y,
                                           FUNSCALEX (shell, snap_distance),
                                           FUNSCALEY (shell, snap_distance))))
         {
           tool_cursor = GIMP_TOOL_CURSOR_HAND;
           modifier    = GIMP_CURSOR_MODIFIER_MOVE;
         }
-      else if ((layer = gimp_image_pick_correlate_layer (display->image,
-                                                         coords->x, coords->y)))
+      else if ((layer = gimp_image_pick_layer (image,
+                                               coords->x, coords->y)))
         {
           /*  if there is a floating selection, and this aint it...  */
-          if (gimp_image_get_floating_selection (display->image) &&
+          if (gimp_image_get_floating_selection (image) &&
               ! gimp_layer_is_floating_sel (layer))
             {
               tool_cursor = GIMP_TOOL_CURSOR_MOVE;
               modifier    = GIMP_CURSOR_MODIFIER_ANCHOR;
             }
-          else if (layer != gimp_image_get_active_layer (display->image))
+          else if (layer != gimp_image_get_active_layer (image))
             {
               tool_cursor = GIMP_TOOL_CURSOR_HAND;
               modifier    = GIMP_CURSOR_MODIFIER_MOVE;
@@ -819,7 +810,7 @@ gimp_move_tool_start_guide (GimpMoveTool        *move,
 {
   GimpTool *tool = GIMP_TOOL (move);
 
-  gimp_display_shell_selection_control (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_selection_control (gimp_display_get_shell (display),
                                         GIMP_SELECTION_PAUSE);
 
   tool->display = display;
@@ -827,8 +818,8 @@ gimp_move_tool_start_guide (GimpMoveTool        *move,
   gimp_tool_control_set_scroll_lock (tool->control, TRUE);
 
   if (move->guide)
-    gimp_display_shell_draw_guide (GIMP_DISPLAY_SHELL (display->shell),
-                                   move->guide, FALSE);
+    gimp_display_shell_draw_guide (gimp_display_get_shell (display),
+                                   move->guide, NULL, FALSE);
 
   move->guide             = NULL;
   move->moving_guide      = TRUE;
