@@ -20,6 +20,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "tools-types.h"
@@ -27,11 +28,11 @@
 #include "core/gimp.h"
 #include "core/gimp-utils.h"
 #include "core/gimpdrawable.h"
+#include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimppaintinfo.h"
 #include "core/gimpprojection.h"
 #include "core/gimptoolinfo.h"
-#include "core/gimpunit.h"
 
 #include "paint/gimppaintcore.h"
 #include "paint/gimppaintoptions.h"
@@ -49,8 +50,7 @@
 #include "gimp-intl.h"
 
 
-#define HANDLE_SIZE    15
-#define STATUSBAR_SIZE 200
+#define HANDLE_SIZE 15
 
 
 static GObject * gimp_paint_tool_constructor (GType                  type,
@@ -83,6 +83,10 @@ static void   gimp_paint_tool_modifier_key   (GimpTool              *tool,
                                               gboolean               press,
                                               GdkModifierType        state,
                                               GimpDisplay           *display);
+static void   gimp_paint_tool_cursor_update  (GimpTool              *tool,
+                                              const GimpCoords      *coords,
+                                              GdkModifierType        state,
+                                              GimpDisplay           *display);
 static void   gimp_paint_tool_oper_update    (GimpTool              *tool,
                                               const GimpCoords      *coords,
                                               GdkModifierType        state,
@@ -116,6 +120,7 @@ gimp_paint_tool_class_init (GimpPaintToolClass *klass)
   tool_class->button_release = gimp_paint_tool_button_release;
   tool_class->motion         = gimp_paint_tool_motion;
   tool_class->modifier_key   = gimp_paint_tool_modifier_key;
+  tool_class->cursor_update  = gimp_paint_tool_cursor_update;
   tool_class->oper_update    = gimp_paint_tool_oper_update;
 
   draw_tool_class->draw      = gimp_paint_tool_draw;
@@ -222,7 +227,8 @@ gimp_paint_tool_control (GimpTool       *tool,
                          GimpDisplay    *display)
 {
   GimpPaintTool *paint_tool = GIMP_PAINT_TOOL (tool);
-  GimpDrawable  *drawable   = gimp_image_get_active_drawable (display->image);
+  GimpImage     *image      = gimp_display_get_image (display);
+  GimpDrawable  *drawable   = gimp_image_get_active_drawable (image);
 
   switch (action)
     {
@@ -254,7 +260,8 @@ gimp_paint_tool_button_press (GimpTool            *tool,
   GimpPaintTool    *paint_tool    = GIMP_PAINT_TOOL (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_TOOL_GET_OPTIONS (tool);
   GimpPaintCore    *core          = paint_tool->core;
-  GimpDrawable     *drawable;
+  GimpImage        *image         = gimp_display_get_image (display);
+  GimpDrawable     *drawable      = gimp_image_get_active_drawable (image);
   GimpCoords        curr_coords;
   gint              off_x, off_y;
   GError           *error = NULL;
@@ -266,9 +273,21 @@ gimp_paint_tool_button_press (GimpTool            *tool,
       return;
     }
 
-  curr_coords = *coords;
+  if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+    {
+      gimp_tool_message_literal (tool, display,
+                                 _("Cannot paint on layer groups."));
+      return;
+    }
 
-  drawable = gimp_image_get_active_drawable (display->image);
+  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+    {
+      gimp_tool_message_literal (tool, display,
+                                 _("The active layer's pixels are locked."));
+      return;
+    }
+
+  curr_coords = *coords;
 
   gimp_item_get_offset (GIMP_ITEM (drawable), &off_x, &off_y);
 
@@ -278,9 +297,9 @@ gimp_paint_tool_button_press (GimpTool            *tool,
   if (gimp_draw_tool_is_active (draw_tool))
     gimp_draw_tool_stop (draw_tool);
 
-  if (tool->display          &&
+  if (tool->display            &&
       tool->display != display &&
-      tool->display->image == display->image)
+      gimp_display_get_image (tool->display) == image)
     {
       /*  if this is a different display, but the same image, HACK around
        *  in tool internals AFTER stopping the current draw_tool, so
@@ -325,7 +344,7 @@ gimp_paint_tool_button_press (GimpTool            *tool,
                                                 press_type, display);
 
   /*  pause the current selection  */
-  gimp_image_selection_control (display->image, GIMP_SELECTION_PAUSE);
+  gimp_image_selection_control (image, GIMP_SELECTION_PAUSE);
 
   /*  Let the specific painting function initialize itself  */
   gimp_paint_core_paint (core, drawable, paint_options,
@@ -343,7 +362,7 @@ gimp_paint_tool_button_press (GimpTool            *tool,
                              GIMP_PAINT_STATE_MOTION, time);
     }
 
-  gimp_projection_flush_now (gimp_image_get_projection (display->image));
+  gimp_projection_flush_now (gimp_image_get_projection (image));
   gimp_display_flush_now (display);
 
   gimp_draw_tool_start (draw_tool, display);
@@ -360,7 +379,8 @@ gimp_paint_tool_button_release (GimpTool              *tool,
   GimpPaintTool    *paint_tool    = GIMP_PAINT_TOOL (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_TOOL_GET_OPTIONS (tool);
   GimpPaintCore    *core          = paint_tool->core;
-  GimpDrawable     *drawable;
+  GimpImage        *image         = gimp_display_get_image (display);
+  GimpDrawable     *drawable      = gimp_image_get_active_drawable (image);
 
   if (gimp_color_tool_is_enabled (GIMP_COLOR_TOOL (tool)))
     {
@@ -370,8 +390,6 @@ gimp_paint_tool_button_release (GimpTool              *tool,
       return;
     }
 
-  drawable = gimp_image_get_active_drawable (display->image);
-
   gimp_draw_tool_pause (GIMP_DRAW_TOOL (tool));
 
   /*  Let the specific painting function finish up  */
@@ -379,7 +397,7 @@ gimp_paint_tool_button_release (GimpTool              *tool,
                          GIMP_PAINT_STATE_FINISH, time);
 
   /*  resume the current selection  */
-  gimp_image_selection_control (display->image, GIMP_SELECTION_RESUME);
+  gimp_image_selection_control (image, GIMP_SELECTION_RESUME);
 
   /*  chain up to halt the tool */
   GIMP_TOOL_CLASS (parent_class)->button_release (tool, coords, time, state,
@@ -390,7 +408,7 @@ gimp_paint_tool_button_release (GimpTool              *tool,
   else
     gimp_paint_core_finish (core, drawable, TRUE);
 
-  gimp_image_flush (display->image);
+  gimp_image_flush (image);
 
   gimp_draw_tool_resume (GIMP_DRAW_TOOL (tool));
 }
@@ -405,7 +423,8 @@ gimp_paint_tool_motion (GimpTool         *tool,
   GimpPaintTool    *paint_tool    = GIMP_PAINT_TOOL (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_TOOL_GET_OPTIONS (tool);
   GimpPaintCore    *core          = paint_tool->core;
-  GimpDrawable     *drawable;
+  GimpImage        *image         = gimp_display_get_image (display);
+  GimpDrawable     *drawable      = gimp_image_get_active_drawable (image);
   GimpCoords        curr_coords;
   gint              off_x, off_y;
 
@@ -416,8 +435,6 @@ gimp_paint_tool_motion (GimpTool         *tool,
 
   curr_coords = *coords;
 
-  drawable = gimp_image_get_active_drawable (display->image);
-
   gimp_item_get_offset (GIMP_ITEM (drawable), &off_x, &off_y);
 
   curr_coords.x -= off_x;
@@ -435,7 +452,7 @@ gimp_paint_tool_motion (GimpTool         *tool,
   gimp_paint_core_interpolate (core, drawable, paint_options,
                                &curr_coords, time);
 
-  gimp_projection_flush_now (gimp_image_get_projection (display->image));
+  gimp_projection_flush_now (gimp_image_get_projection (image));
   gimp_display_flush_now (display);
 
   gimp_draw_tool_resume (GIMP_DRAW_TOOL (tool));
@@ -458,7 +475,7 @@ gimp_paint_tool_modifier_key (GimpTool        *tool,
     {
       if (press)
         {
-          GimpToolInfo *info = gimp_get_tool_info (display->image->gimp,
+          GimpToolInfo *info = gimp_get_tool_info (display->gimp,
                                                    "gimp-color-picker-tool");
 
           if (GIMP_IS_TOOL_INFO (info))
@@ -499,6 +516,53 @@ gimp_paint_tool_modifier_key (GimpTool        *tool,
     }
 }
 
+static void
+gimp_paint_tool_cursor_update (GimpTool         *tool,
+                               const GimpCoords *coords,
+                               GdkModifierType   state,
+                               GimpDisplay      *display)
+{
+  GimpCursorModifier  modifier;
+  GimpCursorModifier  toggle_modifier;
+  GimpCursorModifier  old_modifier;
+  GimpCursorModifier  old_toggle_modifier;
+
+  modifier        = tool->control->cursor_modifier;
+  toggle_modifier = tool->control->toggle_cursor_modifier;
+
+  old_modifier        = modifier;
+  old_toggle_modifier = toggle_modifier;
+
+  if (! gimp_color_tool_is_enabled (GIMP_COLOR_TOOL (tool)))
+    {
+      GimpImage    *image    = gimp_display_get_image (display);
+      GimpDrawable *drawable = gimp_image_get_active_drawable (image);
+
+      if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)) ||
+          gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+        {
+          modifier        = GIMP_CURSOR_MODIFIER_BAD;
+          toggle_modifier = GIMP_CURSOR_MODIFIER_BAD;
+        }
+
+      gimp_tool_control_set_cursor_modifier        (tool->control,
+                                                    modifier);
+      gimp_tool_control_set_toggle_cursor_modifier (tool->control,
+                                                    toggle_modifier);
+    }
+
+  GIMP_TOOL_CLASS (parent_class)->cursor_update (tool, coords, state,
+                                                 display);
+
+  /*  reset old stuff here so we are not interferring with the modifiers
+   *  set by our subclasses
+   */
+  gimp_tool_control_set_cursor_modifier        (tool->control,
+                                                old_modifier);
+  gimp_tool_control_set_toggle_cursor_modifier (tool->control,
+                                                old_toggle_modifier);
+}
+
 static void
 gimp_paint_tool_oper_update (GimpTool         *tool,
                              const GimpCoords *coords,
@@ -510,8 +574,9 @@ gimp_paint_tool_oper_update (GimpTool         *tool,
   GimpDrawTool     *draw_tool     = GIMP_DRAW_TOOL (tool);
   GimpPaintOptions *paint_options = GIMP_PAINT_TOOL_GET_OPTIONS (tool);
   GimpPaintCore    *core          = paint_tool->core;
-  GimpDisplayShell *shell         = GIMP_DISPLAY_SHELL (display->shell);
-  GimpDrawable     *drawable;
+  GimpDisplayShell *shell         = gimp_display_get_shell (display);
+  GimpImage        *image         = gimp_display_get_image (display);
+  GimpDrawable     *drawable      = gimp_image_get_active_drawable (image);
 
   if (gimp_color_tool_is_enabled (GIMP_COLOR_TOOL (tool)))
     {
@@ -527,7 +592,7 @@ gimp_paint_tool_oper_update (GimpTool         *tool,
 
   if (tool->display            &&
       tool->display != display &&
-      tool->display->image == display->image)
+      gimp_display_get_image (tool->display) == image)
     {
       /*  if this is a different display, but the same image, HACK around
        *  in tool internals AFTER stopping the current draw_tool, so
@@ -538,8 +603,6 @@ gimp_paint_tool_oper_update (GimpTool         *tool,
       tool->display = display;
     }
 
-  drawable = gimp_image_get_active_drawable (display->image);
-
   if (drawable && proximity)
     {
       if (display == tool->display && (state & GDK_SHIFT_MASK))
@@ -581,18 +644,17 @@ gimp_paint_tool_oper_update (GimpTool         *tool,
             }
           else
             {
-              GimpImage *image = display->image;
-              gdouble    xres;
-              gdouble    yres;
-              gchar      format_str[64];
+              gdouble xres;
+              gdouble yres;
+              gchar   format_str[64];
 
               gimp_image_get_resolution (image, &xres, &yres);
 
               g_snprintf (format_str, sizeof (format_str), "%%.%df %s.  %%s",
-                          _gimp_unit_get_digits (image->gimp, shell->unit),
-                          _gimp_unit_get_symbol (image->gimp, shell->unit));
+                          gimp_unit_get_digits (shell->unit),
+                          gimp_unit_get_symbol (shell->unit));
 
-              dist = (_gimp_unit_get_factor (image->gimp, shell->unit) *
+              dist = (gimp_unit_get_factor (shell->unit) *
                       sqrt (SQR (dx / xres) +
                             SQR (dy / yres)));
 
