@@ -138,9 +138,10 @@ gimp_posterize_tool_initialize (GimpTool     *tool,
                                 GError      **error)
 {
   GimpPosterizeTool *posterize_tool = GIMP_POSTERIZE_TOOL (tool);
+  GimpImage         *image          = gimp_display_get_image (display);
   GimpDrawable      *drawable;
 
-  drawable = gimp_image_get_active_drawable (display->image);
+  drawable = gimp_image_get_active_drawable (image);
 
   if (! drawable)
     return FALSE;
@@ -154,7 +155,10 @@ gimp_posterize_tool_initialize (GimpTool     *tool,
 
   gimp_config_reset (GIMP_CONFIG (posterize_tool->config));
 
-  GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error);
+  if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
+    {
+      return FALSE;
+    }
 
   gtk_adjustment_set_value (posterize_tool->levels_data,
                             posterize_tool->config->levels);
