@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-1999 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -43,7 +42,7 @@ enum
   COLUMN_LEFT_NUMBER,
   COLUMN_RIGHT_NUMBER,
   COLUMN_TEXT,
-  NUM_COLUMNS
+  N_COLUMNS
 };
 
 
@@ -115,13 +114,6 @@ gimp_rectangle_options_iface_base_init (GimpRectangleOptionsInterface *iface)
                                                                  GIMP_CONFIG_PARAM_FLAGS |
                                                                  GIMP_PARAM_STATIC_STRINGS));
 
-      g_object_interface_install_property (iface,
-                                           g_param_spec_boolean ("highlight",
-                                                                 NULL, NULL,
-                                                                 TRUE,
-                                                                 GIMP_CONFIG_PARAM_FLAGS |
-                                                                 GIMP_PARAM_STATIC_STRINGS));
-
       g_object_interface_install_property (iface,
                                            g_param_spec_enum ("guide",
                                                               NULL, NULL,
@@ -342,12 +334,12 @@ gimp_rectangle_options_get_private (GimpRectangleOptions *options)
     {
       private = g_slice_new0 (GimpRectangleOptionsPrivate);
 
-      private->aspect_history = gtk_list_store_new (NUM_COLUMNS,
+      private->aspect_history = gtk_list_store_new (N_COLUMNS,
                                                     G_TYPE_DOUBLE,
                                                     G_TYPE_DOUBLE,
                                                     G_TYPE_STRING);
 
-      private->size_history = gtk_list_store_new (NUM_COLUMNS,
+      private->size_history = gtk_list_store_new (N_COLUMNS,
                                                   G_TYPE_DOUBLE,
                                                   G_TYPE_DOUBLE,
                                                   G_TYPE_STRING);
@@ -379,9 +371,6 @@ gimp_rectangle_options_install_properties (GObjectClass *klass)
   g_object_class_override_property (klass,
                                     GIMP_RECTANGLE_OPTIONS_PROP_SHRINK_MERGED,
                                     "shrink-merged");
-  g_object_class_override_property (klass,
-                                    GIMP_RECTANGLE_OPTIONS_PROP_HIGHLIGHT,
-                                    "highlight");
   g_object_class_override_property (klass,
                                     GIMP_RECTANGLE_OPTIONS_PROP_GUIDE,
                                     "guide");
