@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -69,6 +68,15 @@ gimp_rectangle_select_options_class_init (GimpRectangleSelectOptionsClass *klass
   object_class->set_property = gimp_rectangle_select_options_set_property;
   object_class->get_property = gimp_rectangle_select_options_get_property;
 
+  /* The 'highlight' property is defined here because we want different
+   * default values for the Crop and the Rectangle Select tools.
+   */
+  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class,
+                                    GIMP_RECTANGLE_OPTIONS_PROP_HIGHLIGHT,
+                                    "highlight", NULL,
+                                    FALSE,
+                                    GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_ROUND_CORNERS,
                                     "round-corners", NULL,
                                     FALSE,
@@ -176,10 +184,6 @@ gimp_rectangle_select_options_gui (GimpToolOptions *tool_options)
     vbox_rectangle = gimp_rectangle_options_gui (tool_options);
     gtk_box_pack_start (GTK_BOX (vbox), vbox_rectangle, FALSE, FALSE, 0);
     gtk_widget_show (vbox_rectangle);
-
-    g_object_set (GIMP_RECTANGLE_OPTIONS (tool_options),
-                  "highlight", FALSE,
-                  NULL);
   }
 
   return vbox;
