@@ -27,6 +27,7 @@
 
 #include "core/gimpchannel.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-pick-layer.h"
 #include "core/gimppickable.h"
 
 #include "display/gimpdisplay.h"
@@ -159,6 +160,7 @@ gimp_selection_tool_oper_update (GimpTool         *tool,
 {
   GimpSelectionTool    *selection_tool = GIMP_SELECTION_TOOL (tool);
   GimpSelectionOptions *options;
+  GimpImage            *image;
   GimpChannel          *selection;
   GimpDrawable         *drawable;
   GimpLayer            *layer;
@@ -169,11 +171,11 @@ gimp_selection_tool_oper_update (GimpTool         *tool,
 
   options = GIMP_SELECTION_TOOL_GET_OPTIONS (tool);
 
-  selection    = gimp_image_get_mask (display->image);
-  drawable     = gimp_image_get_active_drawable (display->image);
-  layer        = gimp_image_pick_correlate_layer (display->image,
-                                                  coords->x, coords->y);
-  floating_sel = gimp_image_get_floating_selection (display->image);
+  image        = gimp_display_get_image (display);
+  selection    = gimp_image_get_mask (image);
+  drawable     = gimp_image_get_active_drawable (image);
+  layer        = gimp_image_pick_layer (image, coords->x, coords->y);
+  floating_sel = gimp_image_get_floating_selection (image);
 
   if (drawable)
     {
