@@ -36,6 +36,7 @@
 #include "widgets/gimpdock.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimppropwidgets.h"
+#include "widgets/gimptextbuffer.h"
 #include "widgets/gimptexteditor.h"
 #include "widgets/gimpviewablebox.h"
 #include "widgets/gimpwidgets-utils.h"
@@ -60,6 +61,8 @@ enum
   PROP_INDENTATION,
   PROP_LINE_SPACING,
   PROP_LETTER_SPACING,
+  PROP_BOX_MODE,
+
   PROP_USE_EDITOR,
 
   PROP_FONT_VIEW_TYPE,
@@ -67,6 +70,7 @@ enum
 };
 
 
+static void  gimp_text_options_finalize           (GObject      *object);
 static void  gimp_text_options_set_property       (GObject      *object,
                                                    guint         property_id,
                                                    const GValue *value,
@@ -95,12 +99,15 @@ G_DEFINE_TYPE_WITH_CODE (GimpTextOptions, gimp_text_options,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_RECTANGLE_OPTIONS,
                                                 NULL))
 
+#define parent_class gimp_text_options_parent_class
+
 
 static void
 gimp_text_options_class_init (GimpTextOptionsClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->finalize     = gimp_text_options_finalize;
   object_class->set_property = gimp_text_options_set_property;
   object_class->get_property = gimp_text_options_get_property;
 
@@ -167,6 +174,12 @@ gimp_text_options_class_init (GimpTextOptionsClass *klass)
                                    -8192.0, 8192.0, 0.0,
                                    GIMP_PARAM_STATIC_STRINGS |
                                    GIMP_CONFIG_PARAM_DEFAULTS);
+  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_BOX_MODE,
+                                "box-mode",
+                                 N_("Text box resize mode"),
+                                 GIMP_TYPE_TEXT_BOX_MODE,
+                                 GIMP_TEXT_BOX_DYNAMIC,
+                                 GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_EDITOR,
                                     "use-editor",
@@ -197,6 +210,20 @@ gimp_text_options_init (GimpTextOptions *options)
   options->size_entry = NULL;
 }
 
+static void
+gimp_text_options_finalize (GObject *object)
+{
+  GimpTextOptions *options = GIMP_TEXT_OPTIONS (object);
+
+  if (options->language)
+    {
+      g_free (options->language);
+      options->language = NULL;
+    }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gimp_text_options_get_property (GObject    *object,
                                 guint       property_id,
@@ -237,6 +264,9 @@ gimp_text_options_get_property (GObject    *object,
     case PROP_LETTER_SPACING:
       g_value_set_double (value, options->letter_spacing);
       break;
+    case PROP_BOX_MODE:
+      g_value_set_enum (value, options->box_mode);
+      break;
 
     case PROP_USE_EDITOR:
       g_value_set_boolean (value, options->use_editor);
@@ -296,6 +326,9 @@ gimp_text_options_set_property (GObject      *object,
     case PROP_LETTER_SPACING:
       options->letter_spacing = g_value_get_double (value);
       break;
+    case PROP_BOX_MODE:
+      options->box_mode = g_value_get_enum (value);
+      break;
 
     case PROP_USE_EDITOR:
       options->use_editor = g_value_get_boolean (value);
@@ -433,6 +466,7 @@ gimp_text_options_gui (GimpToolOptions *tool_options)
   GtkWidget       *box;
   GtkWidget       *label;
   GtkWidget       *spinbutton;
+  GtkWidget       *combo;
   GtkSizeGroup    *size_group;
   gint             row = 0;
 
@@ -517,6 +551,11 @@ gimp_text_options_gui (GimpToolOptions *tool_options)
   gimp_table_attach_stock (GTK_TABLE (table), row++,
                            GIMP_STOCK_LETTER_SPACING, spinbutton, 1, TRUE);
 
+  combo = gimp_prop_enum_combo_box_new (config, "box-mode", 0, 0);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
+                             _("Box:"), 0.0, 0.5,
+                             combo, 1, TRUE);
+
   /*  Only add the language entry if the iso-codes package is available.  */
 
 #ifdef HAVE_ISO_CODES
@@ -580,7 +619,7 @@ gimp_text_options_editor_new (GtkWindow       *parent,
                               GimpTextOptions *options,
                               GimpMenuFactory *menu_factory,
                               const gchar     *title,
-                              GtkTextBuffer   *text_buffer)
+                              GimpTextBuffer  *text_buffer)
 {
   GtkWidget   *editor;
   const gchar *font_name;
@@ -588,6 +627,7 @@ gimp_text_options_editor_new (GtkWindow       *parent,
   g_return_val_if_fail (GIMP_IS_TEXT_OPTIONS (options), NULL);
   g_return_val_if_fail (GIMP_IS_MENU_FACTORY (menu_factory), NULL);
   g_return_val_if_fail (title != NULL, NULL);
+  g_return_val_if_fail (GIMP_IS_TEXT_BUFFER (text_buffer), NULL);
 
   editor = gimp_text_editor_new (title, parent, menu_factory, text_buffer);
 
