@@ -47,6 +47,7 @@ struct _GimpTextOptions
   gdouble                indent;
   gdouble                line_spacing;
   gdouble                letter_spacing;
+  GimpTextBoxMode        box_mode;
 
   GimpViewType           font_view_type;
   GimpViewSize           font_view_size;
@@ -69,7 +70,7 @@ GtkWidget * gimp_text_options_editor_new   (GtkWindow       *parent,
                                             GimpTextOptions *options,
                                             GimpMenuFactory *menu_factory,
                                             const gchar     *title,
-                                            GtkTextBuffer   *text_buffer);
+                                            GimpTextBuffer  *text_buffer);
 
 
 #endif /* __GIMP_TEXT_OPTIONS_H__ */
