@@ -88,6 +88,9 @@ static void       gimp_tool_real_motion         (GimpTool              *tool,
 static gboolean   gimp_tool_real_key_press      (GimpTool              *tool,
                                                  GdkEventKey           *kevent,
                                                  GimpDisplay           *display);
+static gboolean   gimp_tool_real_key_release    (GimpTool              *tool,
+                                                 GdkEventKey           *kevent,
+                                                 GimpDisplay           *display);
 static void       gimp_tool_real_modifier_key   (GimpTool              *tool,
                                                  GdkModifierType        key,
                                                  gboolean               press,
@@ -140,6 +143,7 @@ gimp_tool_class_init (GimpToolClass *klass)
   klass->button_release      = gimp_tool_real_button_release;
   klass->motion              = gimp_tool_real_motion;
   klass->key_press           = gimp_tool_real_key_press;
+  klass->key_release         = gimp_tool_real_key_release;
   klass->modifier_key        = gimp_tool_real_modifier_key;
   klass->active_modifier_key = gimp_tool_real_active_modifier_key;
   klass->oper_update         = gimp_tool_real_oper_update;
@@ -252,7 +256,7 @@ gimp_tool_real_has_image (GimpTool  *tool,
 {
   if (tool->display)
     {
-      if (image && tool->display->image == image)
+      if (image && gimp_display_get_image (tool->display) == image)
         return tool->display;
 
       /*  NULL image means any display  */
@@ -298,8 +302,10 @@ gimp_tool_real_button_press (GimpTool            *tool,
 {
   if (press_type == GIMP_BUTTON_PRESS_NORMAL)
     {
+      GimpImage *image = gimp_display_get_image (display);
+
       tool->display  = display;
-      tool->drawable = gimp_image_get_active_drawable (display->image);
+      tool->drawable = gimp_image_get_active_drawable (image);
 
       gimp_tool_control_activate (tool->control);
     }
@@ -333,6 +339,14 @@ gimp_tool_real_key_press (GimpTool    *tool,
   return FALSE;
 }
 
+static gboolean
+gimp_tool_real_key_release (GimpTool    *tool,
+                            GdkEventKey *kevent,
+                            GimpDisplay *display)
+{
+  return FALSE;
+}
+
 static void
 gimp_tool_real_modifier_key (GimpTool        *tool,
                              GdkModifierType  key,
@@ -428,7 +442,7 @@ gimp_tool_has_image (GimpTool  *tool,
         {
           GimpDisplay *status_display = list->data;
 
-          if (status_display->image == image)
+          if (gimp_display_get_image (status_display) == image)
             return status_display;
         }
 
@@ -546,13 +560,16 @@ gimp_tool_check_click_distance (GimpTool         *tool,
                                 guint32           time,
                                 GimpDisplay      *display)
 {
-  gint double_click_time;
-  gint double_click_distance;
+  GimpDisplayShell *shell;
+  gint              double_click_time;
+  gint              double_click_distance;
 
   if (! tool->in_click_distance)
     return FALSE;
 
-  g_object_get (gtk_widget_get_settings (display->shell),
+  shell = gimp_display_get_shell (display);
+
+  g_object_get (gtk_widget_get_settings (GTK_WIDGET (shell)),
                 "gtk-double-click-time",     &double_click_time,
                 "gtk-double-click-distance", &double_click_distance,
                 NULL);
@@ -563,7 +580,7 @@ gimp_tool_check_click_distance (GimpTool         *tool,
     }
   else
     {
-      GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (display->shell);
+      GimpDisplayShell *shell = gimp_display_get_shell (display);
       gdouble           dx;
       gdouble           dy;
 
@@ -687,6 +704,18 @@ gimp_tool_key_press (GimpTool    *tool,
   return GIMP_TOOL_GET_CLASS (tool)->key_press (tool, kevent, display);
 }
 
+gboolean
+gimp_tool_key_release (GimpTool    *tool,
+                       GdkEventKey *kevent,
+                       GimpDisplay *display)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL (tool), FALSE);
+  g_return_val_if_fail (GIMP_IS_DISPLAY (display), FALSE);
+  g_return_val_if_fail (display == tool->focus_display, FALSE);
+
+  return GIMP_TOOL_GET_CLASS (tool)->key_release (tool, kevent, display);
+}
+
 static void
 gimp_tool_modifier_key (GimpTool        *tool,
                         GdkModifierType  key,
@@ -849,7 +878,7 @@ gimp_tool_oper_update (GimpTool         *tool,
   GIMP_TOOL_GET_CLASS (tool)->oper_update (tool, coords, state, proximity,
                                            display);
 
-  if (G_UNLIKELY (gimp_image_is_empty (display->image) &&
+  if (G_UNLIKELY (gimp_image_is_empty (gimp_display_get_image (display)) &&
                   ! gimp_tool_control_get_handle_empty_image (tool->control)))
     {
       gimp_tool_replace_status (tool, display,
@@ -902,13 +931,13 @@ gimp_tool_push_status (GimpTool    *tool,
   g_return_if_fail (GIMP_IS_DISPLAY (display));
   g_return_if_fail (format != NULL);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   stock_id = gimp_viewable_get_stock_id (GIMP_VIEWABLE (tool->tool_info));
 
   va_start (args, format);
 
-  gimp_statusbar_push_valist (GIMP_STATUSBAR (shell->statusbar),
+  gimp_statusbar_push_valist (gimp_display_shell_get_statusbar (shell),
                               G_OBJECT_TYPE_NAME (tool), stock_id,
                               format, args);
 
@@ -934,11 +963,11 @@ gimp_tool_push_status_coords (GimpTool            *tool,
   g_return_if_fail (GIMP_IS_TOOL (tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   stock_id = gimp_viewable_get_stock_id (GIMP_VIEWABLE (tool->tool_info));
 
-  gimp_statusbar_push_coords (GIMP_STATUSBAR (shell->statusbar),
+  gimp_statusbar_push_coords (gimp_display_shell_get_statusbar (shell),
                               G_OBJECT_TYPE_NAME (tool), stock_id,
                               precision, title, x, separator, y,
                               help);
@@ -961,11 +990,11 @@ gimp_tool_push_status_length (GimpTool            *tool,
   g_return_if_fail (GIMP_IS_TOOL (tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   stock_id = gimp_viewable_get_stock_id (GIMP_VIEWABLE (tool->tool_info));
 
-  gimp_statusbar_push_length (GIMP_STATUSBAR (shell->statusbar),
+  gimp_statusbar_push_length (gimp_display_shell_get_statusbar (shell),
                               G_OBJECT_TYPE_NAME (tool), stock_id,
                               title, axis, value, help);
 
@@ -987,13 +1016,13 @@ gimp_tool_replace_status (GimpTool    *tool,
   g_return_if_fail (GIMP_IS_DISPLAY (display));
   g_return_if_fail (format != NULL);
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
   stock_id = gimp_viewable_get_stock_id (GIMP_VIEWABLE (tool->tool_info));
 
   va_start (args, format);
 
-  gimp_statusbar_replace_valist (GIMP_STATUSBAR (shell->statusbar),
+  gimp_statusbar_replace_valist (gimp_display_shell_get_statusbar (shell),
                                  G_OBJECT_TYPE_NAME (tool), stock_id,
                                  format, args);
 
@@ -1012,9 +1041,9 @@ gimp_tool_pop_status (GimpTool    *tool,
   g_return_if_fail (GIMP_IS_TOOL (tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  shell = GIMP_DISPLAY_SHELL (display->shell);
+  shell = gimp_display_get_shell (display);
 
-  gimp_statusbar_pop (GIMP_STATUSBAR (shell->statusbar),
+  gimp_statusbar_pop (gimp_display_shell_get_statusbar (shell),
                       G_OBJECT_TYPE_NAME (tool));
 
   tool->status_displays = g_list_remove (tool->status_displays, display);
@@ -1034,7 +1063,7 @@ gimp_tool_message (GimpTool    *tool,
 
   va_start (args, format);
 
-  gimp_message_valist (display->image->gimp, G_OBJECT (display),
+  gimp_message_valist (display->gimp, G_OBJECT (display),
                        GIMP_MESSAGE_WARNING, format, args);
 
   va_end (args);
@@ -1049,7 +1078,7 @@ gimp_tool_message_literal (GimpTool    *tool,
   g_return_if_fail (GIMP_IS_DISPLAY (display));
   g_return_if_fail (message != NULL);
 
-  gimp_message_literal (display->image->gimp, G_OBJECT (display),
+  gimp_message_literal (display->gimp, G_OBJECT (display),
 			GIMP_MESSAGE_WARNING, message);
 }
 
@@ -1063,7 +1092,7 @@ gimp_tool_set_cursor (GimpTool           *tool,
   g_return_if_fail (GIMP_IS_TOOL (tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  gimp_display_shell_set_cursor (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_set_cursor (gimp_display_get_shell (display),
                                  cursor, tool_cursor, modifier);
 }
 
