@@ -35,6 +35,7 @@
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpdrawable-transform.h"
+#include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
@@ -54,6 +55,7 @@
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
 #include "display/gimpdisplayshell-appearance.h"
+#include "display/gimpdisplayshell-expose.h"
 #include "display/gimpdisplayshell-transform.h"
 
 #include "gimptoolcontrol.h"
@@ -332,7 +334,21 @@ gimp_transform_tool_initialize (GimpTool     *tool,
                                 GimpDisplay  *display,
                                 GError      **error)
 {
-  GimpTransformTool *tr_tool = GIMP_TRANSFORM_TOOL (tool);
+  GimpTransformTool *tr_tool  = GIMP_TRANSFORM_TOOL (tool);
+  GimpImage         *image    = gimp_display_get_image (display);
+  GimpDrawable      *drawable = gimp_image_get_active_drawable (image);
+
+  if (! GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
+    {
+      return FALSE;
+    }
+
+  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+    {
+      g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
+			   _("The active layer's pixels are locked."));
+      return FALSE;
+    }
 
   if (display != tool->display)
     {
@@ -340,7 +356,7 @@ gimp_transform_tool_initialize (GimpTool     *tool,
 
       /*  Set the pointer to the active display  */
       tool->display  = display;
-      tool->drawable = gimp_image_get_active_drawable (display->image);
+      tool->drawable = drawable;
 
       /*  Initialize the transform tool dialog */
       if (! tr_tool->dialog)
@@ -687,6 +703,7 @@ gimp_transform_tool_cursor_update (GimpTool         *tool,
   GimpTransformOptions *options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tool);
   GimpCursorType        cursor;
   GimpCursorModifier    modifier = GIMP_CURSOR_MODIFIER_NONE;
+  GimpImage            *image    = gimp_display_get_image (display);
 
   cursor = gimp_tool_control_get_cursor (tool->control);
 
@@ -739,12 +756,19 @@ gimp_transform_tool_cursor_update (GimpTool         *tool,
 
   switch (options->type)
     {
+      GimpDrawable *drawable;
+
     case GIMP_TRANSFORM_TYPE_LAYER:
+      drawable = gimp_image_get_active_drawable (image);
+      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+        modifier = GIMP_CURSOR_MODIFIER_BAD;
+      break;
+
     case GIMP_TRANSFORM_TYPE_SELECTION:
       break;
 
     case GIMP_TRANSFORM_TYPE_PATH:
-      if (! gimp_image_get_active_vectors (display->image))
+      if (! gimp_image_get_active_vectors (image))
         modifier = GIMP_CURSOR_MODIFIER_BAD;
       break;
     }
@@ -760,6 +784,7 @@ gimp_transform_tool_draw (GimpDrawTool *draw_tool)
 {
   GimpTool          *tool    = GIMP_TOOL (draw_tool);
   GimpTransformTool *tr_tool = GIMP_TRANSFORM_TOOL (draw_tool);
+  GimpImage         *image   = gimp_display_get_image (tool->display);
   gdouble            z1, z2, z3, z4;
 
   if (tr_tool->use_grid)
@@ -924,7 +949,7 @@ gimp_transform_tool_draw (GimpDrawTool *draw_tool)
       gint            num_groups;
       gint            i;
 
-      gimp_channel_boundary (gimp_image_get_mask (tool->display->image),
+      gimp_channel_boundary (gimp_image_get_mask (image),
                              &orig_in, &orig_out,
                              &num_segs_in, &num_segs_out,
                              0, 0, 0, 0);
@@ -1005,7 +1030,7 @@ gimp_transform_tool_draw (GimpDrawTool *draw_tool)
       GimpStroke  *stroke = NULL;
       GimpMatrix3  matrix = tr_tool->transform;
 
-      vectors = gimp_image_get_active_vectors (tool->display->image);
+      vectors = gimp_image_get_active_vectors (image);
 
       if (vectors)
         {
@@ -1148,46 +1173,60 @@ static void
 gimp_transform_tool_doit (GimpTransformTool *tr_tool,
                           GimpDisplay       *display)
 {
-  GimpTool             *tool        = GIMP_TOOL (tr_tool);
-  GimpTransformOptions *options     = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tool);
-  GimpContext          *context     = GIMP_CONTEXT (options);
-  GimpDisplayShell     *shell       = GIMP_DISPLAY_SHELL (display->shell);
-  GimpItem             *active_item = NULL;
+  GimpTool             *tool           = GIMP_TOOL (tr_tool);
+  GimpTransformOptions *options        = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tool);
+  GimpContext          *context        = GIMP_CONTEXT (options);
+  GimpDisplayShell     *shell          = gimp_display_get_shell (display);
+  GimpImage            *image          = gimp_display_get_image (display);
+  GimpItem             *active_item    = NULL;
   TileManager          *new_tiles;
-  const gchar          *message     = NULL;
+  const gchar          *null_message   = NULL;
+  const gchar          *locked_message = NULL;
   gboolean              new_layer;
   gboolean              mask_empty;
 
   switch (options->type)
     {
     case GIMP_TRANSFORM_TYPE_LAYER:
-      active_item = GIMP_ITEM (gimp_image_get_active_drawable (display->image));
-      message = _("There is no layer to transform.");
+      active_item = GIMP_ITEM (gimp_image_get_active_drawable (image));
+      null_message   = _("There is no layer to transform.");
+      locked_message = _("The active layer's pixels are locked.");
       break;
 
     case GIMP_TRANSFORM_TYPE_SELECTION:
-      active_item = GIMP_ITEM (gimp_image_get_mask (display->image));
-      /* cannot happen, so don't translate this message */
-      message = "There is no selection to transform.";
+      active_item = GIMP_ITEM (gimp_image_get_mask (image));
+      /* cannot happen, so don't translate these messages */
+      null_message   = "There is no selection to transform.";
+      locked_message = "The selection's pixels are locked.";
       break;
 
     case GIMP_TRANSFORM_TYPE_PATH:
-      active_item = GIMP_ITEM (gimp_image_get_active_vectors (display->image));
-      message = _("There is no path to transform.");
+      active_item = GIMP_ITEM (gimp_image_get_active_vectors (image));
+      null_message   = _("There is no path to transform.");
+      locked_message = _("The active path's strokes are locked.");
       break;
     }
 
   if (! active_item)
     {
-      gimp_tool_message_literal (tool, display, message);
+      gimp_tool_message_literal (tool, display, null_message);
+      gimp_transform_tool_halt (tr_tool);
       return;
     }
 
-  gimp_dialog_factory_hide_dialog (tr_tool->dialog);
+  if (gimp_item_is_content_locked (active_item))
+    {
+      gimp_tool_message_literal (tool, display, locked_message);
+      gimp_transform_tool_halt (tr_tool);
+      return;
+    }
 
-  mask_empty = gimp_channel_is_empty (gimp_image_get_mask (display->image));
+  if (tr_tool->dialog)
+    gimp_dialog_factory_hide_dialog (tr_tool->dialog);
 
-  gimp_set_busy (display->image->gimp);
+  mask_empty = gimp_channel_is_empty (gimp_image_get_mask (image));
+
+  gimp_set_busy (display->gimp);
 
   /* undraw the tool before we muck around with the transform matrix */
   gimp_draw_tool_stop (GIMP_DRAW_TOOL (tr_tool));
@@ -1196,7 +1235,7 @@ gimp_transform_tool_doit (GimpTransformTool *tr_tool,
   gimp_tool_control_set_preserve (tool->control, TRUE);
 
   /*  Start a transform undo group  */
-  gimp_image_undo_group_start (display->image, GIMP_UNDO_GROUP_TRANSFORM,
+  gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_TRANSFORM,
                                tr_tool->undo_desc);
 
   /* With the old UI, if original is NULL, then this is the
@@ -1208,7 +1247,7 @@ gimp_transform_tool_doit (GimpTransformTool *tr_tool,
    *  selection pointer, so that the original source can be repeatedly
    *  modified.
    */
-  tool->drawable = gimp_image_get_active_drawable (display->image);
+  tool->drawable = gimp_image_get_active_drawable (image);
 
   switch (options->type)
     {
@@ -1277,16 +1316,16 @@ gimp_transform_tool_doit (GimpTransformTool *tr_tool,
   /*  Make a note of the new current drawable (since we may have
    *  a floating selection, etc now.
    */
-  tool->drawable = gimp_image_get_active_drawable (display->image);
+  tool->drawable = gimp_image_get_active_drawable (image);
 
-  gimp_image_undo_push (display->image, GIMP_TYPE_TRANSFORM_TOOL_UNDO,
+  gimp_image_undo_push (image, GIMP_TYPE_TRANSFORM_TOOL_UNDO,
                         GIMP_UNDO_TRANSFORM, NULL,
                         0,
                         "transform-tool", tr_tool,
                         NULL);
 
   /*  push the undo group end  */
-  gimp_image_undo_group_end (display->image);
+  gimp_image_undo_group_end (image);
 
   /*  We're done dirtying the image, and would like to be restarted
    *  if the image gets dirty while the tool exists
@@ -1301,9 +1340,9 @@ gimp_transform_tool_doit (GimpTransformTool *tr_tool,
       gtk_widget_queue_draw (shell->canvas);
     }
 
-  gimp_unset_busy (display->image->gimp);
+  gimp_unset_busy (display->gimp);
 
-  gimp_image_flush (display->image);
+  gimp_image_flush (image);
 
   gimp_transform_tool_halt (tr_tool);
 }
@@ -1389,7 +1428,7 @@ gimp_transform_tool_force_expose_preview (GimpTransformTool *tr_tool)
   if (! gimp_draw_tool_is_active (GIMP_DRAW_TOOL (tr_tool)))
     return;
 
-  shell = GIMP_DISPLAY_SHELL (GIMP_DRAW_TOOL (tr_tool)->display->shell);
+  shell = gimp_display_get_shell (GIMP_DRAW_TOOL (tr_tool)->display);
 
   gimp_display_shell_transform_xy_f (shell, tr_tool->tx1, tr_tool->ty1,
                                      dx + 0, dy + 0, FALSE);
@@ -1442,7 +1481,7 @@ gimp_transform_tool_halt (GimpTransformTool *tr_tool)
     {
       GimpDisplayShell *shell;
 
-      shell = GIMP_DISPLAY_SHELL (GIMP_DRAW_TOOL (tr_tool)->display->shell);
+      shell = gimp_display_get_shell (GIMP_DRAW_TOOL (tr_tool)->display);
 
       if (gimp_display_shell_get_show_transform (shell))
         {
@@ -1477,8 +1516,7 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
                             GimpDisplay       *display)
 {
   GimpTransformOptions *options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
-
-  g_return_if_fail (GIMP_IS_DISPLAY (display));
+  GimpImage            *image   = gimp_display_get_image (display);
 
   /*  find the boundaries  */
   if (tr_tool->original)
@@ -1498,7 +1536,7 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
             gint          offset_x;
             gint          offset_y;
 
-            drawable = gimp_image_get_active_drawable (display->image);
+            drawable = gimp_image_get_active_drawable (image);
 
             gimp_item_get_offset (GIMP_ITEM (drawable), &offset_x, &offset_y);
 
@@ -1514,7 +1552,7 @@ gimp_transform_tool_bounds (GimpTransformTool *tr_tool,
 
         case GIMP_TRANSFORM_TYPE_SELECTION:
         case GIMP_TRANSFORM_TYPE_PATH:
-          gimp_channel_bounds (gimp_image_get_mask (display->image),
+          gimp_channel_bounds (gimp_image_get_mask (image),
                                &tr_tool->x1, &tr_tool->y1,
                                &tr_tool->x2, &tr_tool->y2);
           break;
@@ -1636,19 +1674,19 @@ gimp_transform_tool_handles_recalc (GimpTransformTool *tr_tool,
   gint x1, y1;
   gint x2, y2;
 
-  gimp_display_shell_transform_xy (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_transform_xy (gimp_display_get_shell (display),
                                    tr_tool->tx1, tr_tool->ty1,
                                    &dx1, &dy1,
                                    FALSE);
-  gimp_display_shell_transform_xy (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_transform_xy (gimp_display_get_shell (display),
                                    tr_tool->tx2, tr_tool->ty2,
                                    &dx2, &dy2,
                                    FALSE);
-  gimp_display_shell_transform_xy (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_transform_xy (gimp_display_get_shell (display),
                                    tr_tool->tx3, tr_tool->ty3,
                                    &dx3, &dy3,
                                    FALSE);
-  gimp_display_shell_transform_xy (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_transform_xy (gimp_display_get_shell (display),
                                    tr_tool->tx4, tr_tool->ty4,
                                    &dx4, &dy4,
                                    FALSE);
@@ -1701,6 +1739,7 @@ gimp_transform_tool_prepare (GimpTransformTool *tr_tool,
                              GimpDisplay       *display)
 {
   GimpTransformOptions *options = GIMP_TRANSFORM_TOOL_GET_OPTIONS (tr_tool);
+  GimpImage            *image   = gimp_display_get_image (display);
   gboolean              show_transform;
 
   show_transform =
@@ -1709,12 +1748,12 @@ gimp_transform_tool_prepare (GimpTransformTool *tr_tool,
      options->type         == GIMP_TRANSFORM_TYPE_LAYER &&
      options->direction    == GIMP_TRANSFORM_FORWARD);
 
-  gimp_display_shell_set_show_transform (GIMP_DISPLAY_SHELL (display->shell),
+  gimp_display_shell_set_show_transform (gimp_display_get_shell (display),
                                          show_transform);
 
   if (tr_tool->dialog)
     {
-      GimpDrawable *drawable = gimp_image_get_active_drawable (display->image);
+      GimpDrawable *drawable = gimp_image_get_active_drawable (image);
 
       gimp_viewable_dialog_set_viewable (GIMP_VIEWABLE_DIALOG (tr_tool->dialog),
                                          GIMP_VIEWABLE (drawable),
@@ -1823,7 +1862,7 @@ gimp_transform_tool_notify_preview (GimpTransformOptions *options,
   GimpDisplayShell *shell = NULL;
 
   if (gimp_draw_tool_is_active (GIMP_DRAW_TOOL (tr_tool)))
-    shell = GIMP_DISPLAY_SHELL (GIMP_DRAW_TOOL (tr_tool)->display->shell);
+    shell = gimp_display_get_shell (GIMP_DRAW_TOOL (tr_tool)->display);
 
   switch (options->preview_type)
     {
