@@ -33,6 +33,7 @@
 #include "core/gimpcontext.h"
 #include "core/gimpchannel-select.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
 #include "core/gimplist.h"
 #include "core/gimpprogress.h"
@@ -243,6 +244,22 @@ gimp_vector_tool_control (GimpTool       *tool,
   GIMP_TOOL_CLASS (parent_class)->control (tool, action, display);
 }
 
+static gboolean
+gimp_vector_tool_check_writable (GimpVectorTool *vector_tool)
+{
+  if (gimp_item_is_content_locked (GIMP_ITEM (vector_tool->vectors)))
+    {
+      gimp_tool_message_literal (GIMP_TOOL (vector_tool),
+                                 GIMP_TOOL (vector_tool)->display,
+                                 _("The active path is locked."));
+      vector_tool->function = VECTORS_FINISHED;
+
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void
 gimp_vector_tool_button_press (GimpTool            *tool,
                                const GimpCoords    *coords,
@@ -254,6 +271,7 @@ gimp_vector_tool_button_press (GimpTool            *tool,
   GimpDrawTool      *draw_tool   = GIMP_DRAW_TOOL (tool);
   GimpVectorTool    *vector_tool = GIMP_VECTOR_TOOL (tool);
   GimpVectorOptions *options     = GIMP_VECTOR_TOOL_GET_OPTIONS (tool);
+  GimpImage         *image       = gimp_display_get_image (display);
   GimpVectors       *vectors;
 
   /* do nothing if we are an FINISHED state */
@@ -288,7 +306,7 @@ gimp_vector_tool_button_press (GimpTool            *tool,
                                      NULL, NULL, NULL, NULL, NULL, &vectors))
         {
           gimp_vector_tool_set_vectors (vector_tool, vectors);
-          gimp_image_set_active_vectors (display->image, vectors);
+          gimp_image_set_active_vectors (image, vectors);
         }
 
       vector_tool->function = VECTORS_FINISHED;
@@ -298,16 +316,16 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   if (vector_tool->function == VECTORS_CREATE_VECTOR)
     {
-      vectors = gimp_vectors_new (display->image, _("Unnamed"));
+      vectors = gimp_vectors_new (image, _("Unnamed"));
 
       /* Undo step gets added implicitely */
       vector_tool->have_undo = TRUE;
 
       vector_tool->undo_motion = TRUE;
 
-      gimp_image_add_vectors (display->image, vectors,
+      gimp_image_add_vectors (image, vectors,
                               GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
-      gimp_image_flush (display->image);
+      gimp_image_flush (image);
 
       gimp_vector_tool_set_vectors (vector_tool, vectors);
 
@@ -318,10 +336,9 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* create a new stroke */
 
-  if (vector_tool->function == VECTORS_CREATE_STROKE)
+  if (vector_tool->function == VECTORS_CREATE_STROKE &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
-      g_return_if_fail (vector_tool->vectors != NULL);
-
       gimp_vector_tool_undo_push (vector_tool, _("Add Stroke"));
 
       vector_tool->cur_stroke = gimp_bezier_stroke_new ();
@@ -338,7 +355,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* add an anchor to an existing stroke */
 
-  if (vector_tool->function == VECTORS_ADD_ANCHOR)
+  if (vector_tool->function == VECTORS_ADD_ANCHOR &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       GimpCoords position = GIMP_COORDS_DEFAULT_VALUES;
 
@@ -368,7 +386,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* insertion of an anchor in a curve segment */
 
-  if (vector_tool->function == VECTORS_INSERT_ANCHOR)
+  if (vector_tool->function == VECTORS_INSERT_ANCHOR &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Insert Anchor"));
 
@@ -398,7 +417,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* move a handle */
 
-  if (vector_tool->function == VECTORS_MOVE_HANDLE)
+  if (vector_tool->function == VECTORS_MOVE_HANDLE &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Drag Handle"));
 
@@ -427,7 +447,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* move an anchor */
 
-  if (vector_tool->function == VECTORS_MOVE_ANCHOR)
+  if (vector_tool->function == VECTORS_MOVE_ANCHOR &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Drag Anchor"));
 
@@ -444,7 +465,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* move multiple anchors */
 
-  if (vector_tool->function == VECTORS_MOVE_ANCHORSET)
+  if (vector_tool->function == VECTORS_MOVE_ANCHORSET &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Drag Anchors"));
 
@@ -466,7 +488,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* move a curve segment directly */
 
-  if (vector_tool->function == VECTORS_MOVE_CURVE)
+  if (vector_tool->function == VECTORS_MOVE_CURVE &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Drag Curve"));
 
@@ -494,7 +517,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* connect two strokes */
 
-  if (vector_tool->function == VECTORS_CONNECT_STROKES)
+  if (vector_tool->function == VECTORS_CONNECT_STROKES &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Connect Strokes"));
 
@@ -524,8 +548,9 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* move a stroke or all strokes of a vectors object */
 
-  if (vector_tool->function == VECTORS_MOVE_STROKE ||
-      vector_tool->function == VECTORS_MOVE_VECTORS)
+  if ((vector_tool->function == VECTORS_MOVE_STROKE ||
+       vector_tool->function == VECTORS_MOVE_VECTORS) &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Drag Path"));
 
@@ -535,7 +560,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* convert an anchor to something that looks like an edge */
 
-  if (vector_tool->function == VECTORS_CONVERT_EDGE)
+  if (vector_tool->function == VECTORS_CONVERT_EDGE &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Convert Edge"));
 
@@ -565,7 +591,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* removal of a node in a stroke */
 
-  if (vector_tool->function == VECTORS_DELETE_ANCHOR)
+  if (vector_tool->function == VECTORS_DELETE_ANCHOR &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       gimp_vector_tool_undo_push (vector_tool, _("Delete Anchor"));
 
@@ -585,7 +612,8 @@ gimp_vector_tool_button_press (GimpTool            *tool,
 
   /* deleting a segment (opening up a stroke) */
 
-  if (vector_tool->function == VECTORS_DELETE_SEGMENT)
+  if (vector_tool->function == VECTORS_DELETE_SEGMENT &&
+      gimp_vector_tool_check_writable (vector_tool))
     {
       GimpStroke *new_stroke;
 
@@ -624,6 +652,7 @@ gimp_vector_tool_button_release (GimpTool              *tool,
                                  GimpDisplay           *display)
 {
   GimpVectorTool *vector_tool = GIMP_VECTOR_TOOL (tool);
+  GimpImage      *image       = gimp_display_get_image (display);
 
   vector_tool->function = VECTORS_FINISHED;
 
@@ -634,10 +663,10 @@ gimp_vector_tool_button_release (GimpTool              *tool,
       GimpUndo            *undo;
       GimpUndoAccumulator  accum = { 0, };
 
-      undo = gimp_undo_stack_pop_undo (display->image->undo_stack,
+      undo = gimp_undo_stack_pop_undo (gimp_image_get_undo_stack (image),
                                        GIMP_UNDO_MODE_UNDO, &accum);
 
-      gimp_image_undo_event (display->image, GIMP_UNDO_EVENT_UNDO_EXPIRED, undo);
+      gimp_image_undo_event (image, GIMP_UNDO_EVENT_UNDO_EXPIRED, undo);
 
       gimp_undo_free (undo, GIMP_UNDO_MODE_UNDO);
       g_object_unref (undo);
@@ -647,7 +676,7 @@ gimp_vector_tool_button_release (GimpTool              *tool,
   vector_tool->undo_motion = FALSE;
 
   gimp_tool_control_halt (tool->control);
-  gimp_image_flush (display->image);
+  gimp_image_flush (image);
 }
 
 static void
@@ -779,7 +808,7 @@ gimp_vector_tool_key_press (GimpTool     *tool,
   if (display != draw_tool->display)
     return FALSE;
 
-  shell = GIMP_DISPLAY_SHELL (draw_tool->display->shell);
+  shell = gimp_display_get_shell (draw_tool->display);
 
   if (kevent->state & GDK_SHIFT_MASK)
     pixels = 10.0;
@@ -847,7 +876,7 @@ gimp_vector_tool_key_press (GimpTool     *tool,
       return FALSE;
     }
 
-  gimp_image_flush (display->image);
+  gimp_image_flush (gimp_display_get_image (display));
 
   return TRUE;
 }
@@ -1204,10 +1233,20 @@ gimp_vector_tool_status_update (GimpTool        *tool,
           break;
 
         case VECTORS_MOVE_HANDLE:
-          status = gimp_suggest_modifiers (_("Click-Drag to move the handle "
-                                             "around"),
-                                           GDK_SHIFT_MASK & ~state,
-                                           NULL, NULL, NULL);
+          if (vector_tool->restriction != GIMP_ANCHOR_FEATURE_SYMMETRIC)
+            {
+              status = gimp_suggest_modifiers (_("Click-Drag to move the "
+                                                 "handle around"),
+                                               GDK_SHIFT_MASK & ~state,
+                                               NULL, NULL, NULL);
+            }
+          else
+            {
+              status = gimp_suggest_modifiers (_("Click-Drag to move the "
+                                                 "handles around symmetrically"),
+                                               GDK_SHIFT_MASK & ~state,
+                                               NULL, NULL, NULL);
+            }
           free_status = TRUE;
           break;
 
@@ -1554,7 +1593,8 @@ gimp_vector_tool_set_vectors (GimpVectorTool *vector_tool,
   gimp_draw_tool_pause (draw_tool);
 
   if (gimp_draw_tool_is_active (draw_tool) &&
-      (! vectors || draw_tool->display->image != gimp_item_get_image (item)))
+      (! vectors ||
+       gimp_display_get_image (draw_tool->display) != gimp_item_get_image (item)))
     {
       gimp_draw_tool_stop (draw_tool);
     }
@@ -1657,7 +1697,8 @@ gimp_vector_tool_set_vectors (GimpVectorTool *vector_tool,
 
   if (! gimp_draw_tool_is_active (draw_tool))
     {
-      if (tool->display && tool->display->image == gimp_item_get_image (item))
+      if (tool->display &&
+          gimp_display_get_image (tool->display) == gimp_item_get_image (item))
         {
           gimp_draw_tool_start (draw_tool, tool->display);
         }
@@ -1669,7 +1710,8 @@ gimp_vector_tool_set_vectors (GimpVectorTool *vector_tool,
           context = gimp_get_user_context (tool->tool_info->gimp);
           display = gimp_context_get_display (context);
 
-          if (! display || display->image != gimp_item_get_image (item))
+          if (! display ||
+              gimp_display_get_image (display) != gimp_item_get_image (item))
             {
               GList *list;
 
@@ -1681,7 +1723,7 @@ gimp_vector_tool_set_vectors (GimpVectorTool *vector_tool,
                 {
                   display = list->data;
 
-                  if (display->image == gimp_item_get_image (item))
+                  if (gimp_display_get_image (display) == gimp_item_get_image (item))
                     {
                       gimp_context_set_display (context, display);
                       break;
