@@ -63,9 +63,8 @@ static void       gimp_vectors_finalize      (GObject           *object);
 static gint64     gimp_vectors_get_memsize   (GimpObject        *object,
                                               gint64            *gui_size);
 
-static gboolean   gimp_vectors_is_attached   (GimpItem          *item);
-static GimpContainer *
-                  gimp_vectors_get_container (GimpItem          *item);
+static gboolean   gimp_vectors_is_attached   (const GimpItem    *item);
+static GimpItemTree * gimp_vectors_get_tree  (GimpItem          *item);
 static GimpItem * gimp_vectors_duplicate     (GimpItem          *item,
                                               GType              new_type);
 static void       gimp_vectors_convert       (GimpItem          *item,
@@ -182,7 +181,7 @@ gimp_vectors_class_init (GimpVectorsClass *klass)
   viewable_class->default_stock_id = "gimp-path";
 
   item_class->is_attached          = gimp_vectors_is_attached;
-  item_class->get_container        = gimp_vectors_get_container;
+  item_class->get_tree             = gimp_vectors_get_tree;
   item_class->duplicate            = gimp_vectors_duplicate;
   item_class->convert              = gimp_vectors_convert;
   item_class->translate            = gimp_vectors_translate;
@@ -266,20 +265,23 @@ gimp_vectors_get_memsize (GimpObject *object,
 }
 
 static gboolean
-gimp_vectors_is_attached (GimpItem *item)
+gimp_vectors_is_attached (const GimpItem *item)
 {
-  return (GIMP_IS_IMAGE (gimp_item_get_image (item)) &&
-          gimp_container_have (gimp_item_get_image (item)->vectors, GIMP_OBJECT (item)));
+  GimpImage *image = gimp_item_get_image (item);
+
+  return (GIMP_IS_IMAGE (image) &&
+          gimp_container_have (gimp_image_get_vectors (image),
+                               GIMP_OBJECT (item)));
 }
 
-static GimpContainer *
-gimp_vectors_get_container (GimpItem *item)
+static GimpItemTree *
+gimp_vectors_get_tree (GimpItem *item)
 {
   if (gimp_item_is_attached (item))
     {
       GimpImage *image = gimp_item_get_image (item);
 
-      return gimp_image_get_vectors (image);
+      return gimp_image_get_vectors_tree (image);
     }
 
   return NULL;
@@ -310,8 +312,9 @@ static void
 gimp_vectors_convert (GimpItem  *item,
                       GimpImage *dest_image)
 {
-  item->width  = gimp_image_get_width  (dest_image);
-  item->height = gimp_image_get_height (dest_image);
+  gimp_item_set_size (item,
+                      gimp_image_get_width  (dest_image),
+                      gimp_image_get_height (dest_image));
 
   GIMP_ITEM_CLASS (parent_class)->convert (item, dest_image);
 }
@@ -584,6 +587,14 @@ gimp_vectors_new (GimpImage   *image,
   return vectors;
 }
 
+GimpVectors *
+gimp_vectors_get_parent (GimpVectors *vectors)
+{
+  g_return_val_if_fail (GIMP_IS_VECTORS (vectors), NULL);
+
+  return GIMP_VECTORS (gimp_viewable_get_parent (GIMP_VIEWABLE (vectors)));
+}
+
 void
 gimp_vectors_freeze (GimpVectors *vectors)
 {
