@@ -4,9 +4,9 @@
  * gimpactionview.c
  * Copyright (C) 2004-2005  Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -45,7 +44,10 @@
 /*  local function prototypes  */
 
 static void     gimp_action_view_dispose         (GObject         *object);
+static void     gimp_action_view_finalize        (GObject         *object);
 
+static void     gimp_action_view_select_path     (GimpActionView  *view,
+                                                  GtkTreePath     *path);
 static gboolean gimp_action_view_accel_find_func (GtkAccelKey     *key,
                                                   GClosure        *closure,
                                                   gpointer         data);
@@ -75,7 +77,8 @@ gimp_action_view_class_init (GimpActionViewClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose = gimp_action_view_dispose;
+  object_class->dispose  = gimp_action_view_dispose;
+  object_class->finalize = gimp_action_view_finalize;
 }
 
 static void
@@ -108,12 +111,26 @@ gimp_action_view_dispose (GObject *object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gimp_action_view_finalize (GObject *object)
+{
+  GimpActionView *view = GIMP_ACTION_VIEW (object);
+
+  if (view->filter)
+    {
+      g_free (view->filter);
+      view->filter = NULL;
+    }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static gboolean
 idle_start_editing (GtkTreeView *tree_view)
 {
   GtkTreePath *path;
 
-  path = g_object_get_data (G_OBJECT (tree_view), "start_editing_path");
+  path = g_object_get_data (G_OBJECT (tree_view), "start-editing-path");
 
   if (path)
     {
@@ -123,7 +140,7 @@ idle_start_editing (GtkTreeView *tree_view)
                                 gtk_tree_view_get_column (tree_view, 1),
                                 TRUE);
 
-      g_object_set_data (G_OBJECT (tree_view), "start_editing_path", NULL);
+      g_object_set_data (G_OBJECT (tree_view), "start-editing-path", NULL);
     }
 
   return FALSE;
@@ -154,7 +171,7 @@ gimp_action_view_button_press (GtkWidget      *widget,
           return FALSE;
         }
 
-      g_object_set_data_full (G_OBJECT (tree_view), "start_editing_path",
+      g_object_set_data_full (G_OBJECT (tree_view), "start-editing-path",
                               path, (GDestroyNotify) gtk_tree_path_free);
 
       g_signal_stop_emission_by_name (tree_view, "button-press-event");
@@ -180,20 +197,23 @@ gimp_action_view_new (GimpUIManager *manager,
   GtkTreeViewColumn *column;
   GtkCellRenderer   *cell;
   GtkTreeStore      *store;
+  GtkTreeModel      *filter;
   GtkAccelGroup     *accel_group;
   GList             *list;
   GtkTreePath       *select_path = NULL;
 
   g_return_val_if_fail (GIMP_IS_UI_MANAGER (manager), NULL);
 
-  store = gtk_tree_store_new (GIMP_ACTION_VIEW_NUM_COLUMNS,
-                              GTK_TYPE_ACTION,        /* COLUMN_ACTION        */
-                              G_TYPE_STRING,          /* COLUMN_STOCK_ID      */
-                              G_TYPE_STRING,          /* COLUMN_LABEL         */
-                              G_TYPE_STRING,          /* COLUMN_NAME          */
-                              G_TYPE_UINT,            /* COLUMN_ACCEL_KEY     */
-                              GDK_TYPE_MODIFIER_TYPE, /* COLUMN_ACCEL_MASK    */
-                              G_TYPE_CLOSURE);        /* COLUMN_ACCEL_CLOSURE */
+  store = gtk_tree_store_new (GIMP_ACTION_VIEW_N_COLUMNS,
+                              G_TYPE_BOOLEAN,         /* COLUMN_VISIBLE        */
+                              GTK_TYPE_ACTION,        /* COLUMN_ACTION         */
+                              G_TYPE_STRING,          /* COLUMN_STOCK_ID       */
+                              G_TYPE_STRING,          /* COLUMN_LABEL          */
+                              G_TYPE_STRING,          /* COLUMN_LABEL_CASEFOLD */
+                              G_TYPE_STRING,          /* COLUMN_NAME           */
+                              G_TYPE_UINT,            /* COLUMN_ACCEL_KEY      */
+                              GDK_TYPE_MODIFIER_TYPE, /* COLUMN_ACCEL_MASK     */
+                              G_TYPE_CLOSURE);        /* COLUMN_ACCEL_CLOSURE  */
 
   accel_group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (manager));
 
@@ -221,9 +241,9 @@ gimp_action_view_new (GimpUIManager *manager,
         {
           GtkAction       *action        = list2->data;
           const gchar     *name          = gtk_action_get_name (action);
-          gchar           *stock_id;
+          const gchar     *stock_id      = gtk_action_get_stock_id (action);
           gchar           *label;
-          gchar           *tmp;
+          gchar           *label_casefold;
           guint            accel_key     = 0;
           GdkModifierType  accel_mask    = 0;
           GClosure        *accel_closure = NULL;
@@ -234,13 +254,7 @@ gimp_action_view_new (GimpUIManager *manager,
               name[0] == '<')
             continue;
 
-          g_object_get (action,
-                        "stock-id", &stock_id,
-                        "label",    &tmp,
-                        NULL);
-
-          label = gimp_strip_uline (tmp);
-          g_free (tmp);
+          label = gimp_strip_uline (gtk_action_get_label (action));
 
           if (! (label && strlen (label)))
             {
@@ -248,6 +262,8 @@ gimp_action_view_new (GimpUIManager *manager,
               label = g_strdup (name);
             }
 
+          label_casefold = g_utf8_casefold (label, -1);
+
           if (show_shortcuts)
             {
               accel_closure = gtk_action_get_accel_closure (action);
@@ -273,17 +289,19 @@ gimp_action_view_new (GimpUIManager *manager,
           gtk_tree_store_append (store, &action_iter, &group_iter);
 
           gtk_tree_store_set (store, &action_iter,
-                              GIMP_ACTION_VIEW_COLUMN_ACTION,        action,
-                              GIMP_ACTION_VIEW_COLUMN_STOCK_ID,      stock_id,
-                              GIMP_ACTION_VIEW_COLUMN_LABEL,         label,
-                              GIMP_ACTION_VIEW_COLUMN_NAME,          name,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,     accel_key,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,    accel_mask,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE, accel_closure,
+                              GIMP_ACTION_VIEW_COLUMN_VISIBLE,        TRUE,
+                              GIMP_ACTION_VIEW_COLUMN_ACTION,         action,
+                              GIMP_ACTION_VIEW_COLUMN_STOCK_ID,       stock_id,
+                              GIMP_ACTION_VIEW_COLUMN_LABEL,          label,
+                              GIMP_ACTION_VIEW_COLUMN_LABEL_CASEFOLD, label_casefold,
+                              GIMP_ACTION_VIEW_COLUMN_NAME,           name,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,      accel_key,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,     accel_mask,
+                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE,  accel_closure,
                               -1);
 
-          g_free (stock_id);
           g_free (label);
+          g_free (label_casefold);
 
           if (select_action && ! strcmp (select_action, name))
             {
@@ -295,12 +313,19 @@ gimp_action_view_new (GimpUIManager *manager,
       g_list_free (actions);
     }
 
+  filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (store), NULL);
+
+  g_object_unref (store);
+
   view = g_object_new (GIMP_TYPE_ACTION_VIEW,
-                       "model",      store,
+                       "model",      filter,
                        "rules-hint", TRUE,
                        NULL);
 
-  g_object_unref (store);
+  g_object_unref (filter);
+
+  gtk_tree_model_filter_set_visible_column (GTK_TREE_MODEL_FILTER (filter),
+                                            GIMP_ACTION_VIEW_COLUMN_VISIBLE);
 
   GIMP_ACTION_VIEW (view)->manager        = g_object_ref (manager);
   GIMP_ACTION_VIEW (view)->show_shortcuts = show_shortcuts;
@@ -377,28 +402,132 @@ gimp_action_view_new (GimpUIManager *manager,
 
   if (select_path)
     {
-      GtkTreeSelection *sel = gtk_tree_view_get_selection (view);
-      GtkTreePath      *expand;
-
-      expand = gtk_tree_path_copy (select_path);
-      gtk_tree_path_up (expand);
-      gtk_tree_view_expand_row (view, expand, FALSE);
-
-      gtk_tree_selection_select_path (sel, select_path);
-
-      gtk_tree_view_scroll_to_cell (view, select_path, NULL,
-                                    TRUE, 0.5, 0.0);
-
+      gimp_action_view_select_path (GIMP_ACTION_VIEW (view), select_path);
       gtk_tree_path_free (select_path);
-      gtk_tree_path_free (expand);
     }
 
   return GTK_WIDGET (view);
 }
 
+void
+gimp_action_view_set_filter (GimpActionView *view,
+                             const gchar    *filter)
+{
+  GtkTreeSelection    *sel;
+  GtkTreeModel        *filtered_model;
+  GtkTreeModel        *model;
+  GtkTreeIter          iter;
+  gboolean             iter_valid;
+  GtkTreeRowReference *selected_row = NULL;
+
+  g_return_if_fail (GIMP_IS_ACTION_VIEW (view));
+
+  filtered_model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+  model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filtered_model));
+
+  if (filter && ! strlen (filter))
+    filter = NULL;
+
+  g_free (view->filter);
+  view->filter = NULL;
+
+  if (filter)
+    view->filter = g_utf8_casefold (filter, -1);
+
+  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+    {
+      GtkTreePath *path = gtk_tree_model_get_path (filtered_model, &iter);
+
+      selected_row = gtk_tree_row_reference_new (filtered_model, path);
+    }
+
+  for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
+       iter_valid;
+       iter_valid = gtk_tree_model_iter_next (model, &iter))
+    {
+      GtkTreeIter child_iter;
+      gboolean    child_valid;
+      gint        n_children = 0;
+
+      for (child_valid = gtk_tree_model_iter_children (model, &child_iter,
+                                                       &iter);
+           child_valid;
+           child_valid = gtk_tree_model_iter_next (model, &child_iter))
+        {
+          gboolean visible = TRUE;
+
+          if (view->filter)
+            {
+              gchar *label;
+              gchar *name;
+
+              gtk_tree_model_get (model, &child_iter,
+                                  GIMP_ACTION_VIEW_COLUMN_LABEL_CASEFOLD, &label,
+                                  GIMP_ACTION_VIEW_COLUMN_NAME,           &name,
+                                  -1);
+
+              visible = label && name && (strstr (label, view->filter) != NULL ||
+                                          strstr (name,  view->filter) != NULL);
+
+              g_free (label);
+              g_free (name);
+            }
+
+          gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
+                              GIMP_ACTION_VIEW_COLUMN_VISIBLE, visible,
+                              -1);
+
+          if (visible)
+            n_children++;
+        }
+
+      gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
+                          GIMP_ACTION_VIEW_COLUMN_VISIBLE, n_children > 0,
+                          -1);
+    }
+
+  if (view->filter)
+    gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
+  else
+    gtk_tree_view_collapse_all (GTK_TREE_VIEW (view));
+
+  gtk_tree_view_columns_autosize (GTK_TREE_VIEW (view));
+
+  if (selected_row)
+    {
+      if (gtk_tree_row_reference_valid (selected_row))
+        {
+          GtkTreePath *path = gtk_tree_row_reference_get_path (selected_row);
+
+          gimp_action_view_select_path (view, path);
+          gtk_tree_path_free (path);
+        }
+
+      gtk_tree_row_reference_free (selected_row);
+    }
+}
+
 
 /*  private functions  */
 
+static void
+gimp_action_view_select_path (GimpActionView *view,
+                              GtkTreePath    *path)
+{
+  GtkTreeView *tv = GTK_TREE_VIEW (view);
+  GtkTreePath *expand;
+
+  expand = gtk_tree_path_copy (path);
+  gtk_tree_path_up (expand);
+  gtk_tree_view_expand_row (tv, expand, FALSE);
+  gtk_tree_path_free (expand);
+
+  gtk_tree_view_set_cursor (tv, path, NULL, FALSE);
+  gtk_tree_view_scroll_to_cell (tv, path, NULL, TRUE, 0.5, 0.0);
+}
+
 static gboolean
 gimp_action_view_accel_find_func (GtkAccelKey *key,
                                   GClosure    *closure,
@@ -422,6 +551,10 @@ gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
   if (! model)
     return;
 
+  model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
+  if (! model)
+    return;
+
   for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
        iter_valid;
        iter_valid = gtk_tree_model_iter_next (model, &iter))
@@ -494,9 +627,9 @@ gimp_action_view_conflict_response (GtkWidget   *dialog,
                                         confirm_data->accel_mask,
                                         TRUE))
         {
-          gimp_message (confirm_data->manager->gimp, G_OBJECT (dialog),
-                        GIMP_MESSAGE_ERROR,
-                        _("Changing shortcut failed."));
+          gimp_message_literal (confirm_data->manager->gimp, G_OBJECT (dialog),
+				GIMP_MESSAGE_ERROR,
+				_("Changing shortcut failed."));
         }
     }
 
@@ -514,19 +647,14 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
 {
   GimpActionGroup *group;
   gchar           *label;
-  gchar           *tmp;
   gchar           *accel_string;
   ConfirmData     *confirm_data;
   GtkWidget       *dialog;
   GimpMessageBox  *box;
 
-  g_object_get (action,
-                "action-group", &group,
-                "label",        &tmp,
-                NULL);
+  g_object_get (action, "action-group", &group, NULL);
 
-  label = gimp_strip_uline (tmp);
-  g_free (tmp);
+  label = gimp_strip_uline (gtk_action_get_label (action));
 
   accel_string = gtk_accelerator_get_label (accel_key, accel_mask);
 
@@ -634,9 +762,9 @@ gimp_action_view_accel_edited (GtkCellRendererAccel *accel,
 
   if (! accel_key)
     {
-      gimp_message (view->manager->gimp, G_OBJECT (view),
-                    GIMP_MESSAGE_ERROR,
-                    _("Invalid shortcut."));
+      gimp_message_literal (view->manager->gimp,
+			    G_OBJECT (view), GIMP_MESSAGE_ERROR,
+			    _("Invalid shortcut."));
     }
   else if (! gtk_accel_map_change_entry (accel_path,
                                          accel_key, accel_mask, FALSE))
@@ -647,6 +775,7 @@ gimp_action_view_accel_edited (GtkCellRendererAccel *accel,
       gboolean      iter_valid;
 
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+      model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
 
       for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
            iter_valid;
@@ -698,9 +827,9 @@ gimp_action_view_accel_edited (GtkCellRendererAccel *accel,
             }
           else
             {
-              gimp_message (view->manager->gimp, G_OBJECT (view),
-                            GIMP_MESSAGE_ERROR,
-                            _("Changing shortcut failed."));
+              gimp_message_literal (view->manager->gimp,
+				    G_OBJECT (view), GIMP_MESSAGE_ERROR,
+				    _("Changing shortcut failed."));
             }
         }
     }
@@ -722,8 +851,8 @@ gimp_action_view_accel_cleared (GtkCellRendererAccel *accel,
 
   if (! gtk_accel_map_change_entry (accel_path, 0, 0, FALSE))
     {
-      gimp_message (view->manager->gimp, G_OBJECT (view),
-                    GIMP_MESSAGE_ERROR,
-                    _("Removing shortcut failed."));
+      gimp_message_literal (view->manager->gimp,
+			    G_OBJECT (view), GIMP_MESSAGE_ERROR,
+			    _("Removing shortcut failed."));
     }
 }
