@@ -4,9 +4,9 @@
  * gimpbrushfactoryview.c
  * Copyright (C) 2001 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -102,9 +101,11 @@ gimp_brush_factory_view_destroy (GtkObject *object)
 
   if (view->spacing_changed_handler_id)
     {
-      GimpContainer *container;
+      GimpDataFactory *factory;
+      GimpContainer   *container;
 
-      container = gimp_container_view_get_container (editor->view);
+      factory   = gimp_data_factory_view_get_data_factory (GIMP_DATA_FACTORY_VIEW (editor));
+      container = gimp_data_factory_get_container (factory);
 
       gimp_container_remove_handler (container,
                                      view->spacing_changed_handler_id);
@@ -155,11 +156,11 @@ gimp_brush_factory_view_new (GimpViewType     view_type,
 
   /*  eek  */
   gtk_box_pack_end (GTK_BOX (editor->view),
-                    factory_view->spacing_scale->parent,
+                    gtk_widget_get_parent (factory_view->spacing_scale),
                     FALSE, FALSE, 0);
 
   factory_view->spacing_changed_handler_id =
-    gimp_container_add_handler (factory->container, "spacing-changed",
+    gimp_container_add_handler (gimp_data_factory_get_container (factory), "spacing-changed",
                                 G_CALLBACK (gimp_brush_factory_view_spacing_changed),
                                 factory_view);
 
@@ -242,7 +243,7 @@ gimp_brush_factory_view_spacing_update (GtkAdjustment        *adjustment,
                                        gimp_brush_factory_view_spacing_changed,
                                        view);
 
-      gimp_brush_set_spacing (brush, adjustment->value);
+      gimp_brush_set_spacing (brush, gtk_adjustment_get_value (adjustment));
 
       g_signal_handlers_unblock_by_func (brush,
                                          gimp_brush_factory_view_spacing_changed,
