@@ -273,7 +273,7 @@ gimp_brush_select_run_callback (GimpPdbDialog  *dialog,
                                         dialog->caller_context,
                                         NULL, error,
                                         dialog->callback_name,
-                                        G_TYPE_STRING,        object->name,
+                                        G_TYPE_STRING,        gimp_object_get_name (object),
                                         G_TYPE_DOUBLE,        gimp_context_get_opacity (dialog->context) * 100.0,
                                         GIMP_TYPE_INT32,      GIMP_BRUSH_SELECT (dialog)->spacing,
                                         GIMP_TYPE_INT32,      gimp_context_get_paint_mode (dialog->context),
