@@ -34,6 +34,7 @@
 
 enum
 {
+  PRE_CLICKED,
   CLICKED,
   LAST_SIGNAL
 };
@@ -91,6 +92,39 @@ gimp_cell_renderer_viewable_class_init (GimpCellRendererViewableClass *klass)
   GObjectClass         *object_class = G_OBJECT_CLASS (klass);
   GtkCellRendererClass *cell_class   = GTK_CELL_RENDERER_CLASS (klass);
 
+  /**
+   * GimpCellRendererViewable::pre-clicked:
+   * @cell:
+   * @path:
+   * @state:
+   *
+   * Called early on a viewable cell when it is clicked, typically
+   * before selection code is invoked for example.
+   *
+   * Returns: %TRUE if the signal handled the event and event
+   *          propagation should stop, for example preventing a
+   *          selection from happening, %FALSE to continue as normal
+   **/
+  viewable_cell_signals[PRE_CLICKED] =
+    g_signal_new ("pre-clicked",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GimpCellRendererViewableClass, pre_clicked),
+                  g_signal_accumulator_true_handled, NULL,
+                  gimp_marshal_BOOLEAN__STRING_FLAGS,
+                  G_TYPE_BOOLEAN, 2,
+                  G_TYPE_STRING,
+                  GDK_TYPE_MODIFIER_TYPE);
+
+  /**
+   * GimpCellRendererViewable::clicked:
+   * @cell:
+   * @path:
+   * @state:
+   *
+   * Called late on a viewable cell when it is clicked, typically
+   * after selection code has been invoked for example.
+   **/
   viewable_cell_signals[CLICKED] =
     g_signal_new ("clicked",
                   G_OBJECT_CLASS_TYPE (object_class),
@@ -198,11 +232,16 @@ gimp_cell_renderer_viewable_get_size (GtkCellRenderer *cell,
                                       gint            *height)
 {
   GimpCellRendererViewable *cellviewable;
+  gfloat                    xalign, yalign;
+  gint                      xpad, ypad;
   gint                      view_width  = 0;
   gint                      view_height = 0;
   gint                      calc_width;
   gint                      calc_height;
 
+  gtk_cell_renderer_get_alignment (cell, &xalign, &yalign);
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+
   cellviewable = GIMP_CELL_RENDERER_VIEWABLE (cell);
 
   if (cellviewable->renderer)
@@ -213,8 +252,8 @@ gimp_cell_renderer_viewable_get_size (GtkCellRenderer *cell,
                      2 * cellviewable->renderer->border_width);
     }
 
-  calc_width  = (gint) cell->xpad * 2 + view_width;
-  calc_height = (gint) cell->ypad * 2 + view_height;
+  calc_width  = (gint) xpad * 2 + view_width;
+  calc_height = (gint) ypad * 2 + view_height;
 
   if (x_offset) *x_offset = 0;
   if (y_offset) *y_offset = 0;
@@ -224,15 +263,14 @@ gimp_cell_renderer_viewable_get_size (GtkCellRenderer *cell,
       if (x_offset)
         {
           *x_offset = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-                        1.0 - cell->xalign : cell->xalign) *
-                       (cell_area->width - calc_width - 2 * cell->xpad));
-          *x_offset = (MAX (*x_offset, 0) + cell->xpad);
+                        1.0 - xalign : xalign) *
+                       (cell_area->width - calc_width - 2 * xpad));
+          *x_offset = (MAX (*x_offset, 0) + xpad);
         }
       if (y_offset)
         {
-          *y_offset = (cell->yalign *
-                       (cell_area->height - calc_height - 2 * cell->ypad));
-          *y_offset = (MAX (*y_offset, 0) + cell->ypad);
+          *y_offset = (yalign * (cell_area->height - calc_height - 2 * ypad));
+          *y_offset = (MAX (*y_offset, 0) + ypad);
         }
     }
 
@@ -311,6 +349,25 @@ gimp_cell_renderer_viewable_new (void)
   return g_object_new (GIMP_TYPE_CELL_RENDERER_VIEWABLE, NULL);
 }
 
+gboolean
+gimp_cell_renderer_viewable_pre_clicked (GimpCellRendererViewable *cell,
+                                         const gchar              *path,
+                                         GdkModifierType           state)
+{
+  gboolean handled = FALSE;
+
+  g_return_val_if_fail (GIMP_IS_CELL_RENDERER_VIEWABLE (cell), FALSE);
+  g_return_val_if_fail (path != NULL, FALSE);
+
+  g_signal_emit (cell,
+                 viewable_cell_signals[PRE_CLICKED],
+                 0 /*detail*/,
+                 path, state,
+                 &handled);
+
+  return handled;
+}
+
 void
 gimp_cell_renderer_viewable_clicked (GimpCellRendererViewable *cell,
                                      const gchar              *path,
