@@ -324,10 +324,9 @@ gimp_colormap_editor_set_image (GimpImageEditor *image_editor,
 
       if (! HAVE_COLORMAP (image))
         {
-          editor->index_adjustment->upper = 0;
-          gtk_adjustment_changed (editor->index_adjustment);
+          gtk_adjustment_set_upper (editor->index_adjustment, 0);
 
-          if (GTK_WIDGET_MAPPED (GTK_WIDGET (editor)))
+          if (gtk_widget_get_mapped (GTK_WIDGET (editor)))
             gimp_colormap_editor_clear (editor, -1);
         }
     }
@@ -350,10 +349,8 @@ gimp_colormap_editor_set_image (GimpImageEditor *image_editor,
         {
           gimp_colormap_editor_draw (editor);
 
-          editor->index_adjustment->upper =
-            gimp_image_get_colormap_size (image) - 1;
-
-          gtk_adjustment_changed (editor->index_adjustment);
+          gtk_adjustment_set_upper (editor->index_adjustment,
+                                    gimp_image_get_colormap_size (image) - 1);
         }
     }
 
@@ -509,13 +506,13 @@ gimp_colormap_editor_create_layout (GtkWidget *widget)
 static void
 gimp_colormap_editor_draw (GimpColormapEditor *editor)
 {
-  GimpImage *image  = GIMP_IMAGE_EDITOR (editor)->image;
-  gint       i, j, k, b;
-  gint       col;
-  guchar    *row;
-  gint       cellsize, ncol, xn, yn;
-  gint       width  = editor->preview->allocation.width;
-  gint       height = editor->preview->allocation.height;
+  GimpImage     *image    = GIMP_IMAGE_EDITOR (editor)->image;
+  const guchar  *colormap = gimp_image_get_colormap (image);
+  GtkAllocation  allocation;
+  gint           i, j, k, b;
+  gint           col;
+  guchar        *row;
+  gint           cellsize, ncol, xn, yn;
 
   ncol = gimp_image_get_colormap_size (image);
 
@@ -525,9 +522,11 @@ gimp_colormap_editor_draw (GimpColormapEditor *editor)
       return;
     }
 
-  cellsize = sqrt (width * height / ncol);
+  gtk_widget_get_allocation (editor->preview, &allocation);
+
+  cellsize = sqrt (allocation.width * allocation.height / ncol);
   while (cellsize >= MIN_CELL_SIZE
-         && (xn = width / cellsize) * (yn = height / cellsize) < ncol)
+         && (xn = allocation.width / cellsize) * (yn = allocation.height / cellsize) < ncol)
     cellsize--;
 
   if (cellsize < MIN_CELL_SIZE)
@@ -549,7 +548,7 @@ gimp_colormap_editor_draw (GimpColormapEditor *editor)
   editor->yn       = yn;
   editor->cellsize = cellsize;
 
-  row = g_new (guchar, MAX (width, xn * cellsize) * 3);
+  row = g_new (guchar, MAX (allocation.width, xn * cellsize) * 3);
   col = 0;
   for (i = 0; i < yn; i++)
     {
@@ -557,17 +556,17 @@ gimp_colormap_editor_draw (GimpColormapEditor *editor)
         {
           for (k = 0; k < cellsize; k++)
             for (b = 0; b < 3; b++)
-              row[(j * cellsize + k) * 3 + b] = image->colormap[col * 3 + b];
+              row[(j * cellsize + k) * 3 + b] = colormap[col * 3 + b];
         }
 
-      if (j * cellsize < width)
-        memset (row + j * cellsize * 3, 255, 3 * (width - j * cellsize));
+      if (j * cellsize < allocation.width)
+        memset (row + j * cellsize * 3, 255, 3 * (allocation.width - j * cellsize));
 
       for (k = 0; k < cellsize; k++)
         {
           gtk_preview_draw_row (GTK_PREVIEW (editor->preview), row, 0,
                                 i * cellsize + k,
-                                width);
+                                allocation.width);
         }
     }
 
@@ -583,10 +582,11 @@ static void
 gimp_colormap_editor_draw_cell (GimpColormapEditor *editor,
                                 gint                col)
 {
-  GimpImage *image    = GIMP_IMAGE_EDITOR (editor)->image;
-  gint       cellsize = editor->cellsize;
-  guchar    *row      = g_alloca (cellsize * 3);
-  gint       x, y, k;
+  GimpImage    *image    = GIMP_IMAGE_EDITOR (editor)->image;
+  const guchar *colormap = gimp_image_get_colormap (image);
+  gint          cellsize = editor->cellsize;
+  guchar       *row      = g_alloca (cellsize * 3);
+  gint          x, y, k;
 
   if (! editor->xn)
     return;
@@ -613,9 +613,9 @@ gimp_colormap_editor_draw_cell (GimpColormapEditor *editor,
 
       for (k = 1; k < cellsize - 1; k++)
         {
-          row[k*3]   = image->colormap[col * 3];
-          row[k*3+1] = image->colormap[col * 3 + 1];
-          row[k*3+2] = image->colormap[col * 3 + 2];
+          row[k*3]   = colormap[col * 3];
+          row[k*3+1] = colormap[col * 3 + 1];
+          row[k*3+2] = colormap[col * 3 + 2];
         }
       for (k = 1; k < cellsize - 1; k+=2)
         gtk_preview_draw_row (GTK_PREVIEW (editor->preview), row,
@@ -632,9 +632,9 @@ gimp_colormap_editor_draw_cell (GimpColormapEditor *editor,
     {
       for (k = 0; k < cellsize; k++)
         {
-          row[k*3]   = image->colormap[col * 3];
-          row[k*3+1] = image->colormap[col * 3 + 1];
-          row[k*3+2] = image->colormap[col * 3 + 2];
+          row[k*3]   = colormap[col * 3];
+          row[k*3+1] = colormap[col * 3 + 1];
+          row[k*3+2] = colormap[col * 3 + 2];
         }
 
       for (k = 0; k < cellsize; k++)
@@ -653,6 +653,7 @@ gimp_colormap_preview_expose (GtkWidget          *widget,
                               GimpColormapEditor *editor)
 {
   GimpImageEditor *image_editor = GIMP_IMAGE_EDITOR (editor);
+  GtkAllocation    allocation;
   gint             x, y;
   gint             width, height;
 
@@ -660,13 +661,15 @@ gimp_colormap_preview_expose (GtkWidget          *widget,
       gimp_image_base_type (image_editor->image) == GIMP_INDEXED)
     return;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   pango_layout_get_pixel_size (editor->layout, &width, &height);
 
-  x = (widget->allocation.width - width) / 2;
-  y = (widget->allocation.height - height) / 2;
+  x = (allocation.width - width) / 2;
+  y = (allocation.height - height) / 2;
 
-  gdk_draw_layout (editor->preview->window,
-                   editor->preview->style->fg_gc[widget->state],
+  gdk_draw_layout (gtk_widget_get_window (editor->preview),
+                   gtk_widget_get_style (editor->preview)->fg_gc[gtk_widget_get_state (widget)],
                    MAX (x, 0), MAX (y, 0),
                    editor->layout);
 }
@@ -675,23 +678,25 @@ static void
 gimp_colormap_editor_clear (GimpColormapEditor *editor,
                             gint                start_row)
 {
-  guchar *row;
-  gint    width  = editor->preview->allocation.width;
-  gint    height = editor->preview->allocation.height;
-  gint    y;
+  GtkAllocation  allocation;
+  guchar        *row;
+  gint           y;
+
+  gtk_widget_get_allocation (editor->preview, &allocation);
 
   if (start_row < 0)
     start_row = 0;
 
-  if (start_row >= height)
+  if (start_row >= allocation.height)
     return;
 
-  row = g_alloca (width * 3);
+  row = g_alloca (allocation.width * 3);
 
-  memset (row, 255, 3 * width);
+  memset (row, 255, 3 * allocation.width);
 
-  for (y = start_row; y < height; y++)
-    gtk_preview_draw_row (GTK_PREVIEW (editor->preview), row, 0, y, width);
+  for (y = start_row; y < allocation.height; y++)
+    gtk_preview_draw_row (GTK_PREVIEW (editor->preview), row,
+                          0, y, allocation.width);
 
   gtk_widget_queue_draw (editor->preview);
 }
@@ -712,12 +717,13 @@ gimp_colormap_editor_update_entries (GimpColormapEditor *editor)
     }
   else
     {
-      gchar  *string;
-      guchar *col;
+      const guchar *colormap = gimp_image_get_colormap (image);
+      const guchar *col;
+      gchar        *string;
 
       gtk_adjustment_set_value (editor->index_adjustment, editor->col_index);
 
-      col = image->colormap + editor->col_index * 3;
+      col = colormap + editor->col_index * 3;
 
       string = g_strdup_printf ("%02x%02x%02x", col[0], col[1], col[2]);
       gtk_entry_set_text (GTK_ENTRY (editor->color_entry), string);
@@ -878,14 +884,8 @@ gimp_colormap_image_colormap_changed (GimpImage          *image,
           gimp_colormap_editor_draw_cell (editor, ncol);
         }
 
-      if (editor->index_adjustment->upper !=
-          (gimp_image_get_colormap_size (image) - 1))
-        {
-          editor->index_adjustment->upper =
-            gimp_image_get_colormap_size (image) - 1;
-
-          gtk_adjustment_changed (editor->index_adjustment);
-        }
+      gtk_adjustment_set_upper (editor->index_adjustment,
+                                gimp_image_get_colormap_size (image) - 1);
     }
   else
     {
