@@ -2,7 +2,7 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpcontainertreeview.c
- * Copyright (C) 2003-2004 Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2003-2010 Michael Natterer <mitch@gimp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "core/gimpviewable.h"
 
 #include "gimpcellrendererviewable.h"
+#include "gimpcontainertreestore.h"
 #include "gimpcontainertreeview.h"
 #include "gimpcontainertreeview-dnd.h"
 #include "gimpcontainertreeview.h"
@@ -57,6 +58,9 @@ static void          gimp_container_tree_view_set_container     (GimpContainerVi
                                                                  GimpContainer               *container);
 static void          gimp_container_tree_view_set_context       (GimpContainerView           *view,
                                                                  GimpContext                 *context);
+static void          gimp_container_tree_view_set_selection_mode(GimpContainerView           *view,
+                                                                 GtkSelectionMode             mode);
+
 static gpointer      gimp_container_tree_view_insert_item       (GimpContainerView           *view,
                                                                  GimpViewable                *viewable,
                                                                  gpointer                     parent_insert_data,
@@ -91,14 +95,17 @@ static gboolean      gimp_container_tree_view_tooltip           (GtkWidget
                                                                  gboolean                     keyboard_tip,
                                                                  GtkTooltip                  *tooltip,
                                                                  GimpContainerTreeView       *tree_view);
-static void          gimp_container_tree_view_renderer_update   (GimpViewRenderer            *renderer,
-                                                                 GimpContainerTreeView       *tree_view);
 static GimpViewable *gimp_container_tree_view_drag_viewable     (GtkWidget                   *widget,
                                                                  GimpContext                **context,
                                                                  gpointer                     data);
 static GdkPixbuf    *gimp_container_tree_view_drag_pixbuf       (GtkWidget                   *widget,
                                                                  gpointer                     data);
 
+static gboolean      gimp_container_tree_view_get_selected_single (GimpContainerTreeView  *tree_view,
+                                                                   GtkTreeIter            *iter);
+static gint          gimp_container_tree_view_get_selected        (GimpContainerView    *view,
+                                                                   GList               **items);
+
 
 G_DEFINE_TYPE_WITH_CODE (GimpContainerTreeView, gimp_container_tree_view,
                          GIMP_TYPE_CONTAINER_BOX,
@@ -138,15 +145,20 @@ gimp_container_tree_view_view_iface_init (GimpContainerViewInterface *iface)
 {
   parent_view_iface = g_type_interface_peek_parent (iface);
 
-  iface->set_container = gimp_container_tree_view_set_container;
-  iface->set_context   = gimp_container_tree_view_set_context;
-  iface->insert_item   = gimp_container_tree_view_insert_item;
-  iface->remove_item   = gimp_container_tree_view_remove_item;
-  iface->reorder_item  = gimp_container_tree_view_reorder_item;
-  iface->rename_item   = gimp_container_tree_view_rename_item;
-  iface->select_item   = gimp_container_tree_view_select_item;
-  iface->clear_items   = gimp_container_tree_view_clear_items;
-  iface->set_view_size = gimp_container_tree_view_set_view_size;
+  if (! parent_view_iface)
+    parent_view_iface = g_type_default_interface_peek (GIMP_TYPE_CONTAINER_VIEW);
+
+  iface->set_container      = gimp_container_tree_view_set_container;
+  iface->set_context        = gimp_container_tree_view_set_context;
+  iface->set_selection_mode = gimp_container_tree_view_set_selection_mode;
+  iface->insert_item        = gimp_container_tree_view_insert_item;
+  iface->remove_item        = gimp_container_tree_view_remove_item;
+  iface->reorder_item       = gimp_container_tree_view_reorder_item;
+  iface->rename_item        = gimp_container_tree_view_rename_item;
+  iface->select_item        = gimp_container_tree_view_select_item;
+  iface->clear_items        = gimp_container_tree_view_clear_items;
+  iface->set_view_size      = gimp_container_tree_view_set_view_size;
+  iface->get_selected       = gimp_container_tree_view_get_selected;
 
   iface->insert_data_free = (GDestroyNotify) gtk_tree_iter_free;
 }
@@ -160,11 +172,8 @@ gimp_container_tree_view_init (GimpContainerTreeView *tree_view)
                                                  GIMP_TYPE_CONTAINER_TREE_VIEW,
                                                  GimpContainerTreeViewPriv);
 
-  tree_view->n_model_columns = GIMP_CONTAINER_TREE_VIEW_N_COLUMNS;
-
-  tree_view->model_columns[GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER]        = GIMP_TYPE_VIEW_RENDERER;
-  tree_view->model_columns[GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME]            = G_TYPE_STRING;
-  tree_view->model_columns[GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME_ATTRIBUTES] = PANGO_TYPE_ATTR_LIST;
+  gimp_container_tree_store_columns_init (tree_view->model_columns,
+                                          &tree_view->n_model_columns);
 
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (box->scrolled_win),
                                        GTK_SHADOW_IN);
@@ -180,7 +189,6 @@ gimp_container_tree_view_constructor (GType                  type,
   GimpContainerTreeView *tree_view;
   GimpContainerView     *view;
   GimpContainerBox      *box;
-  GtkTreeStore          *tree;
   GObject               *object;
 
   object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
@@ -189,19 +197,19 @@ gimp_container_tree_view_constructor (GType                  type,
   view      = GIMP_CONTAINER_VIEW (object);
   box       = GIMP_CONTAINER_BOX (object);
 
-  tree = gtk_tree_store_newv (tree_view->n_model_columns,
-                              tree_view->model_columns);
-  tree_view->model = GTK_TREE_MODEL (tree);
+  tree_view->model = gimp_container_tree_store_new (view,
+                                                    tree_view->n_model_columns,
+                                                    tree_view->model_columns);
 
   tree_view->view = g_object_new (GTK_TYPE_TREE_VIEW,
-                                  "model",           tree,
-                                  "search-column",   GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME,
+                                  "model",           tree_view->model,
+                                  "search-column",   GIMP_CONTAINER_TREE_STORE_COLUMN_NAME,
                                   "enable-search",   FALSE,
                                   "headers-visible", FALSE,
                                   "has-tooltip",     TRUE,
                                   "show-expanders",  GIMP_CONTAINER_VIEW_GET_INTERFACE (view)->model_is_tree,
                                   NULL);
-  g_object_unref (tree);
+  g_object_unref (tree_view->model);
 
   gtk_container_add (GTK_CONTAINER (box->scrolled_win),
                      GTK_WIDGET (tree_view->view));
@@ -222,7 +230,7 @@ gimp_container_tree_view_constructor (GType                  type,
 
   gtk_tree_view_column_set_attributes (tree_view->main_column,
                                        tree_view->renderer_cell,
-                                       "renderer", GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER,
+                                       "renderer", GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER,
                                        NULL);
 
   tree_view->priv->name_cell = gtk_cell_renderer_text_new ();
@@ -233,16 +241,17 @@ gimp_container_tree_view_constructor (GType                  type,
 
   gtk_tree_view_column_set_attributes (tree_view->main_column,
                                        tree_view->priv->name_cell,
-                                       "text",       GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME,
-                                       "attributes", GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME_ATTRIBUTES,
+                                       "text",       GIMP_CONTAINER_TREE_STORE_COLUMN_NAME,
+                                       "attributes", GIMP_CONTAINER_TREE_STORE_COLUMN_NAME_ATTRIBUTES,
+                                       "sensitive",  GIMP_CONTAINER_TREE_STORE_COLUMN_NAME_SENSITIVE,
                                        NULL);
 
   g_signal_connect (tree_view->priv->name_cell, "editing-canceled",
                     G_CALLBACK (gimp_container_tree_view_name_canceled),
                     tree_view);
 
-  tree_view->priv->renderer_cells = g_list_prepend (tree_view->priv->renderer_cells,
-                                                    tree_view->renderer_cell);
+  gimp_container_tree_view_add_renderer_cell (tree_view,
+                                              tree_view->renderer_cell);
 
   tree_view->priv->selection = gtk_tree_view_get_selection (tree_view->view);
 
@@ -318,18 +327,20 @@ gimp_container_tree_view_menu_position (GtkMenu  *menu,
 {
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (data);
   GtkWidget             *widget    = GTK_WIDGET (tree_view->view);
+  GtkAllocation          allocation;
   GtkTreeIter            selected_iter;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
 
-  if (GTK_WIDGET_NO_WINDOW (widget))
+  if (! gtk_widget_get_has_window (widget))
     {
-      *x += widget->allocation.x;
-      *y += widget->allocation.y;
+      *x += allocation.x;
+      *y += allocation.y;
     }
 
-  if (gtk_tree_selection_get_selected (tree_view->priv->selection, NULL,
-                                       &selected_iter))
+  if (gimp_container_tree_view_get_selected_single (tree_view, &selected_iter))
     {
       GtkTreePath  *path;
       GdkRectangle  cell_rect;
@@ -341,9 +352,9 @@ gimp_container_tree_view_menu_position (GtkMenu  *menu,
       gtk_tree_path_free (path);
 
       center = cell_rect.y + cell_rect.height / 2;
-      center = CLAMP (center, 0, widget->allocation.height);
+      center = CLAMP (center, 0, allocation.height);
 
-      *x += widget->allocation.width / 2;
+      *x += allocation.width / 2;
       *y += center;
     }
   else
@@ -409,25 +420,28 @@ gimp_container_tree_view_set_main_column_title (GimpContainerTreeView *tree_view
 }
 
 void
-gimp_container_tree_view_prepend_toggle_cell_renderer (GimpContainerTreeView *tree_view,
-                                                       GtkCellRenderer       *cell_renderer)
+gimp_container_tree_view_add_toggle_cell (GimpContainerTreeView *tree_view,
+                                          GtkCellRenderer       *cell)
 {
   g_return_if_fail (GIMP_IS_CONTAINER_TREE_VIEW (tree_view));
-  g_return_if_fail (GTK_IS_CELL_RENDERER (cell_renderer));
+  g_return_if_fail (GIMP_IS_CELL_RENDERER_TOGGLE (cell));
 
   tree_view->priv->toggle_cells = g_list_prepend (tree_view->priv->toggle_cells,
-                                                  cell_renderer);
+                                                  cell);
 }
 
 void
-gimp_container_tree_view_prepend_cell_renderer (GimpContainerTreeView *tree_view,
-                                                GtkCellRenderer       *cell_renderer)
+gimp_container_tree_view_add_renderer_cell (GimpContainerTreeView *tree_view,
+                                            GtkCellRenderer       *cell)
 {
   g_return_if_fail (GIMP_IS_CONTAINER_TREE_VIEW (tree_view));
-  g_return_if_fail (GTK_IS_CELL_RENDERER (cell_renderer));
+  g_return_if_fail (GIMP_IS_CELL_RENDERER_VIEWABLE (cell));
 
   tree_view->priv->renderer_cells = g_list_prepend (tree_view->priv->renderer_cells,
-                                                    cell_renderer);
+                                                    cell);
+
+  gimp_container_tree_store_add_renderer_cell (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                               cell);
 }
 
 void
@@ -461,40 +475,6 @@ gimp_container_tree_view_connect_name_edited (GimpContainerTreeView *tree_view,
                     data);
 }
 
-static void
-gimp_container_tree_view_set (GimpContainerTreeView *tree_view,
-                              GtkTreeIter           *iter,
-                              GimpViewable          *viewable)
-{
-  GimpContainerView *view = GIMP_CONTAINER_VIEW (tree_view);
-  GimpViewRenderer  *renderer;
-  gchar             *name;
-  gint               view_size;
-  gint               border_width;
-
-  view_size = gimp_container_view_get_view_size (view, &border_width);
-
-  renderer = gimp_view_renderer_new (gimp_container_view_get_context (view),
-                                     G_TYPE_FROM_INSTANCE (viewable),
-                                     view_size, border_width,
-                                     FALSE);
-  gimp_view_renderer_set_viewable (renderer, viewable);
-  gimp_view_renderer_remove_idle (renderer);
-
-  g_signal_connect (renderer, "update",
-                    G_CALLBACK (gimp_container_tree_view_renderer_update),
-                    tree_view);
-
-  name = gimp_viewable_get_description (viewable, NULL);
-
-  gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), iter,
-                      GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, renderer,
-                      GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME,     name,
-                      -1);
-
-  g_free (name);
-  g_object_unref (renderer);
-}
 
 /*  GimpContainerView methods  */
 
@@ -565,24 +545,19 @@ gimp_container_tree_view_set_context (GimpContainerView *view,
   parent_view_iface->set_context (view, context);
 
   if (tree_view->model)
-    {
-      GtkTreeIter iter;
-      gboolean    iter_valid;
+    gimp_container_tree_store_set_context (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                           context);
+}
 
-      for (iter_valid = gtk_tree_model_get_iter_first (tree_view->model, &iter);
-           iter_valid;
-           iter_valid = gtk_tree_model_iter_next (tree_view->model, &iter))
-        {
-          GimpViewRenderer *renderer;
+static void
+gimp_container_tree_view_set_selection_mode (GimpContainerView *view,
+                                             GtkSelectionMode   mode)
+{
+  GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
 
-          gtk_tree_model_get (tree_view->model, &iter,
-                              GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
-                              -1);
+  gtk_tree_selection_set_mode (tree_view->priv->selection, mode);
 
-          gimp_view_renderer_set_context (renderer, context);
-          g_object_unref (renderer);
-        }
-    }
+  parent_view_iface->set_selection_mode (view, mode);
 }
 
 static gpointer
@@ -592,27 +567,23 @@ gimp_container_tree_view_insert_item (GimpContainerView *view,
                                       gint               index)
 {
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
-  GtkTreeIter            iter;
+  GtkTreeIter           *iter;
 
-  if (index == -1)
-    gtk_tree_store_append (GTK_TREE_STORE (tree_view->model), &iter,
-                           parent_insert_data);
-  else
-    gtk_tree_store_insert (GTK_TREE_STORE (tree_view->model), &iter,
-                           parent_insert_data, index);
-
-  gimp_container_tree_view_set (tree_view, &iter, viewable);
+  iter = gimp_container_tree_store_insert_item (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                                viewable,
+                                                parent_insert_data,
+                                                index);
 
   if (parent_insert_data)
     {
-      GtkTreePath *path = gtk_tree_model_get_path (tree_view->model, &iter);
+      GtkTreePath *path = gtk_tree_model_get_path (tree_view->model, iter);
 
       gtk_tree_view_expand_to_path (tree_view->view, path);
 
       gtk_tree_path_free (path);
     }
 
-  return gtk_tree_iter_copy (&iter);
+  return iter;
 }
 
 static void
@@ -623,24 +594,12 @@ gimp_container_tree_view_remove_item (GimpContainerView *view,
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
   GtkTreeIter           *iter      = (GtkTreeIter *) insert_data;
 
+  gimp_container_tree_store_remove_item (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                         viewable,
+                                         iter);
+
   if (iter)
-    {
-      gtk_tree_store_remove (GTK_TREE_STORE (tree_view->model), iter);
-
-      gtk_tree_view_columns_autosize (tree_view->view);
-
-      /*  If the store is empty after this remove, clear out renderers
-       *  from all cells so they don't keep refing the viewables
-       *  (see bug #149906).
-       */
-      if (! gtk_tree_model_iter_n_children (tree_view->model, NULL))
-        {
-          GList *list;
-
-          for (list = tree_view->priv->renderer_cells; list; list = list->next)
-            g_object_set (list->data, "renderer", NULL, NULL);
-        }
-    }
+    gtk_tree_view_columns_autosize (tree_view->view);
 }
 
 static void
@@ -651,30 +610,21 @@ gimp_container_tree_view_reorder_item (GimpContainerView *view,
 {
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
   GtkTreeIter           *iter      = (GtkTreeIter *) insert_data;
+  gboolean               selected  = FALSE;
 
   if (iter)
     {
-      GimpViewable  *parent;
-      GimpContainer *container;
-      GtkTreeIter    selected_iter;
-      gboolean       selected;
+      GtkTreeIter selected_iter;
 
-      parent = gimp_viewable_get_parent (viewable);
-
-      if (parent)
-        container = gimp_viewable_get_children (parent);
-      else
-        container = gimp_container_view_get_container (view);
-
-      selected = gtk_tree_selection_get_selected (tree_view->priv->selection,
-                                                  NULL, &selected_iter);
+      selected = gimp_container_tree_view_get_selected_single (tree_view,
+                                                               &selected_iter);
 
       if (selected)
         {
           GimpViewRenderer *renderer;
 
           gtk_tree_model_get (tree_view->model, &selected_iter,
-                              GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                              GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                               -1);
 
           if (renderer->viewable != viewable)
@@ -682,53 +632,15 @@ gimp_container_tree_view_reorder_item (GimpContainerView *view,
 
           g_object_unref (renderer);
         }
-
-      if (new_index == -1 ||
-          new_index == gimp_container_get_n_children (container) - 1)
-        {
-          gtk_tree_store_move_before (GTK_TREE_STORE (tree_view->model),
-                                      iter, NULL);
-        }
-      else if (new_index == 0)
-        {
-          gtk_tree_store_move_after (GTK_TREE_STORE (tree_view->model),
-                                     iter, NULL);
-        }
-      else
-        {
-          GtkTreePath *path;
-          GtkTreeIter  place_iter;
-          gint         depth;
-          gint        *indices;
-          gint         old_index;
-
-          path = gtk_tree_model_get_path (tree_view->model, iter);
-          indices = gtk_tree_path_get_indices (path);
-
-          depth = gtk_tree_path_get_depth (path);
-
-          old_index = indices[depth - 1];
-
-          if (new_index != old_index)
-            {
-              indices[depth - 1] = new_index;
-
-              gtk_tree_model_get_iter (tree_view->model, &place_iter, path);
-
-              if (new_index > old_index)
-                gtk_tree_store_move_after (GTK_TREE_STORE (tree_view->model),
-                                           iter, &place_iter);
-              else
-                gtk_tree_store_move_before (GTK_TREE_STORE (tree_view->model),
-                                            iter, &place_iter);
-            }
-
-          gtk_tree_path_free (path);
-        }
-
-      if (selected)
-        gimp_container_view_select_item (view, viewable);
     }
+
+  gimp_container_tree_store_reorder_item (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                          viewable,
+                                          new_index,
+                                          iter);
+
+  if (selected)
+    gimp_container_view_select_item (view, viewable);
 }
 
 static void
@@ -739,24 +651,11 @@ gimp_container_tree_view_rename_item (GimpContainerView *view,
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
   GtkTreeIter           *iter      = (GtkTreeIter *) insert_data;
 
-  if (iter)
+  if (gimp_container_tree_store_rename_item (GIMP_CONTAINER_TREE_STORE (tree_view->model),
+                                             viewable,
+                                             iter))
     {
-      gchar *name = gimp_viewable_get_description (viewable, NULL);
-      gchar *old_name;
-
-      gtk_tree_model_get (tree_view->model, iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME, &old_name,
-                          -1);
-
-      gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME, name,
-                          -1);
-
-      if (name && old_name && strlen (name) < strlen (old_name))
-        gtk_tree_view_columns_autosize (tree_view->view);
-
-      g_free (name);
-      g_free (old_name);
+      gtk_tree_view_columns_autosize (tree_view->view);
     }
 }
 
@@ -766,14 +665,22 @@ gimp_container_tree_view_select_item (GimpContainerView *view,
                                       gpointer           insert_data)
 {
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
-  GtkTreeIter           *iter      = (GtkTreeIter *) insert_data;
 
-  if (iter)
+  if (viewable && insert_data)
     {
       GtkTreePath *path;
+      GtkTreePath *parent_path;
+      GtkTreeIter *iter = (GtkTreeIter *) insert_data;
 
       path = gtk_tree_model_get_path (tree_view->model, iter);
 
+      parent_path = gtk_tree_path_copy (path);
+
+      if (gtk_tree_path_up (parent_path))
+        gtk_tree_view_expand_to_path (tree_view->view, parent_path);
+
+      gtk_tree_path_free (parent_path);
+
       g_signal_handlers_block_by_func (tree_view->priv->selection,
                                        gimp_container_tree_view_selection_changed,
                                        tree_view);
@@ -789,8 +696,10 @@ gimp_container_tree_view_select_item (GimpContainerView *view,
 
       gtk_tree_path_free (path);
     }
-  else
+  else if (insert_data == NULL)
     {
+      /* viewable == NULL && insert_data != NULL means multiple selection.
+       * viewable == NULL && insert_data == NULL means no selection. */
       gtk_tree_selection_unselect_all (tree_view->priv->selection);
     }
 
@@ -802,18 +711,7 @@ gimp_container_tree_view_clear_items (GimpContainerView *view)
 {
   GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
 
-  gtk_tree_store_clear (GTK_TREE_STORE (tree_view->model));
-
-  /*  Clear out renderers from all cells so they don't keep refing the
-   *  viewables (see bug #149906).
-   */
-  if (! gtk_tree_model_iter_n_children (tree_view->model, NULL))
-    {
-      GList *list;
-
-      for (list = tree_view->priv->renderer_cells; list; list = list->next)
-        g_object_set (list->data, "renderer", NULL, NULL);
-    }
+  gimp_container_tree_store_clear_items (GIMP_CONTAINER_TREE_STORE (tree_view->model));
 
   parent_view_iface->clear_items (view);
 }
@@ -830,24 +728,7 @@ gimp_container_tree_view_set_view_size (GimpContainerView *view)
   view_size = gimp_container_view_get_view_size (view, &border_width);
 
   if (tree_view->model)
-    {
-      GtkTreeIter iter;
-      gboolean    iter_valid;
-
-      for (iter_valid = gtk_tree_model_get_iter_first (tree_view->model, &iter);
-           iter_valid;
-           iter_valid = gtk_tree_model_iter_next (tree_view->model, &iter))
-        {
-          GimpViewRenderer *renderer;
-
-          gtk_tree_model_get (tree_view->model, &iter,
-                              GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
-                              -1);
-
-          gimp_view_renderer_set_size (renderer, view_size, border_width);
-          g_object_unref (renderer);
-        }
-    }
+    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (tree_view->model));
 
   tree_widget = GTK_WIDGET (tree_view->view);
 
@@ -891,19 +772,19 @@ gimp_container_tree_view_name_canceled (GtkCellRendererText   *cell,
 {
   GtkTreeIter iter;
 
-  if (gtk_tree_selection_get_selected (tree_view->priv->selection, NULL, &iter))
+  if (gimp_container_tree_view_get_selected_single (tree_view, &iter))
     {
       GimpViewRenderer *renderer;
       gchar            *name;
 
       gtk_tree_model_get (tree_view->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
 
       name = gimp_viewable_get_description (renderer->viewable, NULL);
 
       gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME, name,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_NAME, name,
                           -1);
 
       g_free (name);
@@ -915,20 +796,12 @@ static void
 gimp_container_tree_view_selection_changed (GtkTreeSelection      *selection,
                                             GimpContainerTreeView *tree_view)
 {
-  GtkTreeIter iter;
+  GimpContainerView    *view = GIMP_CONTAINER_VIEW (tree_view);
+  GList                *items;
 
-  if (gtk_tree_selection_get_selected (selection, NULL, &iter))
-    {
-      GimpViewRenderer *renderer;
-
-      gtk_tree_model_get (tree_view->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
-                          -1);
-
-      gimp_container_view_item_selected (GIMP_CONTAINER_VIEW (tree_view),
-                                         renderer->viewable);
-      g_object_unref (renderer);
-    }
+  gimp_container_tree_view_get_selected (view, &items);
+  gimp_container_view_multi_selected (view, items);
+  g_list_free (items);
 }
 
 static GtkCellRenderer *
@@ -948,19 +821,22 @@ gimp_container_tree_view_find_click_cell (GtkWidget         *widget,
       gint             start;
       gint             width;
 
-      if (renderer->visible &&
+      if (gtk_cell_renderer_get_visible (renderer) &&
           gtk_tree_view_column_cell_get_position (column, renderer,
                                                   &start, &width))
         {
+          gint xpad, ypad;
           gint x;
 
+          gtk_cell_renderer_get_padding (renderer, &xpad, &ypad);
+
           if (rtl)
             x = column_area->x + column_area->width - start - width;
           else
             x = start + column_area->x;
 
-          if (tree_x >= x + renderer->xpad &&
-              tree_x < x + width - renderer->xpad)
+          if (tree_x >= x + xpad &&
+              tree_x <  x + width - xpad)
             {
               return renderer;
             }
@@ -981,7 +857,7 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
 
   tree_view->priv->dnd_renderer = NULL;
 
-  if (! GTK_WIDGET_HAS_FOCUS (widget))
+  if (! gtk_widget_has_focus (widget))
     gtk_widget_grab_focus (widget);
 
   if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
@@ -994,11 +870,16 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
       GtkCellRenderer          *edit_cell    = NULL;
       GdkRectangle              column_area;
       GtkTreeIter               iter;
+      gboolean                  handled = TRUE;
+      gboolean                  multisel_mode;
+
+      multisel_mode = (gtk_tree_selection_get_mode (tree_view->priv->selection)
+                       == GTK_SELECTION_MULTIPLE);
 
       gtk_tree_model_get_iter (tree_view->model, &iter, path);
 
       gtk_tree_model_get (tree_view->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
 
       tree_view->priv->dnd_renderer = renderer;
@@ -1011,6 +892,34 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
                                                &iter,
                                                FALSE, FALSE);
 
+      if (bevent->button == 1                                     &&
+          gtk_tree_model_iter_has_child (tree_view->model, &iter) &&
+          column == gtk_tree_view_get_expander_column (tree_view->view))
+        {
+          GList *cells;
+
+          cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
+
+          if (! gimp_container_tree_view_find_click_cell (widget,
+                                                          cells,
+                                                          column, &column_area,
+                                                          bevent->x, bevent->y))
+            {
+              /*  we didn't click on any cell, but we clicked on empty
+               *  space in the expander column of a row that has
+               *  children; let GtkTreeView process the button press
+               *  to maybe handle a click on an expander.
+               */
+              g_list_free (cells);
+              gtk_tree_path_free (path);
+              g_object_unref (renderer);
+
+              return FALSE;
+            }
+
+          g_list_free (cells);
+        }
+
       toggled_cell = (GimpCellRendererToggle *)
         gimp_container_tree_view_find_click_cell (widget,
                                                   tree_view->priv->toggle_cells,
@@ -1038,12 +947,35 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
         case 1:
           if (bevent->type == GDK_BUTTON_PRESS)
             {
-              gboolean success = TRUE;
-
               /*  don't select item if a toggle was clicked */
               if (! toggled_cell)
-                success = gimp_container_view_item_selected (container_view,
-                                                             renderer->viewable);
+                {
+                  gchar *path_str = gtk_tree_path_to_string (path);
+
+                  handled = FALSE;
+
+                  if (clicked_cell)
+                    handled =
+                      gimp_cell_renderer_viewable_pre_clicked (clicked_cell,
+                                                               path_str,
+                                                               bevent->state);
+
+                  if (! handled)
+                    {
+                      if (multisel_mode)
+                        {
+                          /* let parent do the work */
+                        }
+                      else
+                        {
+                          handled =
+                            gimp_container_view_item_selected (container_view,
+                                                               renderer->viewable);
+                        }
+                    }
+
+                  g_free (path_str);
+                }
 
               /*  a callback invoked by selecting the item may have
                *  destroyed us, so check if the container is still there
@@ -1094,12 +1026,11 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
                         {
                           const gchar *real_name;
 
-                          real_name =
-                            gimp_object_get_name (GIMP_OBJECT (renderer->viewable));
+                          real_name = gimp_object_get_name (renderer->viewable);
 
                           gtk_tree_store_set (GTK_TREE_STORE (tree_view->model),
                                               &iter,
-                                              GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME,
+                                              GIMP_CONTAINER_TREE_STORE_COLUMN_NAME,
                                               real_name,
                                               -1);
                         }
@@ -1156,6 +1087,8 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
 
       gtk_tree_path_free (path);
       g_object_unref (renderer);
+
+      return multisel_mode ? handled : TRUE;
     }
   else
     {
@@ -1168,9 +1101,9 @@ gimp_container_tree_view_button_press (GtkWidget             *widget,
         default:
           break;
         }
-    }
 
-  return TRUE;
+      return TRUE;
+    }
 }
 
 static gboolean
@@ -1192,7 +1125,7 @@ gimp_container_tree_view_tooltip (GtkWidget             *widget,
     return FALSE;
 
   gtk_tree_model_get (tree_view->model, &iter,
-                      GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                      GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                       -1);
 
   if (renderer)
@@ -1221,25 +1154,6 @@ gimp_container_tree_view_tooltip (GtkWidget             *widget,
   return show_tip;
 }
 
-static void
-gimp_container_tree_view_renderer_update (GimpViewRenderer      *renderer,
-                                          GimpContainerTreeView *tree_view)
-{
-  GimpContainerView *view = GIMP_CONTAINER_VIEW (tree_view);
-  GtkTreeIter       *iter;
-
-  iter = gimp_container_view_lookup (view, renderer->viewable);
-
-  if (iter)
-    {
-      GtkTreePath *path;
-
-      path = gtk_tree_model_get_path (tree_view->model, iter);
-      gtk_tree_model_row_changed (tree_view->model, path, iter);
-      gtk_tree_path_free (path);
-    }
-}
-
 static GimpViewable *
 gimp_container_tree_view_drag_viewable (GtkWidget    *widget,
                                         GimpContext **context,
@@ -1272,3 +1186,80 @@ gimp_container_tree_view_drag_pixbuf (GtkWidget *widget,
 
   return NULL;
 }
+
+static gboolean
+gimp_container_tree_view_get_selected_single (GimpContainerTreeView  *tree_view,
+                                               GtkTreeIter            *iter)
+{
+  GtkTreeSelection         *selection;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view->view));
+
+  if (gtk_tree_selection_count_selected_rows (selection) == 1)
+    {
+      GList    *selected_rows;
+
+      selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_view->model), iter,
+                               (GtkTreePath *) selected_rows->data);
+
+      g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
+      g_list_free (selected_rows);
+
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
+static gint
+gimp_container_tree_view_get_selected (GimpContainerView    *view,
+                                       GList               **items)
+{
+  GimpContainerTreeView *tree_view = GIMP_CONTAINER_TREE_VIEW (view);
+  GtkTreeSelection      *selection;
+  gint                   selected_count;
+  GList                 *selected_rows;
+  GList                 *current_row;
+  GtkTreeIter            iter;
+  GimpViewRenderer      *renderer;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view->view));
+  selected_count = gtk_tree_selection_count_selected_rows (selection);
+
+  if (items == NULL)
+    {
+      /* just provide selected count */
+      return selected_count;
+    }
+
+  selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+  *items = NULL;
+  for (current_row = selected_rows;
+       current_row;
+       current_row = g_list_next (current_row))
+    {
+      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_view->model), &iter,
+                               (GtkTreePath *) current_row->data);
+
+      gtk_tree_model_get (tree_view->model, &iter,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                          -1);
+
+      if (renderer->viewable)
+        *items = g_list_prepend (*items, renderer->viewable);
+
+      g_object_unref (renderer);
+    }
+
+  g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
+  g_list_free (selected_rows);
+
+  *items = g_list_reverse (*items);
+
+  return selected_count;
+}
