@@ -360,8 +360,7 @@ gimp_controller_editor_constructor (GType                  type,
   gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  editor->grab_button = gtk_check_button_new_with_mnemonic (_("_Grab event"));
-  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->grab_button), FALSE);
+  editor->grab_button = gtk_toggle_button_new_with_mnemonic (_("_Grab event"));
   gtk_box_pack_start (GTK_BOX (hbox), editor->grab_button, TRUE, TRUE, 0);
   gtk_widget_show (editor->grab_button);
 
@@ -555,7 +554,7 @@ gimp_controller_editor_row_activated (GtkTreeView          *tv,
                                       GtkTreeViewColumn    *column,
                                       GimpControllerEditor *editor)
 {
-  if (GTK_WIDGET_IS_SENSITIVE (editor->edit_button))
+  if (gtk_widget_is_sensitive (editor->edit_button))
     gtk_button_clicked (GTK_BUTTON (editor->edit_button));
 }
 
@@ -682,7 +681,7 @@ gimp_controller_editor_edit_clicked (GtkWidget            *button,
       g_object_add_weak_pointer (G_OBJECT (editor->edit_dialog),
                                  (gpointer) &editor->edit_dialog);
 
-      gimp_dialog_factory_add_foreign (gimp_dialog_factory_from_name ("toplevel"),
+      gimp_dialog_factory_add_foreign (gimp_dialog_factory_get_singleton (),
                                        "gimp-controller-action-dialog",
                                        editor->edit_dialog);
 
