@@ -452,7 +452,7 @@ gimp_controller_list_row_activated (GtkTreeView        *tv,
                                     GtkTreeViewColumn  *column,
                                     GimpControllerList *list)
 {
-  if (GTK_WIDGET_IS_SENSITIVE (list->add_button))
+  if (gtk_widget_is_sensitive (list->add_button))
     gtk_button_clicked (GTK_BUTTON (list->add_button));
 }
 
@@ -495,7 +495,7 @@ gimp_controller_list_activate_item (GimpContainerView  *view,
                                     gpointer            insert_data,
                                     GimpControllerList *list)
 {
-  if (GTK_WIDGET_IS_SENSITIVE (list->edit_button))
+  if (gtk_widget_is_sensitive (list->edit_button))
     gtk_button_clicked (GTK_BUTTON (list->edit_button));
 }
 
@@ -565,7 +565,7 @@ gimp_controller_list_remove_clicked (GtkWidget          *button,
                                            RESPONSE_DISABLE,
                                            -1);
 
-  name = gimp_object_get_name (GIMP_OBJECT (list->dest_info));
+  name = gimp_object_get_name (list->dest_info);
   gimp_message_box_set_primary_text (GIMP_MESSAGE_DIALOG (dialog)->box,
                                      _("Remove Controller '%s'?"), name);
 
@@ -634,7 +634,7 @@ gimp_controller_list_edit_clicked (GtkWidget          *button,
 
                             NULL);
 
-  gimp_dialog_factory_add_foreign (gimp_dialog_factory_from_name ("toplevel"),
+  gimp_dialog_factory_add_foreign (gimp_dialog_factory_get_singleton (),
                                    "gimp-controller-editor-dialog",
                                    dialog);
 
