@@ -22,56 +22,79 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libgimpconfig/gimpconfig.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "widgets-types.h"
 
+#include "core/gimp.h"
 #include "core/gimpcurve.h"
 #include "core/gimpcurve-map.h"
 #include "core/gimpmarshal.h"
 
+#include "gimpclipboard.h"
 #include "gimpcurveview.h"
 
 
 enum
 {
   PROP_0,
+  PROP_GIMP,
   PROP_BASE_LINE,
   PROP_GRID_ROWS,
   PROP_GRID_COLUMNS
 };
 
+enum
+{
+  CUT_CLIPBOARD,
+  COPY_CLIPBOARD,
+  PASTE_CLIPBOARD,
+  LAST_SIGNAL
+};
 
-static void       gimp_curve_view_finalize       (GObject          *object);
-static void       gimp_curve_view_dispose        (GObject          *object);
-static void       gimp_curve_view_set_property   (GObject          *object,
-                                                  guint             property_id,
-                                                  const GValue     *value,
-                                                  GParamSpec       *pspec);
-static void       gimp_curve_view_get_property   (GObject          *object,
-                                                  guint             property_id,
-                                                  GValue           *value,
-                                                  GParamSpec       *pspec);
 
-static void       gimp_curve_view_style_set      (GtkWidget        *widget,
-                                                  GtkStyle         *prev_style);
-static gboolean   gimp_curve_view_expose         (GtkWidget        *widget,
-                                                  GdkEventExpose   *event);
-static gboolean   gimp_curve_view_button_press   (GtkWidget        *widget,
-                                                  GdkEventButton   *bevent);
-static gboolean   gimp_curve_view_button_release (GtkWidget        *widget,
-                                                  GdkEventButton   *bevent);
-static gboolean   gimp_curve_view_motion_notify  (GtkWidget        *widget,
-                                                  GdkEventMotion   *bevent);
-static gboolean   gimp_curve_view_leave_notify   (GtkWidget        *widget,
-                                                  GdkEventCrossing *cevent);
-static gboolean   gimp_curve_view_key_press      (GtkWidget        *widget,
-                                                  GdkEventKey      *kevent);
+typedef struct
+{
+  GimpCurve *curve;
+  GimpRGB    color;
+} BGCurve;
 
-static void       gimp_curve_view_set_cursor     (GimpCurveView    *view,
-                                                  gdouble           x,
-                                                  gdouble           y);
-static void       gimp_curve_view_unset_cursor   (GimpCurveView *view);
+
+static void       gimp_curve_view_finalize        (GObject          *object);
+static void       gimp_curve_view_dispose         (GObject          *object);
+static void       gimp_curve_view_set_property    (GObject          *object,
+                                                   guint             property_id,
+                                                   const GValue     *value,
+                                                   GParamSpec       *pspec);
+static void       gimp_curve_view_get_property    (GObject          *object,
+                                                   guint             property_id,
+                                                   GValue           *value,
+                                                   GParamSpec       *pspec);
+
+static void       gimp_curve_view_style_set       (GtkWidget        *widget,
+                                                   GtkStyle         *prev_style);
+static gboolean   gimp_curve_view_expose          (GtkWidget        *widget,
+                                                   GdkEventExpose   *event);
+static gboolean   gimp_curve_view_button_press    (GtkWidget        *widget,
+                                                   GdkEventButton   *bevent);
+static gboolean   gimp_curve_view_button_release  (GtkWidget        *widget,
+                                                   GdkEventButton   *bevent);
+static gboolean   gimp_curve_view_motion_notify   (GtkWidget        *widget,
+                                                   GdkEventMotion   *bevent);
+static gboolean   gimp_curve_view_leave_notify    (GtkWidget        *widget,
+                                                   GdkEventCrossing *cevent);
+static gboolean   gimp_curve_view_key_press       (GtkWidget        *widget,
+                                                   GdkEventKey      *kevent);
+
+static void       gimp_curve_view_cut_clipboard   (GimpCurveView    *view);
+static void       gimp_curve_view_copy_clipboard  (GimpCurveView    *view);
+static void       gimp_curve_view_paste_clipboard (GimpCurveView    *view);
+
+static void       gimp_curve_view_set_cursor      (GimpCurveView    *view,
+                                                   gdouble           x,
+                                                   gdouble           y);
+static void       gimp_curve_view_unset_cursor    (GimpCurveView *view);
 
 
 G_DEFINE_TYPE (GimpCurveView, gimp_curve_view,
@@ -79,12 +102,15 @@ G_DEFINE_TYPE (GimpCurveView, gimp_curve_view,
 
 #define parent_class gimp_curve_view_parent_class
 
+static guint curve_view_signals[LAST_SIGNAL] = { 0 };
+
 
 static void
 gimp_curve_view_class_init (GimpCurveViewClass *klass)
 {
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GtkBindingSet  *binding_set;
 
   object_class->finalize             = gimp_curve_view_finalize;
   object_class->dispose              = gimp_curve_view_dispose;
@@ -99,22 +125,70 @@ gimp_curve_view_class_init (GimpCurveViewClass *klass)
   widget_class->leave_notify_event   = gimp_curve_view_leave_notify;
   widget_class->key_press_event      = gimp_curve_view_key_press;
 
+  klass->cut_clipboard               = gimp_curve_view_cut_clipboard;
+  klass->copy_clipboard              = gimp_curve_view_copy_clipboard;
+  klass->paste_clipboard             = gimp_curve_view_paste_clipboard;
+
+  g_object_class_install_property (object_class, PROP_GIMP,
+                                   g_param_spec_object ("gimp",
+                                                        NULL, NULL,
+                                                        GIMP_TYPE_GIMP,
+                                                        GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_BASE_LINE,
                                    g_param_spec_boolean ("base-line",
                                                          NULL, NULL,
                                                          TRUE,
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT_ONLY));
+
   g_object_class_install_property (object_class, PROP_GRID_ROWS,
                                    g_param_spec_int ("grid-rows", NULL, NULL,
                                                      0, 100, 8,
                                                      GIMP_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
+
   g_object_class_install_property (object_class, PROP_GRID_COLUMNS,
                                    g_param_spec_int ("grid-columns", NULL, NULL,
                                                      0, 100, 8,
                                                      GIMP_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
+
+  curve_view_signals[CUT_CLIPBOARD] =
+    g_signal_new ("cut-clipboard",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (GimpCurveViewClass, cut_clipboard),
+                  NULL, NULL,
+                  gimp_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  curve_view_signals[COPY_CLIPBOARD] =
+    g_signal_new ("copy-clipboard",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (GimpCurveViewClass, copy_clipboard),
+                  NULL, NULL,
+                  gimp_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  curve_view_signals[PASTE_CLIPBOARD] =
+    g_signal_new ("paste-clipboard",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (GimpCurveViewClass, paste_clipboard),
+                  NULL, NULL,
+                  gimp_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  binding_set = gtk_binding_set_by_class (klass);
+
+  gtk_binding_entry_add_signal (binding_set, GDK_x, GDK_CONTROL_MASK,
+                                "cut-clipboard", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_c, GDK_CONTROL_MASK,
+                                "copy-clipboard", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_v, GDK_CONTROL_MASK,
+                                "paste-clipboard", 0);
 }
 
 static void
@@ -128,9 +202,12 @@ gimp_curve_view_init (GimpCurveView *view)
   view->xpos        = -1.0;
   view->cursor_x    = -1.0;
   view->cursor_y    = -1.0;
+  view->range_x_min = 0.0;
+  view->range_x_max = 1.0;
+  view->range_y_min = 0.0;
+  view->range_y_max = 1.0;
 
-  GTK_WIDGET_SET_FLAGS (view, GTK_CAN_FOCUS);
-
+  gtk_widget_set_can_focus (GTK_WIDGET (view), TRUE);
   gtk_widget_add_events (GTK_WIDGET (view),
                          GDK_BUTTON_PRESS_MASK   |
                          GDK_BUTTON_RELEASE_MASK |
@@ -167,6 +244,13 @@ gimp_curve_view_dispose (GObject *object)
 
   gimp_curve_view_set_curve (view, NULL);
 
+  while (view->bg_curves)
+    {
+      BGCurve *bg = view->bg_curves->data;
+
+      gimp_curve_view_remove_background (view, bg->curve);
+    }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -180,6 +264,9 @@ gimp_curve_view_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_GIMP:
+      view->gimp = g_value_get_object (value); /* don't ref */
+      break;
     case PROP_GRID_ROWS:
       view->grid_rows = g_value_get_int (value);
       break;
@@ -205,6 +292,9 @@ gimp_curve_view_get_property (GObject    *object,
 
   switch (property_id)
     {
+    case PROP_GIMP:
+      g_value_set_object (value, view->gimp);
+      break;
     case PROP_GRID_ROWS:
       g_value_set_int (value, view->grid_rows);
       break;
@@ -333,13 +423,47 @@ gimp_curve_view_draw_point (GimpCurveView *view,
              0, 2 * G_PI);
 }
 
+static void
+gimp_curve_view_draw_curve (GimpCurveView *view,
+                            cairo_t       *cr,
+                            GimpCurve     *curve,
+                            gint           width,
+                            gint           height,
+                            gint           border)
+{
+  gdouble x, y;
+  gint    i;
+
+  x = 0.0;
+  y = 1.0 - gimp_curve_map_value (curve, 0.0);
+
+  cairo_move_to (cr,
+                 border + (gdouble) width  * x,
+                 border + (gdouble) height * y);
+
+  for (i = 1; i < 256; i++)
+    {
+      x = (gdouble) i / 255.0;
+      y = 1.0 - gimp_curve_map_value (curve, x);
+
+      cairo_line_to (cr,
+                     border + (gdouble) width  * x,
+                     border + (gdouble) height * y);
+    }
+
+  cairo_stroke (cr);
+}
+
 static gboolean
 gimp_curve_view_expose (GtkWidget      *widget,
                         GdkEventExpose *event)
 {
-  GimpCurveView *view  = GIMP_CURVE_VIEW (widget);
-  GtkStyle      *style = gtk_widget_get_style (widget);
+  GimpCurveView *view   = GIMP_CURVE_VIEW (widget);
+  GdkWindow     *window = gtk_widget_get_window (widget);
+  GtkStyle      *style  = gtk_widget_get_style (widget);
+  GtkAllocation  allocation;
   cairo_t       *cr;
+  GList         *list;
   gint           border;
   gint           width;
   gint           height;
@@ -351,10 +475,37 @@ gimp_curve_view_expose (GtkWidget      *widget,
   if (! view->curve)
     return FALSE;
 
-  border = GIMP_HISTOGRAM_VIEW (view)->border_width;
+  gtk_widget_get_allocation (widget, &allocation);
 
-  width  = widget->allocation.width  - 2 * border - 1;
-  height = widget->allocation.height - 2 * border - 1;
+  border = GIMP_HISTOGRAM_VIEW (view)->border_width;
+  width  = allocation.width  - 2 * border - 1;
+  height = allocation.height - 2 * border - 1;
+
+  if (! gimp_histogram_view_get_histogram (GIMP_HISTOGRAM_VIEW (view)) &&
+      ! gimp_histogram_view_get_background (GIMP_HISTOGRAM_VIEW (view)))
+    {
+
+      gdk_draw_rectangle (window,
+                          style->base_gc[GTK_STATE_NORMAL], TRUE,
+                          0, 0,
+                          allocation.width,
+                          allocation.height);
+
+      /*  Draw the outer border  */
+      gdk_draw_rectangle (window,
+                          style->dark_gc[GTK_STATE_NORMAL], FALSE,
+                          border, border,
+                          width - 1, height - 1);
+    }
+
+  if (gtk_widget_has_focus (widget))
+    {
+      gtk_paint_focus (style, window,
+                       gtk_widget_get_state (widget),
+                       &event->area, widget, NULL,
+                       border - 2, border - 2,
+                       width + 4, height + 4);
+    }
 
   cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
@@ -366,28 +517,28 @@ gimp_curve_view_expose (GtkWidget      *widget,
   /*  Draw the grid lines  */
   gimp_curve_view_draw_grid (view, style, cr, width, height, border);
 
+  /*  Draw the background curves  */
+  for (list = view->bg_curves; list; list = g_list_next (list))
+    {
+      BGCurve *bg = list->data;
+
+      cairo_set_source_rgba (cr,
+                             bg->color.r,
+                             bg->color.g,
+                             bg->color.b,
+                             0.5);
+
+      gimp_curve_view_draw_curve (view, cr, bg->curve,
+                                  width, height, border);
+    }
+
   /*  Draw the curve  */
   gdk_cairo_set_source_color (cr, &style->text[GTK_STATE_NORMAL]);
 
-  x = 0.0;
-  y = 1.0 - gimp_curve_map_value (view->curve, 0.0);
-
-  cairo_move_to (cr,
-                 border + (gdouble) width  * x,
-                 border + (gdouble) height * y);
-
-  for (i = 1; i < 256; i++)
-    {
-      x = (gdouble) i / 255.0;
-      y = 1.0 - gimp_curve_map_value (view->curve, x);
-
-      cairo_line_to (cr,
-                     border + (gdouble) width  * x,
-                     border + (gdouble) height * y);
-    }
-
-  cairo_stroke (cr);
+  gimp_curve_view_draw_curve (view, cr, view->curve,
+                              width, height, border);
 
+  /*  Draw the points  */
   if (gimp_curve_get_curve_type (view->curve) == GIMP_CURVE_SMOOTH)
     {
       gdk_cairo_set_source_color (cr, &style->text[GTK_STATE_NORMAL]);
@@ -458,8 +609,19 @@ gimp_curve_view_expose (GtkWidget      *widget,
 
       if (! view->cursor_layout)
         {
-          view->cursor_layout = gtk_widget_create_pango_layout (widget,
-                                                                "x:888 y:888");
+          /*  stupid heuristic: special-case for 0..255  */
+          if (view->range_x_max == 255.0 &&
+              view->range_y_max == 255.0)
+            {
+              view->cursor_layout =
+                gtk_widget_create_pango_layout (widget, "x:888 y:888");
+            }
+          else
+            {
+              view->cursor_layout =
+                gtk_widget_create_pango_layout (widget, "x:0.888 y:0.888");
+            }
+
           pango_layout_get_pixel_extents (view->cursor_layout,
                                           NULL, &view->cursor_rect);
         }
@@ -479,9 +641,29 @@ gimp_curve_view_expose (GtkWidget      *widget,
       cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
       cairo_stroke (cr);
 
-      g_snprintf (buf, sizeof (buf), "x:%3d y:%3d",
-                  (gint) (view->cursor_x         * 255.999),
-                  (gint) ((1.0 - view->cursor_y) * 255.999));
+      /*  stupid heuristic: special-case for 0..255  */
+      if (view->range_x_max == 255.0 &&
+          view->range_y_max == 255.0)
+        {
+          g_snprintf (buf, sizeof (buf), "x:%3d y:%3d",
+                      (gint) (view->cursor_x *
+                              (view->range_x_max - view->range_x_min) +
+                              view->range_x_min),
+                      (gint) ((1.0 - view->cursor_y) *
+                              (view->range_y_max - view->range_y_min) +
+                              view->range_y_min));
+        }
+      else
+        {
+          g_snprintf (buf, sizeof (buf), "x:%0.3f y:%0.3f",
+                      view->cursor_x *
+                      (view->range_x_max - view->range_x_min) +
+                      view->range_x_min,
+                      (1.0 - view->cursor_y) *
+                      (view->range_y_max - view->range_y_min) +
+                      view->range_y_min);
+        }
+
       pango_layout_set_text (view->cursor_layout, buf, -1);
 
       gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_NORMAL]);
@@ -521,6 +703,7 @@ gimp_curve_view_button_press (GtkWidget      *widget,
 {
   GimpCurveView *view  = GIMP_CURVE_VIEW (widget);
   GimpCurve     *curve = view->curve;
+  GtkAllocation  allocation;
   gint           border;
   gint           width, height;
   gdouble        x;
@@ -531,9 +714,11 @@ gimp_curve_view_button_press (GtkWidget      *widget,
   if (! curve || bevent->button != 1)
     return TRUE;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   border = GIMP_HISTOGRAM_VIEW (view)->border_width;
-  width  = widget->allocation.width  - 2 * border;
-  height = widget->allocation.height - 2 * border;
+  width  = allocation.width  - 2 * border;
+  height = allocation.height - 2 * border;
 
   x = (gdouble) (bevent->x - border) / (gdouble) width;
   y = (gdouble) (bevent->y - border) / (gdouble) height;
@@ -592,7 +777,7 @@ gimp_curve_view_button_press (GtkWidget      *widget,
       break;
     }
 
-  if (! GTK_WIDGET_HAS_FOCUS (widget))
+  if (! gtk_widget_has_focus (widget))
     gtk_widget_grab_focus (widget);
 
   return TRUE;
@@ -620,6 +805,7 @@ gimp_curve_view_motion_notify (GtkWidget      *widget,
 {
   GimpCurveView  *view       = GIMP_CURVE_VIEW (widget);
   GimpCurve      *curve      = view->curve;
+  GtkAllocation   allocation;
   GimpCursorType  new_cursor = GDK_X_CURSOR;
   gint            border;
   gint            width, height;
@@ -631,9 +817,11 @@ gimp_curve_view_motion_notify (GtkWidget      *widget,
   if (! curve)
     return TRUE;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   border = GIMP_HISTOGRAM_VIEW (view)->border_width;
-  width  = widget->allocation.width  - 2 * border;
-  height = widget->allocation.height - 2 * border;
+  width  = allocation.width  - 2 * border;
+  height = allocation.height - 2 * border;
 
   x = (gdouble) (mevent->x - border) / (gdouble) width;
   y = (gdouble) (mevent->y - border) / (gdouble) height;
@@ -760,82 +948,147 @@ static gboolean
 gimp_curve_view_key_press (GtkWidget   *widget,
                            GdkEventKey *kevent)
 {
-  GimpCurveView *view   = GIMP_CURVE_VIEW (widget);
-  GimpCurve     *curve  = view->curve;
-  gint           i      = view->selected;
-  gdouble        x, y;
-  gboolean       retval = FALSE;
+  GimpCurveView *view    = GIMP_CURVE_VIEW (widget);
+  GimpCurve     *curve   = view->curve;
+  gboolean       handled = FALSE;
 
-  if (view->grabbed || ! curve ||
-      gimp_curve_get_curve_type (curve) == GIMP_CURVE_FREE)
-    return FALSE;
-
-  gimp_curve_get_point (curve, i, NULL, &y);
-
-  switch (kevent->keyval)
+  if (! view->grabbed && curve &&
+      gimp_curve_get_curve_type (curve) == GIMP_CURVE_SMOOTH)
     {
-    case GDK_Left:
-      for (i = i - 1; i >= 0 && ! retval; i--)
-        {
-          gimp_curve_get_point (curve, i, &x, NULL);
+      gint    i = view->selected;
+      gdouble x, y;
 
-          if (x >= 0.0)
+      gimp_curve_get_point (curve, i, NULL, &y);
+
+      switch (kevent->keyval)
+        {
+        case GDK_Left:
+          for (i = i - 1; i >= 0 && ! handled; i--)
             {
-              gimp_curve_view_set_selected (view, i);
+              gimp_curve_get_point (curve, i, &x, NULL);
 
-              retval = TRUE;
+              if (x >= 0.0)
+                {
+                  gimp_curve_view_set_selected (view, i);
+
+                  handled = TRUE;
+                }
             }
-        }
-      break;
+          break;
 
-    case GDK_Right:
-      for (i = i + 1; i < curve->n_points && ! retval; i++)
-        {
-          gimp_curve_get_point (curve, i, &x, NULL);
-
-          if (x >= 0.0)
+        case GDK_Right:
+          for (i = i + 1; i < curve->n_points && ! handled; i++)
             {
-              gimp_curve_view_set_selected (view, i);
+              gimp_curve_get_point (curve, i, &x, NULL);
 
-              retval = TRUE;
+              if (x >= 0.0)
+                {
+                  gimp_curve_view_set_selected (view, i);
+
+                  handled = TRUE;
+                }
             }
+          break;
+
+        case GDK_Up:
+          if (y < 1.0)
+            {
+              y = y + (kevent->state & GDK_SHIFT_MASK ?
+                       (16.0 / 255.0) : (1.0 / 255.0));
+
+              gimp_curve_move_point (curve, i, CLAMP (y, 0.0, 1.0));
+
+              handled = TRUE;
+            }
+          break;
+
+        case GDK_Down:
+          if (y > 0)
+            {
+              y = y - (kevent->state & GDK_SHIFT_MASK ?
+                       (16.0 / 255.0) : (1.0 / 255.0));
+
+              gimp_curve_move_point (curve, i, CLAMP (y, 0.0, 1.0));
+
+              handled = TRUE;
+            }
+          break;
+
+        default:
+          break;
         }
-      break;
-
-    case GDK_Up:
-      if (y < 1.0)
-        {
-          y = y + (kevent->state & GDK_SHIFT_MASK ?
-                   (16.0 / 255.0) : (1.0 / 255.0));
-
-          gimp_curve_move_point (curve, i, CLAMP (y, 0.0, 1.0));
-
-          retval = TRUE;
-        }
-      break;
-
-    case GDK_Down:
-      if (y > 0)
-        {
-          y = y - (kevent->state & GDK_SHIFT_MASK ?
-                   (16.0 / 255.0) : (1.0 / 255.0));
-
-          gimp_curve_move_point (curve, i, CLAMP (y, 0.0, 1.0));
-
-          retval = TRUE;
-        }
-      break;
-
-    default:
-      break;
     }
 
-  if (retval)
-    set_cursor (view, GDK_TCROSS);
+  if (handled)
+    {
+      set_cursor (view, GDK_TCROSS);
 
-  return retval;
+      return TRUE;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, kevent);
 }
 
+static void
+gimp_curve_view_cut_clipboard (GimpCurveView *view)
+{
+  g_printerr ("%s\n", G_STRFUNC);
+
+  if (! view->curve || ! view->gimp)
+    {
+      gtk_widget_error_bell (GTK_WIDGET (view));
+      return;
+    }
+
+  gimp_curve_view_copy_clipboard (view);
+
+  gimp_curve_reset (view->curve, FALSE);
+}
+
+static void
+gimp_curve_view_copy_clipboard (GimpCurveView *view)
+{
+  GimpCurve *copy;
+
+  g_printerr ("%s\n", G_STRFUNC);
+
+  if (! view->curve || ! view->gimp)
+    {
+      gtk_widget_error_bell (GTK_WIDGET (view));
+      return;
+    }
+
+  copy = GIMP_CURVE (gimp_data_duplicate (GIMP_DATA (view->curve)));
+  gimp_clipboard_set_curve (view->gimp, copy);
+  g_object_unref (copy);
+}
+
+static void
+gimp_curve_view_paste_clipboard (GimpCurveView *view)
+{
+  GimpCurve *copy;
+
+  g_printerr ("%s\n", G_STRFUNC);
+
+  if (! view->curve || ! view->gimp)
+    {
+      gtk_widget_error_bell (GTK_WIDGET (view));
+      return;
+    }
+
+  copy = gimp_clipboard_get_curve (view->gimp);
+
+  if (copy)
+    {
+      gimp_config_copy (GIMP_CONFIG (copy),
+                        GIMP_CONFIG (view->curve), 0);
+      g_object_unref (copy);
+    }
+}
+
+
+/*  public functions  */
+
 GtkWidget *
 gimp_curve_view_new (void)
 {
@@ -876,6 +1129,8 @@ gimp_curve_view_set_curve (GimpCurveView *view,
                         G_CALLBACK (gimp_curve_view_curve_dirty),
                         view);
     }
+
+  gtk_widget_queue_draw (GTK_WIDGET (view));
 }
 
 GimpCurve *
@@ -886,6 +1141,74 @@ gimp_curve_view_get_curve (GimpCurveView *view)
   return view->curve;
 }
 
+void
+gimp_curve_view_add_background (GimpCurveView *view,
+                                GimpCurve     *curve,
+                                const GimpRGB *color)
+{
+  BGCurve *bg;
+
+  g_return_if_fail (GIMP_IS_CURVE_VIEW (view));
+  g_return_if_fail (GIMP_IS_CURVE (curve));
+  g_return_if_fail (color != NULL);
+
+  bg = g_slice_new0 (BGCurve);
+
+  bg->curve = g_object_ref (curve);
+  bg->color = *color;
+
+  view->bg_curves = g_list_append (view->bg_curves, bg);
+
+  gtk_widget_queue_draw (GTK_WIDGET (view));
+}
+
+void
+gimp_curve_view_remove_background (GimpCurveView *view,
+                                   GimpCurve     *curve)
+{
+  GList *list;
+
+  g_return_if_fail (GIMP_IS_CURVE_VIEW (view));
+  g_return_if_fail (GIMP_IS_CURVE (curve));
+
+  for (list = view->bg_curves; list; list = g_list_next (list))
+    {
+      BGCurve *bg = list->data;
+
+      if (bg->curve == curve)
+        {
+          g_object_unref (bg->curve);
+
+          view->bg_curves = g_list_remove (view->bg_curves, bg);
+
+          g_slice_free (BGCurve, bg);
+
+          gtk_widget_queue_draw (GTK_WIDGET (view));
+
+          break;
+        }
+    }
+}
+
+void
+gimp_curve_view_remove_all_backgrounds (GimpCurveView *view)
+{
+  g_return_if_fail (GIMP_IS_CURVE_VIEW (view));
+
+  while (view->bg_curves)
+    {
+      BGCurve *bg = view->bg_curves->data;
+
+      g_object_unref (bg->curve);
+
+      view->bg_curves = g_list_remove (view->bg_curves, bg);
+
+      g_slice_free (BGCurve, bg);
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (view));
+}
+
 void
 gimp_curve_view_set_selected (GimpCurveView *view,
                               gint           selected)
@@ -897,6 +1220,28 @@ gimp_curve_view_set_selected (GimpCurveView *view,
   gtk_widget_queue_draw (GTK_WIDGET (view));
 }
 
+void
+gimp_curve_view_set_range_x (GimpCurveView *view,
+                             gdouble        min,
+                             gdouble        max)
+{
+  g_return_if_fail (GIMP_IS_CURVE_VIEW (view));
+
+  view->range_x_min = min;
+  view->range_x_max = max;
+}
+
+void
+gimp_curve_view_set_range_y (GimpCurveView *view,
+                             gdouble        min,
+                             gdouble        max)
+{
+  g_return_if_fail (GIMP_IS_CURVE_VIEW (view));
+
+  view->range_y_min = min;
+  view->range_y_max = max;
+}
+
 void
 gimp_curve_view_set_xpos (GimpCurveView *view,
                           gdouble        x)
