@@ -36,8 +36,12 @@ struct _GimpCurveView
 {
   GimpHistogramView  parent_instance;
 
+  Gimp              *gimp; /* only needed for copy & paste */
+
   GimpCurve         *curve;
 
+  GList             *bg_curves;
+
   gboolean           draw_base_line;
   gint               grid_rows;
   gint               grid_columns;
@@ -54,6 +58,11 @@ struct _GimpCurveView
   gdouble            xpos;
   PangoLayout       *xpos_layout;
 
+  gdouble            range_x_min;
+  gdouble            range_x_max;
+  gdouble            range_y_min;
+  gdouble            range_y_max;
+
   gdouble            cursor_x;
   gdouble            cursor_y;
   PangoLayout       *cursor_layout;
@@ -63,21 +72,39 @@ struct _GimpCurveView
 struct _GimpCurveViewClass
 {
   GimpHistogramViewClass  parent_class;
+
+  void (* cut_clipboard)   (GimpCurveView *view);
+  void (* copy_clipboard)  (GimpCurveView *view);
+  void (* paste_clipboard) (GimpCurveView *view);
 };
 
 
-GType       gimp_curve_view_get_type     (void) G_GNUC_CONST;
+GType       gimp_curve_view_get_type          (void) G_GNUC_CONST;
 
-GtkWidget * gimp_curve_view_new          (void);
+GtkWidget * gimp_curve_view_new               (void);
 
-void        gimp_curve_view_set_curve    (GimpCurveView *view,
-                                          GimpCurve     *curve);
-GimpCurve * gimp_curve_view_get_curve    (GimpCurveView *view);
+void        gimp_curve_view_set_curve         (GimpCurveView *view,
+                                               GimpCurve     *curve);
+GimpCurve * gimp_curve_view_get_curve         (GimpCurveView *view);
 
-void        gimp_curve_view_set_selected (GimpCurveView *view,
-                                          gint           selected);
-void        gimp_curve_view_set_xpos     (GimpCurveView *view,
-                                          gdouble        x);
+void        gimp_curve_view_add_background    (GimpCurveView *view,
+                                               GimpCurve     *curve,
+                                               const GimpRGB *color);
+void        gimp_curve_view_remove_background (GimpCurveView *view,
+                                               GimpCurve     *curve);
+
+void   gimp_curve_view_remove_all_backgrounds (GimpCurveView *view);
+
+void        gimp_curve_view_set_selected      (GimpCurveView *view,
+                                               gint           selected);
+void        gimp_curve_view_set_range_x       (GimpCurveView *view,
+                                               gdouble        min,
+                                               gdouble        max);
+void        gimp_curve_view_set_range_y       (GimpCurveView *view,
+                                               gdouble        min,
+                                               gdouble        max);
+void        gimp_curve_view_set_xpos          (GimpCurveView *view,
+                                               gdouble        x);
 
 
 #endif /* __GIMP_CURVE_VIEW_H__ */
