@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -45,8 +45,6 @@
 
 #include "vectors/gimpvectors.h"
 
-/*  #define DEBUG_DND  */
-
 #include "gimpdnd.h"
 #include "gimpdnd-xds.h"
 #include "gimppixbuf.h"
@@ -54,6 +52,7 @@
 #include "gimpview.h"
 #include "gimpviewrendererimage.h"
 
+#include "gimp-log.h"
 #include "gimp-intl.h"
 
 
@@ -61,13 +60,6 @@
 #define DRAG_ICON_OFFSET   -8
 
 
-#ifdef DEBUG_DND
-#define D(stmnt) stmnt
-#else
-#define D(stmnt)
-#endif
-
-
 typedef GtkWidget * (* GimpDndGetIconFunc)  (GtkWidget        *widget,
                                              GCallback         get_data_func,
                                              gpointer          get_data_data);
@@ -664,7 +656,7 @@ gimp_dnd_data_drag_begin (GtkWidget      *widget,
   data_type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget),
                                                   "gimp-dnd-get-data-type"));
 
-  D (g_printerr ("\ngimp_dnd_data_drag_begin (%d)\n", data_type));
+  GIMP_LOG (DND, "data type %d", data_type);
 
   if (! data_type)
     return;
@@ -692,6 +684,10 @@ gimp_dnd_data_drag_begin (GtkWidget      *widget,
       GtkWidget *window;
 
       window = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_DND);
+      gtk_window_set_screen (GTK_WINDOW (window),
+                             gtk_widget_get_screen (widget));
+
       gtk_widget_realize (window);
 
       frame = gtk_frame_new (NULL);
@@ -717,9 +713,13 @@ static void
 gimp_dnd_data_drag_end (GtkWidget      *widget,
                         GdkDragContext *context)
 {
-  GtkWidget *icon_widget;
+  GimpDndType  data_type;
+  GtkWidget   *icon_widget;
 
-  D (g_printerr ("\ngimp_dnd_data_drag_end\n"));
+  data_type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget),
+                                                  "gimp-dnd-get-data-type"));
+
+  GIMP_LOG (DND, "data type %d", data_type);
 
   icon_widget = g_object_get_data (G_OBJECT (widget), "gimp-dnd-data-widget");
 
@@ -749,7 +749,7 @@ gimp_dnd_data_drag_handle (GtkWidget        *widget,
   gpointer     get_data_data = NULL;
   GimpDndType  data_type;
 
-  D (g_printerr ("\ngimp_dnd_data_drag_handle(%d)\n", info));
+  GIMP_LOG (DND, "data type %d", info);
 
   for (data_type = GIMP_DND_TYPE_NONE + 1;
        data_type <= GIMP_DND_TYPE_LAST;
@@ -759,8 +759,7 @@ gimp_dnd_data_drag_handle (GtkWidget        *widget,
 
       if (dnd_data->target_entry.info == info)
         {
-          D (g_printerr ("gimp_dnd_data_drag_handle(%s)\n",
-                         dnd_data->target_entry.target));
+          GIMP_LOG (DND, "target %s", dnd_data->target_entry.target);
 
           if (dnd_data->get_data_func_name)
             get_data_func = g_object_get_data (G_OBJECT (widget),
@@ -796,9 +795,9 @@ gimp_dnd_data_drop_handle (GtkWidget        *widget,
 {
   GimpDndType data_type;
 
-  D (g_printerr ("\ngimp_dnd_data_drop_handle(%d)\n", info));
+  GIMP_LOG (DND, "data type %d", info);
 
-  if (selection_data->length <= 0)
+  if (gtk_selection_data_get_length (selection_data) <= 0)
     {
       gtk_drag_finish (context, FALSE, FALSE, time);
       return;
@@ -815,8 +814,7 @@ gimp_dnd_data_drop_handle (GtkWidget        *widget,
           GCallback set_data_func = NULL;
           gpointer  set_data_data = NULL;
 
-          D (g_printerr ("gimp_dnd_data_drop_handle(%s)\n",
-                         dnd_data->target_entry.target));
+          GIMP_LOG (DND, "target %s", dnd_data->target_entry.target);
 
           if (dnd_data->set_data_func_name)
             set_data_func = g_object_get_data (G_OBJECT (widget),
