@@ -2,7 +2,7 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpdrawabletreeview.c
- * Copyright (C) 2001-2003 Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2001-2009 Michael Natterer <mitch@gimp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpwidgets/gimpwidgets.h"
+
 #include "widgets-types.h"
 
 #include "core/gimp.h"
@@ -55,6 +57,7 @@ static gboolean gimp_drawable_tree_view_drop_possible(GimpContainerTreeView *vie
                                                       GimpDndType          src_type,
                                                       GimpViewable        *src_viewable,
                                                       GimpViewable        *dest_viewable,
+                                                      GtkTreePath         *drop_path,
                                                       GtkTreeViewDropPosition  drop_pos,
                                                       GtkTreeViewDropPosition *return_drop_pos,
                                                       GdkDragAction       *return_drag_action);
@@ -116,6 +119,9 @@ gimp_drawable_tree_view_class_init (GimpDrawableTreeViewClass *klass)
   tree_view_class->drop_color    = gimp_drawable_tree_view_drop_color;
 
   item_view_class->set_image     = gimp_drawable_tree_view_set_image;
+
+  item_view_class->lock_content_stock_id = GIMP_STOCK_TOOL_PAINTBRUSH;
+  item_view_class->lock_content_tooltip  = _("Lock pixels");
 }
 
 static void
@@ -195,6 +201,7 @@ gimp_drawable_tree_view_drop_possible (GimpContainerTreeView   *tree_view,
                                        GimpDndType              src_type,
                                        GimpViewable            *src_viewable,
                                        GimpViewable            *dest_viewable,
+                                       GtkTreePath             *drop_path,
                                        GtkTreeViewDropPosition  drop_pos,
                                        GtkTreeViewDropPosition *return_drop_pos,
                                        GdkDragAction           *return_drag_action)
@@ -203,6 +210,7 @@ gimp_drawable_tree_view_drop_possible (GimpContainerTreeView   *tree_view,
                                                                     src_type,
                                                                     src_viewable,
                                                                     dest_viewable,
+                                                                    drop_path,
                                                                     drop_pos,
                                                                     return_drop_pos,
                                                                     return_drag_action))
@@ -210,7 +218,9 @@ gimp_drawable_tree_view_drop_possible (GimpContainerTreeView   *tree_view,
       if (src_type == GIMP_DND_TYPE_COLOR ||
           src_type == GIMP_DND_TYPE_PATTERN)
         {
-          if (! dest_viewable)
+          if (! dest_viewable ||
+              gimp_item_is_content_locked (GIMP_ITEM (dest_viewable)) ||
+              gimp_viewable_get_children (GIMP_VIEWABLE (dest_viewable)))
             return FALSE;
 
           if (return_drop_pos)
