@@ -392,12 +392,7 @@ gimp_editor_set_show_button_bar (GimpDocked *docked,
       editor->show_button_bar = show;
 
       if (editor->button_box)
-        {
-          if (show)
-            gtk_widget_show (editor->button_box);
-          else
-            gtk_widget_hide (editor->button_box);
-        }
+        gtk_widget_set_visible (editor->button_box, show);
     }
 }
 
@@ -637,14 +632,9 @@ gimp_editor_add_action_button (GimpEditor  *editor,
   button_icon_size = gimp_editor_ensure_button_box (editor, &button_relief);
 
   if (GTK_IS_TOGGLE_ACTION (action))
-    {
-      button = gtk_toggle_button_new ();
-      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
-    }
+    button = gtk_toggle_button_new ();
   else
-    {
-      button = gimp_button_new ();
-    }
+    button = gimp_button_new ();
 
   gtk_button_set_relief (GTK_BUTTON (button), button_relief);
 
@@ -780,11 +770,14 @@ gimp_editor_set_box_style (GimpEditor *editor,
 
           if (GTK_IS_IMAGE (child))
             {
-              gchar *stock_id;
+              GtkIconSize  old_size;
+              gchar       *stock_id;
 
-              gtk_image_get_stock (GTK_IMAGE (child), &stock_id, NULL);
-              gtk_image_set_from_stock (GTK_IMAGE (child),
-                                        stock_id, button_icon_size);
+              gtk_image_get_stock (GTK_IMAGE (child), &stock_id, &old_size);
+
+              if (button_icon_size != old_size)
+                gtk_image_set_from_stock (GTK_IMAGE (child),
+                                          stock_id, button_icon_size);
             }
         }
     }
