@@ -112,7 +112,7 @@ gimp_font_select_run_callback (GimpPdbDialog  *dialog,
                                              dialog->caller_context,
                                              NULL, error,
                                              dialog->callback_name,
-                                             G_TYPE_STRING,   object->name,
+                                             G_TYPE_STRING,   gimp_object_get_name (object),
                                              GIMP_TYPE_INT32, closing,
                                              G_TYPE_NONE);
 }
