@@ -169,7 +169,7 @@ gimp_gradient_select_run_callback (GimpPdbDialog  *dialog,
                                         dialog->caller_context,
                                         NULL, error,
                                         dialog->callback_name,
-                                        G_TYPE_STRING,         object->name,
+                                        G_TYPE_STRING,         gimp_object_get_name (object),
                                         GIMP_TYPE_INT32,       array->length / sizeof (gdouble),
                                         GIMP_TYPE_FLOAT_ARRAY, array,
                                         GIMP_TYPE_INT32,       closing,
