@@ -144,6 +144,7 @@
 #define GIMP_HELP_LAYER_DUPLICATE                 "gimp-layer-duplicate"
 #define GIMP_HELP_LAYER_ANCHOR                    "gimp-layer-anchor"
 #define GIMP_HELP_LAYER_MERGE_DOWN                "gimp-layer-merge-down"
+#define GIMP_HELP_LAYER_MERGE_GROUP               "gimp-layer-merge-group"
 #define GIMP_HELP_LAYER_DELETE                    "gimp-layer-delete"
 #define GIMP_HELP_LAYER_TEXT_DISCARD              "gimp-layer-text-discard"
 #define GIMP_HELP_LAYER_TEXT_TO_PATH              "gimp-layer-text-to-path"
@@ -240,6 +241,7 @@
 #define GIMP_HELP_TOOLS_DIALOG                    "gimp-tools-dialog"
 
 #define GIMP_HELP_TOOL_AIRBRUSH                   "gimp-tool-airbrush"
+#define GIMP_HELP_TOOL_ALIGN                      "gimp-tool-align"
 #define GIMP_HELP_TOOL_BLEND                      "gimp-tool-blend"
 #define GIMP_HELP_TOOL_BRIGHTNESS_CONTRAST        "gimp-tool-brightness-contrast"
 #define GIMP_HELP_TOOL_BUCKET_FILL                "gimp-tool-bucket-fill"
@@ -260,6 +262,7 @@
 #define GIMP_HELP_TOOL_FREE_SELECT                "gimp-tool-free-select"
 #define GIMP_HELP_TOOL_FOREGROUND_SELECT          "gimp-tool-foreground-select"
 #define GIMP_HELP_TOOL_FUZZY_SELECT               "gimp-tool-fuzzy-select"
+#define GIMP_HELP_TOOL_GEGL                       "gimp-tool-gegl"
 #define GIMP_HELP_TOOL_HEAL                       "gimp-tool-heal"
 #define GIMP_HELP_TOOL_HUE_SATURATION             "gimp-tool-hue-saturation"
 #define GIMP_HELP_TOOL_INK                        "gimp-tool-ink"
@@ -306,6 +309,18 @@
 #define GIMP_HELP_BRUSH_EDITOR_DIALOG             "gimp-brush-editor-dialog"
 #define GIMP_HELP_BRUSH_EDITOR_EDIT_ACTIVE        "gimp-brush-editor-edit-active"
 
+#define GIMP_HELP_DYNAMICS_EDITOR_DIALOG          "gimp-dynamics-editor-dialog"
+
+#define GIMP_HELP_TOOL_PRESET_EDITOR_DIALOG       "gimp-tool-preset-editor-dialog"
+
+#define GIMP_HELP_DYNAMICS_DIALOG                 "gimp-dynamics-dialog"
+#define GIMP_HELP_DYNAMICS_EDIT                   "gimp-dynamics-edit"
+#define GIMP_HELP_DYNAMICS_NEW                    "gimp-dynamics-new"
+#define GIMP_HELP_DYNAMICS_DUPLICATE              "gimp-dynamics-duplicate"
+#define GIMP_HELP_DYNAMICS_COPY_LOCATION          "gimp-dynamics-copy-location"
+#define GIMP_HELP_DYNAMICS_DELETE                 "gimp-dynamics-delete"
+#define GIMP_HELP_DYNAMICS_REFRESH                "gimp-dynamics-refresh"
+
 #define GIMP_HELP_PATTERN_DIALOG                  "gimp-pattern-dialog"
 #define GIMP_HELP_PATTERN_EDIT                    "gimp-pattern-edit"
 #define GIMP_HELP_PATTERN_OPEN_AS_IMAGE           "gimp-pattern-open-as-image"
@@ -377,6 +392,14 @@
 #define GIMP_HELP_BUFFER_PASTE_AS_NEW             "gimp-buffer-paste-as-new"
 #define GIMP_HELP_BUFFER_DELETE                   "gimp-buffer-delete"
 
+#define GIMP_HELP_TOOL_PRESET_DIALOG              "gimp-tool-preset-dialog"
+#define GIMP_HELP_TOOL_PRESET_EDIT                "gimp-tool-preset-edit"
+#define GIMP_HELP_TOOL_PRESET_NEW                 "gimp-tool-preset-new"
+#define GIMP_HELP_TOOL_PRESET_DUPLICATE           "gimp-tool-preset-duplicate"
+#define GIMP_HELP_TOOL_PRESET_COPY_LOCATION       "gimp-tool-preset-copy-location"
+#define GIMP_HELP_TOOL_PRESET_DELETE              "gimp-tool-preset-delete"
+#define GIMP_HELP_TOOL_PRESET_REFRESH             "gimp-tool-preset-refresh"
+
 #define GIMP_HELP_DOCUMENT_CLEAR                  "gimp-document-clear"
 #define GIMP_HELP_DOCUMENT_COPY_LOCATION          "gimp-document-copy-location"
 #define GIMP_HELP_DOCUMENT_DIALOG                 "gimp-document-dialog"
@@ -424,6 +447,7 @@
 #define GIMP_HELP_PREFS_SESSION                   "gimp-prefs-session"
 #define GIMP_HELP_PREFS_FOLDERS                   "gimp-prefs-folders"
 #define GIMP_HELP_PREFS_FOLDERS_BRUSHES           "gimp-prefs-folders-brushes"
+#define GIMP_HELP_PREFS_FOLDERS_DYNAMICS          "gimp-prefs-folders-dynamics"
 #define GIMP_HELP_PREFS_FOLDERS_PATTERNS          "gimp-prefs-folders-patterns"
 #define GIMP_HELP_PREFS_FOLDERS_PALETTES          "gimp-prefs-folders-palettes"
 #define GIMP_HELP_PREFS_FOLDERS_GRADIENTS         "gimp-prefs-folders-gradients"
@@ -435,6 +459,7 @@
 #define GIMP_HELP_PREFS_FOLDERS_ENVIRONMENT       "gimp-prefs-folders-environment"
 #define GIMP_HELP_PREFS_FOLDERS_THEMES            "gimp-prefs-folders-themes"
 
+#define GIMP_HELP_INPUT_DEVICES                   "gimp-help-input-devices"
 #define GIMP_HELP_KEYBOARD_SHORTCUTS              "gimp-help-keyboard-shortcuts"
 
 #define GIMP_HELP_INDEXED_PALETTE_DIALOG          "gimp-indexed-palette-dialog"
@@ -482,6 +507,8 @@
 #define GIMP_HELP_UNIT_DIALOG                     "gimp-unit-dialog"
 
 #define GIMP_HELP_WINDOWS_SHOW_DOCK               "gimp-windows-show-dock"
+#define GIMP_HELP_WINDOWS_HIDE_DOCKS              "gimp-windows-hide-docks"
+#define GIMP_HELP_WINDOWS_USE_SINGLE_WINDOW_MODE  "gimp-windows-use-single-window-mode"
 #define GIMP_HELP_WINDOWS_OPEN_RECENT_DOCK        "gimp-windows-open-recent-dock"
 
 #define GIMP_HELP_HELP                            "gimp-help"
