@@ -5,9 +5,9 @@
  * Copyright (C) 1999-2004 Michael Natterer <mitch@gimp.org>
  *                         Henrik Brix Andersen <brix@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -321,8 +320,8 @@ gimp_help_browser (Gimp         *gimp,
 
       if (error)
         {
-          gimp_message (gimp, G_OBJECT (progress), GIMP_MESSAGE_ERROR,
-                        "%s", error->message);
+          gimp_message_literal (gimp, G_OBJECT (progress), GIMP_MESSAGE_ERROR,
+				error->message);
           g_error_free (error);
         }
      }
@@ -381,8 +380,7 @@ gimp_help_browser_error (Gimp         *gimp,
 
   gimp_message_box_set_primary_text (GIMP_MESSAGE_DIALOG (dialog)->box,
                                      "%s", primary);
-  gimp_message_box_set_text (GIMP_MESSAGE_DIALOG (dialog)->box,
-                             "%s", text);
+  gimp_message_box_set_text (GIMP_MESSAGE_DIALOG (dialog)->box, "%s", text);
 
   if (gimp_dialog_run (GIMP_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
@@ -430,8 +428,7 @@ gimp_help_call (Gimp         *gimp,
 
       if (error)
         {
-          gimp_message (gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "%s", error->message);
+          gimp_message_literal (gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
           g_error_free (error);
         }
 
@@ -473,8 +470,7 @@ gimp_help_call (Gimp         *gimp,
 
       if (error)
         {
-          gimp_message (gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "%s", error->message);
+          gimp_message_literal (gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
           g_error_free (error);
         }
     }
@@ -508,8 +504,7 @@ gimp_help_call (Gimp         *gimp,
 
       if (error)
         {
-          gimp_message (gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "%s", error->message);
+          gimp_message_literal (gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
           g_error_free (error);
         }
     }
