@@ -40,8 +40,8 @@
 #include "core/gimpcontainer.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-colormap.h"
+#include "core/gimpimage-undo.h"
 #include "core/gimpundostack.h"
-#include "core/gimpunit.h"
 #include "core/gimp-utils.h"
 
 #include "file/file-procedure.h"
@@ -119,7 +119,7 @@ gimp_image_prop_view_init (GimpImagePropView *view)
   gtk_table_set_row_spacings (table, 3);
 
   view->pixel_size_label =
-    gimp_image_prop_view_add_label (table, row++, _("Pixel dimensions:"));
+    gimp_image_prop_view_add_label (table, row++, _("Size in pixels:"));
 
   view->print_size_label =
     gimp_image_prop_view_add_label (table, row++, _("Print size:"));
@@ -319,7 +319,7 @@ static void
 gimp_image_prop_view_label_set_filename (GtkWidget *label,
                                          GimpImage *image)
 {
-  const gchar *uri = gimp_object_get_name (GIMP_OBJECT (image));
+  const gchar *uri = gimp_object_get_name (image);
 
   if (uri)
     {
@@ -453,23 +453,22 @@ gimp_image_prop_view_update (GimpImagePropView *view)
   /*  print size  */
   unit = gimp_get_default_unit ();
 
-  unit_factor = _gimp_unit_get_factor (image->gimp, unit);
-  unit_digits = _gimp_unit_get_digits (image->gimp, unit);
+  unit_digits = gimp_unit_get_digits (unit);
 
   g_snprintf (format_buf, sizeof (format_buf), "%%.%df × %%.%df %s",
               unit_digits + 1, unit_digits + 1,
-              _gimp_unit_get_plural (image->gimp, unit));
+              gimp_unit_get_plural (unit));
   g_snprintf (buf, sizeof (buf), format_buf,
-              gimp_image_get_width  (image) * unit_factor / xres,
-              gimp_image_get_height (image) * unit_factor / yres);
+              gimp_pixels_to_units (gimp_image_get_width  (image), unit, xres),
+              gimp_pixels_to_units (gimp_image_get_height (image), unit, yres));
   gtk_label_set_text (GTK_LABEL (view->print_size_label), buf);
 
   /*  resolution  */
-  unit = gimp_image_get_unit (image);
-  unit_factor = _gimp_unit_get_factor (image->gimp, unit);
+  unit        = gimp_image_get_unit (image);
+  unit_factor = gimp_unit_get_factor (unit);
 
   g_snprintf (format_buf, sizeof (format_buf), _("pixels/%s"),
-              _gimp_unit_get_abbreviation (image->gimp, unit));
+              gimp_unit_get_abbreviation (unit));
   g_snprintf (buf, sizeof (buf), _("%g × %g %s"),
               xres / unit_factor,
               yres / unit_factor,
@@ -502,8 +501,10 @@ gimp_image_prop_view_update (GimpImagePropView *view)
                                           GIMP_OBJECT (image));
 
   /*  undo / redo  */
-  gimp_image_prop_view_label_set_undo (view->undo_label, image->undo_stack);
-  gimp_image_prop_view_label_set_undo (view->redo_label, image->redo_stack);
+  gimp_image_prop_view_label_set_undo (view->undo_label,
+                                       gimp_image_get_undo_stack (image));
+  gimp_image_prop_view_label_set_undo (view->redo_label,
+                                       gimp_image_get_redo_stack (image));
 
   /*  number of layers  */
   g_snprintf (buf, sizeof (buf), "%d",
