@@ -38,13 +38,13 @@
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
 #include "core/gimpitemundo.h"
-#include "core/gimplayer.h"
 #include "core/gimpmarshal.h"
 #include "core/gimptreehandler.h"
 #include "core/gimpundostack.h"
 
 #include "vectors/gimpvectors.h"
 
+#include "gimpcontainertreestore.h"
 #include "gimpcontainerview.h"
 #include "gimpdnd.h"
 #include "gimpdocked.h"
@@ -68,6 +68,12 @@ struct _GimpItemTreeViewPriv
 {
   GimpImage       *image;
 
+  GtkWidget       *options_box;
+  GtkSizeGroup    *options_group;
+  GtkWidget       *lock_box;
+
+  GtkWidget       *lock_content_toggle;
+
   GtkWidget       *edit_button;
   GtkWidget       *new_button;
   GtkWidget       *raise_button;
@@ -82,6 +88,9 @@ struct _GimpItemTreeViewPriv
 
   GimpTreeHandler *visible_changed_handler;
   GimpTreeHandler *linked_changed_handler;
+  GimpTreeHandler *lock_content_changed_handler;
+
+  gboolean         inserting_item; /* EEK */
 };
 
 
@@ -94,6 +103,9 @@ static GObject * gimp_item_tree_view_constructor    (GType              type,
 
 static void   gimp_item_tree_view_destroy           (GtkObject         *object);
 
+static void   gimp_item_tree_view_style_set         (GtkWidget         *widget,
+                                                     GtkStyle          *prev_style);
+
 static void   gimp_item_tree_view_real_set_image    (GimpItemTreeView  *view,
                                                      GimpImage         *image);
 
@@ -110,6 +122,9 @@ static gpointer gimp_item_tree_view_insert_item     (GimpContainerView *view,
                                                      GimpViewable      *viewable,
                                                      gpointer           parent_insert_data,
                                                      gint               index);
+static void   gimp_item_tree_view_insert_item_after (GimpContainerView *view,
+                                                     GimpViewable      *viewable,
+                                                     gpointer           insert_data);
 static gboolean gimp_item_tree_view_select_item     (GimpContainerView *view,
                                                      GimpViewable      *item,
                                                      gpointer           insert_data);
@@ -124,6 +139,7 @@ static gboolean gimp_item_tree_view_drop_possible   (GimpContainerTreeView *view
                                                      GimpDndType        src_type,
                                                      GimpViewable      *src_viewable,
                                                      GimpViewable      *dest_viewable,
+                                                     GtkTreePath       *drop_path,
                                                      GtkTreeViewDropPosition  drop_pos,
                                                      GtkTreeViewDropPosition *return_drop_pos,
                                                      GdkDragAction     *return_drag_action);
@@ -148,10 +164,12 @@ static void   gimp_item_tree_view_name_edited       (GtkCellRendererText *cell,
                                                      const gchar       *new_name,
                                                      GimpItemTreeView  *view);
 
-static void   gimp_item_tree_view_visible_changed   (GimpItem          *item,
-                                                     GimpItemTreeView  *view);
-static void   gimp_item_tree_view_linked_changed    (GimpItem          *item,
-                                                     GimpItemTreeView  *view);
+static void   gimp_item_tree_view_visible_changed      (GimpItem          *item,
+                                                        GimpItemTreeView  *view);
+static void   gimp_item_tree_view_linked_changed       (GimpItem          *item,
+                                                        GimpItemTreeView  *view);
+static void   gimp_item_tree_view_lock_content_changed (GimpItem          *item,
+                                                        GimpItemTreeView  *view);
 
 static void   gimp_item_tree_view_eye_clicked       (GtkCellRendererToggle *toggle,
                                                      gchar             *path,
@@ -161,6 +179,12 @@ static void   gimp_item_tree_view_chain_clicked     (GtkCellRendererToggle *togg
                                                      gchar             *path,
                                                      GdkModifierType    state,
                                                      GimpItemTreeView  *view);
+static void   gimp_item_tree_view_lock_content_toggled
+                                                    (GtkWidget         *widget,
+                                                     GimpItemTreeView  *view);
+
+static void   gimp_item_tree_view_update_options    (GimpItemTreeView  *view,
+                                                     GimpItem          *item);
 
 /*  utility function to avoid code duplication  */
 static void   gimp_item_tree_view_toggle_clicked    (GtkCellRendererToggle *toggle,
@@ -169,6 +193,11 @@ static void   gimp_item_tree_view_toggle_clicked    (GtkCellRendererToggle *togg
                                                      GimpItemTreeView  *view,
                                                      GimpUndoType       undo_type);
 
+static void   gimp_item_tree_view_row_expanded      (GtkTreeView       *tree_view,
+                                                     GtkTreeIter       *iter,
+                                                     GtkTreePath       *path,
+                                                     GimpItemTreeView  *item_view);
+
 
 G_DEFINE_TYPE_WITH_CODE (GimpItemTreeView, gimp_item_tree_view,
                          GIMP_TYPE_CONTAINER_TREE_VIEW,
@@ -187,13 +216,12 @@ static guint view_signals[LAST_SIGNAL] = { 0 };
 static void
 gimp_item_tree_view_class_init (GimpItemTreeViewClass *klass)
 {
-  GObjectClass               *object_class;
-  GtkObjectClass             *gtk_object_class;
+  GObjectClass               *object_class     = G_OBJECT_CLASS (klass);
+  GtkObjectClass             *gtk_object_class = GTK_OBJECT_CLASS (klass);
+  GtkWidgetClass             *widget_class     = GTK_WIDGET_CLASS (klass);
   GimpContainerTreeViewClass *tree_view_class;
 
-  object_class     = G_OBJECT_CLASS (klass);
-  gtk_object_class = GTK_OBJECT_CLASS (klass);
-  tree_view_class  = GIMP_CONTAINER_TREE_VIEW_CLASS (klass);
+  tree_view_class = GIMP_CONTAINER_TREE_VIEW_CLASS (klass);
 
   view_signals[SET_IMAGE] =
     g_signal_new ("set-image",
@@ -209,6 +237,8 @@ gimp_item_tree_view_class_init (GimpItemTreeViewClass *klass)
 
   gtk_object_class->destroy      = gimp_item_tree_view_destroy;
 
+  widget_class->style_set        = gimp_item_tree_view_style_set;
+
   tree_view_class->drop_possible = gimp_item_tree_view_drop_possible;
   tree_view_class->drop_viewable = gimp_item_tree_view_drop_viewable;
 
@@ -235,8 +265,13 @@ gimp_item_tree_view_class_init (GimpItemTreeViewClass *klass)
   klass->lower_bottom_action     = NULL;
   klass->duplicate_action        = NULL;
   klass->delete_action           = NULL;
+
   klass->reorder_desc            = NULL;
 
+  klass->lock_content_stock_id   = NULL;
+  klass->lock_content_tooltip    = NULL;
+  klass->lock_content_help_id    = NULL;
+
   g_type_class_add_private (klass, sizeof (GimpItemTreeViewPriv));
 }
 
@@ -245,12 +280,13 @@ gimp_item_tree_view_view_iface_init (GimpContainerViewInterface *iface)
 {
   parent_view_iface = g_type_interface_peek_parent (iface);
 
-  iface->set_container = gimp_item_tree_view_set_container;
-  iface->set_context   = gimp_item_tree_view_set_context;
-  iface->insert_item   = gimp_item_tree_view_insert_item;
-  iface->select_item   = gimp_item_tree_view_select_item;
-  iface->activate_item = gimp_item_tree_view_activate_item;
-  iface->context_item  = gimp_item_tree_view_context_item;
+  iface->set_container     = gimp_item_tree_view_set_container;
+  iface->set_context       = gimp_item_tree_view_set_context;
+  iface->insert_item       = gimp_item_tree_view_insert_item;
+  iface->insert_item_after = gimp_item_tree_view_insert_item_after;
+  iface->select_item       = gimp_item_tree_view_select_item;
+  iface->activate_item     = gimp_item_tree_view_activate_item;
+  iface->context_item      = gimp_item_tree_view_context_item;
 }
 
 static void
@@ -268,22 +304,15 @@ gimp_item_tree_view_init (GimpItemTreeView *view)
                                             GIMP_TYPE_ITEM_TREE_VIEW,
                                             GimpItemTreeViewPriv);
 
-  /* The following used to read:
-   *
-   * tree_view->model_columns[tree_view->n_model_columns++] = ...
-   *
-   * but combining the two lead to gcc miscompiling the function on ppc/ia64
-   * (model_column_mask and model_column_mask_visible would have the same
-   * value, probably due to bad instruction reordering). See bug #113144 for
-   * more info.
-   */
-  view->priv->model_column_visible = tree_view->n_model_columns;
-  tree_view->model_columns[tree_view->n_model_columns] = G_TYPE_BOOLEAN;
-  tree_view->n_model_columns++;
+  view->priv->model_column_visible =
+    gimp_container_tree_store_columns_add (tree_view->model_columns,
+                                           &tree_view->n_model_columns,
+                                           G_TYPE_BOOLEAN);
 
-  view->priv->model_column_linked = tree_view->n_model_columns;
-  tree_view->model_columns[tree_view->n_model_columns] = G_TYPE_BOOLEAN;
-  tree_view->n_model_columns++;
+  view->priv->model_column_linked =
+    gimp_container_tree_store_columns_add (tree_view->model_columns,
+                                           &tree_view->n_model_columns,
+                                           G_TYPE_BOOLEAN);
 
   gimp_container_tree_view_set_dnd_drop_to_empty (tree_view, TRUE);
 
@@ -301,6 +330,9 @@ gimp_item_tree_view_constructor (GType                  type,
   GimpItemTreeView      *item_view;
   GObject               *object;
   GtkTreeViewColumn     *column;
+  GtkWidget             *hbox;
+  GtkWidget             *image;
+  GtkIconSize            icon_size;
 
   object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
 
@@ -313,18 +345,26 @@ gimp_item_tree_view_constructor (GType                  type,
                                                 G_CALLBACK (gimp_item_tree_view_name_edited),
                                                 item_view);
 
+  g_signal_connect (tree_view->view, "row-expanded",
+                    G_CALLBACK (gimp_item_tree_view_row_expanded),
+                    tree_view);
+
   column = gtk_tree_view_column_new ();
   gtk_tree_view_insert_column (tree_view->view, column, 0);
 
   item_view->priv->eye_cell = gimp_cell_renderer_toggle_new (GIMP_STOCK_VISIBLE);
+  g_object_set (item_view->priv->eye_cell,
+                "xpad", 0,
+                "ypad", 0,
+                NULL);
   gtk_tree_view_column_pack_start (column, item_view->priv->eye_cell, FALSE);
   gtk_tree_view_column_set_attributes (column, item_view->priv->eye_cell,
                                        "active",
                                        item_view->priv->model_column_visible,
                                        NULL);
 
-  gimp_container_tree_view_prepend_toggle_cell_renderer (tree_view,
-                                                         item_view->priv->eye_cell);
+  gimp_container_tree_view_add_toggle_cell (tree_view,
+                                            item_view->priv->eye_cell);
 
   g_signal_connect (item_view->priv->eye_cell, "clicked",
                     G_CALLBACK (gimp_item_tree_view_eye_clicked),
@@ -334,14 +374,18 @@ gimp_item_tree_view_constructor (GType                  type,
   gtk_tree_view_insert_column (tree_view->view, column, 1);
 
   item_view->priv->chain_cell = gimp_cell_renderer_toggle_new (GIMP_STOCK_LINKED);
+  g_object_set (item_view->priv->chain_cell,
+                "xpad", 0,
+                "ypad", 0,
+                NULL);
   gtk_tree_view_column_pack_start (column, item_view->priv->chain_cell, FALSE);
   gtk_tree_view_column_set_attributes (column, item_view->priv->chain_cell,
                                        "active",
                                        item_view->priv->model_column_linked,
                                        NULL);
 
-  gimp_container_tree_view_prepend_toggle_cell_renderer (tree_view,
-                                                         item_view->priv->chain_cell);
+  gimp_container_tree_view_add_toggle_cell (tree_view,
+                                            item_view->priv->chain_cell);
 
   g_signal_connect (item_view->priv->chain_cell, "clicked",
                     G_CALLBACK (gimp_item_tree_view_chain_clicked),
@@ -404,6 +448,36 @@ gimp_item_tree_view_constructor (GType                  type,
                                   GTK_BUTTON (item_view->priv->delete_button),
                                   item_view_class->item_type);
 
+
+  /*  Lock content toggle  */
+
+  hbox = gimp_item_tree_view_get_lock_box (item_view);
+
+  item_view->priv->lock_content_toggle = gtk_toggle_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox), item_view->priv->lock_content_toggle,
+                      FALSE, FALSE, 0);
+  gtk_box_reorder_child (GTK_BOX (hbox),
+                         item_view->priv->lock_content_toggle, 0);
+  gtk_widget_show (item_view->priv->lock_content_toggle);
+
+  g_signal_connect (item_view->priv->lock_content_toggle, "toggled",
+                    G_CALLBACK (gimp_item_tree_view_lock_content_toggled),
+                    item_view);
+
+  gimp_help_set_help_data (item_view->priv->lock_content_toggle,
+                           item_view_class->lock_content_tooltip,
+                           item_view_class->lock_content_help_id);
+
+  gtk_widget_style_get (GTK_WIDGET (item_view),
+                        "button-icon-size", &icon_size,
+                        NULL);
+
+  image = gtk_image_new_from_stock (item_view_class->lock_content_stock_id,
+                                    icon_size);
+  gtk_container_add (GTK_CONTAINER (item_view->priv->lock_content_toggle),
+                     image);
+  gtk_widget_show (image);
+
   return object;
 }
 
@@ -418,6 +492,81 @@ gimp_item_tree_view_destroy (GtkObject *object)
   GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
+static void
+gimp_item_tree_view_style_set (GtkWidget *widget,
+                               GtkStyle  *prev_style)
+{
+  GimpItemTreeView *view = GIMP_ITEM_TREE_VIEW (widget);
+  GList            *children;
+  GList            *list;
+  GtkReliefStyle    button_relief;
+  GtkIconSize       button_icon_size;
+  gint              content_spacing;
+  gint              button_spacing;
+
+  gtk_widget_style_get (widget,
+                        "button-relief",    &button_relief,
+                        "button-icon-size", &button_icon_size,
+                        "content-spacing",  &content_spacing,
+                        "button-spacing",   &button_spacing,
+                        NULL);
+
+  if (view->priv->options_box)
+    {
+      gtk_box_set_spacing (GTK_BOX (view->priv->options_box), content_spacing);
+
+      children =
+        gtk_container_get_children (GTK_CONTAINER (view->priv->options_box));
+
+      for (list = children; list; list = g_list_next (list))
+        {
+          GtkWidget *child = list->data;
+
+          gtk_box_set_spacing (GTK_BOX (child), button_spacing);
+        }
+
+      g_list_free (children);
+    }
+
+  if (view->priv->lock_box)
+    {
+      gtk_box_set_spacing (GTK_BOX (view->priv->lock_box), button_spacing);
+
+      children =
+        gtk_container_get_children (GTK_CONTAINER (view->priv->lock_box));
+
+      for (list = children; list; list = g_list_next (list))
+        {
+          GtkWidget *child = list->data;
+
+          if (GTK_IS_BUTTON (child))
+            {
+              GtkWidget *image;
+
+              gtk_button_set_relief (GTK_BUTTON (child), button_relief);
+
+              image = gtk_bin_get_child (GTK_BIN (child));
+
+              if (GTK_IS_IMAGE (image))
+                {
+                  GtkIconSize  old_size;
+                  gchar       *stock_id;
+
+                  gtk_image_get_stock (GTK_IMAGE (image), &stock_id, &old_size);
+
+                  if (button_icon_size != old_size)
+                    gtk_image_set_from_stock (GTK_IMAGE (image),
+                                              stock_id, button_icon_size);
+                }
+            }
+        }
+
+      g_list_free (children);
+    }
+
+  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+}
+
 GtkWidget *
 gimp_item_tree_view_new (GType            view_type,
                          gint             view_size,
@@ -475,6 +624,91 @@ gimp_item_tree_view_get_image (GimpItemTreeView *view)
   return view->priv->image;
 }
 
+void
+gimp_item_tree_view_add_options (GimpItemTreeView *view,
+                                 const gchar      *label,
+                                 GtkWidget        *options)
+{
+  GtkWidget *hbox;
+  GtkWidget *label_widget;
+  gint       content_spacing;
+  gint       button_spacing;
+  gboolean   box_created = FALSE;
+
+  g_return_if_fail (GIMP_IS_ITEM_TREE_VIEW (view));
+  g_return_if_fail (label != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (options));
+
+  gtk_widget_style_get (GTK_WIDGET (view),
+                        "content-spacing", &content_spacing,
+                        "button-spacing",  &button_spacing,
+                        NULL);
+
+  if (! view->priv->options_box)
+    {
+      GimpItemTreeViewClass *item_view_class;
+
+      item_view_class = GIMP_ITEM_TREE_VIEW_GET_CLASS (view);
+
+      view->priv->options_box = gtk_vbox_new (FALSE, content_spacing);
+      gtk_box_pack_start (GTK_BOX (view), view->priv->options_box,
+                          FALSE, FALSE, 0);
+      gtk_box_reorder_child (GTK_BOX (view), view->priv->options_box, 0);
+      gtk_widget_show (view->priv->options_box);
+
+      if (! view->priv->image ||
+          ! item_view_class->get_active_item (view->priv->image))
+        {
+          gtk_widget_set_sensitive (view->priv->options_box, FALSE);
+        }
+
+      view->priv->options_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+      box_created = TRUE;
+    }
+
+  hbox = gtk_hbox_new (FALSE, button_spacing);
+  gtk_box_pack_start (GTK_BOX (view->priv->options_box), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
+  label_widget = gtk_label_new (label);
+  gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);
+  gtk_size_group_add_widget (view->priv->options_group, label_widget);
+  gtk_box_pack_start (GTK_BOX (hbox), label_widget, FALSE, FALSE, 0);
+  gtk_widget_show (label_widget);
+
+  if (box_created)
+    g_object_unref (view->priv->options_group);
+
+  gtk_box_pack_start (GTK_BOX (hbox), options, TRUE, TRUE, 0);
+  gtk_widget_show (options);
+}
+
+GtkWidget *
+gimp_item_tree_view_get_lock_box (GimpItemTreeView *view)
+{
+  g_return_val_if_fail (GIMP_IS_ITEM_TREE_VIEW (view), NULL);
+
+  if (! view->priv->lock_box)
+    {
+      gint button_spacing;
+
+      gtk_widget_style_get (GTK_WIDGET (view),
+                            "button-spacing", &button_spacing,
+                            NULL);
+
+      view->priv->lock_box = gtk_hbox_new (FALSE, button_spacing);
+
+      gimp_item_tree_view_add_options (view, _("Lock:"), view->priv->lock_box);
+
+      gtk_box_set_child_packing (GTK_BOX (view->priv->options_box),
+                                 gtk_widget_get_parent (view->priv->lock_box),
+                                 FALSE, FALSE, 0, GTK_PACK_END);
+    }
+
+  return view->priv->lock_box;
+}
+
 GtkWidget *
 gimp_item_tree_view_get_new_button (GimpItemTreeView *view)
 {
@@ -607,6 +841,9 @@ gimp_item_tree_view_set_container (GimpContainerView *view,
 
       gimp_tree_handler_disconnect (item_view->priv->linked_changed_handler);
       item_view->priv->linked_changed_handler = NULL;
+
+      gimp_tree_handler_disconnect (item_view->priv->lock_content_changed_handler);
+      item_view->priv->lock_content_changed_handler = NULL;
     }
 
   parent_view_iface->set_container (view, container);
@@ -622,6 +859,11 @@ gimp_item_tree_view_set_container (GimpContainerView *view,
         gimp_tree_handler_connect (container, "linked-changed",
                                    G_CALLBACK (gimp_item_tree_view_linked_changed),
                                    view);
+
+      item_view->priv->lock_content_changed_handler =
+        gimp_tree_handler_connect (container, "lock-content-changed",
+                                   G_CALLBACK (gimp_item_tree_view_lock_content_changed),
+                                   view);
     }
 }
 
@@ -671,9 +913,13 @@ gimp_item_tree_view_insert_item (GimpContainerView *view,
   GimpItem              *item      = GIMP_ITEM (viewable);
   GtkTreeIter           *iter;
 
+  item_view->priv->inserting_item = TRUE;
+
   iter = parent_view_iface->insert_item (view, viewable,
                                          parent_insert_data, index);
 
+  item_view->priv->inserting_item = FALSE;
+
   gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), iter,
                       item_view->priv->model_column_visible,
                       gimp_item_get_visible (item),
@@ -684,12 +930,30 @@ gimp_item_tree_view_insert_item (GimpContainerView *view,
   return iter;
 }
 
+static void
+gimp_item_tree_view_insert_item_after (GimpContainerView *view,
+                                       GimpViewable      *viewable,
+                                       gpointer           insert_data)
+{
+  GimpItemTreeView      *item_view = GIMP_ITEM_TREE_VIEW (view);
+  GimpItemTreeViewClass *item_view_class;
+  GimpItem              *active_item;
+
+  item_view_class = GIMP_ITEM_TREE_VIEW_GET_CLASS (item_view);
+
+  active_item = item_view_class->get_active_item (item_view->priv->image);
+
+  if (active_item == (GimpItem *) viewable)
+    gimp_container_view_select_item (view, viewable);
+}
+
 static gboolean
 gimp_item_tree_view_select_item (GimpContainerView *view,
                                  GimpViewable      *item,
                                  gpointer           insert_data)
 {
-  GimpItemTreeView *tree_view = GIMP_ITEM_TREE_VIEW (view);
+  GimpItemTreeView *tree_view         = GIMP_ITEM_TREE_VIEW (view);
+  gboolean          options_sensitive = FALSE;
   gboolean          success;
 
   success = parent_view_iface->select_item (view, item, insert_data);
@@ -710,10 +974,17 @@ gimp_item_tree_view_select_item (GimpContainerView *view,
 
           gimp_image_flush (tree_view->priv->image);
         }
+
+      options_sensitive = TRUE;
+
+      gimp_item_tree_view_update_options (tree_view, GIMP_ITEM (item));
     }
 
   gimp_ui_manager_update (GIMP_EDITOR (tree_view)->ui_manager, tree_view);
 
+  if (tree_view->priv->options_box)
+    gtk_widget_set_sensitive (tree_view->priv->options_box, options_sensitive);
+
   return success;
 }
 
@@ -751,6 +1022,7 @@ gimp_item_tree_view_drop_possible (GimpContainerTreeView   *tree_view,
                                    GimpDndType              src_type,
                                    GimpViewable            *src_viewable,
                                    GimpViewable            *dest_viewable,
+                                   GtkTreePath             *drop_path,
                                    GtkTreeViewDropPosition  drop_pos,
                                    GtkTreeViewDropPosition *return_drop_pos,
                                    GdkDragAction           *return_drag_action)
@@ -773,6 +1045,7 @@ gimp_item_tree_view_drop_possible (GimpContainerTreeView   *tree_view,
                                                                        src_type,
                                                                        src_viewable,
                                                                        dest_viewable,
+                                                                       drop_path,
                                                                        drop_pos,
                                                                        return_drop_pos,
                                                                        return_drag_action);
@@ -855,13 +1128,10 @@ gimp_item_tree_view_new_dropped (GtkWidget    *widget,
                                  gpointer      data)
 {
   GimpItemTreeViewClass *item_view_class = GIMP_ITEM_TREE_VIEW_GET_CLASS (data);
-  GimpItemTreeView      *view            = GIMP_ITEM_TREE_VIEW (data);
-  GimpContainer         *container;
+  GimpContainerView     *view            = GIMP_CONTAINER_VIEW (data);
 
-  container = gimp_container_view_get_container (GIMP_CONTAINER_VIEW (view));
-
-  if (viewable && gimp_container_have (container, GIMP_OBJECT (viewable)) &&
-      item_view_class->new_default_action)
+  if (item_view_class->new_default_action &&
+      viewable && gimp_container_view_lookup (view, viewable))
     {
       GtkAction *action;
 
@@ -926,12 +1196,12 @@ gimp_item_tree_view_name_edited (GtkCellRendererText *cell,
       GError           *error = NULL;
 
       gtk_tree_model_get (tree_view->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
 
       item = GIMP_ITEM (renderer->viewable);
 
-      old_name = gimp_object_get_name (GIMP_OBJECT (item));
+      old_name = gimp_object_get_name (item);
 
       if (! old_name) old_name = "";
       if (! new_name) new_name = "";
@@ -946,7 +1216,7 @@ gimp_item_tree_view_name_edited (GtkCellRendererText *cell,
           gchar *name = gimp_viewable_get_description (renderer->viewable, NULL);
 
           gtk_tree_store_set (GTK_TREE_STORE (tree_view->model), &iter,
-                              GIMP_CONTAINER_TREE_VIEW_COLUMN_NAME, name,
+                              GIMP_CONTAINER_TREE_STORE_COLUMN_NAME, name,
                               -1);
           g_free (name);
 
@@ -1027,6 +1297,91 @@ gimp_item_tree_view_chain_clicked (GtkCellRendererToggle *toggle,
 }
 
 
+/*  "Lock Content" callbacks  */
+
+static void
+gimp_item_tree_view_lock_content_changed (GimpItem         *item,
+                                          GimpItemTreeView *view)
+{
+  GimpImage *image = view->priv->image;
+  GimpItem  *active_item;
+
+  active_item = GIMP_ITEM_TREE_VIEW_GET_CLASS (view)->get_active_item (image);
+
+  if (active_item == item)
+    gimp_item_tree_view_update_options (view, item);
+}
+
+static void
+gimp_item_tree_view_lock_content_toggled (GtkWidget         *widget,
+                                          GimpItemTreeView  *view)
+{
+  GimpImage *image = view->priv->image;
+  GimpItem  *item;
+
+  item = GIMP_ITEM_TREE_VIEW_GET_CLASS (view)->get_active_item (image);
+
+  if (item)
+    {
+      gboolean lock_content;
+
+      lock_content = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+      if (gimp_item_get_lock_content (item) != lock_content)
+        {
+#if 0
+          GimpUndo *undo;
+#endif
+          gboolean  push_undo = TRUE;
+
+#if 0
+          /*  compress lock content undos  */
+          undo = gimp_image_undo_can_compress (image, GIMP_TYPE_ITEM_UNDO,
+                                               GIMP_UNDO_ITEM_LOCK_CONTENT);
+
+          if (undo && GIMP_ITEM_UNDO (undo)->item == item)
+            push_undo = FALSE;
+#endif
+
+          g_signal_handlers_block_by_func (item,
+                                           gimp_item_tree_view_lock_content_changed,
+                                           view);
+
+          gimp_item_set_lock_content (item, lock_content, push_undo);
+
+          g_signal_handlers_unblock_by_func (item,
+                                             gimp_item_tree_view_lock_content_changed,
+                                             view);
+
+          gimp_image_flush (image);
+        }
+    }
+}
+
+static void
+gimp_item_tree_view_update_options (GimpItemTreeView *view,
+                                    GimpItem         *item)
+{
+  if (gimp_item_get_lock_content (item) !=
+      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->priv->lock_content_toggle)))
+    {
+      g_signal_handlers_block_by_func (view->priv->lock_content_toggle,
+                                       gimp_item_tree_view_lock_content_toggled,
+                                       view);
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->priv->lock_content_toggle),
+                                    gimp_item_get_lock_content (item));
+
+      g_signal_handlers_unblock_by_func (view->priv->lock_content_toggle,
+                                         gimp_item_tree_view_lock_content_toggled,
+                                         view);
+    }
+
+  gtk_widget_set_sensitive (view->priv->lock_content_toggle,
+                            gimp_item_can_lock_content (item));
+}
+
+
 /*  Utility functions used from eye_clicked and chain_clicked.
  *  Would make sense to do this in a generic fashion using
  *  properties, but for now it's better than duplicating the code.
@@ -1087,7 +1442,7 @@ gimp_item_tree_view_toggle_clicked (GtkCellRendererToggle *toggle,
       context = gimp_container_view_get_context (GIMP_CONTAINER_VIEW (view));
 
       gtk_tree_model_get (tree_view->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
       g_object_get (toggle,
                     "active", &active,
@@ -1112,7 +1467,7 @@ gimp_item_tree_view_toggle_clicked (GtkCellRendererToggle *toggle,
                                                       &iter))
             {
               gtk_tree_model_get (tree_view->model, &iter,
-                                  GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                                   -1);
 
               if ((GimpItem *) renderer->viewable != item)
@@ -1209,3 +1564,28 @@ gimp_item_tree_view_toggle_clicked (GtkCellRendererToggle *toggle,
 
   gtk_tree_path_free (path);
 }
+
+
+/*  GtkTreeView callbacks  */
+
+static void
+gimp_item_tree_view_row_expanded (GtkTreeView      *tree_view,
+                                  GtkTreeIter      *iter,
+                                  GtkTreePath      *path,
+                                  GimpItemTreeView *item_view)
+{
+  /*  don't select the item while it is being inserted  */
+  if (! item_view->priv->inserting_item)
+    {
+      GimpItemTreeViewClass *item_view_class;
+      GimpItem              *active_item;
+
+      item_view_class = GIMP_ITEM_TREE_VIEW_GET_CLASS (item_view);
+
+      active_item = item_view_class->get_active_item (item_view->priv->image);
+
+      if (active_item)
+        gimp_container_view_select_item (GIMP_CONTAINER_VIEW (item_view),
+                                         GIMP_VIEWABLE (active_item));
+    }
+}
