@@ -239,7 +239,7 @@ gimp_message_box_size_request (GtkWidget      *widget,
 
   GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
-  if (box->image && GTK_WIDGET_VISIBLE (box->image))
+  if (box->image && gtk_widget_get_visible (box->image))
     {
       GtkRequisition  child_requisition;
       gint            border_width;
@@ -266,7 +266,7 @@ gimp_message_box_size_allocate (GtkWidget     *widget,
 
   rtl = (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL);
 
-  if (box->image)
+  if (box->image && gtk_widget_get_visible (box->image))
     {
       GtkRequisition  child_requisition;
       GtkAllocation   child_allocation;
@@ -306,7 +306,7 @@ gimp_message_box_size_allocate (GtkWidget     *widget,
   allocation->x      -= rtl ? 0 : width;
   allocation->width  += width;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 }
 
 static void
