@@ -155,10 +155,11 @@ gimp_navigation_view_class_init (GimpNavigationViewClass *klass)
 static void
 gimp_navigation_view_init (GimpNavigationView *view)
 {
-  GTK_WIDGET_SET_FLAGS (view, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (GTK_WIDGET (view), TRUE);
+  gtk_widget_add_events (GTK_WIDGET (view),
+                         GDK_POINTER_MOTION_MASK |
+                         GDK_KEY_PRESS_MASK);
 
-  gtk_widget_add_events (GTK_WIDGET (view), (GDK_POINTER_MOTION_MASK |
-                                             GDK_KEY_PRESS_MASK));
   view->x               = 0.0;
   view->y               = 0.0;
   view->width           = 0.0;
@@ -188,7 +189,7 @@ static gboolean
 gimp_navigation_view_expose (GtkWidget      *widget,
                              GdkEventExpose *event)
 {
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       cairo_t *cr;
 
@@ -514,12 +515,15 @@ gimp_navigation_view_draw_marker (GimpNavigationView *nav_view,
 
   if (view->renderer->viewable && nav_view->width && nav_view->height)
     {
-      GtkWidget *widget = GTK_WIDGET (view);
+      GtkWidget     *widget = GTK_WIDGET (view);
+      GtkAllocation  allocation;
 
-      cairo_translate (cr, widget->allocation.x, widget->allocation.y);
+      gtk_widget_get_allocation (widget, &allocation);
+
+      cairo_translate (cr, allocation.x, allocation.y);
       cairo_rectangle (cr,
                        0, 0,
-                       widget->allocation.width, widget->allocation.height);
+                       allocation.width, allocation.height);
       cairo_rectangle (cr,
                        nav_view->p_x, nav_view->p_y,
                        nav_view->p_width, nav_view->p_height);
