@@ -152,7 +152,7 @@ gimp_palette_view_class_init (GimpPaletteViewClass *klass)
 static void
 gimp_palette_view_init (GimpPaletteView *view)
 {
-  GTK_WIDGET_SET_FLAGS (view, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (GTK_WIDGET (view), TRUE);
 
   view->selected  = NULL;
   view->dnd_entry = NULL;
@@ -165,7 +165,7 @@ gimp_palette_view_expose (GtkWidget      *widget,
   GimpPaletteView *pal_view = GIMP_PALETTE_VIEW (widget);
   GimpView        *view     = GIMP_VIEW (widget);
 
-  if (! GTK_WIDGET_DRAWABLE (widget))
+  if (! gtk_widget_is_drawable (widget))
     return FALSE;
 
   GTK_WIDGET_CLASS (parent_class)->expose_event (widget, eevent);
@@ -174,11 +174,14 @@ gimp_palette_view_expose (GtkWidget      *widget,
     {
       GimpViewRendererPalette *renderer;
       GtkStyle                *style = gtk_widget_get_style (widget);
+      GtkAllocation            allocation;
       cairo_t                 *cr;
       gint                     row, col;
 
       renderer = GIMP_VIEW_RENDERER_PALETTE (view->renderer);
 
+      gtk_widget_get_allocation (widget, &allocation);
+
       row = pal_view->selected->position / renderer->columns;
       col = pal_view->selected->position % renderer->columns;
 
@@ -187,8 +190,8 @@ gimp_palette_view_expose (GtkWidget      *widget,
       cairo_clip (cr);
 
       cairo_rectangle (cr,
-                       widget->allocation.x + col * renderer->cell_width  + 0.5,
-                       widget->allocation.y + row * renderer->cell_height + 0.5,
+                       allocation.x + col * renderer->cell_width  + 0.5,
+                       allocation.y + row * renderer->cell_height + 0.5,
                        renderer->cell_width,
                        renderer->cell_height);
 
@@ -215,7 +218,7 @@ gimp_palette_view_button_press (GtkWidget      *widget,
   GimpPaletteView  *view = GIMP_PALETTE_VIEW (widget);
   GimpPaletteEntry *entry;
 
-  if (GTK_WIDGET_CAN_FOCUS (widget) && ! GTK_WIDGET_HAS_FOCUS (widget))
+  if (gtk_widget_get_can_focus (widget) && ! gtk_widget_has_focus (widget))
     gtk_widget_grab_focus (widget);
 
   entry = gimp_palette_view_find_entry (view, bevent->x, bevent->y);
@@ -288,7 +291,7 @@ gimp_palette_view_focus (GtkWidget        *widget,
 
   palette = GIMP_PALETTE (GIMP_VIEW (view)->renderer->viewable);
 
-  if (GTK_WIDGET_CAN_FOCUS (widget) && ! GTK_WIDGET_HAS_FOCUS (widget))
+  if (gtk_widget_get_can_focus (widget) && ! gtk_widget_has_focus (widget))
     {
       gtk_widget_grab_focus (widget);
 
@@ -452,15 +455,18 @@ gimp_palette_view_expose_entry (GimpPaletteView  *view,
   GimpViewRendererPalette *renderer;
   gint                     row, col;
   GtkWidget               *widget = GTK_WIDGET (view);
+  GtkAllocation            allocation;
 
   renderer = GIMP_VIEW_RENDERER_PALETTE (GIMP_VIEW (view)->renderer);
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   row = entry->position / renderer->columns;
   col = entry->position % renderer->columns;
 
   gtk_widget_queue_draw_area (GTK_WIDGET (view),
-                              widget->allocation.x + col * renderer->cell_width,
-                              widget->allocation.y + row * renderer->cell_height,
+                              allocation.x + col * renderer->cell_width,
+                              allocation.y + row * renderer->cell_height,
                               renderer->cell_width  + 1,
                               renderer->cell_height + 1);
 }
