@@ -123,7 +123,7 @@ gimp_pattern_select_run_callback (GimpPdbDialog  *dialog,
                                         dialog->caller_context,
                                         NULL, error,
                                         dialog->callback_name,
-                                        G_TYPE_STRING,        object->name,
+                                        G_TYPE_STRING,        gimp_object_get_name (object),
                                         GIMP_TYPE_INT32,      pattern->mask->width,
                                         GIMP_TYPE_INT32,      pattern->mask->height,
                                         GIMP_TYPE_INT32,      pattern->mask->bytes,
