@@ -4,9 +4,9 @@
  * gimpsettingsbox.h
  * Copyright (C) 2008 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_SETTINGS_BOX_H__
@@ -60,10 +59,13 @@ struct _GimpSettingsBoxClass
 {
   GtkHBoxClass  parent_class;
 
-  void (* import) (GimpSettingsBox *box,
-                   const gchar     *filename);
-  void (* export) (GimpSettingsBox *box,
-                   const gchar     *filename);
+  void (* file_dialog_setup) (GimpSettingsBox      *box,
+                              GtkFileChooserDialog *dialog,
+                              gboolean              export);
+  void (* import)            (GimpSettingsBox      *box,
+                              const gchar          *filename);
+  void (* export)            (GimpSettingsBox      *box,
+                              const gchar          *filename);
 };
 
 
@@ -79,7 +81,8 @@ GtkWidget * gimp_settings_box_new         (Gimp            *gimp,
                                            const gchar     *default_folder,
                                            const gchar     *last_filename);
 
-void        gimp_settings_box_add_current (GimpSettingsBox *box);
+void        gimp_settings_box_add_current (GimpSettingsBox *box,
+                                           gint             max_recent);
 
 
 #endif  /*  __GIMP_SETTINGS_BOX_H__  */
