@@ -32,6 +32,7 @@
 #include "core/gimpcontext.h"
 #include "core/gimptoolinfo.h"
 
+#include "gimpcontainertreestore.h"
 #include "gimpcontainerview.h"
 #include "gimpviewrenderer.h"
 #include "gimptooleditor.h"
@@ -276,8 +277,7 @@ gimp_tool_editor_new (GimpContainer *container,
                                               gimp_tool_editor_eye_data_func,
                                               tree_view, NULL);
 
-    gimp_container_tree_view_prepend_toggle_cell_renderer (tree_view,
-                                                           eye_cell);
+    gimp_container_tree_view_add_toggle_cell (tree_view, eye_cell);
 
     g_signal_connect (eye_cell, "clicked",
                       G_CALLBACK (gimp_tool_editor_eye_clicked),
@@ -292,16 +292,16 @@ gimp_tool_editor_new (GimpContainer *container,
   /* buttons */
   priv->raise_button =
     gimp_editor_add_button (GIMP_EDITOR (tree_view), GTK_STOCK_GO_UP,
-                            _("Raise this tool Raise this tool to the top"),
-                            NULL,
+                            _("Raise this tool"),
+                            _("Raise this tool to the top"),
                             G_CALLBACK (gimp_tool_editor_raise_clicked),
                             G_CALLBACK (gimp_tool_editor_raise_extend_clicked),
                             tool_editor);
 
   priv->lower_button =
     gimp_editor_add_button (GIMP_EDITOR (tree_view), GTK_STOCK_GO_DOWN,
-                            _("Lower this tool Lower this tool to the bottom"),
-                            NULL,
+                            _("Lower this tool"),
+                            _("Lower this tool to the bottom"),
                             G_CALLBACK (gimp_tool_editor_lower_clicked),
                             G_CALLBACK (gimp_tool_editor_lower_extend_clicked),
                             tool_editor);
@@ -493,7 +493,7 @@ gimp_tool_editor_eye_data_func (GtkTreeViewColumn *tree_column,
   gboolean          visible;
 
   gtk_tree_model_get (tree_model, iter,
-                      GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                      GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                       -1);
 
   g_object_get (renderer->viewable, "visible", &visible, NULL);
@@ -524,7 +524,7 @@ gimp_tool_editor_eye_clicked (GtkCellRendererToggle *toggle,
                     "active", &active,
                     NULL);
       gtk_tree_model_get (priv->model, &iter,
-                          GIMP_CONTAINER_TREE_VIEW_COLUMN_RENDERER, &renderer,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
 
       g_object_set (renderer->viewable, "visible", ! active, NULL);
