@@ -22,6 +22,8 @@
 
 #include <string.h>
 
+#undef GSEAL_ENABLE
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -692,10 +694,18 @@ gimp_ui_manager_entry_load (GimpUIManager         *manager,
                             GimpUIManagerUIEntry  *entry,
                             GError               **error)
 {
-  gchar *filename;
+  gchar       *filename           = NULL;
+  const gchar *menus_dir_override = g_getenv ("GIMP_TESTING_MENUS_DIR");
 
-  filename = g_build_filename (gimp_data_directory (), "menus",
-                               entry->basename, NULL);
+  /* In order for test cases to be able to run without GIMP being
+   * installed yet, allow them to override the menus directory to the
+   * menus dir in the source root
+   */
+  if (menus_dir_override)
+    filename = g_build_filename (menus_dir_override, entry->basename, NULL);
+  else
+    filename = g_build_filename (gimp_data_directory (), "menus",
+                                 entry->basename, NULL);
 
   if (manager->gimp->be_verbose)
     g_print ("loading menu '%s' for %s\n",
@@ -943,7 +953,8 @@ gimp_ui_manager_item_key_press (GtkWidget     *widget,
 
       if (! menu_item && GTK_IS_MENU (widget))
         {
-          GdkWindow *window = gtk_widget_get_window (GTK_MENU (widget)->toplevel);
+          GtkWidget *parent = gtk_widget_get_parent (widget);
+          GdkWindow *window = gtk_widget_get_window (parent);
           gint       x, y;
 
           gdk_window_get_pointer (window, &x, &y, NULL);
@@ -1072,7 +1083,7 @@ child_location_foreach (GtkWidget *child,
   struct ChildLocation *child_loc = data;
 
   /* Ignore invisible widgets */
-  if (! GTK_WIDGET_DRAWABLE (child))
+  if (! gtk_widget_is_drawable (child))
     return;
 
   /* (child_loc->x, child_loc->y) are relative to
@@ -1084,20 +1095,24 @@ child_location_foreach (GtkWidget *child,
                                         child_loc->x, child_loc->y,
                                         &x, &y))
     {
+      GtkAllocation child_allocation;
+
+      gtk_widget_get_allocation (child, &child_allocation);
+
 #ifdef DEBUG_TOOLTIP
       g_print ("candidate: %s  alloc=[(%d,%d)  %dx%d]     (%d, %d)->(%d, %d)\n",
                gtk_widget_get_name (child),
-               child->allocation.x,
-               child->allocation.y,
-               child->allocation.width,
-               child->allocation.height,
+               child_allocation.x,
+               child_allocation.y,
+               child_allocation.width,
+               child_allocation.height,
                child_loc->x, child_loc->y,
                x, y);
 #endif /* DEBUG_TOOLTIP */
 
       /* (x, y) relative to child's allocation. */
-      if (x >= 0 && x < child->allocation.width
-          && y >= 0 && y < child->allocation.height)
+      if (x >= 0 && x < child_allocation.width
+          && y >= 0 && y < child_allocation.height)
         {
           if (GTK_IS_CONTAINER (child))
             {
@@ -1136,22 +1151,28 @@ window_to_alloc (GtkWidget *dest_widget,
                  gint      *dest_x,
                  gint      *dest_y)
 {
+  GtkAllocation dest_allocation;
+
+  gtk_widget_get_allocation (dest_widget, &dest_allocation);
+
   /* Translate from window relative to allocation relative */
-  if (! GTK_WIDGET_NO_WINDOW (dest_widget) && dest_widget->parent)
+  if (gtk_widget_get_has_window (dest_widget) &&
+      gtk_widget_get_parent (dest_widget))
     {
       gint wx, wy;
+
       gdk_window_get_position (gtk_widget_get_window (dest_widget), &wx, &wy);
 
       /* Offset coordinates if widget->window is smaller than
        * widget->allocation.
        */
-      src_x += wx - dest_widget->allocation.x;
-      src_y += wy - dest_widget->allocation.y;
+      src_x += wx - dest_allocation.x;
+      src_y += wy - dest_allocation.y;
     }
   else
     {
-      src_x -= dest_widget->allocation.x;
-      src_y -= dest_widget->allocation.y;
+      src_x -= dest_allocation.x;
+      src_y -= dest_allocation.y;
     }
 
   if (dest_x)
