@@ -111,10 +111,14 @@ gimp_view_popup_show (GtkWidget      *widget,
   popup->button_x     = bevent->x;
   popup->button_y     = bevent->y;
 
-  if (GTK_WIDGET_NO_WINDOW (widget))
+  if (! gtk_widget_get_has_window (widget))
     {
-      popup->button_x += widget->allocation.x;
-      popup->button_y += widget->allocation.y;
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (widget, &allocation);
+
+      popup->button_x += allocation.x;
+      popup->button_y += allocation.y;
     }
 
   g_signal_connect (widget, "button-release-event",
