@@ -161,12 +161,12 @@ gimp_view_class_init (GimpViewClass *klass)
 static void
 gimp_view_init (GimpView *view)
 {
-  GTK_WIDGET_SET_FLAGS (view, GTK_NO_WINDOW);
-
-  gtk_widget_add_events (GTK_WIDGET (view), (GDK_BUTTON_PRESS_MASK   |
-                                             GDK_BUTTON_RELEASE_MASK |
-                                             GDK_ENTER_NOTIFY_MASK   |
-                                             GDK_LEAVE_NOTIFY_MASK));
+  gtk_widget_set_has_window (GTK_WIDGET (view), FALSE);
+  gtk_widget_add_events (GTK_WIDGET (view),
+                         GDK_BUTTON_PRESS_MASK   |
+                         GDK_BUTTON_RELEASE_MASK |
+                         GDK_ENTER_NOTIFY_MASK   |
+                         GDK_LEAVE_NOTIFY_MASK);
 
   view->event_window      = NULL;
   view->viewable          = NULL;
@@ -203,16 +203,19 @@ static void
 gimp_view_realize (GtkWidget *widget)
 {
   GimpView      *view = GIMP_VIEW (widget);
+  GtkAllocation  allocation;
   GdkWindowAttr  attributes;
   gint           attributes_mask;
 
   GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.x           = widget->allocation.x;
-  attributes.y           = widget->allocation.y;
-  attributes.width       = widget->allocation.width;
-  attributes.height      = widget->allocation.height;
+  attributes.x           = allocation.x;
+  attributes.y           = allocation.y;
+  attributes.width       = allocation.width;
+  attributes.height      = allocation.height;
   attributes.wclass      = GDK_INPUT_ONLY;
   attributes.event_mask  = gtk_widget_get_events (widget);
 
@@ -369,9 +372,9 @@ gimp_view_size_allocate (GtkWidget     *widget,
   allocation->width  = width;
   allocation->height = height;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     gdk_window_move_resize (view->event_window,
                             allocation->x,
                             allocation->y,
@@ -383,11 +386,15 @@ static gboolean
 gimp_view_expose_event (GtkWidget      *widget,
                         GdkEventExpose *event)
 {
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (widget, &allocation);
+
       gimp_view_renderer_draw (GIMP_VIEW (widget)->renderer,
                                gtk_widget_get_window (widget), widget,
-                               &widget->allocation,
+                               &allocation,
                                &event->area);
     }
 
@@ -422,7 +429,7 @@ gimp_view_button_press_event (GtkWidget      *widget,
       ! view->show_popup)
     return FALSE;
 
-  if (! GTK_WIDGET_REALIZED (widget))
+  if (! gtk_widget_get_realized (widget))
     return FALSE;
 
   if (bevent->type == GDK_BUTTON_PRESS)
@@ -770,19 +777,19 @@ static void
 gimp_view_update_callback (GimpViewRenderer *renderer,
                            GimpView         *view)
 {
-  GtkWidget *widget = GTK_WIDGET (view);
-  gint       width;
-  gint       height;
+  GtkWidget      *widget = GTK_WIDGET (view);
+  GtkRequisition  requisition;
+  gint            width;
+  gint            height;
+
+  gtk_widget_get_requisition (widget, &requisition);
 
   width  = renderer->width  + 2 * renderer->border_width;
   height = renderer->height + 2 * renderer->border_width;
 
-  if (width  != widget->requisition.width ||
-      height != widget->requisition.height)
+  if (width  != requisition.width ||
+      height != requisition.height)
     {
-      widget->requisition.width  = width;
-      widget->requisition.height = height;
-
       gtk_widget_queue_resize (widget);
     }
   else
