@@ -4,9 +4,9 @@
  * gimpviewrenderer.h
  * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_VIEW_RENDERER_H__
@@ -53,12 +52,12 @@ struct _GimpViewRenderer
 
   GimpViewBorderType  border_type;
   GimpRGB             border_color;
-  GdkGC              *gc;
+
+  /*< protected >*/
+  cairo_surface_t    *surface;
 
   /*< private >*/
-  guchar             *buffer;
-  gint                rowstride;
-  gint                bytes;
+  cairo_pattern_t    *pattern;
   GdkPixbuf          *pixbuf;
   gchar              *bg_stock_id;
 
@@ -85,10 +84,9 @@ struct _GimpViewRendererClass
                         GimpContext        *context);
   void (* invalidate)  (GimpViewRenderer   *renderer);
   void (* draw)        (GimpViewRenderer   *renderer,
-                        GdkWindow          *window,
                         GtkWidget          *widget,
-                        const GdkRectangle *draw_area,
-                        const GdkRectangle *expose_area);
+                        cairo_t            *cr,
+                        const GdkRectangle *area);
   void (* render)      (GimpViewRenderer   *renderer,
                         GtkWidget          *widget);
 };
@@ -128,8 +126,6 @@ void   gimp_view_renderer_set_border_color (GimpViewRenderer   *renderer,
 void   gimp_view_renderer_set_background   (GimpViewRenderer   *renderer,
                                             const gchar        *stock_id);
 
-void   gimp_view_renderer_unrealize        (GimpViewRenderer   *renderer);
-
 void   gimp_view_renderer_invalidate       (GimpViewRenderer   *renderer);
 void   gimp_view_renderer_update           (GimpViewRenderer   *renderer);
 void   gimp_view_renderer_update_idle      (GimpViewRenderer   *renderer);
@@ -143,32 +139,21 @@ void   gimp_view_renderer_draw             (GimpViewRenderer   *renderer,
 
 /*  protected  */
 
-void   gimp_view_renderer_default_render_buffer (GimpViewRenderer *renderer,
-                                                 GtkWidget        *widget,
-                                                 TempBuf          *temp_buf);
-void   gimp_view_renderer_default_render_stock  (GimpViewRenderer *renderer,
-                                                 GtkWidget        *widget,
-                                                 const gchar      *stock_id);
-void   gimp_view_renderer_render_buffer         (GimpViewRenderer *renderer,
-                                                 TempBuf          *temp_buf,
-                                                 gint              channel,
-                                                 GimpViewBG        inside_bg,
-                                                 GimpViewBG        outside_bg);
-void    gimp_view_renderer_render_pixbuf        (GimpViewRenderer *renderer,
-                                                 GdkPixbuf        *pixbuf);
+void   gimp_view_renderer_default_render_surface (GimpViewRenderer *renderer,
+                                                  GtkWidget        *widget,
+                                                  TempBuf          *temp_buf);
+void   gimp_view_renderer_default_render_stock   (GimpViewRenderer *renderer,
+                                                  GtkWidget        *widget,
+                                                  const gchar      *stock_id);
+void   gimp_view_renderer_render_surface         (GimpViewRenderer *renderer,
+                                                  TempBuf          *temp_buf,
+                                                  gint              channel,
+                                                  GimpViewBG        inside_bg,
+                                                  GimpViewBG        outside_bg);
 
+void   gimp_view_renderer_render_pixbuf          (GimpViewRenderer *renderer,
+                                                  GdkPixbuf        *pixbuf);
 
-/*  general purpose temp_buf to buffer projection function  */
-
-void   gimp_view_render_to_buffer          (TempBuf    *temp_buf,
-                                            gint        channel,
-                                            GimpViewBG  inside_bg,
-                                            GimpViewBG  outside_bg,
-                                            guchar     *dest_buffer,
-                                            gint        dest_width,
-                                            gint        dest_height,
-                                            gint        dest_rowstride,
-                                            gint        dest_bytes);
 
 
 #endif /* __GIMP_VIEW_RENDERER_H__ */
