@@ -4,9 +4,9 @@
  * gimpviewrendererimage.c
  * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,12 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -77,16 +77,20 @@ gimp_view_renderer_image_render (GimpViewRenderer *renderer,
     {
       gint      view_width;
       gint      view_height;
+      gdouble   xres;
+      gdouble   yres;
       gboolean  scaling_up;
       TempBuf  *render_buf = NULL;
 
-      gimp_viewable_calc_preview_size (image->width,
-                                       image->height,
+      gimp_image_get_resolution (image, &xres, &yres);
+
+      gimp_viewable_calc_preview_size (gimp_image_get_width  (image),
+                                       gimp_image_get_height (image),
                                        renderer->width,
                                        renderer->height,
                                        renderer->dot_for_dot,
-                                       image->xresolution,
-                                       image->yresolution,
+                                       xres,
+                                       yres,
                                        &view_width,
                                        &view_height,
                                        &scaling_up);
@@ -97,8 +101,8 @@ gimp_view_renderer_image_render (GimpViewRenderer *renderer,
 
           temp_buf = gimp_viewable_get_new_preview (renderer->viewable,
                                                     renderer->context,
-                                                    image->width,
-                                                    image->height);
+                                                    gimp_image_get_width  (image),
+                                                    gimp_image_get_height (image));
 
           if (temp_buf)
             {
@@ -141,10 +145,10 @@ gimp_view_renderer_image_render (GimpViewRenderer *renderer,
             component_index =
               gimp_image_get_component_index (image, rendererimage->channel);
 
-          gimp_view_renderer_render_buffer (renderer, render_buf,
-                                            component_index,
-                                            GIMP_VIEW_BG_CHECKS,
-                                            GIMP_VIEW_BG_WHITE);
+          gimp_view_renderer_render_surface (renderer, render_buf,
+                                             component_index,
+                                             GIMP_VIEW_BG_CHECKS,
+                                             GIMP_VIEW_BG_WHITE);
 
           temp_buf_free (render_buf);
 
