@@ -160,24 +160,34 @@ gimp_view_renderer_imagefile_get_icon (GimpImagefile *imagefile,
   GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen (screen);
   GdkPixbuf    *pixbuf     = NULL;
 
-  if (! gimp_object_get_name (GIMP_OBJECT (imagefile)))
+  if (! gimp_object_get_name (imagefile))
     return NULL;
 
   if (! pixbuf)
     {
-      GFile       *file;
-      GFileInfo   *file_info;
-      GtkIconInfo *info;
+      GFile     *file;
+      GFileInfo *file_info;
 
-      file = g_file_new_for_uri (gimp_object_get_name (GIMP_OBJECT (imagefile)));
+      file = g_file_new_for_uri (gimp_object_get_name (imagefile));
       file_info = g_file_query_info (file, "standard::icon", 0, NULL, NULL);
 
       if (file_info)
         {
           GIcon *icon = g_file_info_get_icon (file_info);
 
-          info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon, size, 0);
-          pixbuf = gtk_icon_info_load_icon (info, NULL);
+          if (icon)
+            {
+              GtkIconInfo *info;
+
+              info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon, size, 0);
+              pixbuf = gtk_icon_info_load_icon (info, NULL);
+            }
+          else
+            {
+#ifdef GIMP_UNSTABLE
+              g_printerr ("no icon for: %s\n", gimp_object_get_name (imagefile));
+#endif
+            }
 
           g_object_unref (file_info);
         }
