@@ -27,6 +27,8 @@
 
 #include "widgets-types.h"
 
+#include "core/gimpcontainer.h"
+
 #include "text/gimptextlayer.h"
 
 #include "gimpviewrendererlayer.h"
@@ -69,6 +71,13 @@ gimp_view_renderer_layer_render (GimpViewRenderer *renderer,
     {
       stock_id = gimp_viewable_get_stock_id (renderer->viewable);
     }
+  else
+    {
+      GimpContainer *children = gimp_viewable_get_children (renderer->viewable);
+
+      if (children && gimp_container_get_n_children (children) == 0)
+        stock_id = GTK_STOCK_DIRECTORY;
+    }
 
   if (stock_id)
     gimp_view_renderer_default_render_stock (renderer, widget, stock_id);
