@@ -20,22 +20,7 @@
 
 #include "config.h"
 
-#include <errno.h>
-#include <fcntl.h>
 #include <string.h>
-#include <sys/types.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <glib/gstdio.h>
-
-#include <glib.h>
-
-#ifdef G_OS_WIN32
-#include "libgimpbase/gimpwin32-io.h"
-#endif
 
 #include <gtk/gtk.h>
 
@@ -54,6 +39,9 @@
 
 #include "widgets-types.h"
 
+#include "gimpdialogfactory.h"
+#include "gimpdock.h"
+#include "gimpdockwindow.h"
 #include "gimperrordialog.h"
 #include "gimpwidgets-utils.h"
 
@@ -142,15 +130,19 @@ gimp_button_menu_position (GtkWidget       *button,
                            gint            *y)
 {
   GdkScreen      *screen;
+  GtkAllocation   button_allocation;
   GtkRequisition  menu_requisition;
   GdkRectangle    rect;
   gint            monitor;
 
-  g_return_if_fail (GTK_WIDGET_REALIZED (button));
+  g_return_if_fail (GTK_IS_WIDGET (button));
+  g_return_if_fail (gtk_widget_get_realized (button));
   g_return_if_fail (GTK_IS_MENU (menu));
   g_return_if_fail (x != NULL);
   g_return_if_fail (y != NULL);
 
+  gtk_widget_get_allocation (button, &button_allocation);
+
   if (gtk_widget_get_direction (button) == GTK_TEXT_DIR_RTL)
     {
       switch (position)
@@ -162,7 +154,16 @@ gimp_button_menu_position (GtkWidget       *button,
         }
     }
 
-  gdk_window_get_origin (gtk_widget_get_window (button), x, y);
+  *x = 0;
+  *y = 0;
+
+  if (! gtk_widget_get_has_window (button))
+    {
+      *x += button_allocation.x;
+      *y += button_allocation.y;
+    }
+
+  gdk_window_get_root_coords (gtk_widget_get_window (button), *x, *y, x, y);
 
   gtk_widget_size_request (GTK_WIDGET (menu), &menu_requisition);
 
@@ -173,21 +174,18 @@ gimp_button_menu_position (GtkWidget       *button,
 
   gtk_menu_set_screen (menu, screen);
 
-  if (GTK_WIDGET_NO_WINDOW (button))
-    *x += button->allocation.x;
-
   switch (position)
     {
     case GTK_POS_LEFT:
       *x -= menu_requisition.width;
       if (*x < rect.x)
-        *x += menu_requisition.width + button->allocation.width;
+        *x += menu_requisition.width + button_allocation.width;
       break;
 
     case GTK_POS_RIGHT:
-      *x += button->allocation.width;
+      *x += button_allocation.width;
       if (*x + menu_requisition.width > rect.x + rect.width)
-        *x -= button->allocation.width + menu_requisition.width;
+        *x -= button_allocation.width + menu_requisition.width;
       break;
 
     default:
@@ -195,10 +193,7 @@ gimp_button_menu_position (GtkWidget       *button,
       break;
     }
 
-  if (GTK_WIDGET_NO_WINDOW (button))
-    *y += button->allocation.y;
-
-  *y += button->allocation.height / 2;
+  *y += button_allocation.height / 2;
 
   if (*y + menu_requisition.height > rect.y + rect.height)
     *y -= menu_requisition.height;
@@ -434,6 +429,30 @@ gimp_get_icon_size (GtkWidget   *widget,
   return icon_size;
 }
 
+GimpTabStyle
+gimp_preview_tab_style_to_icon (GimpTabStyle tab_style)
+{
+  switch (tab_style)
+    {
+    case GIMP_TAB_STYLE_PREVIEW:
+      tab_style = GIMP_TAB_STYLE_ICON;
+      break;
+
+    case GIMP_TAB_STYLE_PREVIEW_NAME:
+      tab_style = GIMP_TAB_STYLE_ICON_NAME;
+      break;
+
+    case GIMP_TAB_STYLE_PREVIEW_BLURB:
+      tab_style = GIMP_TAB_STYLE_ICON_BLURB;
+      break;
+
+    default:
+      break;
+    }
+
+  return tab_style;
+}
+
 const gchar *
 gimp_get_mod_name_shift (void)
 {
@@ -822,12 +841,12 @@ gimp_window_get_native (GtkWindow *window)
 #endif
 
 #ifdef GDK_WINDOWING_WIN32
-  if (window && GTK_WIDGET_REALIZED (window))
+  if (window && gtk_widget_get_realized (GTK_WIDGET (window)))
     return (GdkNativeWindow) GDK_WINDOW_HWND (gtk_widget_get_window (GTK_WIDGET (window)));
 #endif
 
 #ifdef GDK_WINDOWING_X11
-  if (window && GTK_WIDGET_REALIZED (window))
+  if (window && gtk_widget_get_realized (GTK_WIDGET (window)))
     return GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (window)));
 #endif
 
@@ -838,7 +857,7 @@ static void
 gimp_window_transient_realized (GtkWidget *window,
                                 GdkWindow *parent)
 {
-  if (GTK_WIDGET_REALIZED (window))
+  if (gtk_widget_get_realized (window))
     gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
 }
 
@@ -863,7 +882,7 @@ gimp_window_set_transient_for (GtkWindow *window,
   if (! parent)
     return;
 
-  if (GTK_WIDGET_REALIZED (window))
+  if (gtk_widget_get_realized (GTK_WIDGET (window)))
     gdk_window_set_transient_for (gtk_widget_get_window (GTK_WIDGET (window)),
                                   parent);
 
@@ -875,125 +894,6 @@ gimp_window_set_transient_for (GtkWindow *window,
 #endif
 }
 
-gboolean
-gimp_text_buffer_load (GtkTextBuffer  *buffer,
-                       const gchar    *filename,
-                       GError        **error)
-{
-  FILE        *file;
-  gchar        buf[2048];
-  gint         remaining = 0;
-  GtkTextIter  iter;
-
-  g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  file = g_fopen (filename, "r");
-
-  if (! file)
-    {
-      g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
-      return FALSE;
-    }
-
-  gtk_text_buffer_set_text (buffer, "", 0);
-  gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
-
-  while (! feof (file))
-    {
-      const char *leftover;
-      gint        count;
-      gint        to_read = sizeof (buf) - remaining - 1;
-
-      count = fread (buf + remaining, 1, to_read, file);
-      buf[count + remaining] = '\0';
-
-      g_utf8_validate (buf, count + remaining, &leftover);
-
-      gtk_text_buffer_insert (buffer, &iter, buf, leftover - buf);
-      gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
-
-      remaining = (buf + remaining + count) - leftover;
-      g_memmove (buf, leftover, remaining);
-
-      if (remaining > 6 || count < to_read)
-        break;
-    }
-
-  if (remaining)
-    g_message (_("Invalid UTF-8 data in file '%s'."),
-               gimp_filename_to_utf8 (filename));
-
-  fclose (file);
-
-  return TRUE;
-}
-
-gboolean
-gimp_text_buffer_save (GtkTextBuffer  *buffer,
-                       const gchar    *filename,
-                       gboolean        selection_only,
-                       GError        **error)
-{
-  GtkTextIter  start_iter;
-  GtkTextIter  end_iter;
-  gint         fd;
-  gchar       *text_contents;
-
-  g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  fd = g_open (filename, O_WRONLY | O_CREAT | O_APPEND, 0666);
-
-  if (fd == -1)
-    {
-      g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
-      return FALSE;
-    }
-
-  if (selection_only)
-    gtk_text_buffer_get_selection_bounds (buffer, &start_iter, &end_iter);
-  else
-    gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
-
-  text_contents = gtk_text_buffer_get_text (buffer,
-                                            &start_iter, &end_iter, TRUE);
-
-  if (text_contents)
-    {
-      gint text_length = strlen (text_contents);
-
-      if (text_length > 0)
-        {
-          gint bytes_written;
-
-          bytes_written = write (fd, text_contents, text_length);
-
-          if (bytes_written != text_length)
-            {
-              g_free (text_contents);
-              close (fd);
-              g_set_error_literal (error, G_FILE_ERROR,
-                                   g_file_error_from_errno (errno),
-                                   g_strerror (errno));
-              return FALSE;
-            }
-        }
-
-      g_free (text_contents);
-    }
-
-  close (fd);
-
-  return TRUE;
-}
-
 void
 gimp_toggle_button_set_visible (GtkToggleButton *toggle,
                                 GtkWidget       *widget)
@@ -1001,10 +901,8 @@ gimp_toggle_button_set_visible (GtkToggleButton *toggle,
   g_return_if_fail (GTK_IS_TOGGLE_BUTTON (toggle));
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
-  if (gtk_toggle_button_get_active (toggle))
-    gtk_widget_show (widget);
-  else
-    gtk_widget_hide (widget);
+  gtk_widget_set_visible (widget,
+                          gtk_toggle_button_get_active (toggle));
 }
 
 static gboolean
@@ -1159,3 +1057,80 @@ gimp_pango_layout_set_weight (PangoLayout *layout,
   pango_layout_set_attributes (layout, attrs);
   pango_attr_list_unref (attrs);
 }
+
+/**
+ * gimp_highlight_widget:
+ * @widget:
+ * @highlight:
+ *
+ * Calls gtk_drag_highlight() on @widget if @highlight is %TRUE,
+ * calls gtk_drag_unhighlight() otherwise.
+ **/
+void
+gimp_highlight_widget (GtkWidget *widget,
+                       gboolean   highlight)
+{
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  if (highlight)
+    gtk_drag_highlight (widget);
+  else
+    gtk_drag_unhighlight (widget);
+}
+
+/**
+ * gimp_dock_with_window_new:
+ * @factory: a #GimpDialogFacotry
+ * @screen:  the #GdkScreen the dock window should appear on
+ * @toolbox: if %TRUE; gives a "gimp-toolbox-window" with a
+ *           "gimp-toolbox", "gimp-dock-window"+"gimp-dock"
+ *           otherwise
+ *
+ * Returns: the newly created #GimpDock with the #GimpDockWindow
+ **/
+GtkWidget *
+gimp_dock_with_window_new (GimpDialogFactory *factory,
+                           GdkScreen         *screen,
+                           gboolean           toolbox)
+{
+  GtkWidget     *dock_window = NULL;
+  GtkWidget     *dock        = NULL;
+  GimpUIManager *ui_manager  = NULL;
+
+  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (factory), NULL);
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+
+  /* Create a dock window to put the dock in. We need to create the
+   * dock window before the dock because the dock has a dependency to
+   * the ui manager in the dock window
+   */
+  dock_window =
+    gimp_dialog_factory_dialog_new (factory,
+                                    screen,
+                                    NULL /*ui_manager*/,
+                                    (toolbox ?
+                                     "gimp-toolbox-window" :
+                                     "gimp-dock-window"),
+                                    -1 /*view_size*/,
+                                    FALSE /*present*/);
+
+  /* Create the dock */
+  ui_manager = gimp_dock_window_get_ui_manager (GIMP_DOCK_WINDOW (dock_window));
+  dock       = gimp_dialog_factory_dialog_new (factory,
+                                               screen,
+                                               ui_manager,
+                                               (toolbox ?
+                                                "gimp-toolbox" :
+                                                "gimp-dock"),
+                                               -1 /*view_size*/,
+                                               FALSE /*present*/);
+  if (dock)
+    {
+      /* Put the dock in the dock window */
+      gimp_dock_window_add_dock (GIMP_DOCK_WINDOW (dock_window),
+                                 GIMP_DOCK (dock),
+                                 -1);
+    }
+
+  return dock;
+}
