@@ -83,28 +83,6 @@ typedef enum
 } GimpColorPickState;
 
 
-#define GIMP_TYPE_CURSOR_FORMAT (gimp_cursor_format_get_type ())
-
-GType gimp_cursor_format_get_type (void) G_GNUC_CONST;
-
-typedef enum
-{
-  GIMP_CURSOR_FORMAT_BITMAP, /*< desc="Black & white" >*/
-  GIMP_CURSOR_FORMAT_PIXBUF  /*< desc="Fancy"         >*/
-} GimpCursorFormat;
-
-
-#define GIMP_TYPE_HELP_BROWSER_TYPE (gimp_help_browser_type_get_type ())
-
-GType gimp_help_browser_type_get_type (void) G_GNUC_CONST;
-
-typedef enum
-{
-  GIMP_HELP_BROWSER_GIMP,        /*< desc="GIMP help browser" >*/
-  GIMP_HELP_BROWSER_WEB_BROWSER  /*< desc="Web browser"       >*/
-} GimpHelpBrowserType;
-
-
 #define GIMP_TYPE_HISTOGRAM_SCALE (gimp_histogram_scale_get_type ())
 
 GType gimp_histogram_scale_get_type (void) G_GNUC_CONST;
@@ -129,7 +107,9 @@ typedef enum
   GIMP_TAB_STYLE_ICON_NAME,     /*< desc="Icon & text"    >*/
   GIMP_TAB_STYLE_ICON_BLURB,    /*< desc="Icon & desc"    >*/
   GIMP_TAB_STYLE_PREVIEW_NAME,  /*< desc="Status & text"  >*/
-  GIMP_TAB_STYLE_PREVIEW_BLURB  /*< desc="Status & desc"  >*/
+  GIMP_TAB_STYLE_PREVIEW_BLURB, /*< desc="Status & desc"  >*/
+  GIMP_TAB_STYLE_UNDEFINED,     /*< desc="Undefined"      >*/
+  GIMP_TAB_STYLE_AUTOMATIC      /*< desc="Automatic"      >*/
 } GimpTabStyle;
 
 
@@ -144,18 +124,6 @@ typedef enum
 } GimpTagEntryMode;
 
 
-#define GIMP_TYPE_WINDOW_HINT (gimp_window_hint_get_type ())
-
-GType gimp_window_hint_get_type (void) G_GNUC_CONST;
-
-typedef enum
-{
-  GIMP_WINDOW_HINT_NORMAL,     /*< desc="Normal window"  >*/
-  GIMP_WINDOW_HINT_UTILITY,    /*< desc="Utility window" >*/
-  GIMP_WINDOW_HINT_KEEP_ABOVE  /*< desc="Keep above"     >*/
-} GimpWindowHint;
-
-
 /*
  * non-registered enums; register them if needed
  */
@@ -314,5 +282,12 @@ typedef enum  /*< skip >*/
   GIMP_FILE_CHOOSER_ACTION_EXPORT
 } GimpFileChooserAction;
 
+typedef enum  /*< skip >*/
+{
+  GIMP_DIALOGS_SHOWN,
+  GIMP_DIALOGS_HIDDEN_EXPLICITLY,  /* user used the Tab key to hide dialogs */
+  GIMP_DIALOGS_HIDDEN_WITH_DISPLAY /* dialogs are hidden with the display   */
+} GimpDialogsState;
+
 
 #endif /* __WIDGETS_ENUMS_H__ */
