@@ -37,13 +37,14 @@ typedef struct _GimpControllerWheel          GimpControllerWheel;
 /*  docks  */
 
 typedef struct _GimpDock                     GimpDock;
-typedef struct _GimpDockSeparator            GimpDockSeparator; /* not a dock */
-typedef struct _GimpImageDock                GimpImageDock;
+typedef struct _GimpDockColumns              GimpDockColumns;
+typedef struct _GimpDockWindow               GimpDockWindow;
 typedef struct _GimpMenuDock                 GimpMenuDock;
 typedef struct _GimpToolbox                  GimpToolbox;
 typedef struct _GimpDockbook                 GimpDockbook;
 typedef struct _GimpDockable                 GimpDockable;
 typedef struct _GimpDocked                   GimpDocked; /* dummy typedef */
+typedef struct _GimpPanedBox                 GimpPanedBox;
 
 
 /*  GimpEditor widgets  */
@@ -59,9 +60,10 @@ typedef struct _GimpToolOptionsEditor        GimpToolOptionsEditor;
 
 typedef struct _GimpDataEditor               GimpDataEditor;
 typedef struct _GimpBrushEditor              GimpBrushEditor;
+typedef struct _GimpDynamicsEditor           GimpDynamicsEditor;
 typedef struct _GimpGradientEditor           GimpGradientEditor;
 typedef struct _GimpPaletteEditor            GimpPaletteEditor;
-
+typedef struct _GimpToolPresetEditor         GimpToolPresetEditor;
 
 /*  GimpImageEditor widgets  */
 
@@ -81,6 +83,8 @@ typedef struct _GimpContainerBox             GimpContainerBox;
 typedef struct _GimpContainerComboBox        GimpContainerComboBox;
 typedef struct _GimpContainerEntry           GimpContainerEntry;
 typedef struct _GimpContainerGridView        GimpContainerGridView;
+typedef struct _GimpContainerIconView        GimpContainerIconView;
+typedef struct _GimpContainerTreeStore       GimpContainerTreeStore;
 typedef struct _GimpContainerTreeView        GimpContainerTreeView;
 typedef struct _GimpItemTreeView             GimpItemTreeView;
 typedef struct _GimpDrawableTreeView         GimpDrawableTreeView;
@@ -107,8 +111,9 @@ typedef struct _GimpToolEditor               GimpToolEditor;
 
 typedef struct _GimpDataFactoryView          GimpDataFactoryView;
 typedef struct _GimpBrushFactoryView         GimpBrushFactoryView;
+typedef struct _GimpDynamicsFactoryView      GimpDynamicsFactoryView;
 typedef struct _GimpPatternFactoryView       GimpPatternFactoryView;
-
+typedef struct _GimpToolPresetFactoryView    GimpToolPresetFactoryView;
 
 /*  menus  */
 
@@ -159,6 +164,9 @@ typedef struct _GimpControllerEditor         GimpControllerEditor;
 typedef struct _GimpControllerList           GimpControllerList;
 typedef struct _GimpCurveView                GimpCurveView;
 typedef struct _GimpDashEditor               GimpDashEditor;
+typedef struct _GimpDeviceEditor             GimpDeviceEditor;
+typedef struct _GimpDeviceInfoEditor         GimpDeviceInfoEditor;
+typedef struct _GimpDynamicsOutputEditor     GimpDynamicsOutputEditor;
 typedef struct _GimpFgBgEditor               GimpFgBgEditor;
 typedef struct _GimpFgBgView                 GimpFgBgView;
 typedef struct _GimpFileProcView             GimpFileProcView;
@@ -169,11 +177,13 @@ typedef struct _GimpHistogramBox             GimpHistogramBox;
 typedef struct _GimpHistogramView            GimpHistogramView;
 typedef struct _GimpImageCommentEditor       GimpImageCommentEditor;
 typedef struct _GimpImageParasiteView        GimpImageParasiteView;
-typedef struct _GimpImagePropView            GimpImagePropView;
 typedef struct _GimpImageProfileView         GimpImageProfileView;
-typedef struct _GimpLanguageStore            GimpLanguageStore;
+typedef struct _GimpImagePropView            GimpImagePropView;
+typedef struct _GimpLanguageComboBox         GimpLanguageComboBox;
 typedef struct _GimpLanguageEntry            GimpLanguageEntry;
+typedef struct _GimpLanguageStore            GimpLanguageStore;
 typedef struct _GimpMessageBox               GimpMessageBox;
+typedef struct _GimpOverlayBox               GimpOverlayBox;
 typedef struct _GimpProgressBox              GimpProgressBox;
 typedef struct _GimpScaleButton              GimpScaleButton;
 typedef struct _GimpSettingsBox              GimpSettingsBox;
@@ -183,7 +193,9 @@ typedef struct _GimpStrokeEditor             GimpStrokeEditor;
 typedef struct _GimpTagEntry                 GimpTagEntry;
 typedef struct _GimpTagPopup                 GimpTagPopup;
 typedef struct _GimpTemplateEditor           GimpTemplateEditor;
+typedef struct _GimpTextStyleEditor          GimpTextStyleEditor;
 typedef struct _GimpThumbBox                 GimpThumbBox;
+typedef struct _GimpTranslationStore         GimpTranslationStore;
 typedef struct _GimpUnitStore                GimpUnitStore;
 typedef struct _GimpUnitComboBox             GimpUnitComboBox;
 typedef struct _GimpWindow                   GimpWindow;
@@ -216,15 +228,18 @@ typedef struct _GimpCellRendererDashes       GimpCellRendererDashes;
 typedef struct _GimpCellRendererViewable     GimpCellRendererViewable;
 
 
-/*  misc utilities & constructors  */
+/*  misc objects  */
 
 typedef struct _GimpDialogFactory            GimpDialogFactory;
+typedef struct _GimpTextBuffer               GimpTextBuffer;
+typedef struct _GimpUIConfigurer             GimpUIConfigurer;
 
 
 /*  session management objects and structs  */
 
 typedef struct _GimpSessionInfo              GimpSessionInfo;
 typedef struct _GimpSessionInfoBook          GimpSessionInfoBook;
+typedef struct _GimpSessionInfoDock          GimpSessionInfoDock;
 typedef struct _GimpSessionInfoDockable      GimpSessionInfoDockable;
 typedef struct _GimpSessionInfoAux           GimpSessionInfoAux;
 
@@ -243,17 +258,20 @@ typedef struct _GimpDialogFactoryEntry       GimpDialogFactoryEntry;
 
 /*  function types  */
 
-typedef void (* GimpActionGroupSetupFunc)  (GimpActionGroup *group);
-typedef void (* GimpActionGroupUpdateFunc) (GimpActionGroup *group,
-                                            gpointer         data);
+typedef void     (* GimpActionGroupSetupFunc)     (GimpActionGroup *group);
+typedef void     (* GimpActionGroupUpdateFunc)    (GimpActionGroup *group,
+                                                   gpointer         data);
 
-typedef void (* GimpUIManagerSetupFunc)    (GimpUIManager   *manager,
-                                            const gchar     *ui_path);
+typedef void     (* GimpUIManagerSetupFunc)       (GimpUIManager   *manager,
+                                                   const gchar     *ui_path);
 
-typedef void (* GimpMenuPositionFunc)      (GtkMenu         *menu,
-                                            gint            *x,
-                                            gint            *y,
-                                            gpointer         data);
+typedef void     (* GimpMenuPositionFunc)         (GtkMenu         *menu,
+                                                   gint            *x,
+                                                   gint            *y,
+                                                   gpointer         data);
+typedef gboolean (* GimpPanedBoxDroppedFunc)      (GtkWidget         *source,
+                                                   gint               insert_index,
+                                                   gpointer           data);
 
 
 #endif /* __WIDGETS_TYPES_H__ */
