@@ -41,13 +41,13 @@
 #include "core/gimpimage-colormap.h"
 #include "core/gimpimage-grid.h"
 #include "core/gimpimage-guides.h"
+#include "core/gimpimage-private.h"
 #include "core/gimpimage-sample-points.h"
 #include "core/gimplayer.h"
 #include "core/gimplayermask.h"
 #include "core/gimpparasitelist.h"
 #include "core/gimpprogress.h"
 #include "core/gimpsamplepoint.h"
-#include "core/gimpunit.h"
 
 #include "text/gimptextlayer.h"
 #include "text/gimptextlayer-xcf.h"
@@ -213,20 +213,19 @@ xcf_save_image (XcfInfo    *info,
                 GimpImage  *image,
                 GError    **error)
 {
-  GimpLayer   *layer;
-  GimpChannel *channel;
-  GList       *list;
-  guint32      saved_pos;
-  guint32      offset;
-  guint32      value;
-  guint        n_layers;
-  guint        n_channels;
-  guint        progress = 0;
-  guint        max_progress;
-  gboolean     have_selection;
-  gint         t1, t2, t3, t4;
-  gchar        version_tag[16];
-  GError      *tmp_error = NULL;
+  GList   *all_layers;
+  GList   *all_channels;
+  GList   *list;
+  guint32  saved_pos;
+  guint32  offset;
+  guint32  value;
+  guint    n_layers;
+  guint    n_channels;
+  guint    progress = 0;
+  guint    max_progress;
+  gint     t1, t2, t3, t4;
+  gchar    version_tag[16];
+  GError  *tmp_error = NULL;
 
   /* write out the tag information for the image */
   if (info->file_version > 0)
@@ -251,16 +250,20 @@ xcf_save_image (XcfInfo    *info,
   xcf_write_int32_check_error (info, &value, 1);
 
   /* determine the number of layers and channels in the image */
-  n_layers   = (guint) gimp_container_get_n_children (image->layers);
-  n_channels = (guint) gimp_container_get_n_children (image->channels);
-
-  max_progress = 1 + n_layers + n_channels;
+  all_layers   = gimp_image_get_layer_list (image);
+  all_channels = gimp_image_get_channel_list (image);
 
   /* check and see if we have to save out the selection */
-  have_selection = gimp_channel_bounds (gimp_image_get_mask (image),
-                                        &t1, &t2, &t3, &t4);
-  if (have_selection)
-    n_channels += 1;
+  if (gimp_channel_bounds (gimp_image_get_mask (image),
+                           &t1, &t2, &t3, &t4))
+    {
+      all_channels = g_list_append (all_channels, gimp_image_get_mask (image));
+    }
+
+  n_layers   = (guint) g_list_length (all_layers);
+  n_channels = (guint) g_list_length (all_channels);
+
+  max_progress = 1 + n_layers + n_channels;
 
   /* write the property information for the image.
    */
@@ -279,11 +282,9 @@ xcf_save_image (XcfInfo    *info,
                                  info->cp + (n_layers + n_channels + 2) * 4,
                                  error));
 
-  for (list = gimp_image_get_layer_iter (image);
-       list;
-       list = g_list_next (list))
+  for (list = all_layers; list; list = g_list_next (list))
     {
-      layer = list->data;
+      GimpLayer *layer = list->data;
 
       /* save the start offset of where we are writing
        *  out the next layer.
@@ -321,21 +322,9 @@ xcf_save_image (XcfInfo    *info,
   saved_pos = info->cp;
   xcf_check_error (xcf_seek_end (info, error));
 
-  list = gimp_image_get_channel_iter (image);
-
-  while (list || have_selection)
+  for (list = all_channels; list; list = g_list_next (list))
     {
-      if (list)
-        {
-          channel = list->data;
-
-          list = g_list_next (list);
-        }
-      else
-        {
-          channel = gimp_image_get_mask (image);
-          have_selection = FALSE;
-        }
+      GimpChannel *channel = list->data;
 
       /* save the start offset of where we are writing
        *  out the next channel.
@@ -364,6 +353,9 @@ xcf_save_image (XcfInfo    *info,
       xcf_check_error (xcf_seek_end (info, error));
     }
 
+  g_list_free (all_layers);
+  g_list_free (all_channels);
+
   /* write out a '0' offset position to indicate the end
    *  of the channel offsets.
    */
@@ -380,10 +372,11 @@ xcf_save_image_props (XcfInfo    *info,
                       GimpImage  *image,
                       GError    **error)
 {
-  GimpParasite *parasite = NULL;
-  GimpUnit      unit     = gimp_image_get_unit (image);
-  gdouble       xres;
-  gdouble       yres;
+  GimpImagePrivate *private  = GIMP_IMAGE_GET_PRIVATE (image);
+  GimpParasite     *parasite = NULL;
+  GimpUnit          unit     = gimp_image_get_unit (image);
+  gdouble           xres;
+  gdouble           yres;
 
   gimp_image_get_resolution (image, &xres, &yres);
 
@@ -411,10 +404,10 @@ xcf_save_image_props (XcfInfo    *info,
   xcf_check_error (xcf_save_prop (info, image, PROP_TATTOO, error,
                                   gimp_image_get_tattoo_state (image)));
 
-  if (unit < _gimp_unit_get_number_of_built_in_units (image->gimp))
+  if (unit < gimp_unit_get_number_of_built_in_units ())
     xcf_check_error (xcf_save_prop (info, image, PROP_UNIT, error, unit));
 
-  if (gimp_container_get_n_children (image->vectors) > 0)
+  if (gimp_container_get_n_children (gimp_image_get_vectors (image)) > 0)
     {
       if (gimp_vectors_compat_is_compatible (image))
         xcf_check_error (xcf_save_prop (info, image, PROP_PATHS, error));
@@ -422,26 +415,26 @@ xcf_save_image_props (XcfInfo    *info,
         xcf_check_error (xcf_save_prop (info, image, PROP_VECTORS, error));
     }
 
-  if (unit >= _gimp_unit_get_number_of_built_in_units (image->gimp))
+  if (unit >= gimp_unit_get_number_of_built_in_units ())
     xcf_check_error (xcf_save_prop (info, image, PROP_USER_UNIT, error, unit));
 
-  if (GIMP_IS_GRID (image->grid))
+  if (gimp_image_get_grid (image))
     {
       GimpGrid *grid = gimp_image_get_grid (image);
 
       parasite = gimp_grid_to_parasite (grid);
-      gimp_parasite_list_add (GIMP_IMAGE (image)->parasites, parasite);
+      gimp_parasite_list_add (private->parasites, parasite);
     }
 
-  if (gimp_parasite_list_length (GIMP_IMAGE (image)->parasites) > 0)
+  if (gimp_parasite_list_length (private->parasites) > 0)
     {
       xcf_check_error (xcf_save_prop (info, image, PROP_PARASITES, error,
-                                      GIMP_IMAGE (image)->parasites));
+                                      private->parasites));
     }
 
   if (parasite)
     {
-      gimp_parasite_list_remove (GIMP_IMAGE (image)->parasites,
+      gimp_parasite_list_remove (private->parasites,
                                  gimp_parasite_name (parasite));
       gimp_parasite_free (parasite);
     }
@@ -461,6 +454,19 @@ xcf_save_layer_props (XcfInfo    *info,
   gint          offset_x;
   gint          offset_y;
 
+  if (gimp_viewable_get_children (GIMP_VIEWABLE (layer)))
+    xcf_check_error (xcf_save_prop (info, image, PROP_GROUP_ITEM, error));
+
+  if (gimp_viewable_get_parent (GIMP_VIEWABLE (layer)))
+    {
+      GList *path;
+
+      path = gimp_item_get_path (GIMP_ITEM (layer));
+      xcf_check_error (xcf_save_prop (info, image, PROP_ITEM_PATH, error,
+                                      path));
+      g_list_free (path);
+    }
+
   if (layer == gimp_image_get_active_layer (image))
     xcf_check_error (xcf_save_prop (info, image, PROP_ACTIVE_LAYER, error));
 
@@ -477,6 +483,8 @@ xcf_save_layer_props (XcfInfo    *info,
                                   gimp_item_get_visible (GIMP_ITEM (layer))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LINKED, error,
                                   gimp_item_get_linked (GIMP_ITEM (layer))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_CONTENT, error,
+                                  gimp_item_get_lock_content (GIMP_ITEM (layer))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_ALPHA, error,
                                   gimp_layer_get_lock_alpha (layer)));
 
@@ -561,6 +569,8 @@ xcf_save_channel_props (XcfInfo      *info,
                                   gimp_item_get_visible (GIMP_ITEM (channel))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LINKED, error,
                                   gimp_item_get_linked (GIMP_ITEM (channel))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_CONTENT, error,
+                                  gimp_item_get_lock_content (GIMP_ITEM (channel))));
   xcf_check_error (xcf_save_prop (info, image, PROP_SHOW_MASKED, error,
                                   gimp_channel_get_show_masked (channel)));
 
@@ -621,6 +631,7 @@ xcf_save_prop (XcfInfo    *info,
     case PROP_ACTIVE_LAYER:
     case PROP_ACTIVE_CHANNEL:
     case PROP_SELECTION:
+    case PROP_GROUP_ITEM:
       size = 0;
 
       xcf_write_prop_type_check_error (info, prop_type);
@@ -697,6 +708,19 @@ xcf_save_prop (XcfInfo    *info,
       }
       break;
 
+    case PROP_LOCK_CONTENT:
+      {
+        guint32 lock_content;
+
+        lock_content = va_arg (args, guint32);
+        size = 4;
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+        xcf_write_int32_check_error (info, &lock_content, 1);
+      }
+      break;
+
     case PROP_LOCK_ALPHA:
       {
         guint32 lock_alpha;
@@ -994,13 +1018,13 @@ xcf_save_prop (XcfInfo    *info,
         unit = va_arg (args, guint32);
 
         /* write the entire unit definition */
-        unit_strings[0] = _gimp_unit_get_identifier (image->gimp, unit);
-        factor          = _gimp_unit_get_factor (image->gimp, unit);
-        digits          = _gimp_unit_get_digits (image->gimp, unit);
-        unit_strings[1] = _gimp_unit_get_symbol (image->gimp, unit);
-        unit_strings[2] = _gimp_unit_get_abbreviation (image->gimp, unit);
-        unit_strings[3] = _gimp_unit_get_singular (image->gimp, unit);
-        unit_strings[4] = _gimp_unit_get_plural (image->gimp, unit);
+        unit_strings[0] = gimp_unit_get_identifier (unit);
+        factor          = gimp_unit_get_factor (unit);
+        digits          = gimp_unit_get_digits (unit);
+        unit_strings[1] = gimp_unit_get_symbol (unit);
+        unit_strings[2] = gimp_unit_get_abbreviation (unit);
+        unit_strings[3] = gimp_unit_get_singular (unit);
+        unit_strings[4] = gimp_unit_get_plural (unit);
 
         size =
           2 * 4 +
@@ -1062,6 +1086,27 @@ xcf_save_prop (XcfInfo    *info,
         xcf_write_int32_check_error (info, &flags, 1);
       }
       break;
+
+    case PROP_ITEM_PATH:
+      {
+        GList *path;
+
+        path = va_arg (args, GList *);
+        size = 4 * g_list_length (path);
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+
+        while (path)
+          {
+            guint32 index = GPOINTER_TO_UINT (path->data);
+
+            xcf_write_int32_check_error (info, &index, 1);
+
+            path = g_list_next (path);
+          }
+      }
+      break;
     }
 
   va_end (args);
@@ -1103,18 +1148,18 @@ xcf_save_layer (XcfInfo    *info,
   xcf_write_int32_check_error (info, &value, 1);
 
   /* write out the layers name */
-  string = gimp_object_get_name (GIMP_OBJECT (layer));
+  string = gimp_object_get_name (layer);
   xcf_write_string_check_error (info, (gchar **) &string, 1);
 
   /* write out the layer properties */
   xcf_save_layer_props (info, image, layer, error);
 
-  /* save the current position which is where the hierarchy offset
+  /*  save the current position which is where the hierarchy offset
    *  will be stored.
    */
   saved_pos = info->cp;
 
-  /* write out the layer tile hierarchy */
+  /*  write out the layer tile hierarchy  */
   xcf_check_error (xcf_seek_pos (info, info->cp + 8, error));
   offset = info->cp;
 
@@ -1124,6 +1169,10 @@ xcf_save_layer (XcfInfo    *info,
 
   xcf_check_error (xcf_seek_pos (info, saved_pos, error));
   xcf_write_int32_check_error (info, &offset, 1);
+
+  /*  save the current position which is where the layer mask offset
+   *  will be stored.
+   */
   saved_pos = info->cp;
 
   /* write out the layer mask */
@@ -1177,7 +1226,7 @@ xcf_save_channel (XcfInfo      *info,
   xcf_write_int32_check_error (info, &value, 1);
 
   /* write out the channels name */
-  string = gimp_object_get_name (GIMP_OBJECT (channel));
+  string = gimp_object_get_name (channel);
   xcf_write_string_check_error (info, (gchar **) &string, 1);
 
   /* write out the channel properties */
@@ -1604,12 +1653,12 @@ xcf_save_old_paths (XcfInfo    *info,
    * then each path:-
    */
 
-  num_paths = gimp_container_get_n_children (image->vectors);
+  num_paths = gimp_container_get_n_children (gimp_image_get_vectors (image));
 
   active_vectors = gimp_image_get_active_vectors (image);
 
   if (active_vectors)
-    active_index = gimp_container_get_child_index (image->vectors,
+    active_index = gimp_container_get_child_index (gimp_image_get_vectors (image),
                                                    GIMP_OBJECT (active_vectors));
 
   xcf_write_int32_check_error (info, &active_index, 1);
@@ -1652,7 +1701,7 @@ xcf_save_old_paths (XcfInfo    *info,
        * we already saved the number of paths and I wont start seeking
        * around to fix that cruft  */
 
-      name     = (gchar *) gimp_object_get_name (GIMP_OBJECT (vectors));
+      name     = (gchar *) gimp_object_get_name (vectors);
       locked   = gimp_item_get_linked (GIMP_ITEM (vectors));
       state    = closed ? 4 : 2;  /* EDIT : ADD  (editing state, 1.2 compat) */
       version  = 3;
@@ -1718,10 +1767,10 @@ xcf_save_vectors (XcfInfo    *info,
   active_vectors = gimp_image_get_active_vectors (image);
 
   if (active_vectors)
-    active_index = gimp_container_get_child_index (image->vectors,
+    active_index = gimp_container_get_child_index (gimp_image_get_vectors (image),
                                                    GIMP_OBJECT (active_vectors));
 
-  num_paths = gimp_container_get_n_children (image->vectors);
+  num_paths = gimp_container_get_n_children (gimp_image_get_vectors (image));
 
   xcf_write_int32_check_error (info, &version,      1);
   xcf_write_int32_check_error (info, &active_index, 1);
@@ -1754,7 +1803,7 @@ xcf_save_vectors (XcfInfo    *info,
 
       parasites = GIMP_ITEM (vectors)->parasites;
 
-      name          = gimp_object_get_name (GIMP_OBJECT (vectors));
+      name          = gimp_object_get_name (vectors);
       visible       = gimp_item_get_visible (GIMP_ITEM (vectors));
       linked        = gimp_item_get_linked (GIMP_ITEM (vectors));
       tattoo        = gimp_item_get_tattoo (GIMP_ITEM (vectors));
