@@ -3,10 +3,10 @@
  *
  * gimpdrawable.h
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_DRAWABLE_H__
@@ -44,11 +43,11 @@ GimpDrawable * gimp_drawable_get                    (gint32         drawable_ID)
 void           gimp_drawable_detach                 (GimpDrawable  *drawable);
 void           gimp_drawable_flush                  (GimpDrawable  *drawable);
 GimpTile     * gimp_drawable_get_tile               (GimpDrawable  *drawable,
-                                                     gint           shadow,
+                                                     gboolean       shadow,
                                                      gint           row,
                                                      gint           col);
 GimpTile     * gimp_drawable_get_tile2              (GimpDrawable  *drawable,
-                                                     gint           shadow,
+                                                     gboolean       shadow,
                                                      gint           x,
                                                      gint           y);
 
@@ -69,7 +68,7 @@ guchar       * gimp_drawable_get_sub_thumbnail_data (gint32         drawable_ID,
                                                      gint          *dest_height,
                                                      gint          *bpp);
 
-void           gimp_drawable_attach_new_parasite    (gint32         drawable_ID,
+gboolean       gimp_drawable_attach_new_parasite    (gint32         drawable_ID,
                                                      const gchar   *name,
                                                      gint           flags,
                                                      gint           size,
