@@ -4,10 +4,10 @@
  * gimpexport.c
  * Copyright (C) 1999-2004 Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -207,6 +206,13 @@ export_add_alpha (gint32  image_ID,
   g_free (layers);
 }
 
+static void
+export_void (gint32  image_ID,
+             gint32 *drawable_ID)
+{
+  /* do nothing */
+}
+
 
 /* a set of predefined actions */
 
@@ -351,6 +357,27 @@ static ExportAction export_action_add_alpha =
 };
 
 
+static ExportFunc
+export_action_get_func (const ExportAction *action)
+{
+  if (action->choice == 0 && action->default_action)
+    return action->default_action;
+
+  if (action->choice == 1 && action->alt_action)
+    return action->alt_action;
+
+  return export_void;
+}
+
+static void
+export_action_perform (const ExportAction *action,
+                       gint32              image_ID,
+                       gint32             *drawable_ID)
+{
+  export_action_get_func (action) (image_ID, drawable_ID);
+}
+
+
 /* dialog functions */
 
 static void
@@ -399,7 +426,8 @@ confirm_save_dialog (const gchar *message,
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
   hbox = gtk_hbox_new (FALSE, 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     hbox);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
   gtk_widget_show (hbox);
 
@@ -481,7 +509,8 @@ export_dialog (GSList      *actions,
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
   hbox = gtk_hbox_new (FALSE, 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     hbox);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
   gtk_widget_show (hbox);
 
@@ -628,6 +657,10 @@ export_dialog (GSList      *actions,
  * user chooses Cancel, GIMP_EXPORT_CANCEL is returned and the
  * save_plugin should quit itself with status #GIMP_PDB_CANCEL.
  *
+ * If @format_name is NULL, no dialogs will be shown and this function
+ * will behave as if the user clicked on the 'Export' button, if a
+ * dialog would have been shown.
+ *
  * Returns: An enum of #GimpExportReturn describing the user_action.
  **/
 GimpExportReturn
@@ -637,7 +670,6 @@ gimp_export_image (gint32                 *image_ID,
                    GimpExportCapabilities  capabilities)
 {
   GSList            *actions = NULL;
-  GSList            *list;
   GimpImageBaseType  type;
   gint32             i;
   gint32             n_layers;
@@ -879,6 +911,8 @@ gimp_export_image (gint32                 *image_ID,
 
   if (retval == GIMP_EXPORT_EXPORT)
     {
+      GSList *list;
+
       *image_ID = gimp_image_duplicate (*image_ID);
       *drawable_ID = gimp_image_get_active_layer (*image_ID);
 
@@ -886,12 +920,7 @@ gimp_export_image (gint32                 *image_ID,
 
       for (list = actions; list; list = list->next)
         {
-          ExportAction *action = list->data;
-
-          if (action->choice == 0 && action->default_action)
-            action->default_action (*image_ID, drawable_ID);
-          else if (action->choice == 1 && action->alt_action)
-            action->alt_action (*image_ID, drawable_ID);
+          export_action_perform (list->data, *image_ID, drawable_ID);
         }
     }
 
@@ -899,3 +928,74 @@ gimp_export_image (gint32                 *image_ID,
 
   return retval;
 }
+
+/**
+ * gimp_export_dialog_new:
+ * @format_name: The short name of the image_format (e.g. JPEG or PNG).
+ * @role:        The dialog's @role which will be set with
+ *               gtk_window_set_role().
+ * @help_id:     The GIMP help id.
+ *
+ * Creates a new export dialog. All file plug-ins should use this
+ * dialog to get a consistent look on the export dialogs. Use
+ * gimp_export_dialog_get_content_area() to get a #GtkVBox to be
+ * filled with export options. The export dialog is a wrapped
+ * #GimpDialog.
+ *
+ * The dialog response when the user clicks on the Export button is
+ * %GTK_RESPONSE_OK, and when the Cancel button is clicked it is
+ * %GTK_RESPONSE_CANCEL.
+ *
+ * Returns: The new export dialog.
+ *
+ * Since: GIMP 2.8
+ **/
+GtkWidget *
+gimp_export_dialog_new (const gchar *format_name,
+                        const gchar *role,
+                        const gchar *help_id)
+{
+  GtkWidget *dialog = NULL;
+  GtkWidget *button = NULL;
+  gchar     *title  = g_strconcat (_("Export Image as "), format_name, NULL);
+
+  dialog = gimp_dialog_new (title, role,
+                            NULL, 0,
+                            gimp_standard_help_func, help_id,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            NULL);
+
+  button = gimp_dialog_add_button (GIMP_DIALOG (dialog),
+                                   _("_Export"), GTK_RESPONSE_OK);
+  gtk_button_set_image (GTK_BUTTON (button),
+                        gtk_image_new_from_stock (GTK_STOCK_SAVE,
+                                                  GTK_ICON_SIZE_BUTTON));
+
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
+
+  g_free (title);
+
+  return dialog;
+}
+
+/**
+ * gimp_export_dialog_get_content_area:
+ * @dialog: A dialog created with gimp_export_dialog_new()
+ *
+ * Returns the #GtkVBox of the passed export dialog to be filled with
+ * export options.
+ *
+ * Returns: The #GtkVBox to fill with export options.
+ *
+ * Since: GIMP 2.8
+ **/
+GtkWidget *
+gimp_export_dialog_get_content_area (GtkWidget *dialog)
+{
+  return gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+}
