@@ -3,10 +3,10 @@
  *
  * gimpmenu.c
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -484,12 +483,13 @@ static void
 gimp_menu_callback (GtkWidget *widget,
                     gpointer   any_ID)
 {
-  GimpMenuCallback callback;
-  gpointer         callback_data;
+  GtkWidget        *parent = gtk_widget_get_parent (widget);
+  GimpMenuCallback  callback;
+  gpointer          callback_data;
 
-  callback = (GimpMenuCallback) g_object_get_data (G_OBJECT (widget->parent),
+  callback = (GimpMenuCallback) g_object_get_data (G_OBJECT (parent),
                                                    "gimp-menu-callback");
-  callback_data = g_object_get_data (G_OBJECT (widget->parent),
+  callback_data = g_object_get_data (G_OBJECT (parent),
                                      "gimp-menu-callback-data");
 
   (* callback) (GPOINTER_TO_INT (any_ID), callback_data);
