@@ -3,10 +3,10 @@
  *
  * gimppatterns.c
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -24,14 +23,15 @@
 #include <string.h>
 
 #include "gimp.h"
+#undef GIMP_DISABLE_DEPRECATED
+#undef __GIMP_PATTERNS_H__
+#include "gimppatterns.h"
 
 /**
  * gimp_patterns_set_pattern:
  * @name: The pattern name.
  *
- * Set the specified pattern as the active pattern.
- *
- * This procedure is deprecated! Use 'gimp_context_set_pattern' instead.
+ * This procedure is deprecated! Use gimp_context_set_pattern() instead.
  *
  * Returns: TRUE on success.
  */
