@@ -4,10 +4,10 @@
  * Datafiles module copyight (C) 1996 Federico Mena Quintero
  * federico@nuclecu.unam.mx
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_DATAFILES_H__
@@ -25,10 +24,13 @@
 
 #include <time.h>
 
+G_BEGIN_DECLS
+
 
 struct _GimpDatafileData
 {
   const gchar *filename;
+  const gchar *dirname;
   const gchar *basename;
 
   time_t       atime;
@@ -38,12 +40,14 @@ struct _GimpDatafileData
 
 
 gboolean   gimp_datafiles_check_extension  (const gchar            *filename,
-					    const gchar            *extension);
+                                            const gchar            *extension);
 
 void       gimp_datafiles_read_directories (const gchar            *path_str,
-					    GFileTest               flags,
-					    GimpDatafileLoaderFunc  loader_func,
-					    gpointer                user_data);
+                                            GFileTest               flags,
+                                            GimpDatafileLoaderFunc  loader_func,
+                                            gpointer                user_data);
 
 
+G_END_DECLS
+
 #endif  /*  __GIMP_DATAFILES_H__ */
