@@ -4,10 +4,10 @@
  * Object properties deserialization routines
  * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -81,6 +80,7 @@ static GTokenType  gimp_config_deserialize_value_array (GValue     *value,
 static GTokenType  gimp_config_deserialize_any         (GValue     *value,
                                                         GParamSpec *prop_spec,
                                                         GScanner   *scanner);
+static GTokenType  gimp_config_skip_unknown_property   (GScanner   *scanner);
 
 static inline gboolean  scanner_string_utf8_valid (GScanner    *scanner,
                                                    const gchar *token_name);
@@ -122,15 +122,15 @@ gimp_config_deserialize_properties (GimpConfig *config,
   guint          i;
   guint          scope_id;
   guint          old_scope_id;
-  GTokenType         token;
-  GTokenType         next;
+  GTokenType     token;
+  GTokenType     next;
 
   g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
 
   klass = G_OBJECT_GET_CLASS (config);
   property_specs = g_object_class_list_properties (klass, &n_property_specs);
 
-  if (!property_specs)
+  if (! property_specs)
     return TRUE;
 
   scope_id = g_type_qname (G_TYPE_FROM_INSTANCE (config));
@@ -157,8 +157,12 @@ gimp_config_deserialize_properties (GimpConfig *config,
     {
       next = g_scanner_peek_next_token (scanner);
 
-      if (next != token)
-        break;
+      if (G_UNLIKELY (next != token &&
+                      ! (token == G_TOKEN_SYMBOL &&
+                         next  == G_TOKEN_IDENTIFIER)))
+        {
+          break;
+        }
 
       token = g_scanner_get_next_token (scanner);
 
@@ -168,6 +172,10 @@ gimp_config_deserialize_properties (GimpConfig *config,
           token = G_TOKEN_SYMBOL;
           break;
 
+        case G_TOKEN_IDENTIFIER:
+          token = gimp_config_skip_unknown_property (scanner);
+          break;
+
         case G_TOKEN_SYMBOL:
           token = gimp_config_deserialize_property (config,
                                                     scanner, nest_level);
@@ -750,3 +758,37 @@ gimp_config_deserialize_any (GValue     *value,
 
   return G_TOKEN_RIGHT_PAREN;
 }
+
+static GTokenType
+gimp_config_skip_unknown_property (GScanner *scanner)
+{
+  gint open_paren = 0;
+
+  while (TRUE)
+    {
+      GTokenType token = g_scanner_peek_next_token (scanner);
+
+      switch (token)
+        {
+        case G_TOKEN_LEFT_PAREN:
+          open_paren++;
+          g_scanner_get_next_token (scanner);
+          break;
+
+        case G_TOKEN_RIGHT_PAREN:
+          if (open_paren == 0)
+            return token;
+
+          open_paren--;
+          g_scanner_get_next_token (scanner);
+          break;
+
+        case G_TOKEN_EOF:
+          return token;
+
+        default:
+          g_scanner_get_next_token (scanner);
+          break;
+        }
+    }
+}
