@@ -4,10 +4,10 @@
  * gimpconfig-path.h
  * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_CONFIG_PATH_H__
@@ -74,11 +73,11 @@ GimpConfigPathType  gimp_param_spec_config_path_type (GParamSpec   *pspec);
 
 gchar             * gimp_config_path_expand          (const gchar  *path,
                                                       gboolean      recode,
-                                                      GError      **error);
+                                                      GError      **error) G_GNUC_MALLOC;
 
-gchar             * gimp_config_build_data_path      (const gchar  *name);
-gchar             * gimp_config_build_writable_path  (const gchar  *name);
-gchar             * gimp_config_build_plug_in_path   (const gchar  *name);
+gchar             * gimp_config_build_data_path      (const gchar  *name) G_GNUC_MALLOC;
+gchar             * gimp_config_build_writable_path  (const gchar  *name) G_GNUC_MALLOC;
+gchar             * gimp_config_build_plug_in_path   (const gchar  *name) G_GNUC_MALLOC;
 
 
 G_END_DECLS
