@@ -4,10 +4,10 @@
  * Object properties serialization routines
  * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -308,6 +307,34 @@ gimp_config_serialize_property (GimpConfig       *config,
   return success;
 }
 
+/**
+ * gimp_config_serialize_property_by_name:
+ * @config:    a #GimpConfig.
+ * @prop_name: the property's name.
+ * @writer:    a #GimpConfigWriter.
+ *
+ * This function serializes a single object property to the @writer.
+ *
+ * Returns: %TRUE if serialization succeeded, %FALSE otherwise
+ *
+ * Since: GIMP 2.6
+ **/
+gboolean
+gimp_config_serialize_property_by_name (GimpConfig       *config,
+                                        const gchar      *prop_name,
+                                        GimpConfigWriter *writer)
+{
+  GParamSpec *pspec;
+
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (config),
+                                        prop_name);
+
+  if (! pspec)
+    return FALSE;
+
+  return gimp_config_serialize_property (config, pspec, writer);
+}
+
 /**
  * gimp_config_serialize_value:
  * @value: a #GValue.
