@@ -4,10 +4,10 @@
  * GimpConfigWriter
  * Copyright (C) 2003  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -567,9 +566,9 @@ gimp_config_writer_linefeed (GimpConfigWriter *writer)
   if (writer->buffer->len == 0 && !writer->comment)
     {
       if (write (writer->fd, "\n", 1) < 0)
-        g_set_error (&writer->error, GIMP_CONFIG_ERROR, GIMP_CONFIG_ERROR_WRITE,
-                     "%s",
-                     g_strerror (errno));
+        g_set_error_literal (&writer->error,
+			     GIMP_CONFIG_ERROR, GIMP_CONFIG_ERROR_WRITE,
+			     g_strerror (errno));
     }
   else
     {
