@@ -1,10 +1,10 @@
 /* LIBGIMP - The GIMP Library
  * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +12,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -312,33 +311,6 @@ gimp_module_db_refresh (GimpModuleDB *db,
                                    db);
 }
 
-/* name must be of the form lib*.so (Unix) or *.dll (Win32) */
-static gboolean
-valid_module_name (const gchar *filename)
-{
-  gchar *basename = g_path_get_basename (filename);
-
-#if !defined(G_OS_WIN32) && !defined(G_WITH_CYGWIN)
-  if (strncmp (basename, "lib", 3))
-    goto no_module;
-
-  if (! gimp_datafiles_check_extension (basename, ".so"))
-    goto no_module;
-#else
-  if (! gimp_datafiles_check_extension (basename, ".dll"))
-    goto no_module;
-#endif
-
-  g_free (basename);
-
-  return TRUE;
-
- no_module:
-  g_free (basename);
-
-  return FALSE;
-}
-
 static void
 gimp_module_db_module_initialize (const GimpDatafileData *file_data,
                                   gpointer                user_data)
@@ -347,7 +319,8 @@ gimp_module_db_module_initialize (const GimpDatafileData *file_data,
   GimpModule   *module;
   gboolean      load_inhibit;
 
-  if (! valid_module_name (file_data->filename))
+  if (! gimp_datafiles_check_extension (file_data->filename,
+                                        "." G_MODULE_SUFFIX))
     return;
 
   /* don't load if we already know about it */
