@@ -4,10 +4,10 @@
  * gimpcolordisplaystack.c
  * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -43,13 +42,15 @@ enum
 };
 
 
-static void   gimp_color_display_stack_finalize        (GObject               *object);
+static void   gimp_color_display_stack_dispose         (GObject               *object);
 
 static void   gimp_color_display_stack_display_changed (GimpColorDisplay      *display,
                                                         GimpColorDisplayStack *stack);
 static void   gimp_color_display_stack_display_enabled (GimpColorDisplay      *display,
                                                         GParamSpec            *pspec,
                                                         GimpColorDisplayStack *stack);
+static void   gimp_color_display_stack_disconnect      (GimpColorDisplayStack *stack,
+                                                        GimpColorDisplay      *display);
 
 
 G_DEFINE_TYPE (GimpColorDisplayStack, gimp_color_display_stack, G_TYPE_OBJECT)
@@ -105,12 +106,12 @@ gimp_color_display_stack_class_init (GimpColorDisplayStackClass *klass)
                   GIMP_TYPE_COLOR_DISPLAY,
                   G_TYPE_INT);
 
-  object_class->finalize = gimp_color_display_stack_finalize;
+  object_class->dispose = gimp_color_display_stack_dispose;
 
-  klass->changed         = NULL;
-  klass->added           = NULL;
-  klass->removed         = NULL;
-  klass->reordered       = NULL;
+  klass->changed        = NULL;
+  klass->added          = NULL;
+  klass->removed        = NULL;
+  klass->reordered      = NULL;
 }
 
 static void
@@ -120,18 +121,27 @@ gimp_color_display_stack_init (GimpColorDisplayStack *stack)
 }
 
 static void
-gimp_color_display_stack_finalize (GObject *object)
+gimp_color_display_stack_dispose (GObject *object)
 {
   GimpColorDisplayStack *stack = GIMP_COLOR_DISPLAY_STACK (object);
 
   if (stack->filters)
     {
-      g_list_foreach (stack->filters, (GFunc) g_object_unref, NULL);
+      GList *list;
+
+      for (list = stack->filters; list; list = g_list_next (list))
+        {
+          GimpColorDisplay *display = list->data;
+
+          gimp_color_display_stack_disconnect (stack, display);
+          g_object_unref (display);
+        }
+
       g_list_free (stack->filters);
       stack->filters = NULL;
     }
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 GimpColorDisplayStack *
@@ -181,12 +191,12 @@ gimp_color_display_stack_add (GimpColorDisplayStack *stack,
 
   stack->filters = g_list_append (stack->filters, g_object_ref (display));
 
-  g_signal_connect_object (display, "changed",
-                           G_CALLBACK (gimp_color_display_stack_display_changed),
-                           G_OBJECT (stack), 0);
-  g_signal_connect_object (display, "notify::enabled",
-                           G_CALLBACK (gimp_color_display_stack_display_enabled),
-                           G_OBJECT (stack), 0);
+  g_signal_connect (display, "changed",
+                    G_CALLBACK (gimp_color_display_stack_display_changed),
+                    G_OBJECT (stack));
+  g_signal_connect (display, "notify::enabled",
+                    G_CALLBACK (gimp_color_display_stack_display_enabled),
+                    G_OBJECT (stack));
 
   g_signal_emit (stack, stack_signals[ADDED], 0,
                  display, g_list_length (stack->filters) - 1);
@@ -202,12 +212,7 @@ gimp_color_display_stack_remove (GimpColorDisplayStack *stack,
   g_return_if_fail (GIMP_IS_COLOR_DISPLAY (display));
   g_return_if_fail (g_list_find (stack->filters, display) != NULL);
 
-  g_signal_handlers_disconnect_by_func (display,
-                                        gimp_color_display_stack_display_changed,
-                                        stack);
-  g_signal_handlers_disconnect_by_func (display,
-                                        gimp_color_display_stack_display_enabled,
-                                        stack);
+  gimp_color_display_stack_disconnect (stack, display);
 
   stack->filters = g_list_remove (stack->filters, display);
 
@@ -306,3 +311,15 @@ gimp_color_display_stack_display_enabled (GimpColorDisplay      *display,
 {
   gimp_color_display_stack_changed (stack);
 }
+
+static void
+gimp_color_display_stack_disconnect (GimpColorDisplayStack *stack,
+                                     GimpColorDisplay      *display)
+{
+  g_signal_handlers_disconnect_by_func (display,
+                                        gimp_color_display_stack_display_changed,
+                                        stack);
+  g_signal_handlers_disconnect_by_func (display,
+                                        gimp_color_display_stack_display_enabled,
+                                        stack);
+}
