@@ -2,8 +2,8 @@
  * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
  *
  * gimpcolorscale.c
- * Copyright (C) 2002  Sven Neumann <sven@gimp.org>
- *                     Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2002-2010  Sven Neumann <sven@gimp.org>
+ *                          Michael Natterer <mitch@gimp.org>
  *
  * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -76,12 +76,13 @@ gimp_color_scale_init (GimpColorScale *scale)
 {
   GtkRange *range = GTK_RANGE (scale);
 
-  GTK_SCALE (scale)->draw_value = FALSE;
-
-  range->slider_size_fixed = TRUE;
-  range->flippable         = TRUE;
+  gtk_range_set_slider_size_fixed (range, TRUE);
   /* range->update_policy     = GTK_UPDATE_DELAYED; */
 
+  gtk_range_set_flippable (GTK_RANGE (scale), TRUE);
+
+  gtk_scale_set_draw_value (GTK_SCALE (scale), FALSE);
+
   scale->channel      = GIMP_COLOR_SELECTOR_VALUE;
   scale->needs_render = TRUE;
 
@@ -115,6 +116,7 @@ gimp_color_scale_size_allocate (GtkWidget     *widget,
 {
   GimpColorScale *scale = GIMP_COLOR_SCALE (widget);
   GtkRange       *range = GTK_RANGE (widget);
+  GdkRectangle    range_rect;
   gint            focus = 0;
   gint            trough_border;
   gint            scale_width;
@@ -124,7 +126,7 @@ gimp_color_scale_size_allocate (GtkWidget     *widget,
                         "trough-border", &trough_border,
                         NULL);
 
-  if (GTK_WIDGET_CAN_FOCUS (widget))
+  if (gtk_widget_get_can_focus (widget))
     {
       gint focus_padding = 0;
 
@@ -135,25 +137,28 @@ gimp_color_scale_size_allocate (GtkWidget     *widget,
       focus += focus_padding;
     }
 
-  range->min_slider_size = (MIN (allocation->width,
-                                 allocation->height) - 2 * focus) / 2;
+  gtk_range_set_min_slider_size (range,
+                                 (MIN (allocation->width,
+                                       allocation->height) - 2 * focus) / 2);
 
   if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
     GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 
-  scale_width  = range->range_rect.width  - 2 * (focus + trough_border);
-  scale_height = range->range_rect.height - 2 * (focus + trough_border);
+  gtk_range_get_range_rect (range, &range_rect);
+
+  scale_width  = range_rect.width  - 2 * (focus + trough_border);
+  scale_height = range_rect.height - 2 * (focus + trough_border);
 
   switch (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)))
     {
     case GTK_ORIENTATION_HORIZONTAL:
-      scale_width  -= range->min_slider_size - 1;
+      scale_width  -= gtk_range_get_min_slider_size (range) - 1;
       scale_height -= 2;
       break;
 
     case GTK_ORIENTATION_VERTICAL:
       scale_width  -= 2;
-      scale_height -= range->min_slider_size - 1;
+      scale_height -= gtk_range_get_min_slider_size (range) - 1;
       break;
     }
 
@@ -175,7 +180,7 @@ static void
 gimp_color_scale_state_changed (GtkWidget    *widget,
                                 GtkStateType  previous_state)
 {
-  if (widget->state == GTK_STATE_INSENSITIVE ||
+  if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE ||
       previous_state == GTK_STATE_INSENSITIVE)
     {
       GIMP_COLOR_SCALE (widget)->needs_render = TRUE;
@@ -239,34 +244,25 @@ gimp_color_scale_expose (GtkWidget      *widget,
   GtkRange       *range  = GTK_RANGE (widget);
   GtkStyle       *style  = gtk_widget_get_style (widget);
   GdkWindow      *window = gtk_widget_get_window (widget);
+  GtkAllocation   allocation;
+  GdkRectangle    range_rect;
   GdkRectangle    expose_area;        /* Relative to widget->allocation */
   GdkRectangle    area;
   gint            focus = 0;
   gint            trough_border;
+  gint            slider_start;
   gint            slider_size;
   gint            x, y;
   gint            w, h;
 
-  if (! scale->buf || ! GTK_WIDGET_DRAWABLE (widget))
+  if (! scale->buf || ! gtk_widget_is_drawable (widget))
     return FALSE;
 
-  /* This is ugly as it relies heavily on GTK+ internals, but I see no
-   * other way to force the range to recalculate its layout. Might
-   * break if GtkRange internals change.
-   */
-  if (range->need_recalc)
-    {
-      GTK_WIDGET_UNSET_FLAGS (widget, GTK_REALIZED);
-      GTK_WIDGET_CLASS (parent_class)->size_allocate (widget,
-                                                      &widget->allocation);
-      GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-    }
-
   gtk_widget_style_get (widget,
                         "trough-border", &trough_border,
                         NULL);
 
-  if (GTK_WIDGET_CAN_FOCUS (widget))
+  if (gtk_widget_get_can_focus (widget))
     {
       gint focus_padding = 0;
 
@@ -277,20 +273,25 @@ gimp_color_scale_expose (GtkWidget      *widget,
       focus += focus_padding;
     }
 
-  x = widget->allocation.x + range->range_rect.x + focus;
-  y = widget->allocation.y + range->range_rect.y + focus;
-  w = range->range_rect.width  - 2 * focus;
-  h = range->range_rect.height - 2 * focus;
+  gtk_widget_get_allocation (widget, &allocation);
 
-  slider_size = range->min_slider_size / 2;
+  gtk_range_get_range_rect (range, &range_rect);
+  gtk_range_get_slider_range (range, &slider_start, NULL);
+
+  x = allocation.x + range_rect.x + focus;
+  y = allocation.y + range_rect.y + focus;
+  w = range_rect.width  - 2 * focus;
+  h = range_rect.height - 2 * focus;
+
+  slider_size = gtk_range_get_min_slider_size (range) / 2;
 
   expose_area = event->area;
-  expose_area.x -= widget->allocation.x;
-  expose_area.y -= widget->allocation.y;
+  expose_area.x -= allocation.x;
+  expose_area.y -= allocation.y;
 
-  if (gdk_rectangle_intersect (&expose_area, &range->range_rect, &area))
+  if (gdk_rectangle_intersect (&expose_area, &range_rect, &area))
     {
-      gboolean sensitive = (GTK_WIDGET_STATE (widget) != GTK_STATE_INSENSITIVE);
+      gboolean sensitive = gtk_widget_is_sensitive (widget);
 
       if (scale->needs_render)
         {
@@ -302,8 +303,8 @@ gimp_color_scale_expose (GtkWidget      *widget,
           scale->needs_render = FALSE;
         }
 
-      area.x += widget->allocation.x;
-      area.y += widget->allocation.y;
+      area.x += allocation.x;
+      area.y += allocation.y;
 
       gtk_paint_box (style, window,
                      sensitive ? GTK_STATE_ACTIVE : GTK_STATE_INSENSITIVE,
@@ -345,18 +346,18 @@ gimp_color_scale_expose (GtkWidget      *widget,
       gdk_gc_set_clip_rectangle (style->black_gc, NULL);
     }
 
-  if (GTK_WIDGET_IS_SENSITIVE (widget) && GTK_WIDGET_HAS_FOCUS (range))
-    gtk_paint_focus (style, window, GTK_WIDGET_STATE (widget),
+  if (gtk_widget_has_focus (widget))
+    gtk_paint_focus (style, window, gtk_widget_get_state (widget),
                      &area, widget, "trough",
-                     widget->allocation.x + range->range_rect.x,
-                     widget->allocation.y + range->range_rect.y,
-                     range->range_rect.width,
-                     range->range_rect.height);
+                     allocation.x + range_rect.x,
+                     allocation.y + range_rect.y,
+                     range_rect.width,
+                     range_rect.height);
 
   switch (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)))
     {
     case GTK_ORIENTATION_HORIZONTAL:
-      area.x      = widget->allocation.x + range->slider_start;
+      area.x      = allocation.x + slider_start;
       area.y      = y + trough_border;
       area.width  = 2 * slider_size + 1;
       area.height = h - 2 * trough_border;
@@ -364,7 +365,7 @@ gimp_color_scale_expose (GtkWidget      *widget,
 
     case GTK_ORIENTATION_VERTICAL:
       area.x      = x + trough_border;
-      area.y      = widget->allocation.y + range->slider_start;
+      area.y      = allocation.y + slider_start;
       area.width  = w - 2 * trough_border;
       area.height = 2 * slider_size + 1;
       break;
@@ -374,7 +375,7 @@ gimp_color_scale_expose (GtkWidget      *widget,
     {
       GdkGC *gc;
 
-      gc = (GTK_WIDGET_IS_SENSITIVE (widget) ?
+      gc = (gtk_widget_is_sensitive (widget) ?
             style->black_gc :
             style->dark_gc[GTK_STATE_INSENSITIVE]);
 
@@ -396,7 +397,7 @@ gimp_color_scale_expose (GtkWidget      *widget,
 
       gdk_gc_set_clip_rectangle (gc, NULL);
 
-      gc = (GTK_WIDGET_IS_SENSITIVE (widget) ?
+      gc = (gtk_widget_is_sensitive (widget) ?
             style->white_gc :
             style->light_gc[GTK_STATE_INSENSITIVE]);
 
@@ -441,7 +442,8 @@ gimp_color_scale_new (GtkOrientation            orientation,
 
   scale->channel = channel;
 
-  GTK_RANGE (scale)->flippable = (orientation == GTK_ORIENTATION_HORIZONTAL);
+  gtk_range_set_flippable (GTK_RANGE (scale),
+                           orientation == GTK_ORIENTATION_HORIZONTAL);
 
   return GTK_WIDGET (scale);
 }
@@ -496,16 +498,23 @@ gimp_color_scale_set_color (GimpColorScale *scale,
 static gboolean
 should_invert (GtkRange *range)
 {
+  gboolean inverted  = gtk_range_get_inverted (range);
+  gboolean flippable = gtk_range_get_flippable (range);
+
   if (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)) ==
       GTK_ORIENTATION_HORIZONTAL)
-    return
-      (range->inverted && !range->flippable) ||
-      (range->inverted && range->flippable &&
-       gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_LTR) ||
-      (!range->inverted && range->flippable &&
-       gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL);
+    {
+      return
+        (inverted && !flippable) ||
+        (inverted && flippable &&
+         gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_LTR) ||
+        (!inverted && flippable &&
+         gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL);
+    }
   else
-    return range->inverted;
+    {
+      return inverted;
+    }
 }
 
 static void
