@@ -8,10 +8,10 @@
  * Colour selector module
  * Copyright (C) 1999 Austin Donnelly <austin@greenend.org.uk>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_COLOR_SELECTOR_H__
@@ -87,6 +86,15 @@ struct _GimpColorSelectorClass
                                   const GimpHSV            *hsv);
   void (* channel_changed)       (GimpColorSelector        *selector,
                                   GimpColorSelectorChannel  channel);
+
+  /*  another virtual function  */
+  void (* set_config)            (GimpColorSelector        *selector,
+                                  GimpColorConfig          *config);
+
+  /* Padding for future expansion */
+  void (* _gimp_reserved2) (void);
+  void (* _gimp_reserved3) (void);
+  void (* _gimp_reserved4) (void);
 };
 
 
@@ -111,6 +119,9 @@ void   gimp_color_selector_set_channel           (GimpColorSelector *selector,
 void   gimp_color_selector_color_changed         (GimpColorSelector *selector);
 void   gimp_color_selector_channel_changed       (GimpColorSelector *selector);
 
+void   gimp_color_selector_set_config            (GimpColorSelector *selector,
+                                                  GimpColorConfig   *config);
+
 
 G_END_DECLS
 
