@@ -23,6 +23,8 @@
 
 #include <string.h>
 
+#undef GSEAL_ENABLE
+
 #include <gtk/gtk.h>
 
 #include "gimpwidgetstypes.h"
@@ -103,7 +105,7 @@ gimp_frame_size_request (GtkWidget      *widget,
   GtkRequisition  child_requisition;
   gint            border_width;
 
-  if (label_widget && GTK_WIDGET_VISIBLE (label_widget))
+  if (label_widget && gtk_widget_get_visible (label_widget))
     {
       gtk_widget_size_request (label_widget, requisition);
     }
@@ -115,7 +117,7 @@ gimp_frame_size_request (GtkWidget      *widget,
 
   requisition->height += gimp_frame_get_label_spacing (frame);
 
-  if (child && GTK_WIDGET_VISIBLE (child))
+  if (child && gtk_widget_get_visible (child))
     {
       gint indent = gimp_frame_get_indent (widget);
 
@@ -140,14 +142,14 @@ gimp_frame_size_allocate (GtkWidget     *widget,
   GtkWidget *label_widget = gtk_frame_get_label_widget (frame);
   GtkWidget *child        = gtk_bin_get_child (GTK_BIN (widget));
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
   gimp_frame_child_allocate (frame, &frame->child_allocation);
 
-  if (child && GTK_WIDGET_VISIBLE (child))
+  if (child && gtk_widget_get_visible (child))
     gtk_widget_size_allocate (child, &frame->child_allocation);
 
-  if (label_widget && GTK_WIDGET_VISIBLE (label_widget))
+  if (label_widget && gtk_widget_get_visible (label_widget))
     {
       GtkAllocation   label_allocation;
       GtkRequisition  label_requisition;
@@ -173,14 +175,16 @@ gimp_frame_child_allocate (GtkFrame      *frame,
 {
   GtkWidget     *widget       = GTK_WIDGET (frame);
   GtkWidget     *label_widget = gtk_frame_get_label_widget (frame);
-  GtkAllocation *allocation   = &widget->allocation;
+  GtkAllocation  allocation;
   gint           border_width;
   gint           spacing      = 0;
   gint           indent       = gimp_frame_get_indent (widget);
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   border_width = gtk_container_get_border_width (GTK_CONTAINER (frame));
 
-  if (label_widget && GTK_WIDGET_VISIBLE (label_widget))
+  if (label_widget && gtk_widget_get_visible (label_widget))
     {
       GtkRequisition  child_requisition;
 
@@ -197,13 +201,13 @@ gimp_frame_child_allocate (GtkFrame      *frame,
 
   child_allocation->y      = border_width + spacing;
   child_allocation->width  = MAX (1,
-                                  allocation->width - 2 * border_width - indent);
+                                  allocation.width - 2 * border_width - indent);
   child_allocation->height = MAX (1,
-                                  allocation->height -
+                                  allocation.height -
                                   child_allocation->y - border_width);
 
-  child_allocation->x += allocation->x;
-  child_allocation->y += allocation->y;
+  child_allocation->x += allocation.x;
+  child_allocation->y += allocation.y;
 }
 
 static void
@@ -221,7 +225,7 @@ static gboolean
 gimp_frame_expose_event (GtkWidget      *widget,
                          GdkEventExpose *event)
 {
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       GtkWidgetClass *widget_class = g_type_class_peek_parent (parent_class);
 
@@ -307,7 +311,7 @@ gimp_frame_get_label_spacing (GtkFrame *frame)
   GtkWidget *label_widget = gtk_frame_get_label_widget (frame);
   gint       spacing      = 0;
 
-  if ((label_widget && GTK_WIDGET_VISIBLE (label_widget)) ||
+  if ((label_widget && gtk_widget_get_visible (label_widget)) ||
       (g_object_get_data (G_OBJECT (frame), GIMP_FRAME_IN_EXPANDER_KEY)))
     {
       gtk_widget_style_get (GTK_WIDGET (frame),
