@@ -4,10 +4,10 @@
  * gimphelpui.c
  * Copyright (C) 2000-2003 Michael Natterer <mitch@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -454,7 +453,7 @@ gimp_context_help_idle_start (gpointer widget)
       cursor = gdk_cursor_new_for_display (gtk_widget_get_display (invisible),
                                            GDK_QUESTION_ARROW);
 
-      status = gdk_pointer_grab (invisible->window, TRUE,
+      status = gdk_pointer_grab (gtk_widget_get_window (invisible), TRUE,
                                  GDK_BUTTON_PRESS_MASK   |
                                  GDK_BUTTON_RELEASE_MASK |
                                  GDK_ENTER_NOTIFY_MASK   |
@@ -470,7 +469,7 @@ gimp_context_help_idle_start (gpointer widget)
           return FALSE;
         }
 
-      if (gdk_keyboard_grab (invisible->window, TRUE,
+      if (gdk_keyboard_grab (gtk_widget_get_window (invisible), TRUE,
                              GDK_CURRENT_TIME) != GDK_GRAB_SUCCESS)
         {
           gdk_display_pointer_ungrab (gtk_widget_get_display (invisible),
