@@ -4,10 +4,10 @@
  * gimpintstore.c
  * Copyright (C) 2004-2007  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -229,15 +228,17 @@ gimp_int_store_row_inserted (GtkTreeModel *model,
 static void
 gimp_int_store_add_empty (GimpIntStore *store)
 {
+  GtkTreeIter iter = { 0, };
+
   g_return_if_fail (store->empty_iter == NULL);
 
-  store->empty_iter = g_new0 (GtkTreeIter, 1);
-
-  gtk_list_store_prepend (GTK_LIST_STORE (store), store->empty_iter);
-  gtk_list_store_set (GTK_LIST_STORE (store), store->empty_iter,
+  gtk_list_store_prepend (GTK_LIST_STORE (store), &iter);
+  gtk_list_store_set (GTK_LIST_STORE (store), &iter,
                       GIMP_INT_STORE_VALUE, -1,
                       GIMP_INT_STORE_LABEL, (_("(Empty)")),
                       -1);
+
+  store->empty_iter = gtk_tree_iter_copy (&iter);
 }
 
 /**
