@@ -12,9 +12,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -225,13 +224,14 @@ gimp_pick_button_clicked (GtkButton *gtk_button)
   widget = button->grab_widget;
   timestamp = gtk_get_current_event_time ();
 
-  if (gdk_keyboard_grab (widget->window, FALSE, timestamp) != GDK_GRAB_SUCCESS)
+  if (gdk_keyboard_grab (gtk_widget_get_window (widget), FALSE,
+                         timestamp) != GDK_GRAB_SUCCESS)
     {
       g_warning ("Failed to grab keyboard to do eyedropper");
       return;
     }
 
-  if (gdk_pointer_grab (widget->window, FALSE,
+  if (gdk_pointer_grab (gtk_widget_get_window (widget), FALSE,
                         GDK_BUTTON_RELEASE_MASK |
                         GDK_BUTTON_PRESS_MASK   |
                         GDK_POINTER_MOTION_MASK,
