@@ -4,10 +4,10 @@
  * gimpquerybox.c
  * Copyright (C) 1999-2000 Michael Natterer <mitch@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -70,6 +69,7 @@ static QueryBox * create_query_box             (const gchar   *title,
                                                 gpointer       callback_data);
 
 static void       query_box_disconnect         (QueryBox      *query_box);
+static void       query_box_destroy            (QueryBox      *query_box);
 
 static void       string_query_box_response    (GtkWidget     *widget,
                                                 gint           response_id,
@@ -118,7 +118,7 @@ create_query_box (const gchar   *title,
   g_return_val_if_fail (object == NULL || G_IS_OBJECT (object), NULL);
   g_return_val_if_fail (object == NULL || signal != NULL, NULL);
 
-  query_box = g_new0 (QueryBox, 1);
+  query_box = g_slice_new0 (QueryBox);
 
   query_box->qbox = gimp_dialog_new (title, "gimp-query-box",
                                      parent, 0,
@@ -158,12 +158,14 @@ create_query_box (const gchar   *title,
 
   if (stock_id)
     {
+      GtkWidget *content_area;
       GtkWidget *image;
 
+      content_area = gtk_dialog_get_content_area (GTK_DIALOG (query_box->qbox));
+
       hbox = gtk_hbox_new (FALSE, 12);
       gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
-      gtk_container_add (GTK_CONTAINER (GTK_DIALOG (query_box->qbox)->vbox),
-                         hbox);
+      gtk_container_add (GTK_CONTAINER (content_area), hbox);
       gtk_widget_show (hbox);
 
       image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
@@ -183,9 +185,12 @@ create_query_box (const gchar   *title,
     }
   else
     {
+      GtkWidget *content_area;
+
+      content_area = gtk_dialog_get_content_area (GTK_DIALOG (query_box->qbox));
+
       gtk_container_set_border_width (GTK_CONTAINER (query_box->vbox), 12);
-      gtk_container_add (GTK_CONTAINER (GTK_DIALOG (query_box->qbox)->vbox),
-                         query_box->vbox);
+      gtk_container_add (GTK_CONTAINER (content_area), query_box->vbox);
     }
 
   gtk_widget_show (query_box->vbox);
@@ -547,6 +552,16 @@ query_box_disconnect (QueryBox *query_box)
                                           query_box);
 }
 
+static void
+query_box_destroy (QueryBox *query_box)
+{
+  /*  Destroy the box  */
+  if (query_box->qbox)
+    gtk_widget_destroy (query_box->qbox);
+
+  g_slice_free (QueryBox, query_box);
+}
+
 static void
 string_query_box_response (GtkWidget *widget,
                            gint       response_id,
@@ -565,11 +580,7 @@ string_query_box_response (GtkWidget *widget,
                                                        string,
                                                        query_box->callback_data);
 
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
 
 static void
@@ -590,11 +601,7 @@ int_query_box_response (GtkWidget *widget,
                                                     value,
                                                     query_box->callback_data);
 
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
 
 static void
@@ -615,11 +622,7 @@ double_query_box_response (GtkWidget *widget,
                                                        value,
                                                        query_box->callback_data);
 
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
 
 static void
@@ -643,11 +646,7 @@ size_query_box_response (GtkWidget *widget,
                                                      unit,
                                                      query_box->callback_data);
 
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
 
 static void
@@ -663,21 +662,12 @@ boolean_query_box_response (GtkWidget *widget,
                                                        GTK_RESPONSE_OK),
                                                       query_box->callback_data);
 
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
 
 static void
 query_box_cancel_callback (QueryBox *query_box)
 {
   query_box_disconnect (query_box);
-
-  /*  Destroy the box  */
-  if (query_box->qbox)
-    gtk_widget_destroy (query_box->qbox);
-
-  g_free (query_box);
+  query_box_destroy (query_box);
 }
