@@ -306,10 +306,8 @@ gimp_scrolled_preview_area_size_allocate (GtkWidget           *widget,
   switch (priv->hscr_policy)
     {
     case GTK_POLICY_AUTOMATIC:
-      if (width > GIMP_PREVIEW (preview)->width)
-        gtk_widget_show (preview->hscr);
-      else
-        gtk_widget_hide (preview->hscr);
+      gtk_widget_set_visible (preview->hscr,
+                              width > GIMP_PREVIEW (preview)->width);
       break;
 
     case GTK_POLICY_ALWAYS:
@@ -326,10 +324,8 @@ gimp_scrolled_preview_area_size_allocate (GtkWidget           *widget,
   switch (priv->vscr_policy)
     {
     case GTK_POLICY_AUTOMATIC:
-      if (height > GIMP_PREVIEW (preview)->height)
-        gtk_widget_show (preview->vscr);
-      else
-        gtk_widget_hide (preview->vscr);
+      gtk_widget_set_visible (preview->vscr,
+                              height > GIMP_PREVIEW (preview)->height);
       break;
 
     case GTK_POLICY_ALWAYS:
@@ -341,16 +337,10 @@ gimp_scrolled_preview_area_size_allocate (GtkWidget           *widget,
       break;
     }
 
-  if (GTK_WIDGET_VISIBLE (preview->vscr) &&
-      GTK_WIDGET_VISIBLE (preview->hscr) &&
-      GIMP_PREVIEW_GET_CLASS (preview)->draw_thumb)
-    {
-      gtk_widget_show (preview->nav_icon);
-    }
-  else
-    {
-      gtk_widget_hide (preview->nav_icon);
-    }
+  gtk_widget_set_visible (preview->nav_icon,
+                          gtk_widget_get_visible (preview->vscr) &&
+                          gtk_widget_get_visible (preview->hscr) &&
+                          GIMP_PREVIEW_GET_CLASS (preview)->draw_thumb);
 
   gimp_scrolled_preview_thaw (preview);
 }
@@ -674,18 +664,21 @@ gimp_scrolled_preview_nav_popup_event (GtkWidget           *widget,
         GdkEventMotion *mevent = (GdkEventMotion *) event;
         GtkAdjustment  *hadj;
         GtkAdjustment  *vadj;
+        GtkAllocation   allocation;
         gint            cx, cy;
         gdouble         x, y;
 
         hadj = gtk_range_get_adjustment (GTK_RANGE (preview->hscr));
         vadj = gtk_range_get_adjustment (GTK_RANGE (preview->vscr));
 
+        gtk_widget_get_allocation (widget, &allocation);
+
         gtk_widget_get_pointer (widget, &cx, &cy);
 
         x = cx * (gtk_adjustment_get_upper (hadj) -
-                  gtk_adjustment_get_lower (hadj)) / widget->allocation.width;
+                  gtk_adjustment_get_lower (hadj)) / allocation.width;
         y = cy * (gtk_adjustment_get_upper (vadj) -
-                  gtk_adjustment_get_lower (vadj)) / widget->allocation.height;
+                  gtk_adjustment_get_lower (vadj)) / allocation.height;
 
         x += (gtk_adjustment_get_lower (hadj) -
               gtk_adjustment_get_page_size (hadj) / 2);
@@ -723,11 +716,14 @@ gimp_scrolled_preview_nav_popup_expose (GtkWidget           *widget,
                                         GimpScrolledPreview *preview)
 {
   GtkAdjustment *adj;
+  GtkAllocation  allocation;
   cairo_t       *cr;
   gdouble        x, y;
   gdouble        w, h;
 
-  adj   = gtk_range_get_adjustment (GTK_RANGE (preview->hscr));
+  adj = gtk_range_get_adjustment (GTK_RANGE (preview->hscr));
+
+  gtk_widget_get_allocation (widget, &allocation);
 
   x = (gtk_adjustment_get_value (adj) /
        (gtk_adjustment_get_upper (adj) -
@@ -748,10 +744,10 @@ gimp_scrolled_preview_nav_popup_expose (GtkWidget           *widget,
   if (w >= 1.0 && h >= 1.0)
     return FALSE;
 
-  x = floor (x * (gdouble) widget->allocation.width);
-  y = floor (y * (gdouble) widget->allocation.height);
-  w = MAX (1, ceil (w * (gdouble) widget->allocation.width));
-  h = MAX (1, ceil (h * (gdouble) widget->allocation.height));
+  x = floor (x * (gdouble) allocation.width);
+  y = floor (y * (gdouble) allocation.height);
+  w = MAX (1, ceil (w * (gdouble) allocation.width));
+  h = MAX (1, ceil (h * (gdouble) allocation.height));
 
   cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
@@ -759,7 +755,7 @@ gimp_scrolled_preview_nav_popup_expose (GtkWidget           *widget,
   cairo_clip (cr);
 
   cairo_rectangle (cr,
-                   0, 0, widget->allocation.width, widget->allocation.height);
+                   0, 0, allocation.width, allocation.height);
 
   cairo_rectangle (cr, x, y, w, h);
 
@@ -781,7 +777,7 @@ gimp_scrolled_preview_nav_popup_expose (GtkWidget           *widget,
 static void
 gimp_scrolled_preview_set_cursor (GimpPreview *preview)
 {
-  if (! GTK_WIDGET_REALIZED (preview->area))
+  if (! gtk_widget_get_realized (preview->area))
     return;
 
   if (preview->xmax - preview->xmin > preview->width  ||
