@@ -197,6 +197,7 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_TOOLS,                    NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_OPTIONS,             NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_DEVICE_STATUS,            NULL,        0, 0, LIBGIMP_DOMAIN },
+  { GIMP_STOCK_INPUT_DEVICE,             NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_CURSOR,                   NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_SAMPLE_POINT,             NULL,        0, 0, LIBGIMP_DOMAIN },
 
@@ -294,6 +295,7 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_TOOL_COLORIZE,            NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_CROP,         N_("Cr_op"),        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_CURVES,              NULL,        0, 0, LIBGIMP_DOMAIN },
+  { GIMP_STOCK_TOOL_DESATURATE,          NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_DODGE,               NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_ELLIPSE_SELECT,      NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_ERASER,              NULL,        0, 0, LIBGIMP_DOMAIN },
@@ -390,6 +392,7 @@ gimp_stock_button_pixbufs[] =
   { GIMP_STOCK_TOOLS,                    stock_tools_24                    },
   { GIMP_STOCK_TOOL_OPTIONS,             stock_tool_options_24             },
   { GIMP_STOCK_DEVICE_STATUS,            stock_device_status_24            },
+  { GIMP_STOCK_INPUT_DEVICE,             stock_input_device_22             },
   { GIMP_STOCK_CURSOR,                   stock_cursor_24                   },
   { GIMP_STOCK_SAMPLE_POINT,             stock_sample_point_24             },
 
@@ -444,6 +447,7 @@ gimp_stock_button_pixbufs[] =
   { GIMP_STOCK_TOOL_COLORIZE,            stock_tool_colorize_22            },
   { GIMP_STOCK_TOOL_CROP,                stock_tool_crop_22                },
   { GIMP_STOCK_TOOL_CURVES,              stock_tool_curves_22              },
+  { GIMP_STOCK_TOOL_DESATURATE,          stock_tool_desaturate_22          },
   { GIMP_STOCK_TOOL_DODGE,               stock_tool_dodge_22               },
   { GIMP_STOCK_TOOL_ELLIPSE_SELECT,      stock_tool_ellipse_select_22      },
   { GIMP_STOCK_TOOL_ERASER,              stock_tool_eraser_22              },
@@ -516,6 +520,7 @@ gimp_stock_menu_pixbufs[] =
   { GIMP_STOCK_TOOLS,                    stock_tools_16                    },
   { GIMP_STOCK_TOOL_OPTIONS,             stock_tool_options_16             },
   { GIMP_STOCK_DEVICE_STATUS,            stock_device_status_16            },
+  { GIMP_STOCK_INPUT_DEVICE,             stock_input_device_16             },
   { GIMP_STOCK_CURSOR,                   stock_cursor_16                   },
   { GIMP_STOCK_SAMPLE_POINT,             stock_sample_point_16             },
 
@@ -606,6 +611,7 @@ gimp_stock_menu_pixbufs[] =
   { GIMP_STOCK_TOOL_COLORIZE,            stock_tool_colorize_16            },
   { GIMP_STOCK_TOOL_CROP,                stock_tool_crop_16                },
   { GIMP_STOCK_TOOL_CURVES,              stock_tool_curves_16              },
+  { GIMP_STOCK_TOOL_DESATURATE,          stock_tool_desaturate_16          },
   { GIMP_STOCK_TOOL_DODGE,               stock_tool_dodge_16               },
   { GIMP_STOCK_TOOL_ELLIPSE_SELECT,      stock_tool_ellipse_select_16      },
   { GIMP_STOCK_TOOL_ERASER,              stock_tool_eraser_16              },
