@@ -110,6 +110,7 @@ G_BEGIN_DECLS
 #define GIMP_STOCK_TOOL_COLORIZE            "gimp-tool-colorize"
 #define GIMP_STOCK_TOOL_CROP                "gimp-tool-crop"
 #define GIMP_STOCK_TOOL_CURVES              "gimp-tool-curves"
+#define GIMP_STOCK_TOOL_DESATURATE          "gimp-tool-desaturate"
 #define GIMP_STOCK_TOOL_DODGE               "gimp-tool-dodge"
 #define GIMP_STOCK_TOOL_ELLIPSE_SELECT      "gimp-tool-ellipse-select"
 #define GIMP_STOCK_TOOL_ERASER              "gimp-tool-eraser"
@@ -214,6 +215,7 @@ G_BEGIN_DECLS
 #define GIMP_STOCK_TOOLS                    "gimp-tools"
 #define GIMP_STOCK_TOOL_OPTIONS             "gimp-tool-options"
 #define GIMP_STOCK_DEVICE_STATUS            "gimp-device-status"
+#define GIMP_STOCK_INPUT_DEVICE             "gimp-input-device"
 #define GIMP_STOCK_CURSOR                   "gimp-cursor"
 #define GIMP_STOCK_SAMPLE_POINT             "gimp-sample-point"
 
@@ -268,13 +270,14 @@ G_BEGIN_DECLS
 /*  missing icons:  */
 
 #define GIMP_STOCK_BRUSH                    GIMP_STOCK_TOOL_PAINTBRUSH
+#define GIMP_STOCK_DYNAMICS                 GIMP_STOCK_TOOL_PAINTBRUSH
 #define GIMP_STOCK_BUFFER                   GTK_STOCK_PASTE
 #define GIMP_STOCK_DETACH                   GTK_STOCK_CONVERT
 #define GIMP_STOCK_FONT                     GTK_STOCK_SELECT_FONT
 #define GIMP_STOCK_GRADIENT                 GIMP_STOCK_TOOL_BLEND
 #define GIMP_STOCK_PALETTE                  GTK_STOCK_SELECT_COLOR
 #define GIMP_STOCK_PATTERN                  GIMP_STOCK_TOOL_BUCKET_FILL
-#define GIMP_STOCK_TOOL_DESATURATE          GIMP_STOCK_CONVERT_GRAYSCALE
+#define GIMP_STOCK_TOOL_PRESET              GIMP_STOCK_TOOL_PAINTBRUSH
 
 
 void   gimp_stock_init (void);
