@@ -4,9 +4,9 @@
  *
  * Copyright (C) 1997-1998 Shuji Narazaki <narazaki@InetQ.or.jp>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -117,11 +116,11 @@ query (void)
 {
   static const GimpParamDef args [] =
   {
-    { GIMP_PDB_INT32,    "run-mode",             "Interactive, non-interactive"},
+    { GIMP_PDB_INT32,    "run-mode",             "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"},
     { GIMP_PDB_IMAGE,    "image",                "Input image"},
     { GIMP_PDB_DRAWABLE, "drawable",             "Input drawable (not used)"},
-    { GIMP_PDB_INT32,    "link-after-alignment", "Link the visible layers after alignment"},
-    { GIMP_PDB_INT32,    "use-bottom",           "use the bottom layer as the base of alignment"}
+    { GIMP_PDB_INT32,    "link-after-alignment", "Link the visible layers after alignment { TRUE, FALSE }"},
+    { GIMP_PDB_INT32,    "use-bottom",           "use the bottom layer as the base of alignment { TRUE, FALSE }"}
   };
 
   gimp_install_procedure (PLUG_IN_PROC,
@@ -146,7 +145,7 @@ run (const gchar      *name,
      gint             *nreturn_vals,
      GimpParam       **return_vals)
 {
-  static GimpParam  values[1];
+  static GimpParam  values[2];
   GimpPDBStatusType status = GIMP_PDB_EXECUTION_ERROR;
   GimpRunMode       run_mode;
   gint              image_id, layer_num;
@@ -168,7 +167,9 @@ run (const gchar      *name,
       gimp_image_get_layers (image_id, &layer_num);
       if (layer_num < 2)
         {
-          g_message (_("There are not enough layers to align."));
+          *nreturn_vals = 2;
+          values[1].type          = GIMP_PDB_STRING;
+          values[1].data.d_string = _("There are not enough layers to align.");
           return;
         }
       gimp_get_data (PLUG_IN_PROC, &VALS);
@@ -415,8 +416,8 @@ align_layers_dialog (void)
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
   gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table,
-                      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      table, FALSE, FALSE, 0);
   gtk_widget_show (table);
 
   combo = gimp_int_combo_box_new (_("None"),                 H_NONE,
