@@ -1,7 +1,8 @@
 /*
- * Animation Playback plug-in version 0.98.8
+ * Animation Playback plug-in version 0.99.1
  *
  * (c) Adam D. Moss : 1997-2000 : adam@gimp.org : adam@foxbox.org
+ * (c) Mircea Purdea : 2009 : someone_else@exhalus.net
  *
  * GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
@@ -27,7 +28,7 @@
  *  speedups (caching?  most bottlenecks seem to be in pixelrgns)
  *    -> do pixelrgns properly!
  *
- *  write other half of the user interface (default timing, disposal &c)
+ *  write other half of the user interface (zoom, disposal &c)
  */
 
 #include "config.h"
@@ -70,7 +71,9 @@ static void        rewind_callback           (GtkAction       *action);
 static void        speed_up_callback         (GtkAction       *action);
 static void        speed_down_callback       (GtkAction       *action);
 static void        speed_reset_callback      (GtkAction       *action);
-static void        speedcombo_changed        (GtkWidget       *scombo,
+static void        speedcombo_changed        (GtkWidget       *combo,
+                                              gpointer         data);
+static void        fpscombo_changed          (GtkWidget       *combo,
                                               gpointer         data);
 static gboolean    repaint_sda               (GtkWidget       *darea,
                                               GdkEventExpose  *event,
@@ -85,6 +88,7 @@ static void        total_alpha_preview       (guchar          *ptr);
 static void        init_preview              (void);
 static void        update_combobox           (void);
 static gdouble     get_duration_factor       (gint             index);
+static gint        get_fps                   (gint             index);
 
 
 /* tag util functions*/
@@ -132,6 +136,7 @@ static GimpImageBaseType  imagetype;
 static guchar            *palette;
 static gint               ncolours;
 static gint               duration_index = 3;
+static gint               default_frame_duration = 100; /* ms */
 
 
 /* for shaping */
@@ -222,7 +227,7 @@ reshape_from_bitmap (const gchar *bitmap)
     {
       GdkBitmap *shape_mask;
 
-      shape_mask = gdk_bitmap_create_from_data (shape_window->window,
+      shape_mask = gdk_bitmap_create_from_data (gtk_widget_get_window (shape_window),
                                                 bitmap,
                                                 width, height);
       gtk_widget_shape_combine_mask (shape_window, shape_mask, 0, 0);
@@ -279,12 +284,12 @@ shape_pressed (GtkWidget      *widget,
       p->y = (gint) event->y;
 
       gtk_grab_add (widget);
-      gdk_pointer_grab (widget->window, TRUE,
+      gdk_pointer_grab (gtk_widget_get_window (widget), TRUE,
                         GDK_BUTTON_RELEASE_MASK |
                         GDK_BUTTON_MOTION_MASK  |
                         GDK_POINTER_MOTION_HINT_MASK,
                         NULL, NULL, 0);
-      gdk_window_raise (widget->window);
+      gdk_window_raise (gtk_widget_get_window (widget));
     }
 
   return FALSE;
@@ -344,7 +349,7 @@ repaint_da (GtkWidget      *darea,
 {
   GtkStyle *style = gtk_widget_get_style (darea);
 
-  gdk_draw_rgb_image (darea->window,
+  gdk_draw_rgb_image (gtk_widget_get_window (darea),
                       style->white_gc,
                       0, 0, width, height,
                       (total_frames == 1) ? GDK_RGB_DITHER_MAX : DITHERTYPE,
@@ -360,7 +365,7 @@ repaint_sda (GtkWidget      *darea,
 {
   GtkStyle *style = gtk_widget_get_style (darea);
 
-  gdk_draw_rgb_image (darea->window,
+  gdk_draw_rgb_image (gtk_widget_get_window (darea),
                       style->white_gc,
                       0, 0, width, height,
                       (total_frames == 1) ? GDK_RGB_DITHER_MAX : DITHERTYPE,
@@ -408,18 +413,18 @@ detach_callback (GtkToggleAction *action)
       gtk_window_set_screen (GTK_WINDOW (shape_window),
                              gtk_widget_get_screen (drawing_area));
 
-      if (GTK_WIDGET_REALIZED (drawing_area))
+      if (gtk_widget_get_realized (drawing_area))
         {
           gint x, y;
 
-          gdk_window_get_origin (drawing_area->window, &x, &y);
+          gdk_window_get_origin (gtk_widget_get_window (drawing_area), &x, &y);
 
           gtk_window_move (GTK_WINDOW (shape_window), x + 6, y + 6);
         }
 
       gtk_widget_show (shape_window);
 
-      gdk_window_set_back_pixmap (shape_drawing_area->window, NULL, TRUE);
+      gdk_window_set_back_pixmap (gtk_widget_get_window (shape_drawing_area), NULL, TRUE);
 
       memset (shape_preview_mask, 0, (width * height) / 8 + height);
       render_frame (frame_number);
@@ -640,15 +645,41 @@ build_dialog (GimpImageBaseType  basetype,
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
+  progress = gtk_progress_bar_new ();
+  gtk_box_pack_end (GTK_BOX (hbox), progress, TRUE, TRUE, 0);
+  gtk_widget_show (progress);
+
   speedcombo = gtk_combo_box_new_text ();
-  gtk_box_pack_start (GTK_BOX (hbox), speedcombo, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), speedcombo, FALSE, FALSE, 0);
+  gtk_widget_show (speedcombo);
+
+  for (index = 0; index < 9; index++)
+    {
+      gchar *text;
+
+      /* list is given in "fps" - frames per second */
+      text = g_strdup_printf  (_("%d fps"), get_fps (index));
+      gtk_combo_box_append_text (GTK_COMBO_BOX (speedcombo), text);
+      g_free (text);
+    }
+
+  gtk_combo_box_set_active (GTK_COMBO_BOX (speedcombo), 0);
+
+  g_signal_connect (speedcombo, "changed",
+                    G_CALLBACK (fpscombo_changed),
+                    NULL);
+
+  gimp_help_set_help_data (speedcombo, _("Default framerate"), NULL);
+
+  speedcombo = gtk_combo_box_new_text ();
+  gtk_box_pack_end (GTK_BOX (hbox), speedcombo, FALSE, FALSE, 0);
   gtk_widget_show (speedcombo);
 
   for (index = 0; index < 7; index++)
     {
       gchar *text;
 
-      text = g_strdup_printf  ("%g %%", 100.0 / get_duration_factor (index));
+      text = g_strdup_printf  ("%g\303\227", (100 / get_duration_factor (index)) / 100);
       gtk_combo_box_append_text (GTK_COMBO_BOX (speedcombo), text);
       g_free (text);
     }
@@ -661,10 +692,6 @@ build_dialog (GimpImageBaseType  basetype,
 
   gimp_help_set_help_data (speedcombo, _("Playback speed"), NULL);
 
-  progress = gtk_progress_bar_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), progress, TRUE, TRUE, 0);
-  gtk_widget_show (progress);
-
   if (total_frames < 2)
     {
       action = gtk_ui_manager_get_action (ui_manager,
@@ -696,11 +723,11 @@ build_dialog (GimpImageBaseType  basetype,
   gtk_widget_add_events (shape_drawing_area, GDK_BUTTON_PRESS_MASK);
   gtk_widget_realize (shape_window);
 
-  gdk_window_set_back_pixmap (shape_window->window, NULL, FALSE);
+  gdk_window_set_back_pixmap (gtk_widget_get_window (shape_window), NULL, FALSE);
 
   cursor = gdk_cursor_new_for_display (gtk_widget_get_display (shape_window),
                                        GDK_HAND2);
-  gdk_window_set_cursor (shape_window->window, cursor);
+  gdk_window_set_cursor (gtk_widget_get_window (shape_window), cursor);
   gdk_cursor_unref (cursor);
 
   g_signal_connect (shape_window, "button-press-event",
@@ -922,7 +949,7 @@ render_frame (gint32 whichframe)
           if (detached)
             {
               reshape_from_bitmap (shape_preview_mask);
-              gdk_draw_rgb_image (shape_drawing_area->window,
+              gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                   shape_style->white_gc,
                                   0, 0, width, height,
                                   (total_frames == 1 ?
@@ -932,7 +959,7 @@ render_frame (gint32 whichframe)
           else
             {
               reshape_from_bitmap (shape_preview_mask);
-              gdk_draw_rgb_image (drawing_area->window,
+              gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                   drawing_style->white_gc,
                                   0, 0, width, height,
                                   (total_frames == 1 ?
@@ -1023,7 +1050,7 @@ render_frame (gint32 whichframe)
                   gint bottom = MIN (rawy + rawheight, height);
 
                   reshape_from_bitmap (shape_preview_mask);
-                  gdk_draw_rgb_image (shape_drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                       shape_style->white_gc,
                                       0, top, width, bottom - top,
                                       (total_frames == 1 ?
@@ -1034,7 +1061,7 @@ render_frame (gint32 whichframe)
               else
                 {
                   reshape_from_bitmap (shape_preview_mask);
-                  gdk_draw_rgb_image (shape_drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                       shape_style->white_gc,
                                       0, 0, width, height,
                                       (total_frames == 1 ?
@@ -1049,7 +1076,7 @@ render_frame (gint32 whichframe)
                   gint top    = MAX (rawy, 0);
                   gint bottom = MIN (rawy + rawheight, height);
 
-                  gdk_draw_rgb_image (drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                       drawing_style->white_gc,
                                       0, top, width, bottom - top,
                                       (total_frames == 1 ?
@@ -1059,7 +1086,7 @@ render_frame (gint32 whichframe)
                 }
               else
                 {
-                  gdk_draw_rgb_image (drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                       drawing_style->white_gc,
                                       0, 0, width, height,
                                       (total_frames == 1 ?
@@ -1151,7 +1178,7 @@ render_frame (gint32 whichframe)
           if (detached)
             {
               reshape_from_bitmap (shape_preview_mask);
-              gdk_draw_rgb_image (shape_drawing_area->window,
+              gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                   shape_style->white_gc,
                                   0, 0, width, height,
                                   (total_frames == 1 ?
@@ -1160,7 +1187,7 @@ render_frame (gint32 whichframe)
             }
           else
             {
-              gdk_draw_rgb_image (drawing_area->window,
+              gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                   drawing_style->white_gc,
                                   0, 0, width, height,
                                   (total_frames == 1 ?
@@ -1256,7 +1283,7 @@ render_frame (gint32 whichframe)
                   gint bottom = MIN (rawy + rawheight, height);
 
                   reshape_from_bitmap (shape_preview_mask);
-                  gdk_draw_rgb_image (shape_drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                       shape_style->white_gc,
                                       0, top, width, bottom - top,
                                       (total_frames == 1 ?
@@ -1267,7 +1294,7 @@ render_frame (gint32 whichframe)
               else
                 {
                   reshape_from_bitmap (shape_preview_mask);
-                  gdk_draw_rgb_image (shape_drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (shape_drawing_area),
                                       shape_style->white_gc,
                                       0, 0, width, height,
                                       (total_frames == 1 ?
@@ -1282,7 +1309,7 @@ render_frame (gint32 whichframe)
                   gint top    = MAX (rawy, 0);
                   gint bottom = MIN (rawy + rawheight, height);
 
-                  gdk_draw_rgb_image (drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                       drawing_style->white_gc,
                                       0, top, width, bottom - top,
                                       (total_frames == 1 ?
@@ -1292,7 +1319,7 @@ render_frame (gint32 whichframe)
                 }
               else
                 {
-                  gdk_draw_rgb_image (drawing_area->window,
+                  gdk_draw_rgb_image (gtk_widget_get_window (drawing_area),
                                       drawing_style->white_gc,
                                       0, 0, width, height,
                                       (total_frames == 1 ?
@@ -1470,6 +1497,34 @@ get_duration_factor (gint index)
     }
 }
 
+static gint
+get_fps (gint index)
+{
+  switch (index)
+    {
+    case 0:
+      return 10;
+    case 1:
+      return 12;
+    case 2:
+      return 15;
+    case 3:
+      return 24;
+    case 4:
+      return 25;
+    case 5:
+      return 30;
+    case 6:
+      return 50;
+    case 7:
+      return 60;
+    case 8:
+      return 72;
+    default:
+      return 10;
+    }
+}
+
 static void
 step_callback (GtkAction *action)
 {
@@ -1549,11 +1604,11 @@ speed_reset_callback (GtkAction *action)
 
 
 static void
-speedcombo_changed (GtkWidget *scombo, gpointer data)
+speedcombo_changed (GtkWidget *combo, gpointer data)
 {
   GtkAction * action;
 
-  duration_index = gtk_combo_box_get_active (GTK_COMBO_BOX (scombo));
+  duration_index = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
 
   action = gtk_ui_manager_get_action (ui_manager,
                                       "/anim-play-popup/speed-reset");
@@ -1568,6 +1623,12 @@ speedcombo_changed (GtkWidget *scombo, gpointer data)
   gtk_action_set_sensitive (action, duration_index > 0);
 }
 
+static void
+fpscombo_changed (GtkWidget *combo, gpointer data)
+{
+  default_frame_duration = 1000 / get_fps(gtk_combo_box_get_active (GTK_COMBO_BOX (combo)));
+}
+
 static void
 update_combobox (void)
 {
@@ -1602,9 +1663,8 @@ get_frame_duration (guint whichframe)
       g_free (layer_name);
     }
 
-  if (duration < 0) duration = 100;  /* FIXME for default-if-not-said  */
-  else
-    if (duration == 0) duration = 100; /* FIXME - 0-wait is nasty */
+  if (duration <= 0)
+    duration = default_frame_duration;
 
   return (guint32) duration;
 }
