@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -128,17 +127,17 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",   "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",   "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",      "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",   "Input drawable" },
     { GIMP_PDB_FLOAT,    "horizontal", "Horizontal radius of gaussian blur (in pixels, > 0.0)" },
     { GIMP_PDB_FLOAT,    "vertical",   "Vertical radius of gaussian blur (in pixels, > 0.0)" },
-    { GIMP_PDB_INT32,    "method",     "IIR (0) or RLE (1)" }
+    { GIMP_PDB_INT32,    "method",     "Blur method { IIR (0), RLE (1) }" }
   };
 
   static const GimpParamDef args1[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",   "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",   "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",      "Input image (unused)" },
     { GIMP_PDB_DRAWABLE, "drawable",   "Input drawable" },
     { GIMP_PDB_FLOAT,    "radius",     "Radius of gaussian blur (in pixels, > 0.0)" },
@@ -148,7 +147,7 @@ query (void)
 
   static const GimpParamDef args2[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",   "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",   "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",      "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",   "Input drawable" },
     { GIMP_PDB_FLOAT,    "horizontal", "Horizontal radius of gaussian blur (in pixels, > 0.0)" },
@@ -265,7 +264,7 @@ run (const gchar      *name,
      gint             *nreturn_vals,
      GimpParam       **return_vals)
 {
-  static GimpParam   values[1];
+  static GimpParam   values[2];
   gint32             image_ID;
   GimpDrawable      *drawable;
   GimpRunMode        run_mode;
@@ -442,8 +441,10 @@ run (const gchar      *name,
         }
       else
         {
-          g_message (_("Cannot operate on indexed color images."));
-          status = GIMP_PDB_EXECUTION_ERROR;
+          status        = GIMP_PDB_EXECUTION_ERROR;
+          *nreturn_vals = 2;
+          values[1].type          = GIMP_PDB_STRING;
+          values[1].data.d_string = _("Cannot operate on indexed color images.");
         }
 
       gimp_drawable_detach (drawable);
@@ -491,7 +492,8 @@ gauss_dialog (gint32        image_ID,
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     main_vbox);
   gtk_widget_show (main_vbox);
 
   preview = gimp_drawable_preview_new (drawable, NULL);
@@ -1198,8 +1200,8 @@ gauss_rle (GimpDrawable *drawable,
 
   if (direct)
     gimp_pixel_rgn_init (&dest_rgn,
-			 drawable, 0, 0, drawable->width, drawable->height,
-			 TRUE, TRUE);
+                         drawable, 0, 0, drawable->width, drawable->height,
+                         TRUE, TRUE);
 
   progress = 0.0;
   max_progress  = (horz <= 0.0) ? 0 : width * height * horz;
@@ -1242,22 +1244,22 @@ gauss_rle (GimpDrawable *drawable,
           for (b = 0; b < bytes; b++)
             {
               gint same =  run_length_encode (src + b, rle, pix, bytes,
-					      height, length, TRUE);
+                                              height, length, TRUE);
 
               if (same > (3 * height) / 4)
-		{
-		  /* encoded_rle is only fastest if there are a lot of
-		   * repeating pixels
-		   */
+                {
+                  /* encoded_rle is only fastest if there are a lot of
+                   * repeating pixels
+                   */
                   do_encoded_lre (rle, pix, dest + b, height, length, bytes,
                                   curve, total, sum);
-		}
-	      else
-		{
-		  /* else a full but more simple algorithm is better */
-		  do_full_lre (pix, dest + b, height, length, bytes,
-			       curve, total);
-		}
+                }
+              else
+                {
+                  /* else a full but more simple algorithm is better */
+                  do_full_lre (pix, dest + b, height, length, bytes,
+                               curve, total);
+                }
             }
 
           if (has_alpha)
@@ -1354,22 +1356,22 @@ gauss_rle (GimpDrawable *drawable,
           for (b = 0; b < bytes; b++)
             {
               gint same = run_length_encode (src + b, rle, pix, bytes,
-					     width, length, TRUE);
+                                             width, length, TRUE);
 
               if (same > (3 * width) / 4)
-		{
-		  /* encoded_rle is only fastest if there are a lot of
-		   * repeating pixels
-		   */
+                {
+                  /* encoded_rle is only fastest if there are a lot of
+                   * repeating pixels
+                   */
                   do_encoded_lre (rle, pix, dest + b, width, length, bytes,
                                   curve, total, sum);
-		}
-	      else
-		{
-		  /* else a full but more simple algorithm is better */
-		  do_full_lre (pix, dest + b, width, length, bytes,
+                }
+              else
+                {
+                  /* else a full but more simple algorithm is better */
+                  do_full_lre (pix, dest + b, width, length, bytes,
                                curve, total);
-		}
+                }
             }
 
 
@@ -1459,15 +1461,15 @@ gauss (GimpDrawable *drawable,
 
   if (method == BLUR_IIR)
     gauss_iir (drawable,
-	       horz, vert, method, preview_buffer, x1, y1, width, height);
+               horz, vert, method, preview_buffer, x1, y1, width, height);
   else
     gauss_rle (drawable,
-	       horz, vert, method, preview_buffer, x1, y1, width, height);
+               horz, vert, method, preview_buffer, x1, y1, width, height);
 
   if (preview)
     {
       gimp_preview_draw_buffer (GIMP_PREVIEW (preview),
-				preview_buffer, width * drawable->bpp);
+                                preview_buffer, width * drawable->bpp);
       g_free (preview_buffer);
     }
   else
@@ -1495,9 +1497,9 @@ transfer_pixels (const gdouble *src1,
       sum = *src1++ + *src2++;
 
       if (sum > 255)
-	sum = 255;
+        sum = 255;
       else if (sum < 0)
-	sum = 0;
+        sum = 0;
 
       *dest++ = (guchar) sum;
     }
@@ -1529,13 +1531,13 @@ find_iir_constants (gdouble *n_p,
 
   n_p [0] = x4 + x6;
   n_p [1] = (exp(x1)*(x7*sin(x3)-(x6+2*x4)*cos(x3)) +
-	     exp(x0)*(x5*sin(x2) - (2*x6+x4)*cos (x2)));
+             exp(x0)*(x5*sin(x2) - (2*x6+x4)*cos (x2)));
   n_p [2] = (2 * exp(x0+x1) *
-	     ((x4+x6)*cos(x3)*cos(x2) - x5*cos(x3)*sin(x2) -
-	      x7*cos(x2)*sin(x3)) +
-	     x6*exp(2*x0) + x4*exp(2*x1));
+             ((x4+x6)*cos(x3)*cos(x2) - x5*cos(x3)*sin(x2) -
+              x7*cos(x2)*sin(x3)) +
+             x6*exp(2*x0) + x4*exp(2*x1));
   n_p [3] = (exp(x1+2*x0) * (x7*sin(x3) - x6*cos(x3)) +
-	     exp(x0+2*x1) * (x5*sin(x2) - x4*cos(x2)));
+             exp(x0+2*x1) * (x5*sin(x2) - x4*cos(x2)));
   n_p [4] = 0.0;
 
   d_p [0] = 0.0;
