@@ -1,9 +1,9 @@
 /* Cubism --- image filter plug-in for GIMP
  * Copyright (C) 1996 Spencer Kimball, Tracy Scott
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * You can contact me at quartic@polloux.fciencias.unam.mx
  * You can contact the original GIMP authors at gimp@xcf.berkeley.edu
@@ -138,12 +137,12 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",        "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",        "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",           "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",        "Input drawable" },
     { GIMP_PDB_FLOAT,    "tile-size",       "Average diameter of each tile (in pixels)" },
     { GIMP_PDB_FLOAT,    "tile-saturation", "Expand tiles by this amount" },
-    { GIMP_PDB_INT32,    "bg-color",        "Background color: { BLACK (0), BG (1) }" }
+    { GIMP_PDB_INT32,    "bg-color",        "Background color { BLACK (0), BG (1) }" }
   };
 
   gimp_install_procedure (PLUG_IN_PROC,
@@ -283,7 +282,8 @@ cubism_dialog (GimpDrawable *drawable)
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     main_vbox);
   gtk_widget_show (main_vbox);
 
   preview = gimp_drawable_preview_new (drawable, &cvals.preview);
