@@ -4,9 +4,9 @@
  * Decompose plug-in (C) 1997 Peter Kirchgessner
  * e-mail: peter@kirchgessner.net, WWW: http://www.kirchgessner.net
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -276,7 +275,7 @@ query (void)
 {
   static GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",       "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",       "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",          "Input image (unused)"         },
     { GIMP_PDB_DRAWABLE, "drawable",       "Input drawable"               },
     { GIMP_PDB_STRING,   "decompose-type", NULL                           },
@@ -309,36 +308,36 @@ query (void)
   args[3].description = type_desc->str;
 
   gimp_install_procedure (PLUG_IN_PROC,
-			  N_("Decompose an image into separate colorspace components"),
-			  "This function creates new gray images with "
-			  "different channel information in each of them",
-			  "Peter Kirchgessner",
-			  "Peter Kirchgessner",
-			  "1997",
-			  N_("_Decompose..."),
-			  "RGB*",
-			  GIMP_PLUGIN,
-			  G_N_ELEMENTS (args),
+                          N_("Decompose an image into separate colorspace components"),
+                          "This function creates new gray images with "
+                          "different channel information in each of them",
+                          "Peter Kirchgessner",
+                          "Peter Kirchgessner",
+                          "1997",
+                          N_("_Decompose..."),
+                          "RGB*",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (args),
                           G_N_ELEMENTS (return_vals),
-			  args, return_vals);
+                          args, return_vals);
 
   gimp_install_procedure (PLUG_IN_PROC_REG,
-			  N_("Decompose an image into separate colorspace components"),
-			  "This function creates new gray images with "
-			  "different channel information in each of them. "
-			  "Pixels in the foreground color will appear black "
+                          N_("Decompose an image into separate colorspace components"),
+                          "This function creates new gray images with "
+                          "different channel information in each of them. "
+                          "Pixels in the foreground color will appear black "
                           "in all output images.  This can be used for "
                           "things like crop marks that have to show up on "
                           "all channels.",
-			  "Peter Kirchgessner",
-			  "Peter Kirchgessner, Clarence Risher",
-			  "1997",
-			  N_("_Decompose..."),
-			  "RGB*",
-			  GIMP_PLUGIN,
-			  G_N_ELEMENTS (args),
+                          "Peter Kirchgessner",
+                          "Peter Kirchgessner, Clarence Risher",
+                          "1997",
+                          N_("_Decompose..."),
+                          "RGB*",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (args),
                           G_N_ELEMENTS (return_vals),
-			  args, return_vals);
+                          args, return_vals);
 
   gimp_plugin_menu_register (PLUG_IN_PROC_REG, "<Image>/Colors/Components");
 
@@ -388,24 +387,24 @@ run (const gchar      *name,
 
       /*  First acquire information with a dialog  */
       if (! decompose_dialog ())
-	return;
+        return;
       break;
 
     case GIMP_RUN_NONINTERACTIVE:
       /*  Make sure all the arguments are there!  */
       if (nparams != 4 && nparams != 5 && nparams != 6)
-	{
-	  status = GIMP_PDB_CALLING_ERROR;
-	}
+        {
+          status = GIMP_PDB_CALLING_ERROR;
+        }
       else
-	{
+        {
           strncpy (decovals.extract_type, param[3].data.d_string,
                    sizeof (decovals.extract_type));
           decovals.extract_type[sizeof (decovals.extract_type)-1] = '\0';
 
           decovals.as_layers = nparams > 4 ? param[4].data.d_int32 : FALSE;
           decovals.use_registration = (strcmp (name, PLUG_IN_PROC_REG) == 0);
-	}
+        }
       break;
 
     case GIMP_RUN_WITH_LAST_VALS:
@@ -435,11 +434,11 @@ run (const gchar      *name,
                               layer_ID_extract);
 
       if (num_images <= 0)
-	{
-	  status = GIMP_PDB_EXECUTION_ERROR;
-	}
+        {
+          status = GIMP_PDB_EXECUTION_ERROR;
+        }
       else
-	{
+        {
           /* create decompose-data parasite */
           GString *data = g_string_new ("");
 
@@ -449,25 +448,25 @@ run (const gchar      *name,
           for (j = 0; j < num_layers; j++)
             g_string_append_printf (data, "%d ", layer_ID_extract[j]);
 
-	  for (j = 0; j < num_images; j++)
-	    {
-	      values[j+1].data.d_int32 = image_ID_extract[j];
+          for (j = 0; j < num_images; j++)
+            {
+              values[j+1].data.d_int32 = image_ID_extract[j];
 
-	      gimp_image_undo_enable (image_ID_extract[j]);
-	      gimp_image_clean_all (image_ID_extract[j]);
+              gimp_image_undo_enable (image_ID_extract[j]);
+              gimp_image_clean_all (image_ID_extract[j]);
 
               gimp_image_attach_new_parasite (image_ID_extract[j],
                                               "decompose-data",
                                               0, data->len + 1, data->str);
 
-	      if (run_mode != GIMP_RUN_NONINTERACTIVE)
-		gimp_display_new (image_ID_extract[j]);
-	    }
+              if (run_mode != GIMP_RUN_NONINTERACTIVE)
+                gimp_display_new (image_ID_extract[j]);
+            }
 
-	  /*  Store data  */
-	  if (run_mode == GIMP_RUN_INTERACTIVE)
-	    gimp_set_data (PLUG_IN_PROC, &decovals, sizeof (DecoVals));
-	}
+          /*  Store data  */
+          if (run_mode == GIMP_RUN_INTERACTIVE)
+            gimp_set_data (PLUG_IN_PROC, &decovals, sizeof (DecoVals));
+        }
     }
 
   values[0].data.d_status = status;
@@ -501,10 +500,10 @@ decompose (gint32       image_ID,
   for (j = 0; j < G_N_ELEMENTS (extract); j++)
     {
       if (g_ascii_strcasecmp (extract_type, extract[j].type) == 0)
-	{
-	  extract_idx = j;
-	  break;
-	}
+        {
+          extract_idx = j;
+          break;
+        }
     }
   if (extract_idx < 0)
     return -1;
@@ -628,7 +627,7 @@ decompose (gint32       image_ID,
 
       /* Extract the channel information */
       extract[extract_idx].extract_fun (src, drawable_src->bpp, scan_lines*width,
-					dst);
+                                        dst);
 
       /* Transfer the registration color */
       if (decovals.use_registration)
@@ -637,8 +636,8 @@ decompose (gint32       image_ID,
 
       /* Set destination pixel regions */
       for (j = 0; j < num_layers; j++)
-	gimp_pixel_rgn_set_rect (&(pixel_rgn_dst[j]), dst[j], 0, i, width,
-				 scan_lines);
+        gimp_pixel_rgn_set_rect (&(pixel_rgn_dst[j]), dst[j], 0, i, width,
+                                 scan_lines);
       i += scan_lines;
 
       gimp_progress_update ((gdouble) i / (gdouble) height);
@@ -728,7 +727,7 @@ create_new_layer (gint32              image_ID,
 
   *drawable = gimp_drawable_get (layer_ID);
   gimp_pixel_rgn_init (pixel_rgn, *drawable, 0, 0, (*drawable)->width,
-		       (*drawable)->height, TRUE, FALSE);
+                       (*drawable)->height, TRUE, FALSE);
 
   return layer_ID;
 }
@@ -773,9 +772,9 @@ transfer_registration_color (const guchar  *src,
 
 static void
 extract_rgb (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *red_dst = dst[0];
@@ -794,9 +793,9 @@ extract_rgb (const guchar  *src,
 
 static void
 extract_rgba (const guchar  *src,
-	      gint           bpp,
-	      gint           numpix,
-	      guchar       **dst)
+              gint           bpp,
+              gint           numpix,
+              guchar       **dst)
 {
   register const guchar *rgba_src = src;
   register guchar *red_dst = dst[0];
@@ -817,9 +816,9 @@ extract_rgba (const guchar  *src,
 
 static void
 extract_red (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *red_dst = dst[0];
@@ -835,9 +834,9 @@ extract_red (const guchar  *src,
 
 static void
 extract_green (const guchar  *src,
-	       gint           bpp,
-	       gint           numpix,
-	       guchar       **dst)
+               gint           bpp,
+               gint           numpix,
+               guchar       **dst)
 {
   register const guchar *rgb_src = src+1;
   register guchar *green_dst = dst[0];
@@ -853,9 +852,9 @@ extract_green (const guchar  *src,
 
 static void
 extract_blue (const guchar  *src,
-	      gint           bpp,
-	      gint           numpix,
-	      guchar       **dst)
+              gint           bpp,
+              gint           numpix,
+              guchar       **dst)
 {
   register const guchar *rgb_src = src+2;
   register guchar *blue_dst = dst[0];
@@ -871,9 +870,9 @@ extract_blue (const guchar  *src,
 
 static void
 extract_alpha (const guchar  *src,
-	       gint           bpp,
-	       gint           numpix,
-	       guchar       **dst)
+               gint           bpp,
+               gint           numpix,
+               guchar       **dst)
 {
   register const guchar *rgb_src = src+3;
   register guchar *alpha_dst = dst[0];
@@ -889,9 +888,9 @@ extract_alpha (const guchar  *src,
 
 static void
 extract_cmy (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *cyan_dst = dst[0];
@@ -911,9 +910,9 @@ extract_cmy (const guchar  *src,
 
 static void
 extract_hsv (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *hue_dst = dst[0];
@@ -935,9 +934,9 @@ extract_hsv (const guchar  *src,
 
 static void
 extract_hue (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *hue_dst = dst[0];
@@ -955,9 +954,9 @@ extract_hue (const guchar  *src,
 
 static void
 extract_sat (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *sat_dst = dst[0];
@@ -975,9 +974,9 @@ extract_sat (const guchar  *src,
 
 static void
 extract_val (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *val_dst = dst[0];
@@ -995,9 +994,9 @@ extract_val (const guchar  *src,
 
 static void
 extract_hsl (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *hue_dst = dst[0];
@@ -1024,9 +1023,9 @@ extract_hsl (const guchar  *src,
 
 static void
 extract_huel (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *hue_dst = dst[0];
@@ -1048,9 +1047,9 @@ extract_huel (const guchar  *src,
 
 static void
 extract_satl (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *sat_dst = dst[0];
@@ -1072,9 +1071,9 @@ extract_satl (const guchar  *src,
 
 static void
 extract_lightness (const guchar  *src,
-		   gint           bpp,
-		   gint           numpix,
-		   guchar       **dst)
+                   gint           bpp,
+                   gint           numpix,
+                   guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *lum_dst = dst[0];
@@ -1095,9 +1094,9 @@ extract_lightness (const guchar  *src,
 
 static void
 extract_cyan (const guchar  *src,
-	      gint           bpp,
-	      gint           numpix,
-	      guchar       **dst)
+              gint           bpp,
+              gint           numpix,
+              guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *cyan_dst = dst[0];
@@ -1113,9 +1112,9 @@ extract_cyan (const guchar  *src,
 
 static void
 extract_magenta (const guchar  *src,
-		 gint           bpp,
-		 gint           numpix,
-		 guchar       **dst)
+                 gint           bpp,
+                 gint           numpix,
+                 guchar       **dst)
 {
   register const guchar *rgb_src = src+1;
   register guchar *magenta_dst = dst[0];
@@ -1131,9 +1130,9 @@ extract_magenta (const guchar  *src,
 
 static void
 extract_yellow (const guchar  *src,
-		gint           bpp,
-		gint           numpix,
-		guchar       **dst)
+                gint           bpp,
+                gint           numpix,
+                guchar       **dst)
 {
   register const guchar *rgb_src = src+2;
   register guchar *yellow_dst = dst[0];
@@ -1149,9 +1148,9 @@ extract_yellow (const guchar  *src,
 
 static void
 extract_cmyk (const guchar  *src,
-	      gint           bpp,
-	      gint           numpix,
-	      guchar       **dst)
+              gint           bpp,
+              gint           numpix,
+              guchar       **dst)
 
 {
   register const guchar *rgb_src = src;
@@ -1159,25 +1158,28 @@ extract_cmyk (const guchar  *src,
   register guchar *magenta_dst = dst[1];
   register guchar *yellow_dst = dst[2];
   register guchar *black_dst = dst[3];
+  register guchar k, s;
   register gint count = numpix, offset = bpp-3;
-  GimpCMYK gcmyk;
-
-  gimp_cmyk_set (&gcmyk, 0,0,0,0);
 
   while (count-- > 0)
     {
-      GimpRGB grgb;
-      guchar  r, g, b;
-
-      r = *rgb_src++;
-      g = *rgb_src++;
-      b = *rgb_src++;
-
-      gimp_rgb_set_uchar (&grgb, r, g, b);
-      gimp_rgb_to_cmyk (&grgb, 1.0, &gcmyk);
-      gimp_cmyk_get_uchar (&gcmyk,
-                           cyan_dst++, magenta_dst++, yellow_dst++,
-                           black_dst++);
+      *cyan_dst = k = 255 - *(rgb_src++);
+      *magenta_dst = s = 255 - *(rgb_src++);
+      if (s < k)
+        k = s;
+      *yellow_dst = s = 255 - *(rgb_src++);
+      if (s < k)
+        k = s;                /* Black intensity is minimum of c, m, y */
+      if (k)
+        {
+          *cyan_dst -= k;     /* Remove common part of c, m, y */
+          *magenta_dst -= k;
+          *yellow_dst -= k;
+        }
+      cyan_dst++;
+      magenta_dst++;
+      yellow_dst++;
+      *(black_dst++) = k;
 
       rgb_src += offset;
     }
@@ -1186,29 +1188,24 @@ extract_cmyk (const guchar  *src,
 
 static void
 extract_cyank (const guchar  *src,
-	       gint           bpp,
-	       gint           numpix,
-	       guchar       **dst)
+               gint           bpp,
+               gint           numpix,
+               guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *cyan_dst = dst[0];
+  register guchar s, k;
   register gint count = numpix, offset = bpp-3;
-  GimpCMYK gcmyk;
-
-  gimp_cmyk_set (&gcmyk, 0,0,0,0);
 
   while (count-- > 0)
     {
-      GimpRGB grgb;
-      guchar  r, g, b;
-
-      r = *rgb_src++;
-      g = *rgb_src++;
-      b = *rgb_src++;
-
-      gimp_rgb_set_uchar (&grgb, r,g,b);
-      gimp_rgb_to_cmyk (&grgb, 1.0, &gcmyk);
-      gimp_cmyk_get_uchar (&gcmyk, cyan_dst++, NULL, NULL, NULL);
+      *cyan_dst = k = 255 - *(rgb_src++);
+      s = 255 - *(rgb_src++);  /* magenta */
+      if (s < k) k = s;
+      s = 255 - *(rgb_src++);  /* yellow */
+      if (s < k) k = s;
+      if (k) *cyan_dst -= k;
+      cyan_dst++;
 
       rgb_src += offset;
     }
@@ -1217,29 +1214,27 @@ extract_cyank (const guchar  *src,
 
 static void
 extract_magentak (const guchar  *src,
-		  gint           bpp,
-		  gint           numpix,
-		  guchar       **dst)
+                  gint           bpp,
+                  gint           numpix,
+                  guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *magenta_dst = dst[0];
+  register guchar s, k;
   register gint count = numpix, offset = bpp-3;
-  GimpCMYK gcmyk;
-
-  gimp_cmyk_set (&gcmyk, 0,0,0,0);
 
   while (count-- > 0)
     {
-      GimpRGB grgb;
-      guchar  r, g, b;
-
-      r = *rgb_src++;
-      g = *rgb_src++;
-      b = *rgb_src++;
-
-      gimp_rgb_set_uchar (&grgb, r,g,b);
-      gimp_rgb_to_cmyk (&grgb, 1.0, &gcmyk);
-      gimp_cmyk_get_uchar (&gcmyk, NULL, magenta_dst++, NULL, NULL);
+      k = 255 - *(rgb_src++);  /* cyan */
+      *magenta_dst = s = 255 - *(rgb_src++);  /* magenta */
+      if (s < k)
+        k = s;
+      s = 255 - *(rgb_src++);  /* yellow */
+      if (s < k)
+        k = s;
+      if (k)
+        *magenta_dst -= k;
+      magenta_dst++;
 
       rgb_src += offset;
     }
@@ -1248,30 +1243,27 @@ extract_magentak (const guchar  *src,
 
 static void
 extract_yellowk (const guchar  *src,
-		 gint           bpp,
-		 gint           numpix,
-		 guchar       **dst)
+                 gint           bpp,
+                 gint           numpix,
+                 guchar       **dst)
 
 {
   register const guchar *rgb_src = src;
   register guchar *yellow_dst = dst[0];
+  register guchar s, k;
   register gint count = numpix, offset = bpp-3;
-  GimpCMYK gcmyk;
-
-  gimp_cmyk_set (&gcmyk, 0,0,0,0);
 
   while (count-- > 0)
     {
-      GimpRGB grgb;
-      guchar  r, g, b;
-
-      r = *rgb_src++;
-      g = *rgb_src++;
-      b = *rgb_src++;
-
-      gimp_rgb_set_uchar (&grgb, r,g,b);
-      gimp_rgb_to_cmyk (&grgb, 1.0, &gcmyk);
-      gimp_cmyk_get_uchar (&gcmyk, NULL, NULL, yellow_dst++, NULL);
+      k = 255 - *(rgb_src++);  /* cyan */
+      s = 255 - *(rgb_src++);  /* magenta */
+      if (s < k) k = s;
+      *yellow_dst = s = 255 - *(rgb_src++);
+      if (s < k)
+        k = s;
+      if (k)
+        *yellow_dst -= k;
+      yellow_dst++;
 
       rgb_src += offset;
     }
@@ -1279,9 +1271,9 @@ extract_yellowk (const guchar  *src,
 
 static void
 extract_lab (const guchar  *src,
-	     gint           bpp,
-	     gint           numpix,
-	     guchar       **dst)
+             gint           bpp,
+             gint           numpix,
+             guchar       **dst)
 {
   register const guchar *rgb_src = src;
   register guchar *l_dst = dst[0];
@@ -1503,8 +1495,8 @@ decompose_dialog (void)
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), main_vbox,
-                      TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
   frame = gimp_frame_new (_("Extract Channels"));
