@@ -350,7 +350,7 @@ load_image (const gchar  *filename,
   gint               bn_size;
   GimpImageBaseType  base_type;
   GimpImageType      image_type;
-  gssize             size;
+  gsize              size;
 
   fd = g_open (filename, O_RDONLY | _O_BINARY, 0);
 
@@ -380,6 +380,21 @@ load_image (const gchar  *filename,
   bh.magic_number = g_ntohl (bh.magic_number);
   bh.spacing      = g_ntohl (bh.spacing);
 
+  /* Sanitize values */
+  if ((bh.width == 0) || (bh.width > GIMP_MAX_IMAGE_SIZE) ||
+      (bh.height == 0) || (bh.height > GIMP_MAX_IMAGE_SIZE) ||
+      ((bh.bytes != 1) && (bh.bytes != 2) && (bh.bytes != 4) &&
+       (bh.bytes != 18)) ||
+      (G_MAXSIZE / bh.width / bh.height / bh.bytes < 1))
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   _("Invalid header data in '%s': width=%lu, height=%lu, "
+                     "bytes=%lu"), gimp_filename_to_utf8 (filename),
+                   (unsigned long int)bh.width, (unsigned long int)bh.height,
+                   (unsigned long int)bh.bytes);
+      return -1;
+    }
+
   switch (bh.version)
     {
     case 1:
@@ -606,6 +621,9 @@ save_image (const gchar  *filename,
   gint          x;
   gint          bpp;
   GimpPixelRgn  pixel_rgn;
+  GimpRGB       gray, white;
+
+  gimp_rgba_set_uchar (&white, 255, 255, 255, 255);
 
   switch (gimp_drawable_type (drawable_ID))
     {
@@ -667,7 +685,7 @@ save_image (const gchar  *filename,
                        0, 0, drawable->width, drawable->height,
                        FALSE, FALSE);
 
-  buffer = g_new (guchar, drawable->width * bpp);
+  buffer = g_new (guchar, drawable->width * MAX (bpp, drawable->bpp));
 
   for (line = 0; line < drawable->height; line++)
     {
@@ -682,9 +700,19 @@ save_image (const gchar  *filename,
           break;
 
         case 2:
-          /*  invert and drop alpha channel  */
           for (x = 0; x < drawable->width; x++)
-            buffer[x] = 255 - buffer[2 * x];
+            {
+              /*  apply alpha channel  */
+              gimp_rgba_set_uchar (&gray,
+                                   buffer[2 * x],
+                                   buffer[2 * x],
+                                   buffer[2 * x],
+                                   buffer[2 * x + 1]);
+              gimp_rgb_composite (&gray, &white, GIMP_RGB_COMPOSITE_BEHIND);
+              gimp_rgba_get_uchar (&gray, &buffer[x], NULL, NULL, NULL);
+              /* invert */
+              buffer[x] = 255 - buffer[x];
+            }
           break;
 
         case 3:
