@@ -119,7 +119,7 @@ query (void)
 {
   static const GimpParamDef load_args[] =
   {
-    { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
     { GIMP_PDB_STRING, "raw-filename", "The name entered"             }
   };
@@ -132,6 +132,12 @@ query (void)
     { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
     { GIMP_PDB_INT32,  "thumb-size",   "Preferred thumbnail size"     }
   };
+  static const GimpParamDef thumb_return_vals[] =
+  {
+    { GIMP_PDB_IMAGE,  "image",        "Output image"                 },
+    { GIMP_PDB_INT32,  "image-width",  "Width of full-sized image"    },
+    { GIMP_PDB_INT32,  "image-height", "Height of full-sized image"   }
+  };
 
   gimp_install_procedure (LOAD_PROC,
                           "Loads files of Compuserve GIF file format",
@@ -163,8 +169,8 @@ query (void)
                           NULL,
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (thumb_args),
-                          G_N_ELEMENTS (load_return_vals),
-                          thumb_args, load_return_vals);
+                          G_N_ELEMENTS (thumb_return_vals),
+                          thumb_args, thumb_return_vals);
 
   gimp_register_thumbnail_loader (LOAD_PROC, LOAD_THUMB_PROC);
 }
@@ -176,7 +182,7 @@ run (const gchar      *name,
      gint             *nreturn_vals,
      GimpParam       **return_vals)
 {
-  static GimpParam   values[2];
+  static GimpParam   values[4];
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
   GError            *error  = NULL;
   gint32             image_ID;
@@ -226,6 +232,15 @@ run (const gchar      *name,
           *nreturn_vals = 2;
           values[1].type         = GIMP_PDB_IMAGE;
           values[1].data.d_image = image_ID;
+
+          if (strcmp (name, LOAD_THUMB_PROC) == 0)
+            {
+              *nreturn_vals = 4;
+              values[2].type         = GIMP_PDB_INT32;
+              values[2].data.d_int32 = gimp_image_width (image_ID);
+              values[3].type         = GIMP_PDB_INT32;
+              values[3].data.d_int32 = gimp_image_height (image_ID);
+            }
         }
       else
         {
@@ -421,6 +436,7 @@ load_image (const gchar  *filename,
               g_message ("EOF / read error on extension function code");
               return image_ID; /* will be -1 if failed on first image! */
             }
+
           DoExtension (fd, c);
           continue;
         }
@@ -697,8 +713,7 @@ LZWReadByte (FILE *fd,
   static gint firstcode, oldcode;
   static gint clear_code, end_code;
   static gint table[2][(1 << MAX_LZW_BITS)];
-#define STACK_SIZE ((1 << (MAX_LZW_BITS)) * 2)
-  static gint stack[STACK_SIZE], *sp;
+  static gint stack[(1 << (MAX_LZW_BITS)) * 2], *sp;
   gint        i;
 
   if (just_reset_LZW)
@@ -744,11 +759,11 @@ LZWReadByte (FILE *fd,
         }
       while (firstcode == clear_code);
 
-      return firstcode & 255;
+      return firstcode;
     }
 
   if (sp > stack)
-    return (*--sp) & 255;
+    return *--sp;
 
   while ((code = GetCode (fd, code_size, FALSE)) >= 0)
     {
@@ -771,9 +786,9 @@ LZWReadByte (FILE *fd,
           sp            = stack;
           firstcode     = oldcode = GetCode (fd, code_size, FALSE);
 
-          return firstcode & 255;
+          return firstcode;
         }
-      else if (code == end_code || code > max_code)
+      else if (code == end_code)
         {
           gint   count;
           guchar buf[260];
@@ -792,14 +807,13 @@ LZWReadByte (FILE *fd,
 
       incode = code;
 
-      if (code == max_code)
+      if (code >= max_code)
         {
-          if (sp < &(stack[STACK_SIZE]))
-            *sp++ = firstcode;
+          *sp++ = firstcode;
           code = oldcode;
         }
 
-      while (code >= clear_code && sp < &(stack[STACK_SIZE]))
+      while (code >= clear_code)
         {
           *sp++ = table[1][code];
           if (code == table[0][code])
@@ -810,8 +824,7 @@ LZWReadByte (FILE *fd,
           code = table[0][code];
         }
 
-      if (sp < &(stack[STACK_SIZE]))
-        *sp++ = firstcode = table[1][code];
+      *sp++ = firstcode = table[1][code];
 
       if ((code = max_code) < (1 << MAX_LZW_BITS))
         {
@@ -829,10 +842,10 @@ LZWReadByte (FILE *fd,
       oldcode = incode;
 
       if (sp > stack)
-        return (*--sp) & 255;
+        return *--sp;
     }
 
-  return code & 255;
+  return code;
 }
 
 static gint32
