@@ -760,7 +760,7 @@ size_adjustment_callback (GtkWidget *widget,
   adj->guides = NULL;
   gimp_displays_flush ();
 
-  *(adj->value) = GTK_ADJUSTMENT (widget)->value;
+  *(adj->value) = gtk_adjustment_get_value (GTK_ADJUSTMENT (widget));
 
   if (adj->orientation == GIMP_ORIENTATION_VERTICAL)
     {
@@ -788,10 +788,8 @@ size_adjustment_callback (GtkWidget *widget,
 
   *(adj->count) = newn;
 
-  if (newn * *(adj->value) != size)
-    gtk_widget_show (GTK_WIDGET (adj->warning_label));
-  else
-    gtk_widget_hide (GTK_WIDGET (adj->warning_label));
+  gtk_widget_set_visible (GTK_WIDGET (adj->warning_label),
+                          newn * *(adj->value) != size);
 
   if (adj->ncells != NULL)
     gtk_adjustment_set_value (GTK_ADJUSTMENT (adj->ncells),
@@ -830,7 +828,7 @@ dim_callback (GtkAdjustment *adjustment,
 {
   gint i;
 
-  gihparams.dim = RINT (adjustment->value);
+  gihparams.dim = RINT (gtk_adjustment_get_value (adjustment));
 
   for (i = 0; i < GIMP_PIXPIPE_MAXDIM; i++)
     {
