@@ -11,9 +11,9 @@
  * Email: ddunbar@diads.com
  * WWW:   http://millennium.diads.com/gimp/
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -22,8 +22,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Version 1.0:
@@ -140,7 +139,7 @@ query (void)
 {
   static const GimpParamDef save_args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",     "Interactive" },
+    { GIMP_PDB_INT32,    "run-mode",     "The run mode { RUN-INTERACTIVE (0) }" },
     { GIMP_PDB_IMAGE,    "image",        "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
     { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
@@ -154,7 +153,7 @@ query (void)
                           "Daniel Dunbar",
                           "1998",
                           _("HTML table"),
-			  "RGB*, GRAY*",
+                          "RGB*, GRAY*",
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
@@ -190,13 +189,13 @@ run (const gchar      *name,
   if (save_dialog (param[1].data.d_int32))
     {
       if (save_image (param[3].data.d_string, drawable, &error))
-	{
-	  gimp_set_data (SAVE_PROC, &gtmvals, sizeof (GTMValues));
-	}
+        {
+          gimp_set_data (SAVE_PROC, &gtmvals, sizeof (GTMValues));
+        }
       else
-	{
-	  status = GIMP_PDB_EXECUTION_ERROR;
-	}
+        {
+          status = GIMP_PDB_EXECUTION_ERROR;
+        }
     }
   else
     {
@@ -215,7 +214,7 @@ run (const gchar      *name,
 
 static gboolean
 save_image (const gchar   *filename,
-	    GimpDrawable  *drawable,
+            GimpDrawable  *drawable,
             GError       **error)
 {
   gint          row,col, cols, rows, x, y;
@@ -406,26 +405,12 @@ save_dialog (gint32 image_ID)
 
   gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-  dialog = gimp_dialog_new (_("Save as HTML table"), PLUG_IN_BINARY,
-                            NULL, 0,
-                            gimp_standard_help_func, SAVE_PROC,
-
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
-
-                            NULL);
-
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
+  dialog = gimp_export_dialog_new (_("HTML table"), PLUG_IN_BINARY, SAVE_PROC);
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), main_vbox,
-		      TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gimp_export_dialog_get_content_area (dialog)),
+                      main_vbox, TRUE, TRUE, 0);
 
   if (gimp_image_width (image_ID) * gimp_image_height (image_ID) > 4096)
     {
@@ -445,8 +430,8 @@ save_dialog (gint32 image_ID)
       gtk_box_pack_start (GTK_BOX (hbox), eek, FALSE, FALSE, 0);
 
       label = gtk_label_new (_("You are about to create a huge\n"
-			       "HTML file which will most likely\n"
-			       "crash your browser."));
+                               "HTML file which will most likely\n"
+                               "crash your browser."));
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
       gtk_widget_show_all (frame);
@@ -466,10 +451,10 @@ save_dialog (gint32 image_ID)
   gtk_widget_show (toggle);
 
   gimp_help_set_help_data (toggle,
-			   _("If checked GTM will output a full HTML document "
-			     "with <HTML>, <BODY>, etc. tags instead of just "
-			     "the table html."),
-			   NULL);
+                           _("If checked GTM will output a full HTML document "
+                             "with <HTML>, <BODY>, etc. tags instead of just "
+                             "the table html."),
+                           NULL);
 
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
@@ -493,10 +478,10 @@ save_dialog (gint32 image_ID)
   gtk_widget_show (toggle);
 
   gimp_help_set_help_data (toggle,
-			   _("If checked GTM will replace any rectangular "
-			     "sections of identically colored blocks with one "
-			     "large cell with ROWSPAN and COLSPAN values."),
-			   NULL);
+                           _("If checked GTM will replace any rectangular "
+                             "sections of identically colored blocks with one "
+                             "large cell with ROWSPAN and COLSPAN values."),
+                           NULL);
 
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
@@ -508,11 +493,11 @@ save_dialog (gint32 image_ID)
   gtk_widget_show (toggle);
 
   gimp_help_set_help_data (toggle,
-			   _("Checking this tag will cause GTM to leave no "
-			     "whitespace between the TD tags and the "
-			     "cellcontent.  This is only necessary for pixel "
-			     "level positioning control."),
-			   NULL);
+                           _("Checking this tag will cause GTM to leave no "
+                             "whitespace between the TD tags and the "
+                             "cellcontent.  This is only necessary for pixel "
+                             "level positioning control."),
+                           NULL);
 
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
@@ -524,9 +509,9 @@ save_dialog (gint32 image_ID)
   gtk_widget_show (toggle);
 
   gimp_help_set_help_data (toggle,
-			   _("Check if you would like to have the table "
-			     "captioned."),
-			   NULL);
+                           _("Check if you would like to have the table "
+                             "captioned."),
+                           NULL);
 
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
@@ -536,7 +521,7 @@ save_dialog (gint32 image_ID)
   gtk_widget_set_size_request (entry, 200, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.captiontxt);
   gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
-		    GTK_FILL | GTK_EXPAND, 0, 0, 0);
+                    GTK_FILL | GTK_EXPAND, 0, 0, 0);
   gtk_widget_show (entry);
 
   gimp_help_set_help_data (entry, _("The text for the table caption."), NULL);
@@ -552,8 +537,8 @@ save_dialog (gint32 image_ID)
   gtk_widget_set_size_request (entry, 200, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.cellcontent);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
-			     _("C_ell content:"), 0.0, 0.5,
-			     entry, 1, FALSE);
+                             _("C_ell content:"), 0.0, 0.5,
+                             entry, 1, FALSE);
   gtk_widget_show (entry);
 
   gimp_help_set_help_data (entry, _("The text to go into each cell."), NULL);
@@ -575,14 +560,14 @@ save_dialog (gint32 image_ID)
   gtk_container_add (GTK_CONTAINER (frame), table);
 
   spinbutton = gimp_spin_button_new (&adj, gtmvals.border,
-				     0, 1000, 1, 10, 0, 1, 0);
+                                     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("_Border:"), 0.0, 0.5,
-			     spinbutton, 1, TRUE);
+                             _("_Border:"), 0.0, 0.5,
+                             spinbutton, 1, TRUE);
 
   gimp_help_set_help_data (spinbutton,
-			   _("The number of pixels in the table border."),
-			   NULL);
+                           _("The number of pixels in the table border."),
+                           NULL);
 
   g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_int_adjustment_update),
@@ -592,13 +577,13 @@ save_dialog (gint32 image_ID)
   gtk_widget_set_size_request (entry, 60, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.clwidth);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("_Width:"), 0.0, 0.5,
-			     entry, 1, TRUE);
+                             _("_Width:"), 0.0, 0.5,
+                             entry, 1, TRUE);
 
   gimp_help_set_help_data (entry,
-			   _("The width for each table cell.  "
-			     "Can be a number or a percent."),
-			   NULL);
+                           _("The width for each table cell.  "
+                             "Can be a number or a percent."),
+                           NULL);
 
   g_signal_connect (entry, "changed",
                     G_CALLBACK (gtm_clwidth_callback),
@@ -608,39 +593,39 @@ save_dialog (gint32 image_ID)
   gtk_widget_set_size_request (entry, 60, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.clheight);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
-			     _("_Height:"), 0.0, 0.5,
-			     entry, 1, TRUE);
+                             _("_Height:"), 0.0, 0.5,
+                             entry, 1, TRUE);
 
   gimp_help_set_help_data (entry,
-			   _("The height for each table cell.  "
-			     "Can be a number or a percent."),
-			   NULL);
+                           _("The height for each table cell.  "
+                             "Can be a number or a percent."),
+                           NULL);
 
   g_signal_connect (entry, "changed",
                     G_CALLBACK (gtm_clheight_callback),
                     NULL);
 
   spinbutton = gimp_spin_button_new (&adj, gtmvals.cellpadding,
-				     0, 1000, 1, 10, 0, 1, 0);
+                                     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
-			     _("Cell-_padding:"), 0.0, 0.5,
-			     spinbutton, 1, TRUE);
+                             _("Cell-_padding:"), 0.0, 0.5,
+                             spinbutton, 1, TRUE);
 
   gimp_help_set_help_data (spinbutton,
-			   _("The amount of cellpadding."), NULL);
+                           _("The amount of cellpadding."), NULL);
 
   g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_int_adjustment_update),
                     &gtmvals.cellpadding);
 
   spinbutton = gimp_spin_button_new (&adj, gtmvals.cellspacing,
-				     0, 1000, 1, 10, 0, 1, 0);
+                                     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 4,
-			     _("Cell-_spacing:"), 0.0, 0.5,
-			     spinbutton, 1, TRUE);
+                             _("Cell-_spacing:"), 0.0, 0.5,
+                             spinbutton, 1, TRUE);
 
   gimp_help_set_help_data (spinbutton,
-			   _("The amount of cellspacing."), NULL);
+                           _("The amount of cellspacing."), NULL);
 
   g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_int_adjustment_update),
@@ -660,7 +645,7 @@ save_dialog (gint32 image_ID)
 
 static gboolean
 color_comp (guchar *buffer,
-	    guchar *buf2)
+            guchar *buf2)
 {
   return (buffer[0] == buf2[0] &&
           buffer[1] == buf2[1] &&
@@ -671,28 +656,28 @@ color_comp (guchar *buffer,
 
 static void
 gtm_caption_callback (GtkWidget *widget,
-		      gpointer   data)
+                      gpointer   data)
 {
   strncpy (gtmvals.captiontxt, gtk_entry_get_text (GTK_ENTRY (widget)), 255);
 }
 
 static void
 gtm_cellcontent_callback (GtkWidget *widget,
-			  gpointer   data)
+                          gpointer   data)
 {
   strncpy (gtmvals.cellcontent, gtk_entry_get_text (GTK_ENTRY (widget)), 255);
 }
 
 static void
 gtm_clwidth_callback (GtkWidget *widget,
-		      gpointer   data)
+                      gpointer   data)
 {
   strncpy (gtmvals.clwidth, gtk_entry_get_text (GTK_ENTRY (widget)), 255);
 }
 
 static void
 gtm_clheight_callback (GtkWidget *widget,
-		       gpointer   data)
+                       gpointer   data)
 {
   strncpy (gtmvals.clheight, gtk_entry_get_text (GTK_ENTRY (widget)), 255);
 }
