@@ -526,28 +526,34 @@ open_document (const gchar  *filename,
                GError      **load_error)
 {
   PopplerDocument *doc;
-  gchar           *uri;
+  GMappedFile     *mapped_file;
   GError          *error = NULL;
 
-  uri = g_filename_to_uri (filename, NULL, &error);
+  mapped_file = g_mapped_file_new (filename, FALSE, &error);
 
-  if (! uri)
+  if (! mapped_file)
     {
       g_set_error (load_error, 0, 0,
-                   "Could not convert '%s' to an URI: %s",
+                   "Could not load '%s' %s",
                    gimp_filename_to_utf8 (filename), error->message);
       g_error_free (error);
       return NULL;
     }
 
-  doc = poppler_document_new_from_file (uri, NULL, &error);
+  doc = poppler_document_new_from_data (g_mapped_file_get_contents (mapped_file),
+                                        g_mapped_file_get_length (mapped_file),
+                                        NULL,
+                                        &error);
 
-  g_free (uri);
+  /* We can't g_mapped_file_unref(mapped_file) as apparently doc has
+   * references to data in there. No big deal, this is just a
+   * short-lived plug-in.
+   */
 
   if (! doc)
     {
       g_set_error (load_error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                   _("Could not open '%s' for reading: %s"),
+                   _("Could not load '%s': %s"),
                    gimp_filename_to_utf8 (filename),
                    error->message);
       g_error_free (error);
@@ -838,6 +844,7 @@ load_dialog (PopplerDocument  *doc,
 
   /* Title */
   title = gimp_prop_label_new (G_OBJECT (doc), "title");
+  gtk_label_set_ellipsize (GTK_LABEL (title), PANGO_ELLIPSIZE_END);
   gtk_box_pack_start (GTK_BOX (vbox), title, FALSE, FALSE, 0);
   gtk_widget_show (title);
 
@@ -1253,20 +1260,30 @@ gimp_resolution_entry_attach_label (GimpResolutionEntry *gre,
 
   if (column == 0)
     {
-      GtkTableChild *child;
-      GList         *list;
+      GList *children;
+      GList *list;
 
-      for (list = GTK_TABLE (gre)->children; list; list = g_list_next (list))
+      children = gtk_container_get_children (GTK_CONTAINER (gre));
+
+      for (list = children; list; list = g_list_next (list))
         {
-          child = list->data;
+          GtkWidget *child = list->data;
+          gint       left_attach;
+          gint       top_attach;
 
-          if (child->left_attach == 1 && child->top_attach == row)
+          gtk_container_child_get (GTK_CONTAINER (gre), child,
+                                   "left-attach", &left_attach,
+                                   "top-attach",  &top_attach,
+                                   NULL);
+
+          if (left_attach == 1 && top_attach == row)
             {
-              gtk_label_set_mnemonic_widget (GTK_LABEL (label),
-                                             child->widget);
+              gtk_label_set_mnemonic_widget (GTK_LABEL (label), child);
               break;
             }
         }
+
+      g_list_free (children);
     }
 
   gtk_misc_set_alignment (GTK_MISC (label), alignment, 0.5);
@@ -1353,7 +1370,7 @@ gimp_resolution_entry_value_callback (GtkWidget *widget,
   GimpResolutionEntryField *gref = (GimpResolutionEntryField *) data;
   gdouble                   new_value;
 
-  new_value = GTK_ADJUSTMENT (widget)->value;
+  new_value = gtk_adjustment_get_value (GTK_ADJUSTMENT (widget));
 
   if (gref->value != new_value)
     gimp_resolution_entry_update_value (gref, new_value);
