@@ -190,7 +190,7 @@ query (void)
                           G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
-  gimp_register_load_handler (LOAD_PROC, "", "");
+  gimp_register_load_handler (LOAD_PROC, "data", "");
 
   gimp_install_procedure (SAVE_PROC,
                           "Dump images to disk in raw format",
@@ -221,6 +221,8 @@ run (const gchar      *name,
   gint32             image_id;
   gint32             drawable_id;
 
+  INIT_I18N ();
+
   run_mode = param[0].data.d_int32;
 
   *nreturn_vals = 1;
