@@ -339,11 +339,7 @@ load_wmf_size (const gchar *filename,
     success = FALSE;
 
   wmf_mem_close (API);
-#if GLIB_CHECK_VERSION(2, 21, 3)
   g_mapped_file_unref (file);
-#else
-  g_mapped_file_free (file);
-#endif
 
   if (width < 1 || height < 1)
     {
@@ -874,11 +870,7 @@ wmf_get_pixbuf (const gchar *filename,
       wmf_api_destroy (API);
     }
 
-#if GLIB_CHECK_VERSION(2, 21, 3)
   g_mapped_file_unref (file);
-#else
-  g_mapped_file_free (file);
-#endif
 
   return pixels;
 }
@@ -963,11 +955,7 @@ wmf_load_file (const gchar  *filename,
       wmf_api_destroy (API);
     }
 
-#if GLIB_CHECK_VERSION(2, 21, 3)
   g_mapped_file_unref (file);
-#else
-  g_mapped_file_free (file);
-#endif
 
   /* FIXME: improve error message */
   g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
