@@ -836,11 +836,7 @@ lcms_image_set_profile (gint32       image,
                                     g_mapped_file_get_length (file),
                                     g_mapped_file_get_contents (file));
 
-#if GLIB_CHECK_VERSION(2, 21, 3)
       g_mapped_file_unref (file);
-#else
-      g_mapped_file_free (file);
-#endif
 
       gimp_image_parasite_attach (image, parasite);
       gimp_parasite_free (parasite);
@@ -1133,11 +1129,7 @@ lcms_load_profile (const gchar *filename,
                  gimp_filename_to_utf8 (filename));
     }
 
-#if GLIB_CHECK_VERSION(2, 21, 3)
   g_mapped_file_unref (file);
-#else
-  g_mapped_file_free (file);
-#endif
 
   return profile;
 }
