@@ -7,9 +7,9 @@
  *   useful discussion, optimisation and improvements.
  * Framework borrowed from many similar plugins...
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -83,12 +82,12 @@ static guchar             background_color[4];
 
 
 const GimpPlugInInfo PLUG_IN_INFO =
-  {
-    NULL,  /* init_proc  */
-    NULL,  /* quit_proc  */
-    query, /* query_proc */
-    run,   /* run_proc   */
-  };
+{
+  NULL,  /* init_proc  */
+  NULL,  /* quit_proc  */
+  query, /* query_proc */
+  run,   /* run_proc   */
+};
 
 
 MAIN ()
@@ -98,17 +97,15 @@ query (void)
 {
   static GimpParamDef args[] =
     {
-      { GIMP_PDB_INT32,      "run-mode",    "Interactive, non-interactive" },
-      { GIMP_PDB_IMAGE,      "image",       "Input image (unused)" },
-      { GIMP_PDB_DRAWABLE,   "drawable",    "Input drawable" },
-      { GIMP_PDB_FLOAT,      "offset-x",    "Effect centre offset in X" },
-      { GIMP_PDB_FLOAT,      "offset-y",    "Effect centre offset in Y" },
-      { GIMP_PDB_FLOAT,      "main-adjust",
-        "Amount of second-order distortion" },
-      { GIMP_PDB_FLOAT,      "edge-adjust",
-        "Amount of fourth-order distortion" },
-      { GIMP_PDB_FLOAT,      "rescale",     "Rescale overall image size" },
-      { GIMP_PDB_FLOAT,      "brighten",    "Adjust brightness in corners" }
+      { GIMP_PDB_INT32,    "run-mode",    "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
+      { GIMP_PDB_IMAGE,    "image",       "Input image (unused)" },
+      { GIMP_PDB_DRAWABLE, "drawable",    "Input drawable" },
+      { GIMP_PDB_FLOAT,    "offset-x",    "Effect centre offset in X" },
+      { GIMP_PDB_FLOAT,    "offset-y",    "Effect centre offset in Y" },
+      { GIMP_PDB_FLOAT,    "main-adjust", "Amount of second-order distortion" },
+      { GIMP_PDB_FLOAT,    "edge-adjust", "Amount of fourth-order distortion" },
+      { GIMP_PDB_FLOAT,    "rescale",     "Rescale overall image size" },
+      { GIMP_PDB_FLOAT,    "brighten",    "Adjust brightness in corners" }
     };
 
   gimp_install_procedure (PLUG_IN_PROC,
@@ -500,7 +497,8 @@ lens_dialog (GimpDrawable *drawable)
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     main_vbox);
   gtk_widget_show (main_vbox);
 
   preview = gimp_zoom_preview_new (drawable);
