@@ -83,8 +83,6 @@
     'film' => { ui => 1 },
     'filter-pack' => { ui => 1 },
     'fractal-trace' => { ui => 1 },
-    'gee' => { ui => 1 },
-    'gee-zoom' => { ui => 1 },
     'gradient-map' => {},
     'grid' => { ui => 1 },
     'guillotine' => {},
@@ -142,7 +140,7 @@
     'video' => { ui => 1 },
     'warp' => { ui => 1 },
     'waves' => { ui => 1 },
-    'web-browser' => {},
+    'web-browser' => { ui => 1 },
     'whirl-pinch' => { ui => 1 },
     'wind' => { ui => 1 }
 );
