@@ -1,23 +1,20 @@
 /*
- * "$Id: sharpen.c 23603 2007-09-21 13:27:33Z neo $"
+ * Sharpen filters for GIMP - The GNU Image Manipulation Program
  *
- *   Sharpen filters for GIMP - The GNU Image Manipulation Program
+ * Copyright 1997-1998 Michael Sweet (mike@easysw.com)
  *
- *   Copyright 1997-1998 Michael Sweet (mike@easysw.com)
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -103,7 +100,7 @@ query (void)
 {
   static const GimpParamDef   args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode", "Interactive, non-interactive"      },
+    { GIMP_PDB_INT32,    "run-mode", "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"      },
     { GIMP_PDB_IMAGE,    "image",    "Input image"                       },
     { GIMP_PDB_DRAWABLE, "drawable", "Input drawable"                    },
     { GIMP_PDB_INT32,    "percent",  "Percent sharpening (default = 10)" }
@@ -294,17 +291,19 @@ sharpen (GimpDrawable *drawable)
 
   filter = NULL;
 
-  gimp_drawable_mask_bounds (drawable->drawable_id,
-                             &x1, &y1, &x2, &y2);
+  if (! gimp_drawable_mask_intersect (drawable->drawable_id,
+                                      &x1, &y1, &sel_width, &sel_height))
+    return;
 
-  sel_width  = x2 - x1;
-  sel_height = y2 - y1;
-  img_bpp    = gimp_drawable_bpp (drawable->drawable_id);
+  x2 = x1 + sel_width;
+  y2 = y1 + sel_height;
+
+  img_bpp = gimp_drawable_bpp (drawable->drawable_id);
 
   /*
    * Let the user know what we're doing...
    */
-  gimp_progress_init( _("Sharpening"));
+  gimp_progress_init (_("Sharpening"));
 
   /*
    * Setup for filter...
@@ -491,7 +490,8 @@ sharpen_dialog (GimpDrawable *drawable)
 
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                     main_vbox);
   gtk_widget_show (main_vbox);
 
   preview = gimp_drawable_preview_new (drawable, NULL);
