@@ -11,9 +11,9 @@
  *
  *          Copyright (C) 2000 Monigotes
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -22,8 +22,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -210,7 +209,7 @@ query (void)
 {
   static const GimpParamDef save_args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",     "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",        "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
     { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
@@ -272,7 +271,7 @@ run (const gchar      *name,
         case GIMP_RUN_INTERACTIVE:
         case GIMP_RUN_WITH_LAST_VALS:
           gimp_ui_init (PLUG_IN_BINARY, FALSE);
-          export = gimp_export_image (&image_id, &drawable_id, "PSD",
+          export = gimp_export_image (&image_id, &drawable_id, NULL,
                                       GIMP_EXPORT_CAN_HANDLE_RGB     |
                                       GIMP_EXPORT_CAN_HANDLE_GRAY    |
                                       GIMP_EXPORT_CAN_HANDLE_INDEXED |
@@ -336,6 +335,9 @@ psd_lmode_layer (gint32  idLayer,
     case GIMP_COLOR_MODE:
       strcpy (psdMode, "colr");
       break;
+    case GIMP_ADDITION_MODE:
+      strcpy (psdMode, "lddg");
+      break;
     case GIMP_MULTIPLY_MODE:
       strcpy (psdMode, "mul ");
       break;
@@ -414,7 +416,7 @@ write_pascalstring (FILE        *fd,
     }
   else
     {
-      write_gint16 (fd, 0, why);
+      write_gchar (fd, 0, why);
     }
 
   /* If total length (length byte + content) is not a multiple of PADDING,
