@@ -1,15 +1,15 @@
 /*
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This is a plug-in for the GIMP.
+ * This is a plug-in for GIMP.
  *
  * Generates images containing vector type drawings.
  *
  * Copyright (C) 1997 Andy Thomas  alt@picnic.demon.co.uk
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -114,7 +113,7 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode", "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode", "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",    "Input image (unused)" },
     { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
     { GIMP_PDB_INT32,    "dummy",    "dummy" }
@@ -603,11 +602,11 @@ load_options (GFigObj *gfig,
     {
       /* Get option name */
 #ifdef DEBUG
-      printf ("num = %d\n", sscanf (load_buf, "%s %s", str_buf, opt_buf));
+      printf ("num = %d\n", sscanf (load_buf, "%255s %255s", str_buf, opt_buf));
 
       printf ("option %s val %s\n", str_buf, opt_buf);
 #else
-      sscanf (load_buf, "%s %s", str_buf, opt_buf);
+      sscanf (load_buf, "%255s %255s", str_buf, opt_buf);
 #endif /* DEBUG */
 
       if (!strcmp (str_buf, "GridSpacing:"))
