@@ -8,9 +8,9 @@
  *
  * Some code & ideas taken from the GNOME help browser.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,8 +19,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -86,7 +85,7 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,       "run-mode", "Interactive" },
+    { GIMP_PDB_INT32,       "run-mode", "The run mode { RUN-INTERACTIVE (0) }" },
     { GIMP_PDB_INT32,       "num-domain-names", ""    },
     { GIMP_PDB_STRINGARRAY, "domain-names",     ""    },
     { GIMP_PDB_INT32,       "num-domain-uris",  ""    },
@@ -129,6 +128,9 @@ run (const gchar      *name,
 
   INIT_I18N ();
 
+  if (! g_thread_supported ())
+    g_thread_init (NULL);
+
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
