@@ -6,9 +6,9 @@
  *                         Michael Natterer <mitch@gimp.org>
  *                         Henrik Brix Andersen <brix@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*  This code is written so that it can also be compiled standalone.
@@ -194,7 +193,7 @@ gimp_help_locale_parse (GimpHelpLocale    *locale,
     }
 
 #ifdef GIMP_HELP_DEBUG
-  g_printerr ("help (%s): parsing '%s' for locale \"%s\"\n",
+  g_printerr ("help (%s): parsing '%s' for \"%s\"\n",
               locale->locale_id, uri, help_domain);
 #endif
 
@@ -219,8 +218,8 @@ gimp_help_locale_parse (GimpHelpLocale    *locale,
                                            cancellable, error);
       if (! info)
         {
-          locale_set_error (error, _("Could not open '%s' for reading: %s"),
-                            file);
+          locale_set_error (error,
+                            _("Could not open '%s' for reading: %s"), file);
           g_object_unref (file);
 
           return FALSE;
@@ -235,7 +234,8 @@ gimp_help_locale_parse (GimpHelpLocale    *locale,
 
   if (! stream)
     {
-      locale_set_error (error, _("Could not open '%s' for reading: %s"), file);
+      locale_set_error (error,
+                        _("Could not open '%s' for reading: %s"), file);
       g_object_unref (file);
 
       return FALSE;
