@@ -1,13 +1,13 @@
 /*
- * This is a plug-in for the GIMP.
+ * This is a plug-in for GIMP.
  *
  * Generates clickable image maps.
  *
  * Copyright (C) 1998-1999 Maurits Rijk  lpeek.mrijk@consunet.nl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -42,14 +41,18 @@ void
 mru_destruct(MRU_t *mru)
 {
    g_list_foreach(mru->list, (GFunc) g_free, NULL);
+   g_list_free (mru->list);
    g_free(mru);
 }
 
 static void
 mru_remove_link(MRU_t *mru, GList *link)
 {
-   g_free(link->data);
-   mru->list = g_list_remove_link(mru->list, link);
+  if (link)
+    {
+      g_free(link->data);
+      mru->list = g_list_remove_link(mru->list, link);
+    }
 }
 
 static GList*
