@@ -35,29 +35,6 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-static void
-set_buttons(Selection_t *data)
-{
-#ifdef _OLD_
-  if (gtk_tree_selection_count_selected_rows (data->selection)) {
-#ifdef _OLD_
-    gtk_widget_set_sensitive(data->arrow_up,
-			     (data->selected_row) ? TRUE : FALSE);
-    if (data->selected_row < GTK_CLIST(data->list)->rows - 1)
-      gtk_widget_set_sensitive(data->arrow_down, TRUE);
-    else
-      gtk_widget_set_sensitive(data->arrow_down, FALSE);
-#endif
-    gtk_widget_set_sensitive(data->remove, TRUE);
-    gtk_widget_set_sensitive(data->edit, TRUE);
-  } else {
-    gtk_widget_set_sensitive(data->arrow_up, FALSE);
-    gtk_widget_set_sensitive(data->arrow_down, FALSE);
-    gtk_widget_set_sensitive(data->remove, FALSE);
-    gtk_widget_set_sensitive(data->edit, FALSE);
-  }
-#endif
-}
 
 static void
 changed_cb(GtkTreeSelection *selection, gpointer param)
@@ -100,8 +77,6 @@ changed_cb(GtkTreeSelection *selection, gpointer param)
 
         g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (selected_rows);
-
-        set_buttons (data);
   }
 }
 
@@ -222,7 +197,6 @@ object_removed_cb(Object_t *obj, gpointer data)
 
   if (selection_find_object (selection, obj, &iter)) {
     gtk_list_store_remove (GTK_LIST_STORE (selection->store), &iter);
-    set_buttons(selection);
   }
 }
 
@@ -232,7 +206,6 @@ object_selected_cb(Object_t *obj, gpointer data)
   Selection_t *selection = (Selection_t*) data;
   gint position = object_get_position_in_list (obj);
   selection_set_selected (selection, position);
-  set_buttons(selection);
 }
 
 static void
@@ -240,13 +213,6 @@ object_moved_cb(Object_t *obj, gpointer data)
 {
   Selection_t *selection = (Selection_t*) data;
   selection->select_lock = TRUE;
-#ifdef _OLD_
-  {
-    gint row = object_get_position_in_list(obj);
-    gtk_clist_set_row_data(GTK_CLIST(selection->list), row, (gpointer) obj);
-    selection_set_selected(selection, row);
-  }
-#endif
 }
 
 static const GtkTargetEntry target_table[] =
@@ -268,25 +234,33 @@ handle_drop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 	    GtkSelectionData *data, guint info, guint time)
 {
   gboolean success = FALSE;
-  if (data->length >= 0 && data->format == 8) {
-    GtkTreePath *path;
-    if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), x, y,
-				       &path, NULL, NULL, NULL)) {
-      GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
-      GtkTreeIter iter;
 
-      if (gtk_tree_model_get_iter (model, &iter, path)) {
-	Object_t *obj = selection_get_object (model, &iter);
-	if (!obj->locked) {
-	  command_list_add(edit_object_command_new (obj));
-	  object_set_url (obj, (const gchar *) data->data);
-	  object_emit_update_signal (obj);
-	  success = TRUE;
-	}
-      }
-      gtk_tree_path_free (path);
+  if (gtk_selection_data_get_length (data) >= 0 &&
+      gtk_selection_data_get_format (data) == 8)
+    {
+      GtkTreePath *path;
+
+      if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), x, y,
+                                         &path, NULL, NULL, NULL))
+        {
+          GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+          GtkTreeIter iter;
+
+          if (gtk_tree_model_get_iter (model, &iter, path))
+            {
+              Object_t *obj = selection_get_object (model, &iter);
+
+              if (!obj->locked)
+                {
+                  command_list_add(edit_object_command_new (obj));
+                  object_set_url (obj, (const gchar *) gtk_selection_data_get_data (data));
+                  object_emit_update_signal (obj);
+                  success = TRUE;
+                }
+            }
+          gtk_tree_path_free (path);
+        }
     }
-  }
   gtk_drag_finish(context, success, FALSE, time);
 }
 
@@ -448,8 +422,6 @@ make_selection(ObjectList_t *object_list)
   gtk_tree_selection_set_mode (data->selection, GTK_SELECTION_MULTIPLE);
   g_signal_connect (data->selection, "changed", G_CALLBACK(changed_cb), data);
 
-  set_buttons (data);
-
   /* Set object list callbacks we're interested in */
   object_list_add_add_cb (object_list, object_added_cb, data);
   object_list_add_update_cb (object_list, object_updated_cb, data);
@@ -463,13 +435,12 @@ make_selection(ObjectList_t *object_list)
 void
 selection_toggle_visibility(Selection_t *selection)
 {
-  if (selection->is_visible) {
-    gtk_widget_hide (selection->container);
-    selection->is_visible = FALSE;
-  } else {
-    gtk_widget_show (selection->container);
-    selection->is_visible = TRUE;
-  }
+  /* Toggle */
+  selection->is_visible = ! selection->is_visible;
+
+  /* Adapt to new state */
+  gtk_widget_set_visible (selection->container,
+                          selection->is_visible);
 }
 
 void
