@@ -102,16 +102,7 @@ static gchar buffer[BUFSIZE];
     We need to block signal handlers for div_entry this time.  We
       happen to know that div_entry's callback data is our old
       "friend", so we pull our friend out from where we stuck him in
-      the entry's userdata...  Hopefully that does it.  */
-
-/* Questions:
-
-     Gosh that was dumb.  Is there a way to
-       signal_handler_block_by_name?
-     That would make life so much nicer.
-
-         You could pass the handler_id around and use
-         gtk_signal_handler_block ().   (Sven)
+      the entry's userdata...  Hopefully that does it.
 */
 
 static void        maze_message          (const gchar *message);
@@ -655,7 +646,7 @@ entscale_int_scale_update (GtkAdjustment *adjustment,
 
   userdata = g_object_get_data (G_OBJECT (adjustment), "userdata");
 
-  new_val = (gint) adjustment->value;
+  new_val = (gint) gtk_adjustment_get_value (adjustment);
 
   *intvar = new_val;
 
@@ -691,23 +682,23 @@ entscale_int_entry_update (GtkWidget *widget,
 
   new_val = atoi (gtk_entry_get_text (GTK_ENTRY (widget)));
   constraint_val = new_val;
-  if ( constraint_val < adjustment->lower )
-    constraint_val = adjustment->lower;
-  if ( constraint_val > adjustment->upper )
-    constraint_val = adjustment->upper;
+
+  if (constraint_val < gtk_adjustment_get_lower (adjustment))
+    constraint_val = gtk_adjustment_get_lower (adjustment);
+
+  if (constraint_val > gtk_adjustment_get_upper (adjustment))
+    constraint_val = gtk_adjustment_get_upper (adjustment);
 
   if ( userdata->constraint )
     *intvar = constraint_val;
   else
     *intvar = new_val;
 
-  adjustment->value = constraint_val;
-
   g_signal_handlers_block_by_func (adjustment,
                                    entscale_int_scale_update,
                                    data);
 
-  gtk_adjustment_value_changed (adjustment);
+  gtk_adjustment_set_value (adjustment, constraint_val);
 
   g_signal_handlers_unblock_by_func (adjustment,
                                      entscale_int_scale_update,
