@@ -48,10 +48,13 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+#include "xmp-schemas.h"
+#include "xmp-model.h"
 #include "interface.h"
 #include "metadata.h"
-#include "xmp-schemas.h"
 #include "xmp-encode.h"
+#include "gimpxmpmodelentry.h"
+#include "gimpxmpmodeltext.h"
 
 
 #define RESPONSE_IMPORT   1
@@ -68,10 +71,10 @@ typedef struct
 } MetadataGui;
 
 static void
-value_edited (GtkCellRendererText *cell,
-        const gchar         *path_string,
-        const gchar         *new_text,
-        gpointer             data)
+tree_value_edited (GtkCellRendererText *cell,
+                   const gchar         *path_string,
+                   const gchar         *new_text,
+                   gpointer             data)
 {
   GtkTreeModel *model = data;
   GtkTreePath  *path  = gtk_tree_path_new_from_string (path_string);
@@ -85,8 +88,8 @@ value_edited (GtkCellRendererText *cell,
   /* FIXME: update value[] array */
   /* FIXME: check widget xref and update other widget if not NULL */
   gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
-          COL_XMP_VALUE, new_text,
-          -1);
+                      COL_XMP_VALUE, new_text,
+                      -1);
 
   gtk_tree_path_free (path);
 }
@@ -135,7 +138,7 @@ add_view_columns (GtkTreeView *treeview)
   g_object_set (renderer, "xalign", 0.0, NULL);
 
   g_signal_connect (renderer, "edited",
-        G_CALLBACK (value_edited), model);
+                    G_CALLBACK (tree_value_edited), model);
   col_offset =
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                                  -1, _("Value"),
@@ -188,80 +191,21 @@ update_icons (MetadataGui *mgui)
   gtk_tree_model_foreach (model, icon_foreach_func, mgui);
 }
 
-/* FIXME: temporary data structure for testing */
-typedef struct
-{
-  const gchar *schema;
-  const gchar *property_name;
-  GSList      *widget_list;
-} WidgetXRef;
-
 static void
-entry_changed (GtkEntry *entry,
-               gpointer  user_data)
-{
-  WidgetXRef *xref = user_data;
-
-  g_print ("XMP: %s %p %p %s\n", xref->property_name, entry, user_data, gtk_entry_get_text (entry)); /* FIXME */
-}
-
-static void
-register_entry_xref (GtkWidget   *entry,
-                     const gchar *schema,
-                     const gchar *property_name)
-{
-  WidgetXRef *xref;
-
-  xref = g_new (WidgetXRef, 1);
-  xref->schema = schema;
-  xref->property_name = property_name;
-  xref->widget_list = g_slist_prepend (NULL, entry);
-  g_signal_connect (GTK_ENTRY (entry), "changed",
-        G_CALLBACK (entry_changed), xref);
-}
-
-static void
-text_changed (GtkTextBuffer *text_buffer,
-             gpointer       user_data)
-{
-  WidgetXRef  *xref = user_data;
-  GtkTextIter  start;
-  GtkTextIter  end;
-
-  gtk_text_buffer_get_bounds (text_buffer, &start, &end);
-  g_print ("XMP: %s %p %p %s\n", xref->property_name, text_buffer, user_data, gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE)); /* FIXME */
-}
-
-static void
-register_text_xref (GtkTextBuffer *text_buffer,
-                    const gchar   *schema,
-                    const gchar   *property_name)
-{
-  WidgetXRef *xref;
-
-  xref = g_new (WidgetXRef, 1);
-  xref->schema = schema;
-  xref->property_name = property_name;
-  xref->widget_list = g_slist_prepend (NULL, text_buffer);
-  g_signal_connect (GTK_TEXT_BUFFER (text_buffer), "changed",
-        G_CALLBACK (text_changed), xref);
-}
-
-static void
-add_description_tab (GtkWidget *notebook)
+add_description_tab (GtkWidget   *notebook,
+                     MetadataGui *mgui)
 {
   GtkWidget     *frame;
   GtkWidget     *table;
   GtkWidget     *entry;
   GtkWidget     *scrolled_window;
   GtkWidget     *text_view;
-  GtkTextBuffer *text_buffer;
 
   frame = gimp_frame_new (_("Description"));
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame,
           gtk_label_new (_("Description")));
   gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
-  /* gtk_widget_show (frame); */
+  gtk_widget_show (frame);
 
   table = gtk_table_new (5, 2, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
@@ -269,17 +213,23 @@ add_description_tab (GtkWidget *notebook)
   gtk_container_add (GTK_CONTAINER (frame), table);
   /* gtk_widget_show (table); */
 
-  entry = gtk_entry_new ();
-  register_entry_xref (entry, XMP_SCHEMA_DUBLIN_CORE, "title");
+  entry = g_object_new (GIMP_TYPE_XMP_MODEL_ENTRY,
+                        "schema-uri",    XMP_SCHEMA_DUBLIN_CORE,
+                        "property-name", "title",
+                        "xmp-model",     mgui->xmp_model,
+                        NULL);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-           _("Image _title:"), 0.0, 0.5,
-           entry, 1, FALSE);
+                             _("Image _title:"), 0.0, 0.5,
+                             entry, 1, FALSE);
 
-  entry = gtk_entry_new ();
-  register_entry_xref (entry, XMP_SCHEMA_DUBLIN_CORE, "creator");
+  entry = g_object_new (GIMP_TYPE_XMP_MODEL_ENTRY,
+                        "schema-uri",    XMP_SCHEMA_DUBLIN_CORE,
+                        "property-name", "creator",
+                        "xmp-model",     mgui->xmp_model,
+                        NULL);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-           _("_Author:"), 0.0, 0.5,
-           entry, 1, FALSE);
+                             _("_Author:"), 0.0, 0.5,
+                             entry, 1, FALSE);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
@@ -287,21 +237,21 @@ add_description_tab (GtkWidget *notebook)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
           GTK_POLICY_AUTOMATIC,
           GTK_POLICY_AUTOMATIC);
-  text_view = gtk_text_view_new ();
-  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
-  gtk_text_buffer_set_text (text_buffer,
-                            "FIXME:\n"
-                            "These widgets are currently disconnected from the XMP model.\n"
-                            "Please use the Advanced tab.",
-                            -1);
-  register_text_xref (text_buffer, XMP_SCHEMA_DUBLIN_CORE, "description");
+  text_view = g_object_new (GIMP_TYPE_XMP_MODEL_TEXT,
+                            "schema-uri", XMP_SCHEMA_DUBLIN_CORE,
+                            "property-name", "description",
+                            "xmp-model", mgui->xmp_model,
+                            NULL);
   gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
            _("_Description:"), 0.0, 0.5,
            scrolled_window, 1, FALSE);
 
-  entry = gtk_entry_new ();
-  register_entry_xref (entry, XMP_SCHEMA_PHOTOSHOP, "CaptionWriter");
+  entry = g_object_new (GIMP_TYPE_XMP_MODEL_ENTRY,
+                        "schema-uri",    XMP_SCHEMA_PHOTOSHOP,
+                        "property-name", "CaptionWriter",
+                        "xmp-model",     mgui->xmp_model,
+                        NULL);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
            _("Description _writer:"), 0.0, 0.5,
            entry, 1, FALSE);
@@ -312,9 +262,11 @@ add_description_tab (GtkWidget *notebook)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
           GTK_POLICY_AUTOMATIC,
           GTK_POLICY_AUTOMATIC);
-  text_view = gtk_text_view_new ();
-  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
-  register_text_xref (text_buffer, XMP_SCHEMA_PDF, "Keywords");
+  text_view = g_object_new (GIMP_TYPE_XMP_MODEL_TEXT,
+                            "schema-uri", XMP_SCHEMA_PDF,
+                            "property-name", "Keywords",
+                            "xmp-model", mgui->xmp_model,
+                            NULL);
   gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 4,
            _("_Keywords:"), 0.0, 0.5,
@@ -696,7 +648,7 @@ metadata_dialog (gint32    image_ID,
   mgui.run_ok = FALSE;
 
   /* add the tabs to the notebook */
-  add_description_tab (notebook);
+  add_description_tab (notebook, &mgui);
   add_copyright_tab (notebook);
   add_origin_tab (notebook);
   add_camera1_tab (notebook);
