@@ -3,9 +3,9 @@
 #   Gimp-Python - allows the writing of Gimp plugins in Python.
 #   Copyright (C) 1997  James Henstridge <james@daa.com.au>
 #
-#   This program is free software; you can redistribute it and/or modify
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation; either version 2 of the License, or
+#   the Free Software Foundation; either version 3 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
 #   GNU General Public License for more details.
 #
 #   You should have received a copy of the GNU General Public License
-#   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from gimpfu import *
 import time
@@ -27,8 +26,12 @@ def foggify(img, layer, name, colour, turbulence, opacity):
     gimp.context_push()
     img.undo_group_start()
 
-    fog = gimp.Layer(img, name, layer.width, layer.height, RGBA_IMAGE,
-                     opacity, NORMAL_MODE)
+    if img.base_type is RGB:
+        type = RGBA_IMAGE
+    else:
+        type = GRAYA_IMAGE
+    fog = gimp.Layer(img, name,
+                     layer.width, layer.height, type, opacity, NORMAL_MODE)
     fog.fill(TRANSPARENT_FILL)
     img.add_layer(fog, 0)
 
