@@ -7,9 +7,9 @@
 #   Gimp-Python - allows the writing of Gimp plugins in Python.
 #   Copyright (C) 2003, 2005  Manish Singh <yosh@gimp.org>
 #
-#   This program is free software; you can redistribute it and/or modify
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation; either version 2 of the License, or
+#   the Free Software Foundation; either version 3 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -18,8 +18,7 @@
 #   GNU General Public License for more details.
 #
 #   You should have received a copy of the GNU General Public License
-#   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # (c) 2003 Manish Singh.
 #"Guillotine implemented ala python, with html output
@@ -36,9 +35,6 @@ gettext.install("gimp20-python", gimp.locale_directory, unicode=True)
 def pyslice(image, drawable, save_path, html_filename,
             image_basename, image_extension, separate,
             image_path, cellspacing, animate, skip_caps):
-
-    cellspacing = int (cellspacing)
-
     if animate:
         count = 0
         drw = []
