@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if 0
@@ -1251,9 +1250,10 @@ script_fu_marshal_procedure_call (scheme  *sc,
                   break;
                 }
 
+              args[i].data.d_parasite.size =
+                sc->vptr->ivalue (sc->vptr->pair_car (temp_val));
               args[i].data.d_parasite.data =
                 sc->vptr->string_value (sc->vptr->pair_car (temp_val));
-              args[i].data.d_parasite.size = strlen (args[i].data.d_parasite.data);
 
 #if DEBUG_MARSHALL
               g_printerr (", size %d\n", args[i].data.d_parasite.size);
@@ -1595,8 +1595,6 @@ script_fu_marshal_procedure_call (scheme  *sc,
                 else
                   {
                     GimpParasite *p = &values[i + 1].data.d_parasite;
-                    gchar        *data = g_strndup (p->data, p->size);
-                    gint          char_cnt = g_utf8_strlen (data, p->size);
                     pointer       temp_val;
 
                     /* don't move the mk_foo() calls outside this function call,
@@ -1608,13 +1606,12 @@ script_fu_marshal_procedure_call (scheme  *sc,
                                    sc->vptr->mk_integer (sc, p->flags),
                                    sc->vptr->cons (sc,
                                      sc->vptr->mk_counted_string (sc,
-                                                                  data,
-                                                                  char_cnt),
+                                                                  p->data,
+                                                                  p->size),
                                      sc->NIL)));
                     return_val = sc->vptr->cons (sc,
                                                  temp_val,
                                                  return_val);
-                    g_free (data);
 
 #if DEBUG_MARSHALL
                     g_printerr ("      name '%s'\n", p->name);
