@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -108,7 +107,7 @@
 #define RSP_LEN_L_BYTE  3
 
 /*
- *  Local Types
+ *  Local Structures
  */
 
 typedef struct
@@ -129,15 +128,6 @@ typedef struct
   gboolean   run;
 } ServerInterface;
 
-typedef union
-{
-  sa_family_t              family;
-  struct sockaddr_storage  ss;
-  struct sockaddr          sa;
-  struct sockaddr_in       sa_in;
-  struct sockaddr_in6      sa_in6;
-} sa_union;
-
 /*
  *  Local Functions
  */
@@ -146,8 +136,7 @@ static void      server_start       (gint         port,
                                      const gchar *logfile);
 static gboolean  execute_command    (SFCommand   *cmd);
 static gint      read_from_client   (gint         filedes);
-static gint      make_socket        (const struct addrinfo
-                                                 *ai);
+static gint      make_socket        (guint        port);
 static void      server_log         (const gchar *format,
                                      ...) G_GNUC_PRINTF (1, 2);
 static void      server_quit        (void);
@@ -161,10 +150,7 @@ static void      print_socket_api_error (const gchar *api_name);
 /*
  *  Local variables
  */
-static gint         server_socks[2],
-                    server_socks_used = 0;
-static const gint   server_socks_len = sizeof (server_socks) /
-                                       sizeof (server_socks[0]);
+static gint         server_sock;
 static GList       *command_queue   = NULL;
 static gint         queue_length    = 0;
 static gint         request_no      = 0;
@@ -298,7 +284,6 @@ script_fu_server_listen (gint timeout)
   struct timeval  tv;
   struct timeval *tvp = NULL;
   SELECT_MASK     fds;
-  gint            sockno;
 
   /*  Set time struct  */
   if (timeout)
@@ -309,10 +294,7 @@ script_fu_server_listen (gint timeout)
     }
 
   FD_ZERO (&fds);
-  for (sockno = 0; sockno < server_socks_used; sockno++)
-    {
-      FD_SET (server_socks[sockno], &fds);
-    }
+  FD_SET (server_sock, &fds);
   g_hash_table_foreach (clients, script_fu_server_add_fd, &fds);
 
   /* Block until input arrives on one or more active sockets
@@ -324,23 +306,15 @@ script_fu_server_listen (gint timeout)
       return;
     }
 
-  /* Service the server sockets if any has input pending. */
-  for (sockno = 0; sockno < server_socks_used; sockno++)
+  /* Service the server socket if it has input pending. */
+  if (FD_ISSET (server_sock, &fds))
     {
-      sa_union                 client;
-      gchar                    clientname[NI_MAXHOST];
+      struct sockaddr_in  clientname;
 
       /* Connection request on original socket. */
-      guint                    size = sizeof (client);
-      gint                     new;
-      guint                    portno;
-
-      if (! FD_ISSET (server_socks[sockno], &fds))
-        {
-          continue;
-        }
-
-      new = accept (server_socks[sockno], &(client.sa), &size);
+      guint size = sizeof (clientname);
+      gint  new  = accept (server_sock,
+                           (struct sockaddr *) &clientname, &size);
 
       if (new < 0)
         {
@@ -349,32 +323,13 @@ script_fu_server_listen (gint timeout)
         }
 
       /*  Associate the client address with the socket  */
-
-      /* If all else fails ... */
-      strncpy (clientname, "(error during host address lookup)", NI_MAXHOST-1);
-
-      /* Lookup address */
-      (void) getnameinfo (&(client.sa), size, clientname, sizeof (clientname),
-                          NULL, 0, NI_NUMERICHOST);
-
-      g_hash_table_insert (clients, GINT_TO_POINTER (new),
-                           g_strdup (clientname));
-
-      /* Determine port number */
-      switch (client.family)
-        {
-          case AF_INET:
-            portno = (guint) g_ntohs (client.sa_in.sin_port);
-            break;
-          case AF_INET6:
-            portno = (guint) g_ntohs (client.sa_in6.sin6_port);
-            break;
-          default:
-            portno = 0;
-        }
+      g_hash_table_insert (clients,
+                           GINT_TO_POINTER (new),
+                           g_strdup (inet_ntoa (clientname.sin_addr)));
 
       server_log ("Server: connect from host %s, port %d.\n",
-                  clientname, portno);
+                  inet_ntoa (clientname.sin_addr),
+                  (unsigned int) ntohs (clientname.sin_port));
     }
 
   /* Service the client sockets. */
@@ -436,46 +391,18 @@ static void
 server_start (gint         port,
               const gchar *logfile)
 {
-  struct addrinfo *ai,
-                  *ai_curr;
-  struct addrinfo  hints;
-  gint             e,
-                   sockno;
-  gchar           *port_s;
+  const gchar *progress;
 
-  const gchar     *progress;
+  /* First of all, create the socket and set it up to accept connections. */
+  /* This may fail if there's a server running on this port already.      */
+  server_sock = make_socket (port);
 
-  memset (&hints, 0, sizeof (hints));
-  hints.ai_flags = AI_PASSIVE | AI_ADDRCONFIG;
-  hints.ai_socktype = SOCK_STREAM;
-
-  port_s = g_strdup_printf ("%d", port);
-  e = getaddrinfo (NULL, port_s, &hints, &ai);
-  g_free (port_s);
-
-  if (e != 0)
+  if (listen (server_sock, 5) < 0)
     {
-      g_printerr ("getaddrinfo: %s", gai_strerror (e));
+      print_socket_api_error ("listen");
       return;
     }
 
-  for (ai_curr = ai, sockno = 0;
-       ai_curr != NULL && sockno < server_socks_len;
-       ai_curr = ai_curr->ai_next, sockno++)
-    {
-      /* Create the socket and set it up to accept connections.          */
-      /* This may fail if there's a server running on this port already. */
-      server_socks[sockno] = make_socket (ai_curr);
-
-      if (listen (server_socks[sockno], 5) < 0)
-        {
-          print_socket_api_error ("listen");
-          return;
-        }
-    }
-
-  server_socks_used = sockno;
-
   /*  Setup up the server log file  */
   if (logfile && *logfile)
     server_log_file = g_fopen (logfile, "a");
@@ -664,10 +591,11 @@ read_from_client (gint filedes)
 }
 
 static gint
-make_socket (const struct addrinfo *ai)
+make_socket (guint port)
 {
-  gint                    sock;
-  gint                    v = 1;
+  struct sockaddr_in name;
+  gint               sock;
+  gint               v = 1;
 
   /*  Win32 needs the winsock library initialized.  */
 #ifdef G_OS_WIN32
@@ -691,7 +619,7 @@ make_socket (const struct addrinfo *ai)
 #endif
 
   /* Create the socket. */
-  sock = socket (ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+  sock = socket (PF_INET, SOCK_STREAM, 0);
   if (sock < 0)
     {
       print_socket_api_error ("socket");
@@ -700,20 +628,12 @@ make_socket (const struct addrinfo *ai)
 
   setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &v, sizeof(v));
 
-#ifdef IPV6_V6ONLY
-  /* Only listen on IPv6 addresses, otherwise bind() will fail. */
-  if (ai->ai_family == AF_INET6)
-    {
-      v = 1;
-      if (setsockopt (sock, IPPROTO_IPV6, IPV6_V6ONLY, &v, sizeof(v)) < 0)
-        {
-          print_socket_api_error ("setsockopt");
-          gimp_quit();
-        }
-    }
-#endif
+  /* Give the socket a name. */
+  name.sin_family      = AF_INET;
+  name.sin_port        = htons (port);
+  name.sin_addr.s_addr = htonl (INADDR_ANY);
 
-  if (bind (sock, ai->ai_addr, ai->ai_addrlen) < 0)
+  if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0)
     {
       print_socket_api_error ("bind");
       gimp_quit ();
@@ -751,12 +671,7 @@ script_fu_server_shutdown_fd (gpointer key,
 static void
 server_quit (void)
 {
-  gint sockno;
-
-  for (sockno = 0; sockno < server_socks_used; sockno++)
-    {
-      CLOSESOCKET (server_socks[sockno]);
-    }
+  CLOSESOCKET (server_sock);
 
   if (clients)
     {
@@ -820,7 +735,7 @@ server_interface (void)
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
   gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
                       table, FALSE, FALSE, 0);
 
   /*  The server port  */
