@@ -1,9 +1,9 @@
 ; GIMP - The GNU Image Manipulation Program
 ; Copyright (C) 1995 Spencer Kimball and Peter Mattis
 ;
-; This program is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
+; the Free Software Foundation; either version 3 of the License, or
 ; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ;
 ;
 ; blend-anim.scm   version 1.03   1999/12/21
@@ -133,7 +132,7 @@
 							   blur
 							   blur)
 					(if (>= blur 1.0)
-						(plug-in-gauss-rle 1
+						(plug-in-gauss-rle RUN-NONINTERACTIVE
 							   image
 							   upper-copy
 							   blur
@@ -150,7 +149,7 @@
 							   blur
 							   blur)
 					(if (>= blur 1.0)
-						(plug-in-gauss-rle 1
+						(plug-in-gauss-rle RUN-NONINTERACTIVE
 							   image
 							   lower-copy
 							   blur
