@@ -1,9 +1,9 @@
-; The GIMP -- an image manipulation program
+; GIMP - The GNU Image Manipulation Program
 ; Copyright (C) 1995 Spencer Kimball and Peter Mattis
 ;
-; This program is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
+; the Free Software Foundation; either version 3 of the License, or
 ; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ;
 ;
 ; drop-shadow.scm   version 1.04   1999/12/21
@@ -93,14 +92,14 @@
                 (set! image-offset-x (- 0 (+ shadow-offset-x
                                              shadow-transl-x)))
                 (set! shadow-offset-x (- 0 shadow-transl-x))
-                (set! new-image-width (- new-image-width image-offset-x))))
+                (set! new-image-width (+ new-image-width image-offset-x))))
 
           (if (< (+ shadow-offset-y shadow-transl-y) 0)
               (begin
                 (set! image-offset-y (- 0 (+ shadow-offset-y
                                              shadow-transl-y)))
                 (set! shadow-offset-y (- 0 shadow-transl-y))
-                (set! new-image-height (- new-image-height image-offset-y))))
+                (set! new-image-height (+ new-image-height image-offset-y))))
 
           (if (> (+ (+ shadow-width shadow-offset-x) shadow-transl-x)
                  new-image-width)
@@ -124,10 +123,10 @@
                                             shadow-width
                                             shadow-height
                                             type
-                                            "Drop-Shadow"
+                                            "Drop Shadow"
                                             shadow-opacity
                                             NORMAL-MODE)))
-   (gimp-image-add-layer image shadow-layer -1)
+    (gimp-image-add-layer image shadow-layer -1)
     (gimp-layer-set-offsets shadow-layer
                             shadow-offset-x
                             shadow-offset-y))
@@ -137,7 +136,7 @@
   (gimp-edit-fill shadow-layer BACKGROUND-FILL)
   (gimp-selection-none image)
   (gimp-layer-set-lock-alpha shadow-layer FALSE)
-  (if (>= shadow-blur 1.0) (plug-in-gauss-rle 1
+  (if (>= shadow-blur 1.0) (plug-in-gauss-rle RUN-NONINTERACTIVE
                                               image
                                               shadow-layer
                                               shadow-blur
@@ -165,8 +164,8 @@
 )
 
 (script-fu-register "script-fu-drop-shadow"
-  _"_Drop-Shadow..."
-  _"Add a drop-shadow to the selected region (or alpha)"
+  _"_Drop Shadow..."
+  _"Add a drop shadow to the selected region (or alpha)"
   "Sven Neumann <sven@gimp.org>"
   "Sven Neumann"
   "1999/12/21"
