@@ -5,9 +5,9 @@
 ; FlatLand - creates a tileable pattern that looks like a map
 ; Copyright (C) 1997 Adrian Likins
 ;
-; This program is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
+; the Free Software Foundation; either version 3 of the License, or
 ; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;
 ; FlatLand
@@ -73,4 +72,4 @@
 )
 
 (script-fu-menu-register "script-fu-flatland"
-                         "<Image>/File/New/Patterns")
+                         "<Image>/File/Create/Patterns")
