@@ -14,9 +14,9 @@
 ;
 ; Thanks to Quartic for helping me debug this thing.
 ;
-; This program is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
+; the Free Software Foundation; either version 3 of the License, or
 ; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
@@ -25,8 +25,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 
@@ -87,4 +86,4 @@
 )
 
 (script-fu-menu-register "script-fu-land"
-                         "<Image>/File/New/Patterns")
+                         "<Image>/File/Create/Patterns")
