@@ -89,17 +89,20 @@ pointer EOF_OBJ;         /* special cell representing end-of-file object */
 pointer oblist;          /* pointer to symbol table */
 pointer global_env;      /* pointer to global environment */
 
+pointer c_nest;          /* stack for nested calls from C */
+
 /* global pointers to special symbols */
-pointer LAMBDA;               /* pointer to syntax lambda */
+pointer LAMBDA;          /* pointer to syntax lambda */
 pointer QUOTE;           /* pointer to syntax quote */
 
-pointer QQUOTE;               /* pointer to symbol quasiquote */
+pointer QQUOTE;          /* pointer to symbol quasiquote */
 pointer UNQUOTE;         /* pointer to symbol unquote */
 pointer UNQUOTESP;       /* pointer to symbol unquote-splicing */
 pointer FEED_TO;         /* => */
 pointer COLON_HOOK;      /* *colon-hook* */
 pointer ERROR_HOOK;      /* *error-hook* */
-pointer SHARP_HOOK;  /* *sharp-hook* */
+pointer SHARP_HOOK;      /* *sharp-hook* */
+pointer COMPILE_HOOK;    /* *compile-hook* */
 
 pointer free_cell;       /* pointer to top of free cells */
 long    fcells;          /* # of free cells */
@@ -110,7 +113,7 @@ pointer save_inport;
 pointer loadport;
 
 #define MAXFIL 64
-port load_stack[MAXFIL];     /* Stack of open files for port -1 (LOADing) */
+port load_stack[MAXFIL]; /* Stack of open files for port -1 (LOADing) */
 int nesting_stack[MAXFIL];
 int file_i;
 int nesting;
@@ -120,7 +123,7 @@ char    no_memory;       /* Whether mem. alloc. has failed */
 
 #define LINESIZE 1024
 char    linebuff[LINESIZE];
-#define STRBUFFSIZE 256
+#define STRBUFFSIZE 1024
 char    strbuff[STRBUFFSIZE];
 
 FILE *tmpfp;
@@ -129,7 +132,7 @@ int print_flag;
 pointer value;
 int op;
 
-void *ext_data;     /* For the benefit of foreign functions */
+void *ext_data;      /* For the benefit of foreign functions */
 long gensym_cnt;
 
 struct scheme_interface *vptr;
@@ -168,6 +171,8 @@ gunichar charvalue(pointer p);
 int is_vector(pointer p);
 
 int is_port(pointer p);
+int is_inport(pointer p);
+int is_outport(pointer p);
 
 int is_pair(pointer p);
 pointer pair_car(pointer p);
@@ -200,3 +205,9 @@ void setimmutable(pointer p);
 #endif
 
 #endif
+
+/*
+Local variables:
+c-file-style: "k&r"
+End:
+*/
