@@ -332,7 +332,7 @@ user directory.\n\
 \n\
 For plug-ins which do not use GTK+, the --build and --install options can be\n\
 appended with -noui for appropriate settings. For plug-ins that use GTK+ but\n\
-not libgumpui, append -nogimpui.\n");
+not libgimpui, append -nogimpui.\n");
   exit (exit_status);
 }
 
@@ -631,7 +631,7 @@ maybe_append_exe (const gchar *what)
   gchar *p = strrchr (what, '.');
 
   if (p == NULL || g_ascii_strcasecmp (p, ".exe") != 0)
-    return g_strconcat (what, ".exe");
+    return g_strconcat (what, ".exe", NULL);
 #endif
 
   return what;
